package br.com.centralit.api.dao.impl;

import java.util.Collection;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ContratoDao;
import br.com.centralit.api.model.Contrato;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Sort;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 25/01/2016 - 15:23:44
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Repository("contratoDao")
public class ContratoDaoHibernate extends CitGenericDAOImpl implements ContratoDao {

	/** Atributo MAX_RESULT_AUTO_COMPLETE. */
	private static final int MAX_RESULT_AUTO_COMPLETE = 10;

	public ContratoDaoHibernate() {
		super(Contrato.class);
	}

	@Override
	public Contrato buscarPorProgramaAcao(Long idProgramaAcao) {
		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterEqual("programaAcao.id", idProgramaAcao);

		return this.searchUnique(searchSeven);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<Contrato> findContrato(String value, Long idOrganizacao) {
		SearchSeven search = new SearchSeven();

		search.addFilter(Filter.ilike("numeroAnoContrato", "%" + value + "%"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addSort(Sort.asc("numeroAnoContrato"));

		search.setMaxResults(MAX_RESULT_AUTO_COMPLETE);

		return this.search(search, Contrato.class);
	}

	/**
	 * Método responsável por listar contratos por numero e organizacao
	 *
	 * @author luis.camargo
	 *
	 * @param numeroAnoContrato
	 * @param idOrganizacao
	 *
	 * @return Collection<Contrato>
	 */
	@Override
	public Collection<Contrato> findAutoCompleteContratoByNumeroAndOrganizacao(String numeroAnoContrato, Long idOrganizacao) {

		SearchSeven searchSeven = new SearchSeven(this.persistentClass);
		searchSeven.addFilterILike("numeroAnoContrato", "%" + numeroAnoContrato + "%");
		searchSeven.addFilterEqual("organizacao.id", idOrganizacao);

		searchSeven.addSort(Sort.asc("numeroAnoContrato"));
		return this.search(searchSeven, this.persistentClass);
	}

	public Collection<Contrato> findAutoCompleteContratoPorNumeroAndOrganizacao(String numeroAnoContrato, Long idOrganizacao) {

		SearchSeven searchSeven = new SearchSeven(this.persistentClass);
		searchSeven.addSort(Sort.asc("numeroAnoContrato"));
		searchSeven.addFilter(Filter.ilike("numeroAnoContrato", "%" + numeroAnoContrato + "%"));
		searchSeven.addFilterEqual("organizacao.id", idOrganizacao);
		searchSeven.setMaxResults(MAX_RESULT_AUTO_COMPLETE);

		return this.search(searchSeven, this.persistentClass);
	}
	
}
