package br.com.centralit.api.dao.impl;

import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ContratoEventoDao;
import br.com.centralit.api.model.ContratoEvento;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.util.UtilDate;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 09/03/2016 - 11:41:28
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Repository("contratoEventoDao")
public class ContratoEventoDaoHibernate extends CitGenericDAOImpl implements ContratoEventoDao {

	public ContratoEventoDaoHibernate() {
		super(ContratoEvento.class);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<ContratoEvento> findAllByContrato(Long contratoId) {

		SearchSeven search = new SearchSeven();

		search.addFilterIn("contrato.id", contratoId);

		search.addSort("dataEvento", false);

		search.addSort("id", false);

		return this.search(search);
	}

	@Override
	public Collection<ContratoEvento> findAllByContratoTipoEvento(Long contratoId, Long tipoEventoId) {
		SearchSeven search = new SearchSeven();

		search.addFilterIn("contrato.id", contratoId);

		search.addFilterEqual("tipoEvento.id", tipoEventoId);

		search.addSort("dataEvento", false);

		search.addSort("id", false);

		return this.search(search);
	}

	@SuppressWarnings("deprecation")
	@Override
	public Collection<ContratoEvento> findAllByContratoIntervalDataEmissao(Long contratoId, Date dataInicial, Date dateFinal) {
		SearchSeven search = new SearchSeven();

		search.addFilterIn("contrato.id", contratoId);
		if(dataInicial != null){
			search.addFilterGreaterOrEqual("dataEvento", UtilDate.dateToCalendar(dataInicial));
		}
		if(dateFinal != null){
			dateFinal.setHours(23);
			dateFinal.setMinutes(59);
			dateFinal.setSeconds(59);
			search.addFilterLessOrEqual("dataEvento", UtilDate.dateToCalendar(dateFinal));
		}

		search.addSort("tipoEvento.descricao", false);
		search.addSort("dataEvento", false);

		return this.search(search);
	}

	@SuppressWarnings("deprecation")
	@Override
	public Collection<ContratoEvento> findAllByContratoTipoTermoDataEmissao(List<Long> codigosTermo, Date dataInicial, Date dateFinal) {

		SearchSeven search = new SearchSeven();

		search.addFilterIn("tipoEvento.codigo", codigosTermo);

		if(dataInicial != null){
			search.addFilterGreaterOrEqual("dataEvento", UtilDate.dateToCalendar(dataInicial));
		}

		if(dateFinal != null){
			dateFinal.setHours(23);
			dateFinal.setMinutes(59);
			dateFinal.setSeconds(59);
			search.addFilterLessOrEqual("dataEvento", UtilDate.dateToCalendar(dateFinal));
		}

		search.addSort("tipoEvento.descricao", false);
		search.addSort("dataEvento", false);

		return this.search(search);
	}

	@Override
	public Collection<ContratoEvento> findAllByTipoEvento(List<Long> tiposEvento) {
		SearchSeven search = new SearchSeven();

		search.addFilterIn("tipoEvento.id", tiposEvento);

		search.addSort("dataEvento", false);

		search.addSort("id", false);

		return this.search(search);
	}

}
