package br.com.centralit.api.dao.impl;

import java.util.Calendar;
import java.util.Collection;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.EstudoTecnicoDao;
import br.com.centralit.api.model.EstudoTecnico;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Search;
import com.googlecode.genericdao.search.Sort;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 05/01/2016 - 15:47:23
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Repository("estudoTecnicoDao")
public class EstudoTecnicoDaoHibernate extends CitGenericDAOImpl implements EstudoTecnicoDao {

	/** Atributo MAX_RESULT_AUTO_COMPLETE. */
	private static final int MAX_RESULT_AUTO_COMPLETE = 10;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	public EstudoTecnicoDaoHibernate() {

		super(EstudoTecnico.class);
	}

	/**
	 * <p><b>História(s):</b> <a href="LINK_PORTAL">2389</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por buscar os estudos tecnicos cadastrados pelo codigo
	 *
	 * @author juliana.barbosa
	 *
	 * @param nome
	 * @return
	 */
	@Override
	public Collection<EstudoTecnico> listarEstudosPorCodigo(String codigo) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterILike("registroSequencial.sequencialAno", "%" + codigo + "%");

		search.addFilterEmpty("dataInativo");

		search.addSort(Sort.asc("registroSequencial.sequencialAno"));

		search.setMaxResults(10);

		return this.search(search, this.persistentClass);
	}

	/**
	 * Método responsável por listar estudos tecnicos por codigo e organizacao
	 *
	 * @author iago.almeida
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @return Collection<EstudoTecnico>
	 */
	@Override
	public Collection<EstudoTecnico> findAutoCompleteEstudoTecnicoPorCodigoAndOrganizacao(String codigo, Long idOrganizacao) {

		SearchSeven searchSeven = new SearchSeven(this.persistentClass);

		searchSeven.addSort(Sort.asc("registroSequencial.sequencialAno"));

		searchSeven.addFilter(Filter.ilike("registroSequencial.sequencialAno", "%" + codigo + "%"));

		searchSeven.addFilterEqual("organizacao.id", idOrganizacao);

		searchSeven.setMaxResults(MAX_RESULT_AUTO_COMPLETE);

		return this.search(searchSeven, this.persistentClass);

	}

	@Override
	public EstudoTecnico buscarUltimoRegistroPorDataEmissao(int ano, Long idOrganizacao) {
		int anoInformado = ano;

		Calendar dataInicial = Calendar.getInstance();
		dataInicial.set(anoInformado, Calendar.JANUARY, 1);
		dataInicial.set(Calendar.HOUR_OF_DAY,00);
		dataInicial.set(Calendar.MINUTE,00);
		dataInicial.set(Calendar.SECOND,00);
		dataInicial.set(Calendar.MILLISECOND,0);

		Calendar dataFinal = Calendar.getInstance();
		dataFinal.set(anoInformado, Calendar.DECEMBER, 31);
		dataFinal.set(Calendar.HOUR_OF_DAY,23);
		dataFinal.set(Calendar.MINUTE,59);
		dataFinal.set(Calendar.SECOND,59);
		dataFinal.set(Calendar.MILLISECOND,999);

		Search search = new Search();
		search.addFilterNotNull("registroSequencial.numeroSequencial");
		search.addFilterEqual("organizacao.id", idOrganizacao);
		search.addFilterGreaterOrEqual("dataCriacao", dataInicial);
		search.addFilterLessOrEqual("dataCriacao", dataFinal);
		search.addSortDesc("registroSequencial.numeroSequencial");
		search.setMaxResults(1);

		return searchUnique(search);
	}

}
