package br.com.centralit.api.dao.impl;

import java.util.Calendar;

import org.springframework.stereotype.Repository;

import com.googlecode.genericdao.search.Search;

import br.com.centralit.api.dao.EventoApostilamentoDao;
import br.com.centralit.api.model.EventoApostilamento;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 10/03/2016 - 09:32:28
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */

@Repository("eventoApostilamentoDao")
public class EventoApostilamentoDaoHibernate extends CitGenericDAOImpl implements EventoApostilamentoDao {

	public EventoApostilamentoDaoHibernate() {
		super(EventoApostilamento.class);
	}

	@Override
	public EventoApostilamento buscarUltimoRegistroPorDataEmissao(String property, Calendar dataEmissao, Long idOrganizacao) {

		return buscarUltimoRegistroPorDataEmissaoPorContrato(property, dataEmissao, idOrganizacao, null);
	}

	@Override
	public EventoApostilamento buscarUltimoRegistroPorDataEmissaoPorContrato(String property, Calendar dataEmissao, Long idOrganizacao, Long contratoId) {

		int anoInformado = dataEmissao.get(Calendar.YEAR);

		Calendar dataInicial = Calendar.getInstance();
		dataInicial.set(anoInformado, Calendar.JANUARY, 1);
		dataInicial.set(Calendar.HOUR_OF_DAY,00);
		dataInicial.set(Calendar.MINUTE,00);
		dataInicial.set(Calendar.SECOND,00);
		dataInicial.set(Calendar.MILLISECOND,0);

		Calendar dataFinal = Calendar.getInstance();
		dataFinal.set(anoInformado, Calendar.DECEMBER, 31);
		dataFinal.set(Calendar.HOUR_OF_DAY,23);
		dataFinal.set(Calendar.MINUTE,59);
		dataFinal.set(Calendar.SECOND,59);
		dataFinal.set(Calendar.MILLISECOND,999);

		Search search = new Search();
		search.addFilterNotNull(property);
		search.addFilterEqual("organizacao.id", idOrganizacao);
		search.addFilterGreaterOrEqual("dataEmissao", dataInicial);
		search.addFilterLessOrEqual("dataEmissao", dataFinal);
		if(contratoId != null){
			search.addFilterEqual("contrato.id", contratoId);
		}
		search.addSortDesc(property);
		search.setMaxResults(1);

		return searchUnique(search);
	}
}
