package br.com.centralit.api.dao.impl;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.List;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.EventoLiberacaoPagamentoDao;
import br.com.centralit.api.model.EventoLiberacaoPagamento;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;

import com.googlecode.genericdao.search.Search;

@Repository("eventoLiberacaoPagamentoDao")
public class EventoLiberacaoPagamentoDaoHibernate extends CitGenericDAOImpl implements EventoLiberacaoPagamentoDao {

	public EventoLiberacaoPagamentoDaoHibernate() {
		super(EventoLiberacaoPagamento.class);
	}

	@Override
	public BigDecimal getValorTotalLiberado(Long contratoId) {
		Search search = new Search();
		search.addFilterEqual("contrato.id", contratoId);
		List<EventoLiberacaoPagamento> result = this.search(search, this.persistentClass);
		BigDecimal valorTotalLiberado = new BigDecimal("0");
		if(result == null || result.size() == 0){
			return valorTotalLiberado;
		}else{
			for (EventoLiberacaoPagamento item : result) {
				if(item.getTotalLiberadoPagamento() != null){
					valorTotalLiberado = valorTotalLiberado.add(item.getTotalLiberadoPagamento());
				}
			}
		}
		return valorTotalLiberado;
	}

	@Override
	public EventoLiberacaoPagamento buscarUltimoRegistroPorDataEmissaoPorContrato(
			String campoBuscaParametrizadaUltimoRegistro, String anoInformado,
			Long organizacaoId, Long contratoId) {

			Calendar dataInicial = Calendar.getInstance();
			dataInicial.set(Integer.valueOf(anoInformado), Calendar.JANUARY, 1);
			dataInicial.set(Calendar.HOUR_OF_DAY, 00);
			dataInicial.set(Calendar.MINUTE, 00);
			dataInicial.set(Calendar.SECOND, 00);
			dataInicial.set(Calendar.MILLISECOND, 0);

			Calendar dataFinal = Calendar.getInstance();
			dataFinal.set(Integer.valueOf(anoInformado), Calendar.DECEMBER, 31);
			dataFinal.set(Calendar.HOUR_OF_DAY, 23);
			dataFinal.set(Calendar.MINUTE, 59);
			dataFinal.set(Calendar.SECOND, 59);
			dataFinal.set(Calendar.MILLISECOND, 999);

			Search search = new Search();
			search.addFilterNotNull(campoBuscaParametrizadaUltimoRegistro);
			search.addFilterEqual("organizacao.id", organizacaoId);
			search.addFilterGreaterOrEqual("dataLiberacao", dataInicial);
			search.addFilterLessOrEqual("dataLiberacao", dataFinal);
			search.addFilterEqual("contrato.id", contratoId);
			search.addSortDesc(campoBuscaParametrizadaUltimoRegistro);
			search.setMaxResults(1);

			return searchUnique(search);
		}


}
