package br.com.centralit.api.dao.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.persistence.Query;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ObjetivoEstrategicoDao;
import br.com.centralit.api.model.ObjetivoEstrategico;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;

import com.googlecode.genericdao.search.SearchResult;

@Repository("objetivoEstrategicoDao")
public class ObjetivoEstrategicoDaoHibernate extends CitGenericDAOImpl implements ObjetivoEstrategicoDao {

	public ObjetivoEstrategicoDaoHibernate() {
		super(ObjetivoEstrategico.class);
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por trazer todos os objetivos estratégicos da organizacao do usuário logado.
	 *
	 * @author luis.camargo
	 *
	 * @param organizacaoId ID da organizacao a se buscar os objetivos estratégicos
	 * @return
	 */
	@Override
	public Collection<ObjetivoEstrategico> findAllByOrganizacao(Long organizacaoId) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("organizacao.id", organizacaoId);

		return this.search(search, this.persistentClass);
	}

	@SuppressWarnings("unchecked")
	@Override
	public SearchResult<ObjetivoEstrategico> findAllObjetivosEstrategico(String nomeObjetivo, String descricaoNecessidade, String keyword, int limiteResultado, int pagina) {
		StringBuilder sql = new StringBuilder();
		List<ObjetivoEstrategico> listaFinal = new ArrayList<ObjetivoEstrategico>();
		@SuppressWarnings("rawtypes")
		SearchResult searchResult = new SearchResult<ObjetivoEstrategico>();

		sql.append(" SELECT ObjEst.ID, ObjEst.DESCRICAO AS OBJETIVO, ObjEst.NOME, ObjEstNec.DESCRICAO AS NECESSIDADE");
		sql.append(" FROM cnt_objetivoestrategico ObjEst ");
		sql.append(" INNER JOIN cnt_oe_necessidade ObjEstNec ON ObjEstNec.ObjetivoEstrategico_ID = ObjEst.ID ");
		sql.append(" WHERE ObjEst.DATABLOQUEIO IS NULL AND ObjEst.DATAINATIVO IS NULL  ");

		StringBuilder sqlCount = new StringBuilder();
		sqlCount.append(" SELECT ObjEst.ID, COUNT(ObjEstNec.DESCRICAO) ");
		sqlCount.append(" FROM cnt_objetivoestrategico ObjEst  ");
		sqlCount.append(" INNER JOIN cnt_oe_necessidade ObjEstNec ON ObjEstNec.ObjetivoEstrategico_ID = ObjEst.ID  ");
		sqlCount.append(" WHERE ObjEst.DATABLOQUEIO IS NULL AND ObjEst.DATAINATIVO IS NULL   ");

		if(StringUtils.isNotBlank(nomeObjetivo)){
			sql.append(" AND ObjEst.NOME LIKE '%"+ nomeObjetivo +"%'  ");
			sqlCount.append(" AND ObjEst.NOME LIKE '%"+ nomeObjetivo +"%' ");
		}

		if(StringUtils.isNotBlank(descricaoNecessidade)){
			sql.append(" AND ObjEstNec.DESCRICAO LIKE '%"+ descricaoNecessidade +"%' ");
			sqlCount.append(" AND ObjEstNec.DESCRICAO LIKE '%"+ descricaoNecessidade +"%' ");
		}

		if(StringUtils.isNotBlank(keyword)){
			sql.append(" AND (ObjEst.NOME LIKE '%"+ keyword +"%' OR  ObjEstNec.DESCRICAO LIKE '%"+ keyword +"%' )");
			sqlCount.append(" AND (ObjEst.NOME LIKE '%"+ keyword +"%' OR  ObjEstNec.DESCRICAO LIKE '%"+ keyword +"%' )");
		}

		sqlCount.append(" GROUP BY ObjEst.ID ");
		sqlCount.append(" ORDER BY ObjEst.ID ");

		Query queryCount = em().createNativeQuery(sqlCount.toString());
		List result = queryCount.getResultList();
		if(result != null){
			if(result.size() < limiteResultado){
				queryCount.setMaxResults(limiteResultado);
			}else {
				queryCount.setFirstResult(limiteResultado * (pagina - 1)).setMaxResults(limiteResultado);
			}

			List<Object[]> resultIds = queryCount.getResultList();
			if (!resultIds.isEmpty()) {
				StringBuilder ids = new StringBuilder();
				ids.append(" AND ObjEst.ID IN (");
				for (Object[] objeto : resultIds) {
					ids.append(objeto[0]);
					ids.append(",");
				}
				ids.deleteCharAt(ids.length() -1);
				ids.append(") ");
				sql.append(ids);
			}
			sql.append(" GROUP BY ObjEst.ID, ObjEst.DESCRICAO, ObjEst.NOME, ObjEstNec.DESCRICAO  ");
			sql.append(" ORDER BY ObjEst.ID ");

			Query query = em().createNativeQuery(sql.toString());
			if(result != null){

				List<Object[]> resultado = query.getResultList();
				long idAux = 0;
				ObjetivoEstrategico objEstrategico = null;
				StringBuilder necessidade = new StringBuilder();
				necessidade.append("<ul>");
				int count = 0;
				for (Object[] objeto : resultado) {
					count ++;
					if(idAux == 0){
						idAux = Long.valueOf(objeto[0].toString());
						objEstrategico = new ObjetivoEstrategico();
						objEstrategico.setId(Long.valueOf(objeto[0].toString()));
						objEstrategico.setDescricao(objeto[1].toString());
						objEstrategico.setNome(objeto[2].toString());

						necessidade.append("<li>");
						necessidade.append(objeto[3].toString());
						necessidade.append("</li>");

						if(result.size() == 1){
							necessidade.append("</ul>");
							objEstrategico.setNecessidadeElecandasDesricao(necessidade.toString());
							listaFinal.add(objEstrategico);
							break;
						}else{
							continue;
						}
					}
					if(idAux == Long.valueOf(objeto[0].toString())){
						necessidade.append("<li>");
						necessidade.append(objeto[3].toString());
						necessidade.append("</li>");

					}else{
						necessidade.append("</ul>");
						objEstrategico.setNecessidadeElecandasDesricao(necessidade.toString());
						listaFinal.add(objEstrategico);

						idAux = Long.valueOf(objeto[0].toString());
						objEstrategico = new ObjetivoEstrategico();
						objEstrategico.setId(Long.valueOf(objeto[0].toString()));
						objEstrategico.setDescricao(objeto[1].toString());
						objEstrategico.setNome(objeto[2].toString());
						necessidade = new StringBuilder();
						necessidade.append("<ul>");
						necessidade.append("<li>");
						necessidade.append(objeto[3].toString());
						necessidade.append("</li>");
					}

					if(count == resultado.size()){
						necessidade.append("</ul>");
						objEstrategico.setNecessidadeElecandasDesricao(necessidade.toString());
						listaFinal.add(objEstrategico);
					}
				}

				searchResult.setTotalCount(result.size());
				searchResult.setResult(listaFinal);
			}
		}
		return searchResult;
	}

}
