package br.com.centralit.api.dao.impl;

import java.util.Collection;

import javax.persistence.Query;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ProgramaDao;
import br.com.centralit.api.model.Programa;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Sort;

@Repository("programaDao")
public class ProgramaDaoHibernate extends CitGenericDAOImpl implements ProgramaDao {

	/** Atributo MAX_RESULT_AUTO_COMPLETE. */
	private static final int MAX_RESULT_AUTO_COMPLETE = 10;

	public ProgramaDaoHibernate() {
		super(Programa.class);
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por trazer todos os programas da organizacao do usuário logado.
	 *
	 * @author luis.camargo
	 *
	 * @param organizacaoId ID da organizacao a se buscar os programas
	 * @return
	 */
	@Override
	public Collection<Programa> findAllByOrganizacao(Long organizacaoId) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("organizacao.id", organizacaoId);

		return this.search(search, this.persistentClass);
	}


	/**
	 * Método responsável por listar programas por numero ou nome e organizacao
	 *
	 * @author iago.almeida
	 *
	 * @param numeroOrNome
	 * @param idOrganizacao
	 * @return Collection<Programa>
	 */
	@Override
	public Collection<Programa> findAutoCompleteProgramaByNumeroOrNomeAndOrganizacao(String numeroOrNome, Long idOrganizacao) {

		SearchSeven searchSeven = new SearchSeven(this.persistentClass);

		searchSeven.addSort(Sort.asc("nome"));

		if (StringUtils.isNumeric(numeroOrNome)) {
			searchSeven.addFilterOr(Filter.equal("codigo", Double.parseDouble(numeroOrNome)), Filter.ilike("nome", "%" + numeroOrNome + "%"));
		} else {
			searchSeven.addFilterILike("nome", "%" + numeroOrNome + "%");
		}

		searchSeven.addFilterEqual("organizacao.id", idOrganizacao);

		searchSeven.setMaxResults(MAX_RESULT_AUTO_COMPLETE);

		return this.search(searchSeven, this.persistentClass);

	}
	
	@Override
	public Programa findByProgramaAcao(Long idProgramaAcao) {

		StringBuilder query = new StringBuilder();
		query.append("select p.* ")
		.append(" from cnt_programa p ")
		.append(" inner join cnt_p_acao pa on pa.programa_id = p.id ")
		.append(" where pa.id = :idProgramaAcao ");

		Query resultado = em().createNativeQuery(query.toString(), Programa.class);
		resultado.setParameter("idProgramaAcao", idProgramaAcao);
		return (Programa) resultado.getSingleResult();
	}

}
