package br.com.centralit.api.dao.impl;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.RegistroOcorrenciaDao;
import br.com.centralit.api.model.RegistroOcorrencia;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.arquitetura.PersistentObject;

import com.googlecode.genericdao.search.Search;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 15/02/2016 - 15:01:19
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */
@Repository("registroOcorrenciaDao")
public class RegistroOcorrenciaDaoHibernate extends CitGenericDAOImpl implements RegistroOcorrenciaDao {

	public RegistroOcorrenciaDaoHibernate() {
		super(RegistroOcorrencia.class);
	}

	@Override
	public Collection<RegistroOcorrencia> findAllPorContrato(Long idContrato) {
		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("contrato.id", idContrato);

		return this.search(search, this.persistentClass);
	}

	@Override
	public PersistentObject buscarUltimoRegistroPorDataEmissao(String property, Calendar dataEmissaoOcorrencia, Long idOrganizacao) {

		int anoInformado = dataEmissaoOcorrencia.get(Calendar.YEAR);

		Calendar dataInicial = Calendar.getInstance();
		dataInicial.set(anoInformado, Calendar.JANUARY, 1, 00, 00, 00);
		dataInicial.set(Calendar.MILLISECOND,0);

		Calendar dataFinal = Calendar.getInstance();
		dataFinal.set(anoInformado, Calendar.DECEMBER, 31, 23, 59, 59);
		dataFinal.set(Calendar.MILLISECOND,999);

		Search search = new Search();
		search.addFilterNotNull(property);
		search.addFilterEqual("organizacao.id", idOrganizacao);
		search.addFilterGreaterOrEqual("dataEmissaoOcorrencia", dataInicial);
		search.addFilterLessOrEqual("dataEmissaoOcorrencia", dataFinal);
		search.addSortDesc(property);
		search.setMaxResults(1);

		return searchUnique(search);
	}

	@Override
	public BigDecimal getValorTotalGlosaOrdem(Long ordemFornecimentoId) {
		Search search = new Search();
		search.addFilterEqual("ordemFornecimento.id", ordemFornecimentoId);
		List<RegistroOcorrencia> result = this.search(search, this.persistentClass);
		BigDecimal totalGlosa = new BigDecimal("0");
		if(result == null || result.size() == 0){
			return totalGlosa;
		}else{
			for (RegistroOcorrencia registroOcorrencia : result) {
				if(registroOcorrencia.getValorGlosa() != null){
					totalGlosa = totalGlosa.add(registroOcorrencia.getValorGlosa());
				}
			}
		}
		return totalGlosa;
	}

}