package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 19/01/2016 - 14:04:42
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Audited
@AuditTable(value = "cnt_contrato_aud")
@Entity
@Table(name = "cnt_contrato")
@JsonIgnoreProperties({ "valorTotalItens", "cnpjFornecedor", "valorTotalLiberado", "ultimaDataVigenciaFinal" })
public class Contrato extends PersistentObjectAuditOrganizacao {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	/** Atributo programaAcao. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private ProgramaAcao programaAcao;

	/** Atributo numeroProcesso. */
	@Column(length = 30, nullable = false)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.OrdemFornecimentoEditView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.ContratoEventoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoPenalidadeEditView.class,
				ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Long numeroProcesso;

	/** Atributo exercicio. */
	@Column(name = "exercicio", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Calendar exercicio;

	/** Atributo numeroAnoContrato. */
	@Column(length = 30, nullable = false)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.OrdemFornecimentoListView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.NotificacaoContratoEditView.class,
				ViewsContrato.ContratoEventoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoPenalidadeEditView.class,
				ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private String numeroAnoContrato;

	/** Atributo numeroAnoContrato. */
	@Transient
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.OrdemFornecimentoListView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.NotificacaoContratoEditView.class,
				ViewsContrato.EncerramentoContratoListView.class,
				ViewsContrato.ContratoEventoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoPenalidadeEditView.class,
				ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private String numeroAnoContratoFormatado;

	/** Atributo dominioTipoContrato. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class })
	private Dominio dominioTipoContrato;

	/** Atributo dominioTipoAquisicao. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.OrdemFornecimentoEditView.class,
				ViewsContrato.EncerramentoContratoEditView.class })
	private Dominio dominioTipoAquisicao;

	/** Atributo dataCelebracao. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.EncerramentoContratoEditView.class })
	private Calendar dataCelebracao;

	/** Atributo dataVigenciaInicial. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.OrdemFornecimentoEditView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.NotificacaoContratoEditView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Calendar dataVigenciaInicial;

	/** Atributo dataVigenciaFinal. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.OrdemFornecimentoEditView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.NotificacaoContratoEditView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Calendar dataVigenciaFinal;

	@Transient
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.OrdemFornecimentoEditView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.NotificacaoContratoEditView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Calendar dataUltimaVigenciaFinal;

	/** Atributo dominioSituacaoAtual. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.OrdemFornecimentoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.ContratoEventoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class })
	private Dominio dominioSituacaoAtual;

	/** Atributo fornecedor. */
	@NotAudited
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.OrdemFornecimentoEditView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.NotificacaoContratoEditView.class,
				ViewsContrato.EncerramentoContratoListView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoPenalidadeEditView.class })
	private Fornecedor fornecedor;

	/** Atributo disponivel. */
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private boolean renovavel;

	/** Atributo objetoContratacao. */
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.EventoApostilamentoEditView.class,
				ViewsContrato.EventoRecebimentoObjetoEditView.class })
	@Column(length = 1024, nullable = false)
	private String objetoContratacao;

	/** Atributo valorContrato. */
	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.ContratoEventoEditView.class })
	private BigDecimal valorContrato;

	@Transient
	@JsonView({ ViewsContrato.ContratoEditView.class,
				ViewsContrato.ContratoAutoCompleteView.class,
				ViewsContrato.CronogramaExecucaoEditView.class,
				ViewsContrato.EncerramentoContratoEditView.class,
				ViewsContrato.RegistroOcorrenciaEditView.class,
				ViewsContrato.ContratoEventoEditView.class })
	private BigDecimal valorTotalContrato;

	/** Atributo dominioFormaPagamentoContrato. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JoinColumn(name = "formapagamentocontrato_id")
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Dominio dominioFormaPagamentoContrato;

	/** Atributo numeroParcela. */
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Long numeroParcela;

	/** Atributo ordensFornecimento. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contrato", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Collection<OrdemFornecimento> ordensFornecimento;

	/** Atributo ctrtValoresPagamento. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contrato", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Collection<ContratoValorPagamento> ctrtValoresPagamento;

	/** Atributo ctrtEmpenhos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contratoEmpenho", cascade = CascadeType.ALL, targetEntity = ContratoEmpenho.class)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Collection<ContratoEmpenho> ctrtEmpenhos;

	/** Atributo ctrtEmpenhos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contratoAditivo", cascade = CascadeType.ALL, targetEntity = EventoAditivo.class)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class,
				ViewsContrato.ContratoEventoEditView.class,
				ViewsContrato.ContratoEditView.class })
	private Collection<EventoAditivo> aditivos;

	/** Atributo valorGarantia. */
	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private BigDecimal valorTotalEmpenhado;

	/** Atributo numeroAnoEditalLicitacao. */
	@Column(length = 10, nullable = false)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private String numeroAnoEditalLicitacao;

	/** Atributo dataPublicacaoLicitacao. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Calendar dataPublicacaoLicitacao;

	/** Atributo dominioModalidadeLicitacao. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Dominio dominioModalidadeLicitacao;

	/** Atributo dominioTipoLicitacao. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Dominio dominioTipoLicitacao;

	/** Atributo ctrtEmpenhos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contrato", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.CronogramaExecucaoEditView.class })
	private Collection<ContratoItem> ctrtItens;

	/** Atributo garantias. ctrtEnvolvidos */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contrato", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Collection<ContratoEnvolvido> ctrtEnvolvidos;

	/** Atributo dominioModalidadeGarantia. */
	@ManyToOne(fetch = FetchType.LAZY)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Dominio dominioModalidadeGarantia;

	/** Atributo dataVigencia. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Calendar dataVigenciaGarantia;

	/** Atributo valorGarantia. */
	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private BigDecimal valorGarantia;

	/** Atributo percentual. */
	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private BigDecimal percentualGarantia;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Collection<ContratoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Collection<ContratoObservacao> observacoes;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.ContratoAutoCompleteView.class })
	private TermoReferencia termoReferencia;

	@Transient
	@JsonView({ ViewsContrato.ContratoEditView.class })
	private Long rascunho;
	
	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>programaAcao</code>
	 *
	 * @return <code>ProgramaAcao</code>
	 */
	public ProgramaAcao getProgramaAcao() {

		return programaAcao;
	}

	/**
	 * Define o valor do atributo <code>programaAcao</code>.
	 *
	 * @param programaAcao
	 */
	public void setProgramaAcao(ProgramaAcao programaAcao) {

		this.programaAcao = programaAcao;
	}

	/**
	 * Retorna o valor do atributo <code>numeroProcesso</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getNumeroProcesso() {

		return numeroProcesso;
	}

	/**
	 * Define o valor do atributo <code>numeroProcesso</code>.
	 *
	 * @param numeroProcesso
	 */
	public void setNumeroProcesso(Long numeroProcesso) {

		this.numeroProcesso = numeroProcesso;
	}

	/**
	 * Retorna o valor do atributo <code>exercicio</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getExercicio() {

		return exercicio;
	}

	/**
	 * Define o valor do atributo <code>exercicio</code>.
	 *
	 * @param exercicio
	 */
	public void setExercicio(Calendar exercicio) {

		this.exercicio = exercicio;
	}

	/**
	 * Retorna o valor do atributo <code>numeroAnoContrato</code> formatado com
	 * máscara
	 *
	 * @return <code>String</code>
	 */
	public String getNumeroAnoContratoFormatado() {
		if (this.numeroAnoContrato != null
				&& this.numeroAnoContrato.length() == 13) {
			this.numeroAnoContratoFormatado = numeroAnoContrato.substring(0, 9)
					+ "/" + numeroAnoContrato.substring(9, 13);
		}
		return this.numeroAnoContratoFormatado;
	}

	/**
	 * Retorna o valor do atributo <code>numeroAnoContrato</code>
	 *
	 * @return <code>String</code>
	 */
	public String getNumeroAnoContrato() {

		return numeroAnoContrato;
	}

	/**
	 * Define o valor do atributo <code>numeroAnoContrato</code>.
	 *
	 * @param numeroAnoContrato
	 */
	public void setNumeroAnoContrato(String numeroAnoContrato) {

		this.numeroAnoContrato = numeroAnoContrato;
	}

	/**
	 * Retorna o valor do atributo <code>dominioTipoContrato</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoContrato() {

		return dominioTipoContrato;
	}

	/**
	 * Define o valor do atributo <code>dominioTipoContrato</code>.
	 *
	 * @param dominioTipoContrato
	 */
	public void setDominioTipoContrato(Dominio dominioTipoContrato) {

		this.dominioTipoContrato = dominioTipoContrato;
	}

	/**
	 * Retorna o valor do atributo <code>dominioTipoAquisicao</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoAquisicao() {

		return dominioTipoAquisicao;
	}

	/**
	 * Define o valor do atributo <code>dominioTipoAquisicao</code>.
	 *
	 * @param dominioTipoAquisicao
	 */
	public void setDominioTipoAquisicao(Dominio dominioTipoAquisicao) {

		this.dominioTipoAquisicao = dominioTipoAquisicao;
	}

	/**
	 * Retorna o valor do atributo <code>dataCelebracao</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataCelebracao() {

		return dataCelebracao;
	}

	/**
	 * Define o valor do atributo <code>dataCelebracao</code>.
	 *
	 * @param dataCelebracao
	 */
	public void setDataCelebracao(Calendar dataCelebracao) {

		this.dataCelebracao = dataCelebracao;
	}

	/**
	 * Retorna o valor do atributo <code>dataVigenciaInicial</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataVigenciaInicial() {

		return dataVigenciaInicial;
	}

	/**
	 * Define o valor do atributo <code>dataVigenciaInicial</code>.
	 *
	 * @param dataVigenciaInicial
	 */
	public void setDataVigenciaInicial(Calendar dataVigenciaInicial) {

		this.dataVigenciaInicial = dataVigenciaInicial;
	}

	/**
	 * Retorna o valor do atributo <code>dataVigenciaFinal</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataVigenciaFinal() {

		return dataVigenciaFinal;
	}

	/**
	 * Define o valor do atributo <code>dataVigenciaFinal</code>.
	 *
	 * @param dataVigenciaFinal
	 */
	public void setDataVigenciaFinal(Calendar dataVigenciaFinal) {

		this.dataVigenciaFinal = dataVigenciaFinal;
	}

	/**
	 * Retorna o valor do atributo <code>dataUltimaVigenciaFinal</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataUltimaVigenciaFinal() {

		if (this.aditivos == null || this.aditivos.isEmpty()) {
			return this.dataVigenciaFinal;
		} else {
			this.dataUltimaVigenciaFinal = this.dataVigenciaFinal;
			for (EventoAditivo aditivo : this.aditivos) {
				if (aditivo.getAditivoDePrazo()
						&& aditivo.getDataFim().after(
								this.dataUltimaVigenciaFinal)) {
					this.dataUltimaVigenciaFinal = aditivo.getDataFim();
				}
			}
		}
		return this.dataUltimaVigenciaFinal;
	}

	/**
	 * Define o valor do atributo <code>dataUltimaVigenciaFinal</code>.
	 *
	 * @param dataUltimaVigenciaFinal
	 */
	public void setDataUltimaVigenciaFinal(Calendar dataUltimaVigenciaFinal) {

		this.dataUltimaVigenciaFinal = dataUltimaVigenciaFinal;
	}

	/**
	 * Retorna o valor do atributo <code>dominioSituacaoAtual</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioSituacaoAtual() {

		return dominioSituacaoAtual;
	}

	/**
	 * Define o valor do atributo <code>dominioSituacaoAtual</code>.
	 *
	 * @param dominioSituacaoAtual
	 */
	public void setDominioSituacaoAtual(Dominio dominioSituacaoAtual) {

		this.dominioSituacaoAtual = dominioSituacaoAtual;
	}

	/**
	 * Retorna o valor do atributo <code>fornecedor</code>
	 *
	 * @return <code>Fornecedor</code>
	 */
	public Fornecedor getFornecedor() {

		return fornecedor;
	}

	/**
	 * Define o valor do atributo <code>fornecedor</code>.
	 *
	 * @param fornecedor
	 */
	public void setFornecedor(Fornecedor fornecedor) {

		this.fornecedor = fornecedor;
	}

	/**
	 * Retorna o valor do atributo <code>renovavel</code>
	 *
	 * @return <code>boolean</code>
	 */
	public boolean isRenovavel() {

		return renovavel;
	}

	/**
	 * Define o valor do atributo <code>renovavel</code>.
	 *
	 * @param renovavel
	 */
	public void setRenovavel(boolean renovavel) {

		this.renovavel = renovavel;
	}

	/**
	 * Retorna o valor do atributo <code>objetoContratacao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getObjetoContratacao() {

		return objetoContratacao;
	}

	/**
	 * Define o valor do atributo <code>objetoContratacao</code>.
	 *
	 * @param objetoContratacao
	 */
	public void setObjetoContratacao(String objetoContratacao) {

		this.objetoContratacao = objetoContratacao;
	}

	/**
	 * Retorna o valor do atributo <code>valorContrato</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorContrato() {

		return valorContrato;
	}

	/**
	 * Define o valor do atributo <code>valorContrato</code>.
	 *
	 * @param valorContrato
	 */
	public void setValorContrato(BigDecimal valorContrato) {

		this.valorContrato = valorContrato;
	}

	/**
	 * Retorna o valor do atributo <code>valorTotalContrato</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorTotalContrato() {

		if (this.aditivos == null || this.aditivos.isEmpty()) {
			return this.valorContrato;
		} else {
			this.valorTotalContrato = this.valorContrato;
			for (EventoAditivo aditivo : this.aditivos) {
				if (aditivo.getAditivoDeValor()
						&& aditivo
								.getTipoAlteracaoValor()
								.getCodigo()
								.equals(DominioContrato.CODIGO_TIPO_ALTERACAO_VALOR_ACRESCIMO)) {
					this.valorTotalContrato = this.valorTotalContrato
							.add(aditivo.getValorAditivo());
				} else if (aditivo.getAditivoDeValor()
						&& aditivo
								.getTipoAlteracaoValor()
								.getCodigo()
								.equals(DominioContrato.CODIGO_TIPO_ALTERACAO_VALOR_SUPRESSAO)) {
					this.valorTotalContrato = this.valorTotalContrato
							.subtract(aditivo.getValorAditivo());
				}
			}
		}
		return this.valorTotalContrato;
	}

	/**
	 * Define o valor do atributo <code>valorTotalContrato</code>.
	 *
	 * @param valorTotalContrato
	 */
	public void setValorTotalContrato(BigDecimal valorTotalContrato) {

		this.valorTotalContrato = valorTotalContrato;
	}

	/**
	 * Retorna o valor do atributo <code>dominioFormaPagamentoContrato</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioFormaPagamentoContrato() {

		return dominioFormaPagamentoContrato;
	}

	/**
	 * Define o valor do atributo <code>dominioFormaPagamentoContrato</code>.
	 *
	 * @param dominioFormaPagamentoContrato
	 */
	public void setDominioFormaPagamentoContrato(
			Dominio dominioFormaPagamentoContrato) {

		this.dominioFormaPagamentoContrato = dominioFormaPagamentoContrato;
	}

	/**
	 * Retorna o valor do atributo <code>numeroParcela</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getNumeroParcela() {

		return numeroParcela;
	}

	/**
	 * Define o valor do atributo <code>numeroParcela</code>.
	 *
	 * @param numeroParcela
	 */
	public void setNumeroParcela(Long numeroParcela) {

		this.numeroParcela = numeroParcela;
	}

	/**
	 * Retorna o valor do atributo <code>ctrtValoresPagamento</code>
	 *
	 * @return <code>Collection<ContratoValorPagamento></code>
	 */
	public Collection<ContratoValorPagamento> getCtrtValoresPagamento() {

		return ctrtValoresPagamento;
	}

	/**
	 * Define o valor do atributo <code>ctrtValoresPagamento</code>.
	 *
	 * @param ctrtValoresPagamento
	 */
	public void setCtrtValoresPagamento(
			Collection<ContratoValorPagamento> ctrtValoresPagamento) {

		this.ctrtValoresPagamento = ctrtValoresPagamento;
	}

	/**
	 * Retorna o valor do atributo <code>ctrtEmpenhos</code>
	 *
	 * @return <code>Collection<ContratoEmpenho></code>
	 */
	public Collection<ContratoEmpenho> getCtrtEmpenhos() {

		return ctrtEmpenhos;
	}

	/**
	 * Define o valor do atributo <code>ctrtEmpenhos</code>.
	 *
	 * @param ctrtEmpenhos
	 */
	public void setCtrtEmpenhos(Collection<ContratoEmpenho> ctrtEmpenhos) {

		this.ctrtEmpenhos = ctrtEmpenhos;
	}

	/**
	 * Retorna o valor do atributo <code>aditivos</code>
	 *
	 * @return <code>Collection<EventoAditivo></code>
	 */
	public Collection<EventoAditivo> getAditivos() {

		return aditivos;
	}

	/**
	 * Define o valor do atributo <code>aditivos</code>.
	 *
	 * @param aditivos
	 */
	public void setAditivos(Collection<EventoAditivo> aditivos) {

		this.aditivos = aditivos;
	}

	/**
	 * Retorna o valor do atributo <code>valorTotalEmpenhado</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorTotalEmpenhado() {

		return valorTotalEmpenhado;
	}

	/**
	 * Define o valor do atributo <code>valorTotalEmpenhado</code>.
	 *
	 * @param valorTotalEmpenhado
	 */
	public void setValorTotalEmpenhado(BigDecimal valorTotalEmpenhado) {

		this.valorTotalEmpenhado = valorTotalEmpenhado;
	}

	/**
	 * Retorna o valor do atributo <code>numeroAnoEditalLicitacao</code>
	 *
	 * @return <code>Long</code>
	 */
	public String getNumeroAnoEditalLicitacao() {

		return numeroAnoEditalLicitacao;
	}

	/**
	 * Define o valor do atributo <code>numeroAnoEditalLicitacao</code>.
	 *
	 * @param numeroAnoEditalLicitacao
	 */
	public void setNumeroAnoEditalLicitacao(String numeroAnoEditalLicitacao) {

		this.numeroAnoEditalLicitacao = numeroAnoEditalLicitacao;
	}

	/**
	 * Retorna o valor do atributo <code>dataPublicacaoLicitacao</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataPublicacaoLicitacao() {

		return dataPublicacaoLicitacao;
	}

	/**
	 * Define o valor do atributo <code>dataPublicacaoLicitacao</code>.
	 *
	 * @param dataPublicacaoLicitacao
	 */
	public void setDataPublicacaoLicitacao(Calendar dataPublicacaoLicitacao) {

		this.dataPublicacaoLicitacao = dataPublicacaoLicitacao;
	}

	/**
	 * Retorna o valor do atributo <code>dominioModalidadeLicitacao</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioModalidadeLicitacao() {

		return dominioModalidadeLicitacao;
	}

	/**
	 * Define o valor do atributo <code>dominioModalidadeLicitacao</code>.
	 *
	 * @param dominioModalidadeLicitacao
	 */
	public void setDominioModalidadeLicitacao(Dominio dominioModalidadeLicitacao) {

		this.dominioModalidadeLicitacao = dominioModalidadeLicitacao;
	}

	/**
	 * Retorna o valor do atributo <code>dominioTipoLicitacao</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoLicitacao() {

		return dominioTipoLicitacao;
	}

	/**
	 * Define o valor do atributo <code>dominioTipoLicitacao</code>.
	 *
	 * @param dominioTipoLicitacao
	 */
	public void setDominioTipoLicitacao(Dominio dominioTipoLicitacao) {

		this.dominioTipoLicitacao = dominioTipoLicitacao;
	}

	/**
	 * Retorna o valor do atributo <code>ctrtItens</code>
	 *
	 * @return <code>Collection<ContratoItem></code>
	 */
	public Collection<ContratoItem> getCtrtItens() {

		return ctrtItens;
	}

	/**
	 * Define o valor do atributo <code>ctrtItens</code>.
	 *
	 * @param ctrtItens
	 */
	public void setCtrtItens(Collection<ContratoItem> ctrtItens) {

		this.ctrtItens = ctrtItens;
	}

	/**
	 * Retorna o valor do atributo <code>ctrtEnvolvidos</code>
	 *
	 * @return <code>Collection<ContratoEnvolvido></code>
	 */
	public Collection<ContratoEnvolvido> getCtrtEnvolvidos() {

		return ctrtEnvolvidos;
	}

	/**
	 * Define o valor do atributo <code>ctrtEnvolvidos</code>.
	 *
	 * @param ctrtEnvolvidos
	 */
	public void setCtrtEnvolvidos(Collection<ContratoEnvolvido> ctrtEnvolvidos) {

		this.ctrtEnvolvidos = ctrtEnvolvidos;
	}

	/**
	 * @return the ordensFornecimento
	 */
	public Collection<OrdemFornecimento> getOrdensFornecimento() {

		return ordensFornecimento;
	}

	/**
	 * @param ordensFornecimento
	 *            the ordensFornecimento to set
	 */
	public void setOrdensFornecimento(Collection<OrdemFornecimento> ordensFornecimento) {

		this.ordensFornecimento = ordensFornecimento;
	}

	/**
	 * Retorna o valor do atributo <code>dominioModalidadeGarantia</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioModalidadeGarantia() {

		return dominioModalidadeGarantia;
	}

	/**
	 * Define o valor do atributo <code>dominioModalidadeGarantia</code>.
	 *
	 * @param dominioModalidadeGarantia
	 */
	public void setDominioModalidadeGarantia(Dominio dominioModalidadeGarantia) {

		this.dominioModalidadeGarantia = dominioModalidadeGarantia;
	}

	/**
	 * Retorna o valor do atributo <code>dataVigenciaGarantia</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataVigenciaGarantia() {

		return dataVigenciaGarantia;
	}

	/**
	 * Define o valor do atributo <code>dataVigenciaGarantia</code>.
	 *
	 * @param dataVigenciaGarantia
	 */
	public void setDataVigenciaGarantia(Calendar dataVigenciaGarantia) {

		this.dataVigenciaGarantia = dataVigenciaGarantia;
	}

	/**
	 * Retorna o valor do atributo <code>valorGarantia</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorGarantia() {

		return valorGarantia;
	}

	/**
	 * Define o valor do atributo <code>valorGarantia</code>.
	 *
	 * @param valorGarantia
	 */
	public void setValorGarantia(BigDecimal valorGarantia) {

		this.valorGarantia = valorGarantia;
	}

	/**
	 * Retorna o valor do atributo <code>percentualGarantia</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getPercentualGarantia() {

		return percentualGarantia;
	}

	/**
	 * Define o valor do atributo <code>percentualGarantia</code>.
	 *
	 * @param percentualGarantia
	 */
	public void setPercentualGarantia(BigDecimal percentualGarantia) {

		this.percentualGarantia = percentualGarantia;
	}

	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<ContratoDocumento></code>
	 */
	public Collection<ContratoDocumento> getDocumentos() {

		return documentos;
	}

	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<ContratoDocumento> documentos) {

		this.documentos = documentos;
	}

	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<ContratoObservacao></code>
	 */
	public Collection<ContratoObservacao> getObservacoes() {

		return observacoes;
	}

	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<ContratoObservacao> observacoes) {

		this.observacoes = observacoes;
	}

	public TermoReferencia getTermoReferencia() {
		return termoReferencia;
	}

	public void setTermoReferencia(TermoReferencia termoReferencia) {
		this.termoReferencia = termoReferencia;
	}

	public Long getRascunho() {
		return rascunho;
	}
	
	public void setRascunho(Long rascunho) {
		this.rascunho = rascunho;
	}
}
