package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.model.Dominio;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 19/01/2016 - 14:04:42
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Audited
@AuditTable(value = "cnt_cn_empenho_aud")
@Entity
@Table(name = "cnt_cn_empenho")
@JsonIgnoreProperties({ "$checked" })
public class ContratoEmpenho extends ContratoEvento {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

	/** Atributo numeroEmpenho. */
	@Column(length = 30, nullable=false)
	@JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.EventoLiberacaoPagamentoView.class, ViewsContrato.ContratoEmpenhoView.class, ViewsContrato.ContratoEmpenhoEditView.class })
	private String numeroEmpenho;

	/** Atributo data. */
    @Temporal(TemporalType.TIMESTAMP)
    @Column(nullable=false)
    @JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.ContratoEmpenhoEditView.class})
    private Calendar data;

    /** Atributo dominioFinalidadeEmpenho. */
    @ManyToOne(fetch = FetchType.LAZY, optional=false)
    @JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.ContratoEmpenhoEditView.class })
    private Dominio dominioFinalidadeEmpenho;

    /** Atributo dominioCategoriaEconomica. */
    @ManyToOne(fetch = FetchType.LAZY, optional=false)
    @JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.ContratoEmpenhoEditView.class })
    private Dominio dominioCategoriaEconomica;

    /** Atributo classificacaoDespesa. */
	@Column(length = 30)
	@JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.ContratoEmpenhoEditView.class})
	private String classificacaoDespesa;

	/** Atributo valorEmpenhado. */
    @Column(precision = 20, scale = 4, nullable=false)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.ContratoEventoEditView.class, ViewsContrato.ContratoEmpenhoView.class, ViewsContrato.ContratoEmpenhoEditView.class})
    private BigDecimal valorEmpenhado;

    /** Atributo contratoEmpenho. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contratoEmpenho;

	/** Atributo contratoEmpenhoInativo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contratoEmpenhoInativo;

	@Column(length = 300)
	@JsonView({ ViewsContrato.ContratoEmpenhoEditView.class })
	private String justificativaEmpenho;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contratoEmpenho", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.ContratoEmpenhoEditView.class, ViewsContrato.ContratoEditView.class })
	private Collection<ContratoEmpenhoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contratoEmpenho", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.ContratoEmpenhoEditView.class, ViewsContrato.ContratoEditView.class })
	private Collection<ContratoEmpenhoObservacao> observacoes;

	/**
	 * Retorna o valor do atributo <code>numeroEmpenho</code>
	 *
	 * @return <code>String</code>
	 */
	public String getNumeroEmpenho() {

		return numeroEmpenho;
	}


	/**
	 * Define o valor do atributo <code>numeroEmpenho</code>.
	 *
	 * @param numeroEmpenho
	 */
	public void setNumeroEmpenho(String numeroEmpenho) {

		this.numeroEmpenho = numeroEmpenho;
	}


	/**
	 * Retorna o valor do atributo <code>data</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getData() {

		return data;
	}


	/**
	 * Define o valor do atributo <code>data</code>.
	 *
	 * @param data
	 */
	public void setData(Calendar data) {

		this.data = data;
	}


	/**
	 * Retorna o valor do atributo <code>dominioFinalidadeEmpenho</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioFinalidadeEmpenho() {

		return dominioFinalidadeEmpenho;
	}


	/**
	 * Define o valor do atributo <code>dominioFinalidadeEmpenho</code>.
	 *
	 * @param dominioFinalidadeEmpenho
	 */
	public void setDominioFinalidadeEmpenho(Dominio dominioFinalidadeEmpenho) {

		this.dominioFinalidadeEmpenho = dominioFinalidadeEmpenho;
	}


	/**
	 * Retorna o valor do atributo <code>dominioCategoriaEconomica</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioCategoriaEconomica() {

		return dominioCategoriaEconomica;
	}


	/**
	 * Define o valor do atributo <code>dominioCategoriaEconomica</code>.
	 *
	 * @param dominioCategoriaEconomica
	 */
	public void setDominioCategoriaEconomica(Dominio dominioCategoriaEconomica) {

		this.dominioCategoriaEconomica = dominioCategoriaEconomica;
	}


	/**
	 * Retorna o valor do atributo <code>classificacaoDespesa</code>
	 *
	 * @return <code>String</code>
	 */
	public String getClassificacaoDespesa() {

		return classificacaoDespesa;
	}


	/**
	 * Define o valor do atributo <code>classificacaoDespesa</code>.
	 *
	 * @param classificacaoDespesa
	 */
	public void setClassificacaoDespesa(String classificacaoDespesa) {

		this.classificacaoDespesa = classificacaoDespesa;
	}


	/**
	 * Retorna o valor do atributo <code>valorEmpenhado</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorEmpenhado() {

		return valorEmpenhado;
	}


	/**
	 * Define o valor do atributo <code>valorEmpenhado</code>.
	 *
	 * @param valorEmpenhado
	 */
	public void setValorEmpenhado(BigDecimal valorEmpenhado) {

		this.valorEmpenhado = valorEmpenhado;
	}



	/**
	 * Retorna o valor do atributo <code>contratoEmpenho</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContratoEmpenho() {

		return contratoEmpenho;
	}



	/**
	 * Define o valor do atributo <code>contratoEmpenho</code>.
	 *
	 * @param contratoEmpenho
	 */
	public void setContratoEmpenho(Contrato contratoEmpenho) {

		this.contratoEmpenho = contratoEmpenho;
	}



	/**
	 * Retorna o valor do atributo <code>contratoEmpenhoInativo</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContratoEmpenhoInativo() {

		return contratoEmpenhoInativo;
	}



	/**
	 * Define o valor do atributo <code>contratoEmpenhoInativo</code>.
	 *
	 * @param contratoEmpenhoInativo
	 */
	public void setContratoEmpenhoInativo(Contrato contratoEmpenhoInativo) {

		this.contratoEmpenhoInativo = contratoEmpenhoInativo;
	}


	public String getJustificativaEmpenho() {
		return justificativaEmpenho;
	}


	public void setJustificativaEmpenho(String justificativaEmpenho) {
		this.justificativaEmpenho = justificativaEmpenho;
	}


	public Collection<ContratoEmpenhoDocumento> getDocumentos() {
		return documentos;
	}


	public void setDocumentos(Collection<ContratoEmpenhoDocumento> documentos) {
		this.documentos = documentos;
	}


	public Collection<ContratoEmpenhoObservacao> getObservacoes() {
		return observacoes;
	}


	public void setObservacoes(Collection<ContratoEmpenhoObservacao> observacoes) {
		this.observacoes = observacoes;
	}

}
