package br.com.centralit.api.model;

import java.math.BigDecimal;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 19/01/2016 - 16:29:12
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Audited
@AuditTable(value = "cnt_cn_item_aud")
@Entity
@Table(name="cnt_cn_item")
@JsonIgnoreProperties({ "$checked" })
public class ContratoItem extends PersistentObjectAudit{

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Atributo id. */
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @JsonView({ Views.GenericView.class })
    private Long id;

	/** Atributo contrato. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contrato;

	/** Atributo contratoInativo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contratoInativo;

    /** Atributo dominioFinalidadeEmpenho. */
    @ManyToOne(fetch = FetchType.LAZY, optional=false)
    @JsonView({ ViewsContrato.ContratoEditView.class })
    private EstudoTecnico estudoTecnico;

	/** Atributo estudoTecnicoBemServicoSolucao. */
	@ManyToOne(fetch = FetchType.LAZY, optional=true)
	@JoinColumn(name = "etbemservicosolucao_id")
	@JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.OrdemFornecimentoEditView.class, ViewsContrato.ContratoItemAutoCompleteView.class, ViewsContrato.CronogramaExecucaoEditView.class })
	private EstudoTecnicoBemServicoSolucao estudoTecnicoBemServicoSolucao;

	/** Atributo quantidade. */
	@Column(nullable=false)
	@JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.OrdemFornecimentoEditView.class, ViewsContrato.ContratoItemAutoCompleteView.class })
	private Long quantidade;

	/** Atributo valorUnitarioItem. */
    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.ContratoEditView.class, ViewsContrato.OrdemFornecimentoEditView.class, ViewsContrato.ContratoItemAutoCompleteView.class })
    private BigDecimal valorUnitarioItem;

    /** Atributo valorTotalItem. */
    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.ContratoEditView.class })
    private BigDecimal valorTotalItem;


	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}


	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}


	/**
	 * Retorna o valor do atributo <code>contrato</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContrato() {

		return contrato;
	}


	/**
	 * Define o valor do atributo <code>contrato</code>.
	 *
	 * @param contrato
	 */
	public void setContrato(Contrato contrato) {

		this.contrato = contrato;
	}


	/**
	 * Retorna o valor do atributo <code>contratoInativo</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContratoInativo() {

		return contratoInativo;
	}


	/**
	 * Define o valor do atributo <code>contratoInativo</code>.
	 *
	 * @param contratoInativo
	 */
	public void setContratoInativo(Contrato contratoInativo) {

		this.contratoInativo = contratoInativo;
	}


	/**
	 * Retorna o valor do atributo <code>estudoTecnico</code>
	 *
	 * @return <code>EstudoTecnico</code>
	 */
	public EstudoTecnico getEstudoTecnico() {

		return estudoTecnico;
	}


	/**
	 * Define o valor do atributo <code>estudoTecnico</code>.
	 *
	 * @param estudoTecnico
	 */
	public void setEstudoTecnico(EstudoTecnico estudoTecnico) {

		this.estudoTecnico = estudoTecnico;
	}


	/**
	 * Retorna o valor do atributo <code>estudoTecnicoBemServicoSolucao</code>
	 *
	 * @return <code>EstudoTecnicoBemServicoSolucao</code>
	 */
	public EstudoTecnicoBemServicoSolucao getEstudoTecnicoBemServicoSolucao() {

		return estudoTecnicoBemServicoSolucao;
	}


	/**
	 * Define o valor do atributo <code>estudoTecnicoBemServicoSolucao</code>.
	 *
	 * @param estudoTecnicoBemServicoSolucao
	 */
	public void setEstudoTecnicoBemServicoSolucao(EstudoTecnicoBemServicoSolucao estudoTecnicoBemServicoSolucao) {

		this.estudoTecnicoBemServicoSolucao = estudoTecnicoBemServicoSolucao;
	}


	/**
	 * Retorna o valor do atributo <code>quantidade</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getQuantidade() {

		return quantidade;
	}


	/**
	 * Define o valor do atributo <code>quantidade</code>.
	 *
	 * @param quantidade
	 */
	public void setQuantidade(Long quantidade) {

		this.quantidade = quantidade;
	}


	/**
	 * Retorna o valor do atributo <code>valorUnitarioItem</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorUnitarioItem() {

		return valorUnitarioItem;
	}


	/**
	 * Define o valor do atributo <code>valorUnitarioItem</code>.
	 *
	 * @param valorUnitarioItem
	 */
	public void setValorUnitarioItem(BigDecimal valorUnitarioItem) {

		this.valorUnitarioItem = valorUnitarioItem;
	}


	/**
	 * Retorna o valor do atributo <code>valorTotalItem</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorTotalItem() {

		return valorTotalItem;
	}


	/**
	 * Define o valor do atributo <code>valorTotalItem</code>.
	 *
	 * @param valorTotalItem
	 */
	public void setValorTotalItem(BigDecimal valorTotalItem) {

		this.valorTotalItem = valorTotalItem;
	}

}
