package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 22/01/2016 - 14:07:24
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Audited
@AuditTable(value = "cnt_cn_valorpagamento_aud")
@Entity
@Table(name = "cnt_cn_valorpagamento")
@JsonIgnoreProperties({ "$checked"})
public class ContratoValorPagamento extends PersistentObjectAuditOrganizacao {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Atributo id. */
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @JsonView({ Views.GenericView.class })
    private Long id;

	/** Atributo contrato. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contrato;

	/** Atributo contratoInativo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contratoInativo;

    /** Atributo valorParcela. */
    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.ContratoEditView.class })
    private BigDecimal valorParcela;

    /** Atributo dataVencimento. */
	@Column(nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsContrato.ContratoEditView.class })
    private Calendar dataVencimento;


	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}


	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}


	/**
	 * Retorna o valor do atributo <code>contrato</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContrato() {

		return contrato;
	}


	/**
	 * Define o valor do atributo <code>contrato</code>.
	 *
	 * @param contrato
	 */
	public void setContrato(Contrato contrato) {

		this.contrato = contrato;
	}


	/**
	 * Retorna o valor do atributo <code>contratoInativo</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContratoInativo() {

		return contratoInativo;
	}


	/**
	 * Define o valor do atributo <code>contratoInativo</code>.
	 *
	 * @param contratoInativo
	 */
	public void setContratoInativo(Contrato contratoInativo) {

		this.contratoInativo = contratoInativo;
	}


	/**
	 * Retorna o valor do atributo <code>valorParcela</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorParcela() {

		return valorParcela;
	}


	/**
	 * Define o valor do atributo <code>valorParcela</code>.
	 *
	 * @param valorParcela
	 */
	public void setValorParcela(BigDecimal valorParcela) {

		this.valorParcela = valorParcela;
	}


	/**
	 * Retorna o valor do atributo <code>dataVencimento</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataVencimento() {

		return dataVencimento;
	}


	/**
	 * Define o valor do atributo <code>dataVencimento</code>.
	 *
	 * @param dataVencimento
	 */
	public void setDataVencimento(Calendar dataVencimento) {

		this.dataVencimento = dataVencimento;
	}

}
