package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

/**
 * @author luis.camargo
 *
 */
@Audited
@AuditTable(value = "cnt_cronogramaexecucao_aud")
@Entity
@Table(name="cnt_cronogramaexecucao")
public class CronogramaExecucao extends PersistentObjectAuditOrganizacao {


	private static final long serialVersionUID = 4447061172675458262L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	@Column(length = 10, nullable=false)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private String codigo;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class, ViewsContrato.ContratoAutoCompleteView.class })
	private Contrato contrato;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "cronogramaExecucao", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private Collection<CronogramaExecucaoItem> itensCronogramaExecucao;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "cronogramaExecucao", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private Collection<CronogramaExecucaoDocumento> documentos;

	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "cronogramaExecucao", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private Collection<CronogramaExecucaoObservacao> observacoes;

    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private BigDecimal valorTotal;

	@Override
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getCodigo() {
		return codigo;
	}

	public void setCodigo(String codigo) {
		this.codigo = codigo;
	}

	public Contrato getContrato() {
		return contrato;
	}

	public void setContrato(Contrato contrato) {
		this.contrato = contrato;
	}

	public Collection<CronogramaExecucaoItem> getItensCronogramaExecucao() {
		return itensCronogramaExecucao;
	}

	public void setItensCronogramaExecucao(
			Collection<CronogramaExecucaoItem> itensCronogramaExecucao) {
		this.itensCronogramaExecucao = itensCronogramaExecucao;
	}

	public Collection<CronogramaExecucaoDocumento> getDocumentos() {
		return documentos;
	}

	public void setDocumentos(Collection<CronogramaExecucaoDocumento> documentos) {
		this.documentos = documentos;
	}

	public Collection<CronogramaExecucaoObservacao> getObservacoes() {
		return observacoes;
	}

	public void setObservacoes(Collection<CronogramaExecucaoObservacao> observacoes) {
		this.observacoes = observacoes;
	}

	public BigDecimal getValorTotal() {
		return valorTotal;
	}

	public void setValorTotal(BigDecimal valorTotal) {
		this.valorTotal = valorTotal;
	}

}
