package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;
import br.com.centralit.framework.util.UtilColecao;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

/**
 * @author luis.camargo
 *
 */
@Audited
@AuditTable(value = "cnt_ce_item_aud")
@Entity
@Table(name="cnt_ce_item")
@JsonIgnoreProperties({ "$checked", "$index", "numeroParcelas" })
public class CronogramaExecucaoItem extends PersistentObjectAudit {

	private static final long serialVersionUID = 2875055351555925314L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	@Column(nullable=false)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private boolean pagamentoParcelado;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private ContratoItem contratoItem;

	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private CronogramaExecucao cronogramaExecucao;
	
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private CronogramaExecucao cronogramaExecucaoInativo;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "cronogramaExecucaoItem", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private Collection<CronogramaExecucaoItemParcela> parcelas;
	
	@Transient
	@JsonDeserialize(using = MoneyDeserializer.class)
	private BigDecimal valorTotalItem; 
	
	@Override
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public boolean isPagamentoParcelado() {
		return pagamentoParcelado;
	}

	public void setPagamentoParcelado(boolean pagamentoParcelado) {
		this.pagamentoParcelado = pagamentoParcelado;
	}

	public ContratoItem getContratoItem() {
		return contratoItem;
	}

	public void setContratoItem(ContratoItem contratoItem) {
		this.contratoItem = contratoItem;
	}

	public CronogramaExecucao getCronogramaExecucao() {
		return cronogramaExecucao;
	}

	public void setCronogramaExecucao(CronogramaExecucao cronogramaExecucao) {
		this.cronogramaExecucao = cronogramaExecucao;
	}

	public CronogramaExecucao getCronogramaExecucaoInativo() {
		return cronogramaExecucaoInativo;
	}
	
	public void setCronogramaExecucaoInativo(CronogramaExecucao cronogramaExecucaoInativo) {
		this.cronogramaExecucaoInativo = cronogramaExecucaoInativo;
	}
	
	public Collection<CronogramaExecucaoItemParcela> getParcelas() {
		return parcelas;
	}

	public void setParcelas(Collection<CronogramaExecucaoItemParcela> parcelas) {
		this.parcelas = parcelas;
	}

	public BigDecimal getValorTotalItem() {
		if (!UtilColecao.isVazio(parcelas)){
			valorTotalItem = BigDecimal.ZERO;
			for (CronogramaExecucaoItemParcela parcela : parcelas) {
				valorTotalItem.add(parcela.getValorTotalParcela());
			}
		}
		return valorTotalItem;
	}
	
	public void setValorTotalItem(BigDecimal valorTotalItem) {
		this.valorTotalItem = valorTotalItem;
	}
}
