package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * @author luis.camargo
 *
 */
@Audited
@AuditTable(value = "cnt_cei_parcela_aud")
@Entity
@Table(name="cnt_cei_parcela")
@JsonIgnoreProperties({ "sequencia", "$checked", "$index" })
public class CronogramaExecucaoItemParcela extends PersistentObjectAudit {

	private static final long serialVersionUID = 5671794537593241328L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	@Column(nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private Calendar dataEntrega;

	@Column(nullable=false)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private Long quantidade;

    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private BigDecimal valorUnitario;
	
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private CronogramaExecucaoItem cronogramaExecucaoItem;
	
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JoinColumn(name = "ceiteminativo_id")
	private CronogramaExecucaoItem cronogramaExecucaoItemInativo;

	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.CronogramaExecucaoEditView.class })
	private BigDecimal valorTotalParcela;
	
	@Override
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Calendar getDataEntrega() {
		return dataEntrega;
	}

	public void setDataEntrega(Calendar dataEntrega) {
		this.dataEntrega = dataEntrega;
	}

	public Long getQuantidade() {
		return quantidade;
	}

	public void setQuantidade(Long quantidade) {
		this.quantidade = quantidade;
	}

	public BigDecimal getValorUnitario() {
		return valorUnitario;
	}

	public void setValorUnitario(BigDecimal valorUnitario) {
		this.valorUnitario = valorUnitario;
	}

	public CronogramaExecucaoItem getCronogramaExecucaoItem() {
		return cronogramaExecucaoItem;
	}

	public void setCronogramaExecucaoItem(CronogramaExecucaoItem cronogramaExecucaoItem) {
		this.cronogramaExecucaoItem = cronogramaExecucaoItem;
	}

	public CronogramaExecucaoItem getCronogramaExecucaoItemInativo() {
		return cronogramaExecucaoItemInativo;
	}

	public void setCronogramaExecucaoItemInativo(
			CronogramaExecucaoItem cronogramaExecucaoItemInativo) {
		this.cronogramaExecucaoItemInativo = cronogramaExecucaoItemInativo;
	}

	public BigDecimal getValorTotalParcela() {
		return valorTotalParcela;
	}
	
	public void setValorTotalParcela(BigDecimal valorTotalParcela) {
		this.valorTotalParcela = valorTotalParcela;
	}
}
