package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 24/02/2016 - 09:03:01
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_encerramentocontrato_aud")
@Entity
@Table(name="cnt_encerramentocontrato")
@JsonIgnoreProperties({ "$checked", "existePendencia" })
public class EncerramentoContrato extends PersistentObjectAudit {

    /** Atributo serialVersionUID. */
	private static final long serialVersionUID = -177989736011074198L;

	/** Atributo id. */
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @JsonView({ Views.GenericView.class })
    private Long id;

	/** Atributo contrato. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.EncerramentoContratoListView.class })
	private Contrato contrato;

	/** Atributo contratante. */
	@NotAudited
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private Organizacao contratante;

    /** Atributo codigo. */
	@Column(length = 30, unique = true, nullable = false)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
    private String numTermo;

	/** Atributo numeroAnoContrato. */
	@Transient
	@JsonView({ ViewsContrato.EncerramentoContratoListView.class })
	private String numTermoFormatado;

	/** Atributo dataEmissao referente à dataEncerramento da tela de cadastro */
	@Column(nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsContrato.EncerramentoContratoListView.class, ViewsContrato.ContratoEditView.class })
    private Calendar dataEmissao;

	@Column(length = 2000, nullable = false)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private String introducao;

	@Column(length = 2000, nullable = false)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private String motivo;

	@Column(length = 8000, nullable = false)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private String encaminhamentos;

	@Column(length = 2000, nullable = true)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private String pendencias;

	@Column(length = 2000, nullable = false)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private String decisao;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "encerramentoContrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private Collection<EncerramentoContratoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "encerramentoContrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EncerramentoContratoEditView.class })
	private Collection<EncerramentoContratoObservacao> observacoes;

	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}


	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}


	/**
	 * Retorna o valor do atributo <code>contrato</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContrato() {

		return contrato;
	}


	/**
	 * Define o valor do atributo <code>contrato</code>.
	 *
	 * @param contrato
	 */
	public void setContrato(Contrato contrato) {

		this.contrato = contrato;
	}



	/**
	 * Retorna o valor do atributo <code>contratante</code>
	 *
	 * @return <code>Organizacao</code>
	 */
	public Organizacao getContratante() {

		return contratante;
	}



	/**
	 * Define o valor do atributo <code>contratante</code>.
	 *
	 * @param contratante
	 */
	public void setContratante(Organizacao contratante) {

		this.contratante = contratante;
	}



	/**
	 * Retorna o valor do atributo <code>numTermo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getNumTermo() {

		return numTermo;
	}

	/**
	 * Retorna o valor do atributo <code>numeroAnoContrato</code> formatado com máscara
	 *
	 * @return <code>String</code>
	 */
	public String getNumTermoFormatado() {
		if (this.numTermo != null && this.numTermo.length() == 8) {
			this.numTermoFormatado = numTermo.substring(0, 4) + "/" + numTermo.substring(4, 8);
		}
		return this.numTermoFormatado;
	}



	/**
	 * Define o valor do atributo <code>numTermo</code>.
	 *
	 * @param numTermo
	 */
	public void setNumTermo(String numTermo) {

		this.numTermo = numTermo;
	}



	/**
	 * Retorna o valor do atributo <code>dataEmissao</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataEmissao() {

		return dataEmissao;
	}



	/**
	 * Define o valor do atributo <code>dataEmissao</code>.
	 *
	 * @param dataEmissao
	 */
	public void setDataEmissao(Calendar dataEmissao) {

		this.dataEmissao = dataEmissao;
	}



	/**
	 * Retorna o valor do atributo <code>introducao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getIntroducao() {

		return introducao;
	}



	/**
	 * Define o valor do atributo <code>introducao</code>.
	 *
	 * @param introducao
	 */
	public void setIntroducao(String introducao) {

		this.introducao = introducao;
	}



	/**
	 * Retorna o valor do atributo <code>motivo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getMotivo() {

		return motivo;
	}



	/**
	 * Define o valor do atributo <code>motivo</code>.
	 *
	 * @param motivo
	 */
	public void setMotivo(String motivo) {

		this.motivo = motivo;
	}



	/**
	 * Retorna o valor do atributo <code>encaminhamentos</code>
	 *
	 * @return <code>String</code>
	 */
	public String getEncaminhamentos() {

		return encaminhamentos;
	}



	/**
	 * Define o valor do atributo <code>encaminhamentos</code>.
	 *
	 * @param encaminhamentos
	 */
	public void setEncaminhamentos(String encaminhamentos) {

		this.encaminhamentos = encaminhamentos;
	}



	/**
	 * Retorna o valor do atributo <code>pendencias</code>
	 *
	 * @return <code>String</code>
	 */
	public String getPendencias() {

		return pendencias;
	}



	/**
	 * Define o valor do atributo <code>pendencias</code>.
	 *
	 * @param pendencias
	 */
	public void setPendencias(String pendencias) {

		this.pendencias = pendencias;
	}



	/**
	 * Retorna o valor do atributo <code>decisao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDecisao() {

		return decisao;
	}



	/**
	 * Define o valor do atributo <code>decisao</code>.
	 *
	 * @param decisao
	 */
	public void setDecisao(String decisao) {

		this.decisao = decisao;
	}



	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<EncerramentoContratoDocumento></code>
	 */
	public Collection<EncerramentoContratoDocumento> getDocumentos() {

		return documentos;
	}



	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<EncerramentoContratoDocumento> documentos) {

		this.documentos = documentos;
	}



	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<EncerramentoContratoObservacao></code>
	 */
	public Collection<EncerramentoContratoObservacao> getObservacoes() {

		return observacoes;
	}



	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<EncerramentoContratoObservacao> observacoes) {

		this.observacoes = observacoes;
	}

}
