package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 05/01/2016 - 14:04:42
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Audited
@AuditTable(value = "cnt_estudoTecnico_aud")
@Entity
@Table(name="cnt_estudoTecnico")
public class EstudoTecnico extends PersistentObjectAuditOrganizacao {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = -8798203126933021163L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	@Embedded
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class, Views.GenericView.class })
	private RegistroSequencial registroSequencial;

	/** Atributo numeroPortaria. */
	@Column(length = 30, nullable=false)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private String numeroPortaria;

	/** Atributo dataInstituicaoEquipe. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Calendar dataInstituicaoEquipe;

	/** Atributo integranteTecnico. */
	@ManyToOne(fetch = FetchType.LAZY, optional=false)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private Colaborador integranteTecnico;

	/** Atributo integranteRequisitante. */
	@ManyToOne(fetch = FetchType.LAZY, optional=false)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private Colaborador integranteRequisitante;

	/** Atributo integranteAdministrativo. */
	@ManyToOne(fetch = FetchType.LAZY, optional=false)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private Colaborador integranteAdministrativo;

	/** Atributo descricaoSolucao. */
	@Column(length = 1000, nullable=false)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private String descricaoSolucao;

	/** Atributo etRequisitosNegocio. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private Collection<EstudoTecnicoRequisitoNegocio> etRequisitosNegocio;

	/** Atributo requisitosTecnicos. */
	@Column(length = 1000, nullable=false)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private String requisitosTecnicos;

	/** Atributo requisitosCapacitacao. */
	@Column(length = 1000)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private String requisitosCapacitacao;

	/** Atributo etLevantamentoAlternativas. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Collection<EstudoTecnicoLevantamentoAlternativas> etLevantamentoAlternativas;

	/** Atributo descricaoJustificativaSolucao. */
	@Column(length = 1000, nullable=false)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private String descricaoJustificativaSolucao;

	/** Atributo roiJustificativaSolucao. */
	@Column(length = 1000)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private String roiJustificativaSolucao;

	/** Atributo etBensServicosSolucoes. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class, ViewsContrato.EstudoTecnicoAutoCompleteView.class, ViewsContrato.ContratoEditView.class })
	private Collection<EstudoTecnicoBemServicoSolucao> etBensServicosSolucoes;

	/** Atributo etBeneficiosEsperados. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class, ViewsContrato.TermoReferenciaEditView.class })
	private Collection<EstudoTecnicoBeneficioEsperado> etBeneficiosEsperados;

	/** Atributo descricaoNecessidadeAdqcaoAmbiente. */
	@Column(length = 1000, name = "descNecessidadeAdqcaoAmbiente")
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private String descricaoNecessidadeAdqcaoAmbiente;

	/** Atributo etRecursosMateriais. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Collection<EstudoTecnicoRecursoMaterial> etRecursosMateriais;

	/** Atributo etRecursosHumanos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Collection<EstudoTecnicoRecursoHumano> etRecursosHumanos;

	/** Atributo etAcaoContingencia. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Collection<EstudoTecnicoAcaoContingencia> etAcoesContingencia;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Collection<EstudoTecnicoObservacao> observacoes;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "estudoTecnico", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Collection<EstudoTecnicoDocumento> documentos;

	@Transient
	@JsonView({ ViewsContrato.EstudoTecnicoEditView.class })
	private Long rascunho;
	
	public RegistroSequencial getRegistroSequencial() {

		return registroSequencial;
	}

	public void setRegistroOcorrenciaSequencial(RegistroSequencial registroSequencial) {

		this.registroSequencial = registroSequencial;
	}

	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}


	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>numeroPortaria</code>
	 *
	 * @return <code>String</code>
	 */
	public String getNumeroPortaria() {

		return numeroPortaria;
	}


	/**
	 * Define o valor do atributo <code>numeroPortaria</code>.
	 *
	 * @param numeroPortaria
	 */
	public void setNumeroPortaria(String numeroPortaria) {

		this.numeroPortaria = numeroPortaria;
	}


	/**
	 * Retorna o valor do atributo <code>dataInstituicaoEquipe</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataInstituicaoEquipe() {

		return dataInstituicaoEquipe;
	}


	/**
	 * Define o valor do atributo <code>dataInstituicaoEquipe</code>.
	 *
	 * @param dataInstituicaoEquipe
	 */
	public void setDataInstituicaoEquipe(Calendar dataInstituicaoEquipe) {

		this.dataInstituicaoEquipe = dataInstituicaoEquipe;
	}

	/**
	 * Retorna o valor do atributo <code>descricaoSolucao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDescricaoSolucao() {

		return descricaoSolucao;
	}

	/**
	 * Define o valor do atributo <code>descricaoSolucao</code>.
	 *
	 * @param descricaoSolucao
	 */
	public void setDescricaoSolucao(String descricaoSolucao) {

		this.descricaoSolucao = descricaoSolucao;
	}

	/**
	 * Retorna o valor do atributo <code>etRequisitosNegocio</code>
	 *
	 * @return <code>Collection<EstudoTecnicoRequisitoNegocio></code>
	 */
	public Collection<EstudoTecnicoRequisitoNegocio> getEtRequisitosNegocio() {

		return etRequisitosNegocio;
	}

	/**
	 * Define o valor do atributo <code>etRequisitosNegocio</code>.
	 *
	 * @param etRequisitosNegocio
	 */
	public void setEtRequisitosNegocio(Collection<EstudoTecnicoRequisitoNegocio> etRequisitosNegocio) {

		this.etRequisitosNegocio = etRequisitosNegocio;
	}

	/**
	 * Retorna o valor do atributo <code>requisitosTecnicos</code>
	 *
	 * @return <code>String</code>
	 */
	public String getRequisitosTecnicos() {

		return requisitosTecnicos;
	}


	/**
	 * Define o valor do atributo <code>requisitosTecnicos</code>.
	 *
	 * @param requisitosTecnicos
	 */
	public void setRequisitosTecnicos(String requisitosTecnicos) {

		this.requisitosTecnicos = requisitosTecnicos;
	}


	/**
	 * Retorna o valor do atributo <code>requisitosCapacitacao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getRequisitosCapacitacao() {

		return requisitosCapacitacao;
	}


	/**
	 * Define o valor do atributo <code>requisitosCapacitacao</code>.
	 *
	 * @param requisitosCapacitacao
	 */
	public void setRequisitosCapacitacao(String requisitosCapacitacao) {

		this.requisitosCapacitacao = requisitosCapacitacao;
	}


	/**
	 * Retorna o valor do atributo <code>etLevantamentoAlternativas</code>
	 *
	 * @return <code>Collection<EstudoTecnicoLevantamentoAlternativas></code>
	 */
	public Collection<EstudoTecnicoLevantamentoAlternativas> getEtLevantamentoAlternativas() {

		return etLevantamentoAlternativas;
	}

	/**
	 * Define o valor do atributo <code>etLevantamentoAlternativas</code>.
	 *
	 * @param etLevantamentoAlternativas
	 */
	public void setEtLevantamentoAlternativas(Collection<EstudoTecnicoLevantamentoAlternativas> etLevantamentoAlternativas) {

		this.etLevantamentoAlternativas = etLevantamentoAlternativas;
	}

	/**
	 * Retorna o valor do atributo <code>descricaoJustificativaSolucao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDescricaoJustificativaSolucao() {

		return descricaoJustificativaSolucao;
	}


	/**
	 * Define o valor do atributo <code>descricaoJustificativaSolucao</code>.
	 *
	 * @param descricaoJustificativaSolucao
	 */
	public void setDescricaoJustificativaSolucao(String descricaoJustificativaSolucao) {

		this.descricaoJustificativaSolucao = descricaoJustificativaSolucao;
	}


	/**
	 * Retorna o valor do atributo <code>roiJustificativaSolucao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getRoiJustificativaSolucao() {

		return roiJustificativaSolucao;
	}


	/**
	 * Define o valor do atributo <code>roiJustificativaSolucao</code>.
	 *
	 * @param roiJustificativaSolucao
	 */
	public void setRoiJustificativaSolucao(String roiJustificativaSolucao) {

		this.roiJustificativaSolucao = roiJustificativaSolucao;
	}


	/**
	 * Retorna o valor do atributo <code>etBensServicosSolucoes</code>
	 *
	 * @return <code>Collection<EstudoTecnicoBemServicoSolucao></code>
	 */
	public Collection<EstudoTecnicoBemServicoSolucao> getEtBensServicosSolucoes() {

		return etBensServicosSolucoes;
	}

	/**
	 * Define o valor do atributo <code>etBensServicosSolucoes</code>.
	 *
	 * @param etBensServicosSolucoes
	 */
	public void setEtBensServicosSolucoes(Collection<EstudoTecnicoBemServicoSolucao> etBensServicosSolucoes) {

		this.etBensServicosSolucoes = etBensServicosSolucoes;
	}

	/**
	 * Retorna o valor do atributo <code>etBeneficiosEsperados</code>
	 *
	 * @return <code>Collection<EstudoTecnicoBeneficioEsperado></code>
	 */
	public Collection<EstudoTecnicoBeneficioEsperado> getEtBeneficiosEsperados() {

		return etBeneficiosEsperados;
	}

	/**
	 * Define o valor do atributo <code>etBeneficiosEsperados</code>.
	 *
	 * @param etBeneficiosEsperados
	 */
	public void setEtBeneficiosEsperados(Collection<EstudoTecnicoBeneficioEsperado> etBeneficiosEsperados) {

		this.etBeneficiosEsperados = etBeneficiosEsperados;
	}

	/**
	 * Retorna o valor do atributo <code>descricaoNecessidadeAdqcaoAmbiente</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDescricaoNecessidadeAdqcaoAmbiente() {

		return descricaoNecessidadeAdqcaoAmbiente;
	}

	/**
	 * Define o valor do atributo <code>descricaoNecessidadeAdqcaoAmbiente</code>.
	 *
	 * @param descricaoNecessidadeAdqcaoAmbiente
	 */
	public void setDescricaoNecessidadeAdqcaoAmbiente(String descricaoNecessidadeAdqcaoAmbiente) {

		this.descricaoNecessidadeAdqcaoAmbiente = descricaoNecessidadeAdqcaoAmbiente;
	}


	/**
	 * Retorna o valor do atributo <code>etRecursosMateriais</code>
	 *
	 * @return <code>Collection<EstudoTecnicoRecursoMaterial></code>
	 */
	public Collection<EstudoTecnicoRecursoMaterial> getEtRecursosMateriais() {

		return etRecursosMateriais;
	}


	/**
	 * Define o valor do atributo <code>etRecursosMateriais</code>.
	 *
	 * @param etRecursosMateriais
	 */
	public void setEtRecursosMateriais(Collection<EstudoTecnicoRecursoMaterial> etRecursosMateriais) {

		this.etRecursosMateriais = etRecursosMateriais;
	}


	/**
	 * Retorna o valor do atributo <code>etRecursosHumanos</code>
	 *
	 * @return <code>Collection<EstudoTecnicoRecursoHumano></code>
	 */
	public Collection<EstudoTecnicoRecursoHumano> getEtRecursosHumanos() {

		return etRecursosHumanos;
	}


	/**
	 * Define o valor do atributo <code>etRecursosHumanos</code>.
	 *
	 * @param etRecursosHumanos
	 */
	public void setEtRecursosHumanos(Collection<EstudoTecnicoRecursoHumano> etRecursosHumanos) {

		this.etRecursosHumanos = etRecursosHumanos;
	}


	/**
	 * Retorna o valor do atributo <code>etAcoesContingencia</code>
	 *
	 * @return <code>Collection<EstudoTecnicoAcaoContingencia></code>
	 */
	public Collection<EstudoTecnicoAcaoContingencia> getEtAcoesContingencia() {

		return etAcoesContingencia;
	}

	/**
	 * Define o valor do atributo <code>etAcoesContingencia</code>.
	 *
	 * @param etAcoesContingencia
	 */
	public void setEtAcoesContingencia(Collection<EstudoTecnicoAcaoContingencia> etAcoesContingencia) {

		this.etAcoesContingencia = etAcoesContingencia;
	}

	/**
	 * Retorna o valor do atributo <code>integranteTecnico</code>
	 *
	 * @return <code>Colaborador</code>
	 */
	public Colaborador getIntegranteTecnico() {

		return integranteTecnico;
	}


	/**
	 * Define o valor do atributo <code>integranteTecnico</code>.
	 *
	 * @param integranteTecnico
	 */
	public void setIntegranteTecnico(Colaborador integranteTecnico) {

		this.integranteTecnico = integranteTecnico;
	}


	/**
	 * Retorna o valor do atributo <code>integranteRequisitante</code>
	 *
	 * @return <code>Colaborador</code>
	 */
	public Colaborador getIntegranteRequisitante() {

		return integranteRequisitante;
	}


	/**
	 * Define o valor do atributo <code>integranteRequisitante</code>.
	 *
	 * @param integranteRequisitante
	 */
	public void setIntegranteRequisitante(Colaborador integranteRequisitante) {

		this.integranteRequisitante = integranteRequisitante;
	}


	/**
	 * Retorna o valor do atributo <code>integranteAdministrativo</code>
	 *
	 * @return <code>Colaborador</code>
	 */
	public Colaborador getIntegranteAdministrativo() {

		return integranteAdministrativo;
	}


	/**
	 * Define o valor do atributo <code>integranteAdministrativo</code>.
	 *
	 * @param integranteAdministrativo
	 */
	public void setIntegranteAdministrativo(Colaborador integranteAdministrativo) {

		this.integranteAdministrativo = integranteAdministrativo;
	}


	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<EstudoTecnicoObservacao></code>
	 */
	public Collection<EstudoTecnicoObservacao> getObservacoes() {

		return observacoes;
	}


	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<EstudoTecnicoObservacao> observacoes) {

		this.observacoes = observacoes;
	}


	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<EstudoTecnicoDocumento></code>
	 */
	public Collection<EstudoTecnicoDocumento> getDocumentos() {

		return documentos;
	}


	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<EstudoTecnicoDocumento> documentos) {

		this.documentos = documentos;
	}

	public Long getRascunho() {
		return rascunho;
	}
	
	public void setRascunho(Long rascunho) {
		this.rascunho = rascunho;
	}
}
