package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 09/03/2016 - 16:40:20
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_ev_aditivo_aud")
@Entity
@Table(name="cnt_ev_aditivo")
@JsonIgnoreProperties({ "$checked" })
public class EventoAditivo extends ContratoEvento {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

    @Embedded
	@JsonView({ Views.GenericView.class })
	private RegistroSequencial registroSequencial;

    /** Atributo dataEmissao. */
    @Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class, ViewsContrato.ContratoEditView.class })
	private Calendar dataEmissao;

	/** Atributo tipoAlteracao. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class })
    private Dominio tipoAlteracao;

	/** Atributo aditivoDePrazo. */
	@Column(nullable = false)
    @JsonView({ Views.GenericView.class })
	private Boolean aditivoDePrazo = Boolean.FALSE;

	/** Atributo aditivoDeValor. */
	@Column(nullable = false)
	@JsonView({ Views.GenericView.class })
	private Boolean aditivoDeValor = Boolean.FALSE;

	/** Atributo aditivoDeClausula. */
	@Column(nullable = false)
	@JsonView({ Views.GenericView.class })
	private Boolean aditivoDeClausula = Boolean.FALSE;

	/** Atributo dataEmissao. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class })
	private Calendar dataInicio;

	/** Atributo dataEmissao. */
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class })
	private Calendar dataFim;

	/** Atributo tipoObjeto. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
    @JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class })
    private Dominio tipoObjeto;

	/** Atributo tipoAlteracaoValor. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
    @JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class })
    private Dominio tipoAlteracaoValor;

	/** Atributo valorContrato. */
    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class })
    private BigDecimal valorAditivo;

    /** Atributo valorContrato. */
    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEventoEditView.class })
    private BigDecimal percentualAditivo;

    /** Atributo descricaoAditivo. */
	@Column(length = 5000, nullable = true)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class })
    private String descricaoAditivo;

	/** Atributo justificativa. */
	@Column(length = 2000, nullable = false)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class })
    private String justificativa;

	/** Atributo manifestacaoContratada. */
	@Column(length = 2000, nullable = true)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class })
    private String manifestacaoContratada;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoAditivo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEditView.class })
	private Collection<EventoAditivoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoAditivo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoAditivoEditView.class, ViewsContrato.ContratoEditView.class })
	private Collection<EventoAditivoObservacao> observacoes;

	/** Atributo contratoAditivo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contratoAditivo;

	/** Atributo contratoAditivoInativo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Contrato contratoAditivoInativo;


	/**
	 * Retorna o valor do atributo <code>registroSequencial</code>
	 *
	 * @return <code>RegistroSequencial</code>
	 */
	public RegistroSequencial getRegistroSequencial() {

		return registroSequencial;
	}


	/**
	 * Define o valor do atributo <code>registroSequencial</code>.
	 *
	 * @param registroSequencial
	 */
	public void setRegistroSequencial(RegistroSequencial registroSequencial) {

		this.registroSequencial = registroSequencial;
	}


	/**
	 * Retorna o valor do atributo <code>dataEmissao</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataEmissao() {

		return dataEmissao;
	}


	/**
	 * Define o valor do atributo <code>dataEmissao</code>.
	 *
	 * @param dataEmissao
	 */
	public void setDataEmissao(Calendar dataEmissao) {

		this.dataEmissao = dataEmissao;
	}


	/**
	 * Retorna o valor do atributo <code>tipoAlteracao</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getTipoAlteracao() {

		return tipoAlteracao;
	}


	/**
	 * Define o valor do atributo <code>tipoAlteracao</code>.
	 *
	 * @param tipoAlteracao
	 */
	public void setTipoAlteracao(Dominio tipoAlteracao) {

		this.tipoAlteracao = tipoAlteracao;
	}


	/**
	 * Retorna o valor do atributo <code>aditivoDePrazo</code>
	 *
	 * @return <code>Boolean</code>
	 */
	public Boolean getAditivoDePrazo() {

		return aditivoDePrazo;
	}


	/**
	 * Define o valor do atributo <code>aditivoDePrazo</code>.
	 *
	 * @param aditivoDePrazo
	 */
	public void setAditivoDePrazo(Boolean aditivoDePrazo) {

		this.aditivoDePrazo = aditivoDePrazo;
	}


	/**
	 * Retorna o valor do atributo <code>aditivoDeValor</code>
	 *
	 * @return <code>Boolean</code>
	 */
	public Boolean getAditivoDeValor() {

		return aditivoDeValor;
	}


	/**
	 * Define o valor do atributo <code>aditivoDeValor</code>.
	 *
	 * @param aditivoDeValor
	 */
	public void setAditivoDeValor(Boolean aditivoDeValor) {

		this.aditivoDeValor = aditivoDeValor;
	}


	/**
	 * Retorna o valor do atributo <code>aditivoDeClausula</code>
	 *
	 * @return <code>Boolean</code>
	 */
	public Boolean getAditivoDeClausula() {

		return aditivoDeClausula;
	}


	/**
	 * Define o valor do atributo <code>aditivoDeClausula</code>.
	 *
	 * @param aditivoDeClausula
	 */
	public void setAditivoDeClausula(Boolean aditivoDeClausula) {

		this.aditivoDeClausula = aditivoDeClausula;
	}


	/**
	 * Retorna o valor do atributo <code>dataInicio</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataInicio() {

		return dataInicio;
	}


	/**
	 * Define o valor do atributo <code>dataInicio</code>.
	 *
	 * @param dataInicio
	 */
	public void setDataInicio(Calendar dataInicio) {

		this.dataInicio = dataInicio;
	}


	/**
	 * Retorna o valor do atributo <code>dataFim</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataFim() {

		return dataFim;
	}


	/**
	 * Define o valor do atributo <code>dataFim</code>.
	 *
	 * @param dataFim
	 */
	public void setDataFim(Calendar dataFim) {

		this.dataFim = dataFim;
	}


	/**
	 * Retorna o valor do atributo <code>tipoObjeto</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getTipoObjeto() {

		return tipoObjeto;
	}


	/**
	 * Define o valor do atributo <code>tipoObjeto</code>.
	 *
	 * @param tipoObjeto
	 */
	public void setTipoObjeto(Dominio tipoObjeto) {

		this.tipoObjeto = tipoObjeto;
	}


	/**
	 * Retorna o valor do atributo <code>tipoAlteracaoValor</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getTipoAlteracaoValor() {

		return tipoAlteracaoValor;
	}


	/**
	 * Define o valor do atributo <code>tipoAlteracaoValor</code>.
	 *
	 * @param tipoAlteracaoValor
	 */
	public void setTipoAlteracaoValor(Dominio tipoAlteracaoValor) {

		this.tipoAlteracaoValor = tipoAlteracaoValor;
	}


	/**
	 * Retorna o valor do atributo <code>valorAditivo</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorAditivo() {

		return valorAditivo;
	}


	/**
	 * Define o valor do atributo <code>valorAditivo</code>.
	 *
	 * @param valorAditivo
	 */
	public void setValorAditivo(BigDecimal valorAditivo) {

		this.valorAditivo = valorAditivo;
	}


	/**
	 * Retorna o valor do atributo <code>percentualAditivo</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getPercentualAditivo() {

		return percentualAditivo;
	}


	/**
	 * Define o valor do atributo <code>percentualAditivo</code>.
	 *
	 * @param percentualAditivo
	 */
	public void setPercentualAditivo(BigDecimal percentualAditivo) {

		this.percentualAditivo = percentualAditivo;
	}



	/**
	 * Retorna o valor do atributo <code>descricaoAditivo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDescricaoAditivo() {

		return descricaoAditivo;
	}



	/**
	 * Define o valor do atributo <code>descricaoAditivo</code>.
	 *
	 * @param descricaoAditivo
	 */
	public void setDescricaoAditivo(String descricaoAditivo) {

		this.descricaoAditivo = descricaoAditivo;
	}


	/**
	 * Retorna o valor do atributo <code>justificativa</code>
	 *
	 * @return <code>String</code>
	 */
	public String getJustificativa() {

		return justificativa;
	}


	/**
	 * Define o valor do atributo <code>justificativa</code>.
	 *
	 * @param justificativa
	 */
	public void setJustificativa(String justificativa) {

		this.justificativa = justificativa;
	}


	/**
	 * Retorna o valor do atributo <code>manifestacaoContratada</code>
	 *
	 * @return <code>String</code>
	 */
	public String getManifestacaoContratada() {

		return manifestacaoContratada;
	}


	/**
	 * Define o valor do atributo <code>manifestacaoContratada</code>.
	 *
	 * @param manifestacaoContratada
	 */
	public void setManifestacaoContratada(String manifestacaoContratada) {

		this.manifestacaoContratada = manifestacaoContratada;
	}


	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<EventoAditivoDocumento></code>
	 */
	public Collection<EventoAditivoDocumento> getDocumentos() {

		return documentos;
	}


	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<EventoAditivoDocumento> documentos) {

		this.documentos = documentos;
	}


	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<EventoAditivoObservacao></code>
	 */
	public Collection<EventoAditivoObservacao> getObservacoes() {

		return observacoes;
	}


	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<EventoAditivoObservacao> observacoes) {

		this.observacoes = observacoes;
	}



	/**
	 * Retorna o valor do atributo <code>contratoAditivo</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContratoAditivo() {

		return contratoAditivo;
	}



	/**
	 * Define o valor do atributo <code>contratoAditivo</code>.
	 *
	 * @param contratoAditivo
	 */
	public void setContratoAditivo(Contrato contratoAditivo) {

		this.contratoAditivo = contratoAditivo;
	}



	/**
	 * Retorna o valor do atributo <code>contratoAditivoInativo</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContratoAditivoInativo() {

		return contratoAditivoInativo;
	}



	/**
	 * Define o valor do atributo <code>contratoAditivoInativo</code>.
	 *
	 * @param contratoAditivoInativo
	 */
	public void setContratoAditivoInativo(Contrato contratoAditivoInativo) {

		this.contratoAditivoInativo = contratoAditivoInativo;
	}

}
