package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 09/03/2016 - 18:07:56
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_ev_apostilamento_aud")
@Entity
@Table(name="cnt_ev_apostilamento")
@JsonIgnoreProperties({ "cnpjFornecedor" })
public class EventoApostilamento extends ContratoEvento {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

	@Embedded
	@JsonView({ ViewsContrato.EventoApostilamentoEditView.class })
	private RegistroSequencial registroSequencial;

	/** Atributo dataEmissao. */
    @Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
    @JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsContrato.EventoApostilamentoEditView.class, ViewsContrato.ContratoEditView.class })
	private Calendar dataEmissao;

    /** Atributo justificativa. */
	@Column(length = 2000, nullable = false)
	@JsonView({ ViewsContrato.EventoApostilamentoEditView.class })
    private String justificativa;

	/** Atributo justificativa. */
	@Column(length = 2000, nullable = false)
	@JsonView({ ViewsContrato.EventoApostilamentoEditView.class })
    private String registroAdministrativo;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoApostilamento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoApostilamentoEditView.class })
	private Collection<EventoApostilamentoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoApostilamento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoApostilamentoEditView.class })
	private Collection<EventoApostilamentoObservacao> observacoes;


	public RegistroSequencial getRegistroSequencial() {

		return registroSequencial;
	}



	public void setRegistroSequencial(RegistroSequencial registroSequencial) {

		this.registroSequencial = registroSequencial;
	}


	/**
	 * Retorna o valor do atributo <code>dataEmissao</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataEmissao() {

		return dataEmissao;
	}


	/**
	 * Define o valor do atributo <code>dataEmissao</code>.
	 *
	 * @param dataEmissao
	 */
	public void setDataEmissao(Calendar dataEmissao) {

		this.dataEmissao = dataEmissao;
	}


	/**
	 * Retorna o valor do atributo <code>justificativa</code>
	 *
	 * @return <code>String</code>
	 */
	public String getJustificativa() {

		return justificativa;
	}


	/**
	 * Define o valor do atributo <code>justificativa</code>.
	 *
	 * @param justificativa
	 */
	public void setJustificativa(String justificativa) {

		this.justificativa = justificativa;
	}


	/**
	 * Retorna o valor do atributo <code>registroAdministrativo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getRegistroAdministrativo() {

		return registroAdministrativo;
	}


	/**
	 * Define o valor do atributo <code>registroAdministrativo</code>.
	 *
	 * @param registroAdministrativo
	 */
	public void setRegistroAdministrativo(String registroAdministrativo) {

		this.registroAdministrativo = registroAdministrativo;
	}


	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<EventoApostilamentoDocumento></code>
	 */
	public Collection<EventoApostilamentoDocumento> getDocumentos() {

		return documentos;
	}


	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<EventoApostilamentoDocumento> documentos) {

		this.documentos = documentos;
	}


	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<EventoApostilamentoObservacao></code>
	 */
	public Collection<EventoApostilamentoObservacao> getObservacoes() {

		return observacoes;
	}


	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<EventoApostilamentoObservacao> observacoes) {

		this.observacoes = observacoes;
	}

}
