package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 09/03/2016 - 11:27:42
 *
 * @version 1.0.0
 *
 * @author erick.sato
 *
 */
@Audited
@AuditTable(value = "cnt_ev_lib_pagamento_aud")
@Entity
@Table(name = "cnt_ev_liberacao_pagamento")
@JsonIgnoreProperties({ "valorLiberadoPagamento", "valorExecutado", "percentualEmpenho", "liberarPagamentoValorContrata", "valorTotalOS", "valorGlosa" })
public class EventoLiberacaoPagamento extends ContratoEvento {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1L;

	@Column(length = 30, nullable = false, updatable = true)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class, Views.GenericView.class, ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private String numeroLiberacao;

	@Column(name = "emissao", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private Calendar dataLiberacao;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private ContratoEnvolvido responsavel;

	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private BigDecimal totalLiberadoOS;

	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private BigDecimal totalEmpenhoLiberado;

	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class, Views.GenericView.class })
	private BigDecimal totalLiberadoPagamento;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "liberacaoPagamento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private Collection<EventoLiberacaoPagamentoOrdemFornecimento> ordensFornecimento;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "liberacaoPagamento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private Collection<EventoLiberacaoPagamentoEmpenho> empenhos;

	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "liberacaoPagamento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private Collection<EventoLiberacaoPagamentoObservacao> observacoes;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "liberacaoPagamento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoLiberacaoPagamentoView.class })
	private Collection<EventoLiberacaoPagamentoDocumento> documentos;

	public String getNumeroLiberacao() {
		return numeroLiberacao;
	}

	public void setNumeroLiberacao(String numeroLiberacao) {
		this.numeroLiberacao = numeroLiberacao;
	}

	public Calendar getDataLiberacao() {
		return dataLiberacao;
	}

	public void setDataLiberacao(Calendar dataLiberacao) {
		this.dataLiberacao = dataLiberacao;
	}

	public ContratoEnvolvido getResponsavel() {
		return responsavel;
	}

	public void setResponsavel(ContratoEnvolvido responsavel) {
		this.responsavel = responsavel;
	}

	public Collection<EventoLiberacaoPagamentoOrdemFornecimento> getOrdensFornecimento() {
		return ordensFornecimento;
	}

	public void setOrdensFornecimento(
			Collection<EventoLiberacaoPagamentoOrdemFornecimento> ordensFornecimento) {
		this.ordensFornecimento = ordensFornecimento;
	}

	public Collection<EventoLiberacaoPagamentoEmpenho> getEmpenhos() {
		return empenhos;
	}

	public void setEmpenhos(Collection<EventoLiberacaoPagamentoEmpenho> empenhos) {
		this.empenhos = empenhos;
	}

	public Collection<EventoLiberacaoPagamentoObservacao> getObservacoes() {
		return observacoes;
	}

	public void setObservacoes(
			Collection<EventoLiberacaoPagamentoObservacao> observacoes) {
		this.observacoes = observacoes;
	}

	public Collection<EventoLiberacaoPagamentoDocumento> getDocumentos() {
		return documentos;
	}

	public void setDocumentos(Collection<EventoLiberacaoPagamentoDocumento> documentos) {
		this.documentos = documentos;
	}

	public BigDecimal getTotalEmpenhoLiberado() {
		return totalEmpenhoLiberado;
	}

	public void setTotalEmpenhoLiberado(BigDecimal totalEmpenhoLiberado) {
		this.totalEmpenhoLiberado = totalEmpenhoLiberado;
	}

	public BigDecimal getTotalLiberadoOS() {
		return totalLiberadoOS;
	}

	public void setTotalLiberadoOS(BigDecimal totalLiberadoOS) {
		this.totalLiberadoOS = totalLiberadoOS;
	}

	public BigDecimal getTotalLiberadoPagamento() {
		return totalLiberadoPagamento;
	}

	public void setTotalLiberadoPagamento(BigDecimal totalLiberadoPagamento) {
		this.totalLiberadoPagamento = totalLiberadoPagamento;
	}

}
