package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;


/**
 * 
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 09/03/2016 - 16:40:20
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_ev_penalidade_aud")
@Entity
@Table(name="cnt_ev_penalidade")
@JsonIgnoreProperties({ "$checked", "cnpjFornecedor" })
public class EventoPenalidade extends ContratoEvento {

	private static final long serialVersionUID = 6446962115083653467L;

	/** Atributo dataAtualizacao. */
    @Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsContrato.EventoPenalidadeEditView.class })
	private Calendar dataPenalizacao;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JsonView({ ViewsContrato.EventoPenalidadeEditView.class })
    private TermoReferenciaPenalidade termoReferenciaPenalidade;

	/** Atributo registroOcorrencia. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoPenalidade", cascade = CascadeType.ALL, orphanRemoval = true)
    @JsonView({ ViewsContrato.EventoPenalidadeEditView.class })
	private Collection<EventoPenalidadeRegistroOcorrencia> ocorrencias;

	/** Atributo justificativa. */
	@Column(length = 500, nullable = true)
	@JsonView({ ViewsContrato.EventoPenalidadeEditView.class })
    private String respostaContratada;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoPenalidade", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoPenalidadeEditView.class })
	private Collection<EventoPenalidadeDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoPenalidade", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoPenalidadeEditView.class })
	private Collection<EventoPenalidadeObservacao> observacoes;

	public Calendar getDataPenalizacao() {
		return dataPenalizacao;
	}

	public void setDataPenalizacao(Calendar dataPenalizacao) {
		this.dataPenalizacao = dataPenalizacao;
	}

	public TermoReferenciaPenalidade getTermoReferenciaPenalidade() {
		return termoReferenciaPenalidade;
	}

	public void setTermoReferenciaPenalidade(TermoReferenciaPenalidade termoReferenciaPenalidade) {
		this.termoReferenciaPenalidade = termoReferenciaPenalidade;
	}

	public Collection<EventoPenalidadeRegistroOcorrencia> getOcorrencias() {
		return ocorrencias;
	}
	
	public void setOcorrencias(Collection<EventoPenalidadeRegistroOcorrencia> ocorrencias) {
		this.ocorrencias = ocorrencias;
	}
	
	public String getRespostaContratada() {
		return respostaContratada;
	}

	public void setRespostaContratada(String respostaContratada) {
		this.respostaContratada = respostaContratada;
	}

	public Collection<EventoPenalidadeDocumento> getDocumentos() {
		return documentos;
	}

	public void setDocumentos(Collection<EventoPenalidadeDocumento> documentos) {
		this.documentos = documentos;
	}

	public Collection<EventoPenalidadeObservacao> getObservacoes() {
		return observacoes;
	}

	public void setObservacoes(Collection<EventoPenalidadeObservacao> observacoes) {
		this.observacoes = observacoes;
	}

}