package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.model.Dominio;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 *
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 09/03/2016 - 16:40:20
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_ev_recebimentoobjeto_aud")
@Entity
@Table(name = "cnt_ev_recebimentoobjeto")
@JsonIgnoreProperties({ "$checked", "$index", "originalElement"})
public class EventoRecebimentoObjeto extends ContratoEvento {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** Atributo numeroRecebimento. */
	@Embedded
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private RegistroSequencial registroSequencial;

	/** Atributo dataRecebimento. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Calendar dataRecebimento;

	/** Atributo tipoRecebimento. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Dominio tipoRecebimento;

	/** Atributo status. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Dominio tipoStatus;

	/** Atributo dominioTipoDocumento. */
	@ManyToOne(cascade=CascadeType.ALL, fetch=FetchType.LAZY)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Dominio tipoAvaliacaoPrazo;

	@ManyToOne(cascade=CascadeType.ALL, fetch=FetchType.LAZY)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Dominio tipoAvaliacaoAdesao;

	/** Atributo justificativa. */
	@Column(length = 2000, nullable = true)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private String justificativa;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRecebimentoObjeto", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Collection<EventoRecebimentoObjetoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRecebimentoObjeto", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRecebimentoObjetoEditView.class })
	private Collection<EventoRecebimentoObjetoObservacao> observacoes;

	public RegistroSequencial getRegistroSequencial() {

		return registroSequencial;
	}


	public void setRegistroSequencial(RegistroSequencial registroSequencial) {

		this.registroSequencial = registroSequencial;
	}

	/**
	 * Retorna o valor do atributo <code>dataRecebimento</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataRecebimento() {

		return dataRecebimento;
	}

	/**
	 * Define o valor do atributo <code>dataRecebimento</code>.
	 *
	 * @param dataRecebimento
	 */
	public void setDataRecebimento(Calendar dataRecebimento) {

		this.dataRecebimento = dataRecebimento;
	}

	/**
	 * Retorna o valor do atributo <code>tipoRecebimento</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getTipoRecebimento() {

		return tipoRecebimento;
	}

	/**
	 * Define o valor do atributo <code>tipoRecebimento</code>.
	 *
	 * @param tipoRecebimento
	 */
	public void setTipoRecebimento(Dominio tipoRecebimento) {

		this.tipoRecebimento = tipoRecebimento;
	}

	public Dominio getTipoStatus() {
		return tipoStatus;
	}


	public void setTipoStatus(Dominio tipoStatus) {
		this.tipoStatus = tipoStatus;
	}

	public Dominio getTipoAvaliacaoPrazo() {

		return tipoAvaliacaoPrazo;
	}


	public void setTipoAvaliacaoPrazo(Dominio tipoAvaliacaoPrazo) {

		this.tipoAvaliacaoPrazo = tipoAvaliacaoPrazo;
	}


	public Dominio getTipoAvaliacaoAdesao() {

		return tipoAvaliacaoAdesao;
	}


	public void setTipoAvaliacaoAdesao(Dominio tipoAvaliacaoAdesao) {

		this.tipoAvaliacaoAdesao = tipoAvaliacaoAdesao;
	}

	/**
	 * Retorna o valor do atributo <code>justificativa</code>
	 *
	 * @return <code>String</code>
	 */
	public String getJustificativa() {

		return justificativa;
	}

	/**
	 * Define o valor do atributo <code>justificativa</code>.
	 *
	 * @param justificativa
	 */
	public void setJustificativa(String justificativa) {

		this.justificativa = justificativa;
	}

	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<EventoRecebimentoObjetoDocumento></code>
	 */
	public Collection<EventoRecebimentoObjetoDocumento> getDocumentos() {

		return documentos;
	}

	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<EventoRecebimentoObjetoDocumento> documentos) {

		this.documentos = documentos;
	}

	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<EventoRecebimentoObjetoObservacao></code>
	 */
	public Collection<EventoRecebimentoObjetoObservacao> getObservacoes() {

		return observacoes;
	}

	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<EventoRecebimentoObjetoObservacao> observacoes) {

		this.observacoes = observacoes;
	}

}
