package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.MoneyDeserializer;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 09/03/2016 - 18:07:56
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_ev_registro_nc_aud")
@Entity
@Table(name="cnt_ev_registro_nc")
@JsonIgnoreProperties({ "$checked"})
public class EventoRegistroNaoConformidade extends ContratoEvento {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Atributo dataNaoConformidade. */
    @Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsContrato.EventoRegistroNaoConformidadeView.class })
	private Calendar dataNaoConformidade;

    /** Atributo tipoNaoConformidade. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.EventoRegistroNaoConformidadeView.class })
    private TermoReferenciaSancao tipoNaoConformidade;

	/** Atributo percentualMulta. */
    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.EventoRegistroNaoConformidadeView.class })
    private BigDecimal percentualMulta;

    /** Atributo valorMulta. */
    @Column(precision = 20, scale = 4)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.EventoRegistroNaoConformidadeView.class })
    private BigDecimal valorMulta;

    /** Atributo respostaContratada. */
	@Column(length = 2000, nullable = true)
	@JsonView({ ViewsContrato.EventoRegistroNaoConformidadeView.class })
    private String respostaContratada;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRegistroNaoConformidade", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRegistroNaoConformidadeView.class })
	private Collection<EventoRegistroNaoConformidadeDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRegistroNaoConformidade", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRegistroNaoConformidadeView.class })
	private Collection<EventoRegistroNaoConformidadeObservacao> observacoes;


	/**
	 * Retorna o valor do atributo <code>dataNaoConformidade</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataNaoConformidade() {

		return dataNaoConformidade;
	}


	/**
	 * Define o valor do atributo <code>dataNaoConformidade</code>.
	 *
	 * @param dataNaoConformidade
	 */
	public void setDataNaoConformidade(Calendar dataNaoConformidade) {

		this.dataNaoConformidade = dataNaoConformidade;
	}


	public TermoReferenciaSancao getTipoNaoConformidade() {
		return tipoNaoConformidade;
	}


	public void setTipoNaoConformidade(TermoReferenciaSancao tipoNaoConformidade) {
		this.tipoNaoConformidade = tipoNaoConformidade;
	}


	/**
	 * Retorna o valor do atributo <code>percentualMulta</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getPercentualMulta() {

		return percentualMulta;
	}


	/**
	 * Define o valor do atributo <code>percentualMulta</code>.
	 *
	 * @param percentualMulta
	 */
	public void setPercentualMulta(BigDecimal percentualMulta) {

		this.percentualMulta = percentualMulta;
	}


	/**
	 * Retorna o valor do atributo <code>valorMulta</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorMulta() {

		return valorMulta;
	}


	/**
	 * Define o valor do atributo <code>valorMulta</code>.
	 *
	 * @param valorMulta
	 */
	public void setValorMulta(BigDecimal valorMulta) {

		this.valorMulta = valorMulta;
	}


	/**
	 * Retorna o valor do atributo <code>respostaContratada</code>
	 *
	 * @return <code>String</code>
	 */
	public String getRespostaContratada() {

		return respostaContratada;
	}


	/**
	 * Define o valor do atributo <code>respostaContratada</code>.
	 *
	 * @param respostaContratada
	 */
	public void setRespostaContratada(String respostaContratada) {

		this.respostaContratada = respostaContratada;
	}


	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<EventoRegistroNaoConformidadeDocumento></code>
	 */
	public Collection<EventoRegistroNaoConformidadeDocumento> getDocumentos() {

		return documentos;
	}


	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<EventoRegistroNaoConformidadeDocumento> documentos) {

		this.documentos = documentos;
	}


	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<EventoRegistroNaoConformidadeObservacao></code>
	 */
	public Collection<EventoRegistroNaoConformidadeObservacao> getObservacoes() {

		return observacoes;
	}


	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<EventoRegistroNaoConformidadeObservacao> observacoes) {

		this.observacoes = observacoes;
	}

}
