package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;


/**
 * 
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 19/03/2016 - 09:20:20
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_ev_repactuacao_aud")
@Entity
@Table(name="cnt_ev_repactuacao")
public class EventoRepactuacao extends ContratoEvento {
	
	private static final long serialVersionUID = 6779992005903893890L;
	
	/** Atributo numeroRepactuacao. */
	@Column(length = 30, nullable = false)
	@JsonView({ ViewsContrato.EventoRepactuacaoEditView.class })
    private String numeroRepactuacao;

	/** Atributo dataRepactuacao. */
    @Column(nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsContrato.EventoRepactuacaoEditView.class })
	private Calendar dataRepactuacao;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JsonView({ ViewsContrato.EventoRepactuacaoEditView.class })
    private RegistroOcorrencia registroOcorrencia;

	/** Atributo justificativaRepactuacao. */
	@Column(length = 500, nullable = false)
	@JsonView({ ViewsContrato.EventoRepactuacaoEditView.class })
    private String justificativaRepactuacao;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRepactuacao", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRepactuacaoEditView.class })
	private Collection<EventoRepactuacaoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRepactuacao", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRepactuacaoEditView.class })
	private Collection<EventoRepactuacaoObservacao> observacoes;

	/**
	 * @return the numeroRepactuacao
	 */
	public String getNumeroRepactuacao() {
		return numeroRepactuacao;
	}

	/**
	 * @param numeroRepactuacao the numeroRepactuacao to set
	 */
	public void setNumeroRepactuacao(String numeroRepactuacao) {
		this.numeroRepactuacao = numeroRepactuacao;
	}

	/**
	 * @return the dataRepactuacao
	 */
	public Calendar getDataRepactuacao() {
		return dataRepactuacao;
	}

	/**
	 * @param dataRepactuacao the dataRepactuacao to set
	 */
	public void setDataRepactuacao(Calendar dataRepactuacao) {
		this.dataRepactuacao = dataRepactuacao;
	}

	/**
	 * @return the registroOcorrencia
	 */
	public RegistroOcorrencia getRegistroOcorrencia() {
		return registroOcorrencia;
	}

	/**
	 * @param registroOcorrencia the registroOcorrencia to set
	 */
	public void setRegistroOcorrencia(RegistroOcorrencia registroOcorrencia) {
		this.registroOcorrencia = registroOcorrencia;
	}

	/**
	 * @return the justificativaRepactuacao
	 */
	public String getJustificativaRepactuacao() {
		return justificativaRepactuacao;
	}

	/**
	 * @param justificativaRepactuacao the justificativaRepactuacao to set
	 */
	public void setJustificativaRepactuacao(String justificativaRepactuacao) {
		this.justificativaRepactuacao = justificativaRepactuacao;
	}

	public Collection<EventoRepactuacaoDocumento> getDocumentos() {
		return documentos;
	}

	public void setDocumentos(Collection<EventoRepactuacaoDocumento> documentos) {
		this.documentos = documentos;
	}

	public Collection<EventoRepactuacaoObservacao> getObservacoes() {
		return observacoes;
	}

	public void setObservacoes(Collection<EventoRepactuacaoObservacao> observacoes) {
		this.observacoes = observacoes;
	}

}