package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.model.Dominio;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

/**
 *
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 09/03/2016 - 18:07:56
 *
 * @version 1.0.0
 *
 * @author erick.sato
 *
 */
@Audited
@AuditTable(value = "cnt_ev_rescisaocontrato_aud")
@Entity
@Table(name = "cnt_ev_rescisaocontrato")
@JsonIgnoreProperties({ "$checked" })
public class EventoRescisaoContrato extends ContratoEvento {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1L;

	@Column(length = 30, nullable = false, updatable = true)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private String numeroRescisao;

	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private Calendar dataRescisao;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private Dominio tipoRescisao;

	@Column(nullable = false)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private Boolean devolucaoValor = Boolean.FALSE;

	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private BigDecimal valorADevolver;

	/** Atributo justificativa. */
	@Column(length = 2000, nullable = false)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private String justificativa;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRescisaoContrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private Collection<EventoRescisaoContratoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "eventoRescisaoContrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.EventoRescisaoContratoView.class })
	private Collection<EventoRescisaoContratoObservacao> observacoes;

	public Calendar getDataRescisao() {
		return dataRescisao;
	}

	public void setDataRescisao(Calendar dataRescisao) {
		this.dataRescisao = dataRescisao;
	}

	public Boolean getDevolucaoValor() {
		return devolucaoValor;
	}

	public void setDevolucaoValor(Boolean devolucaoValor) {
		this.devolucaoValor = devolucaoValor;
	}

	public BigDecimal getValorADevolver() {
		return valorADevolver;
	}

	public void setValorADevolver(BigDecimal valorADevolver) {
		this.valorADevolver = valorADevolver;
	}

	public String getJustificativa() {
		return justificativa;
	}

	public void setJustificativa(String justificativa) {
		this.justificativa = justificativa;
	}

	public Collection<EventoRescisaoContratoDocumento> getDocumentos() {
		return documentos;
	}

	public void setDocumentos(
			Collection<EventoRescisaoContratoDocumento> documentos) {
		this.documentos = documentos;
	}

	public Collection<EventoRescisaoContratoObservacao> getObservacoes() {
		return observacoes;
	}

	public void setObservacoes(
			Collection<EventoRescisaoContratoObservacao> observacoes) {
		this.observacoes = observacoes;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	public String getNumeroRescisao() {
		return numeroRescisao;
	}

	public void setNumeroRescisao(String numeroRescisao) {
		this.numeroRescisao = numeroRescisao;
	}

	public Dominio getTipoRescisao() {
		return tipoRescisao;
	}

	public void setTipoRescisao(Dominio tipoRescisao) {
		this.tipoRescisao = tipoRescisao;
	}

}
