package br.com.centralit.api.model;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;
import br.com.centralit.framework.util.UtilColecao;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 24/02/2016 - 08:39:42
 *
 * @version 1.0.0
 *
 * @author erick.sato
 *
 */
@Audited
@AuditTable(value = "cnt_notificacao_aud")
@Entity
@Table(name="cnt_notificacao")
public class NotificacaoContrato extends PersistentObjectAuditOrganizacao {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Atributo id. */
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @JsonView({ Views.GenericView.class })
    private Long id;

    @ManyToOne(fetch = FetchType.LAZY, optional=false)
    @JsonView({ Views.GenericView.class, ViewsContrato.NotificacaoContratoEditView.class })
    private Contrato contrato;

	@Column(length = 30, nullable = false, updatable = true, unique = true)
	@JsonView({ ViewsContrato.GenericView.class })
	private String codigo;

	@Column(name = "emissao", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.GenericView.class })
	private Calendar emissao;

	@Column(name = "associarOcorrencia", nullable = true)
	@JsonView({ ViewsContrato.GenericView.class })
	private Boolean associarOcorrencia;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "notificacaoContrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.GenericView.class })
	private Collection<NotificacaoContratoOcorrencia> ocorrencias;

	@ManyToOne(fetch = FetchType.LAZY)
    @JsonView({ ViewsContrato.GenericView.class })
    private Dominio dominioTipoSancao;

	@Column(length = 500, nullable = false)
	@JsonView({ ViewsContrato.GenericView.class })
	private String descricao;

	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "notificacaoContrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.GenericView.class })
	private Collection<NotificacaoContratoObservacao> observacoes;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "notificacaoContrato", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.GenericView.class })
	private Collection<NotificacaoContratoDocumento> documentos;

	@Transient
	@JsonView({ ViewsContrato.GenericView.class })
	private String codigoNotificacaoFormatado;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Contrato getContrato() {
		return contrato;
	}

	public void setContrato(Contrato contrato) {
		this.contrato = contrato;
	}

	public String getCodigo() {
		return codigo;
	}

	public void setCodigo(String codigo) {
		this.codigo = codigo;
	}

	public Calendar getEmissao() {
		return emissao;
	}

	public void setEmissao(Calendar emissao) {
		this.emissao = emissao;
	}

	public Boolean getAssociarOcorrencia() {
		return associarOcorrencia;
	}

	public void setAssociarOcorrencia(Boolean associarOcorrencia) {
		this.associarOcorrencia = associarOcorrencia;
	}

	public Collection<NotificacaoContratoOcorrencia> getOcorrencias() {
		List<NotificacaoContratoOcorrencia> ocorrenciasList = new ArrayList<NotificacaoContratoOcorrencia>();
		if(!UtilColecao.isVazio(ocorrencias)){
			for (NotificacaoContratoOcorrencia notificacaoContratoOcorrencia : ocorrencias) {
				if(notificacaoContratoOcorrencia.getDataInativo() != null){
					continue;
				}
				ocorrenciasList.add(notificacaoContratoOcorrencia);
			}
		}
		return ocorrenciasList;
	}

	public void setOcorrencias(Collection<NotificacaoContratoOcorrencia> ocorrencias) {
		this.ocorrencias = ocorrencias;
	}

	public String getCodigoNotificacaoFormatado() {
		if (StringUtils.isNotBlank(codigo)) {
			this.codigoNotificacaoFormatado = codigo.substring(0, codigo.length() - 4) + "/" + codigo.substring(codigo.length() - 4, codigo.length());
		}
		return codigoNotificacaoFormatado;
	}

	public void setCodigoNotificacaoFormatado(String codigoNotificacaoFormatado) {
		this.codigoNotificacaoFormatado = codigoNotificacaoFormatado;
	}

	public Dominio getDominioTipoSancao() {
		return dominioTipoSancao;
	}

	public void setDominioTipoSancao(Dominio dominioTipoSancao) {
		this.dominioTipoSancao = dominioTipoSancao;
	}

	public String getDescricao() {
		return descricao;
	}

	public void setDescricao(String descricao) {
		this.descricao = descricao;
	}

	public Collection<NotificacaoContratoObservacao> getObservacoes() {
		return observacoes;
	}

	public void setObservacoes(Collection<NotificacaoContratoObservacao> observacoes) {
		this.observacoes = observacoes;
	}

	public Collection<NotificacaoContratoDocumento> getDocumentos() {
		return documentos;
	}

	public void setDocumentos(Collection<NotificacaoContratoDocumento> documentos) {
		this.documentos = documentos;
	}

}
