package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 *
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 03/02/2016 - 15:25:05
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_ordemfornecimento_aud")
@Entity
@Table(name = "cnt_ordemfornecimento")
public class OrdemFornecimento extends PersistentObjectAuditOrganizacao {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class,
			ViewsContrato.OrdemFornecimentoEditView.class })
	private Long id;

	/** Atributo numOrdemFornecimento. */
	@Column(name = "numOrdemFornecimento", length = 20, nullable = false)
	@JsonView({ ViewsContrato.OrdemFornecimentoListView.class,
			ViewsContrato.RegistroOcorrenciaEditView.class,
			ViewsContrato.EventoLiberacaoPagamentoView.class, ViewsContrato.ContratoEditView.class })
	private String numOrdemFornecimento;

	/** Atributo dataEmissao. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.OrdemFornecimentoListView.class,
			ViewsContrato.EventoLiberacaoPagamentoView.class, ViewsContrato.ContratoEditView.class })
	private Calendar dataEmissao;

	/** Atributo dominioSituacao. */
	@ManyToOne(fetch = FetchType.EAGER, optional = true)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
	private Dominio dominioSituacao;

	/** Atributo contrato. */
	@ManyToOne(fetch = FetchType.EAGER, optional = false)
	@JsonView({ ViewsContrato.OrdemFornecimentoListView.class })
	private Contrato contrato;

	/** Atributo localPrestacaoServico. */
	@Column(length = 500, nullable = true)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
	private String localPrestacaoServico;

	/** Atributo itensBensServicos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "ordemFornecimento", cascade = CascadeType.ALL)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
	private Collection<OrdemFornecimentoItem> itensBensServicos;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "ordemFornecimento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class, ViewsContrato.ContratoEditView.class })
	private Collection<OrdemFornecimentoDocumento> documentos;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "ordemFornecimento", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class, ViewsContrato.ContratoEditView.class })
	private Collection<OrdemFornecimentoObservacao> observacoes;

	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	public String getNumOrdemFornecimento() {
		return numOrdemFornecimento;
	}

	public void setNumOrdemFornecimento(String numOrdemFornecimento) {
		this.numOrdemFornecimento = numOrdemFornecimento;
	}

	/**
	 * Retorna o valor do atributo <code>dataEmissao</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataEmissao() {

		return dataEmissao;
	}

	/**
	 * Define o valor do atributo <code>dataEmissao</code>.
	 *
	 * @param dataEmissao
	 */
	public void setDataEmissao(Calendar dataEmissao) {

		this.dataEmissao = dataEmissao;
	}

	/**
	 * Retorna o valor do atributo <code>dominioSituacao</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioSituacao() {

		return dominioSituacao;
	}

	/**
	 * Define o valor do atributo <code>dominioSituacao</code>.
	 *
	 * @param dominioSituacao
	 */
	public void setDominioSituacao(Dominio dominioSituacao) {

		this.dominioSituacao = dominioSituacao;
	}

	/**
	 * Retorna o valor do atributo <code>contrato</code>
	 *
	 * @return <code>Contrato</code>
	 */
	public Contrato getContrato() {

		return contrato;
	}

	/**
	 * Define o valor do atributo <code>contrato</code>.
	 *
	 * @param contrato
	 */
	public void setContrato(Contrato contrato) {

		this.contrato = contrato;
	}

	/**
	 * Retorna o valor do atributo <code>localPrestacaoServico</code>
	 *
	 * @return <code>String</code>
	 */
	public String getLocalPrestacaoServico() {

		return localPrestacaoServico;
	}

	/**
	 * Define o valor do atributo <code>localPrestacaoServico</code>.
	 *
	 * @param localPrestacaoServico
	 */
	public void setLocalPrestacaoServico(String localPrestacaoServico) {

		this.localPrestacaoServico = localPrestacaoServico;
	}

	/**
	 * Retorna o valor do atributo <code>itensBensServicos</code>
	 *
	 * @return <code>Collection<OrdemFornecimentoItem></code>
	 */
	public Collection<OrdemFornecimentoItem> getItensBensServicos() {

		return itensBensServicos;
	}

	/**
	 * Define o valor do atributo <code>itensBensServicos</code>.
	 *
	 * @param itensBensServicos
	 */
	public void setItensBensServicos(
			Collection<OrdemFornecimentoItem> itensBensServicos) {

		this.itensBensServicos = itensBensServicos;
	}

	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<OrdemFornecimentoDocumento></code>
	 */
	public Collection<OrdemFornecimentoDocumento> getDocumentos() {

		return documentos;
	}

	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<OrdemFornecimentoDocumento> documentos) {

		this.documentos = documentos;
	}

	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<OrdemFornecimentoObservacao></code>
	 */
	public Collection<OrdemFornecimentoObservacao> getObservacoes() {

		return observacoes;
	}

	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(
			Collection<OrdemFornecimentoObservacao> observacoes) {

		this.observacoes = observacoes;
	}

}
