package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;


/**
 * 
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 03/02/2016 - 15:42:58
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Audited
@AuditTable(value = "cnt_of_item_aud")
@Entity
@Table(name="cnt_of_item")
@JsonIgnoreProperties({ "$checked", "$index" })
public class OrdemFornecimentoItem extends PersistentObjectAudit{

    /** Atributo serialVersionUID. */
	private static final long serialVersionUID = 5399088518739879093L;

	/** Atributo id. */
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @JsonView({ Views.GenericView.class })
    private Long id;

    /** Atributo contrato. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private OrdemFornecimento ordemFornecimento;

	/** Atributo contratoInativo. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private OrdemFornecimento ordemFornecimentoInativo;
	
	/** Atributo itemBemServico. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
	private ContratoItem itemBemServico;
	
	/** Atributo dataInicioPrevisto. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
	private Calendar dataInicioPrevisto;
	
	/** Atributo dataTerminoPrevisto. */
	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
	private Calendar dataTerminoPrevisto;
	
	/** Atributo quantidade. */
	@Column(nullable = false)
	@JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
	private Long quantidade;

	/** Atributo valorUnitarioItem. */
    @Column(precision = 20, scale = 4, nullable = false)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
    private BigDecimal valorUnitarioItem;

    /** Atributo valorTotalItem. */
    @Column(precision = 20, scale = 4, nullable = false)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ ViewsContrato.OrdemFornecimentoEditView.class })
    private BigDecimal valorTotalItem;

	
	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {
	
		return id;
	}

	
	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id 
	 */
	public void setId(Long id) {
	
		this.id = id;
	}

	
	/**
	 * Retorna o valor do atributo <code>ordemFornecimento</code>
	 *
	 * @return <code>OrdemFornecimento</code>
	 */
	public OrdemFornecimento getOrdemFornecimento() {
	
		return ordemFornecimento;
	}

	
	/**
	 * Define o valor do atributo <code>ordemFornecimento</code>.
	 *
	 * @param ordemFornecimento 
	 */
	public void setOrdemFornecimento(OrdemFornecimento ordemFornecimento) {
	
		this.ordemFornecimento = ordemFornecimento;
	}

	
	/**
	 * Retorna o valor do atributo <code>ordemFornecimentoInativo</code>
	 *
	 * @return <code>OrdemFornecimento</code>
	 */
	public OrdemFornecimento getOrdemFornecimentoInativo() {
	
		return ordemFornecimentoInativo;
	}

	
	/**
	 * Define o valor do atributo <code>ordemFornecimentoInativo</code>.
	 *
	 * @param ordemFornecimentoInativo 
	 */
	public void setOrdemFornecimentoInativo(OrdemFornecimento ordemFornecimentoInativo) {
	
		this.ordemFornecimentoInativo = ordemFornecimentoInativo;
	}

	
	/**
	 * Retorna o valor do atributo <code>itemBemServico</code>
	 *
	 * @return <code>ContratoItem</code>
	 */
	public ContratoItem getItemBemServico() {
	
		return itemBemServico;
	}

	
	/**
	 * Define o valor do atributo <code>itemBemServico</code>.
	 *
	 * @param itemBemServico 
	 */
	public void setItemBemServico(ContratoItem itemBemServico) {
	
		this.itemBemServico = itemBemServico;
	}

	
	/**
	 * Retorna o valor do atributo <code>dataInicioPrevisto</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataInicioPrevisto() {
	
		return dataInicioPrevisto;
	}

	
	/**
	 * Define o valor do atributo <code>dataInicioPrevisto</code>.
	 *
	 * @param dataInicioPrevisto 
	 */
	public void setDataInicioPrevisto(Calendar dataInicioPrevisto) {
	
		this.dataInicioPrevisto = dataInicioPrevisto;
	}

	
	/**
	 * Retorna o valor do atributo <code>dataTerminoPrevisto</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataTerminoPrevisto() {
	
		return dataTerminoPrevisto;
	}

	
	/**
	 * Define o valor do atributo <code>dataTerminoPrevisto</code>.
	 *
	 * @param dataTerminoPrevisto 
	 */
	public void setDataTerminoPrevisto(Calendar dataTerminoPrevisto) {
	
		this.dataTerminoPrevisto = dataTerminoPrevisto;
	}

	
	/**
	 * Retorna o valor do atributo <code>quantidade</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getQuantidade() {
	
		return quantidade;
	}

	
	/**
	 * Define o valor do atributo <code>quantidade</code>.
	 *
	 * @param quantidade 
	 */
	public void setQuantidade(Long quantidade) {
	
		this.quantidade = quantidade;
	}

	
	/**
	 * Retorna o valor do atributo <code>valorUnitarioItem</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorUnitarioItem() {
	
		return valorUnitarioItem;
	}

	
	/**
	 * Define o valor do atributo <code>valorUnitarioItem</code>.
	 *
	 * @param valorUnitarioItem 
	 */
	public void setValorUnitarioItem(BigDecimal valorUnitarioItem) {
	
		this.valorUnitarioItem = valorUnitarioItem;
	}

	
	/**
	 * Retorna o valor do atributo <code>valorTotalItem</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorTotalItem() {
	
		return valorTotalItem;
	}

	
	/**
	 * Define o valor do atributo <code>valorTotalItem</code>.
	 *
	 * @param valorTotalItem 
	 */
	public void setValorTotalItem(BigDecimal valorTotalItem) {
	
		this.valorTotalItem = valorTotalItem;
	}

}
