package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 * Registro de ocorr&ecirc;ncias que um determinado contrato pode ter. Um
 * contrato pode possuir v&aacute;rias ocorr&ecirc;ncias, onde cada uma pode
 * emitir uma glosa ou n&atilde;o.
 *
 * @since 15/02/2016 - 10:29:25
 * @version 1.0.0
 * @author ciro.junior (<a
 *         href="mailto:ciro.junior@centralit.com.br">ciro.junior
 *         @centralit.com.br</a>)
 *
 */
@Audited
@AuditTable(value = "cnt_registroocorrencia_aud")
@Entity
@Table(name="cnt_registroocorrencia")
@JsonIgnoreProperties({"cnpjFornecedor"})
public class RegistroOcorrencia extends PersistentObjectAuditOrganizacao {

	private static final long serialVersionUID = 3729415605391436265L;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	@Embedded
	@JsonView({ Views.GenericView.class })
	private RegistroSequencial registroSequencial;

	/**
	 * Data de cria&ccedil;&atilde;o da ocorr&ecirc;ncia. A data da Emiss&atilde;o dever&aacute; ser MAIOR que &quot;Data de In&iacute;cio
	 * da Vig&ecirc;ncia do contrato&quot; e MENOR que &quot;Data de T&eacute;rmino da Vig&ecirc;ncia do contrato&quot;
	 */
	@Column(name = "dataEmissaoOcorrencia", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private Calendar dataEmissaoOcorrencia;

	/** Atributo utilizado para l&oacute;gica de valida&ccedil;&atilde;o dos campos justificativaGlosa, valorGlosa */
	@Transient
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private Boolean emitirGlosa;

	/** Valor da glosa a ser aplicada para determinado registro de ocorr&ecirc;ncia. &Eacute; opcional caso emitir glosa seja false */
	@Column(name = "valorGlosa", precision = 20, scale = 4, nullable = true)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private BigDecimal valorGlosa;

	/**
	 * Justificativa para a glosa a ser aplicada em determinado registro de ocorr&ecirc;ncia. &Eacute; opcional caso emitir glosa seja false
	 */
	@Column(name = "justificativaGlosa", length = 200, nullable = true)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private String justificativaGlosa;

	/** T&iacute;tulo da ocorr&ecirc;ncia a ser registrada. &Eacute; um campo opcional. */
	@Column(name = "tituloOcorrencia", length = 200, nullable = false)
	@JsonView({ Views.GenericView.class })
	private String tituloOcorrencia;

	/** Descricao da ocorr&ecirc;ncia que est&aacute; sendo aplicada para determinado contrato associado */
	@Column(name = "descricaoOcorrencia", length = 600, nullable = true)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class, ViewsContrato.EventoPenalidadeEditView.class, Views.GenericView.class })
	private String descricaoOcorrencia;

	/**
	 * Prazo para o cumprimento das exig&ecirc;ncias determidadas para uma ocorr&ecirc;ncia de um contrato. A data do prazo n&atilde;o pode
	 * ser inferior &agrave; data da emiss&atilde;o da ocorr&ecirc;ncia
	 */
	@Column(name = "prazoCumprimento")
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private Calendar prazoCumprimento;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private Contrato contrato;

	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private OrdemFornecimento ordemFornecimento;

	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "registroOcorrencia", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private Collection<RegistroOcorrenciaObservacao> observacoes;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "registroOcorrencia", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.RegistroOcorrenciaEditView.class })
	private Collection<RegistroOcorrenciaDocumento> documentos;

	public Long getId() {

		return id;
	}

	public void setId(Long id) {

		this.id = id;
	}

	public RegistroSequencial getRegistroSequencial() {

		return registroSequencial;
	}


	public void setRegistroOcorrenciaSequencial(RegistroSequencial registroSequencial) {

		this.registroSequencial = registroSequencial;
	}

	public Calendar getDataEmissaoOcorrencia() {

		return dataEmissaoOcorrencia;
	}

	public void setDataEmissaoOcorrencia(Calendar dataEmissaoOcorrencia) {

		this.dataEmissaoOcorrencia = dataEmissaoOcorrencia;
	}

	public Boolean getEmitirGlosa() {

		return emitirGlosa;
	}

	public void setEmitirGlosa(Boolean emitirGlosa) {

		this.emitirGlosa = emitirGlosa;
	}

	public BigDecimal getValorGlosa() {

		return valorGlosa;
	}

	public void setValorGlosa(BigDecimal valorGlosa) {

		this.valorGlosa = valorGlosa;
	}

	public String getJustificativaGlosa() {

		return justificativaGlosa;
	}

	public void setJustificativaGlosa(String justificativaGlosa) {

		this.justificativaGlosa = justificativaGlosa;
	}

	public String getTituloOcorrencia() {

		return tituloOcorrencia;
	}

	public void setTituloOcorrencia(String tituloOcorrencia) {

		this.tituloOcorrencia = tituloOcorrencia;
	}

	public String getDescricaoOcorrencia() {

		return descricaoOcorrencia;
	}

	public void setDescricaoOcorrencia(String descricaoOcorrencia) {

		this.descricaoOcorrencia = descricaoOcorrencia;
	}

	public Calendar getPrazoCumprimento() {

		return prazoCumprimento;
	}

	public void setPrazoCumprimento(Calendar prazoCumprimento) {

		this.prazoCumprimento = prazoCumprimento;
	}

	public Contrato getContrato() {

		return contrato;
	}

	public void setContrato(Contrato contrato) {

		this.contrato = contrato;
	}

	public OrdemFornecimento getOrdemFornecimento() {

		return ordemFornecimento;
	}

	public void setOrdemFornecimento(OrdemFornecimento ordemFornecimento) {

		this.ordemFornecimento = ordemFornecimento;
	}

	public Collection<RegistroOcorrenciaObservacao> getObservacoes() {

		return observacoes;
	}

	public void setObservacoes(Collection<RegistroOcorrenciaObservacao> observacoes) {

		this.observacoes = observacoes;
	}

	public Collection<RegistroOcorrenciaDocumento> getDocumentos() {

		return documentos;
	}

	public void setDocumentos(Collection<RegistroOcorrenciaDocumento> documentos) {

		this.documentos = documentos;
	}

}
