package br.com.centralit.api.model;

import java.util.Calendar;
import java.util.Collection;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Type;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSerializer;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

@Audited
@AuditTable(value = "cnt_termoreferencia_aud")
@Entity
@Table(name="cnt_termoreferencia")
@JsonIgnoreProperties({"integrantesEstudoTecnico", "$index"})
public class TermoReferencia extends PersistentObjectAuditOrganizacao {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ ViewsContrato.TermoReferenciaListView.class, ViewsContrato.ContratoEditView.class })
	private Long id;

	@Column(length = 30)
	@JsonView({ ViewsContrato.TermoReferenciaListView.class, ViewsContrato.ContratoEditView.class, ViewsContrato.ContratoAutoCompleteView.class })
	private String codigo;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.TermoReferenciaListView.class })
	private EstudoTecnico estudoTecnico;

	@Column(length = 4000, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String objetoContratacao;

	@Basic(fetch = FetchType.LAZY, optional = false)
	@Type(type = "org.hibernate.type.StringClobType")
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String deveresContratante;

	@Basic(fetch = FetchType.LAZY, optional = false)
	@Type(type = "org.hibernate.type.StringClobType")
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String deveresContratada;

	@Basic(fetch = FetchType.LAZY)
	@Type(type = "org.hibernate.type.StringClobType")
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String deveresOrgaoRegistroPreco;

	@Basic(fetch = FetchType.LAZY)
	@Type(type = "org.hibernate.type.StringClobType")
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String mecanismoComunicacao;

	@Column(length = 1000, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String documentacaoMinima;

	@Column(length = 1000)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String procedimentosInspecao;

	@Column(length = 1000)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String adequacaoOrcamentaria;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Dominio empreitada;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Dominio adjudicacaoObjeto;

	@Column(length = 500, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String justificativa;

	@Column(length = 500)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String qualificacaoTecnica;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Dominio modalidade;

	@Column(length = 500, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String justificativaModalidade;

	@Column(length = 500)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String justificativaPreferencia;

	@Column(length = 500)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String justificativaContratacaoDir;

	@Column(length = 500)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String criterioHabilitacao;

	@Column(name = "dataTermoGerado")
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSerializer.class)
	@JsonView({ ViewsContrato.TermoReferenciaListView.class, ViewsContrato.EstudoTecnicoListView.class })
	private Calendar dataTermoGerado;

	/** Atributo detalhamentosSolucoes. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "termo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Collection<TermoReferenciaDetalhamentoSolucao> detalhamentosSolucoes;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "termo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class, ViewsContrato.ContratoAutoCompleteView.class })
	private Collection<TermoReferenciaPenalidade> penalidades;

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "termo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class, ViewsContrato.ContratoAutoCompleteView.class })
	private Collection<TermoReferenciaSancao> sancoes;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "termo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Collection<TermoReferenciaObservacao> observacoes;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "termo", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Collection<TermoReferenciaDocumento> documentos;

	@Transient
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Long rascunho;
	
	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>codigo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getCodigo() {

		return codigo;
	}

	/**
	 * Define o valor do atributo <code>codigo</code>.
	 *
	 * @param codigo
	 */
	public void setCodigo(String codigo) {

		this.codigo = codigo;
	}

	/**
	 * Retorna o valor do atributo <code>objetoContratacao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getObjetoContratacao() {

		return objetoContratacao;
	}

	/**
	 * Define o valor do atributo <code>objetoContratacao</code>.
	 *
	 * @param objetoContratacao
	 */
	public void setObjetoContratacao(String objetoContratacao) {

		this.objetoContratacao = objetoContratacao;
	}

	/**
	 * Retorna o valor do atributo <code>deveresContratante</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDeveresContratante() {

		return deveresContratante;
	}

	/**
	 * Define o valor do atributo <code>deveresContratante</code>.
	 *
	 * @param deveresContratante
	 */
	public void setDeveresContratante(String deveresContratante) {

		this.deveresContratante = deveresContratante;
	}

	/**
	 * Retorna o valor do atributo <code>deveresContratada</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDeveresContratada() {

		return deveresContratada;
	}

	/**
	 * Define o valor do atributo <code>deveresContratada</code>.
	 *
	 * @param deveresContratada
	 */
	public void setDeveresContratada(String deveresContratada) {

		this.deveresContratada = deveresContratada;
	}

	/**
	 * Retorna o valor do atributo <code>deveresOrgaoRegistroPreco</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDeveresOrgaoRegistroPreco() {

		return deveresOrgaoRegistroPreco;
	}

	/**
	 * Define o valor do atributo <code>deveresOrgaoRegistroPreco</code>.
	 *
	 * @param deveresOrgaoRegistroPreco
	 */
	public void setDeveresOrgaoRegistroPreco(String deveresOrgaoRegistroPreco) {

		this.deveresOrgaoRegistroPreco = deveresOrgaoRegistroPreco;
	}

	/**
	 * Retorna o valor do atributo <code>mecanismoComunicacao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getMecanismoComunicacao() {

		return mecanismoComunicacao;
	}

	/**
	 * Define o valor do atributo <code>mecanismoComunicacao</code>.
	 *
	 * @param mecanismoComunicacao
	 */
	public void setMecanismoComunicacao(String mecanismoComunicacao) {

		this.mecanismoComunicacao = mecanismoComunicacao;
	}

	/**
	 * Retorna o valor do atributo <code>documentacaoMinima</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDocumentacaoMinima() {

		return documentacaoMinima;
	}

	/**
	 * Define o valor do atributo <code>documentacaoMinima</code>.
	 *
	 * @param documentacaoMinima
	 */
	public void setDocumentacaoMinima(String documentacaoMinima) {

		this.documentacaoMinima = documentacaoMinima;
	}

	/**
	 * Retorna o valor do atributo <code>procedimentosInspecao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getProcedimentosInspecao() {

		return procedimentosInspecao;
	}

	/**
	 * Define o valor do atributo <code>procedimentosInspecao</code>.
	 *
	 * @param procedimentosInspecao
	 */
	public void setProcedimentosInspecao(String procedimentosInspecao) {

		this.procedimentosInspecao = procedimentosInspecao;
	}

	/**
	 * Retorna o valor do atributo <code>adequacaoOrcamentaria</code>
	 *
	 * @return <code>String</code>
	 */
	public String getAdequacaoOrcamentaria() {

		return adequacaoOrcamentaria;
	}

	/**
	 * Define o valor do atributo <code>adequacaoOrcamentaria</code>.
	 *
	 * @param adequacaoOrcamentaria
	 */
	public void setAdequacaoOrcamentaria(String adequacaoOrcamentaria) {

		this.adequacaoOrcamentaria = adequacaoOrcamentaria;
	}

	/**
	 * Retorna o valor do atributo <code>empreitada</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getEmpreitada() {

		return empreitada;
	}

	/**
	 * Define o valor do atributo <code>empreitada</code>.
	 *
	 * @param empreitada
	 */
	public void setEmpreitada(Dominio empreitada) {

		this.empreitada = empreitada;
	}

	/**
	 * Retorna o valor do atributo <code>adjudicacaoObjeto</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getAdjudicacaoObjeto() {

		return adjudicacaoObjeto;
	}

	/**
	 * Define o valor do atributo <code>adjudicacaoObjeto</code>.
	 *
	 * @param adjudicacaoObjeto
	 */
	public void setAdjudicacaoObjeto(Dominio adjudicacaoObjeto) {

		this.adjudicacaoObjeto = adjudicacaoObjeto;
	}

	/**
	 * Retorna o valor do atributo <code>justificativa</code>
	 *
	 * @return <code>String</code>
	 */
	public String getJustificativa() {

		return justificativa;
	}

	/**
	 * Define o valor do atributo <code>justificativa</code>.
	 *
	 * @param justificativa
	 */
	public void setJustificativa(String justificativa) {

		this.justificativa = justificativa;
	}

	/**
	 * Retorna o valor do atributo <code>qualificacaoTecnica</code>
	 *
	 * @return <code>String</code>
	 */
	public String getQualificacaoTecnica() {

		return qualificacaoTecnica;
	}

	/**
	 * Define o valor do atributo <code>qualificacaoTecnica</code>.
	 *
	 * @param qualificacaoTecnica
	 */
	public void setQualificacaoTecnica(String qualificacaoTecnica) {

		this.qualificacaoTecnica = qualificacaoTecnica;
	}

	/**
	 * Retorna o valor do atributo <code>modalidade</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getModalidade() {

		return modalidade;
	}

	/**
	 * Define o valor do atributo <code>modalidade</code>.
	 *
	 * @param modalidade
	 */
	public void setModalidade(Dominio modalidade) {

		this.modalidade = modalidade;
	}

	/**
	 * Retorna o valor do atributo <code>justificativaModalidade</code>
	 *
	 * @return <code>String</code>
	 */
	public String getJustificativaModalidade() {

		return justificativaModalidade;
	}

	/**
	 * Define o valor do atributo <code>justificativaModalidade</code>.
	 *
	 * @param justificativaModalidade
	 */
	public void setJustificativaModalidade(String justificativaModalidade) {

		this.justificativaModalidade = justificativaModalidade;
	}

	/**
	 * Retorna o valor do atributo <code>justificativaPreferencia</code>
	 *
	 * @return <code>String</code>
	 */
	public String getJustificativaPreferencia() {

		return justificativaPreferencia;
	}

	/**
	 * Define o valor do atributo <code>justificativaPreferencia</code>.
	 *
	 * @param justificativaPreferencia
	 */
	public void setJustificativaPreferencia(String justificativaPreferencia) {

		this.justificativaPreferencia = justificativaPreferencia;
	}

	/**
	 * Retorna o valor do atributo <code>justificativaContratacaoDir</code>
	 *
	 * @return <code>String</code>
	 */
	public String getJustificativaContratacaoDir() {

		return justificativaContratacaoDir;
	}

	/**
	 * Define o valor do atributo <code>justificativaContratacaoDir</code>.
	 *
	 * @param justificativaContratacaoDir
	 */
	public void setJustificativaContratacaoDir(String justificativaContratacaoDir) {

		this.justificativaContratacaoDir = justificativaContratacaoDir;
	}

	/**
	 * Retorna o valor do atributo <code>criterioHabilitacao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getCriterioHabilitacao() {

		return criterioHabilitacao;
	}

	/**
	 * Define o valor do atributo <code>criterioHabilitacao</code>.
	 *
	 * @param criterioHabilitacao
	 */
	public void setCriterioHabilitacao(String criterioHabilitacao) {

		this.criterioHabilitacao = criterioHabilitacao;
	}

	/**
	 * Retorna o valor do atributo <code>estudoTecnico</code>
	 *
	 * @return <code>EstudoTecnico</code>
	 */
	public EstudoTecnico getEstudoTecnico() {

		return estudoTecnico;
	}

	/**
	 * Define o valor do atributo <code>estudoTecnico</code>.
	 *
	 * @param estudoTecnico
	 */
	public void setEstudoTecnico(EstudoTecnico estudoTecnico) {

		this.estudoTecnico = estudoTecnico;
	}

	/**
	 * Retorna o valor do atributo <code>detalhamentosSolucoes</code>
	 *
	 * @return <code>Collection<TermoReferenciaDetalhamentoSolucao></code>
	 */
	public Collection<TermoReferenciaDetalhamentoSolucao> getDetalhamentosSolucoes() {

		return detalhamentosSolucoes;
	}

	/**
	 * Define o valor do atributo <code>detalhamentosSolucoes</code>.
	 *
	 * @param detalhamentosSolucoes
	 */
	public void setDetalhamentosSolucoes(Collection<TermoReferenciaDetalhamentoSolucao> detalhamentosSolucoes) {

		this.detalhamentosSolucoes = detalhamentosSolucoes;
	}

	/**
	 * Retorna o valor do atributo <code>penalidades</code>
	 *
	 * @return <code>Collection<TermoReferenciaPenalidade></code>
	 */
	public Collection<TermoReferenciaPenalidade> getPenalidades() {

		return penalidades;
	}

	/**
	 * Define o valor do atributo <code>penalidades</code>.
	 *
	 * @param penalidades
	 */
	public void setPenalidades(Collection<TermoReferenciaPenalidade> penalidades) {

		this.penalidades = penalidades;
	}

	/**
	 * Retorna o valor do atributo <code>sancoes</code>
	 *
	 * @return <code>Collection<TermoReferenciaSancao></code>
	 */
	public Collection<TermoReferenciaSancao> getSancoes() {

		return sancoes;
	}

	/**
	 * Define o valor do atributo <code>sancoes</code>.
	 *
	 * @param sancoes
	 */
	public void setSancoes(Collection<TermoReferenciaSancao> sancoes) {

		this.sancoes = sancoes;
	}

	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<TermoReferenciaObservacao></code>
	 */
	public Collection<TermoReferenciaObservacao> getObservacoes() {

		return observacoes;
	}

	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<TermoReferenciaObservacao> observacoes) {

		this.observacoes = observacoes;
	}

	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<TermoReferenciaDocumento></code>
	 */
	public Collection<TermoReferenciaDocumento> getDocumentos() {

		return documentos;
	}

	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<TermoReferenciaDocumento> documentos) {

		this.documentos = documentos;
	}


	public Calendar getDataTermoGerado() {

		return dataTermoGerado;
	}


	public void setDataTermoGerado(Calendar dataTermoGerado) {

		this.dataTermoGerado = dataTermoGerado;
	}

	public Long getRascunho() {
		return rascunho;
	}
	
	public void setRascunho(Long rascunho) {
		this.rascunho = rascunho;
	}
}
