package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 * Termo de Referencia - Detalhamento da Solucao
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Redmine(s):</b> <a href="LINK_PORTAL">2389</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 05/01/2016 - 17:17:06
 *
 * @version 1.0.0
 *
 * @author juliana.barbosa
 *
 */
@Audited
@AuditTable(value = "cnt_tr_detsolucao_aud")
@Entity
@Table(name="cnt_tr_detalhamentosolucao")
@JsonIgnoreProperties({ "$checked", "$index" })
public class TermoReferenciaDetalhamentoSolucao extends PersistentObjectAuditOrganizacao {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Long id;

	// EstudoTecnicoBemServicoSolucao

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private EstudoTecnicoBemServicoSolucao bemServicoSolucao;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Dominio formaPagamento;

	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Long numeroParcelas;

	@Column(name = "prazoEntrega", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Calendar prazoEntrega;

	@Column(length = 150, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String localEntrega;

	@Column(nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Long quantidade;

	@Column(nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Long unidade;

	@Column(precision = 20, scale = 4, nullable = false)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private BigDecimal valor;

	@Column(precision = 20, scale = 4, nullable = false)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private BigDecimal total;

	/** Atributo fornecedor. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Parceiro fornecedor;

	/** Atributo orgaoExterno. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private Parceiro orgaoExterno;

	@Column(length = 150, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String metrica;

	@Column(length = 150, nullable = true)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String indicador;

	@Column(length = 150, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String nivelServico;

	@Column(length = 500, nullable = false)
	@JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private String descricaoGlosa;

	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	// @JsonView({ ViewsContrato.TermoReferenciaEditView.class })
	private TermoReferencia termo;

	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	// @JsonView({ ViewsContrato.TermoReferenciaEditView.class})
	@JoinColumn(name = "termoinativo_id")
	private TermoReferencia termoInativo;

	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>formaPagamento</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getFormaPagamento() {

		return formaPagamento;
	}

	/**
	 * Define o valor do atributo <code>formaPagamento</code>.
	 *
	 * @param formaPagamento
	 */
	public void setFormaPagamento(Dominio formaPagamento) {

		this.formaPagamento = formaPagamento;
	}

	/**
	 * Retorna o valor do atributo <code>numeroParcelas</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getNumeroParcelas() {

		return numeroParcelas;
	}

	/**
	 * Define o valor do atributo <code>numeroParcelas</code>.
	 *
	 * @param numeroParcelas
	 */
	public void setNumeroParcelas(Long numeroParcelas) {

		this.numeroParcelas = numeroParcelas;
	}

	/**
	 * Retorna o valor do atributo <code>prazoEntrega</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getPrazoEntrega() {

		return prazoEntrega;
	}

	/**
	 * Define o valor do atributo <code>prazoEntrega</code>.
	 *
	 * @param prazoEntrega
	 */
	public void setPrazoEntrega(Calendar prazoEntrega) {

		this.prazoEntrega = prazoEntrega;
	}

	/**
	 * Retorna o valor do atributo <code>localEntrega</code>
	 *
	 * @return <code>String</code>
	 */
	public String getLocalEntrega() {

		return localEntrega;
	}

	/**
	 * Define o valor do atributo <code>localEntrega</code>.
	 *
	 * @param localEntrega
	 */
	public void setLocalEntrega(String localEntrega) {

		this.localEntrega = localEntrega;
	}

	/**
	 * Retorna o valor do atributo <code>quantidade</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getQuantidade() {

		return quantidade;
	}

	/**
	 * Define o valor do atributo <code>quantidade</code>.
	 *
	 * @param quantidade
	 */
	public void setQuantidade(Long quantidade) {

		this.quantidade = quantidade;
	}

	/**
	 * Retorna o valor do atributo <code>unidade</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getUnidade() {

		return unidade;
	}

	/**
	 * Define o valor do atributo <code>unidade</code>.
	 *
	 * @param unidade
	 */
	public void setUnidade(Long unidade) {

		this.unidade = unidade;
	}

	/**
	 * Retorna o valor do atributo <code>valor</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValor() {

		return valor;
	}

	/**
	 * Define o valor do atributo <code>valor</code>.
	 *
	 * @param valor
	 */
	public void setValor(BigDecimal valor) {

		this.valor = valor;
	}

	/**
	 * Retorna o valor do atributo <code>total</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getTotal() {

		return total;
	}

	/**
	 * Define o valor do atributo <code>total</code>.
	 *
	 * @param total
	 */
	public void setTotal(BigDecimal total) {

		this.total = total;
	}

	/**
	 * Retorna o valor do atributo <code>fornecedor</code>
	 *
	 * @return <code>Parceiro</code>
	 */
	public Parceiro getFornecedor() {

		return fornecedor;
	}

	/**
	 * Define o valor do atributo <code>fornecedor</code>.
	 *
	 * @param fornecedor
	 */
	public void setFornecedor(Parceiro fornecedor) {

		this.fornecedor = fornecedor;
	}

	/**
	 * Retorna o valor do atributo <code>orgaoExterno</code>
	 *
	 * @return <code>Parceiro</code>
	 */
	public Parceiro getOrgaoExterno() {

		return orgaoExterno;
	}

	/**
	 * Define o valor do atributo <code>orgaoExterno</code>.
	 *
	 * @param orgaoExterno
	 */
	public void setOrgaoExterno(Parceiro orgaoExterno) {

		this.orgaoExterno = orgaoExterno;
	}

	/**
	 * Retorna o valor do atributo <code>metrica</code>
	 *
	 * @return <code>String</code>
	 */
	public String getMetrica() {

		return metrica;
	}

	/**
	 * Define o valor do atributo <code>metrica</code>.
	 *
	 * @param metrica
	 */
	public void setMetrica(String metrica) {

		this.metrica = metrica;
	}

	/**
	 * Retorna o valor do atributo <code>indicador</code>
	 *
	 * @return <code>String</code>
	 */
	public String getIndicador() {

		return indicador;
	}

	/**
	 * Define o valor do atributo <code>indicador</code>.
	 *
	 * @param indicador
	 */
	public void setIndicador(String indicador) {

		this.indicador = indicador;
	}

	/**
	 * Retorna o valor do atributo <code>nivelServico</code>
	 *
	 * @return <code>String</code>
	 */
	public String getNivelServico() {

		return nivelServico;
	}

	/**
	 * Define o valor do atributo <code>nivelServico</code>.
	 *
	 * @param nivelServico
	 */
	public void setNivelServico(String nivelServico) {

		this.nivelServico = nivelServico;
	}

	/**
	 * Retorna o valor do atributo <code>descricaoGlosa</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDescricaoGlosa() {

		return descricaoGlosa;
	}

	/**
	 * Define o valor do atributo <code>descricaoGlosa</code>.
	 *
	 * @param descricaoGlosa
	 */
	public void setDescricaoGlosa(String descricaoGlosa) {

		this.descricaoGlosa = descricaoGlosa;
	}

	/**
	 * Retorna o valor do atributo <code>bemServicoSolucao</code>
	 *
	 * @return <code>EstudoTecnicoBemServicoSolucao</code>
	 */
	public EstudoTecnicoBemServicoSolucao getBemServicoSolucao() {

		return bemServicoSolucao;
	}

	/**
	 * Define o valor do atributo <code>bemServicoSolucao</code>.
	 *
	 * @param bemServicoSolucao
	 */
	public void setBemServicoSolucao(EstudoTecnicoBemServicoSolucao bemServicoSolucao) {

		this.bemServicoSolucao = bemServicoSolucao;
	}

	/**
	 * Retorna o valor do atributo <code>termo</code>
	 *
	 * @return <code>TermoReferencia</code>
	 */
	public TermoReferencia getTermo() {

		return termo;
	}

	/**
	 * Define o valor do atributo <code>termo</code>.
	 *
	 * @param termo
	 */
	public void setTermo(TermoReferencia termo) {

		this.termo = termo;
	}

	/**
	 * Retorna o valor do atributo <code>termoInativo</code>
	 *
	 * @return <code>TermoReferencia</code>
	 */
	public TermoReferencia getTermoInativo() {

		return termoInativo;
	}

	/**
	 * Define o valor do atributo <code>termoInativo</code>.
	 *
	 * @param termoInativo
	 */
	public void setTermoInativo(TermoReferencia termoInativo) {

		this.termoInativo = termoInativo;
	}

}
