package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.ContratoEmpenhoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.ContratoEmpenho;
import br.com.centralit.api.model.ContratoEmpenhoDocumento;
import br.com.centralit.api.model.ContratoEmpenhoObservacao;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ContratoEmpenhoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.Util;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 09/03/2016 - 11:46:17
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("contratoEmpenhoService")
public class ContratoEmpenhoServiceImpl extends GenericServiceImpl<ContratoEmpenho, Long> implements ContratoEmpenhoService {

	private ContratoEmpenhoDao contratoEmpenhoDao;

	@Autowired
	private AnexoService anexoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	private Dominio idioma;

	@Autowired
	private ReportController reportService;

	@Autowired
	public ContratoEmpenhoServiceImpl( ContratoEmpenhoDao contratoEmpenhoDao ) {
		this.dao = contratoEmpenhoDao;
		this.contratoEmpenhoDao = contratoEmpenhoDao;
	}

	public ContratoEmpenho save(ContratoEmpenho entity) {

		entity.setDataEvento(entity.getData());
		String descricaoEmpenho = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EMPENHO", getIdioma());
		descricaoEmpenho = descricaoEmpenho.replace(":VALOR:", Util.convertBigDecimalToString(entity.getValorEmpenhado()));
		descricaoEmpenho = descricaoEmpenho.replace(":DATAEVENTO:", UtilDate.formatarData(entity.getData().getTime(), "dd/MM/yyyy"));
		entity.setDescricao(descricaoEmpenho);

		entity.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_EMPENHO));

		this.montarDocsObs(entity);
		ContratoEmpenho entitySaved = super.save(entity);
		return entitySaved;
	}

	@Override
	public ContratoEmpenho merge(ContratoEmpenho entity) {
		entity.setDataEvento(entity.getData());
		this.montarDocsObs(entity);
		ContratoEmpenho entitySaved = super.save(entity);
		
		return entitySaved;
	}
	
	private void montarDocsObs(ContratoEmpenho entity) {

		if (!UtilColecao.isVazio(entity.getObservacoes())) {

			for (ContratoEmpenhoObservacao observacao : entity.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(observacao.getAutor())) {
					idAutor = observacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId();
				}

				observacao.setAutor(this.usuarioService.find(idAutor));

				observacao.setContratoEmpenho(entity);
			}
		}

		if (!UtilColecao.isVazio(entity.getDocumentos())) {

			for (ContratoEmpenhoDocumento documento : entity.getDocumentos()) {

				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(documento.getEmitente()) && UtilObjeto.isReferencia(documento.getEmitente().getId())) {

					documento.setEmitente(this.parceiroServiceFornecedor.find(documento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(documento.getAnexos())) {
					Anexo anexo = ( (List<Anexo>) documento.getAnexos() ).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					documento.setAnexos(listaAnexo);
				}

				documento.setContratoEmpenho(entity);
			}
		}
	}

	@Override
	public Collection<ContratoEmpenho> findAllByContrato(Long contratoId) {

		return this.contratoEmpenhoDao.findAllByContrato(contratoId);
	}

	@Override
	public Collection<ContratoEmpenho> findEmpenhoParaLiberacao(Long contratoId) {

		return this.contratoEmpenhoDao.findEmpenhoParaLiberacao(contratoId);
	}

	public Dominio getIdioma() {

		if (!UtilObjeto.isReferencia(idioma)) {
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public void gerarPdfEmpenho(Long idEmpenho, boolean download, HttpServletResponse response) {

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;

		try {
			conn = reportService.getConnection();
			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
			String titulo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.TITULO_DOCUMENTO_EMPENHO", this.getIdioma());

			// Responsavel por verificar a presenca de dados no relatorio
			reportService.gerarRelatorio(download, "pdf", this.getJasperPrintEmpenho(idEmpenho, conn, usuarioLogado, titulo), response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		} catch (Exception e) {
			throw new BusinessException();
		} finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}

	@Override
	public JasperPrint getJasperPrintEmpenho(Long id, Connection conn, Usuario usuarioLogado, String titulo) throws JRException {

		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/eventoEmpenho.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("EMPENHO_ID", id);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally {
			UtilDataBase.closeResources(inputSub);
		}
	}

	@Override
	public JasperPrint getJasperPrintRelatorioEmpenho(Long idContrato, Date dataInicial, Date dataFinal, Connection conn, Usuario usuarioLogado, String titulo) throws JRException {

		InputStream input = null;
		InputStream inputSub = null;
		JasperPrint jasperPrint = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/relatorioEventoEmpenho.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("contratoId", idContrato);
			subReportParameters.put("dataInicial", dataInicial);
			subReportParameters.put("dataFinal", dataFinal);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally {
			UtilDataBase.closeResources(inputSub);
		}
		return jasperPrint;
	}

}
