package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRPrintPage;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.ContratoEventoDao;
import br.com.centralit.api.model.Contrato;
import br.com.centralit.api.model.ContratoEvento;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoAditivo;
import br.com.centralit.api.service.ContratoEmpenhoService;
import br.com.centralit.api.service.ContratoEventoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoAditivoService;
import br.com.centralit.api.service.EventoApostilamentoService;
import br.com.centralit.api.service.EventoLiberacaoPagamentoService;
import br.com.centralit.api.service.EventoPenalidadeService;
import br.com.centralit.api.service.EventoRecebimentoObjetoService;
import br.com.centralit.api.service.EventoRegistroNaoConformidadeService;
import br.com.centralit.api.service.EventoRepactuacaoService;
import br.com.centralit.api.service.EventoRescisaoContratoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p>
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 * </p>
 *
 * <p>
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 * </p>
 *
 * <p>
 * <p><b>Title: </b></p>
 * </p>
 *
 * <p>
 * <p><b>Description: </b></p>
 * </p>
 *
 * <p>
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 * </p>
 *
 * <p>
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 * </p>
 *
 * @since 09/03/2016 - 11:39:48
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("contratoEventoService")
public class ContratoEventoServiceImpl extends GenericServiceImpl<ContratoEvento, Long> implements ContratoEventoService {

	/** Atributo contratoEventoDao. */
	private ContratoEventoDao contratoEventoDao;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private EventoAditivoService eventoAditivoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private ReportController reportService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private EventoAditivoService aditivoService;

	@Autowired
	private EventoRepactuacaoService repactuacaoService;

	@Autowired
	private EventoRescisaoContratoService rescisaoContratoService;

	@Autowired
	private EventoRecebimentoObjetoService recebimentoObjetoService;

	@Autowired
	private ContratoEmpenhoService contratoEmpenhoService;

	@Autowired
	private EventoApostilamentoService apostilamentoService;

	@Autowired
	private EventoLiberacaoPagamentoService eventoLiberacaoPagamentoService;

	@Autowired
	private EventoRegistroNaoConformidadeService eventoRegistroNaoConformidadeService;

	@Autowired
	private EventoPenalidadeService eventoPenalidadeService;

	@Autowired
	private ContratoService contratoService;

	private Dominio idioma;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contratoEnvolvidoDao
	 * @param validator
	 */
	@Autowired
	public ContratoEventoServiceImpl( ContratoEventoDao contratoEventoDao ) {

		this.dao = contratoEventoDao;

		this.contratoEventoDao = contratoEventoDao;
	}

	@Override
	public Collection<ContratoEvento> findAllByContrato(Long contratoId) {

		return this.contratoEventoDao.findAllByContrato(contratoId);
	}

	@Override
	public Boolean removeEventoContrato(String justificativaRemocao, Long idEvento) {

		if (StringUtils.isEmpty(justificativaRemocao)) {
			throw new BusinessException(this.internacionalizacaoService.getTranslate("MSG.MN001", this.getIdioma()));
		}
		ContratoEvento contratoEvento = this.find(idEvento);

		if (contratoEvento instanceof EventoAditivo) {
			EventoAditivo aditivo = (EventoAditivo) contratoEvento;
			aditivo.setContratoAditivoInativo(aditivo.getContratoAditivo());
			aditivo.setContratoAditivo(null);
			this.eventoAditivoService.remove(aditivo);
		}

		contratoEvento.setJustificativaExclusao(justificativaRemocao);
		contratoEvento.setContratoInativo(contratoEvento.getContrato());
		contratoEvento.setContrato(null);
		this.remove(contratoEvento);
		return true;
	}

	public Dominio getIdioma() {

		if (!UtilObjeto.isReferencia(idioma)) {
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public Collection<ContratoEvento> findAllByContratoTipoEvento(Long contratoId, String tipoEvento) {

		return this.contratoEventoDao.findAllByContratoTipoEvento(contratoId, this.dominioService.findByChaveAndNome(DominioContrato.TIPO_EVENTO_CONTRATO, tipoEvento).getId());
	}

	@Override
	public Collection<ContratoEvento> findAllByContratoIntervalDataEmissao(Long contratoId, Date dataInicial, Date dateFinal) {

		return this.contratoEventoDao.findAllByContratoIntervalDataEmissao(contratoId, dataInicial, dateFinal);
	}

	@Override
	public Collection<ContratoEvento> findAllByContratoTipoTermoDataEmissao(Long[] codigosTermo, Date dataInicial, Date dateFinal) {
		return this.contratoEventoDao.findAllByContratoTipoTermoDataEmissao(Arrays.asList(codigosTermo), dataInicial, dateFinal);
	}

	@SuppressWarnings("deprecation")
	@Override
	public void gerarTermos(Long[] tiposTermo, Long idContrato, Date dataInicial, Date dateFinal, boolean download, HttpServletResponse response) {

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream inputSub = null;
		InputStream input = null;
		List<Long> idsContratos = getListaIdsContrato(tiposTermo, idContrato, dataInicial, dateFinal);
		if(idsContratos.isEmpty()){
			lancarBusinessException("CONTRATOS.LABEL.SEM_TERMOS_A_SER_GERADO", CodigoErro.REGRA_NEGOCIO.getValue(), true);
		}

		if(dateFinal != null){
			dateFinal.setHours(23);
			dateFinal.setMinutes(59);
			dateFinal.setSeconds(59);
		}

		try {
			conn = reportService.getConnection();
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/consultaTermo.jrxml");
			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);
			JasperPrint jasperPrint = null;
			String titulo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.TITULO_CONSULTA_TERMOS", getIdioma());
			List<JasperPrint> jasperPrintsCol = new ArrayList<JasperPrint>();
			Integer index = 0;
			for (Long id : idsContratos) {
				Contrato contrato = contratoService.find(id);

				Map<String, Object> subReportParameters = new HashMap<String, Object>();
				subReportParameters.put("periodo", getPeriodoInformado(dataInicial, dateFinal));
				subReportParameters.put("TIPO_TERMO", Arrays.asList(tiposTermo));
				subReportParameters.put("ID_CONTRATO", id);
				subReportParameters.put("dataInicial", dataInicial);
				subReportParameters.put("dataFinal", dateFinal);
				subReportParameters.put("contrato", contrato.getNumeroAnoContratoFormatado());
				subReportParameters.put("REPORT_CONNECTION", conn);


				Map<String, Object> parameters = new HashMap<String, Object>();
				parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
				parameters.put("TITULO", titulo);
				parameters.put("SUBREPORT_DIR", jasperReportSub);
				parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
				parameters.put("REPORT_CONNECTION", conn);

				if(index.equals(0)){
					jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);
				} else {
					jasperPrintsCol.add(JasperFillManager.fillReport(jasperReport, parameters, conn));
				}
				index ++;
			}

			if (!UtilColecao.isVazio(jasperPrintsCol)){
				for (JasperPrint jasperPrintPage : jasperPrintsCol) {
					for (int j = 0; j < jasperPrintPage.getPages().size(); j++) {
						JRPrintPage object = (JRPrintPage) jasperPrintPage.getPages().get(j);
						jasperPrint.addPage(object);
					}
				}

			}
			reportService.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);
		} catch (JRException jre) {
			throw new RuntimeException(jre);
		}  catch (Exception e) {
			throw new BusinessException();
		}finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}

	}

	private String getPeriodoInformado(Date dataInicial, Date dateFinal) {
		String periodo;
		if(dataInicial != null && dateFinal == null){
			periodo = UtilDate.formatarData(dataInicial) + " - " + this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.DATA_FINAL_NAO_INFORMADA", getIdioma());
		}else if(dataInicial == null && dateFinal != null){
			periodo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.DATA_INICIAL_NAO_INFORMADA", getIdioma()) + " - " + UtilDate.formatarData(dateFinal)  ;
		}else if(dataInicial != null && dateFinal != null){
			periodo = UtilDate.formatarData(dataInicial) + " - " + UtilDate.formatarData(dateFinal);
		}else{
			periodo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.PERIODO_NAO_INFORMADO", getIdioma());
		}
		return periodo;
	}

	private List<Long> getListaIdsContrato(Long[] tiposTermo, Long idContrato, Date dataInicial, Date dateFinal) {
		List<Long> idsContratos = new ArrayList<Long>();
		if(idContrato != null){
			idsContratos.add(idContrato);
		}else{
			Collection<ContratoEvento> eventos = this.findAllByContratoTipoTermoDataEmissao(tiposTermo, dataInicial, dateFinal);
			boolean addContrato = true;
			for (ContratoEvento contratoEvento : eventos) {
				for (Long contratoId : idsContratos) {
					if(contratoEvento.getContrato().getId() == contratoId){
						addContrato = false;
						break;
					}
				}
				if(addContrato){
					idsContratos.add(contratoEvento.getContrato().getId());
				}
				addContrato = true;
			}
		}
		return idsContratos;
	}

	@Override
	public void gerarRelatorios(Long[] idsTipoEvento, Long idContrato, Date dataInicial, Date dataFinal, boolean download, HttpServletResponse response) {

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream inputCabecalho = null;
		JasperPrint jasperPrint = null;

		try {
			conn = reportService.getConnection();

			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());

			List<JasperPrint> jasperPrints = new ArrayList<JasperPrint>();
			JasperPrint jasperPrincipal = new JasperPrint();

			if (UtilObjeto.isReferencia(idsTipoEvento)) {
				Integer index = 1;

				if (idContrato == null) {

					List<Long> idsContratos = obtemIdsContratosPorEvento(idsTipoEvento);
					
					for (Long contrato : idsContratos) {

						for (Long idTipoEvento : idsTipoEvento) {
							Dominio dominioTipoRelatorio = dominioService.find(idTipoEvento);
							
							jasperPrint = selecionaRelatorio(contrato, dataInicial, dataFinal, conn, jasperPrint, usuarioLogado, dominioTipoRelatorio);
							
							jasperPrincipal = addJasperPrint(jasperPrint, jasperPrints, jasperPrincipal, index);
							index++;
							continue;
						}
					} 
					

				} else {

					for (Long idTipoEvento : idsTipoEvento) {
						Dominio dominioTipoRelatorio = dominioService.find(idTipoEvento);

						jasperPrint = selecionaRelatorio(idContrato, dataInicial, dataFinal, conn, jasperPrint, usuarioLogado, dominioTipoRelatorio);

						jasperPrincipal = addJasperPrint(jasperPrint, jasperPrints, jasperPrincipal, index);
						index++;
						continue;
					}
				}

				if (!UtilColecao.isVazio(jasperPrints)) {
					for (JasperPrint jasper : jasperPrints) {
						for (int j = 0; j < jasper.getPages().size(); j++) {
							JRPrintPage object = (JRPrintPage) jasper.getPages().get(j);
							jasperPrincipal.addPage(object);
						}
					}
				}
				reportService.gerarRelatorio(download, "pdf", jasperPrincipal, response, baos, "");
			}

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		} finally {
			UtilDataBase.closeResources(conn, inputCabecalho, baos);
		}
	}

	private JasperPrint selecionaRelatorio(Long idContrato, Date dataInicial, Date dataFinal, Connection conn, JasperPrint jasperPrint, Usuario usuarioLogado, Dominio dominioTipoRelatorio)
			throws JRException {
		
		String titulo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.CONSULTA_RELATORIOS", this.getIdioma());

		if (dominioTipoRelatorio.getCodigo() == DominioContrato.TIPO_EVENTO_EMPENHO) {
			jasperPrint = this.contratoEmpenhoService.getJasperPrintRelatorioEmpenho(idContrato, dataInicial, dataFinal, conn, usuarioLogado, titulo);

		} else if (dominioTipoRelatorio.getCodigo() == DominioContrato.TIPO_EVENTO_LIBERA_PAGAMENTO) {
			jasperPrint = this.eventoLiberacaoPagamentoService.getJasperPrintRelatorioLiberacaoPagamento(idContrato, dataInicial, dataFinal, conn, usuarioLogado, titulo);

		} else if (dominioTipoRelatorio.getCodigo() == DominioContrato.TIPO_EVENTO_REPACTUACAO) {
			jasperPrint = this.repactuacaoService.getJasperPrintRelatorioRepactuacao(idContrato, dataInicial, dataFinal, conn, usuarioLogado, titulo);

		} else if (dominioTipoRelatorio.getCodigo() == DominioContrato.TIPO_EVENTO_NAO_CONFORMIDADE) {
			jasperPrint = this.eventoRegistroNaoConformidadeService.getJasperPrintRelatorioRegistroNaoConformidade(idContrato, dataInicial, dataFinal, conn, usuarioLogado, titulo);

		} else if (dominioTipoRelatorio.getCodigo() == DominioContrato.TIPO_EVENTO_PENALIDADE) {
			jasperPrint = this.eventoPenalidadeService.getJasperPrintRelatorioPenalidade(idContrato, dataInicial, dataFinal, conn, usuarioLogado, titulo);
		}

		return jasperPrint;
	}
	
	private JasperPrint addJasperPrint(JasperPrint jasperPrint, List<JasperPrint> jasperPrints, JasperPrint jasperPrincipal, Integer index) {

		if (index.equals(1)) {
			jasperPrincipal = jasperPrint;
		} else {
			jasperPrints.add(jasperPrint);
		}
		return jasperPrincipal;
	}

	private List<Long> obtemIdsContratosPorEvento(Long[] idsTipoEvento) {
		Collection<ContratoEvento> contratoEventos = contratoEventoDao.findAllByTipoEvento(Arrays.asList(idsTipoEvento));
		List<Long> idsContratos = new ArrayList<Long>();
		for (ContratoEvento contratoEvento : contratoEventos) {
			if (!idsContratos.contains(contratoEvento.getContrato().getId())) {
				idsContratos.add(contratoEvento.getContrato().getId());
			}
		}
		return idsContratos;
	}

}
