package br.com.centralit.api.service.impl;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Currency;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ContratoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.Colaborador;
import br.com.centralit.api.model.Contrato;
import br.com.centralit.api.model.ContratoDocumento;
import br.com.centralit.api.model.ContratoEmpenho;
import br.com.centralit.api.model.ContratoEnvolvido;
import br.com.centralit.api.model.ContratoEnvolvidoJustificativa;
import br.com.centralit.api.model.ContratoItem;
import br.com.centralit.api.model.ContratoObservacao;
import br.com.centralit.api.model.ContratoValorPagamento;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoAditivo;
import br.com.centralit.api.model.OrdemFornecimento;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ColaboradorService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ContratoEmpenhoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EstudoTecnicoService;
import br.com.centralit.api.service.EventoAditivoService;
import br.com.centralit.api.service.FornecedorService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.RascunhoService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.Util;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 25/01/2016 - 15:23:44
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Service("contratoService")
public class ContratoServiceImpl extends GenericServiceImpl<Contrato, Long>	implements ContratoService {

	/** Atributo contratoDao. */
	private ContratoDao contratoDao;

	/** Atributo usuarioService. */
	@Autowired
	private UsuarioService usuarioService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo parceiroServiceFornecedor. */
	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	/** Atributo anexoService. */
	@Autowired
	private AnexoService anexoService;

	/** Atributo colaboradorService. */
	@Autowired
	private ColaboradorService colaboradorService;

	/** Atributo fornecedorService. */
	@Autowired
	private FornecedorService fornecedorService;

	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	@Autowired
	private EstudoTecnicoService estudoTecnicoService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ContratoEmpenhoService contratoEmpenhoService;

	@Autowired
	private EventoAditivoService aditivoService;

	@Autowired
	private RascunhoService rascunhoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param contratoDao
	 * @param validator
	 */
	@Autowired
	public ContratoServiceImpl(ContratoDao contratoDao, @Qualifier("contratoValidator") Validator validator) {

		this.dao = contratoDao;

		this.contratoDao = contratoDao;

		this.validator = validator;
	}

	/**
	 * Método responsável por salvar todas as informações informadas na tela de cadastro
	 * montar objeto de contrato removendo poss\u00edveis problemas de transient
	 *
	 * @author iago.almeida
	 *
	 * @param contrato
	 * 			objeto que será salvo
	 */
	@Override
	public Contrato save(Contrato contrato) {

		// realiza a validação do objeto com a classe ContratoValidator.java
		this.validarEntidade(contrato, this.validator);

		// monta objeto contrato com todas as suas listas e objetos transients
		this.montarObjetoContrato(contrato);

		if (contrato.getValorGarantia() != null) {
			validarValorGarantia(contrato);
		}

		validarExistenciaContratoMesmoNumeroAno(contrato);

		validarPapeisEnvolvidos(contrato);

		rascunhoService.removeById(contrato.getRascunho());
		
		return super.save(contrato);
	}

	/**
	 * Método responsável por atualizar todas as informações informadas na tela de cadastro
	 * montar objeto de contrato removendo poss\u00edveis problemas de transient
	 *
	 * @author iago.almeida
	 *
	 * @param contrato
	 * 			objeto que será atualizado
	 */
	@Override
	public Contrato merge(Contrato contrato) {

		// realiza a validação do objeto com a classe EstudoTecnicoValidator.java
		this.validarEntidade(contrato, this.validator);

		// monta objeto estudoTecnico com todas as suas listas e objetos transients
		this.montarObjetoContrato(contrato);

		validarPapeisEnvolvidos(contrato);

		return super.save(contrato);
	}

	/**
	 * Método responsável por tratar objetos imcompletos (transients) que sao necessarios
	 * para que seja salvo o objeto contrato
	 *
	 * @author iago.almeida
	 *
	 * @param contrato
	 * 			objeto que será resolvido os transients
	 */
	private void montarObjetoContrato(Contrato contrato) {

		// resolve os transients dos documentos e observacoes do contrato
		this.montarDocsObsContrato(contrato);

		contrato.setDominioFormaPagamentoContrato(this.dominioService.find(contrato.getDominioFormaPagamentoContrato().getId()));

		contrato.setDominioModalidadeLicitacao(this.dominioService.find(contrato.getDominioModalidadeLicitacao().getId()));
		contrato.setDominioSituacaoAtual(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_SITUACAO_CONTRATO, contrato.getDominioSituacaoAtual().getCodigo()));
		contrato.setDominioTipoAquisicao(this.dominioService.find(contrato.getDominioTipoAquisicao().getId()));
		contrato.setDominioTipoContrato(this.dominioService.find(contrato.getDominioTipoContrato().getId()));

		// Tipo de Licitação nao é obrigatorio para modalidades de licitação que não sejam pregao e concurso, quando vier da tela busca a referencia
		if (UtilObjeto.isReferencia(contrato.getDominioTipoLicitacao())){
			contrato.setDominioTipoLicitacao(this.dominioService.find(contrato.getDominioTipoLicitacao().getId()));
		}
		contrato.setFornecedor(this.fornecedorService.find(contrato.getFornecedor().getId()));

		// Modalidade da garantia nao é obrigatoria, quando vier da tela busca a referencia
		if (UtilObjeto.isReferencia(contrato.getDominioModalidadeGarantia())){
			contrato.setDominioModalidadeGarantia(this.dominioService.find(contrato.getDominioModalidadeGarantia().getId()));
		}
		// cria vinculo do contrato com as ctrtValoresPagamento
		if (!UtilColecao.isVazio(contrato.getCtrtValoresPagamento())) {

			for (ContratoValorPagamento contratoValorPagamento : contrato.getCtrtValoresPagamento()) {

				contratoValorPagamento.setContrato(contrato);
			}
		}

		// cria vinculo do contrato com as ctrtEmpenhos
		if (!UtilColecao.isVazio(contrato.getCtrtEmpenhos())){

			for (ContratoEmpenho contratoEmpenho : contrato.getCtrtEmpenhos()) {

				contratoEmpenho.setDominioCategoriaEconomica(this.dominioService.find(contratoEmpenho.getDominioCategoriaEconomica().getId()));

				contratoEmpenho.setDominioFinalidadeEmpenho(this.dominioService.find(contratoEmpenho.getDominioFinalidadeEmpenho().getId()));

				contratoEmpenho.setContrato(contrato);
				contratoEmpenho.setContratoEmpenho(contrato);

				contratoEmpenho.setDataEvento(contratoEmpenho.getData());

				contratoEmpenho.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_EMPENHO));

				Dominio idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
				String descricaoEmpenho = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EMPENHO", idioma);
				descricaoEmpenho = descricaoEmpenho.replace(":VALOR:", Util.convertBigDecimalToString(contratoEmpenho.getValorEmpenhado()));
				descricaoEmpenho = descricaoEmpenho.replace(":DATAEVENTO:", UtilDate.formatarData(contratoEmpenho.getData().getTime(), "dd/MM/yyyy"));

				contratoEmpenho.setDescricao(descricaoEmpenho);

				if(UtilObjeto.isReferencia(contratoEmpenho.getId())){
					ContratoEmpenho empenho = this.contratoEmpenhoService.find(contratoEmpenho.getId());
					if(!UtilColecao.isVazio(empenho.getObservacoes())){
						contratoEmpenho.setObservacoes(empenho.getObservacoes());
					}
					if(!UtilColecao.isVazio(empenho.getDocumentos())){
						contratoEmpenho.setDocumentos(empenho.getDocumentos());
					}
				}

			}
		}

		// cria vinculo do contrato com as ctrtItens
		if (!UtilColecao.isVazio(contrato.getCtrtItens())){

			for (ContratoItem contratoItem : contrato.getCtrtItens()) {

				contratoItem.setContrato(contrato);

				contratoItem.setEstudoTecnico(this.estudoTecnicoService.find(contratoItem.getEstudoTecnico().getId()));

				if (UtilObjeto.isReferencia(contratoItem.getEstudoTecnico().getId())){

					contratoItem.setEstudoTecnicoBemServicoSolucao(contratoItem.getEstudoTecnicoBemServicoSolucao());
				}
			}
		}

		// cria vinculo do contrato com os envolvidos
		if (CollectionUtils.isNotEmpty(contrato.getCtrtEnvolvidos())){
			for (ContratoEnvolvido contratoEnvolvido : contrato.getCtrtEnvolvidos()) {
				contratoEnvolvido.setEnvolvido(this.colaboradorService.find(contratoEnvolvido.getEnvolvido().getId()));
				contratoEnvolvido.setContrato(contrato);
				if (CollectionUtils.isNotEmpty(contratoEnvolvido.getJustificativas())){
					for(ContratoEnvolvidoJustificativa justificativa : contratoEnvolvido.getJustificativas()){
						justificativa.setContratoEnvolvido(contratoEnvolvido);
						justificativa.setAutor(this.usuarioService.find(justificativa.getAutor().getId()));
					}
				}
			}
			}

		if (!UtilColecao.isVazio(contrato.getAditivos())){
			for (EventoAditivo aditivo : contrato.getAditivos()) {
				EventoAditivo eventoAd = this.aditivoService.find(aditivo.getId());
				aditivo.setDataEmissao(eventoAd.getDataEmissao());
				aditivo.setTipoAlteracao(eventoAd.getTipoAlteracao());
				aditivo.setJustificativa(eventoAd.getJustificativa());
				if(!UtilColecao.isVazio(eventoAd.getObservacoes())){
					aditivo.setObservacoes(eventoAd.getObservacoes());
				}
				if(!UtilColecao.isVazio(eventoAd.getDocumentos())){
					aditivo.setDocumentos(eventoAd.getDocumentos());
				}
			}
		}

		if (!UtilColecao.isVazio(contrato.getOrdensFornecimento())){
			for (OrdemFornecimento ordem : contrato.getOrdensFornecimento()) {
				ordem.setContrato(contrato);
			}
		}

	}

	/**
	 * Método responsável por resolver os transients de documentos e observacoes
	 *
	 * @author iago.almeida
	 *
	 * @param contrato
	 */
	@SuppressWarnings("rawtypes")
	private void montarDocsObsContrato(Contrato contrato) {

		// cria vinculo do contrato com as Observacoes
		if (!UtilColecao.isVazio(contrato.getObservacoes())) {

			for (ContratoObservacao contratoObservacao : contrato.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(contratoObservacao.getAutor())) {
					idAutor = contratoObservacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				contratoObservacao.setAutor(this.usuarioService.find(idAutor));

				contratoObservacao.setContrato(contrato);
			}
		}

		// cria vinculo do contrato com os Documentos
		if (!UtilColecao.isVazio(contrato.getDocumentos())) {

			for (ContratoDocumento contratoDocumento : contrato.getDocumentos()) {

				contratoDocumento.setDominioTipoDocumento(this.dominioService.find(contratoDocumento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(contratoDocumento.getEmitente()) && UtilObjeto.isReferencia(contratoDocumento.getEmitente().getId())) {

					contratoDocumento.setEmitente(this.parceiroServiceFornecedor.find(contratoDocumento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(contratoDocumento.getAnexos())){
					Anexo anexo = (Anexo) ((List)contratoDocumento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					contratoDocumento.setAnexos(listaAnexo);
				}

				contratoDocumento.setContrato(contrato);
			}
		}
	}

	private void validarValorGarantia(Contrato contrato) {
		BigDecimal valorContratoReferenciaGarantia = this.configuracaoParametroSistemaService.obterValorContratoReferenciaGarantia();
		BigDecimal percentualLimiteGarantia = BigDecimal.ZERO;
		BigDecimal valorLimiteGarantia = BigDecimal.ZERO;

		if (contrato.getValorContrato().compareTo(valorContratoReferenciaGarantia) > 0) {
			percentualLimiteGarantia = new BigDecimal(10);
		} else {
			percentualLimiteGarantia = new BigDecimal(5);
		}
		valorLimiteGarantia = (contrato.getValorContrato().multiply(percentualLimiteGarantia)).divide(new BigDecimal(100));

		boolean condicao = (contrato.getPercentualGarantia().multiply(new BigDecimal(100))).compareTo(percentualLimiteGarantia) == 1;

		NumberFormat formatter = NumberFormat.getCurrencyInstance();
		formatter.setCurrency(Currency.getInstance("BRL"));

		lancarBusinessException(
				condicao, "CONTRATOS.VALIDACAO.VALOR_GARANTIA_ACIMA_LIMITE_MAXIMO", CodigoErro.REGRA_NEGOCIO.getValue(), formatter.format(valorLimiteGarantia));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<Contrato> findContrato(String value, Long idOrganizacao) {
		return this.contratoDao.findContrato(value, idOrganizacao);
	}

	@Override
	public Colaborador getGestor(Long idContrato) {
		Contrato contrato = this.find(idContrato);
		for (Iterator<ContratoEnvolvido> iterator = contrato.getCtrtEnvolvidos().iterator(); iterator.hasNext();) {
			ContratoEnvolvido contratoEnvolvido = iterator.next();
			if (contratoEnvolvido.getDominioPapelEnvolvidoContrato().getCodigo() == DominioContrato.CODIGO_TIPO_PAPEL_ENVOLVIDO_CONTRATO_GESTOR) {
				return contratoEnvolvido.getEnvolvido();
			}
		}
		return null;
	}

	/**
	 * Método responsável por listar contratos por numero e organizacao
	 *
	 * @author luis.camargo
	 *
	 * @param numeroAnoContrato
	 * @param idOrganizacao
	 *
	 * @return Collection<Contrato>
	 */
	@Override
	public Collection<Contrato> findAutoCompleteContratoByNumeroAndOrganizacao(String numeroAnoContrato, Long idOrganizacao) {
		return contratoDao.findAutoCompleteContratoByNumeroAndOrganizacao(numeroAnoContrato, idOrganizacao);
	}

	@Override
	public Collection<Contrato> findAutoCompleteContratoPorNumeroAndOrganizacao(String numeroAnoContrato, Long idOrganizacao) {

		return contratoDao.findAutoCompleteContratoPorNumeroAndOrganizacao(numeroAnoContrato, idOrganizacao);
	}

	/**
	 * Método responsável validar se os papéis obrigatórios dos envolvidos no contrado foram informados.
	 *
	 * @author luis.camargo
	 *
	 * @param contrato
	 *
	 */
	private void validarPapeisEnvolvidos(Contrato contrato) {
		List<Long> papeisInformados = new ArrayList<Long>();

		for (ContratoEnvolvido envolvido : contrato.getCtrtEnvolvidos()) {
			papeisInformados.add(envolvido.getDominioPapelEnvolvidoContrato().getCodigo());
		}

		StringBuilder complemento = new StringBuilder().append(
				DominioContrato.DESCRICAO_TIPO_PAPEL_ENVOLVIDO_CONTRATO_GESTOR).append(", ").append(
						DominioContrato.DESCRICAO_TIPO_PAPEL_ENVOLVIDO_CONTRATO_FISCAL_ADMINISTRATIVO).append(" e ").append(
								DominioContrato.DESCRICAO_TIPO_PAPEL_ENVOLVIDO_CONTRATO_FISCAL_TECNICO);

		boolean hasObrigatorios = papeisInformados.contains(DominioContrato.CODIGO_TIPO_PAPEL_ENVOLVIDO_CONTRATO_GESTOR) &&
				papeisInformados.contains(DominioContrato.CODIGO_TIPO_PAPEL_ENVOLVIDO_CONTRATO_FISCAL_TECNICO) &&
				papeisInformados.contains(DominioContrato.CODIGO_TIPO_PAPEL_ENVOLVIDO_CONTRATO_FISCAL_ADMINISTRATIVO);
		lancarBusinessException(!hasObrigatorios, "CONTRATOS.VALIDACAO.PAPEIS_OBRIGATORIOS_CONTRATO_FALTANDO", CodigoErro.REGRA_NEGOCIO.getValue(), complemento.toString());
	}

	/**
	 * Método responsável validar se existe contrato com o mesmo numero/ano.
	 *
	 * @author ronan.camargo
	 *
	 * @param contrato
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void validarExistenciaContratoMesmoNumeroAno(Contrato contrato) {

		List<Contrato> contratos = (List) contratoDao.buscarPorAtributo("numeroAnoContrato", contrato.getNumeroAnoContrato());

		boolean isExistente = contratos != null && contratos.size() > 0;

		lancarBusinessException("CONTRATOS.VALIDACAO.NUMERO_CONTRATO_DUPLICADO", CodigoErro.REGRA_NEGOCIO.getValue(), isExistente);
	}
}
