package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.CronogramaExecucaoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.CronogramaExecucao;
import br.com.centralit.api.model.CronogramaExecucaoDocumento;
import br.com.centralit.api.model.CronogramaExecucaoItem;
import br.com.centralit.api.model.CronogramaExecucaoItemParcela;
import br.com.centralit.api.model.CronogramaExecucaoObservacao;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.CronogramaExecucaoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

@Service("cronogramaExecucaoService")
public class CronogramaExecucaoServiceImpl extends GenericServiceImpl<CronogramaExecucao, Long> implements CronogramaExecucaoService {

	private CronogramaExecucaoDao cronogramaExecucaoDao;

	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;
	@Autowired
	private UsuarioService usuarioService;
	@Autowired
	private DominioService dominioService;
	@Autowired
	private ParceiroService parceiroServiceFornecedor;
	@Autowired
	private AnexoService anexoService;
	@Autowired
	private ReportController reportService;
	@Autowired
	private InternacionalizacaoService internacionalizacaoService;
	@Autowired
	private ContratoService contratoService;

	@Autowired
	public CronogramaExecucaoServiceImpl(CronogramaExecucaoDao cronogramaExecucaoDao, @Qualifier("cronogramaExecucaoValidator") Validator validator ) {

		this.cronogramaExecucaoDao = cronogramaExecucaoDao;
		this.dao = cronogramaExecucaoDao;
		this.validator = validator;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por salvar um <code>CronogramaExecucao</code>
	 *
	 * @author luis.camargo
	 *
	 * @param cronogramaExecucao
	 * @return <code>CronogramaExecucao</code>
	 */
	@Override
	public CronogramaExecucao save(CronogramaExecucao cronogramaExecucao) {

		validarEntidade(cronogramaExecucao, this.validator);

		configurarCodigo(cronogramaExecucao);

		montarObjetoCronograma(cronogramaExecucao);

		validarDataEntregaNaVigenciaContrato(cronogramaExecucao);

		validarValorTotalItensAcimaValorContrato(cronogramaExecucao);

		return (CronogramaExecucao) this.cronogramaExecucaoDao.save(cronogramaExecucao);
	}

	@Override
	public CronogramaExecucao merge(CronogramaExecucao cronogramaExecucao) {

		validarEntidade(cronogramaExecucao, this.validator);

		montarObjetoCronograma(cronogramaExecucao);

		validarDataEntregaNaVigenciaContrato(cronogramaExecucao);

		validarValorTotalItensAcimaValorContrato(cronogramaExecucao);

		return (CronogramaExecucao) this.cronogramaExecucaoDao.merge(cronogramaExecucao);
	}

	/**
	 * Método responsável por gerar o relatório de cronograma de execução
	 *
	 * @author luis.camargo
	 *
	 * @param idCronograma
	 * @param download
	 * @param response
	 */
	public void gerarPDFCronogramaExecucao(Long idCronograma, boolean download, HttpServletResponse response) {
		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;

		try {
			conn = reportService.getConnection();
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/cronogramaExecucao.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);

			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			CronogramaExecucao cronograma = find(idCronograma);
			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("CRONOGRAMA_EXECUCAO_ID", idCronograma);
			subReportParameters.put("OBSERVACAO_NAO_APLICA", cronograma.getObservacoes().size() > 0);
			subReportParameters.put("REPORT_CONNECTION", conn);

			Dominio idioma = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
			String titulo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.CRONOGRAMA_EXECUCAO", idioma);

			Usuario user = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", user.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			JasperPrint jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);

			reportService.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		} catch (Exception e) {
			throw new BusinessException();
		} finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}
	private void validarDataEntregaNaVigenciaContrato(CronogramaExecucao cronogramaExecucao) {
		for (CronogramaExecucaoItem item : cronogramaExecucao.getItensCronogramaExecucao()) {
			for (CronogramaExecucaoItemParcela parcela : item.getParcelas()) {

				boolean isBefore = parcela.getDataEntrega().before(cronogramaExecucao.getContrato().getDataVigenciaInicial());
				boolean isAfter = parcela.getDataEntrega().after(cronogramaExecucao.getContrato().getDataUltimaVigenciaFinal());

				lancarBusinessException("CONTRATOS.VALIDACAO.DATA_ENTREGA_CRONOGRAMA_FORA_VIGENCIA", CodigoErro.REGRA_NEGOCIO.getValue(), (isBefore || isAfter));
			}
		}
	}

	private void validarValorTotalItensAcimaValorContrato(CronogramaExecucao cronogramaExecucao) {
		boolean condicao = cronogramaExecucao.getValorTotal().compareTo(cronogramaExecucao.getContrato().getValorTotalContrato()) == 1;
		lancarBusinessException("CONTRATOS.VALIDACAO.VALOR_TOTAL_CRONOGRAMA_ACIMA_VALOR_CONTRATO", CodigoErro.REGRA_NEGOCIO.getValue(), condicao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por settar o atributo codigo.
	 *
	 * @author luis.camargo
	 *
	 * @param entity
	 */
	private void configurarCodigo(CronogramaExecucao entity) {
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		String ano = UtilDate.getAnoAtual();

		if (!UtilObjeto.isReferencia(entity.getOrganizacao())) {
			entity.setOrganizacao(this.getUsuario().getOrganizacao());
		}

		CronogramaExecucao ultimoRegistro = (CronogramaExecucao) cronogramaExecucaoDao.buscarUltimoAnoRegistroComOrdenadoParametrizada("codigo", ano, usuarioLogado.getOrganizacao().getId());

		Long sequencial = 0L;
		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getCodigo() == null ? 0 : Long.parseLong(ultimoRegistro.getCodigo().split("/")[0]);
		}

		Calendar calendar = Calendar.getInstance();
		String anoAtual = String.valueOf(calendar.get(Calendar.YEAR));
		String codigo =  StringUtils.leftPad((++sequencial).toString(), 5, "0");

		entity.setCodigo(codigo.concat("/").concat(anoAtual));
	}

	private void montarObjetoCronograma(CronogramaExecucao cronogramaExecucao) {

		cronogramaExecucao.setContrato(contratoService.getReference(cronogramaExecucao.getContrato().getId()));

		// resolve os transients dos documentos e observacoes do cronograma
		montarDocsObsCronograma(cronogramaExecucao);

		// cria vinculo do CronogramaExecucao com as CronogramaExecucaoItem
		for (CronogramaExecucaoItem itemCronograma : cronogramaExecucao.getItensCronogramaExecucao()) {

			itemCronograma.setCronogramaExecucao(cronogramaExecucao);

			// cria vinculo do CronogramaExecucaoItem com as CronogramaExecucaoItemParcela
			for (CronogramaExecucaoItemParcela parcela : itemCronograma.getParcelas()) {
				parcela.setCronogramaExecucaoItem(itemCronograma);
			}
		}
	}

	/**
	 * Método responsável por resolver os transients de documentos e observacoes
	 *
	 * @author luis.camargo
	 *
	 * @param cronogramaExecucao
	 */
	@SuppressWarnings("rawtypes")
	private void montarDocsObsCronograma(CronogramaExecucao cronogramaExecucao) {

		// cria vinculo do contrato com as Observacoes
		if (!UtilColecao.isVazio(cronogramaExecucao.getObservacoes())) {

			for (CronogramaExecucaoObservacao observacao : cronogramaExecucao.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(observacao.getAutor())) {
					idAutor = observacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				observacao.setAutor(this.usuarioService.find(idAutor));

				observacao.setCronogramaExecucao(cronogramaExecucao);
			}
		}

		// cria vinculo do contrato com os Documentos
		if (!UtilColecao.isVazio(cronogramaExecucao.getDocumentos())) {

			for (CronogramaExecucaoDocumento documento : cronogramaExecucao.getDocumentos()) {

				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(documento.getEmitente()) && UtilObjeto.isReferencia(documento.getEmitente().getId())) {

					documento.setEmitente(this.parceiroServiceFornecedor.find(documento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(documento.getAnexos())){
					Anexo anexo = (Anexo) ((List)documento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					documento.setAnexos(listaAnexo);
				}

				documento.setCronogramaExecucao(cronogramaExecucao);
			}
		}
	}
}
