package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EncerramentoContratoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.EncerramentoContrato;
import br.com.centralit.api.model.EncerramentoContratoDocumento;
import br.com.centralit.api.model.EncerramentoContratoObservacao;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EncerramentoContratoService;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 24/02/2016 - 15:51:45
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("encerramentoContratoService")
public class EncerramentoContratoServiceImpl extends GenericServiceImpl<EncerramentoContrato, Long>	implements EncerramentoContratoService {

	/** Atributo contratoDao. */
	private EncerramentoContratoDao encerramentoContratoDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private OrganizacaoService organizacaoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private ParceiroService parceiroService;

	@Autowired
	private AnexoService anexoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param encerramentoContratoDao
	 * @param validator
	 */
	@Autowired
	public EncerramentoContratoServiceImpl(EncerramentoContratoDao encerramentoContratoDao, @Qualifier("encerramentoContratoValidator") Validator validator) {

		this.dao = encerramentoContratoDao;

		this.encerramentoContratoDao = encerramentoContratoDao;

		this.validator = validator;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public EncerramentoContrato save(EncerramentoContrato encerramentoContrato) {

		this.montarObjeto(encerramentoContrato);

		configurarNumeroTermo(encerramentoContrato);;

		if (!UtilObjeto.isReferencia(encerramentoContrato.getNumTermo()) && UtilObjeto.isReferencia(encerramentoContrato.getDataEmissao())) {
			encerramentoContrato.setNumTermo(this.geraNumeroTermo(encerramentoContrato.getDataEmissao()));
		}

		this.validarEntidade(encerramentoContrato, this.validator);

		return super.save(encerramentoContrato);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public EncerramentoContrato merge(EncerramentoContrato encerramentoContrato) {

		this.montarObjeto(encerramentoContrato);

		this.validarEntidade(encerramentoContrato, this.validator);

		return super.save(encerramentoContrato);

	}

	/**
	 *
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por
	 *
	 * @author geovane.filho
	 *
	 * @param encerramentoContrato
	 */
	private void montarObjeto(EncerramentoContrato encerramentoContrato) {

		// resolve os transients dos documentos e observacoes
		this.montarDocsObsContrato(encerramentoContrato);

		if (UtilObjeto.isReferencia(encerramentoContrato.getContrato())) {
			encerramentoContrato.setContrato(this.contratoService.find(encerramentoContrato.getContrato().getId()));
		}

		if (UtilObjeto.isReferencia(encerramentoContrato.getContratante())) {
			encerramentoContrato.setContratante(this.organizacaoService.find(encerramentoContrato.getContratante().getId()));
		}

	}

	/**
	 * Método responsável por resolver os transients de documentos e observacoes
	 *
	 * @author iago.almeida
	 *
	 * @param encerramentoContrato
	 */
	@SuppressWarnings("rawtypes")
	private void montarDocsObsContrato(EncerramentoContrato encerramentoContrato) {

		// cria vinculo do contrato com as Observacoes
		if (!UtilColecao.isVazio(encerramentoContrato.getObservacoes())) {

			for (EncerramentoContratoObservacao encerramentoObservacao : encerramentoContrato.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(encerramentoObservacao.getAutor())) {
					idAutor = encerramentoObservacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				encerramentoObservacao.setAutor(this.usuarioService.find(idAutor));

				encerramentoObservacao.setEncerramentoContrato(encerramentoContrato);
			}
		}

		// cria vinculo do contrato com os Documentos
		if (!UtilColecao.isVazio(encerramentoContrato.getDocumentos())) {

			for (EncerramentoContratoDocumento encerramentoDocumento : encerramentoContrato.getDocumentos()) {

				encerramentoDocumento.setDominioTipoDocumento(this.dominioService.find(encerramentoDocumento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(encerramentoDocumento.getEmitente()) && UtilObjeto.isReferencia(encerramentoDocumento.getEmitente().getId())) {

					encerramentoDocumento.setEmitente(this.parceiroService.find(encerramentoDocumento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(encerramentoDocumento.getAnexos())){
					Anexo anexo = (Anexo) ((List)encerramentoDocumento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					encerramentoDocumento.setAnexos(listaAnexo);
				}

				encerramentoDocumento.setEncerramentoContrato(encerramentoContrato);
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String geraNumeroTermo(Calendar dataEmissao) {
		return this.encerramentoContratoDao.geraNumeroTermo(dataEmissao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por settar o atributo num.
	 *
	 * @author luis.camargo
	 *
	 * @param entity
	 */
	private void configurarNumeroTermo(EncerramentoContrato entity) {

		Integer ano = entity.getDataEmissao().get(Calendar.YEAR);
		EncerramentoContrato ultimoRegistro = (EncerramentoContrato) encerramentoContratoDao.buscarUltimoAnoRegistroComOrdenadoParametrizada("numTermo", ano.toString(), null);
		Long sequencial = null;

		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getNumTermo().replace(ano.toString(), "") == null ? null : Long.parseLong(ultimoRegistro.getNumTermo().replace(ano.toString(), "") );
		}else{
			sequencial = 0l;
		}

		entity.setNumTermo(StringUtils.leftPad((Long.valueOf(sequencial + 1)).toString(), 5, "0") + ano);
	}

}
