package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EstudoTecnicoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.EstudoTecnico;
import br.com.centralit.api.model.EstudoTecnicoAcaoContingencia;
import br.com.centralit.api.model.EstudoTecnicoBemServicoSolucao;
import br.com.centralit.api.model.EstudoTecnicoBeneficioEsperado;
import br.com.centralit.api.model.EstudoTecnicoDocumento;
import br.com.centralit.api.model.EstudoTecnicoLevantamentoAlternativas;
import br.com.centralit.api.model.EstudoTecnicoObservacao;
import br.com.centralit.api.model.EstudoTecnicoQuestionarioResposta;
import br.com.centralit.api.model.EstudoTecnicoRecursoHumano;
import br.com.centralit.api.model.EstudoTecnicoRecursoMaterial;
import br.com.centralit.api.model.EstudoTecnicoRequisitoNegocio;
import br.com.centralit.api.model.Fornecedor;
import br.com.centralit.api.model.OrgaoExterno;
import br.com.centralit.api.model.QuestionarioPergunta;
import br.com.centralit.api.model.RegistroSequencial;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ColaboradorService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EstudoTecnicoService;
import br.com.centralit.api.service.FuncaoService;
import br.com.centralit.api.service.ObjetivoEstrategicoNecessidadeService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.RascunhoService;
import br.com.centralit.api.service.TermoReferenciaService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 05/01/2016 - 15:30:24
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Service("estudoTecnicoService")
public class EstudoTecnicoServiceImpl extends GenericServiceImpl<EstudoTecnico, Long> implements EstudoTecnicoService {

	private static final String MSG_EXCLUSAO_NEGADA_TERMO_REFERENCIA = "MSG.EXCLUSAO_ET_NEGADA_TERMO_REFERENCIA";

	/** Atributo estudoTecnicoDao. */
	private EstudoTecnicoDao estudoTecnicoDao;

	/** Atributo usuarioService. */
	@Autowired
	private UsuarioService usuarioService;

	/** Atributo configuracaoParametroSistemaService. */
	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	/** Atributo anexoService. */
	@Autowired
	private AnexoService anexoService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo parceiroServiceFornecedor. */
	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	/** Atributo parceiroServiceOrgaoExterno. */
	@Autowired
	private ParceiroService parceiroServiceOrgaoExterno;

	/** Atributo colaboradorService. */
	@Autowired
	private ColaboradorService colaboradorService;

	/** Atributo objetivoEstrategicoNecessidadeService. */
	@Autowired
	private ObjetivoEstrategicoNecessidadeService objetivoEstrategicoNecessidadeService ;

	@Autowired
	private TermoReferenciaService termoReferenciaService;

	@Autowired
	private FuncaoService funcaoService;

	@Autowired
	private RascunhoService rascunhoService;
	
	/** Atributo statico CAMPO_BUSCA_PARAMETRIZADA_ULTIMO_REGISTRO. */
	public static final String CAMPO_BUSCA_PARAMETRIZADA_ULTIMO_REGISTRO = "codigo";

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param estudoTecnicoDao
	 * @param validator
	 */
	@Autowired
	public EstudoTecnicoServiceImpl( EstudoTecnicoDao estudoTecnicoDao, @Qualifier("estudoTecnicoValidator") Validator validator ) {

		this.dao = estudoTecnicoDao;

		this.estudoTecnicoDao = estudoTecnicoDao;

		this.validator = validator;
	}

	/**
	 * Método responsável por salvar todas as informações informadas na tela de cadastro
	 * montar objeto da estudoTecnico removendo poss\u00edveis problemas de transient
	 *
	 * @author iago.almeida
	 *
	 * @param estudoTecnico
	 * 			objeto que será salvo
	 */
	@Override
	public EstudoTecnico save(EstudoTecnico estudoTecnico) {

		// realiza a validação do objeto com a classe EstudoTecnicoValidator.java
		this.validarEntidade(estudoTecnico, this.validator);

		// monta objeto estudoTecnico com todas as suas listas e objetos transients
		this.montarObjetoEstudoTecnico(estudoTecnico);

		rascunhoService.removeById(estudoTecnico.getRascunho());
		
		// verifica se tem codigo, senao gera e salva entidade
		if (UtilObjeto.isReferencia(estudoTecnico.getRegistroSequencial())) {

			return super.save(estudoTecnico);

		} else {

			this.configurarCodigo(estudoTecnico);

			EstudoTecnico entitySaved = super.save(estudoTecnico);

			return entitySaved;

		}

	}

	/**
	 * Método responsável por atualizar todas as informações informadas na tela de cadastro
	 * montar objeto da estudoTecnico removendo poss\u00edveis problemas de transient
	 *
	 * @author iago.almeida
	 *
	 * @param estudoTecnico
	 * 			objeto que será atualizado
	 */
	@Override
	public EstudoTecnico merge(EstudoTecnico estudoTecnico) {

		// realiza a validação do objeto com a classe EstudoTecnicoValidator.java
		this.validarEntidade(estudoTecnico, this.validator);

		// monta objeto estudoTecnico com todas as suas listas e objetos transients
		this.montarObjetoEstudoTecnico(estudoTecnico);

		// verifica se tem codigo, senao gera e salva entidade
		if (UtilObjeto.isReferencia(estudoTecnico.getRegistroSequencial())) {

			return super.save(estudoTecnico);

		} else {

			this.configurarCodigo(estudoTecnico);

			EstudoTecnico entitySaved = super.save(estudoTecnico);

			return entitySaved;
		}
	}

	/**
	 * Método responsável por tratar objetos imcompletos (transients) que sao necessarios
	 * para que seja salvo o objeto estudoTecnico
	 *
	 * @author iago.almeida
	 *
	 * @param estudoTecnico
	 * 			objeto que será resolvido os transients
	 */
	private void montarObjetoEstudoTecnico(EstudoTecnico estudoTecnico) {

		// resolve os transients de integrante do estudo tecnico
		this.montarIntegranteEstudoTecnico(estudoTecnico);

		// resolve os transients dos documentos e observacoes do estudo tecnico
		this.montarDocsObsEstudoTecnico(estudoTecnico);

		// cria vinculo do estudo tecnico com as etAcoesContingencia
		if (!UtilColecao.isVazio(estudoTecnico.getEtAcoesContingencia())){

			// itera sobre estudoTecnicoAcaoContingencia para resolucao de seus atributos transients
			for (EstudoTecnicoAcaoContingencia estudoTecnicoAcaoContingencia : estudoTecnico.getEtAcoesContingencia()) {

				if (UtilObjeto.isReferencia(estudoTecnicoAcaoContingencia.getResponsavel()) && UtilObjeto.isReferencia(estudoTecnicoAcaoContingencia.getResponsavel().getId())){

					// busca para o estudoTecnicoAcaoContingencia o responsavel
					estudoTecnicoAcaoContingencia.setResponsavel(this.funcaoService.find(estudoTecnicoAcaoContingencia.getResponsavel().getId()));
				}

				estudoTecnicoAcaoContingencia.setEstudoTecnico(estudoTecnico);
			}
		}

		// cria vinculo do estudo tecnico com as etBeneficiosEsperados
		if (!UtilColecao.isVazio(estudoTecnico.getEtBeneficiosEsperados())){

			for (EstudoTecnicoBeneficioEsperado estudoTecnicoBeneficioEsperado : estudoTecnico.getEtBeneficiosEsperados()) {

				estudoTecnicoBeneficioEsperado.setEstudoTecnico(estudoTecnico);
			}
		}

		// cria vinculo do estudo tecnico com as etBensServicosSolucoes
		if (!UtilColecao.isVazio(estudoTecnico.getEtBensServicosSolucoes())){

			for (EstudoTecnicoBemServicoSolucao estudoTecnicoBemServicoSolucao : estudoTecnico.getEtBensServicosSolucoes()) {

				estudoTecnicoBemServicoSolucao.setEstudoTecnico(estudoTecnico);
			}
		}

		// cria vinculo do estudo tecnico com as etLevantamentoAlternativas
		if (!UtilColecao.isVazio(estudoTecnico.getEtLevantamentoAlternativas())){

			// itera sobre estudoTecnicoLevantamentoAlternativas para resolucao de seus atributos transients
			for (EstudoTecnicoLevantamentoAlternativas estudoTecnicoLevantamentoAlternativas : estudoTecnico.getEtLevantamentoAlternativas()) {

				if (UtilObjeto.isReferencia(estudoTecnicoLevantamentoAlternativas.getFornecedor()) && UtilObjeto.isReferencia(estudoTecnicoLevantamentoAlternativas.getFornecedor().getId())){

					// busca para o estudoTecnicoLevantamentoAlternativas o fornecedor
					estudoTecnicoLevantamentoAlternativas.setFornecedor((Fornecedor) this.parceiroServiceFornecedor.find(estudoTecnicoLevantamentoAlternativas.getFornecedor().getId()));
				}

				if (UtilObjeto.isReferencia(estudoTecnicoLevantamentoAlternativas.getOrgaoExterno()) && UtilObjeto.isReferencia(estudoTecnicoLevantamentoAlternativas.getOrgaoExterno().getId())){

					// busca para o estudoTecnicoLevantamentoAlternativas o orgaoExterno
					estudoTecnicoLevantamentoAlternativas.setOrgaoExterno((OrgaoExterno) this.parceiroServiceOrgaoExterno.find(estudoTecnicoLevantamentoAlternativas.getOrgaoExterno().getId()));
				}

				if (UtilObjeto.isReferencia(estudoTecnicoLevantamentoAlternativas.getEtlaQuestionario())) {
					estudoTecnicoLevantamentoAlternativas.getEtlaQuestionario().setEstudoTecnico(estudoTecnico);

					for (EstudoTecnicoQuestionarioResposta resposta : estudoTecnicoLevantamentoAlternativas.getEtlaQuestionario().getEtQuestionarioRespostas()) {
						resposta.setEtlaQuestionario(estudoTecnicoLevantamentoAlternativas.getEtlaQuestionario());
						QuestionarioPergunta pergunta = resposta.getQuestionarioPergunta();
						pergunta.setDominioTipoResposta(this.dominioService.find(pergunta.getDominioTipoResposta().getId()));
					}
				}

				estudoTecnicoLevantamentoAlternativas.setEstudoTecnico(estudoTecnico);
			}
		}

		// cria vinculo do estudo tecnico com as etRecursosHumanos
		if (!UtilColecao.isVazio(estudoTecnico.getEtRecursosHumanos())){

			for (EstudoTecnicoRecursoHumano recursoHumano : estudoTecnico.getEtRecursosHumanos()) {

				recursoHumano.setEstudoTecnico(estudoTecnico);
			}
		}

		// cria vinculo do estudo tecnico com as etRecursosMateriais
		if (!UtilColecao.isVazio(estudoTecnico.getEtRecursosMateriais())){

			for (EstudoTecnicoRecursoMaterial recursoMaterial : estudoTecnico.getEtRecursosMateriais()) {

				recursoMaterial.setEstudoTecnico(estudoTecnico);
			}
		}

		// cria vinculo do estudo tecnico com as etRequisitosNegocio
		if (!UtilColecao.isVazio(estudoTecnico.getEtRequisitosNegocio())){

			for (EstudoTecnicoRequisitoNegocio estudoTecnicoRequisitoNegocio : estudoTecnico.getEtRequisitosNegocio()) {

				if (UtilObjeto.isReferencia(estudoTecnicoRequisitoNegocio.getObjetivoEstrategicoNecessidade()) && UtilObjeto.isReferencia(estudoTecnicoRequisitoNegocio.getObjetivoEstrategicoNecessidade().getId())){

					 // busca para o estudoTecnicoRequisitoNegocio objetivoEstrategicoNecessidade
					 estudoTecnicoRequisitoNegocio.setObjetivoEstrategicoNecessidade(this.objetivoEstrategicoNecessidadeService.find(estudoTecnicoRequisitoNegocio.getObjetivoEstrategicoNecessidade().getId()));
				}

				estudoTecnicoRequisitoNegocio.setEstudoTecnico(estudoTecnico);
			}
		}
	}

	/**
	 * Método responsável por resolver os transients de integrante do estudo tecnicos
	 *
	 * @author iago.almeida
	 *
	 * @param estudoTecnico
	 */
	private void montarIntegranteEstudoTecnico(EstudoTecnico estudoTecnico) {

		// busca para o estudo tecnico o colaborador integrante tecnico
		estudoTecnico.setIntegranteTecnico(this.colaboradorService.find(estudoTecnico.getIntegranteTecnico().getId()));

		// busca para o estudo tecnico o colaborador integrante administrativo
		estudoTecnico.setIntegranteAdministrativo(this.colaboradorService.find(estudoTecnico.getIntegranteAdministrativo().getId()));

		// busca para o estudo tecnico o colaborador integrante requisitante
		estudoTecnico.setIntegranteRequisitante(this.colaboradorService.find(estudoTecnico.getIntegranteRequisitante().getId()));
	}

	/**
	 * Método responsável por resolver os transients de documentos e observacoes
	 *
	 * @author iago.almeida
	 *
	 * @param estudoTecnico
	 */
	@SuppressWarnings("rawtypes")
	private void montarDocsObsEstudoTecnico(EstudoTecnico estudoTecnico) {

		// cria vinculo do estudo tecnico com as Observacoes
		if (!UtilColecao.isVazio(estudoTecnico.getObservacoes())) {

			for (EstudoTecnicoObservacao estudoTecnicoObservacao : estudoTecnico.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(estudoTecnicoObservacao.getAutor())) {
					idAutor = estudoTecnicoObservacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				estudoTecnicoObservacao.setAutor(this.usuarioService.find(idAutor));

				estudoTecnicoObservacao.setEstudoTecnico(estudoTecnico);
			}
		}

		// cria vinculo do estudo tecnico com os Documentos
		if (!UtilColecao.isVazio(estudoTecnico.getDocumentos())) {

			for (EstudoTecnicoDocumento estudoTecnicoDocumento : estudoTecnico.getDocumentos()) {

				estudoTecnicoDocumento.setDominioTipoDocumento(this.dominioService.find(estudoTecnicoDocumento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(estudoTecnicoDocumento.getEmitente()) && UtilObjeto.isReferencia(estudoTecnicoDocumento.getEmitente().getId())) {

					estudoTecnicoDocumento.setEmitente(this.parceiroServiceFornecedor.find(estudoTecnicoDocumento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(estudoTecnicoDocumento.getAnexos())){
					Anexo anexo = (Anexo) ((List)estudoTecnicoDocumento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					estudoTecnicoDocumento.setAnexos(listaAnexo);
				}

				estudoTecnicoDocumento.setEstudoTecnico(estudoTecnico);
			}
		}
	}

	/**
	 * Método responsável por settar o atributo codigo.
	 *
	 * @author iago.almeida
	 *
	 * @param entity
	 */
	private void configurarCodigo(EstudoTecnico entitySaved) {

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Calendar date = Calendar.getInstance();
		EstudoTecnico ultimoRegistro = (EstudoTecnico) estudoTecnicoDao.buscarUltimoRegistroPorDataEmissao(date.get(Calendar.YEAR), usuarioLogado.getOrganizacao().getId());

		Long sequencial = null;
		if (ultimoRegistro != null) {
			sequencial = ultimoRegistro.getRegistroSequencial().getNumeroSequencial() == null ? null : ultimoRegistro.getRegistroSequencial().getNumeroSequencial();
		}
		if (sequencial != null){
			sequencial++;
		} else {
			sequencial = 1l;
		}
		String numeroAnoOcorrencia = StringUtils.leftPad(sequencial.toString() + date.get(Calendar.YEAR), 9, "0");
		RegistroSequencial registroSequencial = new RegistroSequencial(sequencial, numeroAnoOcorrencia);
		entitySaved.setRegistroOcorrenciaSequencial(registroSequencial);
	}

	@Override
	public Collection<EstudoTecnico> listarEstudosPorCodigo(String codigo) {

		Collection<EstudoTecnico> listaEstudos = this.estudoTecnicoDao.listarEstudosPorCodigo(codigo);

		return UtilColecao.isVazio(listaEstudos) ? new ArrayList<EstudoTecnico>() : listaEstudos;
	}

	@Override
	public boolean removeById(Long id) {

		if (this.termoReferenciaService.existeVinculo("estudoTecnico.id", id)) {
			lancarBusinessException(MSG_EXCLUSAO_NEGADA_TERMO_REFERENCIA, CodigoErro.REGRA_NEGOCIO.getValue(), true);
		}

		//TODO criar validação para contrato quando o mesmo estiver pronto

		return super.removeById(id);
	}
	/**
	 * Método responsável por listar estudos tecnicos por codigo e organizacao
	 *
	 * @author iago.almeida
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @return Collection<EstudoTecnico>
	 */
	@Override
	public Collection<EstudoTecnico> findAutoCompleteEstudoTecnicoPorCodigoAndOrganizacao(String codigo, Long idOrganizacao) {

		return this.estudoTecnicoDao.findAutoCompleteEstudoTecnicoPorCodigoAndOrganizacao(codigo, idOrganizacao);
	}

	@Override
	public Map<String, Boolean> findEstudoTecnicoRecursoObservacao(Long idEstudoTecnico) {
		Map<String, Boolean> map = new HashMap<String, Boolean>();

		EstudoTecnico estudoTecnico = this.find(idEstudoTecnico);

		if(estudoTecnico.getEtRecursosMateriais() == null || estudoTecnico.getEtRecursosMateriais().size() == 0 ){
			map.put("ERECURSOMATERIAL_NAO_APLICA", Boolean.TRUE);
		}else{
			map.put("ERECURSOMATERIAL_NAO_APLICA", Boolean.FALSE);
		}

		if(estudoTecnico.getEtRecursosHumanos() == null || estudoTecnico.getEtRecursosHumanos().size() == 0 ){
			map.put("ERECURSOHUMANO_NAO_APLICA", Boolean.TRUE);
		}else{
			map.put("ERECURSOHUMANO_NAO_APLICA", Boolean.FALSE);
		}

		if(estudoTecnico.getObservacoes() == null || estudoTecnico.getObservacoes().size() == 0 ){
			map.put("OBSERVACAO_NAO_APLICA", Boolean.TRUE);
		}else{
			map.put("OBSERVACAO_NAO_APLICA", Boolean.FALSE);
		}

		return map;
	}
}
