package br.com.centralit.api.service.impl;

import java.io.InputStream;
import java.sql.Connection;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoAditivoDao;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoAditivo;
import br.com.centralit.api.model.EventoAditivoDocumento;
import br.com.centralit.api.model.EventoAditivoObservacao;
import br.com.centralit.api.model.RegistroSequencial;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoAditivoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.Util;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 15/03/2016 - 18:33:28
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("eventoAditivoService")
public class EventoAditivoServiceImpl extends GenericServiceImpl<EventoAditivo, Long> implements EventoAditivoService {

	private EventoAditivoDao eventoAditivoDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroService;

	private Dominio idioma;

	@Autowired
	public EventoAditivoServiceImpl(EventoAditivoDao aditivoDao, @Qualifier("eventoAditivoValidator") Validator validator) {

		this.dao = aditivoDao;

		this.eventoAditivoDao = aditivoDao;

		this.validator = validator;
	}

	public EventoAditivo save(EventoAditivo entity) {
		montarObjeto(entity);

		// realiza a validação do objeto com a classe EventoAditivoValidator.java
		this.validarEntidade(entity, this.validator);

		EventoAditivo entitySaved = super.save(entity);
		return entitySaved;
	}

	private void gerarNumeroSequencial(EventoAditivo entity) {

		// busca todos os registros dentro do ano de emissao e ordena decrescente pelo numeroSequencial.
		EventoAditivo ultimoRegistro = (EventoAditivo) eventoAditivoDao.buscarUltimoRegistroPorDataEmissaoEContrato("registroSequencial.numeroSequencial",
				entity.getDataEmissao(), entity.getContrato().getId(), entity.getOrganizacao().getId());

		Long sequencial = null;
		if (ultimoRegistro != null) {
			sequencial = ultimoRegistro.getRegistroSequencial().getNumeroSequencial() == null ? null : ultimoRegistro.getRegistroSequencial().getNumeroSequencial();
		}
		if (sequencial != null){
			sequencial++;
		} else {
			sequencial = 1l;
		}
		DecimalFormat sequencialFormatado = new DecimalFormat("00000");
		String sequencialAno = sequencialFormatado.format(sequencial) + entity.getDataEmissao().get(Calendar.YEAR);
		RegistroSequencial registroSequencial = new RegistroSequencial(sequencial, sequencialAno);
		entity.setRegistroSequencial(registroSequencial);
	}

	@Override
	public EventoAditivo merge(EventoAditivo aditivo) {
		montarObjeto(aditivo);
		// realiza a validação do objeto com a classe EventoAditivoValidator.java
		this.validarEntidade(aditivo, this.validator);

		super.save(aditivo);
		return aditivo;
	}

	private void montarObjeto(EventoAditivo eventoAditivo) {

		if (eventoAditivo.getOrganizacao() == null) {
			eventoAditivo.setOrganizacao(this.getUsuario().getOrganizacao());
		}

		if(eventoAditivo.getId() == null){
			gerarNumeroSequencial(eventoAditivo);
		}

		eventoAditivo.setContrato(this.contratoService.find(eventoAditivo.getContrato().getId()));
		eventoAditivo.setContratoAditivo(this.contratoService.find(eventoAditivo.getContrato().getId()));
		eventoAditivo.setTipoAlteracao(this.dominioService.find(eventoAditivo.getTipoAlteracao().getId()));

		if (UtilObjeto.isReferencia(eventoAditivo.getTipoAlteracaoValor())) {
			eventoAditivo.setTipoAlteracaoValor(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_ALTERACAO_VALOR, eventoAditivo.getTipoAlteracaoValor().getCodigo()));
		}

		eventoAditivo.setDataEvento(eventoAditivo.getDataEmissao());

		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");

		String sequencialAno = eventoAditivo.getRegistroSequencial().getSequencialAno();
		String sequencialAnoFormatado = sequencialAno.substring(0, sequencialAno.length() - 4).concat("/").concat(sequencialAno.substring(sequencialAno.length() - 4));

		String descricaoEventoAditivo = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO", this.getIdioma());
		descricaoEventoAditivo = String.format(descricaoEventoAditivo, sequencialAnoFormatado, sdf.format(eventoAditivo.getDataEmissao().getTime()));

		if (eventoAditivo.getAditivoDePrazo()) {
			descricaoEventoAditivo = descricaoEventoAditivo.concat(this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO_PRAZO", this.getIdioma()));
			descricaoEventoAditivo = String.format(descricaoEventoAditivo, sdf.format(eventoAditivo.getDataInicio().getTime()), sdf.format(eventoAditivo.getDataFim().getTime()));
		}

		if (eventoAditivo.getAditivoDeValor()) {
			descricaoEventoAditivo = descricaoEventoAditivo.concat(this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO_VALOR", this.getIdioma()));
			descricaoEventoAditivo = String.format(descricaoEventoAditivo, eventoAditivo.getTipoAlteracaoValor().getDescricao(), Util.convertBigDecimalToString(eventoAditivo.getValorAditivo()));
		}

		if (eventoAditivo.getAditivoDeClausula()) {
			descricaoEventoAditivo = descricaoEventoAditivo.concat(this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO_CLAUSULA", this.getIdioma()));
		}
		eventoAditivo.setDescricao(descricaoEventoAditivo);

		eventoAditivo.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_ADITIVO));

		// Observacoes
		if (CollectionUtils.isNotEmpty(eventoAditivo.getObservacoes())) {
			Collection<EventoAditivoObservacao> observacoes = eventoAditivo.getObservacoes();

			for (EventoAditivoObservacao observacao : observacoes) {
				Long idAutor = null;
				if (observacao.getAutor() != null) {
					idAutor = observacao.getAutor().getId();
					observacao.setAutor(this.usuarioService.find(idAutor));
				}
				observacao.setEventoAditivo(eventoAditivo);
			}
			eventoAditivo.setObservacoes(observacoes);
		}

		// Documentos
		if (CollectionUtils.isNotEmpty(eventoAditivo.getDocumentos())) {
			Collection<EventoAditivoDocumento> documentos = eventoAditivo.getDocumentos();

			for (EventoAditivoDocumento documento : documentos) {
				documento.setEventoAditivo(eventoAditivo);
				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (documento.getEmitente() != null && documento.getEmitente().getId() != null) {
					documento.setEmitente(this.parceiroService.getReference(documento.getEmitente().getId()));
				}
			}
			eventoAditivo.setDocumentos(documentos);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<EventoAditivo> findAllByContrato(Long contratoId) {
		return this.eventoAditivoDao.findAllByContrato(contratoId);
	}

	private Dominio getIdioma() {
		if(!UtilObjeto.isReferencia(idioma)){
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	public JasperPrint getJasperPrintAditivo(Long idAditivo, Connection conn, Usuario user, String titulo) throws JRException {
		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/eventoAditivo.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);

			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("ADITIVO_ID", idAditivo);
			subReportParameters.put("REPORT_CONNECTION", conn);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", user.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}

	@Override
	public Boolean permiteExclusaoAditivo(Long idAditivo) {
		EventoAditivo aditivo = this.find(idAditivo);

		if(aditivo.getDataInicio().before(GregorianCalendar.getInstance()) ){
			return false;
		}

		if(UtilDate.isMesmaData(aditivo.getDataInicio(), GregorianCalendar.getInstance())){
			return false;
		}

		if(this.eventoAditivoDao.existeAditivoPosterior(aditivo.getDataInicio(), aditivo.getContrato().getId())){
			return false;
		}

		return true;
	}

	@Override
	public Boolean existeAditivoPrazoPosterior(Long idAditivo) {
		EventoAditivo aditivo = this.find(idAditivo);
		if(this.eventoAditivoDao.existeAditivoPosterior(aditivo.getDataInicio(), aditivo.getContrato().getId())){
			return true;
		}
		return false;
	}

}
