package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoApostilamentoDao;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoApostilamento;
import br.com.centralit.api.model.EventoApostilamentoDocumento;
import br.com.centralit.api.model.EventoApostilamentoObservacao;
import br.com.centralit.api.model.RegistroSequencial;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoApostilamentoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilObjeto;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 10/03/2016 - 10:10:48
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */
@Service("eventoApostilamentoService")
public class EventoApostilamentoServiceImpl extends GenericServiceImpl<EventoApostilamento, Long> implements EventoApostilamentoService {

	private static final Logger LOG = Logger.getLogger(EventoApostilamentoServiceImpl.class);

	@Autowired
	private EventoApostilamentoDao eventoApostilamentoDao;

	@Autowired
	private ReportController reportService;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroService;

	private Dominio idioma;

	@Autowired
	public EventoApostilamentoServiceImpl( EventoApostilamentoDao eventoApostilamentoDao, @Qualifier("eventoApostilamentoValidator") Validator validator ) {

		this.dao = eventoApostilamentoDao;
		this.eventoApostilamentoDao = eventoApostilamentoDao;
		this.validator = validator;
	}

	@Override
	public EventoApostilamento save(EventoApostilamento eventoApostilamento) {

		this.validarEntidade(eventoApostilamento, this.validator);

		montarObjeto(eventoApostilamento);

		return super.save(eventoApostilamento);
	}

	@Override
	public EventoApostilamento merge (EventoApostilamento eventoApostilamento){

		this.validarEntidade(eventoApostilamento, this.validator);

		montarObjeto(eventoApostilamento);

		return super.merge(eventoApostilamento);
	}

	/**
	 *
	 * M&eacute;todo respons&aacute;vel por montar os objetos
	 *
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 *
	 * @param eventoApostilamento
	 */
	private void montarObjeto(EventoApostilamento eventoApostilamento) {

		if (eventoApostilamento.getOrganizacao() == null) {
			eventoApostilamento.setOrganizacao(this.getUsuario().getOrganizacao());
		}

		if(eventoApostilamento.getId() == null){
			gerarNumeroSequencial(eventoApostilamento);
		}

		eventoApostilamento.setContrato(this.contratoService.getReference(eventoApostilamento.getContrato().getId()));
		eventoApostilamento.setDataEvento(eventoApostilamento.getDataEmissao());
		String descricaoEventoApostilamento = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_APOSTILAMENTO", this.getIdioma());
		if(StringUtils.isNotEmpty(descricaoEventoApostilamento)){
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			String date = sdf.format(new Date());
			String sequencialAno = eventoApostilamento.getRegistroSequencial().getSequencialAno();
			String sequencialAnoFormatado = sequencialAno.substring(0, sequencialAno.length() - 4).concat("/").concat(sequencialAno.substring(sequencialAno.length() - 4));
			eventoApostilamento.setDescricao(String.format(descricaoEventoApostilamento, sequencialAnoFormatado, date));
		} else {
			eventoApostilamento.setDescricao("");
		}

		eventoApostilamento.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_APOSTILAMENTO));

		// Observacoes
		if (CollectionUtils.isNotEmpty(eventoApostilamento.getObservacoes())) {
			Collection<EventoApostilamentoObservacao> observacoes = eventoApostilamento.getObservacoes();

			for (EventoApostilamentoObservacao observacao : observacoes) {
				Long idAutor = null;
				if (observacao.getAutor() != null) {
					idAutor = observacao.getAutor().getId();
					observacao.setAutor(this.usuarioService.find(idAutor));
				}
				observacao.setEventoApostilamento(eventoApostilamento);
			}
			eventoApostilamento.setObservacoes(observacoes);
		}

		// Documentos
		if (CollectionUtils.isNotEmpty(eventoApostilamento.getDocumentos())) {
			Collection<EventoApostilamentoDocumento> documentos = eventoApostilamento.getDocumentos();

			for (EventoApostilamentoDocumento documento : documentos) {
				documento.setEventoApostilamento(eventoApostilamento);
				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (documento.getEmitente() != null && documento.getEmitente().getId() != null) {
					documento.setEmitente(this.parceiroService.getReference(documento.getEmitente().getId()));
				}
			}
			eventoApostilamento.setDocumentos(documentos);
		}
	}

	/**
	 * M&eacute;todo respons&aacute;vel por gerar o n&uacute;mero sequencial identificador e &uacute;nico para cada objeto salvo.
	 * Esta sequencia &eacute; reiniciada a cada ano. Por exemplo: o n&uacute;mero 1/2016 at&eacute; 999/2016, no
	 * pr&oacute;ximo ano a sequencia inicia-se novamente 1/2017 at&eacute; 999/2017
	 *
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 * @param entity
	 */
	@Override
	public void gerarNumeroSequencial(EventoApostilamento entity) {

		// busca todos os registros dentro do ano de emissao e ordena decrescente pelo numeroSequencial.
		EventoApostilamento ultimoRegistro = (EventoApostilamento) eventoApostilamentoDao.buscarUltimoRegistroPorDataEmissaoPorContrato("registroSequencial.numeroSequencial",
				entity.getDataEmissao(), entity.getOrganizacao().getId(), entity.getContrato().getId());

		Long sequencial = null;
		if (ultimoRegistro != null) {
			sequencial = ultimoRegistro.getRegistroSequencial().getNumeroSequencial() == null ? null : ultimoRegistro.getRegistroSequencial().getNumeroSequencial();
		}
		if (sequencial != null){
			sequencial++;
		} else {
			sequencial = 1l;
		}
		DecimalFormat sequencialFormatado = new DecimalFormat("00000");
		String sequencialAno = sequencialFormatado.format(sequencial) + entity.getDataEmissao().get(Calendar.YEAR);
		RegistroSequencial registroSequencial = new RegistroSequencial(sequencial, sequencialAno);
		entity.setRegistroSequencial(registroSequencial);
	}

	@Override
	public void gerarPdfApostilamento(Long idApostilamento, boolean download, HttpServletResponse response) {
		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;

		try {
			conn = reportService.getConnection();
			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
			String titulo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.TITULO_DOCUMENTO_APOSTILAMENTO", this.getIdioma());

			// Responsavel por verificar a presenca de dados no relatorio
			reportService.gerarRelatorio(download, "pdf", this.getJasperPrintApostilamento(idApostilamento, conn, usuarioLogado, titulo), response, baos, titulo);

		} catch (JRException jre) {
			LOG.info("N\u00e3o consegui gerar o relat\u00f3rio de apostilamento. Veja a exce\u00e7\u00e3o para identificar a causa", jre.getCause());
			throw new RuntimeException(jre);
		} catch (Exception e) {
			throw new BusinessException();
		} finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}

	public Dominio getIdioma() {
		if(!UtilObjeto.isReferencia(idioma)){
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public JasperPrint getJasperPrintApostilamento(Long id, Connection conn, Usuario usuarioLogado, String titulo) throws JRException {
		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/eventoApostilamento.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("APOSTILAMENTO_ID", id);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}
}
