package br.com.centralit.api.service.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoAtualizacaoStatusDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.Contrato;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoAtualizacaoStatus;
import br.com.centralit.api.model.EventoAtualizacaoStatusDocumento;
import br.com.centralit.api.model.EventoAtualizacaoStatusObservacao;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoAtualizacaoStatusService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 10/03/2016 - 16:04:45
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("eventoAtualizacaoStatusService")
public class EventoAtualizacaoStatusServiceImpl extends GenericServiceImpl<EventoAtualizacaoStatus, Long> implements EventoAtualizacaoStatusService {

    private EventoAtualizacaoStatusDao eventoAtualizacaoStatusDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private AnexoService anexoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	private Dominio idioma;

    @Autowired
	public EventoAtualizacaoStatusServiceImpl(EventoAtualizacaoStatusDao eventoAtualizacaoStatusDao, @Qualifier("eventoAtualizacaoStatusValidator") Validator validator ) {
    	this.dao = eventoAtualizacaoStatusDao;
        this.eventoAtualizacaoStatusDao = eventoAtualizacaoStatusDao;
        this.validator = validator;
	}

	public EventoAtualizacaoStatus save(EventoAtualizacaoStatus entity) {

		Contrato contrato = this.contratoService.find(entity.getContrato().getId());
		entity.setContrato(contrato);
		this.validarEntidade(entity, this.validator);

		entity.setDataEvento(entity.getDataAtualizacao());
		String descricao = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_ATUALIZACAO_STATUS", this.getIdioma());
		if(StringUtils.isNotEmpty(descricao)){
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			String date = sdf.format(new Date());
			entity.setDescricao(String.format(descricao, date, contrato.getDominioSituacaoAtual().getDescricao(), entity.getTipoSituacaoNova().getDescricao()));
		}
		entity.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_ATUALIZA_STATUS));

		this.montarDocsObs(entity);
		entity.setTipoSituacaoAntiga(entity.getContrato().getDominioSituacaoAtual());
		contrato.setDominioSituacaoAtual(entity.getTipoSituacaoNova());
		EventoAtualizacaoStatus entitySaved = super.save(entity);
		return entitySaved;
	}

	private void montarDocsObs(EventoAtualizacaoStatus eventoAtualizacaoStatus) {

		if (!UtilColecao.isVazio(eventoAtualizacaoStatus.getObservacoes())) {

			for (EventoAtualizacaoStatusObservacao observacao : eventoAtualizacaoStatus.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(observacao.getAutor())) {
					idAutor = observacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				observacao.setAutor(this.usuarioService.find(idAutor));

				observacao.setEventoAtualizacaoStatus(eventoAtualizacaoStatus);
			}
		}

		if (!UtilColecao.isVazio(eventoAtualizacaoStatus.getDocumentos())) {

			for (EventoAtualizacaoStatusDocumento documento : eventoAtualizacaoStatus.getDocumentos()) {

				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(documento.getEmitente()) && UtilObjeto.isReferencia(documento.getEmitente().getId())) {

					documento.setEmitente(this.parceiroServiceFornecedor.find(documento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(documento.getAnexos())){
					Anexo anexo = (Anexo) ((List)documento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					documento.setAnexos(listaAnexo);
				}

				documento.setEventoAtualizacaoStatus(eventoAtualizacaoStatus);
			}
		}
	}

	public Dominio getIdioma() {
		if(!UtilObjeto.isReferencia(idioma)){
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}
}
