package br.com.centralit.api.service.impl;

import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoLiberacaoPagamentoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoLiberacaoPagamento;
import br.com.centralit.api.model.EventoLiberacaoPagamentoDocumento;
import br.com.centralit.api.model.EventoLiberacaoPagamentoEmpenho;
import br.com.centralit.api.model.EventoLiberacaoPagamentoObservacao;
import br.com.centralit.api.model.EventoLiberacaoPagamentoOrdemFornecimento;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ContratoEmpenhoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoLiberacaoPagamentoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.Util;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

@Service("liberacaoPagamentoService")
public class EventoLiberacaoPagamentoServiceImpl extends GenericServiceImpl<EventoLiberacaoPagamento, Long> implements EventoLiberacaoPagamentoService {

	private EventoLiberacaoPagamentoDao liberacaoPagamentoDao;

	public static final String CAMPO_BUSCA_PARAMETRIZADA_ULTIMO_REGISTRO = "numeroLiberacao";

	@Autowired
	private DominioService dominioService;

	@Autowired
	private AnexoService anexoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ContratoEmpenhoService contratoEmpenhoService;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	public EventoLiberacaoPagamentoServiceImpl(EventoLiberacaoPagamentoDao liberacaoPagamentoDao, @Qualifier("eventoLiberacaoPagamentoValidator") Validator validator ) {

		this.dao = liberacaoPagamentoDao;
		this.liberacaoPagamentoDao = liberacaoPagamentoDao;
		this.validator = validator;
	}

	public EventoLiberacaoPagamento save(EventoLiberacaoPagamento entity) {

		this.validarEntidade(entity, this.validator);

		this.configurarCodigo(entity);
		this.montarLiberacaoPagamento(entity);
		EventoLiberacaoPagamento entitySaved = super.save(entity);
		return entitySaved;
	}

	@Override
	public EventoLiberacaoPagamento merge(EventoLiberacaoPagamento liberacaoPagamento) {

		if(liberacaoPagamento.getDataLiberacao() == null && liberacaoPagamento.getDataEvento() != null){
			liberacaoPagamento.setDataLiberacao(liberacaoPagamento.getDataEvento());
		}

		this.montarLiberacaoPagamento(liberacaoPagamento);
		super.merge(liberacaoPagamento);
		return liberacaoPagamento;
	}

	private void montarLiberacaoPagamento(EventoLiberacaoPagamento liberacaoPagamento) {

		liberacaoPagamento.setContrato(contratoService.find(liberacaoPagamento.getContrato().getId()));

		liberacaoPagamento.setDataEvento(liberacaoPagamento.getDataLiberacao());

		if(liberacaoPagamento.getTotalEmpenhoLiberado() != null && liberacaoPagamento.getTotalEmpenhoLiberado().doubleValue() > 0){
			liberacaoPagamento.setTotalLiberadoPagamento(liberacaoPagamento.getTotalEmpenhoLiberado());
		}

		Dominio idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		Integer ano = liberacaoPagamento.getDataLiberacao().get(Calendar.YEAR);
		String[] numeroLiberacaoMask = liberacaoPagamento.getNumeroLiberacao().split(ano.toString());

		String descricaoEmpenho = this.internacionalizacaoService.getTranslate("LIBERACAO_PAGAMENTO.DESCRICAO_EVENTO", idioma);
		descricaoEmpenho = descricaoEmpenho.replace("{numeroLiberacaoPagamento}", numeroLiberacaoMask.length > 0 ? numeroLiberacaoMask[0] + "/" + ano : liberacaoPagamento.getNumeroLiberacao());
		descricaoEmpenho = descricaoEmpenho.replace("{valor}", Util.convertBigDecimalToString(liberacaoPagamento.getTotalLiberadoPagamento()));
		descricaoEmpenho = descricaoEmpenho.replace("{data}", UtilDate.formatarData(liberacaoPagamento.getDataLiberacao().getTime(), "dd/MM/yyyy"));

		liberacaoPagamento.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_LIBERA_PAGAMENTO));

		liberacaoPagamento.setDescricao(descricaoEmpenho);
		this.montarDocsObs(liberacaoPagamento);

		this.setLiberacaoOS(liberacaoPagamento);

		this.setLiberacaoEmpenho(liberacaoPagamento);

	}

	private void setLiberacaoEmpenho(EventoLiberacaoPagamento liberacaoPagamento) {
		if (!UtilColecao.isVazio(liberacaoPagamento.getEmpenhos())) {
			for (EventoLiberacaoPagamentoEmpenho empenho : liberacaoPagamento.getEmpenhos()) {
				empenho.setContratoEmpenho(this.contratoEmpenhoService.getReference(empenho.getContratoEmpenho().getId()));
				empenho.setLiberacaoPagamento(liberacaoPagamento);
			}
		}
	}

	private void setLiberacaoOS(EventoLiberacaoPagamento liberacaoPagamento) {
		if (!UtilColecao.isVazio(liberacaoPagamento.getOrdensFornecimento())) {
			for (EventoLiberacaoPagamentoOrdemFornecimento ordem : liberacaoPagamento.getOrdensFornecimento()) {
				ordem.setLiberacaoPagamento(liberacaoPagamento);
			}
		}
	}

	private void montarDocsObs(EventoLiberacaoPagamento liberacaoPagamento) {

		if (!UtilColecao.isVazio(liberacaoPagamento.getObservacoes())) {

			for (EventoLiberacaoPagamentoObservacao observacao : liberacaoPagamento.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(observacao.getAutor())) {
					idAutor = observacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				observacao.setAutor(this.usuarioService.find(idAutor));

				observacao.setLiberacaoPagamento(liberacaoPagamento);
			}
		}

		if (!UtilColecao.isVazio(liberacaoPagamento.getDocumentos())) {

			for (EventoLiberacaoPagamentoDocumento documento : liberacaoPagamento.getDocumentos()) {

				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(documento.getEmitente()) && UtilObjeto.isReferencia(documento.getEmitente().getId())) {

					documento.setEmitente(this.parceiroServiceFornecedor.find(documento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(documento.getAnexos())){
					Anexo anexo = (Anexo) ((List<Anexo>)documento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					documento.setAnexos(listaAnexo);
				}

				documento.setLiberacaoPagamento(liberacaoPagamento);
			}
		}
	}

	private void configurarCodigo(EventoLiberacaoPagamento entitySaved) {
		Integer ano = entitySaved.getDataLiberacao().get(Calendar.YEAR);
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		EventoLiberacaoPagamento ultimoRegistro = (EventoLiberacaoPagamento) this.liberacaoPagamentoDao.buscarUltimoRegistroPorDataEmissaoPorContrato(CAMPO_BUSCA_PARAMETRIZADA_ULTIMO_REGISTRO, ano.toString(), usuarioLogado.getOrganizacao().getId(), entitySaved.getContrato().getId());
		Long sequencial = null;

		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getNumeroLiberacao().replace(ano.toString(), "") == null ? null : Long.parseLong(ultimoRegistro.getNumeroLiberacao().replace(ano.toString(), "") );
		}else{
			sequencial = 0l;
		}

		entitySaved.setNumeroLiberacao(StringUtils.leftPad((Long.valueOf(sequencial + 1)).toString(), 5, "0") + ano);
	}

	@Override
	public BigDecimal getValorTotalLiberado(Long contratoId) {
		return liberacaoPagamentoDao.getValorTotalLiberado(contratoId);
	}

	@Override
	public JasperPrint getJasperPrintRelatorioLiberacaoPagamento(Long idContrato, Date dataInicial, Date dataFinal,
			Connection conn, Usuario usuarioLogado, String titulo) throws JRException {

		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/relatorioLiberacoesPagamento.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("CONTRATO_ID", idContrato);
			subReportParameters.put("DATA_INICIAL", dataInicial);
			subReportParameters.put("DATA_FINAL", dataFinal);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}

}
