package br.com.centralit.api.service.impl;

import java.io.InputStream;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoPenalidadeDao;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoPenalidade;
import br.com.centralit.api.model.EventoPenalidadeDocumento;
import br.com.centralit.api.model.EventoPenalidadeObservacao;
import br.com.centralit.api.model.EventoPenalidadeRegistroOcorrencia;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoPenalidadeService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDataBase;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 15/03/2016 - 14:38:48
 * @version 1.0.0
 * @author luis.camargo (<a href="mailto:luis.camargo@centralit.com.br">luis.camargo@centralit.com.br</a>)
 *
 */
@Service("eventoPenalidadeService")
public class EventoPenalidadeServiceImpl extends GenericServiceImpl<EventoPenalidade, Long> implements EventoPenalidadeService {

	private static final Logger LOG = Logger.getLogger(EventoPenalidadeServiceImpl.class);

	@Autowired
	private EventoPenalidadeDao eventoPenalidadeDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	public EventoPenalidadeServiceImpl(EventoPenalidadeDao eventoPenalidadeDao, @Qualifier("eventoPenalidadeValidator") Validator validator ) {

		this.dao = eventoPenalidadeDao;
		this.eventoPenalidadeDao = eventoPenalidadeDao;
		this.validator = validator;
	}

	@Override
	public EventoPenalidade save(EventoPenalidade eventoPenalidade) {
		eventoPenalidade.setContrato(this.contratoService.getReference(eventoPenalidade.getContrato().getId()));
		
		this.validarEntidade(eventoPenalidade, this.validator);

		montarObjeto(eventoPenalidade);

		return super.save(eventoPenalidade);
	}

	@Override
	public EventoPenalidade merge(EventoPenalidade eventoPenalidade) {
		eventoPenalidade.setContrato(this.contratoService.getReference(eventoPenalidade.getContrato().getId()));

		this.validarEntidade(eventoPenalidade, this.validator);
		
		montarObjeto(eventoPenalidade);
		
		super.save(eventoPenalidade);
		
		return eventoPenalidade;
	}
	
	/**
	 *
	 * Método responsável por montar os objetos
	 *
	 * @author luis.camargo (<a href="mailto:luis.camargo@centralit.com.br">luis.camargo@centralit.com.br</a>)
	 *
	 * @param eventoPenalidade
	 */
	private void montarObjeto(EventoPenalidade eventoPenalidade) {

		for (EventoPenalidadeRegistroOcorrencia penalidadeRegistroOcorrencia : eventoPenalidade.getOcorrencias()) {
			penalidadeRegistroOcorrencia.setEventoPenalidade(eventoPenalidade);
		}

		String descricao = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_APLICAR_PENALIDADE_DATA", this.getIdioma());
		if(StringUtils.isNotEmpty(descricao)){
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			String date = sdf.format(new Date());
			eventoPenalidade.setDescricao(String.format(descricao, date));
		}

		eventoPenalidade.setDataEvento(eventoPenalidade.getDataPenalizacao());
		eventoPenalidade.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_PENALIDADE));

		// Observacoes
		if (CollectionUtils.isNotEmpty(eventoPenalidade.getObservacoes())) {
			Collection<EventoPenalidadeObservacao> observacoes = eventoPenalidade.getObservacoes();

			for (EventoPenalidadeObservacao observacao : observacoes) {
				Long idAutor = null;
				if (observacao.getAutor() != null) {
					idAutor = observacao.getAutor().getId();
					observacao.setAutor(this.usuarioService.find(idAutor));
				}
				observacao.setEventoPenalidade(eventoPenalidade);
			}
			eventoPenalidade.setObservacoes(observacoes);
		}

		// Documentos
		if (CollectionUtils.isNotEmpty(eventoPenalidade.getDocumentos())) {
			Collection<EventoPenalidadeDocumento> documentos = eventoPenalidade.getDocumentos();

			for (EventoPenalidadeDocumento documento : documentos) {
				documento.setEventoPenalidade(eventoPenalidade);
				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (documento.getEmitente() != null && documento.getEmitente().getId() != null) {
					documento.setEmitente(this.parceiroService.getReference(documento.getEmitente().getId()));
				}
			}
			eventoPenalidade.setDocumentos(documentos);
		}
	}

	public Dominio getIdioma() {
		return dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
	}

	@Override
	public JasperPrint getJasperPrintRelatorioPenalidade(Long idContrato, Date dataInicial, Date dataFinal, 
			Connection conn, Usuario usuarioLogado, String titulo) throws JRException {

		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/relatorioPenalidades.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("CONTRATO_ID", idContrato);
			subReportParameters.put("DATA_INICIAL", dataInicial);
			subReportParameters.put("DATA_FINAL", dataFinal);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}

}
