package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoRecebimentoObjetoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoRecebimentoObjeto;
import br.com.centralit.api.model.EventoRecebimentoObjetoDocumento;
import br.com.centralit.api.model.EventoRecebimentoObjetoObservacao;
import br.com.centralit.api.model.RegistroSequencial;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoRecebimentoObjetoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilObjeto;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

/**
 *
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 10/03/2016 - 16:04:45
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("eventoRecebimentoObjetoService")
public class EventoRecebimentoObjetoServiceImpl extends GenericServiceImpl<EventoRecebimentoObjeto, Long> implements EventoRecebimentoObjetoService {

	private static final Logger LOG = Logger.getLogger(EventoRecebimentoObjeto.class);

	private EventoRecebimentoObjetoDao eventoRecebimentoObjetoDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private AnexoService anexoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ReportController reportService;

	private Dominio idioma;

	@Autowired
	public EventoRecebimentoObjetoServiceImpl( EventoRecebimentoObjetoDao eventoRecebimentoObjetoDao,
			@Qualifier("eventoRecebimentoObjetoValidator") Validator validator ) {

		this.dao = eventoRecebimentoObjetoDao;
		this.eventoRecebimentoObjetoDao = eventoRecebimentoObjetoDao;
		this.validator = validator;
	}

	@Override
	public EventoRecebimentoObjeto save(EventoRecebimentoObjeto entity) {

		this.validarEntidade(entity, this.validator);
		montarObjeto(entity);
		return super.save(entity);
	}

	@Override
	public EventoRecebimentoObjeto merge (EventoRecebimentoObjeto entity){
		this.validarEntidade(entity, this.validator);
		montarObjeto(entity);
		return super.merge(entity);
	}

	private void montarObjeto (EventoRecebimentoObjeto entity) {

		if (entity.getOrganizacao() == null) {
			entity.setOrganizacao(this.getUsuario().getOrganizacao());
		}
		if(entity.getId() == null){
			gerarNumeroSequencial(entity);
		}

		entity.setContrato(this.contratoService.getReference(entity.getContrato().getId()));

		entity.setDataEvento(entity.getDataRecebimento());

		String descricaoRecebimento = this.internacionalizacaoService.getTranslate("MSG.DESCRICAO_EVENTO_RECEBIMENTO_OBJETO", this.getIdioma());
		if(StringUtils.isNotEmpty(descricaoRecebimento)){
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			String date = sdf.format(new Date());
			String sequencialAno = entity.getRegistroSequencial().getSequencialAno();
			String sequencialAnoFormatado = sequencialAno.substring(0, sequencialAno.length() - 4).concat("/").concat(sequencialAno.substring(sequencialAno.length() - 4));
			entity.setDescricao(String.format(descricaoRecebimento, sequencialAnoFormatado, date));
		} else {
			entity.setDescricao("");
		}

		entity.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_RECEBIMENTO_OBJETO));
		entity.setTipoRecebimento(this.dominioService.getReference(entity.getTipoRecebimento().getId()));
		entity.setTipoStatus(this.dominioService.getReference(entity.getTipoStatus().getId()));
		entity.setTipoAvaliacaoPrazo(this.dominioService.getReference(entity.getTipoAvaliacaoPrazo().getId()));
		entity.setTipoAvaliacaoAdesao(this.dominioService.getReference(entity.getTipoAvaliacaoAdesao().getId()));

		this.montarDocsObs(entity);
	}
	private void montarDocsObs(EventoRecebimentoObjeto evento) {

		if (!UtilColecao.isVazio(evento.getObservacoes())) {

			for (EventoRecebimentoObjetoObservacao observacao : evento.getObservacoes()) {
				Long idAutor = null;
				if (observacao.getAutor() != null) {
					idAutor = observacao.getAutor().getId();
				}
				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}
				observacao.setAutor(this.usuarioService.find(idAutor));
				observacao.setEventoRecebimentoObjeto(evento);
			}
		}

		if (!UtilColecao.isVazio(evento.getDocumentos())) {

			for (EventoRecebimentoObjetoDocumento documento : evento.getDocumentos()) {
				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));
				if (documento.getEmitente() != null && documento.getEmitente().getId() != null) {
					documento.setEmitente(this.parceiroServiceFornecedor.find(documento.getEmitente().getId()));
				}
				if (CollectionUtils.isNotEmpty(documento.getAnexos())) {
					Anexo anexo = (Anexo) ((List<Anexo>) documento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					documento.setAnexos(listaAnexo);
				}
				documento.setEventoRecebimentoObjeto(evento);
			}
		}
	}

	public Dominio getIdioma() {

		if (!UtilObjeto.isReferencia(idioma)) {
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public void gerarNumeroSequencial(EventoRecebimentoObjeto entity) {

		// busca todos os registros dentro do ano de recebimento do objeto por contrato e ordena decrescente pelo numeroSequencial.
		EventoRecebimentoObjeto ultimoRegistro = (EventoRecebimentoObjeto) eventoRecebimentoObjetoDao.
				buscarUltimoRegistroPorDataEmissaoPorContrato("registroSequencial.numeroSequencial",
						entity.getDataRecebimento(), entity.getOrganizacao().getId(), entity.getContrato().getId());

		Long sequencial = null;
		if (ultimoRegistro != null) {
			sequencial = ultimoRegistro.getRegistroSequencial().getNumeroSequencial() == null ? null : ultimoRegistro.getRegistroSequencial().getNumeroSequencial();
		}
		if (sequencial != null){
			sequencial++;
		} else {
			sequencial = 1l;
		}
		DecimalFormat sequencialFormatado = new DecimalFormat("00000");
		String numeroAnoRecebimento = sequencialFormatado.format(sequencial) + entity.getDataRecebimento().get(Calendar.YEAR);
		RegistroSequencial registroSequencial = new RegistroSequencial(sequencial, numeroAnoRecebimento);
		entity.setRegistroSequencial(registroSequencial);
	}

	@Override
	public void gerarPdfRecebimento(Long idRecebimentoObjeto, boolean download, HttpServletResponse response) {
		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;

		try {
			conn = this.reportService.getConnection();
			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
			String titulo = this.internacionalizacaoService.getTranslate("LABEL.TITULO_DOCUMENTO_RECEBIMENTO", this.getIdioma());

			// Responsavel por verificar a presenca de dados no relatorio
			reportService.gerarRelatorio(download, "pdf", this.getJasperPrintRecebimentoObjeto(idRecebimentoObjeto, conn, usuarioLogado, titulo), response, baos, titulo);

		} catch (JRException jre) {
			LOG.info("N\u00e3o consegui gerar o relat\u00f3rio de recebimento de objetos. Veja a exce\u00e7\u00e3o para identificar a causa", jre.getCause());
			throw new RuntimeException(jre);
		} catch (Exception e) {
			throw new BusinessException();
		} finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}

	@Override
	public JasperPrint getJasperPrintRecebimentoObjeto(Long id, Connection conn, Usuario usuarioLogado, String titulo) throws JRException {
		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/eventoRecebimentoObjeto.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("RECEBIMENTO_ID", id);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}

}
