package br.com.centralit.api.service.impl;

import java.io.InputStream;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoRegistroNaoConformidadeDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoRegistroNaoConformidade;
import br.com.centralit.api.model.EventoRegistroNaoConformidadeDocumento;
import br.com.centralit.api.model.EventoRegistroNaoConformidadeObservacao;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoRegistroNaoConformidadeService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 10/03/2016 - 16:04:45
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("eventoRegistroNaoConformidadeService")
public class EventoRegistroNaoConformidadeServiceImpl extends GenericServiceImpl<EventoRegistroNaoConformidade, Long> implements EventoRegistroNaoConformidadeService {

    private EventoRegistroNaoConformidadeDao eventoRegistroNaoConformidadeDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private AnexoService anexoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	private Dominio idioma;

    @Autowired
	public EventoRegistroNaoConformidadeServiceImpl(EventoRegistroNaoConformidadeDao eventoRegistroNaoConformidadeDao, @Qualifier("eventoRegistroNaoConformidadeValidator") Validator validator ) {
    	this.dao = eventoRegistroNaoConformidadeDao;
        this.eventoRegistroNaoConformidadeDao = eventoRegistroNaoConformidadeDao;
        this.validator = validator;
	}

	public EventoRegistroNaoConformidade save(EventoRegistroNaoConformidade entity) {

		entity.setContrato(this.contratoService.getReference(entity.getContrato().getId()));
		this.validarEntidade(entity, this.validator);

		entity.setDataEvento(entity.getDataNaoConformidade());
		String descricao = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_NAO_CONFORMIDADE", this.getIdioma());
		if(StringUtils.isNotEmpty(descricao)){
			SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
			String date = sdf.format(new Date());
			entity.setDescricao(String.format(descricao, entity.getTipoNaoConformidade().getDominioTipoSancao().getDescricao() ,date));
		}
		entity.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_NAO_CONFORMIDADE));

		this.montarDocsObs(entity);
		EventoRegistroNaoConformidade entitySaved = super.save(entity);
		return entitySaved;
	}

	public EventoRegistroNaoConformidade merge(EventoRegistroNaoConformidade entity) {
		this.validarEntidade(entity, this.validator);
		this.montarDocsObs(entity);
		return super.merge(entity);
	}

	private void montarDocsObs(EventoRegistroNaoConformidade evento) {

		if (!UtilColecao.isVazio(evento.getObservacoes())) {

			for (EventoRegistroNaoConformidadeObservacao observacao : evento.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(observacao.getAutor())) {
					idAutor = observacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				observacao.setAutor(this.usuarioService.find(idAutor));

				observacao.setEventoRegistroNaoConformidade(evento);
			}
		}

		if (!UtilColecao.isVazio(evento.getDocumentos())) {

			for (EventoRegistroNaoConformidadeDocumento documento : evento.getDocumentos()) {

				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(documento.getEmitente()) && UtilObjeto.isReferencia(documento.getEmitente().getId())) {

					documento.setEmitente(this.parceiroServiceFornecedor.find(documento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(documento.getAnexos())){
					Anexo anexo = ((List<Anexo>)documento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					documento.setAnexos(listaAnexo);
				}

				documento.setEventoRegistroNaoConformidade(evento);
			}
		}
	}

	public Dominio getIdioma() {
		if(!UtilObjeto.isReferencia(idioma)){
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public List<String> aplicacoesNaoConformidadeContrato(Long idContrato) {
		return this.eventoRegistroNaoConformidadeDao.aplicacoesNaoConformidadeContrato(idContrato);
	}

	@Override
	public Boolean existeVinculoSancoesComNaoConformidade(Long sancaoId) {
		return this.eventoRegistroNaoConformidadeDao.existeVinculoSancoesComNaoConformidade(sancaoId);
	}

	@Override
	public JasperPrint getJasperPrintRelatorioRegistroNaoConformidade(Long idContrato, Date dataInicial, Date dataFinal,
			Connection conn, Usuario usuarioLogado, String titulo) throws JRException {

		InputStream input = null;
		InputStream inputSub = null;
		JasperPrint jasperPrint = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/relatorioEventoNaoConformidade.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("contratoId", idContrato);
			subReportParameters.put("dataInicial", dataInicial);
			subReportParameters.put("dataFinal", dataFinal);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally {
			UtilDataBase.closeResources(inputSub);
		}
		return jasperPrint;
	}
}
