package br.com.centralit.api.service.impl;

import java.io.InputStream;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoRepactuacaoDao;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoRepactuacao;
import br.com.centralit.api.model.EventoRepactuacaoDocumento;
import br.com.centralit.api.model.EventoRepactuacaoObservacao;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoRepactuacaoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.RegistroOcorrenciaService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilObjeto;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 19/03/2016 - 09:22:20
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("eventoRepactuacaoService")
public class EventoRepactuacaoServiceImpl extends GenericServiceImpl<EventoRepactuacao, Long> implements EventoRepactuacaoService {

	@Autowired
	private EventoRepactuacaoDao eventoRepactuacaoDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private RegistroOcorrenciaService registroOcorrenciaService;

	@Autowired
	public EventoRepactuacaoServiceImpl(EventoRepactuacaoDao eventoRepactuacaoDao, @Qualifier("eventoRepactuacaoValidator") Validator validator ) {

		this.dao = eventoRepactuacaoDao;
		this.eventoRepactuacaoDao = eventoRepactuacaoDao;
		this.validator = validator;
	}

	@Override
	public EventoRepactuacao save(EventoRepactuacao eventoRepactuacao) {

		configurarCodigo(eventoRepactuacao);
		montarObjeto(eventoRepactuacao);
		this.validarEntidade(eventoRepactuacao, this.validator);

		return super.save(eventoRepactuacao);
	}

	@Override
	public EventoRepactuacao merge(EventoRepactuacao eventoRepactuacao) {
		montarObjeto(eventoRepactuacao);
		this.validarEntidade(eventoRepactuacao, this.validator);
		
		EventoRepactuacao entityMerged = super.save(eventoRepactuacao);
		return entityMerged;
	}
	/**
	 *
	 * @param eventoRepactuacao
	 */
	private void montarObjeto(EventoRepactuacao eventoRepactuacao) {

		eventoRepactuacao.setRegistroOcorrencia(this.registroOcorrenciaService.find(eventoRepactuacao.getRegistroOcorrencia().getId()));

		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		Integer ano = eventoRepactuacao.getDataRepactuacao().get(Calendar.YEAR);
		String[] numeroMask = eventoRepactuacao.getNumeroRepactuacao().split(ano.toString());

		String descricao = internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_REPACTUACAO", this.getIdioma());
		descricao = String.format(descricao, numeroMask.length > 0 ? numeroMask[0] + "/" + ano : eventoRepactuacao.getNumeroRepactuacao(), sdf.format(eventoRepactuacao.getDataRepactuacao().getTime()));
		eventoRepactuacao.setDescricao(descricao);
		eventoRepactuacao.setContrato(this.contratoService.getReference(eventoRepactuacao.getContrato().getId()));
		eventoRepactuacao.setDataEvento(eventoRepactuacao.getDataRepactuacao());
		eventoRepactuacao.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_REPACTUACAO));

		// Observacoes
		if (CollectionUtils.isNotEmpty(eventoRepactuacao.getObservacoes())) {
			Collection<EventoRepactuacaoObservacao> observacoes = eventoRepactuacao.getObservacoes();

			for (EventoRepactuacaoObservacao observacao : observacoes) {
				Long idAutor = null;
				if (observacao.getAutor() != null) {
					idAutor = observacao.getAutor().getId();
					observacao.setAutor(this.usuarioService.find(idAutor));
				}
				observacao.setEventoRepactuacao(eventoRepactuacao);
			}
			eventoRepactuacao.setObservacoes(observacoes);
		}

		// Documentos
		if (CollectionUtils.isNotEmpty(eventoRepactuacao.getDocumentos())) {
			Collection<EventoRepactuacaoDocumento> documentos = eventoRepactuacao.getDocumentos();

			for (EventoRepactuacaoDocumento documento : documentos) {
				documento.setEventoRepactuacao(eventoRepactuacao);
				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (documento.getEmitente() != null && documento.getEmitente().getId() != null) {
					documento.setEmitente(this.parceiroService.getReference(documento.getEmitente().getId()));
				}
			}
			eventoRepactuacao.setDocumentos(documentos);
		}
	}

	public Dominio getIdioma() {
		return dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
	}

	@Override
	public JasperPrint getJasperPrintRepactuacao(Long id, Connection conn, Usuario user, String titulo) throws JRException {
		InputStream inputSub = null;
		InputStream input = null;
		try {

			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/eventoRepactuacao.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);

			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("REPACTUACAO_ID", id);
			subReportParameters.put("REPORT_CONNECTION", conn);


			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", user.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}

	@Override
	public JasperPrint getJasperPrintRelatorioRepactuacao(Long idContrato, Date dataInicial, Date dataFinal, 
			Connection conn, Usuario usuarioLogado, String titulo) throws JRException {

		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/relatorioRepactuacoes.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("CONTRATO_ID", idContrato);
			subReportParameters.put("DATA_INICIAL", dataInicial);
			subReportParameters.put("DATA_FINAL", dataFinal);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}

	private void configurarCodigo(EventoRepactuacao entitySaved) {
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Integer ano = entitySaved.getDataRepactuacao().get(Calendar.YEAR);
		EventoRepactuacao ultimoRegistro = (EventoRepactuacao) this.eventoRepactuacaoDao.buscarUltimoRegistroPorDataEmissaoPorContrato("numeroRepactuacao", ano.toString(), usuarioLogado.getOrganizacao().getId(), entitySaved.getContrato().getId());
		Long sequencial = null;

		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getNumeroRepactuacao().replace(ano.toString(), "") == null ? null : Long.parseLong(ultimoRegistro.getNumeroRepactuacao().replace(ano.toString(), "") );
		}else{
			sequencial = 0l;
		}

		entitySaved.setNumeroRepactuacao(StringUtils.leftPad((Long.valueOf(sequencial + 1)).toString(), 5, "0") + ano);
	}
}
