package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.EventoRescisaoContratoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.Contrato;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoRescisaoContrato;
import br.com.centralit.api.model.EventoRescisaoContratoDocumento;
import br.com.centralit.api.model.EventoRescisaoContratoObservacao;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoRescisaoContratoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.Util;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 10/03/2016 - 16:04:45
 *
 * @version 1.0.0
 *
 * @author erick.sato
 *
 */
@Service("eventoRescisaoContratoService")
public class EventoRescisaoContratoServiceImpl extends GenericServiceImpl<EventoRescisaoContrato, Long> implements EventoRescisaoContratoService {

    private EventoRescisaoContratoDao eventoRescisaoContratoDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private AnexoService anexoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroServiceFornecedor;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ReportController reportService;

	private Dominio idioma;

    @Autowired
	public EventoRescisaoContratoServiceImpl(EventoRescisaoContratoDao eventoRescisaoContratoDao, @Qualifier("eventoRescisaoContratoValidator") Validator validator ) {
    	this.dao = eventoRescisaoContratoDao;
        this.eventoRescisaoContratoDao = eventoRescisaoContratoDao;
        this.validator = validator;
	}

	public EventoRescisaoContrato save(EventoRescisaoContrato entity) {

		Contrato contrato = this.contratoService.find(entity.getContrato().getId());
		entity.setContrato(contrato);
		configurarCodigo(entity);
		this.validarEntidade(entity, this.validator);
		alteraStatusContratoParaRescindido(contrato);
		entity.setDataEvento(entity.getDataRescisao());
		String descricao = "";
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		String date = sdf.format(new Date());
		Integer ano = entity.getDataRescisao().get(Calendar.YEAR);
		String[] numeroMask = entity.getNumeroRescisao().split(ano.toString());
		if(entity.getDevolucaoValor()){
			descricao = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_RESCISAO_CONTRATO_COM_DEVOLUCAO_VALOR", this.getIdioma());
			entity.setDescricao(String.format(descricao, numeroMask.length > 0 ? numeroMask[0] + "/" + ano : entity.getNumeroRescisao(), date, Util.convertBigDecimalToString(entity.getValorADevolver())));
		}else{
			descricao = this.internacionalizacaoService.getTranslate("CONTRATOS.MSG.DESCRICAO_EVENTO_RESCISAO_CONTRATO_SEM_DEVOLUCAO_VALOR", this.getIdioma());
			entity.setDescricao(String.format(descricao, numeroMask.length > 0 ? numeroMask[0] + "/" + ano : entity.getNumeroRescisao(), date));
		}
		entity.setTipoEvento(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_EVENTO_CONTRATO, DominioContrato.TIPO_EVENTO_RESCISAO));

		this.montarDocsObs(entity);
		EventoRescisaoContrato entitySaved = super.save(entity);
		return entitySaved;
	}

	private void alteraStatusContratoParaRescindido(Contrato contrato) {
		contrato.setDominioSituacaoAtual(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_SITUACAO_CONTRATO, DominioContrato.CODIGO_TIPO_SITUACAO_CONTRATO_RESCINDIDO));
	}

	private void montarDocsObs(EventoRescisaoContrato entity) {

		if (!UtilColecao.isVazio(entity.getObservacoes())) {

			for (EventoRescisaoContratoObservacao observacao : entity.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(observacao.getAutor())) {
					idAutor = observacao.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				observacao.setAutor(this.usuarioService.find(idAutor));

				observacao.setEventoRescisaoContrato(entity);
			}
		}

		if (!UtilColecao.isVazio(entity.getDocumentos())) {

			for (EventoRescisaoContratoDocumento documento : entity.getDocumentos()) {

				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(documento.getEmitente()) && UtilObjeto.isReferencia(documento.getEmitente().getId())) {

					documento.setEmitente(this.parceiroServiceFornecedor.find(documento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(documento.getAnexos())){
					Anexo anexo = (Anexo) ((List)documento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					documento.setAnexos(listaAnexo);
				}

				documento.setEventoRescisaoContrato(entity);
			}
		}
	}

	public Dominio getIdioma() {
		if(!UtilObjeto.isReferencia(idioma)){
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public void gerarPdfTermo(Long idRescisaoContrato, boolean download, HttpServletResponse response) {
		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;

		try {
			conn = reportService.getConnection();
			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
			String titulo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.TITULO_DOCUMENTO_RESCISAO_CONTRATO", this.getIdioma());

			reportService.gerarRelatorio(download, "pdf", this.getJasperPrintRescisao(idRescisaoContrato, conn, usuarioLogado, titulo), response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		} catch (Exception e) {
			throw new BusinessException();
		} finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}

	@Override
	public JasperPrint getJasperPrintRescisao(Long id, Connection conn, Usuario usuarioLogado, String titulo) throws JRException {
		InputStream inputSub = null;
		InputStream input = null;
		try {
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/eventoRescisaoContrato.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("RESCISAO_ID", id);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			return JasperFillManager.fillReport(jasperReport, parameters, conn);
		} finally{
			UtilDataBase.closeResources(inputSub);
		}
	}

	private void configurarCodigo(EventoRescisaoContrato entitySaved) {
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Integer ano = entitySaved.getDataRescisao().get(Calendar.YEAR);
		EventoRescisaoContrato ultimoRegistro = (EventoRescisaoContrato) this.eventoRescisaoContratoDao.buscarUltimoRegistroPorDataEmissaoPorContrato("numeroRescisao", ano.toString(), usuarioLogado.getOrganizacao().getId(), entitySaved.getContrato().getId());

		Long sequencial = null;

		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getNumeroRescisao().replace(ano.toString(), "") == null ? null : Long.parseLong(ultimoRegistro.getNumeroRescisao().replace(ano.toString(), "") );
		}else{
			sequencial = 0l;
		}

		entitySaved.setNumeroRescisao(StringUtils.leftPad((Long.valueOf(sequencial + 1)).toString(), 5, "0") + ano);
	}
}
