package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.GrupoService;
import br.com.centralit.api.service.MenuGrupoService;
import br.com.centralit.api.service.MenuService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.api.service.PaginaService;
import br.com.centralit.framework.model.Grupo;
import br.com.centralit.framework.model.Menu;
import br.com.centralit.framework.model.MenuFile;
import br.com.centralit.framework.model.MenuGrupo;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.model.Pagina;
import br.com.centralit.framework.model.UtilStartup;
import br.com.centralit.framework.util.UtilColecao;

@Service("inicializarContratosService")
public class InicializarContratosServiceImpl extends UtilStartup {

	@Autowired
	private PaginaService paginaService;

	/** Atributo menuService. */
	@Autowired
	private MenuService menuService;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	@Autowired
	private ModuloService moduloService;
	
	/** Atributo menuContratos. */
	Menu menuContratos;

	public void gerarCarga() {

		criarPaginasMenus();
		this.menuService.adicionarGrupoMasterMenu(menuContratos);
	}

	/**
	 *
	 * Método responsável por criar paginas e menus do modulo
	 *
	 * @author iago.almeida
	 *
	 */
	private void criarPaginasMenus() {

		dominioJS = this.dominioService.findByChaveAndNome(TIPO_FILE, TIPO_FILE_JS);

		moduloSelecionado = this.moduloService.getModuloPorBaseUrl(BASE_URL_CONTRATO, null);

		// Menu pai Contratos
		this.menuContratos = new Menu("Contratos", null, null, null, 5, COR_CONTRATOS, CSS_MENU_CONTRATOS, OPACIDADEMENUCONTRATOS, CLASSE_CONTRATOS, ICONE_MENU_CONTRATO, moduloSelecionado);
		this.menuContratos = this.menuService.mergeIfNotExist(this.menuContratos);

		// Submenu Demandas
//		criarSubMenuDemandas(moduloSelecionado);
		// Submenu Planejamento da contratação
		criarSubMenuPlanejamentoContratacao(moduloSelecionado);
		// Submenu Seleção de fornecedor
//		criarSubMenuSelecaoFornecedor(moduloSelecionado);
		// Submenu Gestão e fiscalização de contrato
		criarSubMenuGestaoFiscalizacaoContrato(moduloSelecionado);
		// Submenu Controles
		criarSubMenuControles(moduloSelecionado);
		// Submenu Termos e relatórios
		criarSubMenuTermosRelatorios(moduloSelecionado);

		Logger.getLogger(InicializarContratosServiceImpl.class).info("Menu executado - InicializarContratoServiceImpl!");
	}

	private void criarSubMenuDemandas(Modulo modulo) {
		Menu menuDemandas = new Menu("Demandas", null, this.menuContratos, 0, 1, null, null, null, null, modulo, "DEMANDAS");
		menuDemandas = this.menuService.mergeIfNotExist(menuDemandas);

		// Menu Solicitação de demanda
		criarOpcaoSolicitacaoDemanda(modulo, menuDemandas);
		// Menu Acompanhamento de demandas
		criarOpcaoAtendimentoDemendas(modulo, menuDemandas);
	}

	private void criarSubMenuPlanejamentoContratacao(Modulo modulo) {
		Menu menuPlanejamentoContrat = new Menu("Planejamento da contratação", null, this.menuContratos, 0, 2, null, null, null, null, modulo, "PLANEJAMENTO_DA_CONTRATACAO");
		menuPlanejamentoContrat = this.menuService.mergeIfNotExist(menuPlanejamentoContrat);

		// Menu Programa
		criarOpcaoPrograma(modulo, menuPlanejamentoContrat);
		// Menu Alinhamento estratégico
		criarOpcaoAlinhamentoEstrategico(modulo, menuPlanejamentoContrat);
		// Menu Estudo técnico
		criarOpcaoEstudoTecnico(modulo, menuPlanejamentoContrat);
		// Menu Termo de referência / Projeto básico
		criarOpcaoTermoReferenciaProjetoBasico(modulo, menuPlanejamentoContrat);
		// Menu Gestão de riscos
//		criarOpcaoGestaoRiscos(modulo, menuPlanejamentoContrat);
	}

	private void criarSubMenuSelecaoFornecedor(Modulo modulo) {
		Menu menuSelecaoFornecedor = new Menu("Seleção de fornecedor", null, this.menuContratos, 0, 3, null, null, null, null, modulo, "SELECAO_DE_FORNECEDOR");
		menuSelecaoFornecedor = this.menuService.mergeIfNotExist(menuSelecaoFornecedor);

		// Menu Cadastro de pesquisa de mercado
		criarOpcaoCadastroPesquisaMercado(modulo, menuSelecaoFornecedor);
		// Menu Cadastro de minuta de edital
		criarOpcaoCadastroMinutaEdital(modulo, menuSelecaoFornecedor);
	}

	private void criarSubMenuGestaoFiscalizacaoContrato(Modulo modulo) {
		Menu menuGestaoFiscalContrato = new Menu("Gestão e fiscalização de contrato", null, this.menuContratos, 0, 4, null, null, null, null, modulo, "GESTAO_FISCALIZACAO_CONTRATO");
		menuGestaoFiscalContrato = this.menuService.mergeIfNotExist(menuGestaoFiscalContrato);

		// Menu Contrato
		criarOpcaoContrato(modulo, menuGestaoFiscalContrato);
		// Menu Cronograma de execução
		criarOpcaoCronogramaExecucao(modulo, menuGestaoFiscalContrato);
		// Menu Ordem de fornecimento de bens e serviços
		criarOpcaoOrdemFornecimentoBensServicos(modulo, menuGestaoFiscalContrato);
		// Menu Acompanhamento
		criarOpcaoAcompanhamento(modulo, menuGestaoFiscalContrato);
		// Menu Registro de ocorrências
		criarOpcaoRegistroOcorrencias(modulo, menuGestaoFiscalContrato);
		// Menu Notificação
		criarOpcaoNotificacao(modulo, menuGestaoFiscalContrato);
		// Menu Encerramento de contratos
		criarOpcaoEncerramentoContratos(modulo, menuGestaoFiscalContrato);
	}

	private void criarSubMenuControles(Modulo modulo) {
		Menu menuControles = new Menu("Controles", null, this.menuContratos, 1, 1, null, null, null, null, modulo, "CONTROLES");
		menuControles = this.menuService.mergeIfNotExist(menuControles);

		// Menu Controle de registro de preços
//		criarOpcaoControleRegistroPrecos(modulo, menuControles);
		// Menu Checklist de verificação do planejamento da contratação
//		criarOpcaoChecklistVerificacaoPlanejamentoContratacao(modulo, menuControles);
		// Menu Questionarios
		criarOpcaoQuestionario(modulo, menuControles);
		// Menu Alertas
//		criarOpcaoAlertas(modulo, menuControles);
		// Menu Indicadores
//		criarOpcaoIndicacores(modulo, menuControles);
		// Menu Definições de contratos
//		criarOpcaoDefinicoesContratos(modulo, menuControles);
	}

	private void criarSubMenuTermosRelatorios(Modulo modulo) {
		Menu menuTermosRelatorios = new Menu("Termos e relatórios", null, this.menuContratos, 1, 2, null, null, null, null, modulo, "TERMOS_E_RELATORIOS");
		menuTermosRelatorios = this.menuService.mergeIfNotExist(menuTermosRelatorios);

		criarConsultaTermo(modulo, menuTermosRelatorios);
		// Menu Consulta de relatórios
		criarOpcaoConsultaRelatorios(modulo, menuTermosRelatorios);
	}

	private void criarOpcaoSolicitacaoDemanda(Modulo modulo, Menu menuDemandas) {

		Pagina paginaSolicitacaoDemanda = new Pagina("Solicitação de demanda", "/cit-contratos-web/html/solicitacaoDemanda/solicitacaoDemanda.html");
		paginaSolicitacaoDemanda = this.paginaService.saveIfNotExist(paginaSolicitacaoDemanda);
		Menu menuSolicitacaoDemanda = new Menu("Solicitação de demanda", paginaSolicitacaoDemanda, menuDemandas, null, 1, null, null, null, null, modulo, "SOLICITACAO_DEMANDA");

		this.menuService.mergeIfNotExist(menuSolicitacaoDemanda);
	}

	private void criarOpcaoAtendimentoDemendas(Modulo modulo, Menu menuDemandas) {

		Pagina paginaAtendimentoDemandas = new Pagina("Atendimento de demandas", "/cit-contratos-web/html/atendimentoDemandas/atendimentoDemandas.html");
		paginaAtendimentoDemandas = this.paginaService.saveIfNotExist(paginaAtendimentoDemandas);
		Menu menuAtendimentoDemandas = new Menu("Atendimento de demandas", paginaAtendimentoDemandas, menuDemandas, null, 2, null, null, null, null, modulo, "ATENDIMENTO_DEMANDAS");

		this.menuService.mergeIfNotExist(menuAtendimentoDemandas);
	}

	private void criarOpcaoPrograma(Modulo modulo, Menu menuPlanejamentoContrat) {

		Pagina paginaPrograma = new Pagina("Programas", "/cit-contratos-web/html/programa/programa.html");
		paginaPrograma = this.paginaService.saveIfNotExist(paginaPrograma);
		Menu menuPrograma = new Menu("Programas", paginaPrograma, menuPlanejamentoContrat, null, 1, null, null, null, null, modulo, "PROGRAMAS");

		List<MenuFile> filesMenuFilePrograma = new ArrayList<MenuFile>();
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaListController.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaListController.min.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaController.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaController.min.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ProgramaRepository.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ProgramaRepository.min.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaAcaoListController.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaAcaoListController.min.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaAcaoController.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ProgramaAcaoController.min.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ProgramaAcaoRepository.js", this.dominioJS, menuPrograma));
		filesMenuFilePrograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ProgramaAcaoRepository.min.js", this.dominioJS, menuPrograma));
		menuPrograma.setIncludes(filesMenuFilePrograma);

		this.menuService.mergeIfNotExist(menuPrograma);
	}

	private void criarOpcaoAlinhamentoEstrategico(Modulo modulo, Menu menuPlanejamentoContrat) {

		Pagina paginaAlinhamentoEstrategico = new Pagina("Alinhamento estratégico", "/cit-contratos-web/html/objetivoEstrategico/objetivoEstrategico.html");
		paginaAlinhamentoEstrategico = this.paginaService.saveIfNotExist(paginaAlinhamentoEstrategico);
		Menu menuAlinhamentoEstrategico = new Menu("Alinhamento estratégico", paginaAlinhamentoEstrategico, menuPlanejamentoContrat, null, 2, null, null, null, null, modulo, "ALINHAMENTO_ESTRATEGICO");

		List<MenuFile> filesMenuFileAlinhamentoEstrategico = new ArrayList<MenuFile>();
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ObjetivoEstrategicoListController.js", this.dominioJS, menuAlinhamentoEstrategico));
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ObjetivoEstrategicoListController.min.js", this.dominioJS, menuAlinhamentoEstrategico));
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ObjetivoEstrategicoController.js", this.dominioJS, menuAlinhamentoEstrategico));
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ObjetivoEstrategicoController.min.js", this.dominioJS, menuAlinhamentoEstrategico));
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ObjetivoEstrategicoRepository.js", this.dominioJS, menuAlinhamentoEstrategico));
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ObjetivoEstrategicoRepository.min.js", this.dominioJS, menuAlinhamentoEstrategico));
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ObjetivoEstrategicoNecessidadeRepository.js", this.dominioJS, menuAlinhamentoEstrategico));
		filesMenuFileAlinhamentoEstrategico.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ObjetivoEstrategicoNecessidadeRepository.min.js", this.dominioJS, menuAlinhamentoEstrategico));
		menuAlinhamentoEstrategico.setIncludes(filesMenuFileAlinhamentoEstrategico);

		this.menuService.mergeIfNotExist(menuAlinhamentoEstrategico);
	}

	private void criarOpcaoEstudoTecnico(Modulo modulo, Menu menuPlanejamentoContrat) {

		Pagina paginaEstudoTecnico = new Pagina("Estudo técnico", "/cit-contratos-web/html/estudoTecnico/estudoTecnico.html");
		paginaEstudoTecnico = this.paginaService.saveIfNotExist(paginaEstudoTecnico);
		Menu menuEstudoTecnio = new Menu("Estudo técnico", paginaEstudoTecnico, menuPlanejamentoContrat, null, 3, null, null, null, null, modulo, "ESTUDO_TECNICO");

		List<MenuFile> filesMenuFileQuestionatio = new ArrayList<MenuFile>();
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EstudoTecnicoListController.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EstudoTecnicoListController.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EstudoTecnicoController.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EstudoTecnicoController.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRequisitoNegocioRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRequisitoNegocioRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoLevantamentoAlternativasRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoLevantamentoAlternativasRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoQuestionarioRespostaRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoQuestionarioRespostaRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoBemServicoSolucaoRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoBemServicoSolucaoRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoBeneficioEsperadoRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoBeneficioEsperadoRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoAcaoContingenciaRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoAcaoContingenciaRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRecursoMaterialRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRecursoMaterialRepository.min.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRecursoHumanoRepository.js", this.dominioJS, menuEstudoTecnio));
		filesMenuFileQuestionatio.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EstudoTecnicoRecursoHumanoRepository.min.js", this.dominioJS, menuEstudoTecnio));
		menuEstudoTecnio.setIncludes(filesMenuFileQuestionatio);

		this.menuService.mergeIfNotExist(menuEstudoTecnio);
	}

	private void criarOpcaoTermoReferenciaProjetoBasico(Modulo modulo, Menu menuPlanejamentoContrat) {

		Pagina paginaTermo = new Pagina("Termo de referência/Projeto básico", "/cit-contratos-web/html/termoReferencia/termoReferencia.html");
		paginaTermo = this.paginaService.saveIfNotExist(paginaTermo);
		Menu menuTermo = new Menu("Termo de referência/Projeto básico", paginaTermo, menuPlanejamentoContrat, null, 4, null, null, null, null, modulo, "TERMO_REFERENCIA");

		List<MenuFile> filesMenuFileTermo = new ArrayList<MenuFile>();
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/TermoReferenciaListController.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/TermoReferenciaListController.min.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/TermoReferenciaController.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/TermoReferenciaController.min.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaRepository.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaRepository.min.js", this.dominioJS, menuTermo));

		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaSancaoRepository.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaSancaoRepository.min.js", this.dominioJS, menuTermo));

		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaPenalidadeRepository.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaPenalidadeRepository.min.js", this.dominioJS, menuTermo));

		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaDetalhamentoSolucaoRepository.js", this.dominioJS, menuTermo));
		filesMenuFileTermo.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/TermoReferenciaDetalhamentoSolucaoRepository.min.js", this.dominioJS, menuTermo));
		menuTermo.setIncludes(filesMenuFileTermo);

		this.menuService.mergeIfNotExist(menuTermo);
	}

	private void criarOpcaoGestaoRiscos(Modulo modulo, Menu menuPlanejamentoContrat) {

		Pagina paginaGestaoRisco = new Pagina("Gestão de riscos", "/cit-contratos-web/html/gestaoRiscos/gestaoRiscos.html");
		paginaGestaoRisco = this.paginaService.saveIfNotExist(paginaGestaoRisco);
		Menu menuGestaoRisco = new Menu("Gestão de riscos", paginaGestaoRisco, menuPlanejamentoContrat, null, 5, null, null, null, null, modulo, "GESTAO_RISCOS");

		List<MenuFile> filesMenuFileGestao = new ArrayList<MenuFile>();
		menuGestaoRisco.setIncludes(filesMenuFileGestao);

		this.menuService.mergeIfNotExist(menuGestaoRisco);
	}

	private void criarOpcaoCadastroPesquisaMercado(Modulo modulo, Menu menuSelecaoFornecedor) {

		Pagina paginaPesquisaMercado = new Pagina("Cadastro de pesquisa de mercado", "/cit-contratos-web/html/pesquisaMercado/pesquisaMercado.html");
		paginaPesquisaMercado = this.paginaService.saveIfNotExist(paginaPesquisaMercado);
		Menu menuPesquisaMercado = new Menu("Cadastro de pesquisa de mercado", paginaPesquisaMercado, menuSelecaoFornecedor, null, 1, null, null, null, null, modulo, "PESQUISA_MERCADO");

		this.menuService.mergeIfNotExist(menuPesquisaMercado);
	}

	private void criarOpcaoCadastroMinutaEdital(Modulo modulo, Menu menuSelecaoFornecedor) {

		Pagina paginaMinutaEdital = new Pagina("Cadastro de minuta de edital", "/cit-contratos-web/html/minutaEdital/minutaEdital.html");
		paginaMinutaEdital = this.paginaService.saveIfNotExist(paginaMinutaEdital);
		Menu menuMinutaEdital = new Menu("Cadastro de minuta de edital", paginaMinutaEdital, menuSelecaoFornecedor, null, 2, null, null, null, null, modulo, "MINUTA_EDITAL");

		this.menuService.mergeIfNotExist(menuMinutaEdital);
	}

	private void criarOpcaoContrato(Modulo modulo, Menu menuGestaoFiscalContrato) {

		Pagina pgContrato = new Pagina("Contrato", "/cit-contratos-web/html/contrato/contrato.html");
		pgContrato = this.paginaService.saveIfNotExist(pgContrato);
		Menu menuContrato = new Menu("Contrato", pgContrato, menuGestaoFiscalContrato, null, 1, null, null, null, null, modulo, "CONTRATO");

		List<MenuFile> filesMenuFileContrato = this.gerarArquivosMenu(menuContrato, CIT_CONTRATOS_WEB_ANGULAR_CUSTOM, "Contrato", true, true, true);
		filesMenuFileContrato.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoGarantiaRepository.js", this.dominioJS, menuContrato));
		filesMenuFileContrato.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoGarantiaRepository.min.js", this.dominioJS, menuContrato));
		filesMenuFileContrato.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoEnvolvidoRepository.js", this.dominioJS, menuContrato));
		filesMenuFileContrato.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoEnvolvidoRepository.min.js", this.dominioJS, menuContrato));
		filesMenuFileContrato.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoItemRepository.js", this.dominioJS, menuContrato));
		filesMenuFileContrato.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoItemRepository.min.js", this.dominioJS, menuContrato));
		menuContrato.setIncludes(filesMenuFileContrato);

		this.menuService.mergeIfNotExist(menuContrato);
	}

	private void criarOpcaoCronogramaExecucao(Modulo modulo, Menu menuGestaoFiscalContrato) {

		Pagina pgCronograma = new Pagina("Cronograma de execução", "/cit-contratos-web/html/cronogramaExecucao/cronogramaExecucao.html");
		pgCronograma = this.paginaService.saveIfNotExist(pgCronograma);
		Menu menuCronograma = new Menu("Cronograma de execução", pgCronograma, menuGestaoFiscalContrato, null, 2, null, null, null, null, modulo, "CRONOGRAMA_DE_EXECUCAO");

		List<MenuFile> filesMenuFileCronograma = this.gerarArquivosMenu(menuCronograma, CIT_CONTRATOS_WEB_ANGULAR_CUSTOM, "CronogramaExecucao", true, true, true);
		filesMenuFileCronograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/CronogramaExecucaoItemRepository.js", this.dominioJS, menuCronograma));
		filesMenuFileCronograma.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/CronogramaExecucaoItemRepository.min.js", this.dominioJS, menuCronograma));
		menuCronograma.setIncludes(filesMenuFileCronograma);

		this.menuService.mergeIfNotExist(menuCronograma);
	}

	private void criarOpcaoOrdemFornecimentoBensServicos(Modulo modulo, Menu menuGestaoFiscalContrato) {

		Pagina pgOrdemFornecimentoBensServicos = new Pagina("Ordem de fornecimento de bens e serviços", "/cit-contratos-web/html/ordemFornecimento/ordemFornecimento.html");
		pgOrdemFornecimentoBensServicos = this.paginaService.saveIfNotExist(pgOrdemFornecimentoBensServicos);
		Menu menuOrdemFornecimento = new Menu("Ordem de fornecimento de bens e serviços", pgOrdemFornecimentoBensServicos, menuGestaoFiscalContrato, null, 3, null, null, null, null, modulo, "ORDEM_FORNECIMENTO");

		List<MenuFile> filesMenuFileOrdemFornecimento = this.gerarArquivosMenu(menuOrdemFornecimento, CIT_CONTRATOS_WEB_ANGULAR_CUSTOM, "OrdemFornecimento", true, true, true);
		menuOrdemFornecimento.setIncludes(filesMenuFileOrdemFornecimento);

		this.menuService.mergeIfNotExist(menuOrdemFornecimento);
	}

	private void criarOpcaoAcompanhamento(Modulo modulo, Menu menuGestaoFiscalContrato) {

		Pagina pgOrdemAcompanhamento = new Pagina("Acompanhamento", "/cit-contratos-web/html/visaoContrato/visaoContrato.html");
		pgOrdemAcompanhamento = this.paginaService.saveIfNotExist(pgOrdemAcompanhamento);
		Menu menuAcompanhamento = new Menu("Acompanhamento", pgOrdemAcompanhamento, menuGestaoFiscalContrato, null, 4, null, null, null, null, modulo, "ACOMPANHAMENTO");

		List<MenuFile> filesMenuAcompanhamento = this.gerarArquivosMenu(menuAcompanhamento, CIT_CONTRATOS_WEB_ANGULAR_CUSTOM, "VisaoContrato", true, false, false);
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoEventoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoEventoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoAtualizacaoStatusRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoAtualizacaoStatusRepository.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoAtualizacaoStatusController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoAtualizacaoStatusController.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoLiberacaoPagamentoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoLiberacaoPagamentoController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoLiberacaoPagamentoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoLiberacaoPagamentoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoEmpenhoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/ContratoEmpenhoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoApostilamentoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoApostilamentoController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoApostilamentoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoApostilamentoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoPenalidadeController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoPenalidadeController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoPenalidadeRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoPenalidadeRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoAditivoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoAditivoController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoAditivoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoAditivoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRegistroNaoConformidadeController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRegistroNaoConformidadeController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRegistroNaoConformidadeRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRegistroNaoConformidadeRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRescisaoContratoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRescisaoContratoController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRescisaoContratoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRescisaoContratoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/TermoEventoContratoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/TermoEventoContratoController.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRepactuacaoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRepactuacaoController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRepactuacaoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRepactuacaoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoEmpenhoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoEmpenhoController.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRecebimentoObjetoController.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/EventoRecebimentoObjetoController.min.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRecebimentoObjetoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoRecebimentoObjetoRepository.min.js", this.dominioJS, menuAcompanhamento));

		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoLiberacaoPagamentoEmpenhoRepository.js", this.dominioJS, menuAcompanhamento));
		filesMenuAcompanhamento.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/EventoLiberacaoPagamentoEmpenhoRepository.min.js", this.dominioJS, menuAcompanhamento));


		menuAcompanhamento.setIncludes(filesMenuAcompanhamento);

		this.menuService.mergeIfNotExist(menuAcompanhamento);
	}

	private void criarOpcaoRegistroOcorrencias(Modulo modulo, Menu menuGestaoFiscalContrato) {

		Pagina pgRegistroOcorrencias = new Pagina("Registro de ocorrências", "/cit-contratos-web/html/registroOcorrencias/registroOcorrencias.html");
		pgRegistroOcorrencias = this.paginaService.saveIfNotExist(pgRegistroOcorrencias);
		Menu menuRegistroOcorrencias = new Menu("Registro de ocorrências", pgRegistroOcorrencias, menuGestaoFiscalContrato, null, 5, null, null, null, null, modulo, "REGISTRO_OCORRENCIAS");

		List<MenuFile> filesMenuFileRegistroOcorrencias = this.gerarArquivosMenu(menuRegistroOcorrencias, CIT_CONTRATOS_WEB_ANGULAR_CUSTOM, "RegistroOcorrencia", true, true, true);
		menuRegistroOcorrencias.setIncludes(filesMenuFileRegistroOcorrencias);

		this.menuService.mergeIfNotExist(menuRegistroOcorrencias);
	}

	private void criarOpcaoNotificacao(Modulo modulo, Menu menuGestaoFiscalContrato) {

		Pagina pgNotificacao = new Pagina("Notificação", "/cit-contratos-web/html/notificacao/notificacao.html");
		pgNotificacao = this.paginaService.saveIfNotExist(pgNotificacao);
		Menu menuNotificacao = new Menu("Notificação", pgNotificacao, menuGestaoFiscalContrato, null, 6, null, null, null, null, modulo, "NOTIFICACAO_CONTRATO");

		List<MenuFile> filesMenuNotificacao = this.gerarArquivosMenu(menuNotificacao, CIT_CONTRATOS_WEB_ANGULAR_CUSTOM, "NotificacaoContrato", true, true, true);
		filesMenuNotificacao.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/NotificacaoContratoOcorrenciaRepository.js", this.dominioJS, menuNotificacao));
		filesMenuNotificacao.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/NotificacaoContratoOcorrenciaRepository.min.js", this.dominioJS, menuNotificacao));
		menuNotificacao.setIncludes(filesMenuNotificacao);

		this.menuService.mergeIfNotExist(menuNotificacao);
	}

	private void criarOpcaoEncerramentoContratos(Modulo modulo, Menu menuGestaoFiscalContrato) {

		Pagina pgEncerramentoContratos = new Pagina("Encerramento de contratos", "/cit-contratos-web/html/encerramentoContrato/encerramentoContrato.html");
		pgEncerramentoContratos = this.paginaService.saveIfNotExist(pgEncerramentoContratos);
		Menu menuEncerramentoContratos = new Menu("Encerramento de contratos", pgEncerramentoContratos, menuGestaoFiscalContrato, null, 7, null, null, null, null, modulo, "ENCERRAMENTO_CONTRATOS");

		List<MenuFile> filesMenuFileEncerramentoContrato = this.gerarArquivosMenu(menuEncerramentoContratos, CIT_CONTRATOS_WEB_ANGULAR_CUSTOM, "EncerramentoContrato", true, true, true);
		menuEncerramentoContratos.setIncludes(filesMenuFileEncerramentoContrato);

		this.menuService.mergeIfNotExist(menuEncerramentoContratos);
	}

	private void criarOpcaoControleRegistroPrecos(Modulo modulo, Menu menuControles) {

		Pagina pgControleRegistroPrecos = new Pagina("Controle de registro de preços", "/cit-contratos-web/html/controleRegistroPrecos/controleRegistroPrecos.html");
		pgControleRegistroPrecos = this.paginaService.saveIfNotExist(pgControleRegistroPrecos);
		Menu menuControleRegistroPrecos = new Menu("Controle de registro de preços", pgControleRegistroPrecos, menuControles, null, 1, null, null, null, null, modulo, "CONTROLE_REGISTRO_PRECOS");

		this.menuService.mergeIfNotExist(menuControleRegistroPrecos);
	}

	private void criarOpcaoChecklistVerificacaoPlanejamentoContratacao(Modulo modulo, Menu menuControles) {

		Pagina pgChecklistVerificacaoPlanejContratacao = new Pagina("Checklist de verificação do planejamento da contratação", "/cit-contratos-web/html/checklistVerificacaoPlanejamentoContratacao/checklistVerificacaoPlanejamentoContratacao.html");
		pgChecklistVerificacaoPlanejContratacao = this.paginaService.saveIfNotExist(pgChecklistVerificacaoPlanejContratacao);
		Menu menuChecklistVerificacaoPlanejContratacao = new Menu("Checklist de verificação do planejamento da contratação", pgChecklistVerificacaoPlanejContratacao, menuControles, null, 2, null, null, null, null, modulo, "CHECKLIST_VERIFICACAO_PLANEJAMENTO_CONTRATACAO");

		this.menuService.mergeIfNotExist(menuChecklistVerificacaoPlanejContratacao);
	}

	private void criarOpcaoQuestionario(Modulo modulo, Menu menuControles) {
		Pagina paginaQuestionarios = new Pagina("Questionários", "/cit-contratos-web/html/questionario/questionario.html");
		paginaQuestionarios = this.paginaService.saveIfNotExist(paginaQuestionarios);
		Menu menuQuestionario = new Menu("Questionários", paginaQuestionarios, menuControles, null, 3, null, null, null, null, modulo, "QUESTIONARIOS");

		List<MenuFile> filesMenuQuestionario = new ArrayList<MenuFile>();
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/QuestionarioController.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/QuestionarioController.min.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/QuestionarioListController.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/QuestionarioListController.min.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/QuestionarioRepository.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/QuestionarioRepository.min.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/QuestionarioPerguntaRepository.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/QuestionarioPerguntaRepository.min.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/PossivelRespostaQuestionarioPerguntaRepository.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/PossivelRespostaQuestionarioPerguntaRepository.min.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/QuestionarioPerguntaRepository.js", this.dominioJS, menuQuestionario));
		filesMenuQuestionario.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "repository/QuestionarioPerguntaRepository.min.js", this.dominioJS, menuQuestionario));
		menuQuestionario.setIncludes(filesMenuQuestionario);

		this.menuService.mergeIfNotExist(menuQuestionario);
	}

	private void criarOpcaoAlertas(Modulo modulo, Menu menuControles) {

		Pagina pgAlertas = new Pagina("Alertas", "/cit-contratos-web/html/alertas/alertas.html");
		pgAlertas = this.paginaService.saveIfNotExist(pgAlertas);
		Menu menuAlertas = new Menu("Alertas", pgAlertas, menuControles, null, 4, null, null, null, null, modulo, "ALERTAS");

		this.menuService.mergeIfNotExist(menuAlertas);
	}

	private void criarOpcaoIndicacores(Modulo modulo, Menu menuControles) {

		Pagina pgIndicadores = new Pagina("Indicadores", "/cit-contratos-web/html/indicadores/indicadores.html");
		pgIndicadores = this.paginaService.saveIfNotExist(pgIndicadores);
		Menu menuIndicadores = new Menu("Indicadores", pgIndicadores, menuControles, null, 5, null, null, null, null, modulo, "INDICADORES");

		this.menuService.mergeIfNotExist(menuIndicadores);
	}

	private void criarOpcaoDefinicoesContratos(Modulo modulo, Menu menuControles) {

		Pagina pgDefinicoesContratos = new Pagina("Definições de contratos", "/cit-contratos-web/html/definicoesContratos/definicoesContratos.html");
		pgDefinicoesContratos = this.paginaService.saveIfNotExist(pgDefinicoesContratos);
		Menu menuDefinicoesContratos = new Menu("Definições de contratos", pgDefinicoesContratos, menuControles, null, 6, null, null, null, null, modulo, "DEFINICOES_CONTRATOS");

		this.menuService.mergeIfNotExist(menuDefinicoesContratos);
	}

	private void criarConsultaTermo(Modulo modulo, Menu menuTermosRelatorios) {

		Pagina pgRelatorioRecisao = new Pagina("Consulta de termos", "/cit-contratos-web/html/consultaTermo/consultaTermo.html");
		pgRelatorioRecisao = this.paginaService.saveIfNotExist(pgRelatorioRecisao);
		Menu menuRelatorioRecisao = new Menu("Consulta de termos", pgRelatorioRecisao, menuTermosRelatorios, null, 1, null, null, null, null, modulo, "CONSULTA_TERMOS");

		this.menuService.mergeIfNotExist(menuRelatorioRecisao);
	}

	private void criarOpcaoConsultaRelatorios(Modulo modulo, Menu menuTermosRelatorios) {
		
		Pagina pgConsultaRelatorios = new Pagina("Consulta de relatórios", "/cit-contratos-web/html/consultaRelatorios/consultaRelatorios.html");
		pgConsultaRelatorios = this.paginaService.saveIfNotExist(pgConsultaRelatorios);
		Menu menuConsultaRelatorios = new Menu("Consulta de relatórios", pgConsultaRelatorios, menuTermosRelatorios, null, 11, null, null, null, null, modulo, "CONSULTA_RELATORIO");
		
		List<MenuFile> filesMenuConsultaRelatorios = new ArrayList<MenuFile>();
		filesMenuConsultaRelatorios.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ConsultaRelatoriosController.js", this.dominioJS, menuConsultaRelatorios));
		filesMenuConsultaRelatorios.add(new MenuFile(CIT_CONTRATOS_WEB_ANGULAR_CUSTOM + "controller/ConsultaRelatoriosController.min.js", this.dominioJS, menuConsultaRelatorios));
		menuConsultaRelatorios.setIncludes(filesMenuConsultaRelatorios);

		this.menuService.mergeIfNotExist(menuConsultaRelatorios);
	}

}
