package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.NotificacaoContratoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.NotificacaoContrato;
import br.com.centralit.api.model.NotificacaoContratoDocumento;
import br.com.centralit.api.model.NotificacaoContratoObservacao;
import br.com.centralit.api.model.NotificacaoContratoOcorrencia;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.NotificacaoContratoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.RegistroOcorrenciaService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

@Service("notificacaoContratoService")
public class NotificacaoContratoServiceImpl extends GenericServiceImpl<NotificacaoContrato, Long> implements NotificacaoContratoService {

	private NotificacaoContratoDao notificacaoContratoDao;

	public static final String CAMPO_BUSCA_PARAMETRIZADA_ULTIMO_REGISTRO = "codigo";

	@Autowired
	private UsuarioService usuarioService;
	@Autowired
	private DominioService dominioService;
	@Autowired
	private AnexoService anexoService;
	@Autowired
	private RegistroOcorrenciaService registroOcorrenciaService;
	@Autowired
	private ParceiroService parceiroServiceFornecedor;
	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;
	@Autowired
	private ReportController reportController;
	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	public NotificacaoContratoServiceImpl(NotificacaoContratoDao notificacaoContratoDao, @Qualifier("notificacaoContratoValidator") Validator validator) {

		this.dao = notificacaoContratoDao;
		this.notificacaoContratoDao = notificacaoContratoDao;
		this.validator = validator;
	}

	public NotificacaoContrato save(NotificacaoContrato entity) {

		validarEntidade(entity, this.validator);
		this.montarNotificacaoContrato(entity);

		if (UtilObjeto.isReferencia(entity.getCodigo())) {
			return super.save(entity);
		} else {
			this.configurarCodigo(entity);
			NotificacaoContrato entitySaved = super.save(entity);
			return entitySaved;
		}
	}

	@Override
	public NotificacaoContrato merge(NotificacaoContrato notificacaoContrato) {

		this.montarNotificacaoContrato(notificacaoContrato);

		if (UtilObjeto.isReferencia(notificacaoContrato.getCodigo())) {
			super.save(notificacaoContrato);
		} else {
			this.configurarCodigo(notificacaoContrato);
			super.save(notificacaoContrato);
		}
		return notificacaoContrato;
	}

	@Override
	public void gerarRelatorioNotificacaoContrato(Long idNotificacaoContrato, boolean download, HttpServletResponse response) {
		Dominio idioma = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);

		Usuario user = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;
		try {
			conn = reportController.getConnection();
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/notificacaoContrato.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);

			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			NotificacaoContrato notificacao = find(idNotificacaoContrato);
			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("NOTIFICACAO_CONTRATO_ID", idNotificacaoContrato);
			subReportParameters.put("VINCULACAO_OCORRENCIA_NAO_APLICA", notificacao.getOcorrencias().size() > 0);
			subReportParameters.put("REPORT_CONNECTION", conn);

			String titulo = this.internacionalizacaoService.getTranslate("NOTIFICACAO_CONTRATO.NOTIFICACAO_TITULO_TERMO", idioma);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", user.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			JasperPrint jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);

			// Responsavel por verificar a presenca de dados no relatorio
			reportController.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		}  catch (Exception e) {
			throw new BusinessException();
		}finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}

	private void montarNotificacaoContrato(NotificacaoContrato notificacaoContrato) {
		this.montarDocsObsNotificacaoContrato(notificacaoContrato);
		atualizaOcorrenciaComNotificacao(notificacaoContrato);
	}

	private void atualizaOcorrenciaComNotificacao(NotificacaoContrato notificacaoContrato) {
		if (!UtilColecao.isVazio(notificacaoContrato.getOcorrencias())) {
			for (NotificacaoContratoOcorrencia ocorrencia : notificacaoContrato.getOcorrencias()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(ocorrencia.getAutor())) {
					idAutor = ocorrencia.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				ocorrencia.setAutor(this.usuarioService.find(idAutor));
				ocorrencia.setRegistroOcorrencia(this.registroOcorrenciaService.find(ocorrencia.getRegistroOcorrencia().getId()));
				ocorrencia.setNotificacaoContrato(notificacaoContrato);
			}
		}
	}

	private void montarDocsObsNotificacaoContrato(NotificacaoContrato notificacaoContrato) {

		if (!UtilColecao.isVazio(notificacaoContrato.getObservacoes())) {

			for (NotificacaoContratoObservacao notificacaoCont : notificacaoContrato.getObservacoes()) {

				Long idAutor = null;

				if (UtilObjeto.isReferencia(notificacaoCont.getAutor())) {
					idAutor = notificacaoCont.getAutor().getId();
				}

				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}

				notificacaoCont.setAutor(this.usuarioService.find(idAutor));

				notificacaoCont.setNotificacaoContrato(notificacaoContrato);
			}
		}

		if (!UtilColecao.isVazio(notificacaoContrato.getDocumentos())) {

			for (NotificacaoContratoDocumento notificacaocontratoDocumento : notificacaoContrato.getDocumentos()) {

				notificacaocontratoDocumento.setDominioTipoDocumento(this.dominioService.find(notificacaocontratoDocumento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(notificacaocontratoDocumento.getEmitente()) && UtilObjeto.isReferencia(notificacaocontratoDocumento.getEmitente().getId())) {

					notificacaocontratoDocumento.setEmitente(this.parceiroServiceFornecedor.find(notificacaocontratoDocumento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(notificacaocontratoDocumento.getAnexos())){
					Anexo anexo = (Anexo) ((List)notificacaocontratoDocumento.getAnexos()).get(0);
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					notificacaocontratoDocumento.setAnexos(listaAnexo);
				}

				notificacaocontratoDocumento.setNotificacaoContrato(notificacaoContrato);
			}
		}
	}

	private void configurarCodigo(NotificacaoContrato entitySaved) {
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Integer ano = entitySaved.getEmissao().get(Calendar.YEAR);
		NotificacaoContrato ultimoRegistro = (NotificacaoContrato) this.notificacaoContratoDao.buscarUltimoAnoRegistroComOrdenadoParametrizada(CAMPO_BUSCA_PARAMETRIZADA_ULTIMO_REGISTRO, ano.toString(), usuarioLogado.getOrganizacao().getId());
		Long sequencial = null;

		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getCodigo().replace(ano.toString(), "") == null ? null : Long.parseLong(ultimoRegistro.getCodigo().replace(ano.toString(), "") );
		}else{
			sequencial = 0l;
		}

		entitySaved.setCodigo(StringUtils.leftPad((Long.valueOf(sequencial + 1)).toString(), 5, "0") + ano);
	}

}
