package br.com.centralit.api.service.impl;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ObjetivoEstrategicoNecessidadeDao;
import br.com.centralit.api.model.ObjetivoEstrategico;
import br.com.centralit.api.model.ObjetivoEstrategicoNecessidade;
import br.com.centralit.api.service.EstudoTecnicoRequisitoNegocioService;
import br.com.centralit.api.service.ObjetivoEstrategicoNecessidadeService;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 08/01/2016 - 12:07:57
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Service("objetivoEstrategicoNecessidadeService")
public class ObjetivoEstrategicoNecessidadeServiceImpl extends GenericServiceImpl<ObjetivoEstrategicoNecessidade, Long> implements ObjetivoEstrategicoNecessidadeService{

	/** Atributo objetivoEstrategicoNecessidadeDao. */
	private ObjetivoEstrategicoNecessidadeDao objetivoEstrategicoNecessidadeDao;

	@Autowired
	private EstudoTecnicoRequisitoNegocioService estudoTecnicoRequisitoNegocioService;

	@Autowired
	public ObjetivoEstrategicoNecessidadeServiceImpl(ObjetivoEstrategicoNecessidadeDao objetivoEstrategicoNecessidadeDao, @Qualifier("objetivoEstrategicoNecessidadeValidator") Validator validator ) {

		this.objetivoEstrategicoNecessidadeDao = objetivoEstrategicoNecessidadeDao;
		this.dao = objetivoEstrategicoNecessidadeDao;
		this.validator = validator;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<ObjetivoEstrategicoNecessidade> findAllByObjetivoEstrategico(Long objEstrategicoId) {
		return this.objetivoEstrategicoNecessidadeDao.findAllByObjetivoEstrategico(objEstrategicoId);
	}

	@Override
	public boolean removeById(Long id) {
		validarExclusao(id);
		
		ObjetivoEstrategicoNecessidade entity = getReference(id);
		
		return super.removePhysically(entity);
	}

	@Override
	public void removeList(List<ObjetivoEstrategicoNecessidade> listEntity) {
		validarListaExclusao(listEntity);
		super.removeList(listEntity);
	}
	
	@Override
	public void salvarListaNecessidades(Collection<ObjetivoEstrategicoNecessidade> necessidades, ObjetivoEstrategico objetivoEstrategico) {

		for(ObjetivoEstrategicoNecessidade necessidade : necessidades){

			necessidade.setObjetivoEstrategico(objetivoEstrategico);

			validarEntidade(necessidade, validator);
			
			if (necessidade.getId() == null) {
				super.save(necessidade);
			} else {
				super.merge(necessidade);
			}
		}

	}

	private void validarListaExclusao(Collection<ObjetivoEstrategicoNecessidade> necessidades) {
		for(ObjetivoEstrategicoNecessidade necessidade : necessidades) {
			validarExclusao(necessidade.getId());
		}
	}

	private void validarExclusao(Long id) {
		Boolean isVinculado = estudoTecnicoRequisitoNegocioService.existeVinculo("objetivoEstrategicoNecessidade.id", id);
		lancarBusinessException("VALIDACAO.REMOCAO_COM_VINCULO", CodigoErro.REGRA_NEGOCIO.getValue(), isVinculado);
	}
	
}
