package br.com.centralit.api.service.impl;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import com.googlecode.genericdao.search.SearchResult;

import br.com.centralit.api.dao.ObjetivoEstrategicoDao;
import br.com.centralit.api.model.ObjetivoEstrategico;
import br.com.centralit.api.model.ObjetivoEstrategicoNecessidade;
import br.com.centralit.api.service.EstudoTecnicoRequisitoNegocioService;
import br.com.centralit.api.service.ObjetivoEstrategicoNecessidadeService;
import br.com.centralit.api.service.ObjetivoEstrategicoService;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;

@Service("objetivoEstrategicoService")
public class ObjetivoEstrategicoServiceImpl extends GenericServiceImpl<ObjetivoEstrategico, Long> implements ObjetivoEstrategicoService {

	private ObjetivoEstrategicoDao objetivoEstrategicoDao;

	@Autowired
	private EstudoTecnicoRequisitoNegocioService estudoTecnicoRequisitoNegocioService;

	@Autowired
	private ObjetivoEstrategicoNecessidadeService objetivoEstrategicoNecessidadeService;

	@Autowired
	public ObjetivoEstrategicoServiceImpl(ObjetivoEstrategicoDao objetivoEstrategicoDao, @Qualifier("objetivoEstrategicoValidator") Validator validator ) {

		this.objetivoEstrategicoDao = objetivoEstrategicoDao;
		this.dao = objetivoEstrategicoDao;
		this.validator = validator;
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por trazer todos os objetivos estratégicos da organizacao do usuário logado.
	 *
	 * @author luis.camargo
	 *
	 * @param organizacaoId ID da organizacao a se buscar os objetivos estratégicos
	 * @return
	 */
	@Override
	public Collection<ObjetivoEstrategico> findAllByOrganizacao(Long organizacaoId) {
		return this.objetivoEstrategicoDao.findAllByOrganizacao(organizacaoId);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por salvar a <code>ObjetivoEstrategico</code>
	 *
	 * @author luis.camargo
	 *
	 * @param objetivoEstrategico
	 * @return <code>ObjetivoEstrategico</code>
	 */
	@Override
	public ObjetivoEstrategico save(ObjetivoEstrategico objeticoEstrategico) {

		this.validarEntidade(objeticoEstrategico, this.validator);

		ObjetivoEstrategico entity = (ObjetivoEstrategico) this.objetivoEstrategicoDao.save(objeticoEstrategico);

		objetivoEstrategicoNecessidadeService.salvarListaNecessidades(objeticoEstrategico.getNecessidadesElencadas(), entity);

		return entity;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por alterar a <code>ObjetivoEstrategico</code>
	 *
	 * @author luis.camargo
	 *
	 * @param objetivoEstrategico
	 * @return <code>ObjetivoEstrategico</code>
	 */
	@Override
	public ObjetivoEstrategico merge(ObjetivoEstrategico objetivoEstrategico) {

		this.validarEntidade(objetivoEstrategico, this.validator);

		ObjetivoEstrategico entity = (ObjetivoEstrategico) this.objetivoEstrategicoDao.merge(objetivoEstrategico);

		objetivoEstrategicoNecessidadeService.salvarListaNecessidades(objetivoEstrategico.getNecessidadesElencadas(), entity);

		return entity;
	}

	@Override
	public boolean removeById(Long id) {
		ObjetivoEstrategico objetivo = getReference(id);
		removerNecessidades(objetivo.getNecessidadesElencadas());
		return super.removeById(id);
	}

	private void removerNecessidades(Collection<ObjetivoEstrategicoNecessidade> necessidadesElencadas) {
		objetivoEstrategicoNecessidadeService.removeList((List<ObjetivoEstrategicoNecessidade>) necessidadesElencadas);
	}

	@Override
	public SearchResult<ObjetivoEstrategico> findAllObjetivosEstrategico(String nomeObjetivo, String descricaoNecessidade, String keyword, int limiteResultado, int pagina) {
		return this.objetivoEstrategicoDao.findAllObjetivosEstrategico(nomeObjetivo, descricaoNecessidade, keyword, limiteResultado, pagina);
	}

}
