package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.OrdemFornecimentoDao;
import br.com.centralit.api.model.Anexo;
import br.com.centralit.api.model.OrdemFornecimento;
import br.com.centralit.api.model.OrdemFornecimentoDocumento;
import br.com.centralit.api.model.OrdemFornecimentoItem;
import br.com.centralit.api.model.OrdemFornecimentoObservacao;
import br.com.centralit.api.service.AnexoService;
import br.com.centralit.api.service.ContratoItemService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.OrdemFornecimentoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 03/02/2016 - 16:59:53
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Service("ordemFornecimentoService")
public class OrdemFornecimentoServiceImpl extends GenericServiceImpl<OrdemFornecimento, Long> implements OrdemFornecimentoService {

	/** Atributo ordemFornecimentoDao. */
	private OrdemFornecimentoDao ordemFornecimentoDao;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private ContratoItemService contratoItemService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ParceiroService parceiroService;

	@Autowired
	private AnexoService anexoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param ordemFornecimentoDao
	 * @param validator
	 */
	@Autowired
	public OrdemFornecimentoServiceImpl( OrdemFornecimentoDao ordemFornecimentoDao, @Qualifier("ordemFornecimentoValidator") Validator validator) {
		this.dao = ordemFornecimentoDao;
		this.ordemFornecimentoDao = ordemFornecimentoDao;
		this.validator = validator;
	}

	/**
	 * Método responsável por salvar todas as informações informadas na tela de cadastro
	 * montar objeto da ordem de fornecimento removendo possíveis problemas de transient
	 *
	 * @author geovane.filho
	 *
	 * @param ordemFornecimento
	 * 			objeto que será salvo
	 */
	@Override
	public OrdemFornecimento save(OrdemFornecimento ordemFornecimento) {

		configurarCodigo(ordemFornecimento);

		// monta objeto ordemFornecimento com todas as suas listas e objetos transients
		this.montarObjetoOrdemFornecimento(ordemFornecimento);

		// realiza a validação do objeto com a classe OrdemFornecimentoValidator.java
		this.validarEntidade(ordemFornecimento, this.validator);

		return super.save(ordemFornecimento);
	}

	/**
	 * Método responsável por atualizar todas as informações informadas na tela de cadastro
	 * montar objeto da ordem de fornecimento removendo possíveis problemas de transient
	 *
	 * @author geovane.filho
	 *
	 * @param ordemFornecimento
	 * 			objeto que será atualizado
	 */
	@Override
	public OrdemFornecimento merge(OrdemFornecimento ordemFornecimento) {

		if (!UtilObjeto.isReferencia(ordemFornecimento.getNumOrdemFornecimento())) {
			configurarCodigo(ordemFornecimento);
		}

		// monta objeto ordemFornecimento com todas as suas listas e objetos transients
		this.montarObjetoOrdemFornecimento(ordemFornecimento);

		// realiza a validação do objeto com a classe OrdemFornecimentoValidator.java
		this.validarEntidade(ordemFornecimento, this.validator);

		return super.save(ordemFornecimento);
	}

	/**
	 * Método responsável por resolver os transients da ordem de fornecimento
	 *
	 * @author geovane.filho
	 *
	 * @param ordemFornecimento
	 */
	private void montarObjetoOrdemFornecimento(OrdemFornecimento ordemFornecimento) {

		if (UtilObjeto.isReferencia(ordemFornecimento.getDominioSituacao())) {
			ordemFornecimento.setDominioSituacao(this.dominioService.find(ordemFornecimento.getDominioSituacao().getId()));
		}

		if (UtilObjeto.isReferencia(ordemFornecimento.getContrato())) {
			ordemFornecimento.setContrato(this.contratoService.find(ordemFornecimento.getContrato().getId()));
		}

		for (OrdemFornecimentoItem ordemFornItem : ordemFornecimento.getItensBensServicos()) {
			ordemFornItem.setItemBemServico(this.contratoItemService.find(ordemFornItem.getItemBemServico().getId()));
			ordemFornItem.setOrdemFornecimento(ordemFornecimento);
		}

		// cria vinculo da ordem de fornecimento com as Observacoes
		if (!UtilColecao.isVazio(ordemFornecimento.getObservacoes())) {
			for (OrdemFornecimentoObservacao ordemFornecimentoObservacao : ordemFornecimento.getObservacoes()) {
				Long idAutor = null;
				if (UtilObjeto.isReferencia(ordemFornecimentoObservacao.getAutor())) {
					idAutor = ordemFornecimentoObservacao.getAutor().getId();
				}
				if (idAutor == null) {
					idAutor = ((Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
				}
				ordemFornecimentoObservacao.setAutor(this.usuarioService.find(idAutor));
				ordemFornecimentoObservacao.setOrdemFornecimento(ordemFornecimento);
			}
		}

		// cria vinculo do estudo tecnico com os Documentos
		if (!UtilColecao.isVazio(ordemFornecimento.getDocumentos())) {

			for (OrdemFornecimentoDocumento ordemFornecimentoDocumento : ordemFornecimento.getDocumentos()) {

				ordemFornecimentoDocumento.setDominioTipoDocumento(this.dominioService.find(ordemFornecimentoDocumento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(ordemFornecimentoDocumento.getEmitente()) && UtilObjeto.isReferencia(ordemFornecimentoDocumento.getEmitente().getId())) {
					ordemFornecimentoDocumento.setEmitente(this.parceiroService.find(ordemFornecimentoDocumento.getEmitente().getId()));
				}

				if (!UtilColecao.isVazio(ordemFornecimentoDocumento.getAnexos())){
					List<Anexo> listaAnexo = new ArrayList<Anexo>();
					for (Anexo anexo : ordemFornecimentoDocumento.getAnexos()) {
						listaAnexo.add(this.anexoService.getReference(anexo.getId()));
					}
					ordemFornecimentoDocumento.setAnexos(listaAnexo);
				}

				ordemFornecimentoDocumento.setOrdemFornecimento(ordemFornecimento);
			}
		}

	}

	@Override
	public Collection<OrdemFornecimento> findOrdemFornecimentoPorContrato(Long idContrato) {

		return this.ordemFornecimentoDao.findOrdemFornecimentoPorContrato(idContrato);
	}

	private void configurarCodigo(OrdemFornecimento entity) {
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Integer ano = entity.getDataEmissao().get(Calendar.YEAR);

		if (!UtilObjeto.isReferencia(entity.getOrganizacao())) {
			entity.setOrganizacao(this.getUsuario().getOrganizacao());
		}

		OrdemFornecimento ultimoRegistro = (OrdemFornecimento) ordemFornecimentoDao.buscarUltimoAnoRegistroComOrdenadoParametrizada("numOrdemFornecimento", ano.toString(), usuarioLogado.getOrganizacao().getId());

		Long sequencial = 0L;
		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getNumOrdemFornecimento().replace(ano.toString(), "") == null ? null : Long.parseLong(ultimoRegistro.getNumOrdemFornecimento().replace(ano.toString(), "") );
		}else{
			sequencial = 0l;
		}

		entity.setNumOrdemFornecimento(StringUtils.leftPad((Long.valueOf(sequencial + 1)).toString(), 5, "0") + ano);

	}

}
