package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.QuestionarioDao;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.PossivelRespostaQuestionarioPergunta;
import br.com.centralit.api.model.Questionario;
import br.com.centralit.api.model.QuestionarioPergunta;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.QuestionarioService;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;



@Service("questionarioService")
public class QuestionarioServiceImpl extends GenericServiceImpl<Questionario, Long> implements QuestionarioService {

	/** Atributo NOME_QUEST_PADRAO. */
	private static final String NOME_QUEST_PADRAO = "Checklist de Análise";

	private static final String MSG_CADASTRO_DUPLICADO_QUEST_ANALISE_SOLUCAO = "MSG.CADASTRO_DUPLICADO_QUEST_ANALISE_SOLUCAO";

	private static final String QUEST_1_QUEST_PADRAO = "A Solução encontra-se implantada em outro órgão ou entidade da Administração Públic Federal";
	
	private static final String QUEST_2_QUEST_PADRAO = "A Solução está disponível no Portal do Software Público Brasileiro";
	
	private static final String QUEST_3_QUEST_PADRAO = "A Solução é um Software Livre ou Software Público?";
	
	private static final String QUEST_4_QUEST_PADRAO = "A Solução é aderente às Políticas, Premissas e Especificações Técnicas definidas pelos padrões e-PING e e-MAG?";
	
	private static final String QUEST_5_QUEST_PADRAO = "A Solução é aderente às Regulamentações da ICP-Brasil?";
	
	private static final String QUEST_6_QUEST_PADRAO = "A Solução é aderente às Orientações, Premissas e Especificações Técnicas e Funcionais do e-ARQ Brasil?";

	private static final String SIM = "Sim";

	private static final String NAO = "Não";

	private static final String NAO_SE_APLICA = "Não se aplica";
	
	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo RegraEventoDao. */
	private QuestionarioDao questionarioDao;	
	
	Questionario questionario = new Questionario();


	@Autowired
	public QuestionarioServiceImpl( QuestionarioDao questionarioDao) {

		this.dao = questionarioDao;

		this.questionarioDao = questionarioDao;

	}


	@Override
	public Collection<Questionario> listarQuestionarios(String nome) {

		Collection<Questionario> listaQuestionarios = this.questionarioDao.listarQuestionarios(nome);

		return UtilColecao.isVazio(listaQuestionarios) ? new ArrayList<Questionario>() : listaQuestionarios;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Questionario save(Questionario questionario) {
		
		if (questionario.getDominioTipoQuestionario().getCodigo().equals(DominioContrato.TIPO_QUESTIONARIO_ANALISE_SOLUCAO_CODIGO)) {
			if (!findAllByDominioTipoQuestionario(DominioContrato.TIPO_QUESTIONARIO_ANALISE_SOLUCAO_CODIGO, this.getUsuario().getOrganizacao().getId()).isEmpty()) {
				lancarBusinessException(MSG_CADASTRO_DUPLICADO_QUEST_ANALISE_SOLUCAO, CodigoErro.REGRA_NEGOCIO.getValue(), true);
			}
		}

		atualizarAtributos(questionario);
		
		Questionario entitySaved = (Questionario) this.dao.save(questionario);
		
		return entitySaved;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Questionario merge(Questionario questionario) {
		
		if (questionario.getDominioTipoQuestionario().getCodigo().equals(DominioContrato.TIPO_QUESTIONARIO_ANALISE_SOLUCAO_CODIGO)) {
			Collection<Questionario> quests = this.findAllByDominioTipoQuestionario(DominioContrato.TIPO_QUESTIONARIO_ANALISE_SOLUCAO_CODIGO, this.getUsuario().getOrganizacao().getId());
			
			for (Questionario quest : quests) {
				if (!quest.getId().equals(questionario.getId())) {
					lancarBusinessException(MSG_CADASTRO_DUPLICADO_QUEST_ANALISE_SOLUCAO, CodigoErro.REGRA_NEGOCIO.getValue(), true);
				}
			}
		}

		atualizarAtributos(questionario);

		return (Questionario) this.dao.merge(questionario);
	}

	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por atualizar (merge) dos atributos do questionario
	 *
	 * @author geovane.filho
	 *
	 * @param questionario Questionario a se atualizar os atributos
	 */
	private void atualizarAtributos(Questionario questionario) {
		
		questionario.setDominioTipoQuestionario(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_QUESTIONARIO, questionario.getDominioTipoQuestionario().getCodigo()));
		
		atualizarAtributosEmPerguntas(questionario);
	}
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por atualizar (merge) dos atributos do questionario
	 *
	 * @author geovane.filho
	 *
	 * @param questionario Questionario a se atualizar os atributos das perguntas
	 */
	private void atualizarAtributosEmPerguntas(Questionario questionario) {
		
		for (QuestionarioPergunta pergunta : questionario.getQuestoes()) {
			pergunta.setQuestionario(questionario);
			pergunta.setDominioTipoResposta(this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_RESPOSTA_QUESTAO, pergunta.getDominioTipoResposta().getCodigo()));
			
			if (pergunta.getRespostasEscolha() != null) {
				for (PossivelRespostaQuestionarioPergunta possivelResposta : pergunta.getRespostasEscolha()) {
					possivelResposta.setQuestionarioPergunta(pergunta);
				}
			}
		}
		
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por trazer todos os questionarios da organizacao do usuário logado.
	 *
	 * @author geovane.filho
	 *
	 * @param organizacaoId ID da organizacao a se buscar os questionarios
	 * @return
	 */
	@Override
	public Collection<Questionario> findAllByOrganizacao(Long organizacaoId) {
		return this.questionarioDao.findAllByOrganizacao(organizacaoId);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<Questionario> findAllByDominioTipoQuestionario(Long tipoQuestCodigo, Long organizacaoId) {
		Dominio tipoQuest = this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_QUESTIONARIO, tipoQuestCodigo);
		Long idTipoQuest = null;
		if (UtilObjeto.isReferencia(tipoQuest)) {
			idTipoQuest = tipoQuest.getId();
		}
		return this.questionarioDao.findAllByDominioTipoQuestionario(idTipoQuest, organizacaoId);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Questionario createQuestionarioPadrao() {
		Collection<Questionario> quests = this.findAllByDominioTipoQuestionario(DominioContrato.TIPO_QUESTIONARIO_ANALISE_SOLUCAO_CODIGO, this.getUsuario().getOrganizacao().getId());
		if (quests.isEmpty()) {
			Dominio tipoRespostaUnicaEscolha = this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_RESPOSTA_QUESTAO, DominioContrato.TIPO_RESPOSTA_QUESTAO_UNICA_ESCOLHA_CODIGO);
			Collection<QuestionarioPergunta> questoes = new ArrayList<QuestionarioPergunta>();
			questoes.add(new QuestionarioPergunta(QUEST_1_QUEST_PADRAO, tipoRespostaUnicaEscolha, getPossiveisRespostas()));
			questoes.add(new QuestionarioPergunta(QUEST_2_QUEST_PADRAO, tipoRespostaUnicaEscolha, getPossiveisRespostas()));
			questoes.add(new QuestionarioPergunta(QUEST_3_QUEST_PADRAO, tipoRespostaUnicaEscolha, getPossiveisRespostas()));
			questoes.add(new QuestionarioPergunta(QUEST_4_QUEST_PADRAO, tipoRespostaUnicaEscolha, getPossiveisRespostas()));
			questoes.add(new QuestionarioPergunta(QUEST_5_QUEST_PADRAO, tipoRespostaUnicaEscolha, getPossiveisRespostas()));
			questoes.add(new QuestionarioPergunta(QUEST_6_QUEST_PADRAO, tipoRespostaUnicaEscolha, getPossiveisRespostas()));
			
			Questionario quest = new Questionario(NOME_QUEST_PADRAO, this.dominioService.findByChaveAndCodigo(DominioContrato.TIPO_QUESTIONARIO, DominioContrato.TIPO_QUESTIONARIO_ANALISE_SOLUCAO_CODIGO), questoes);
			return this.save(quest);
		} else {
			return quests.iterator().next();
		}		
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por criar as possiveis respostas padrão do questionario padrão
	 *
	 * @author geovane.filho
	 *
	 * @return
	 */
	private Collection<PossivelRespostaQuestionarioPergunta> getPossiveisRespostas() {
		Collection<PossivelRespostaQuestionarioPergunta> respostas = new ArrayList<PossivelRespostaQuestionarioPergunta>();
		respostas.add(new PossivelRespostaQuestionarioPergunta(SIM));
		respostas.add(new PossivelRespostaQuestionarioPergunta(NAO));
		respostas.add(new PossivelRespostaQuestionarioPergunta(NAO_SE_APLICA));
		return respostas;
	}
}
