package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Connection;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.RegistroOcorrenciaDao;
import br.com.centralit.api.model.RegistroOcorrencia;
import br.com.centralit.api.model.RegistroOcorrenciaDocumento;
import br.com.centralit.api.model.RegistroOcorrenciaObservacao;
import br.com.centralit.api.model.RegistroSequencial;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.OrdemFornecimentoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.RegistroOcorrenciaService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * @since 15/02/2016 - 15:24:52
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */
@Service("registroOcorrenciaService")
public class RegistroOcorrenciaServiceImpl extends GenericServiceImpl<RegistroOcorrencia, Long> implements RegistroOcorrenciaService {

	private static final Logger LOG = Logger.getLogger(RegistroOcorrenciaServiceImpl.class);

	@Autowired
	private RegistroOcorrenciaDao registroOcorrenciaDao;

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private OrdemFornecimentoService ordemFornecimentoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private UsuarioService usuarioService;

	//TODO - verificar a necessidade deste atributo para anexo de documentos
	@Autowired
	private ParceiroService parceiroService;

	@Autowired
	private ReportController reportService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private HttpServletRequest request;

	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	private Dominio idioma;

	@Autowired
	public RegistroOcorrenciaServiceImpl( RegistroOcorrenciaDao registroOcorrenciaDao,
			@Qualifier("registroOcorrenciaValidator") Validator validator ) {

		this.dao = registroOcorrenciaDao;
		this.registroOcorrenciaDao = registroOcorrenciaDao;
		this.validator = validator;
	}

	@Override
	public RegistroOcorrencia save(RegistroOcorrencia registroOcorrencia) {

		this.validarEntidade(registroOcorrencia, this.validator);

		montarObjetoRegistroOcorrencia(registroOcorrencia);

		return super.save(registroOcorrencia);
	}

	@Override
	public RegistroOcorrencia merge (RegistroOcorrencia registroOcorrencia){

		this.validarEntidade(registroOcorrencia, this.validator);

		montarObjetoRegistroOcorrencia(registroOcorrencia);

		return super.merge(registroOcorrencia);
	}
	/**
	 *
	 * M&eacute;todo respons&aacute;vel por
	 *
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 *
	 * @param registroOcorrencia
	 */
	private void montarObjetoRegistroOcorrencia(RegistroOcorrencia registroOcorrencia) {

		if (registroOcorrencia.getOrganizacao() == null) {
			registroOcorrencia.setOrganizacao(this.getUsuario().getOrganizacao());
		}

		if(registroOcorrencia.getId() == null){

			gerarNumeroSequencial(registroOcorrencia);
		}

		registroOcorrencia.setContrato(this.contratoService.getReference(registroOcorrencia.getContrato().getId()));

		if (registroOcorrencia.getOrdemFornecimento() != null && registroOcorrencia.getOrdemFornecimento().getId() != null){
			registroOcorrencia.setOrdemFornecimento(this.ordemFornecimentoService.getReference(registroOcorrencia.getOrdemFornecimento().getId()));
		} else {
			registroOcorrencia.setOrdemFornecimento(null);
		}

		if (!registroOcorrencia.getEmitirGlosa()){
			registroOcorrencia.setJustificativaGlosa(null);
		}

		// Observacoes
		if (CollectionUtils.isNotEmpty(registroOcorrencia.getObservacoes())) {
			Collection<RegistroOcorrenciaObservacao> observacoes = registroOcorrencia.getObservacoes();

			for (RegistroOcorrenciaObservacao observacao : observacoes) {
				Long idAutor = null;
				if (observacao.getAutor() != null) {
					idAutor = observacao.getAutor().getId();
					observacao.setAutor(this.usuarioService.find(idAutor));
				}
				observacao.setRegistroOcorrencia(registroOcorrencia);
			}
			registroOcorrencia.setObservacoes(observacoes);
		}

		// Documentos
		if (CollectionUtils.isNotEmpty(registroOcorrencia.getDocumentos())) {
			Collection<RegistroOcorrenciaDocumento> documentos = registroOcorrencia.getDocumentos();

			for (RegistroOcorrenciaDocumento documento : documentos) {
				documento.setRegistroOcorrencia(registroOcorrencia);
				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (documento.getEmitente() != null && documento.getEmitente().getId() != null) {
					documento.setEmitente(this.parceiroService.getReference(documento.getEmitente().getId()));
				}
			}
			registroOcorrencia.setDocumentos(documentos);
		}
	}

	/**
	 *
	 * M&eacute;todo respons&aacute;vel por gerar o n&uacute;mero sequencial identificador e &uacute;nico para cada registro
	 * de ocorr&ecirc;ncia. Esta sequencia &eacute; reiniciada a cada ano. Por exemplo: o n&uacute;mero 1/2016 at&eacute; 999/2016, no
	 * pr&oacute;ximo ano a sequencia inicia-se novamente 1/2017 at&eacute; 999/2017
	 *
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 * @param registroOcorrencia
	 */
	private void gerarNumeroSequencial(RegistroOcorrencia entity) {

		// busca todos os registros dentro do ano de emissao da ocorrencia e ordena decrescente pelo numeroSequencial.
		RegistroOcorrencia ultimoRegistro = (RegistroOcorrencia) registroOcorrenciaDao.buscarUltimoRegistroPorDataEmissao("registroSequencial.numeroSequencial",
				entity.getDataEmissaoOcorrencia(), entity.getOrganizacao().getId());

		Long sequencial = null;
		if (ultimoRegistro != null) {
			sequencial = ultimoRegistro.getRegistroSequencial().getNumeroSequencial() == null ? null : ultimoRegistro.getRegistroSequencial().getNumeroSequencial();
		}
		if (sequencial != null){
			sequencial++;
		} else {
			sequencial = 1l;
		}

		String numeroAnoOcorrencia = StringUtils.leftPad((Long.valueOf(sequencial.toString())).toString(), 5, "0") + entity.getDataEmissaoOcorrencia().get(Calendar.YEAR);
		RegistroSequencial registroSequencial = new RegistroSequencial(sequencial, numeroAnoOcorrencia);
		entity.setRegistroOcorrenciaSequencial(registroSequencial);
	}

	@Override
	public void gerarPdfRegistroOcorrencia(Long registroOcorrenciaId, boolean download, HttpServletResponse response) {

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;

		try {
			conn = reportService.getConnection();
			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());

			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/registroOcorrencia.jrxml");
			String titulo = this.internacionalizacaoService.getTranslate("LABEL.TITULO_REGISTRO_OCORRENCIA", this.getIdioma());

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("registroOcorrenciaId", registroOcorrenciaId);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			JasperPrint jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);

			// Responsavel por verificar a presenca de dados no relatorio
			reportService.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);

		} catch (JRException jre) {
			LOG.info("N\u00e3o consegui gerar o relat\u00f3rio de registro de ocorr\u00eancias. Veja a exce\u00e7\u00e3o para identificar a causa:", jre.getCause());
			throw new RuntimeException(jre);
		} catch (Exception e) {
			throw new BusinessException();
		} finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}

	public Dominio getIdioma() {
		if(!UtilObjeto.isReferencia(idioma)){
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public Collection<RegistroOcorrencia> findAllPorContrato(Long idContrato) {
		return this.registroOcorrenciaDao.findAllPorContrato(idContrato);
	}

	@Override
	public BigDecimal getValorTotalGlosaOrdem(Long ordemFornecimentoId) {
		return this.registroOcorrenciaDao.getValorTotalGlosaOrdem(ordemFornecimentoId);
	}

}
