package br.com.centralit.api.service.impl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.TermoReferenciaDao;
import br.com.centralit.api.model.TermoReferencia;
import br.com.centralit.api.model.TermoReferenciaDetalhamentoSolucao;
import br.com.centralit.api.model.TermoReferenciaDocumento;
import br.com.centralit.api.model.TermoReferenciaObservacao;
import br.com.centralit.api.model.TermoReferenciaPenalidade;
import br.com.centralit.api.model.TermoReferenciaSancao;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.RascunhoService;
import br.com.centralit.api.service.TermoReferenciaDetalhamentoSolucaoService;
import br.com.centralit.api.service.TermoReferenciaPenalidadeService;
import br.com.centralit.api.service.TermoReferenciaSancaoService;
import br.com.centralit.api.service.TermoReferenciaService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDataBase;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 * Termo Referencia
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">2359</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 06/01/2016 - 10:36:39
 *
 * @version 1.0.0
 *
 * @author juliana.barbosa
 *
 */
@Service("termoReferenciaService")
public class TermoReferenciaServiceImpl extends GenericServiceImpl<TermoReferencia, Long> implements TermoReferenciaService {

	public static final Logger LOG = Logger.getLogger(TermoReferenciaServiceImpl.class);

	@Autowired
	private TermoReferenciaDao termoReferenciaDao;

	@Autowired
	private ParceiroService parceiroService;

	@Autowired
	private DominioService dominioService;

	//TODO - Ciro 21/01/2016- Substituir esta dependencia por chamada rest
	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	@Autowired
	private TermoReferenciaPenalidadeService penalidadeService;

	@Autowired
	private TermoReferenciaDetalhamentoSolucaoService detalhamentoSolucaoService;

	@Autowired
	private TermoReferenciaSancaoService sancaoService;

	@Autowired
	private ReportController reportService;

	//TODO - Ciro 21/01/2016- Substituir esta dependencia por chamada rest
	@Autowired
	private InternacionalizacaoService internacionalizacaoService;
	
	@Autowired
	private ContratoService contratoService;

	@Autowired
	private RascunhoService rascunhoService;

	private Dominio idioma;

	@Autowired
	public TermoReferenciaServiceImpl( TermoReferenciaDao termoReferenciaDao, @Qualifier("termoReferenciaValidator") Validator validator ) {

		this.dao = termoReferenciaDao;
		this.termoReferenciaDao = termoReferenciaDao;
		this.validator = validator;
	}

	@Override
	public TermoReferencia save(TermoReferencia termoReferencia) {

		this.validarEntidade(termoReferencia, this.validator);

		montarObjetoTermoReferencia(termoReferencia);

		termoReferencia = (TermoReferencia) termoReferenciaDao.save(termoReferencia);

		configurarCodigo(termoReferencia);
		
		rascunhoService.removeById(termoReferencia.getRascunho());

		return termoReferencia;
	}

	@Override
	public TermoReferencia merge(TermoReferencia termo) {
		this.validarEntidade(termo, this.validator);
		montarObjetoTermoReferencia(termo);

		return (TermoReferencia) this.termoReferenciaDao.merge(termo);
	}

	@Override
	public boolean removeById(Long id) {
		validarExclusao(id);
		
		TermoReferencia termoReferencia = find(id);
		if (CollectionUtils.isNotEmpty(termoReferencia.getPenalidades())){
			penalidadeService.removeList((List<TermoReferenciaPenalidade>) termoReferencia.getPenalidades());
		}
		if(CollectionUtils.isNotEmpty(termoReferencia.getDetalhamentosSolucoes())){
			detalhamentoSolucaoService.removeList((List<TermoReferenciaDetalhamentoSolucao>) termoReferencia.getDetalhamentosSolucoes());
		}
		if(CollectionUtils.isNotEmpty(termoReferencia.getSancoes())){
			sancaoService.removeList((List<TermoReferenciaSancao>) termoReferencia.getSancoes());
		}
		return termoReferenciaDao.removeById(id);
	}
	
	private void validarExclusao(Long id) {
		Boolean isVinculado = contratoService.existeVinculo("termoReferencia.id", id);
		lancarBusinessException("VALIDACAO.REMOCAO_COM_VINCULO", CodigoErro.REGRA_NEGOCIO.getValue(), isVinculado);
	}

	private void montarObjetoTermoReferencia(TermoReferencia termoReferencia) {

		// TermoReferenciaDetalhamentoSolucao
		if (UtilObjeto.isReferencia(termoReferencia.getDetalhamentosSolucoes())) {
			List<TermoReferenciaDetalhamentoSolucao> listaDetalhamentoSolucao = new ArrayList<>();
			for (TermoReferenciaDetalhamentoSolucao detalhamentoSolucao : termoReferencia.getDetalhamentosSolucoes()) {
				montarObjetoDetalhamentoSolucao(detalhamentoSolucao);
				detalhamentoSolucao.setTermo(termoReferencia);
				listaDetalhamentoSolucao.add(montarObjetoDetalhamentoSolucao(detalhamentoSolucao));
			}
			termoReferencia.setDetalhamentosSolucoes(listaDetalhamentoSolucao);
		}

		// Dominios
		termoReferencia.setAdjudicacaoObjeto(this.dominioService.find(termoReferencia.getAdjudicacaoObjeto().getId()));
		termoReferencia.setEmpreitada(this.dominioService.find(termoReferencia.getEmpreitada().getId()));
		termoReferencia.setModalidade(this.dominioService.find(termoReferencia.getModalidade().getId()));

		//Penalidade
		if(CollectionUtils.isNotEmpty(termoReferencia.getPenalidades())) {
			Collection<TermoReferenciaPenalidade> penalidades = termoReferencia.getPenalidades();
			for(TermoReferenciaPenalidade penalidade : penalidades) {
				penalidade.setTermo(termoReferencia);
			}
			termoReferencia.setPenalidades(penalidades);
		}

		//Sansoes
		if(CollectionUtils.isNotEmpty(termoReferencia.getSancoes())) {
			Collection<TermoReferenciaSancao> sancoes = termoReferencia.getSancoes();
			for(TermoReferenciaSancao sancao : sancoes) {
				sancao.setTermo(termoReferencia);
			}
			termoReferencia.setSancoes(sancoes);
		}

		//Observacoes
		if(CollectionUtils.isNotEmpty(termoReferencia.getObservacoes())) {
			Collection<TermoReferenciaObservacao> observacoes = termoReferencia.getObservacoes();

			for(TermoReferenciaObservacao observacao : observacoes) {
				Long idAutor = null;
				if (observacao.getAutor() != null) {
					idAutor = observacao.getAutor().getId();
					observacao.setAutor(this.usuarioService.find(idAutor));
				}
				observacao.setTermo(termoReferencia);
			}
			termoReferencia.setObservacoes(observacoes);
		}

		//Documentos
		if(CollectionUtils.isNotEmpty(termoReferencia.getDocumentos())) {
			Collection<TermoReferenciaDocumento> documentos = termoReferencia.getDocumentos();

			for(TermoReferenciaDocumento documento : documentos) {
				documento.setTermo(termoReferencia);
				documento.setDominioTipoDocumento(this.dominioService.find(documento.getDominioTipoDocumento().getId()));

				if (UtilObjeto.isReferencia(documento.getEmitente()) && UtilObjeto.isReferencia(documento.getEmitente().getId())) {

					documento.setEmitente(this.parceiroService.find(documento.getEmitente().getId()));
				}
			}
			termoReferencia.setDocumentos(documentos);
		}
	}

	private TermoReferenciaDetalhamentoSolucao montarObjetoDetalhamentoSolucao(TermoReferenciaDetalhamentoSolucao detalhe) {

		if (detalhe.getFornecedor() != null) {
			detalhe.setFornecedor(this.parceiroService.find(detalhe.getFornecedor().getId()));
		}

		if (detalhe.getOrgaoExterno() != null) {
			detalhe.setOrgaoExterno(this.parceiroService.find(detalhe.getOrgaoExterno().getId()));
		}

		detalhe.setFormaPagamento(this.dominioService.find(detalhe.getFormaPagamento().getId()));
		return detalhe;
	}


	private void configurarCodigo(TermoReferencia entity) {

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		if (!UtilObjeto.isReferencia(entity.getOrganizacao())) {
			entity.setOrganizacao(this.getUsuario().getOrganizacao());
		}

		Integer ano = Calendar.getInstance().get(Calendar.YEAR);

		TermoReferencia ultimoRegistro =
				(TermoReferencia) termoReferenciaDao.buscarUltimoAnoRegistroComOrdenadoParametrizada("codigo", ano.toString(), usuarioLogado.getOrganizacao().getId());

		Long sequencial = null;
		if (ultimoRegistro != null) {
			sequencial = Long.valueOf(ultimoRegistro.getCodigo().split(ano.toString())[0]);
		}
		if (sequencial != null){
			sequencial++;
		} else {
			sequencial = 1l;
		}
		String codigo = StringUtils.leftPad(sequencial.toString() + ano, 9, "0");

		entity.setCodigo(codigo);
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 *
	 */
	public void gerarPdfTermoReferencia (Long termoReferenciaId, boolean download, HttpServletResponse response) {

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;

		TermoReferencia termoReferencia = this.find(termoReferenciaId);

		try {
			conn = reportService.getConnection();
			Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication()
					.getPrincipal() ).getId());

			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/termoReferencia.jrxml");
			String titulo = this.internacionalizacaoService.getTranslate("LABEL.TITULO_TERMO_REFERENCIA", this.getIdioma());

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("termoReferenciaId", termoReferenciaId);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", usuarioLogado.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			JasperPrint jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);

			// Responsavel por verificar a presenca de dados no relatorio
			reportService.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);

			termoReferencia.setDataTermoGerado(Calendar.getInstance());
			termoReferenciaDao.merge(termoReferencia);

		} catch (JRException jre) {
			LOG.info("N\u00e3o consegui gerar o relat\u00f3rio do termo de refer\u00eancia. Veja a exce\u00e7\u00e3o para identificar a causa:", jre.getCause());
			throw new RuntimeException(jre);
		} catch (Exception e) {
			throw new BusinessException();
		} finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}
	}

	public Dominio getIdioma() {
		if(!UtilObjeto.isReferencia(idioma)){
			idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return idioma;
	}

	@Override
	public TermoReferencia findTermoByEstudoTecnico(long estudoTecnicoId) {
		return this.termoReferenciaDao.findTermoByEstudoTecnico(estudoTecnicoId);
	}

	@Override
	public Collection<TermoReferencia> findAutoCompleteTermoReferencia(String value, Long idOrganizacao) {
		return this.termoReferenciaDao.findAutoCompleteTermoReferencia(value, idOrganizacao);
	}

}
