package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.Contrato;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.util.UtilColecao;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 25/01/2016 - 15:23:44
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Component("contratoValidator")
public class ContratoValidator implements Validator {

	/** Atributo CODIGO_DOMINIO_FORMA_PAGAMENTO_CONTRATO_PARCELADO. */
	private static final Long CODIGO_DOMINIO_FORMA_PAGAMENTO_CONTRATO_PARCELADO = 2L;

	/** Atributo CODIGO_DOMINIO_MODALIDADE_LICITACAO_PREGAO. */
	private static final Long CODIGO_DOMINIO_MODALIDADE_LICITACAO_PREGAO = 1L;

	/** Atributo CODIGO_DOMINIO_MODALIDADE_LICITACAO_CONCURSO. */
	private static final Long CODIGO_DOMINIO_MODALIDADE_LICITACAO_CONCURSO = 2L;

    @Override
	public boolean supports(Class<?> clazz) {

		return Contrato.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		Contrato contrato = (Contrato) target;

		ValidationUtils.rejectIfEmpty(errors, "numeroProcesso", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.NUMERO_PROCESSO");

		ValidationUtils.rejectIfEmpty(errors, "exercicio", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.EXERCICIO");

		ValidationUtils.rejectIfEmpty(errors, "numeroAnoContrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.NUMERO_ANO_CONTRATO");

        ValidationUtils.rejectIfEmpty(errors, "dominioTipoContrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.TIPO_CONTRATO");

        ValidationUtils.rejectIfEmpty(errors, "dominioTipoAquisicao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.TIPO_AQUISICAO");

        ValidationUtils.rejectIfEmpty(errors, "dataCelebracao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_CELEBRACAO");

        ValidationUtils.rejectIfEmpty(errors, "dataVigenciaInicial", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_VIGENCIA_INICIAL");

        ValidationUtils.rejectIfEmpty(errors, "dataVigenciaFinal", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_VIGENCIA_FINAL");

        ValidationUtils.rejectIfEmpty(errors, "dominioSituacaoAtual", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.SITUACAO_ATUAL");

        ValidationUtils.rejectIfEmpty(errors, "objetoContratacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.OBJETO_CONTRATACAO");

        ValidationUtils.rejectIfEmpty(errors, "valorContrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.VALOR_CONTRATO");

        ValidationUtils.rejectIfEmpty(errors, "dominioFormaPagamentoContrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.FORMA_PAGAMENTO_CONTRATO");


        if (contrato.getDominioFormaPagamentoContrato().getCodigo() == CODIGO_DOMINIO_FORMA_PAGAMENTO_CONTRATO_PARCELADO){

        	if (UtilColecao.isVazio(contrato.getCtrtValoresPagamento())){

    			errors.rejectValue("ctrtValoresPagamento", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.VALORES_PAGAMENTO");
    		}
        }

		if (UtilColecao.isVazio(contrato.getCtrtEmpenhos())){

			errors.rejectValue("ctrtEmpenhos", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.INFORMACOES_EMPENHO");
		}

		ValidationUtils.rejectIfEmpty(errors, "numeroAnoEditalLicitacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.NUMERO_ANO_EDITAL");

		ValidationUtils.rejectIfEmpty(errors, "dominioModalidadeLicitacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.MODALIDADE_LICITACAO");

		if (contrato.getDominioModalidadeLicitacao().getCodigo() != CODIGO_DOMINIO_MODALIDADE_LICITACAO_PREGAO && contrato.getDominioModalidadeLicitacao().getCodigo() != CODIGO_DOMINIO_MODALIDADE_LICITACAO_CONCURSO){

			ValidationUtils.rejectIfEmpty(errors, "dominioTipoLicitacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.TIPO_LICITACAO");
		}

		if (UtilColecao.isVazio(contrato.getCtrtItens())){

			errors.rejectValue("ctrtItens", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.INFORMACOES_ITENS_CONTRATO");
		}


		if (UtilColecao.isVazio(contrato.getCtrtEnvolvidos())){

			errors.rejectValue("ctrtEnvolvidos", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.INFORMACOES_ENVOLVIDOS");
		}
		
		if (contrato.getDominioModalidadeGarantia() != null || contrato.getValorGarantia() != null) {
			ValidationUtils.rejectIfEmpty(errors, "dominioModalidadeGarantia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.MODALIDADE_GARANTIA");
			ValidationUtils.rejectIfEmpty(errors, "valorGarantia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.VALOR_GARANTIA");
		}
	}
}

