package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EstudoTecnico;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 05/01/2016 - 17:28:25
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Component("estudoTecnicoValidator")
public class EstudoTecnicoValidator implements Validator{

	@Override
	public boolean supports(Class<?> clazz) {

		return EstudoTecnicoValidator.class.isAssignableFrom(clazz);
	}

	/**
	 * Método responsável por validacao dos campos obrigatorios do estudoTecnico
	 * bem como tambem campos finhos desta entidade
	 *
	 * @author iago.almeida
	 *
	 * @param target
	 * @param errors
	 */
	@Override
	public void validate(Object target, Errors errors) {

		EstudoTecnico estudoTecnico = (EstudoTecnico) target;

		ValidationUtils.rejectIfEmpty(errors, "numeroPortaria", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.NUMERO_PORTARIA");

		ValidationUtils.rejectIfEmpty(errors, "dataInstituicaoEquipe", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_INSTITUICAO_EQUIPE");

		if (!UtilObjeto.isReferencia(estudoTecnico.getIntegranteTecnico()) || !UtilObjeto.isReferencia(estudoTecnico.getIntegranteTecnico().getId())){

			errors.rejectValue("integranteTecnico", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.INTEGRANTE_TECNICO");
		}

		if (!UtilObjeto.isReferencia(estudoTecnico.getIntegranteRequisitante()) || !UtilObjeto.isReferencia(estudoTecnico.getIntegranteRequisitante().getId())){

			errors.rejectValue("integranteRequisitante", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.INTEGRANTE_REQUISITANTE");
		}

		if (!UtilObjeto.isReferencia(estudoTecnico.getIntegranteAdministrativo()) || !UtilObjeto.isReferencia(estudoTecnico.getIntegranteAdministrativo().getId())){

			errors.rejectValue("integranteAdministrativo", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.INTEGRANTE_ADMINISTRATIVO");
		}

		ValidationUtils.rejectIfEmpty(errors, "descricaoSolucao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DESCRICAO_SOLUCAO");

		if (UtilColecao.isVazio(estudoTecnico.getEtRequisitosNegocio())){

			errors.rejectValue("etRequisitosNegocio", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.REQUISITOS_NEGOCIO");

		}

		ValidationUtils.rejectIfEmpty(errors, "requisitosTecnicos", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.REQUISITOS_TECNICOS");

		if (UtilColecao.isVazio(estudoTecnico.getEtLevantamentoAlternativas())){

			errors.rejectValue("etLevantamentoAlternativas", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.SOLUCAO");

		}

		ValidationUtils.rejectIfEmpty(errors, "descricaoJustificativaSolucao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DESCRICAO_JUSTIFICATIVA_SOLUCAO");

		if (UtilColecao.isVazio(estudoTecnico.getEtBensServicosSolucoes())){

			errors.rejectValue("etBensServicosSolucoes", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DESCRICAO_BENS_SERVICO");

		}

		if (UtilColecao.isVazio(estudoTecnico.getEtBeneficiosEsperados())){

			errors.rejectValue("etBeneficiosEsperados", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.BENEFICIOS_ESPERADOS");

		}

		if (UtilColecao.isVazio(estudoTecnico.getEtAcoesContingencia())){

			errors.rejectValue("etAcoesContingencia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.ACOES_CONTINGENCIA");

		}
	}
}
