package br.com.centralit.api.service.validation;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Calendar;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.model.EventoAditivo;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 16/03/2016 - 11:10:02
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Component("eventoAditivoValidator")
public class EventoAditivoValidator implements Validator{

	/** Atributo PERCENTAGEM_MAXIMA_REFORMA_IMOVEIS. */
	private static final BigDecimal PERCENTAGEM_MAXIMA_REFORMA_IMOVEIS = new BigDecimal(0.5);
	/** Atributo PERCENTAGEM_MAXIMA_OBRAS_SERVICOS_COMPRAS. */
	private static final BigDecimal PERCENTAGEM_MAXIMA_OBRAS_SERVICOS_COMPRAS = new BigDecimal(0.25);
	/** Atributo MAXIMO_ADITIVO_PRAZO. */
	private static final int MAXIMO_ADITIVO_PRAZO = 5;

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoAditivoValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoAditivo aditivo = (EventoAditivo) target;
		Calendar dataFinal = null;
		if(UtilObjeto.isReferencia(aditivo.getContrato())){
			dataFinal = aditivo.getContrato().getDataVigenciaFinal();
		}
		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");

		if (UtilObjeto.isReferencia(aditivo.getContrato()) && !UtilObjeto.isReferencia(aditivo.getId())) {
			dataFinal = aditivo.getContrato().getDataUltimaVigenciaFinal();
		}else{
			if (UtilObjeto.isReferencia(aditivo.getContrato().getAditivos())) {
				for (EventoAditivo adContrato : aditivo.getContrato().getAditivos()) {
					if( adContrato.getAditivoDePrazo() && adContrato.getDataFim().before(aditivo.getDataInicio()) && adContrato.getDataFim().after(dataFinal)){
						dataFinal = adContrato.getDataFim();
					}
				}
			}
		}

		if(aditivo.getDataEmissao().after(aditivo.getDataInicio())){
			errors.rejectValue("dataEmissao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.ADITIVO_DATA_EMISSAO_MAIOR_DATA_INICIO");
		}

		ValidationUtils.rejectIfEmpty(errors, "registroSequencial", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.NUM_ADITIVO");

		ValidationUtils.rejectIfEmpty(errors, "dataEmissao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_EMISSAO_OCORRENCIA");

		if (UtilObjeto.isReferencia(aditivo.getDataEmissao()) && UtilObjeto.isReferencia(aditivo.getContrato()) &&
				(aditivo.getDataEmissao().before(aditivo.getContrato().getDataVigenciaInicial()) || aditivo.getDataEmissao().after(dataFinal))) {
			errors.rejectValue("dataEmissao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.DATA_EMISSAO_DENTRO_REFERENCIA");
		}

		ValidationUtils.rejectIfEmpty(errors, "tipoAlteracao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.TIPO_ALTERACAO");

		if (!(aditivo.getAditivoDeValor() || aditivo.getAditivoDePrazo() || aditivo.getAditivoDeClausula())) {
			errors.rejectValue("aditivoDePrazo", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.TIPO_ADITIVO");
			errors.rejectValue("aditivoDeValor", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.TIPO_ADITIVO");
			errors.rejectValue("aditivoDeClausula", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.TIPO_ADITIVO");
		}

		if (aditivo.getAditivoDePrazo()) {
			if (UtilObjeto.isReferencia(aditivo.getDataEmissao())) {
				Calendar ultimaDataEmissaoAditivoAnterior = null;
				Calendar ultimaDataEmissaoAditivoPosterior = null;
				int qntAditivoPrazo = 0;
				if (UtilObjeto.isReferencia(aditivo.getContrato().getAditivos())) {
					for (EventoAditivo adContrato : aditivo.getContrato().getAditivos()) {
						if (((!UtilObjeto.isReferencia(aditivo.getId())) || (UtilObjeto.isReferencia(aditivo.getId()) && !adContrato.getId().equals(aditivo.getId()))) && adContrato.getAditivoDePrazo()) {
							qntAditivoPrazo++;
							if (adContrato.getDataEmissao().before(aditivo.getDataEmissao())) {
								ultimaDataEmissaoAditivoAnterior = adContrato.getDataEmissao();
							}
							if(adContrato.getDataEmissao().after(aditivo.getDataEmissao())){
								ultimaDataEmissaoAditivoPosterior = adContrato.getDataEmissao();
							}
						}
					}
				}

				if (qntAditivoPrazo >= MAXIMO_ADITIVO_PRAZO) {
					errors.rejectValue("aditivoDePrazo", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.MAXIMO_ADITIVO_PRAZO");
				}

				if (UtilObjeto.isReferencia(ultimaDataEmissaoAditivoAnterior)) {

					if (ultimaDataEmissaoAditivoAnterior.get(Calendar.YEAR) < aditivo.getDataEmissao().get(Calendar.YEAR)) {
						if ( ultimaDataEmissaoAditivoAnterior.get(Calendar.MONTH) <= aditivo.getDataEmissao().get(Calendar.MONTH)) {
							if ((ultimaDataEmissaoAditivoAnterior.get(Calendar.MONTH) == aditivo.getDataEmissao().get(Calendar.MONTH)) &&
									ultimaDataEmissaoAditivoAnterior.get(Calendar.DAY_OF_MONTH) > aditivo.getDataEmissao().get(Calendar.DAY_OF_MONTH)) {
								errors.rejectValue("dataEmissao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.DATA_EMISSAO_12_MESES");
							}
						}
					} else {
						errors.rejectValue("dataEmissao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.DATA_EMISSAO_12_MESES");
					}
				}
				if (UtilObjeto.isReferencia(ultimaDataEmissaoAditivoPosterior)) {
					if (ultimaDataEmissaoAditivoPosterior.get(Calendar.YEAR) > aditivo.getDataEmissao().get(Calendar.YEAR)) {
						if (ultimaDataEmissaoAditivoPosterior.get(Calendar.MONTH) >= aditivo.getDataEmissao().get(Calendar.MONTH)) {
							if ((ultimaDataEmissaoAditivoPosterior.get(Calendar.MONTH) == aditivo.getDataEmissao().get(Calendar.MONTH)) &&
									ultimaDataEmissaoAditivoPosterior.get(Calendar.DAY_OF_MONTH) < aditivo.getDataEmissao().get(Calendar.DAY_OF_MONTH)) {
								errors.rejectValue("dataEmissao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.DATA_EMISSAO_12_MESES_POSTERIOR");
							}
						}
					} else {
						errors.rejectValue("dataEmissao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.DATA_EMISSAO_12_MESES_POSTERIOR");
					}
				}
			}

			ValidationUtils.rejectIfEmpty(errors, "dataInicio", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.DATA_INICIO_ADITIVO");
			ValidationUtils.rejectIfEmpty(errors, "dataFim", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.DATA_FINAL_ADITIVO");

			if (UtilObjeto.isReferencia(aditivo.getDataInicio()) && dataFinal.compareTo(aditivo.getDataInicio()) >= 0){
				errors.rejectValue("dataInicio", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.DATA_INICIO_ADITIVO_MAIOR_DATA_FINAL");
			}

			if (UtilObjeto.isReferencia(aditivo.getDataInicio()) && UtilObjeto.isReferencia(aditivo.getDataFim()) && aditivo.getDataInicio().compareTo(aditivo.getDataFim()) >= 0) {
				errors.rejectValue("dataInicio", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.DATA_INICIO_ADITIVO_MAIOR_DATA_FIM_ADITIVO");
			}
		}

		if (aditivo.getAditivoDeValor()) {
			ValidationUtils.rejectIfEmpty(errors, "tipoAlteracaoValor", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.TIPO_ALTERACAO");
			ValidationUtils.rejectIfEmpty(errors, "tipoObjeto", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.TIPO_OBJETO");
			ValidationUtils.rejectIfEmpty(errors, "valorAditivo", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.VALOR_ADITIVO");
			ValidationUtils.rejectIfEmpty(errors, "percentualAditivo", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.PERCENTUAL_ADITIVO");

			Calendar dataInicioPrazo = null;
			Calendar dataFimPrazo = null;

			if (UtilObjeto.isReferencia(aditivo.getContrato())) {
				if (dataDentroPrazo(aditivo.getDataEmissao(), aditivo.getContrato().getDataVigenciaInicial(), aditivo.getContrato().getDataVigenciaFinal())) {
					dataInicioPrazo = aditivo.getContrato().getDataVigenciaInicial();
					dataFimPrazo = aditivo.getContrato().getDataVigenciaFinal();
				} else {
					if (UtilObjeto.isReferencia(aditivo.getContrato().getAditivos())) {
						for (EventoAditivo adContrato : aditivo.getContrato().getAditivos()) {
							if (((!UtilObjeto.isReferencia(aditivo.getId())) || (UtilObjeto.isReferencia(aditivo.getId()) && !adContrato.getId().equals(aditivo.getId()))) && adContrato.getAditivoDePrazo() &&
									dataDentroPrazo(aditivo.getDataEmissao(), adContrato.getDataInicio(), adContrato.getDataFim())) {
								dataInicioPrazo = adContrato.getDataInicio();
								dataFimPrazo = adContrato.getDataFim();
							}
						}
					}
				}
			}

			if (UtilObjeto.isReferencia(dataInicioPrazo) && UtilObjeto.isReferencia(dataFimPrazo)) {
				BigDecimal percentualTotalAditivo = BigDecimal.ZERO;

				if (aditivo.getTipoAlteracaoValor().getCodigo().equals(DominioContrato.CODIGO_TIPO_ALTERACAO_VALOR_ACRESCIMO)) {
					percentualTotalAditivo = percentualTotalAditivo.add(aditivo.getPercentualAditivo());
				} else if (aditivo.getTipoAlteracaoValor().getCodigo().equals(DominioContrato.CODIGO_TIPO_ALTERACAO_VALOR_SUPRESSAO)) {
					percentualTotalAditivo = percentualTotalAditivo.subtract(aditivo.getPercentualAditivo());
				}

				if (UtilObjeto.isReferencia(aditivo.getContrato().getAditivos())) {
					for (EventoAditivo adContrato : aditivo.getContrato().getAditivos()) {
						if (((!UtilObjeto.isReferencia(aditivo.getId())) || (UtilObjeto.isReferencia(aditivo.getId()) && !adContrato.getId().equals(aditivo.getId()))) && adContrato.getAditivoDeValor() && dataDentroPrazo(adContrato.getDataEmissao(), dataInicioPrazo, dataFimPrazo) &&
								adContrato.getTipoObjeto().getCodigo().equals(aditivo.getTipoObjeto().getCodigo())) {
							if (adContrato.getTipoAlteracaoValor().getCodigo().equals(DominioContrato.CODIGO_TIPO_ALTERACAO_VALOR_ACRESCIMO)) {
								percentualTotalAditivo = percentualTotalAditivo.add(adContrato.getPercentualAditivo());
							} else if (adContrato.getTipoAlteracaoValor().getCodigo().equals(DominioContrato.CODIGO_TIPO_ALTERACAO_VALOR_SUPRESSAO)) {
								percentualTotalAditivo = percentualTotalAditivo.subtract(adContrato.getPercentualAditivo());
							}
						}
					}
				}

				if (aditivo.getTipoObjeto().getCodigo().equals(DominioContrato.CODIGO_TIPO_OBJETO_OBRAS_SERVICOS_COMPRAS)) {
					if (percentualTotalAditivo.abs().compareTo(PERCENTAGEM_MAXIMA_OBRAS_SERVICOS_COMPRAS) > 0) {
						errors.rejectValue("percentualAditivo", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.PERCENTAGEM_MAIOR_OBRAS_SERVICOS_COMPRAS");
					}
				} else if (aditivo.getTipoObjeto().getCodigo().equals(DominioContrato.CODIGO_TIPO_OBJETO_REFORMA_IMOVEIS_EQUIPAMENTOS)) {
					if (percentualTotalAditivo.abs().compareTo(PERCENTAGEM_MAXIMA_REFORMA_IMOVEIS) > 0) {
						errors.rejectValue("percentualAditivo", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.MSG.PERCENTAGEM_MAIOR_REFORMA_IMOVEIS");
					}
				}

			}
		}

		if (aditivo.getAditivoDeClausula()) {
			ValidationUtils.rejectIfEmpty(errors, "descricao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.DESCRICAO_ADITIVO");
		}

		ValidationUtils.rejectIfEmpty(errors, "justificativa", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.JUSTIFICATIVA_ADITIVO");
	}

	/**
	 *
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por verificar se uma determinada data esta dentro de um determinado prazo definido
	 *
	 * @author geovane.filho
	 *
	 * @param data a se verificar se esta dentro do prazo
	 * @param prazoInicial Data inicial do prazo
	 * @param prazoFinal Data final do prazo
	 * @return
	 */
	private boolean dataDentroPrazo(Calendar data, Calendar prazoInicial, Calendar prazoFinal) {
		if ((data.compareTo(prazoInicial) >= 0) && (data.compareTo(prazoFinal) <= 0)) {
			return true;
		} else {
			return false;
		}
	}
}
