package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EventoApostilamento;
import br.com.centralit.framework.exception.CodigoErro;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 11/03/2016 - 08:33:20
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */
@Component("eventoApostilamentoValidator")
public class EventoApostilamentoValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoApostilamentoValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoApostilamento eventoApostilamento = (EventoApostilamento) target;

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");
		ValidationUtils.rejectIfEmpty(errors, "dataEmissao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_EMISSAO_APOSTILAMENTO");

		if (eventoApostilamento.getDataEmissao().getTimeInMillis() < eventoApostilamento.getContrato().getDataVigenciaInicial().getTimeInMillis()
				|| eventoApostilamento.getDataEmissao().getTimeInMillis() > eventoApostilamento.getContrato().getDataUltimaVigenciaFinal().getTimeInMillis()){
				ValidationUtils.rejectIfEmpty(errors, "dataEmissao", "MSG.DATA_APOSTILAMENTO_INVALIDA", "LABEL.DATA_EMISSAO_APOSTILAMENTO");
			}

		ValidationUtils.rejectIfEmpty(errors, "justificativa", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.JUSTIFICATIVA_APOSTILAMENTO");
		ValidationUtils.rejectIfEmpty(errors, "registroAdministrativo", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DESCRICAO_REGISTRO_ADMINISTRATIVO");

	}

}
