package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EventoAtualizacaoStatus;
import br.com.centralit.framework.exception.CodigoErro;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 */
@Component("eventoAtualizacaoStatusValidator")
public class EventoAtualizacaoStatusValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoAtualizacaoStatusValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoAtualizacaoStatus evento = (EventoAtualizacaoStatus) target;

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");
		ValidationUtils.rejectIfEmpty(errors, "dataAtualizacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.DATA_ATUALIZACAO");

		if (evento.getDataAtualizacao().getTimeInMillis() < evento.getContrato().getDataVigenciaInicial().getTimeInMillis()
				|| evento.getDataAtualizacao().getTimeInMillis() > evento.getContrato().getDataUltimaVigenciaFinal().getTimeInMillis()) {

			errors.rejectValue("dataAtualizacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.MSG.DATA_ATUALIZACAO_STATUS_INVALIDA");
		}

	}

}
