package br.com.centralit.api.service.validation;

import java.util.Calendar;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EventoLiberacaoPagamento;
import br.com.centralit.framework.exception.CodigoErro;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 */
@Component("eventoLiberacaoPagamentoValidator")
public class EventoLiberacaoPagamentoValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoLiberacaoPagamentoValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoLiberacaoPagamento evento = (EventoLiberacaoPagamento) target;

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");
		ValidationUtils.rejectIfEmpty(errors, "dataLiberacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LIBERACAO_PAGAMENTO.DATA_LIBERACAO");

		if (evento.getDataLiberacao().getTimeInMillis() < evento.getContrato().getDataVigenciaInicial().getTimeInMillis()) {
			errors.rejectValue("dataLiberacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LIBERACAO_PAGAMENTO.MSG.DATA_LIBERACAO");
		}

    	if(evento.getContrato() != null && evento.getTotalLiberadoPagamento() != null && evento.getContrato().getValorTotalContrato() != null &&
    			evento.getTotalLiberadoPagamento().longValue() > evento.getContrato().getValorTotalContrato().longValue()){
    		errors.rejectValue("totalLiberadoPagamento", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "CONTRATOS.VALIDACAO.VALOR_TOTAL_A_LIBERAR_ULTRAPASSA_VALOR_CONTRATO");
			return;
    	}

	}

}
