package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EventoPenalidade;
import br.com.centralit.framework.exception.CodigoErro;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 11/03/2016 - 08:33:20
 * @version 1.0.0
 * @author luis.camargo (<a href="mailto:luis.camargo@centralit.com.br">luis.camargo@centralit.com.br</a>)
 *
 */
@Component("eventoPenalidadeValidator")
public class EventoPenalidadeValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoPenalidadeValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoPenalidade eventoPenalidade = (EventoPenalidade) target;

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");
		ValidationUtils.rejectIfEmpty(errors, "dataPenalizacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_PENALIZACAO");
		ValidationUtils.rejectIfEmpty(errors, "termoReferenciaPenalidade", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.PENALIDADE");

		if (eventoPenalidade.getDataPenalizacao().getTimeInMillis() < eventoPenalidade.getContrato().getDataVigenciaInicial().getTimeInMillis()
				|| eventoPenalidade.getDataPenalizacao().getTimeInMillis() > eventoPenalidade.getContrato().getDataUltimaVigenciaFinal().getTimeInMillis()) {

			errors.rejectValue("dataPenalizacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.VALIDACAO.DATA_PENALIZACAO_INVALIDA");
		}

	}

}
