package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EventoRecebimentoObjeto;
import br.com.centralit.framework.exception.CodigoErro;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 17/03/2016 - 09:48:29
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */
@Component("eventoRecebimentoObjetoValidator")
public class EventoRecebimentoObjetoValidator implements Validator{

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoRecebimentoObjetoValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoRecebimentoObjeto eventoRecebimentoObjeto = (EventoRecebimentoObjeto) target;

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");
		ValidationUtils.rejectIfEmpty(errors, "tipoRecebimento", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.TIPO_RECEBIMENTO");
		ValidationUtils.rejectIfEmpty(errors, "tipoStatus", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.STATUS");
		ValidationUtils.rejectIfEmpty(errors, "tipoAvaliacaoPrazo", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.AVALIACAO_OBJETO");
		ValidationUtils.rejectIfEmpty(errors, "tipoAvaliacaoAdesao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.AVALIACAO_OBJETO");

		if (eventoRecebimentoObjeto.getDataRecebimento().getTimeInMillis() < eventoRecebimentoObjeto.getContrato().getDataVigenciaInicial().getTimeInMillis()
				&& eventoRecebimentoObjeto.getDataRecebimento().getTimeInMillis() > eventoRecebimentoObjeto.getContrato().getDataUltimaVigenciaFinal().getTimeInMillis()){
				ValidationUtils.rejectIfEmpty(errors, "dataRecebimento", "CONTRATOS.MSG.DATA_RECEBIMENTO_INVALIDA", "CONTRATOS.LABEL.DATA_RECEBIMENTO_OBJETO");
		}
	}

}
