package br.com.centralit.api.service.validation;

import java.util.Calendar;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EventoAditivo;
import br.com.centralit.api.model.EventoRepactuacao;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.util.UtilObjeto;


/**
 * 
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 19/03/2016 - 09:32:20
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Component("eventoRepactuacaoValidator")
public class EventoRepactuacaoValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoRepactuacaoValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoRepactuacao eventoRepactuacao = (EventoRepactuacao) target;

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");
		ValidationUtils.rejectIfEmpty(errors, "numeroRepactuacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.NUM_REPACTUACAO");
		ValidationUtils.rejectIfEmpty(errors, "dataRepactuacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.DATA_REPACTUACAO");
		ValidationUtils.rejectIfEmpty(errors, "registroOcorrencia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.NUMERO_OCORRENCIA");
		ValidationUtils.rejectIfEmpty(errors, "justificativaRepactuacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.LABEL.JUSTIFICATIVA_REPACTUACAO");

		Calendar dataFinal = null;
		
		if (UtilObjeto.isReferencia(eventoRepactuacao.getContrato())) {
			dataFinal = eventoRepactuacao.getContrato().getDataVigenciaFinal();
			
			if (UtilObjeto.isReferencia(eventoRepactuacao.getContrato().getAditivos())) {
				for (EventoAditivo adContrato : eventoRepactuacao.getContrato().getAditivos()) {
					if (adContrato.getAditivoDePrazo() && adContrato.getDataFim().after(dataFinal)) {
						dataFinal = adContrato.getDataFim();
					}
				}
			}
		}
		
		if (eventoRepactuacao.getDataRepactuacao().getTimeInMillis() < eventoRepactuacao.getContrato().getDataVigenciaInicial().getTimeInMillis()
				|| eventoRepactuacao.getDataRepactuacao().getTimeInMillis() > dataFinal.getTimeInMillis()) {

			errors.rejectValue("dataRepactuacao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.VALIDACAO.DATA_REPACTUACAO_VIGENCIA");
		}

	}

}
