package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.EventoRescisaoContrato;
import br.com.centralit.framework.exception.CodigoErro;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 */
@Component("eventoRescisaoContratoValidator")
public class EventoRescisaoContratoValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {

		return EventoRescisaoContratoValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		EventoRescisaoContrato evento = (EventoRescisaoContrato) target;

		ValidationUtils.rejectIfEmpty(errors, "dataRescisao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.RESCISAO_CONTRATO.DATA_RESCISAO");
		ValidationUtils.rejectIfEmpty(errors, "tipoRescisao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.RESCISAO_CONTRATO.TIPO_RESCISAO");

		if (evento.getDataRescisao().getTimeInMillis() < evento.getContrato().getDataVigenciaInicial().getTimeInMillis()
				|| evento.getDataRescisao().getTimeInMillis() > evento.getContrato().getDataUltimaVigenciaFinal().getTimeInMillis()) {

			errors.rejectValue("dataRescisao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.MSG.DATA_RESCISAO_INVALIDA");
		}

    	  if(evento.getDevolucaoValor() != null && evento.getValorADevolver() != null && evento.getValorADevolver().doubleValue() > evento.getContrato().getValorTotalContrato().doubleValue()){
    		  errors.rejectValue("valorADevolver", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "CONTRATOS.MSG.RESCISAO_CONTRATO_VALOR_DEVOLVIDO_MAIOR_CONTRATO");
        	return;
  	  }

	}

}
