package br.com.centralit.api.service.validation;

import java.util.HashMap;
import java.util.Map;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.OrdemFornecimento;
import br.com.centralit.api.model.OrdemFornecimentoItem;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.util.UtilObjeto;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 03/02/2016 - 17:04:51
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Component("ordemFornecimentoValidator")
public class OrdemFornecimentoValidator implements Validator{
	@Override
	public boolean supports(Class<?> clazz) {

		return OrdemFornecimentoValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		ValidationUtils.rejectIfEmpty(errors, "numOrdemFornecimento", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.NUM_ORDEM_FORNECIMENTO");

		ValidationUtils.rejectIfEmpty(errors, "dataEmissao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.DATA_EMISSAO");

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.NUM_CONTRATO");

		ValidationUtils.rejectIfEmpty(errors, "itensBensServicos", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.ESPECIFICACAO_PRODUTOS_SERVICOS_VOLUMES");

		OrdemFornecimento ordemForn = (OrdemFornecimento) target;

		Map<Long, Long> quantidades = new HashMap<Long, Long>();
		for (OrdemFornecimentoItem item : ordemForn.getItensBensServicos()) {
			if (item.getDataTerminoPrevisto().before(item.getDataInicioPrevisto())) {
				errors.rejectValue("itensBensServicos", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "VALIDACAO.ITEM_PERIODO_INVALIDO");
			}

			if (item.getDataInicioPrevisto().before(ordemForn.getContrato().getDataVigenciaInicial()) ||
					ordemForn.getContrato().getDataUltimaVigenciaFinal().before(item.getDataTerminoPrevisto())) {
				errors.rejectValue("itensBensServicos", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "VALIDACAO.ITEM_PERIODO_FORA_VIGENCIA_CONTRATO");
			}
			if (UtilObjeto.isReferencia(quantidades.get(item.getItemBemServico().getId()))) {
				quantidades.put(item.getItemBemServico().getId(), quantidades.get(item.getItemBemServico().getId()) + item.getQuantidade());
			} else {
				quantidades.put(item.getItemBemServico().getId(), item.getQuantidade());
			}
		}
		
		for (OrdemFornecimento ordemCadastrada : ordemForn.getContrato().getOrdensFornecimento()) {
			if ((!UtilObjeto.isReferencia(ordemForn.getId()) || (UtilObjeto.isReferencia(ordemForn.getId()) && !ordemForn.getId().equals(ordemCadastrada.getId())))) {
				for (OrdemFornecimentoItem item : ordemCadastrada.getItensBensServicos()) {
					if (UtilObjeto.isReferencia(quantidades.get(item.getItemBemServico().getId()))) {
						quantidades.put(item.getItemBemServico().getId(), quantidades.get(item.getItemBemServico().getId()) + item.getQuantidade());
					} else {
						quantidades.put(item.getItemBemServico().getId(), item.getQuantidade());
					}
				}
			}
		}

		for (Long contratoItemId : quantidades.keySet()) {
			for (OrdemFornecimentoItem ordemItem : ordemForn.getItensBensServicos()) {
				if (ordemItem.getItemBemServico().getId() == contratoItemId) {
					if (quantidades.get(contratoItemId) > ordemItem.getItemBemServico().getQuantidade()) {
						errors.rejectValue("itensBensServicos", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "VALIDACAO.QUANTIDADE_INSERIDA_TOTAL_MAIOR_QUANTIDADE_CONTRATO");
					}
				}
			}
		}
	}
}
