package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.RegistroOcorrencia;
import br.com.centralit.framework.exception.CodigoErro;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 15/02/2016 - 15:04:44
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */
@Component("registroOcorrenciaValidator")
public class RegistroOcorrenciaValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {

		return RegistroOcorrenciaValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {

		RegistroOcorrencia registroOcorrencia = (RegistroOcorrencia) target;

		ValidationUtils.rejectIfEmpty(errors, "contrato", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CONTRATO");
		ValidationUtils.rejectIfEmpty(errors, "dataEmissaoOcorrencia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DATA_EMISSAO_OCORRENCIA");

		if (registroOcorrencia.getDataEmissaoOcorrencia().getTimeInMillis() < registroOcorrencia.getContrato().getDataVigenciaInicial().getTimeInMillis()
			&& registroOcorrencia.getDataEmissaoOcorrencia().getTimeInMillis() > registroOcorrencia.getContrato().getDataUltimaVigenciaFinal().getTimeInMillis()){

			ValidationUtils.rejectIfEmpty(errors, "dataEmissaoOcorrencia", "MSG.DATA_OCORRENCIA_INVALIDA", "LABEL.DATA_EMISSAO_OCORRENCIA");
		}

		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "tituloOcorrencia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.TITULO_OCORRENCIA");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "descricaoOcorrencia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DESCRICAO_OCORRENCIA");

		if(registroOcorrencia.getEmitirGlosa() != null){
			if(registroOcorrencia.getEmitirGlosa().booleanValue()){
				ValidationUtils.rejectIfEmptyOrWhitespace(errors, "valorGlosa", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.VALOR_GLOSA");
				ValidationUtils.rejectIfEmptyOrWhitespace(errors, "justificativaGlosa", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.JUSTIFICATIVA_GLOSA");
			}
		}
	}

}
