package br.com.centralit.api.service.impl;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.when;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.stubbing.Answer;

import br.com.centralit.api.dao.EstudoTecnicoDao;
import br.com.centralit.api.model.Colaborador;
import br.com.centralit.api.model.EstudoTecnico;
import br.com.centralit.api.model.Fornecedor;
import br.com.centralit.api.model.ObjetivoEstrategicoNecessidade;
import br.com.centralit.api.model.OrgaoExterno;
import br.com.centralit.api.service.ColaboradorService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.ObjetivoEstrategicoNecessidadeService;
import br.com.centralit.api.service.ParceiroService;
import br.com.centralit.api.service.validation.EstudoTecnicoValidator;
import br.com.six2six.fixturefactory.Fixture;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="http://goo.gl/xRjvgR">607</a>
 * </p>
 *
 * @since 06/01/2015 - 14:39:48
 * @version 1.0.0
 * @author iago.almeida (<a href="mailto:iago.almeida@centralit.com.br">iago.almeida@centralit.com.br</a>)
 *
 */
@RunWith(MockitoJUnitRunner.class)
public class EstudoTecnicoServiceImplTest extends GenericServiceImplTest<EstudoTecnicoServiceImpl> {

	/** Atributo LOG para a classe EstudoTecnicoServiceImplTest. */
	public static final Logger LOG = Logger.getLogger(EstudoTecnicoServiceImplTest.class);

	/** Atributo CODIGO_INICIAL_ESTUDO_TECNICO_TESTE_SAVE_SEM_CODIGO. */
	public static final String CODIGO_INICIAL_ESTUDO_TECNICO_TESTE_SAVE_SEM_CODIGO  = "000001";

	/** Atributo estudoTecnicoDao.
	 * foi usado @InjectMocks apenas para que todas as dependecias do atributo em questao possam ser utilizados nos metodos de teste
	 * quando necessarios
	 * */
	@InjectMocks
    private EstudoTecnicoServiceImpl estudoTecnicoServiceImpl;

	/** Atributo mockado estudoTecnicoDao.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
    private EstudoTecnicoDao estudoTecnicoDao;

	/** Atributo mockado estudoTecnicoValidator.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
    private EstudoTecnicoValidator estudoTecnicoValidator;

	/** Atributo mockado parceiroServiceFornecedor.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
	private ParceiroService parceiroServiceFornecedor;

	/** Atributo mockado parceiroServiceOrgaoExterno.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
	private ParceiroService parceiroServiceOrgaoExterno;

	/** Atributo mockado colaboradorService.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
	private ColaboradorService colaboradorService;

	/** Atributo mockado dominioService.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
	private DominioService dominioService;

	/** Atributo mockado objetivoEstrategicoNecessidadeService.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
	private ObjetivoEstrategicoNecessidadeService objetivoEstrategicoNecessidadeService;

	/** Atributo mockado configuracaoParametroSistema.
	 * foi usado @Mock apenas para que esse atributo seja utilizado no construtor do Service a ser testado
	 * */
	@Mock
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	@Override
	protected void inicializarMocksDasDependencias() {

	}

	@Before
	public void setUp(){

		LOG.info("==================== Iniciando teste EstudoTecnicoServiceImplTest! ==================== ");
		definirAtributo(this.estudoTecnicoServiceImpl, "validator", this.estudoTecnicoValidator);
		definirAtributo(this.estudoTecnicoServiceImpl, "parceiroServiceFornecedor", this.parceiroServiceFornecedor);
		definirAtributo(this.estudoTecnicoServiceImpl, "parceiroServiceOrgaoExterno", this.parceiroServiceOrgaoExterno);
		definirAtributo(this.estudoTecnicoServiceImpl, "colaboradorService", this.colaboradorService);
		definirAtributo(this.estudoTecnicoServiceImpl, "dominioService", this.dominioService);
		definirAtributo(this.estudoTecnicoServiceImpl, "objetivoEstrategicoNecessidadeService", this.objetivoEstrategicoNecessidadeService);
		definirAtributo(this.estudoTecnicoServiceImpl, "configuracaoParametroSistemaService", this.configuracaoParametroSistemaService);
		MockitoAnnotations.initMocks(this);
	}

	@After
	public void tearDown(){

		LOG.info("==================== Finalizando teste EstudoTecnicoServiceImplTest! ==================== ");
	}

	@Test
	public void testSavePrimeiroRegistroEstudoTecnicoSemCodigo(){

		LOG.info("==================== Inicializando testSavePrimeiroRegistroEstudoTecnicoSemCodigo ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");

		Colaborador colaborador = Fixture.from(Colaborador.class).gimme("valid");

		Fornecedor fornecedor = Fixture.from(Fornecedor.class).gimme("valid");

		OrgaoExterno orgaoExterno = Fixture.from(OrgaoExterno.class).gimme("valid");

		ObjetivoEstrategicoNecessidade objetivoEstrategicoNecessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");

		// Quando for chamado colaboradorService.find(Long) criar retornará o objeto
		when(this.colaboradorService.find(anyLong())).thenReturn(colaborador);

		// Quando for chamado parceiroServiceFornecedor.find(Long) criar retornará o objeto
		when(this.parceiroServiceFornecedor.find(anyLong())).thenReturn(fornecedor);

		// Quando for chamado parceiroServiceOrgaoExterno.find(Long) criar retornará o objeto
		when(this.parceiroServiceOrgaoExterno.find(anyLong())).thenReturn(orgaoExterno);

		// Quando for chamado objetivoEstrategicoNecessidadeService.find(Long) irá retornará o objeto
		when(this.objetivoEstrategicoNecessidadeService.find(anyLong())).thenReturn(objetivoEstrategicoNecessidade);

		// Quando for chamado configuracaoParametroSistemaService.gerarNumeroIdentificacaoSeguencial(Long) irá retornará o objeto
		when(this.configuracaoParametroSistemaService.gerarNumeroIdentificacaoSeguencial(anyLong())).thenReturn(CODIGO_INICIAL_ESTUDO_TECNICO_TESTE_SAVE_SEM_CODIGO);

		// Quando for chamado estudoTecnicoDao.save(PersistentObject) irá retornará o objeto
		when(this.estudoTecnicoDao.save(any(EstudoTecnico.class)))
        .thenAnswer(new Answer<EstudoTecnico>() {
            @Override
            public EstudoTecnico answer(InvocationOnMock invocation) throws Throwable {
            	EstudoTecnico estudoTecnico = (EstudoTecnico) invocation.getArguments()[0];
            	estudoTecnico.setId(1L);
                return estudoTecnico;
            }
        });

		EstudoTecnico estudoTecnicoSaved = this.estudoTecnicoServiceImpl.save(estudoTecnico);

		assertNotNull(estudoTecnicoSaved.getId());
		assertTrue(estudoTecnicoSaved.getId() > 0);
		assertNotNull(estudoTecnicoSaved.getRegistroSequencial());
		LOG.info("==================== Finalizando testSavePrimeiroRegistroEstudoTecnicoSemCodigo ==================== ");

	}

	@Test
	public void testMergeRegistroEstudoTecnicoComCodigo(){

		LOG.info("==================== Inicializando testMergeRegistroEstudoTecnicoComCodigo ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setId(1L);

		Colaborador colaborador = Fixture.from(Colaborador.class).gimme("valid");

		Fornecedor fornecedor = Fixture.from(Fornecedor.class).gimme("valid");

		OrgaoExterno orgaoExterno = Fixture.from(OrgaoExterno.class).gimme("valid");

		ObjetivoEstrategicoNecessidade objetivoEstrategicoNecessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");

		// Quando for chamado colaboradorService.find(Long) criar retornará o objeto
		when(this.colaboradorService.find(anyLong())).thenReturn(colaborador);

		// Quando for chamado parceiroServiceFornecedor.find(Long) criar retornará o objeto
		when(this.parceiroServiceFornecedor.find(anyLong())).thenReturn(fornecedor);

		// Quando for chamado parceiroServiceOrgaoExterno.find(Long) criar retornará o objeto
		when(this.parceiroServiceOrgaoExterno.find(anyLong())).thenReturn(orgaoExterno);

		// Quando for chamado objetivoEstrategicoNecessidadeService.find(Long) irá retornará o objeto
		when(this.objetivoEstrategicoNecessidadeService.find(anyLong())).thenReturn(objetivoEstrategicoNecessidade);

		// Quando for chamado estudoTecnicoDao.save(PersistentObject) irá retornará o objeto
		when(this.estudoTecnicoDao.save(any(EstudoTecnico.class))).thenReturn(estudoTecnico);

		EstudoTecnico estudoTecnicoSaved = this.estudoTecnicoServiceImpl.merge(estudoTecnico);

		assertNotNull(estudoTecnicoSaved.getId());
		assertTrue(estudoTecnicoSaved.getId() > 0);

		LOG.info("==================== Finalizando testMergeRegistroEstudoTecnicoComCodigo ==================== ");

	}
}
