package br.com.centralit.api.service.impl;

import static org.mockito.Matchers.anyLong;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.Collection;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.runners.MockitoJUnitRunner;

import br.com.centralit.api.dao.ObjetivoEstrategicoNecessidadeDao;
import br.com.centralit.api.model.ObjetivoEstrategico;
import br.com.centralit.api.model.ObjetivoEstrategicoNecessidade;
import br.com.centralit.api.service.EstudoTecnicoRequisitoNegocioService;
import br.com.centralit.api.service.validation.ObjetivoEstrategicoNecessidadeValidator;
import br.com.centralit.framework.exception.BusinessException;
import br.com.six2six.fixturefactory.Fixture;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="http://goo.gl/xRjvgR">607</a>
 * </p>
 *
 * @since 12/01/2015 - 17:14:31
 * @version 1.0.0
 * @author luis.camargo
 *
 */
@RunWith(MockitoJUnitRunner.class)
public class ObjetivoEstrategicoNecessidadeServiceImplTest extends GenericServiceImplTest<ObjetivoEstrategicoNecessidadeServiceImpl> {

	public static final Logger LOG = Logger.getLogger(ObjetivoEstrategicoNecessidadeServiceImplTest.class);


	@InjectMocks
	private ObjetivoEstrategicoNecessidadeServiceImpl service;
	
	@Mock
    private ObjetivoEstrategicoNecessidadeDao objetivoEstrategicoNecessidadeDao;
	@Mock
	private ObjetivoEstrategicoNecessidadeValidator objetivoEstrategicoNecessidadeValidator;
	@Mock
	private EstudoTecnicoRequisitoNegocioService estudoTecnicoRequisitoNegocioService;

	@Override
	protected void inicializarMocksDasDependencias() {
		definirAtributo(service, "objetivoEstrategicoNecessidadeDao", objetivoEstrategicoNecessidadeDao);
		definirAtributo(service, "validator", objetivoEstrategicoNecessidadeValidator);
		definirAtributo(service, "estudoTecnicoRequisitoNegocioService", estudoTecnicoRequisitoNegocioService);

		MockitoAnnotations.initMocks(this);
	}

	@Before
	public void setUp(){
		LOG.info("==================== Iniciando teste ObjetivoEstrategicoNecessidadeServiceImplTest! ==================== ");
	}

	@After
	public void tearDown(){
		LOG.info("==================== Finalizando teste ObjetivoEstrategicoNecessidadeServiceImplTest! ==================== ");
	}

	@Test
	public void testFindAllByObjetivoEstrategico() {
		ObjetivoEstrategico objetivo = Fixture.from(ObjetivoEstrategico.class).gimme("valid");
		ObjetivoEstrategicoNecessidade necessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");
		necessidade.setObjetivoEstrategico(objetivo);

		when(this.objetivoEstrategicoNecessidadeDao.findAllByObjetivoEstrategico(anyLong())).thenReturn(Arrays.asList(necessidade));

		Collection<ObjetivoEstrategicoNecessidade> lista = service.findAllByObjetivoEstrategico(objetivo.getId());

		Assert.assertFalse(lista.isEmpty());
	}

	@Test
	public void testRemoveByIdSucesso() {
		ObjetivoEstrategicoNecessidade necessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");

		when(this.estudoTecnicoRequisitoNegocioService.existeVinculo(anyString(), anyLong())).thenReturn(false);
		when(this.objetivoEstrategicoNecessidadeDao.removeById(anyLong())).thenReturn(true);
		
		Boolean excluido = service.removeById(necessidade.getId());
		
		Assert.assertTrue(excluido);
	}
	
	@Test(expected=BusinessException.class)
	public void testRemoveByIdVinculadoEstudoTecnico() {
		ObjetivoEstrategicoNecessidade necessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");
		
		when(this.estudoTecnicoRequisitoNegocioService.existeVinculo(anyString(), anyLong())).thenReturn(true);
		
		Boolean excluido = service.removeById(necessidade.getId());
		
		Assert.assertTrue(excluido);
	}

	@Test
	public void testRemoveListSucesso() {
		ObjetivoEstrategicoNecessidade necessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");

		when(this.estudoTecnicoRequisitoNegocioService.existeVinculo(anyString(), anyLong())).thenReturn(false);
		
		service.removeList(Arrays.asList(necessidade));
	}

	@Test(expected=BusinessException.class)
	public void testRemoveListVinculadoEstudoTecnico() {
		ObjetivoEstrategicoNecessidade necessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");
		
		when(this.estudoTecnicoRequisitoNegocioService.existeVinculo(anyString(), anyLong())).thenReturn(true);
		
		service.removeList(Arrays.asList(necessidade));
	}

	@Test
	public void testSalvarListaNecessidadesSucessoNovoItem() {
		ObjetivoEstrategico objetivo = Fixture.from(ObjetivoEstrategico.class).gimme("valid");
		ObjetivoEstrategicoNecessidade necessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("noId");
		objetivo.setNecessidadesElencadas(Arrays.asList(necessidade));
		
		service.salvarListaNecessidades(objetivo.getNecessidadesElencadas(), objetivo);
	}
	
	@Test
	public void testSalvarListaNecessidadesSucessoItemExistente() {
		ObjetivoEstrategico objetivo = Fixture.from(ObjetivoEstrategico.class).gimme("valid");
		ObjetivoEstrategicoNecessidade necessidade = Fixture.from(ObjetivoEstrategicoNecessidade.class).gimme("valid");
		objetivo.setNecessidadesElencadas(Arrays.asList(necessidade));
		
		service.salvarListaNecessidades(objetivo.getNecessidadesElencadas(), objetivo);
	}
}
