package br.com.centralit.api.service.impl;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyLong;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.Collection;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.runners.MockitoJUnitRunner;

import br.com.centralit.api.dao.ObjetivoEstrategicoDao;
import br.com.centralit.api.model.ObjetivoEstrategico;
import br.com.centralit.api.service.ObjetivoEstrategicoNecessidadeService;
import br.com.centralit.api.service.validation.ObjetivoEstrategicoValidator;
import br.com.six2six.fixturefactory.Fixture;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governan&ccedil;a Corporativa -
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="http://goo.gl/xRjvgR">607</a>
 * </p>
 *
 * @since 12/01/2015 - 17:14:31
 * @version 1.0.0
 * @author luis.camargo
 *
 */
@RunWith(MockitoJUnitRunner.class)
public class ObjetivoEstrategicoServiceImplTest extends GenericServiceImplTest<ObjetivoEstrategicoServiceImpl> {

	public static final Logger LOG = Logger.getLogger(ObjetivoEstrategicoServiceImplTest.class);


	@InjectMocks
	private ObjetivoEstrategicoServiceImpl objetivoEstrategicoServiceImpl;
	
	@Mock
    private ObjetivoEstrategicoDao objetivoEstrategicoDao;
	@Mock
	private ObjetivoEstrategicoValidator objetivoEstrategicoValidator;
	@Mock
	private ObjetivoEstrategicoNecessidadeService objetivoEstrategicoNecessidadeService;

	@Override
	protected void inicializarMocksDasDependencias() {
		definirAtributo(objetivoEstrategicoServiceImpl, "objetivoEstrategicoDao", objetivoEstrategicoDao);
		definirAtributo(objetivoEstrategicoServiceImpl, "validator", objetivoEstrategicoValidator);
		definirAtributo(objetivoEstrategicoServiceImpl, "objetivoEstrategicoNecessidadeService", objetivoEstrategicoNecessidadeService);

		MockitoAnnotations.initMocks(this);
	}

	@Before
	public void setUp(){
		LOG.info("==================== Iniciando teste ObjetivoEstrategicoServiceImplTest! ==================== ");
	}

	@After
	public void tearDown(){
		LOG.info("==================== Finalizando teste ObjetivoEstratégicoServiceImplTest! ==================== ");
	}

	@Test
	public void testSaveSucesso() {
		ObjetivoEstrategico objetivoEstrategico = Fixture.from(ObjetivoEstrategico.class).gimme("noId");
		ObjetivoEstrategico objetivoSaved = Fixture.from(ObjetivoEstrategico.class).gimme("valid");
		
		when(this.objetivoEstrategicoDao.save(any(ObjetivoEstrategico.class))).thenReturn(objetivoSaved);

		ObjetivoEstrategico entity = objetivoEstrategicoServiceImpl.save(objetivoEstrategico);
		Assert.assertNotNull(entity);
	}
	
	@Test
	public void testMergeSucesso() {
		ObjetivoEstrategico objetivoEstrategico = Fixture.from(ObjetivoEstrategico.class).gimme("valid");
		
		when(this.objetivoEstrategicoDao.merge(any(ObjetivoEstrategico.class))).thenReturn(objetivoEstrategico);
		
		ObjetivoEstrategico entity = objetivoEstrategicoServiceImpl.merge(objetivoEstrategico);
		
		Assert.assertNotNull(entity);
	}
	
	@Test
	public void testFindAllByOrganizacao() {
		ObjetivoEstrategico objetivo = Fixture.from(ObjetivoEstrategico.class).gimme("valid");
		
		when(this.objetivoEstrategicoDao.findAllByOrganizacao(anyLong())).thenReturn(Arrays.asList(objetivo));

		Collection<ObjetivoEstrategico> lista = objetivoEstrategicoServiceImpl.findAllByOrganizacao(objetivo.getId());
		
		Assert.assertFalse(lista.isEmpty());
	}
	
	@Test
	public void testRemoveByIdSucesso() {
		ObjetivoEstrategico objetivo = Fixture.from(ObjetivoEstrategico.class).gimme("valid");

		when(this.objetivoEstrategicoDao.getReference(anyLong())).thenReturn(objetivo);
		when(this.objetivoEstrategicoDao.removeById(anyLong())).thenReturn(true);
		
		Boolean excluido = objetivoEstrategicoServiceImpl.removeById(objetivo.getId());
		
		Assert.assertTrue(excluido);
	}
	
}
