package br.com.centralit.api.service.validation;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.validation.BeanPropertyBindingResult;
import org.springframework.validation.Errors;

import br.com.centralit.api.model.EstudoTecnico;
import br.com.centralit.api.service.impl.EstudoTecnicoServiceImplTest;
import br.com.six2six.fixturefactory.Fixture;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 12/01/2016 - 15:20:20
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
public class EstudoTecnicoValidatorTest extends GenericValidatorTest{

	/** Atributo LOG para a classe EstudoTecnicoValidatorTest. */
	public static final Logger LOG = Logger.getLogger(EstudoTecnicoServiceImplTest.class);

	/** Atributo estudoTecnicoValidator. Utilizado para acessar os validadores da classe EstudoTecnico */
	private EstudoTecnicoValidator estudoTecnicoValidator;

	@Before
	public void setUp(){

		LOG.info("==================== Iniciando test EstudoTecnicoValidatorTest! ==================== ");
		estudoTecnicoValidator = new EstudoTecnicoValidator();
	}

	@After
	public void tearDown(){

		LOG.info("==================== Finalizando test EstudoTecnicoValidatorTest! ==================== ");
	}


	@Test
	public void testValidationEstudoTecnicoValid(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoValid ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertFalse(errors.hasErrors());

		LOG.info("==================== Finalizando testValidationEstudoTecnicoValid ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoAllInvalid(){

		LOG.info("==================== Inicializando testeValidationEstudoTecnicoAllInvalid ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setNumeroPortaria("");
		estudoTecnico.setDataInstituicaoEquipe(null);
		estudoTecnico.setIntegranteTecnico(null);
		estudoTecnico.setIntegranteRequisitante(null);
		estudoTecnico.setIntegranteAdministrativo(null);
		estudoTecnico.setDescricaoSolucao("");
		estudoTecnico.setEtRequisitosNegocio(null);
		estudoTecnico.setEtLevantamentoAlternativas(null);
		estudoTecnico.setDescricaoJustificativaSolucao("");
		estudoTecnico.setEtBensServicosSolucoes(null);
		estudoTecnico.setEtBeneficiosEsperados(null);
		estudoTecnico.setEtAcoesContingencia(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("numeroPortaria"));
		assertNotNull(errors.getFieldError("dataInstituicaoEquipe"));
		assertNotNull(errors.getFieldError("integranteTecnico"));
		assertNotNull(errors.getFieldError("integranteRequisitante"));
		assertNotNull(errors.getFieldError("integranteAdministrativo"));
		assertNotNull(errors.getFieldError("etRequisitosNegocio"));
		assertNotNull(errors.getFieldError("etLevantamentoAlternativas"));
		assertNotNull(errors.getFieldError("descricaoJustificativaSolucao"));
		assertNotNull(errors.getFieldError("etBensServicosSolucoes"));
		assertNotNull(errors.getFieldError("etBeneficiosEsperados"));
		assertNotNull(errors.getFieldError("etAcoesContingencia"));

		LOG.info("==================== Finalizando testeValidationEstudoTecnicoAllInvalid ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComNumeroPortariaVazio(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComNumeroPortariaVazio ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setNumeroPortaria("");

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("numeroPortaria"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComNumeroPortariaVazio ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComDataInstituicaoEquipeNull(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComDataInstituicaoEquipeNull ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setDataInstituicaoEquipe(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("dataInstituicaoEquipe"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComDataInstituicaoEquipeNull ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComIntegranteTecnicoNull(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComIntegranteTecnicoNull ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setIntegranteTecnico(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("integranteTecnico"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComIntegranteTecnicoNull ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComIntegranteRequisitanteNull(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComIntegranteRequisitanteNull ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setIntegranteRequisitante(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("integranteRequisitante"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComIntegranteRequisitanteNull ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComIntegranteAdministrativoNull(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComIntegranteAdministrativoNull ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setIntegranteAdministrativo(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("integranteAdministrativo"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComIntegranteAdministrativoNull ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComDescricaoSolucaoVazio(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComDescricaoSolucaoVazio ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setDescricaoSolucao("");

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("descricaoSolucao"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComDescricaoSolucaoVazio ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComListaEtRequisitosNegocioVazia(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComListaEtRequisitosNegocioVazia ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setEtRequisitosNegocio(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("etRequisitosNegocio"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComListaEtRequisitosNegocioVazia ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComRequisitosTecnicosVazio(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComRequisitosTecnicosVazio ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setRequisitosTecnicos("");

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("requisitosTecnicos"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComRequisitosTecnicosVazio ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComListaEtLevantamentoAlternativasVazia(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComListaEtLevantamentoAlternativasVazia ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setEtLevantamentoAlternativas(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("etLevantamentoAlternativas"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComListaEtLevantamentoAlternativasVazia ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComDescricaoJustificativaSolucaoVazio(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComDescricaoJustificativaSolucaoVazio ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setDescricaoJustificativaSolucao("");

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("descricaoJustificativaSolucao"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComDescricaoJustificativaSolucaoVazio ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComEtBensServicosSolucoesNull(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComEtBensServicosSolucoesNull ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setEtBensServicosSolucoes(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("etBensServicosSolucoes"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComEtBensServicosSolucoesNull ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComEtBeneficiosEsperadosNull(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComEtBeneficiosEsperadosNull ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setEtBeneficiosEsperados(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("etBeneficiosEsperados"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComEtBeneficiosEsperadosNull ==================== ");
	}

	@Test
	public void testValidationEstudoTecnicoComEtAcoesContingenciaNull(){

		LOG.info("==================== Inicializando testValidationEstudoTecnicoComEtAcoesContingenciaNull ==================== ");

		EstudoTecnico estudoTecnico = Fixture.from(EstudoTecnico.class).gimme("valid");
		estudoTecnico.setEtAcoesContingencia(null);

		Errors errors = new BeanPropertyBindingResult(estudoTecnico, "estudoTecnico");
		this.estudoTecnicoValidator.validate(estudoTecnico, errors);

		assertTrue(errors.hasErrors());
		assertNotNull(errors.getFieldError("etAcoesContingencia"));

		LOG.info("==================== Finalizando testValidationEstudoTecnicoComEtAcoesContingenciaNull ==================== ");
	}
}
