package br.com.centralit.controller;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.Contrato;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilDataBase;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 25/01/2016 - 15:23:44
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Controller
@RequestMapping("/rest/contrato")
public class ContratoController extends GenericController<Contrato> {

	/** Atributo contratoService. */
	private ContratoService contratoService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private ReportController reportController;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contratoService
	 */
	@Autowired
	public ContratoController( ContratoService contratoService ) {

		super(contratoService);

		this.contratoService = contratoService;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.ContratoListView> getListView() {

		return ViewsContrato.ContratoListView.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.ContratoEditView> getEditView() {

		return ViewsContrato.ContratoEditView.class;
	}
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por procurar contratos pelo número/ano
	 *
	 * @author geovane.filho
	 *
	 * @param numeroAnoContrato
	 * @param idOrganizacao
	 * @return
	 */
	@RequestMapping(value = "/findContrato", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContrato(@RequestParam(value = "numeroAnoContrato") String numeroAnoContrato, @RequestParam(value = "idOrganizacao") Long idOrganizacao) {

		Collection<Contrato> listaContratos = this.contratoService.findContrato(numeroAnoContrato, idOrganizacao);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaContratos, ViewsContrato.ContratoAutoCompleteView.class);

		return responseBody;
	}
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por buscar o gestor do contrato
	 *
	 * @author geovane.filho
	 *
	 * @param idContrato
	 * @return
	 */
	@RequestMapping(value = "/getGestor", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper getGestor(@RequestParam(value = "idContrato") Long idContrato) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(contratoService.getGestor(idContrato), Views.ModulosParceiroView.class);

		return responseBody;
	}

	@RequestMapping(method = RequestMethod.GET, value = "/gerarTermoContrato")
	@ResponseBody
	public void generatePdfRMB(@RequestParam(value="idContrato") Long idContrato, @RequestParam(value="download") boolean download, HttpServletResponse response,
			HttpServletRequest request) throws SQLException, JRException, IOException {

		Dominio idioma = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);

		Usuario user = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;
		try {
			conn = reportController.getConnection();
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/termoContrato.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);

			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("CONTRATO_ID", idContrato);
			subReportParameters.put("REPORT_CONNECTION", conn);

			String titulo = this.internacionalizacaoService.getTranslate("LABEL.TITULO_CONTRATO", idioma);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", user.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			JasperPrint jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);

			// Responsavel por verificar a presenca de dados no relatorio
			reportController.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		}  catch (Exception e) {
			throw new BusinessException();
		}finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}

	}
	
	/**
	 * Método responsável por listar contratos por numero e organizacao
	 *
	 * @author luis.camargo
	 *
	 * @param numeroAnoContrato
	 * 
	 * @param idOrganizacao
	 *
	 * @return Collection<Contrato>
	 */
	@RequestMapping(value = "/findAutoCompleteContratoByNumeroAndOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAutoCompleteContratoByNumeroAndOrganizacao(@RequestParam(value = "numeroAnoContrato") String numeroAnoContrato, @RequestParam(value = "idOrganizacao") Long idOrganizacao) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.contratoService.findAutoCompleteContratoByNumeroAndOrganizacao(numeroAnoContrato, idOrganizacao), ViewsContrato.ContratoAutoCompleteView.class);

		return responseBody;
	}

}
