package br.com.centralit.controller;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.ContratoEmpenho;
import br.com.centralit.api.service.ContratoEmpenhoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import net.sf.jasperreports.engine.JRException;

@Controller
@RequestMapping("/rest/contratoEmpenho")
public class ContratoEmpenhoController extends GenericController<ContratoEmpenho>{

	/** Atributo materialImagemService. */
	private ContratoEmpenhoService contratoEmpenhoService;

	public ContratoEmpenhoController() {
		super();
	}

	@Autowired
	private DominioService dominioService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contratoAnexoService
	 */
	@Autowired
	private ContratoEmpenhoController( ContratoEmpenhoService contratoEmpenhoService ) {

		super(contratoEmpenhoService);

		this.contratoEmpenhoService = contratoEmpenhoService;

	}

	@RequestMapping(value = "/findAllByContrato", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper findAllByContrato(@RequestBody Long contratoId) {

		Collection<ContratoEmpenho> empenhos = this.contratoEmpenhoService.findAllByContrato(contratoId);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(empenhos, ViewsContrato.ContratoEmpenhoView.class);

		return responseBody;
	}
	
	@RequestMapping(value = "/findEmpenhoParaLiberacao", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper findEmpenhoParaLiberacao(@RequestBody Long contratoId) {

		Collection<ContratoEmpenho> empenhos = this.contratoEmpenhoService.findEmpenhoParaLiberacao(contratoId);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(empenhos, ViewsContrato.ContratoEmpenhoView.class);

		return responseBody;
	}
	
	@Override
	public Class<ViewsContrato.ContratoEmpenhoEditView> getEditView() {

		return ViewsContrato.ContratoEmpenhoEditView.class;
	}
	
	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, value = "/pdfGerarEmpenho")
	public void gerarPdfApostilamento(@RequestParam(value = "idEmpenho") Long idEmpenho,
										@RequestParam(value = "download") boolean download, HttpServletResponse response)
										throws SQLException, JRException, IOException {

		contratoEmpenhoService.gerarPdfEmpenho (idEmpenho, download, response);
	}
}
