package br.com.centralit.controller;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.ContratoEvento;
import br.com.centralit.api.model.DominioContrato;
import br.com.centralit.api.service.ContratoEmpenhoService;
import br.com.centralit.api.service.ContratoEventoService;
import br.com.centralit.api.service.ContratoService;
import br.com.centralit.api.viewHelper.ContratoEventosVH;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilString;
import net.sf.jasperreports.engine.JRException;

/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 09/03/2016 - 11:36:22
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Controller
@RequestMapping("/rest/contratoEvento")
public class ContratoEventoController extends GenericController<ContratoEvento> {

	@Autowired
	private ContratoService contratoService;

	@Autowired
	private ContratoEventoService contratoEventoService;

	@Autowired
	private ContratoEmpenhoService contratoEmpenhoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contratoService
	 */
	@Autowired
	public ContratoEventoController( ContratoEventoService contratoEventoService ) {

		super(contratoEventoService);

		this.contratoEventoService = contratoEventoService;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.ContratoEventoListView> getListView() {

		return ViewsContrato.ContratoEventoListView.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.ContratoEventoEditView> getEditView() {

		return ViewsContrato.ContratoEventoEditView.class;
	}

	@RequestMapping(value = "/buscaEventosContrato", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper buscaOcorrenciasContrato(@RequestBody Long contratoId) {

		ContratoEventosVH ocorrenciaContratoVH = new ContratoEventosVH();

		ocorrenciaContratoVH.setContrato(this.contratoService.find(contratoId));
		ocorrenciaContratoVH.setOcorrencias(this.contratoEventoService.findAllByContrato(contratoId));
		ocorrenciaContratoVH.setEmpenhos(this.contratoEmpenhoService.findAllByContrato(contratoId));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(ocorrenciaContratoVH, ViewsContrato.ContratoEventoEditView.class);

		return responseBody;
	}

	@RequestMapping(value = "/removeEventoContrato", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public Boolean removeEventoContrato(@RequestParam(value = "justificativaRemocao") String justificativaRemocao, @RequestParam(value = "idEvento") Long idEvento) {
		return this.contratoEventoService.removeEventoContrato(justificativaRemocao, idEvento);
	}


	@RequestMapping(value = "/buscaEventosContratoPorTipo", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper buscaEventosContratoPorTipo(@RequestParam(value = "contratoId") Long contratoId, @RequestParam(value = "tipoEvento") String tipoEvento) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(contratoEventoService.findAllByContratoTipoEvento(contratoId, tipoEvento), ViewsContrato.TermoEventoContrato.class);

		return responseBody;
	}

	@RequestMapping(method = RequestMethod.GET, value = "/buscaTermosContrato")
	@ResponseBody
	public ResponseBodyWrapper buscaTermosContrato(
			@RequestParam(value = "contratoId") Long contratoId,
			@RequestParam(value = "dataInicial", required = false) String dataInicio, @RequestParam(value = "dataFinal", required = false) String dataFim){

		List<ContratoEventosVH> lista = new ArrayList<ContratoEventosVH>();

		Date dataInicial = StringUtils.isNotBlank(dataInicio) ? UtilDate.jsonToDate(dataInicio) : null;
		Date dateFinal = StringUtils.isNotBlank(dataFim) ? UtilDate.jsonToDate(dataFim) : null;

		Collection<ContratoEvento> listaEventos = this.contratoEventoService.findAllByContratoIntervalDataEmissao(contratoId, dataInicial, dateFinal);

		ContratoEventosVH contratoEventosVH = new ContratoEventosVH();
		long idAuxDominioEvento = 0;
		int countListaEventos = 0;
		for (ContratoEvento contratoEvento : listaEventos) {

			//TERMOS PDF NAO IMPLEMENTADOS
			if(contratoEvento.getTipoEvento().getCodigo() == DominioContrato.TIPO_EVENTO_PENALIDADE ||  contratoEvento.getTipoEvento().getCodigo() == DominioContrato.TIPO_EVENTO_ATUALIZA_STATUS
					|| contratoEvento.getTipoEvento().getCodigo() == DominioContrato.TIPO_EVENTO_LIBERA_PAGAMENTO || contratoEvento.getTipoEvento().getCodigo() == DominioContrato.TIPO_EVENTO_NAO_CONFORMIDADE
					|| contratoEvento.getTipoEvento().getCodigo() == DominioContrato.TIPO_EVENTO_EMPENHO || contratoEvento.getTipoEvento().getCodigo() == DominioContrato.TIPO_EVENTO_REPACTUACAO){
				continue;
			}

			countListaEventos ++;
			if(idAuxDominioEvento == 0){
				contratoEventosVH.setTipoEvento(contratoEvento.getTipoEvento());
				contratoEventosVH.addEventos(contratoEvento);
				idAuxDominioEvento = contratoEvento.getTipoEvento().getId();
				continue;
			}

			if(idAuxDominioEvento == contratoEvento.getTipoEvento().getId()){
				contratoEventosVH.addEventos(contratoEvento);
			}else{
				lista.add(contratoEventosVH);
				contratoEventosVH = new ContratoEventosVH();
				contratoEventosVH.setTipoEvento(contratoEvento.getTipoEvento());
				contratoEventosVH.addEventos(contratoEvento);
				idAuxDominioEvento = contratoEvento.getTipoEvento().getId();
			}

			if(countListaEventos == listaEventos.size()){
				lista.add(contratoEventosVH);
				contratoEventosVH = new ContratoEventosVH();
			}

		}

		if(contratoEventosVH.getEventos() != null && !contratoEventosVH.getEventos().isEmpty()){
			lista.add(contratoEventosVH);
		}

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(lista, ViewsContrato.ContratoEventoEditView.class);

		return responseBody;
	}

	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, value = "/pdfGerarTermos")
	public void gerarGerarTermos(@RequestParam(value = "tiposTermo") String tiposTermoSelecionado,
			@RequestParam(value = "idContrato", required = false) Long idContrato,
			@RequestParam(value = "dataInicial", required = false) String dataInicio,
			@RequestParam(value = "dataFinal", required = false) String dataFim,
			@RequestParam(value = "download") boolean download, HttpServletResponse response) throws SQLException, JRException, IOException {

		Date dataInicial = StringUtils.isNotBlank(dataInicio) ? UtilDate.getData(UtilDate.dataInicioFormatada(dataInicio)) : null;
		Date dateFinal = StringUtils.isNotBlank(dataFim) ? UtilDate.getData(UtilDate.dataInicioFormatada(dataFim)) : null;
		try {
			this.contratoEventoService.gerarTermos(getArrayLong(tiposTermoSelecionado), idContrato, dataInicial, dateFinal, download, response);
		} catch (BusinessException e) {
			response.setStatus(HttpServletResponse.SC_PRECONDITION_FAILED);
		}
	}

	private Long[] getArrayLong(String idsEventos) {
		Long[] arrayLong = null;
        if(!UtilString.isNullOrEmpty(idsEventos)){
            String[] array = idsEventos.split(",");
            int leng = array.length;
            arrayLong = new Long[leng];
            for (int i = 0; i < leng; i++) {
                 arrayLong[i] = Long.parseLong(array[i]);
            }
      }
		return arrayLong;
	}

	@RequestMapping(method = RequestMethod.GET, value = "/listaIdEventosTermo")
	@ResponseBody
	public ResponseBodyWrapper listaIdEventosTermo( @RequestParam(value = "termos") String termos, @RequestParam(value = "dataInicial", required = false) String dataInicio, @RequestParam(value = "dataFinal", required = false) String dataFim){

		Long[] codigosTermo = getArrayLong(termos);
		Date dataInicial = dataInicio != null ? UtilDate.jsonToDate(dataInicio) : null;
		Date dateFinal = dataFim != null ? UtilDate.jsonToDate(dataFim) : null;
		List<Long> idsEventos = new ArrayList<Long>();
		Collection<ContratoEvento> eventos = this.contratoEventoService.findAllByContratoTipoTermoDataEmissao(codigosTermo, dataInicial, dateFinal);
		for (ContratoEvento contratoEvento : eventos) {
			idsEventos.add(contratoEvento.getId());
		}
		return new ResponseBodyWrapper(idsEventos, ViewsContrato.ContratoEventoEditView.class);

	}

	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, value = "/pdfGerarRelatorios")
	public void gerarRelatorios(
			@RequestParam(value = "idsTiposRelatorio") String idsRelatoriosSelecionados,
			@RequestParam(value = "idContrato", required = false) String idContrato,
			@RequestParam(value = "dataInicial", required = false) String dataInicial,
			@RequestParam(value = "dataFinal", required = false) String dataFinal,
			@RequestParam(value = "download") boolean download, HttpServletResponse response) throws SQLException, JRException, IOException {

		Long[] arrayLong = getArrayLong(idsRelatoriosSelecionados);

		Long contratoId = StringUtils.isNumeric(idContrato) ? Long.parseLong(idContrato) : null;
		Date dataInicio = StringUtils.isBlank(dataInicial) ? null : UtilDate.jsonToDate(UtilDate.dataInicioFormatada(dataInicial));
		Date dataFim = StringUtils.isBlank(dataFinal) ? null : UtilDate.jsonToDate(UtilDate.dataInicioFormatada(dataFinal));

		this.contratoEventoService.gerarRelatorios(arrayLong, contratoId, dataInicio, dataFim, download, response);
	}

}
