package br.com.centralit.controller;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.ContratoItem;
import br.com.centralit.api.service.ContratoItemService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;

@Controller
@RequestMapping("/rest/contratoItem")
public class ContratoItemController extends GenericController<ContratoItem>{

	private ContratoItemService contratoItemService;

	public ContratoItemController() {
		super();
	}

	@Autowired
	private DominioService dominioService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contratoItemService
	 */
	@Autowired
	private ContratoItemController( ContratoItemService contratoItemService ) {

		super(contratoItemService);

		this.contratoItemService = contratoItemService;

	}

	@RequestMapping(value = "/findAllByContrato", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper findAllByContrato(@RequestBody Long contratoId) {

		Collection<ContratoItem> itens = this.contratoItemService.findAllByContrato(contratoId);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(itens, ViewsContrato.ContratoItemAutoCompleteView.class);

		return responseBody;
	}
	
	/**
	 * Método responsável por listar os itens do contrato por nome
	 *
	 * @author luis.camargo
	 *
	 * @param nome
	 * @param idContrato
	 *
	 * @return Collection<ContratoItem>
	 */
	@RequestMapping(value = "/findAutoCompleteContratoItemByNomeAndContrato", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAutoCompleteContratoItemByNomeAndContrato(
			@RequestParam(value = "nome") String nome, @RequestParam(value = "idContrato") Long idContrato) {
		
		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.contratoItemService.findAutoCompleteContratoItemByNomeAndContrato(nome, idContrato), ViewsContrato.ContratoItemAutoCompleteView.class);
		
		return responseBody;
	}
	
}
