package br.com.centralit.controller;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.EncerramentoContrato;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EncerramentoContratoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilDataBase;

/**
 * 
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 24/02/2016 - 16:15:13
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Controller
@RequestMapping("/rest/encerramentoContrato")
public class EncerramentoContratoController extends GenericController<EncerramentoContrato> {

	/** Atributo encerramentoContratoService. */
	private EncerramentoContratoService encerramentoContratoService;
	
	@Autowired
	private InternacionalizacaoService internacionalizacaoService;
	
	@Autowired
	private DominioService dominioService;
	
	@Autowired
	private ReportController reportController;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contratoService
	 */
	@Autowired
	public EncerramentoContratoController( EncerramentoContratoService encerramentoContratoService ) {

		super(encerramentoContratoService);

		this.encerramentoContratoService = encerramentoContratoService;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.EncerramentoContratoListView> getListView() {

		return ViewsContrato.EncerramentoContratoListView.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.EncerramentoContratoEditView> getEditView() {

		return ViewsContrato.EncerramentoContratoEditView.class;
	}
	
	@RequestMapping(method = RequestMethod.GET, value = "/gerarTermoEncerramentoContrato")
	@ResponseBody
	public void generatePdfRMB(@RequestParam(value="idEncerramentoContrato") Long idEncerramentoContrato, @RequestParam(value="download") boolean download, HttpServletResponse response,
			HttpServletRequest request) throws SQLException, JRException, IOException {

		Dominio idioma = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);

		Usuario user = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;
		try {
			conn = reportController.getConnection();
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/encerramentoContrato.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);

			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("ENCERRAMENTO_CONTRATO_ID", idEncerramentoContrato);
			subReportParameters.put("REPORT_CONNECTION", conn);

			String titulo = this.internacionalizacaoService.getTranslate("LABEL.TITULO_ENCERRAMENTO_CONTRATO", idioma);

			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", user.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			JasperPrint jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);

			// Responsavel por verificar a presenca de dados no relatorio
			reportController.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		}  catch (Exception e) {
			throw new BusinessException();
		}finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}

	}

}
