package br.com.centralit.controller;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.EstudoTecnico;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EstudoTecnicoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilDataBase;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 05/01/2016 - 15:23:44
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Controller
@RequestMapping("/rest/estudoTecnico")
public class EstudoTecnicoController extends GenericController<EstudoTecnico>{

	/** Atributo estudoTecnicoService. */
	private EstudoTecnicoService estudoTecnicoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private OrganizacaoService organizacaoService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ReportController reportController;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param estudoTecnicoService
	 */
	@Autowired
	public EstudoTecnicoController(EstudoTecnicoService estudoTecnicoService) {

		super(estudoTecnicoService);

		this.estudoTecnicoService = estudoTecnicoService;
	}

	@RequestMapping(value = "/listarEstudosPorCodigo", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarEstudosPorCodigo(@RequestParam(value = "codigo") String codigo) {

		final Collection<EstudoTecnico> listaEstudos = this.estudoTecnicoService.listarEstudosPorCodigo(codigo);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaEstudos, ViewsContrato.TermoReferenciaEditView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar estudos tecnicos por codigo e organizacao
	 *
	 * @author iago.almeida
	 *
	 * @param codigo
	 * @param idOrganizacao
	 *
	 * @return Collection<EstudoTecnico>
	 */
	@RequestMapping(value = "/findAutoCompleteEstudoTecnicoPorCodigoAndOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAutoCompleteEstudoTecnicoPorCodigoAndOrganizacao(@RequestParam(value = "codigo") String codigo, @RequestParam(value = "idOrganizacao") Long idOrganizacao) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.estudoTecnicoService.findAutoCompleteEstudoTecnicoPorCodigoAndOrganizacao(codigo, idOrganizacao), ViewsContrato.EstudoTecnicoAutoCompleteView.class);

		return responseBody;
	}

	@Override
	public Class<ViewsContrato.EstudoTecnicoEditView> getEditView() {

		return ViewsContrato.EstudoTecnicoEditView.class;
	}

	@Override
	public Class<ViewsContrato.EstudoTecnicoListView> getListView() {

		return ViewsContrato.EstudoTecnicoListView.class;
	}

	@RequestMapping(method = RequestMethod.GET, value = "/gerarEstudoTecnico")
	@ResponseBody
	public void generatePdfRMB(@RequestParam(value="idEstudoTecnico") Long idEstudoTecnico, @RequestParam(value="download") boolean download, HttpServletResponse response,
			HttpServletRequest request) throws SQLException, JRException, IOException {

		Dominio idioma = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);

		Usuario user = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;
		InputStream inputSubSub = null;
		try {
			conn = reportController.getConnection();
			input = this.getClass().getResourceAsStream("/reports/templateRetrato.jrxml");
			inputSub = this.getClass().getResourceAsStream("/reports/estudoTecnico.jrxml");
			inputSubSub = this.getClass().getResourceAsStream("/reports/estudoTecnico_levantamentoAlternativas.jrxml");

			JasperDesign jasperDesign = JRXmlLoader.load(input);
			JasperDesign jasperDesignSub = JRXmlLoader.load(inputSub);
			JasperDesign jasperDesignSubSub = JRXmlLoader.load(inputSubSub);

			JasperReport jasperReport = JasperCompileManager.compileReport(jasperDesign);
			JasperReport jasperReportSub = JasperCompileManager.compileReport(jasperDesignSub);
			JasperReport jasperReportSubSub = JasperCompileManager.compileReport(jasperDesignSubSub);

			Map<String, Object> subSubReportParameters = new HashMap<String, Object>();
			subSubReportParameters.put("ESTUDO_TECNICO_ID", idEstudoTecnico);

			Map<String, Object> subReportParameters = new HashMap<String, Object>();
			subReportParameters.put("ESTUDO_TECNICO_ID", idEstudoTecnico);
			subReportParameters.put("SUBREPORT_DIR", jasperReportSubSub);
			subReportParameters.put("SUBREPORT_PARAMETERS_MAP", subSubReportParameters);
			subReportParameters.put("REPORT_CONNECTION", conn);
			subReportParameters.putAll(this.estudoTecnicoService.findEstudoTecnicoRecursoObservacao(idEstudoTecnico));

			String titulo = this.internacionalizacaoService.getTranslate("LABEL.TITULO_ESTUDO_TECNICO", idioma);

			// System.out.println("Creating JasperPrint Object");
			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("ORGANIZACAO_ID", user.getOrganizacao().getId());
			parameters.put("TITULO", titulo);
			parameters.put("SUBREPORT_DIR", jasperReportSub);
			parameters.put("SUBREPORT_PARAMETERS_MAP", subReportParameters);
			parameters.put("REPORT_CONNECTION", conn);

			JasperPrint jasperPrint = JasperFillManager.fillReport(jasperReport, parameters, conn);

			// Responsavel por verificar a presenca de dados no relatorio
			reportController.gerarRelatorio(download, "pdf", jasperPrint, response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		}  catch (Exception e) {
			throw new BusinessException();
		}finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}

	}

}
