package br.com.centralit.controller;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.EventoRepactuacao;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EventoRepactuacaoService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.controller.ReportController;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilDataBase;


/**
 *
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 19/03/2016 - 09:44:20
 *
 * @version 1.0.0
 *
 * @author geovane.filho
 *
 */
@Controller
@RequestMapping("/rest/eventoRepactuacao")
public class EventoRepactuacaoController extends GenericController<EventoRepactuacao> {

	private EventoRepactuacaoService eventoRepactuacaoService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ReportController reportController;

	public EventoRepactuacaoController() {
		super();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.EventoRepactuacaoEditView> getEditView() {

		return ViewsContrato.EventoRepactuacaoEditView.class;
	}

	@Autowired
	public EventoRepactuacaoController (EventoRepactuacaoService eventoRepactuacaoService) {

		super(eventoRepactuacaoService);
		this.eventoRepactuacaoService = (EventoRepactuacaoService) eventoRepactuacaoService;
	}

	@RequestMapping(method = RequestMethod.GET, value = "/gerarRepactuacao")
	@ResponseBody
	public void generatePdfRMB(@RequestParam(value="idRepactuacao") Long idRepactuacao, @RequestParam(value="download") boolean download, HttpServletResponse response,
			HttpServletRequest request) throws SQLException, JRException, IOException {

		Dominio idioma = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);

		Usuario user = (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal();

		Connection conn = null;
		ByteArrayOutputStream baos = null;
		InputStream input = null;
		InputStream inputSub = null;
		try {
			conn = reportController.getConnection();
			String titulo = this.internacionalizacaoService.getTranslate("CONTRATOS.LABEL.TITULO_RELATORIO_REPACTUACAO", idioma);

			// Responsavel por verificar a presenca de dados no relatorio
			reportController.gerarRelatorio(download, "pdf", this.eventoRepactuacaoService.getJasperPrintRepactuacao(idRepactuacao, conn, user, titulo), response, baos, titulo);

		} catch (JRException jre) {
			throw new RuntimeException(jre);
		}  catch (Exception e) {
			throw new BusinessException();
		}finally {
			UtilDataBase.closeResources(conn, input, inputSub, baos);
		}

	}

}
