package br.com.centralit.controller;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.ObjetivoEstrategico;
import br.com.centralit.api.service.ObjetivoEstrategicoService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.model.SearchParams;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.view.GridVH;
import br.com.centralit.framework.view.ResultResponseVH;

import com.googlecode.genericdao.search.SearchResult;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b> Monitor </p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">415</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 06/01/2015 - 09:49:58
 *
 * @version 1.0.0
 *
 * @author luis.camargo
 *
 */
@Controller
@RequestMapping("/rest/objetivoEstrategico")
public class ObjetivoEstrategicoController extends GenericController<ObjetivoEstrategico> {

	/** Atributo regraEventoService. */
	private ObjetivoEstrategicoService objetivoEstrategicoService;

	@Autowired
	private ObjetivoEstrategicoNecessidadeController objetivoEstrategicoNecessidadeController;

	@Autowired
	private UsuarioService usuarioService;


	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param objetivoEstrategicoService
	 */
	@Autowired
	public ObjetivoEstrategicoController(ObjetivoEstrategicoService objetivoEstrategicoService) {

		super(objetivoEstrategicoService);

		this.objetivoEstrategicoService = objetivoEstrategicoService;
	}


	@Override
	@RequestMapping(value = "/getPage", method = RequestMethod.POST)
	@ResponseBody
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public ResponseBodyWrapper findGrid(@RequestBody SearchParams searchParams) {

		SearchResult searchResult = this.objetivoEstrategicoService.findAllObjetivosEstrategico(searchParams.getFilters().get(0).getValue(),
				searchParams.getFilters().get(1).getValue(),
				searchParams.getKeywordValue(),
				searchParams.getLimit(),
				searchParams.getStartIntValue() == null || searchParams.getStartIntValue() == 0 ? 1 : searchParams.getStartIntValue());

		GridVH gridVH = new GridVH();
		gridVH.setObjects(searchResult.getResult());
		gridVH.addTotalItensTotalPages(searchParams, Long.valueOf(searchResult.getTotalCount()));

		ResultResponseVH resultResponseVH = new ResultResponseVH(gridVH);
		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, this.getListView());
		return responseBody;

	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por trazer todos os objetivos estratégicos da organizacao do usuário logado.
	 *
	 * @author luis.camargo
	 *
	 * @return
	 */
	@RequestMapping(value = "/findAllByOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAllByOrganizacao() {

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Long organizacaoId = usuarioLogado.getOrganizacao().getId();

		final Collection<ObjetivoEstrategico> objetivosEstrategicos = this.objetivoEstrategicoService.findAllByOrganizacao(organizacaoId);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(objetivosEstrategicos, ViewsContrato.ObjetivoEstrategicoListView.class);

		return responseBody;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.ObjetivoEstrategicoListView> getListView() {

		return ViewsContrato.ObjetivoEstrategicoListView.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.ObjetivoEstrategicoEditView> getEditView() {

		return ViewsContrato.ObjetivoEstrategicoEditView.class;
	}
}
