package br.com.centralit.controller;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.Questionario;
import br.com.centralit.api.service.QuestionarioService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.view.ResultResponseVH;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b> Monitor </p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">415</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 27/02/2015 - 14:45:58
 *
 * @version 1.0.0
 *
 * @author juliana.barbosa
 *	
 */
@Controller
@RequestMapping("/rest/questionario")
public class QuestionarioController extends GenericController<Questionario> {

	/** Atributo regraEventoService. */
	private QuestionarioService questionarioService;
	
	@Autowired
	private UsuarioService usuarioService;


	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param questionarioService
	 */
	@Autowired
	public QuestionarioController( QuestionarioService questionarioService ) {

		super(questionarioService);
		
		this.questionarioService = questionarioService;
	}


	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">415</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por
	 *
	 * @author juliana.barbosa
	 *
	 * @param nome
	 * @return
	 */
	@RequestMapping(value = "/listarQuestionarios", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarQuestionarios(@RequestParam(value = "nome") String nome) {

		final Collection<Questionario> listaQuestionarios = this.questionarioService.listarQuestionarios(nome);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaQuestionarios, Views.QuestionarioAutoCompleteView.class);

		return responseBody;
	}
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por trazer todos os questionarios da organizacao do usuário logado.
	 *
	 * @author geovane.filho
	 *
	 * @return
	 */
	@RequestMapping(value = "/findAllByOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAllByOrganizacao() {

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Long organizacaoId = usuarioLogado.getOrganizacao().getId();
		
		final Collection<Questionario> questionarios = this.questionarioService.findAllByOrganizacao(organizacaoId);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(questionarios, Views.QuestionarioAutoCompleteView.class);

		return responseBody;
	}
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por trazer todos os questionarios da organizacao do usuário logado.
	 *
	 * @author geovane.filho
	 *
	 * @return
	 */
	@RequestMapping(value = "/findAllByDominioTipoQuestionario", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAllByDominioTipoQuestionario(@RequestParam(value = "tipoQuestCodigo") Long tipoQuestCodigo) {
		
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Long organizacaoId = usuarioLogado.getOrganizacao().getId();
		
		final Collection<Questionario> questionarios = this.questionarioService.findAllByDominioTipoQuestionario(tipoQuestCodigo, organizacaoId);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(questionarios, Views.QuestionarioEditView.class);

		return responseBody;
	}
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por criar o questionario padrão utilizado no estudo técnico
	 *
	 * @author geovane.filho
	 *
	 * @param idOrganizacao
	 * @return
	 */
	@RequestMapping(value = "/createQuestionarioPadrao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper createQuestionarioPadrao() {

		ResultResponseVH<Questionario> resultResponseVH = new ResultResponseVH<Questionario>(this.questionarioService.createQuestionarioPadrao());

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, Views.QuestionarioEditView.class);

		return responseBody;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<Views.QuestionarioListView> getListView() {

		return Views.QuestionarioListView.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<Views.QuestionarioEditView> getEditView() {

		return Views.QuestionarioEditView.class;
	}	
}
