package br.com.centralit.controller;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.RegistroOcorrencia;
import br.com.centralit.api.service.RegistroOcorrenciaService;
import br.com.centralit.api.viewHelper.ViewsContrato;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governan&ccedil;a Corporativa - </p>
 *
 * <p><b>Description: </b></p>
 *
 * @since 16/02/2016 - 10:58:24
 * @version 1.0.0
 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
 *
 */
@Controller
@RequestMapping("/rest/registroOcorrencia")
public class RegistroOcorrenciaController extends GenericController<RegistroOcorrencia> {

	private RegistroOcorrenciaService registroOcorrenciaService;

	@Autowired
	public RegistroOcorrenciaController(RegistroOcorrenciaService registroOcorrenciaService ) {

		super(registroOcorrenciaService);
		this.registroOcorrenciaService = registroOcorrenciaService;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.RegistroOcorrenciaListView> getListView() {

		return ViewsContrato.RegistroOcorrenciaListView.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<ViewsContrato.RegistroOcorrenciaEditView> getEditView() {

		return ViewsContrato.RegistroOcorrenciaEditView.class;
	}

	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, value = "/pdfGerarRegistroOcorrencia")
	public void gerarPdfRegistroOcorrencia(@RequestParam(value = "registroOcorrenciaId") Long registroOcorrenciaId,
										@RequestParam(value = "download") boolean download, HttpServletResponse response)
										throws SQLException, JRException, IOException {

		registroOcorrenciaService.gerarPdfRegistroOcorrencia (registroOcorrenciaId, download, response);
	}


	@RequestMapping(value = "/findAllOcorrenciasPorContrato", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findAllOcorrenciasPorContrato(@RequestParam(value = "idContrato") Long idContrato) {

		final Collection<RegistroOcorrencia> listaOcorrencias = this.registroOcorrenciaService.findAllPorContrato(idContrato);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaOcorrencias, Views.GenericView.class);

		return responseBody;
	}

	@RequestMapping(value = "/getValorTotalGlosaOrdem", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public BigDecimal getValorTotalGlosaOrdem(@RequestParam(value = "ordemFornecimentoId") Long ordemFornecimentoId) {
		return this.registroOcorrenciaService.getValorTotalGlosaOrdem(ordemFornecimentoId);
	}

}
