'use strict';

citApp.controller('ConsultaRelatoriosController', ['$scope', '$filter', '$timeout', '$translate', 'ContratoRepository', 'DominioRepository', 'ContratoEventoRepository',
                                                   function VisaoContratoController($scope, $filter, $timeout, $translate, ContratoRepository, DominioRepository, ContratoEventoRepository) {

	$scope.contrato = null;
	$scope.dataInicial = '';
	$scope.dataFinal = '';
	$scope.tiposRelatorios = [];
	$scope.isTiposRelatorioSelecionado = false;

	$scope.findContrato = function (value) {
		return ContratoRepository.findContrato(value, $scope.usuarioLogado.organizacao.id).then(function (result) {
			return result;
		});
	};

	DominioRepository.findAllDominio('tipoEventoContrato').then(function (result) {
		angular.forEach(result, function(dominio, key) {
			if(dominio.codigo != 1 &&  dominio.codigo != 3 && dominio.codigo != 4 && dominio.codigo != 8 && dominio.codigo != 10){
				$scope.tiposRelatorios.push(dominio);
			}
		});
	});

	$scope.resetForm = function () {
		$scope.contrato = null;
		$timeout(function () {
			$scope.consultaRelatoriosForm.$submitted = false;
			$scope.consultaRelatoriosForm.$setPristine();
		});
	};

	// Seleciona todos os tipos de relatórios da lista
	$scope.checkAll = function(value) {
		angular.forEach($scope.tiposRelatorios, function (item, key) {
			item.$selected = value;
			item.$index = key;
		});
		$scope.atualizaTiposRelatorioSelecionado();
	};

	$scope.atualizaTiposRelatorioSelecionado = function() {
		$scope.isTiposRelatorioSelecionado = false;
		angular.forEach($scope.tiposRelatorios, function (item) {
			if (item.$selected) {
				$scope.isTiposRelatorioSelecionado = true;
			}
		});
	};

	$scope.gerar = function () {
		$scope.consultaRelatoriosForm.$submitted = true;

		if ($scope.dataInicial && $scope.dataFinal) {
			if (!isPeriodoValido($scope.dataInicial, $scope.dataFinal)){
				$scope.showAlert('error', $translate.instant('VALIDACAO.INTERVALO_DATAS_INVALIDO'), " ", false);
				return;
			}
		}
		if(!$scope.isTiposRelatorioSelecionado) {
			//Mensagem de erro de campos obrigatorios não preenchidos
			$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
			return;
		}

		var idsRelatoriosSelecionados = '';
		angular.forEach($scope.tiposRelatorios, function(relatorio) {
			if(relatorio.$selected){
				idsRelatoriosSelecionados += relatorio.id + ',';
			}
		});

		var contratoId = $scope.contrato ? $scope.contrato.id : null;
		var dataInicial = $scope.dataInicial == null ? '' : $scope.dataInicial;
		var dataFinal = $scope.dataFinal == null ? '' : $scope.dataFinal;
		$scope.url = '/cit-contratos-web/rest/contratoEvento/pdfGerarRelatorios?' +
					'idsTiposRelatorio='+ idsRelatoriosSelecionados +
					'&idContrato=' + contratoId +
					'&dataInicial=' + dataInicial +
					'&dataFinal=' + dataFinal;
		$scope.visualizarRelatorio($scope.url, $translate.instant('CONTRATOS.LABEL.RELATORIOS'));
	};

}]);