'use strict';

citApp.controller('ContratoEquipeGestoraListController', ['$scope', 'ContratoEquipeGestoraRepository', '$translate',
                                                          function ContratoEquipeGestoraListController($scope, ContratoEquipeGestoraRepository, $translate) {
    $scope.$showAdvancedFilters = false;

    $scope.totalPages = 10;
    $scope.totalItens = 0;
    $scope.limit = 10;

    $scope.resetForm = function() {
        angular.element("#editContratoEquipeGestora").scope().resetForm();
    };

    // default criteria that will be sent to the server
    $scope.filterCriteria = {
        start : 1,
        dir : 'asc',
        sort : 'id',
        limit : 10,
        fields: ['id', 'dataBloqueio' , 'pessoa', 'dominioPapelEquipeContratos', 'contrato'],
        filters : [ {type : 'string', field : 'pessoa.nome'} , {type : 'string', field : 'dominioPapelEquipeContratos.descricao'} , {type : 'string', field : 'contrato.nome'}]
    };

    // Will be called when filtering the grid, will reset the page number to one
    $scope.filterResult = function(generic) {
        if (generic == undefined) {
            $scope.filterCriteria.keywordValue = null;
        }
        $scope.filterCriteria.start = 1;
        $scope.fetchResult().then(function() {
            // The request fires correctly but sometimes the ui doesn't update,
            // that's a fix
            $scope.filterCriteria.start = 1;
        });
    };

    // call back function that we passed to our custom directive sortBy, will be
    // called when clicking on any field to sort
    $scope.onSort = function(sortedBy, sortDir) {
        $scope.filterCriteria.dir = sortDir;
        $scope.filterCriteria.sort = sortedBy;
        $scope.filterCriteria.start = 1;
        $scope.fetchResult().then(function() {
            // The request fires correctly but sometimes the ui doesn't update,
            // that's a fix
            $scope.filterCriteria.start = 1;
        });
    };

    // RECUPERA LISTA DE PAIS E SETA RESULTADO NA GRID
    $scope.fetchResult = function(page) {
        $scope.setLoadingPesquisa(true);

        return ContratoEquipeGestoraRepository.getListPage($scope.filterCriteria).then(function(result) {
            $scope.contratoEquipeGestoras = result.originalElement.objects;
            $scope.totalPages = result.originalElement.totalPages;
            $scope.totalItens =  result.originalElement.totalItens;

            $scope.setLoading(false);
        }, function() {
            $scope.contratoEquipeGestoras = [];
            $scope.totalPages = 0;
            $scope.totalItens = 0;
        });
    };

    // MODAL QUE CONFIRMA REMOVER DO CIDADE
    $scope.remove = function(){
        var contratoEquipeGestora = $scope.contratoEquipeGestoraChecked;

        if(contratoEquipeGestora === null) {
            $scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));

            return ;
        }

        angular.element('#editContratoEquipeGestora').scope().remove(contratoEquipeGestora);
    };

    // BUSCA LISTA DE CIDADE
    $scope.fetchResult();

    // ABRI CIDADE SELECIONADA
    $scope.abrirContratoEquipeGestora = function(edit){
        var contratoEquipeGestora = $scope.contratoEquipeGestoraChecked;

        if(!contratoEquipeGestora) {
            $scope.showAlert('warning', !edit ? $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO') : $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
            return;
        }else{
            angular.element('#editContratoEquipeGestora').scope().getContratoEquipeGestora(contratoEquipeGestora, edit);
            $scope.$showPageEditWorkspace($scope.workspace);
        }

        angular.element('#editContratoEquipeGestora').scope().getContratoEquipeGestora(contratoEquipeGestora, edit);
        $scope.$showPageEditWorkspace($scope.workspace);
    };
}]);
