'use strict';

citApp.controller('ContratoInfracaoController', ['$scope', 'ContratoInfracaoRepository', '$translate', '$timeout', 'ContratoRepository', 'PenalidadeRepository',
                                                 function ContratoInfracaoController($scope, ContratoInfracaoRepository, $translate, $timeout, ContratoRepository , PenalidadeRepository ) {

    $scope.contratoInfracao = {};

    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        $scope.limparContratoInfracao();
        $scope.edit = true;
        $timeout(function(){
            $scope.contratoInfracaoForm.$submitted = false;
            $scope.contratoInfracaoForm.$setPristine();
        });
    };

    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchContratoInfracao').scope().fetchResult();
    };

    // MODAL QUE CONFIRMA REMOVER DA CIDADE
    $scope.remove = function(contratoInfracao){
        $scope.contratoInfracao = contratoInfracao;
        $scope.$openModalConfirm({
            message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
            callback: function () {
                ContratoInfracaoRepository.remove($scope.contratoInfracao).then(function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                    angular.element('#searchContratoInfracao').scope().fetchResult();

                    $scope.resetForm();
                });
            }
        });
    };

    // SALVA O ContratoInfracao
    $scope.saveOrUpdate = function(){
        $scope.contratoInfracaoForm.$submitted = true;

        //verifica se o formulario está valido para salvar
        if($scope.contratoInfracaoForm.$valid){

            $scope.setLoadingSalva(true);

            ContratoInfracaoRepository.save($scope.contratoInfracao).then(function(result) {
                $scope.contratoInfracao = result.originalElement;
                $scope.showAlert("success", $translate.instant('MSG.REGISTRO_SALVO'));
                $scope.contratoInfracaoForm.$submitted = false;
            });
            $scope.setLoading(false);
        }else{
            //Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        }
    };

    // Limpa o formulario preenchido
    $scope.limparContratoInfracao = function(){
        $scope.contratoInfracao = {};
    };

    // Consulta entidade e mostra no formulario
    $scope.getContratoInfracao = function(contratoInfracao, edit){
        $scope.setLoadingGet(true);

        ContratoInfracaoRepository.get(contratoInfracao.id).then(function(result) {
            $scope.contratoInfracao = result.originalElement;
            $scope.edit = edit;
            $scope.setLoading(false);
        });
    };

    
    $scope.findAutoCompleteContrato = function(value){
        return ContratoRepository.findAutoComplete('id', value).then(function(result) {
            return result;
        });
    };
    
    $scope.findAutoCompletePenalidades = function(value){
        return PenalidadeRepository.findAutoComplete('id', value).then(function(result) {
            return result;
        });
    };
    
    $scope.setPenalidades = function(item){
        if(!$scope.contratoInfracao.penalidades){
            $scope.contratoInfracao.penalidades = [];
        }
        $scope.contratoInfracao.penalidades.push({
            penalidade : item
        });
        $timeout(function(){
            $scope.penalidade = null;
        });
    };

    $scope.removerPenalidades = function(){
        if($scope.penalidadeChecked){
            $scope.$openModalConfirm({message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'), callback: $scope.excluir, item: $scope.penalidadeChecked});
        }
    };

    $scope.excluir = function() {
        var obj = this.item;
        if (obj.id) {
            $scope.showAlertSucesso("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
        }
        $scope.contratoInfracao.penalidades.splice(obj.$index, 1);
        $scope.$modalConfirmInstance.dismiss('cancel');
    };      

}]);
