'use strict';

citApp.controller('ContratoListController', ['$scope', 'ContratoRepository', 'DominioRepository', '$translate', function ContratoListController($scope, ContratoRepository, DominioRepository, $translate) {

    $scope.$showAdvancedFilters = false;

    // ACESSA CONTROLLER DO FORMULARIO DE CADASTRO E RESETA FORMULARIO
    $scope.resetForm = function() {
        angular.element("#editContrato").scope().resetForm();
    };

    // DEFINE CABEÇALHOS DA LISTAGEM
    $scope.headers = [ { title : $translate.instant('LABEL.NUMERO_ANO_CONTRATO'), value : 'numeroAnoContrato', typeMask: 'MASK_CONTRATO', mask : [new RegExp(/(\d{3})(\d{1,4})$/), '$1/$2'], tamanho : 20 } ,
                       { title : $translate.instant('LABEL.TIPO_CONTRATO'), value : 'dominioTipoContrato.descricao', tamanho : 15 } ,
                       { title : $translate.instant('LABEL.DATA_VIGENCIA_INICIAL'), value : 'dataVigenciaInicial', filter : 'dateBR', tamanho : 20 },
                       { title : $translate.instant('LABEL.DATA_VIGENCIA_FINAL'), value : 'dataVigenciaFinal', filter : 'dateBR', tamanho : 20 },
                       { title : $translate.instant('LABEL.SITUACAO_ATUAL'), value : 'dominioSituacaoAtual.descricao', tamanho : 20 }];

    // DIFINE FILTROS DE PESQUISA E CAMPOS QUE DEVEM SER APRESENTADOS NA LISTAGEM
    $scope.filterCriteria = {
        start : 1,
        dir : 'asc',
        sort : 'id',
        limit : 10,
        fields: ['id', 'numeroAnoContrato', 'dominioTipoContrato.descricao', 'dataVigenciaInicial', 'dataVigenciaFinal', 'dominioSituacaoAtual.descricao'],
        filters : [ { type : 'string', field : 'numeroAnoContrato' } ,
                    { type : 'string', field : 'dominioTipoContrato.descricao', listaDominio : [] } ,
                    { type : 'date-range', field : 'dataVigenciaInicial' } ,
                    { type : 'date-range', field : 'dataVigenciaFinal' } ,
                    { type : 'string', field : 'dominioSituacaoAtual.descricao', listaDominio : []  }]
    };

	DominioRepository.findAllDominio('tipoContrato').then(function(result) {
		$scope.filterCriteria.filters[1].listaDominio = result;
	});

	DominioRepository.findAllDominio('tipoSituacaoContrato').then(function(result) {
		$scope.filterCriteria.filters[4].listaDominio = result;
	});

    // ABRI CONTRATO SELECIONADO PARA VISUALIZACAO OU EDICAO
    $scope.abrirVisualizar = function(edit){

        var contrato = $scope.contratoChecked;

        if(!contrato) {

            $scope.showAlert('warning', !edit ? $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO') : $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
            return;
        }

        angular.element('#editContrato').scope().getContrato(contrato.id, edit);
        $scope.$showPageEditWorkspace($scope.workspace);
    };

    $scope.gerarRelatorioTermoContrato = function() {

        if(!$scope.contratoChecked){
        	$scope.showAlert("warning", $translate.instant("MSG.SELECIONE_UM_ITEM_PARA_RELATORIO") + " ", false );
        } else{
        	$scope.url = '/cit-contratos-web/rest/contrato/gerarTermoContrato?idContrato='+ $scope.contratoChecked.id;
        	$scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.TITULO_CONTRATO"));
        }

    };

}]);