'use strict';

citApp.controller('ContratoTermoAditivoController', ['$scope', 'ContratoTermoAditivoRepository', '$translate', '$timeout',
                                                     function ContratoTermoAditivoController($scope, ContratoTermoAditivoRepository, $translate, $timeout) {

    $scope.contratoTermoAditivo = {};

    $scope.isBloquear = false;
    $scope.isDesbloquear = false;

    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        $scope.limparContratoTermoAditivo();
        $scope.edit = true;
        $scope.isBloquear = false;
        $scope.isDesbloquear = $scope.isBloquear;
        $timeout(function(){
            $scope.contratoTermoAditivoForm.$submitted = false;
            $scope.contratoTermoAditivoForm.$setPristine();
        });
    };

    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchContratoTermoAditivo').scope().fetchResult();
    };

    // Bloqueia a ContratoTermoAditivo
    $scope.confirmarBloqueioContratoTermoAditivo = function(){
        if($scope.contratoTermoAditivo.dataBloqueio && $scope.contratoTermoAditivo.dataBloqueio !== ""){
            ContratoTermoAditivoRepository.save($scope.contratoTermoAditivo).then(function(result) {
                $scope.contratoTermoAditivo = result.originalElement;

                $scope.isBloquear = false;
                $scope.isDesbloquear = !$scope.isBloquear;
                $scope.edit = false;
                $scope.showAlert("success", $translate.instant('MSG.SUCESSO_BLOQUEIO'));
            });
        }else{
            $scope.showAlert('error', $translate.instant('MSG.MG011'), " ", false);
        }
    };

    // Desbloqueia o ContratoTermoAditivo
    $scope.desbloquear = function() {
        $scope.setLoadingSalva(true);

        $scope.contratoTermoAditivo.dataBloqueio = null;

        ContratoTermoAditivoRepository.save($scope.contratoTermoAditivo).then(function(result){
            $scope.setLoadingSalva(false);

            $scope.contratoTermoAditivo = result.originalElement;
            $scope.edit = true;
            $scope.isDesbloquear = true;
            $scope.isBloquear = false;
            $scope.showAlert("success", $translate.instant('MSG.SUCESSO_DESBLOQUEIO'));
        });
    };

    // MODAL QUE CONFIRMA REMOVER DA CIDADE
    $scope.remove = function(contratoTermoAditivo){
        $scope.contratoTermoAditivo = contratoTermoAditivo;
        $scope.$openModalConfirm({
            message: $translate.instant('MSG.CONFIRMA_EXCLUIR'),
            callback: function () {
                ContratoTermoAditivoRepository.remove($scope.contratoTermoAditivo).then(function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');
                    $scope.showAlert("success", $translate.instant('MSG.SUCESSO_EXCLUIR'));
                    angular.element('#searchContratoTermoAditivo').scope().fetchResult();

                    $scope.resetForm();
                });
            }
        });
    };

    // SALVA O ContratoTermoAditivo
    $scope.saveOrUpdate = function(){
        $scope.contratoTermoAditivoForm.$submitted = true;

        //verifica se o formulario está valido para salvar
        if($scope.contratoTermoAditivoForm.$valid){

            $scope.setLoadingSalva(true);

            ContratoTermoAditivoRepository.save($scope.contratoTermoAditivo).then(function(result) {
                $scope.contratoTermoAditivo = result.originalElement;
                $scope.isBloquear = true;
                $scope.isDesbloquear = !$scope.isBloquear;
                $scope.showAlert("success", $translate.instant('MSG.SUCESSO_SALVAR'));
                $scope.contratoTermoAditivoForm.$submitted = false;
            });
            $scope.setLoading(false);
        }else{
            //Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        }
    };

    // Limpa o formulario preenchido
    $scope.limparContratoTermoAditivo = function(){
        $scope.contratoTermoAditivo = {};
    };

    // Consulta entidade e mostra no formulario
    $scope.getContratoTermoAditivo = function(contratoTermoAditivo, edit){
        $scope.setLoadingGet(true);

        ContratoTermoAditivoRepository.get(contratoTermoAditivo.id).then(function(result) {
            $scope.contratoTermoAditivo = result.originalElement;

            if($scope.contratoTermoAditivo.dataBloqueio === null || $scope.contratoTermoAditivo.dataBloqueio === undefined){
                $scope.isBloquear = true;
                $scope.isDesbloquear = !$scope.isBloquear;
                $scope.edit = edit;
            }else{
                $scope.isBloquear = false;
                $scope.isDesbloquear = !$scope.isBloquear;
                $scope.edit = false;
            }

            $scope.setLoading(false);
        });
    };   

    

}]);
