'use strict';

citApp.controller('ContratoTermoAditivoListController', ['$scope', 'ContratoTermoAditivoRepository', '$translate',
                                                         function ContratoTermoAditivoListController($scope, ContratoTermoAditivoRepository, $translate) {
    $scope.$showAdvancedFilters = false;

    $scope.totalPages = 10;
    $scope.totalItens = 0;
    $scope.limit = 10;

    $scope.resetForm = function() {
        angular.element("#editContratoTermoAditivo").scope().resetForm();
    };

    // default criteria that will be sent to the server
    $scope.filterCriteria = {
        start : 1,
        dir : 'asc',
        sort : 'id',
        limit : 10,
        fields: ['id', 'dataBloqueio' , 'objetoContratual', 'fornecedor', 'valor', 'dataVigenciaInicial', 'dataVigenciaFinal', 'garantias'],
        filters : [ {type : 'string', field : 'objetoContratual'} , {type : 'string', field : 'fornecedor.nome'} , {type : 'money', field : 'valor'} , {type : 'date', field : 'dataVigenciaInicial'} , {type : 'date', field : 'dataVigenciaFinal'} , {type : 'string', field : 'garantias.nome'}]
    };

    // Will be called when filtering the grid, will reset the page number to one
    $scope.filterResult = function(generic) {
        if (generic == undefined) {
            $scope.filterCriteria.keywordValue = null;
        }
        $scope.filterCriteria.start = 1;
        $scope.fetchResult().then(function() {
            // The request fires correctly but sometimes the ui doesn't update,
            // that's a fix
            $scope.filterCriteria.start = 1;
        });
    };

    // call back function that we passed to our custom directive sortBy, will be
    // called when clicking on any field to sort
    $scope.onSort = function(sortedBy, sortDir) {
        $scope.filterCriteria.dir = sortDir;
        $scope.filterCriteria.sort = sortedBy;
        $scope.filterCriteria.start = 1;
        $scope.fetchResult().then(function() {
            // The request fires correctly but sometimes the ui doesn't update,
            // that's a fix
            $scope.filterCriteria.start = 1;
        });
    };

    // RECUPERA LISTA DE PAIS E SETA RESULTADO NA GRID
    $scope.fetchResult = function(page) {
        $scope.setLoadingPesquisa(true);

        return ContratoTermoAditivoRepository.getListPage($scope.filterCriteria).then(function(result) {
            $scope.contratoTermoAditivos = result.originalElement.objects;
            $scope.totalPages = result.originalElement.totalPages;
            $scope.totalItens =  result.originalElement.totalItens;

            $scope.setLoading(false);
        }, function() {
            $scope.contratoTermoAditivos = [];
            $scope.totalPages = 0;
            $scope.totalItens = 0;
        });
    };

    // MODAL QUE CONFIRMA REMOVER DO CIDADE
    $scope.remove = function(){
        var contratoTermoAditivo = $scope.contratoTermoAditivoChecked;

        if(contratoTermoAditivo === null) {
            $scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));

            return ;
        }

        angular.element('#editContratoTermoAditivo').scope().remove(contratoTermoAditivo);
    };

    // BUSCA LISTA DE CIDADE
    $scope.fetchResult();

    // ABRI CIDADE SELECIONADA
    $scope.abrirContratoTermoAditivo = function(edit){
        var contratoTermoAditivo = $scope.contratoTermoAditivoChecked;

        if(!contratoTermoAditivo) {
            $scope.showAlert('warning', !edit ? $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO') : $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
            return;
        }else{
            angular.element('#editContratoTermoAditivo').scope().getContratoTermoAditivo(contratoTermoAditivo, edit);
            $scope.$showPageEditWorkspace($scope.workspace);
        }

        angular.element('#editContratoTermoAditivo').scope().getContratoTermoAditivo(contratoTermoAditivo, edit);
        $scope.$showPageEditWorkspace($scope.workspace);
    };
}]);
