'use strict';

citApp.controller('ContratosFilterController', ['$scope', 'ContratoRepository', 'DominioRepository', 'PessoaRepository', '$translate', '$timeout',
                                                function ContratosFilterController($scope, ContratoRepository, DominioRepository, PessoaRepository, $translate, $timeout) {
	
	$scope.filtros = {
			start : 1,
			limit : 10,
			tiposSituacao : []
	};
	
	DominioRepository.findAllDominio('tipoSituacaoContrato').then(function(result) {
		$scope.tiposSituacao = result;
	});
	
	// Seleciona todas as situações na lista
	$scope.checkAllSiuacoes = function(value) {
		angular.forEach($scope.tiposSituacao, function (item, key) {
			item.$selected = value;
			$scope.atualizarFiltroSituacao(item);
        });
	};
	
	$scope.findGestor = function (value){//Procura colaboradores a partir do nome para selecionar como gestor
		return PessoaRepository.findColaboradorPorNomeAndOrganizacao(value, $scope.usuarioLogado.organizacao.id).then(function(result) {
			   return result;
		   });
	};
	
	$scope.atualizarFiltroSituacao = function(situacao) {
		if (situacao.$selected) {
			$scope.filtros.tiposSituacao.push(situacao.originalElement);
		} else {
			angular.forEach($scope.filtros.tiposSituacao, function (item, idx) {
				if (item.id == situacao.id) {
					$scope.filtros.tiposSituacao.splice(idx, 1);
				}
	        });
		}
	};
	
	// Limpa formulário para novo cadastro
	$scope.resetForm = function() {
		$scope.limparQuestionario();
		$scope.edit = true;
		$scope.pgEdit = true;
		$scope.isBloquear = false;
		$scope.isDesbloquear = $scope.isBloquear;
		$timeout(function(){
			$scope.questionarioForm.$submitted = false;
			$scope.questionarioForm.$setPristine();
		});
	};
	
	// Gera o relatório de contratos de acordo com os filtros
    $scope.geraRelatorio = function() {
        $scope.setLoadingPesquisa(true);

        return ContratoRepository.gerarRelatorio($scope.filtros).then(function(result) {
            $scope.contratos = result.originalElement.objects;
            $scope.totalPages = result.originalElement.totalPages;
            $scope.totalItens =  result.originalElement.totalItens;

            $scope.setLoading(false);
        }, function() {
            $scope.contratos = [];
            $scope.totalPages = 0;
            $scope.totalItens = 0;
        });
    };
    
    $scope.getRelatorio = function() {
		ContratoRepository.getRelatorio($scope.filtros);
	};
    
    $scope.imprimirRelatorio = function(){
    	$timeout(function(){
			$scope.setLoadingSalva(true);
			
			var info = getAcrobatInfo();
			// console.log(info.browser+ " " + info.acrobat + " " + info.acrobatVersion);
	
			var situacoes = "?situacoes=";
			if ($scope.filtros.tiposSituacao) {
				angular.forEach($scope.filtros.tiposSituacao, function (item) {
					situacoes += ";" + item.id;
		        });
			}
			
			var apostilamento = "&apostilamento=";
			if ($scope.filtros.apostilamento) {
				apostilamento += "true";
			} else {
				apostilamento += "false";
			}
			
			var gestorId = "&gestor=";
			if ($scope.filtros.gestor) {
				gestorId += $scope.filtros.gestor.id; 
			}
			
			var vigInicial = "&vigInicial=";
			if ($scope.filtros.vigInicial) {
				vigInicial += $scope.filtros.vigInicial;
			}
			
			var vigFinal = "&vigFinal=";
			if ($scope.filtros.vigFinal) {
				vigFinal += $scope.filtros.vigFinal;
			}
			
			$scope.url = '/cit-contratos-web/rest/contrato/imprimirRelatorio' + situacoes + apostilamento + gestorId + vigInicial + vigFinal;
	
			// SE EXISTIR ACROBAT INSTALADO ABRE A MODAL COM O REPORT
			if(info.acrobat){
				//$scope.url += '&download=false';
				$scope.$openModal('rmbModal', 'lg');
			}else{
				$scope.url += '&download=true';
				window.location = $scope.url;
			}
			
			$scope.setLoading(false);
    	});
	};
	
	// Limpa o formulario preenchido
	$scope.limparQuestionario = function(){
		$scope.questionario = {
				questoes : []
		};
	};
}]);


