'use strict';
citApp.controller('CronogramaExecucaoController', ['$scope', '$translate', '$timeout', '$filter', 'FileUploader',
                                                   'CronogramaExecucaoRepository', 'ContratoRepository', 'ContratoItemRepository', 'CronogramaExecucaoItemRepository',
                                             function CronogramaExecucaoController($scope, $translate, $timeout, $filter, FileUploader,
                                            		 CronogramaExecucaoRepository, ContratoRepository, ContratoItemRepository, CronogramaExecucaoItemRepository) {

	$scope.cronogramaExecucao = {
			itensCronogramaExecucao : []
	};

	$scope.itemCronogramaExecucao = {
			parcelas : []
	};

	$scope.parcelaItem = {};
	$scope.contratoItem = {};

	$scope.radioValueList = [{descricao : $translate.instant('LABEL.SIM'), valor : true},
	                         {descricao : $translate.instant('LABEL.NAO'), valor : false}];

	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 99,
        url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	// Limpa formulário para novo cadastro
	$scope.resetForm = function() {
		$scope.limparCronogramaExecucao();
		$scope.itemCronogramaExecucao = {
				parcelas : []
		};
		$scope.parcelaItem = {};
		$scope.contratoItem = {};

		$scope.edit = true;
		$scope.pgEdit = true;
		$scope.atualizarListaSizeSelect();
		$timeout(function(){
			limparCronogramaExecucaoForm();
		});
	};

	$scope.resetFormItens = function() {
		limparItemCronogramaExecucao();
		$scope.parcelaItem = {};
		$scope.contratoItem = {};
	};

	function limparCronogramaExecucaoForm() {
		$scope.cronogramaExecucaoForm.$submitted = false;
		$scope.cronogramaExecucaoForm.$setPristine();
	}

	// Atualiza pagina de pesquisa
	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#searchCronogramaExecucao').scope().fetchResult();
	};

	// Consulta entidade e mostra no formulario
	$scope.getCronogramaExecucao = function(cronogramaExecucao, edit) {
		$scope.setLoadingGet(true);

		CronogramaExecucaoRepository.get(cronogramaExecucao.id).then(function(result) {
			$scope.cronogramaExecucao = result.originalElement;

			calcularValorTotalCronograma();

			if($scope.cronogramaExecucao.dataBloqueio === null || $scope.cronogramaExecucao.dataBloqueio === undefined) {
				$scope.edit = edit;
			} else {
				$scope.edit = false;
			}
			$scope.atualizarListaSizeSelect();
			$scope.pgEdit = edit;
			$scope.setLoading(false);
		});
	};

	function calcularValorTotalCronograma() {
		angular.forEach($scope.cronogramaExecucao.itensCronogramaExecucao, function(item) {
			item.valorTotalItem = 0.0;
			angular.forEach(item.parcelas, function(parcela) {
				item.valorTotalItem += (parcela.valorUnitario * parcela.quantidade);
			});
		});
	};

    $scope.findContrato = function(value) {
        return ContratoRepository.findAutoCompleteContratoByNumeroAndOrganizacao(value, $scope.usuarioLogado.organizacao.id).then(function(result) {
            return result;
        });
    };

    $scope.findContratoItem = function(value) {
    	return ContratoItemRepository.findAutoCompleteContratoItemByNomeAndContrato(value, $scope.cronogramaExecucao.contrato.id).then(function(result) {
    		return result;
    	});
    };

	$scope.gerarRelatorioCronograma = function() {
		$scope.url = '/cit-contratos-web/rest/cronogramaExecucao/gerarCronograma?idCronograma='+ $scope.cronogramaExecucao.id;
		$scope.visualizarRelatorio($scope.url, $translate.instant("CONTRATOS.LABEL.CRONOGRAMA_EXECUCAO"));
	};

    $scope.openModalItensCronograma = function() {

    	if ($scope.cronogramaExecucao.contrato) {
    		$scope.editItemCronograma = true;
    		$scope.showButtonSalvarItem = false;

    		limparItemCronogramaExecucao();

    		$timeout(function() {
    			$scope.$openModal('modal-itens-cronograma.html', 'lg');
    		});
    	} else {
    		$scope.showAlert('error', $translate.instant('CONTRATOS.VALIDACAO.NECESSARIO_CONTRATO_INFO_CRONOGRAMA'), " ", false);
    	}
    };

	function limparItemCronogramaExecucao() {
		$scope.itemCronogramaExecucao = {
				parcelas : []
		};
	};

	// MODAL QUE CONFIRMA REMOÇÃO DO CRONOGRAMA DE EXECUÇÃO
	$scope.remove = function(cronogramaExecucao) {
		$scope.$openModalConfirm({
			message: $translate.instant('CONTRATOS.MSG.CONFIRMA_EXCLUSAO_CRONOGRAMA'),
			callback: function () {
				CronogramaExecucaoRepository.remove(cronogramaExecucao).then(function() {
					$scope.limparCronogramaExecucao();

					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.showAlert("success", $translate.instant('CONTRATOS.MSG.SUCESSO_EXCLUSAO_CRONOGRAMA'));
					angular.element('#searchCronogramaExecucao').scope().fetchResult();

					$scope.resetForm();
				});
			}
		});
	};

	// SALVA O CRONOGRAMA DE EXECUÇÃO
	$scope.saveOrUpdate = function(){
		$scope.cronogramaExecucaoForm.$submitted = true;

		try{
			//verifica se o formulario está valido para salvar
			if($scope.cronogramaExecucaoForm.$valid){

				$scope.setLoadingSalva(true);

				var cronogramaExecucaoTemp = {
					documentos: []
				};

				// preencher lista de documentos temporarios para verificacao dos anexos. verificar qual anexo e de qual documento pelo numero do documento
				angular.forEach($scope.cronogramaExecucao.documentos, function(documento, key) {
					if (documento.uploadsDocumento) {
						documento.uuid = getUuid();
						cronogramaExecucaoTemp.documentos.push({
							file: documento.uploadsDocumento[0],
							uuid: clone(documento.uuid)
						});
						// deleta esse atributo por causa de um erro de ciclo no json
						delete documento.uploadsDocumento;
					}
				});

				CronogramaExecucaoRepository.save($scope.cronogramaExecucao).then(function(result) {
					$scope.cronogramaExecucao = result.originalElement;
					$scope.showAlert("success", $translate.instant('CONTRATOS.MSG.SUCESSO_SALVAR_CRONOGRAMA'));

		    		// depois de salvar a baixa ja tendo uma referencia do id da baixa, esse metodo ira salvar os documentos se existir
		    		salvarListaDocumentos(cronogramaExecucaoTemp);

					$scope.cronogramaExecucaoForm.$submitted = false;
					$scope.$showPageSearchWorkspace($scope.workspace);
					$scope.atualizaPaginaPesquisa();
				});
				$scope.setLoading(false);
			}else{
				//Mensagem de erro de campos obrigatorios não preenchidos
				$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
			}
		}catch(exception){
			// se der algum erro no save seta o loading false
			$scope.setLoadingSalva(false);
		}
	};

    // metodo utilizado para percorrer a lista de documentos persistidos e verificar com a lista de documentos temporarios para verificacao de qual anexo pertence a qual documento
    var salvarListaDocumentos = function(cronogramaExecucaoTemp) {
    	$scope.uploader.queue = [];
    	
        angular.forEach($scope.cronogramaExecucao.documentos, function(documentoSalvo, key) {
            // verifica se a entidade documento foi salva
            if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
                // percorre todos os documentos temporarios para verificacao do numero do documento
                angular.forEach(cronogramaExecucaoTemp.documentos, function(documentoTransient, key) {
                    // verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
                    if (documentoTransient.file && documentoSalvo.uuid === documentoTransient.uuid) {
                        //salva o anexo ao documento que tiver o mesmo numero do documento transient
                    	
                    	documentoTransient.file.url = documentoTransient.file.url + documentoSalvo.id;
                    	$scope.uploader.queue.push(documentoTransient.file);
                    }
                });
            }
        });
        
        if($scope.uploader.queue.length > 0){
        	$scope.uploader.uploadAll();
        }
        delete $scope.cronogramaExecucaoTemp;
    };

	// Limpa o formulario preenchido
	$scope.limparCronogramaExecucao = function(){
		$scope.cronogramaExecucao = {
				itensCronogramaExecucao : []
		};
	};

	// Seleciona todos as itensCronogramaExecucao na lista
	$scope.checkAll = function(value) {
		angular.forEach($scope.cronogramaExecucao.itensCronogramaExecucao, function (item, key) {
			item.$checked = value;
			item.$index = key;
        });
		$scope.atualizarListaSizeSelect();
	};

	// Abre modal perguntando se deseja excluir e exclui os elementos selecionados na lista de acoes caso confirmado
	$scope.excluirItensCronogramaSelecionados = function(){
		var lista = recuperarItensSelecionadas();
		if(lista.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {

					angular.forEach($scope.cronogramaExecucao.itensCronogramaExecucao, function (item, idx) {
						if (item.$checked) {
							$scope.cronogramaExecucao.itensCronogramaExecucao.splice(idx, 1);
						}
					});
					$scope.atualizarListaSizeSelect();
					$scope.$modalConfirmInstance.dismiss('cancel');
				}
			});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	$scope.editItemCronogramaExecucao = function(edit) {

		limparItemCronogramaExecucao();
		$scope.editItemCronograma = edit;
		$scope.showButtonSalvarItem = edit;

		var lista = recuperarItensSelecionadas();
		if (lista.length === 0) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}
		if (lista.length > 1) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_APENAS_UM_ITEM_PARA_EDICAO'));
			return;
		}

		$scope.itemCronogramaExecucao.parcelas = clone(lista[0].parcelas);
		angular.forEach($scope.itemCronogramaExecucao.parcelas, function(parcela, key) {
			$scope.itemCronogramaExecucao.parcelas[key].dataEntrega = $filter('date')(parcela.dataEntrega, "dd/MM/yyyy");
		});
		$scope.itemCronogramaExecucao.$index = clone(lista[0].$index);
		$scope.contratoItem = clone(lista[0].contratoItem);

		$timeout(function() {
			$scope.$openModal('modal-itens-cronograma.html', 'lg');
		});

	};

	//Recupera todas as acoes selecionadas na lista
	var recuperarItensSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.cronogramaExecucao.itensCronogramaExecucao, function (item) {
			if(item.$checked){
				lista.push(item);
			}
        });
		return lista;
	};

    // ####################################################################################################################### \\
    // ################## DAQUI PARA BAIXO SAO TODOS OS METODOS RELACIONADOS À MODAL DE ITENS DO CRONOGRAMA ################## \\
    // ####################################################################################################################### \\

	$scope.setDadosContratoItem = function (contratoItem) {
		$scope.contratoItem = contratoItem;

		$scope.parcelaItem.quantidade = contratoItem.quantidade;
		$scope.parcelaItem.valorUnitario = contratoItem.valorUnitarioItem;
	};

	$scope.selecionaRadioPagamentoParcelado = function() {
		limparParcelasItemCronogramaExecucao();

		if ($scope.itemCronogramaExecucao.pagamentoParcelado === false) {
			$scope.itemCronogramaExecucao.numeroParcelas = null;
			$scope.parcelaItem.quantidade = $scope.contratoItem.quantidade;
		}
	};

	function limparParcelasItemCronogramaExecucao() {
		$scope.itemCronogramaExecucao.parcelas = [];
	};

	function limparItemCronogramaExecucaoForm(formDialogItemCronogramaExecucao) {
		var parcelas = angular.copy($scope.itemCronogramaExecucao.parcelas);
		var index =  angular.copy($scope.itemCronogramaExecucao.$index);
		limparItemCronogramaExecucao();
		$scope.parcelaItem = {};

		$scope.itemCronogramaExecucao.parcelas = parcelas;
		$scope.itemCronogramaExecucao.$index = index;

		formDialogItemCronogramaExecucao.$submitted = false;
		formDialogItemCronogramaExecucao.$setPristine();
	}

	$scope.addParcela = function(formDialogItemCronogramaExecucao) {
		formDialogItemCronogramaExecucao.$submitted = true;

		if (formDialogItemCronogramaExecucao.$valid) {

			if ($scope.parcelaItem.$index != undefined && $scope.parcelaItem.$index != null) {
				$scope.itemCronogramaExecucao.parcelas[$scope.parcelaItem.$index] = angular.copy($scope.parcelaItem);
			} else {

				if ($scope.itemCronogramaExecucao.pagamentoParcelado === true && $scope.itemCronogramaExecucao.numeroParcelas) {
					var numeroParcelas = Number($scope.itemCronogramaExecucao.numeroParcelas);
					for (var i=0; i < numeroParcelas; i++) {
						var dataEntrega = angular.copy(converterStringEmDate($scope.parcelaItem.dataEntrega));
						if (i == 0) {
							$scope.parcelaItem.dataEntrega = $filter('date')(dataEntrega, "dd/MM/yyyy");
						} else {
							$scope.parcelaItem.dataEntrega = $filter('date')(dataEntrega.setMonth(dataEntrega.getMonth() + 1), "dd/MM/yyyy");
						}
						adicionarParcelaItemLista(i+1);
					}
				} else {
					limparParcelasItemCronogramaExecucao();
					adicionarParcelaItemLista(1);
				}
			}

			angular.forEach($scope.itemCronogramaExecucao.parcelas, function(parcela, key) {
				$scope.itemCronogramaExecucao.parcelas[key].dataEntrega = $filter('date')(parcela.dataEntrega, "dd/MM/yyyy");
			});

			limparItemCronogramaExecucaoForm(formDialogItemCronogramaExecucao);
		} else {
			$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
		}

	};

	function adicionarParcelaItemLista(sequencia) {

		$scope.parcelaItem.valorTotalParcela = 0.0;
		$scope.parcelaItem.valorTotalParcela = ($scope.parcelaItem.quantidade * $scope.parcelaItem.valorUnitario);

		$scope.parcelaItem.sequencia = sequencia;
		$scope.parcelaItem.dataEntrega = $filter('date')(clone($scope.parcelaItem.dataEntrega), "dd/MM/yyyy");

		$scope.itemCronogramaExecucao.parcelas.push(angular.copy($scope.parcelaItem));
	}

	$scope.atualizaValorTotalParcela = function (index) {
		var quantidade = $scope.itemCronogramaExecucao.parcelas[index].quantidade;
		var valorUnitario = $scope.itemCronogramaExecucao.parcelas[index].valorUnitario;
		$scope.itemCronogramaExecucao.parcelas[index].valorTotalParcela = (quantidade * valorUnitario);

		calcularValorTotalItemDialog();
	};

	$scope.editarParcelaItemCronogramaExecucao = function () {

		var lista = recuperarItensSelecionadas();

		if (lista.length > 0) {
			$scope.itemCronogramaExecucao.contratoItem = clone(lista[0].contratoItem);
			$scope.itemCronogramaExecucao.pagamentoParcelado = clone(lista[0].pagamentoParcelado);
		} else {
			$scope.itemCronogramaExecucao.contratoItem = $scope.contratoItem;
			$scope.itemCronogramaExecucao.pagamentoParcelado = $scope.itemCronogramaExecucao.parcelas > 1;
		}

		$scope.itemCronogramaExecucao.numeroParcelas = $scope.itemCronogramaExecucao.parcelas.length;

		var parcelaChecked = $scope.itemCronogramaExecucao.parcelas.filter(function(parcela) {
			return parcela.$checked === true;
		});

		$scope.parcelaItem = clone(parcelaChecked[0]);
		$scope.parcelaItem.dataEntrega = $filter('date')($scope.parcelaItem.dataEntrega, "dd/MM/yyyy");
	};

    $scope.removerParcelaItemCronogramaExecucao = function (){

    	var parcelaChecked = $scope.itemCronogramaExecucao.parcelas.filter(function(parcela) {
    		return parcela.$checked === true;
    	});

    	if (parcelaChecked.length > 0){

    		$scope.itemCronogramaExecucao.parcelas = $scope.itemCronogramaExecucao.parcelas.filter(function(parcela) {
				return parcela.$checked !== true;
			});

    		if ($scope.itemCronogramaExecucao.parcelas.length > 0){
    			$scope.itemCronogramaExecucao.numeroParcelas = $scope.itemCronogramaExecucao.parcelas.length;

    		} else {
    			$scope.itemCronogramaExecucao.numeroParcelas = null;
    		}
    	} else {
    		$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_ITEM_EXCLUIR'));
    		return;
    	}
    };

    $scope.addItemCronogramaExecucao = function(formDialogItemCronogramaExecucao, continuar) {

    	var itemCronogramaExecucao = angular.copy($scope.itemCronogramaExecucao);

    	if (itemCronogramaExecucao.parcelas.length > 0) {

    		var hasInconsistencia = false;
    		var textParcelasComEntregaForaVigencia = "";

    		angular.forEach(itemCronogramaExecucao.parcelas, function(parcela) {

    			var hasErrorVigenciaInicial = isPeriodoValido(parcela.dataEntrega, $scope.cronogramaExecucao.contrato.dataVigenciaInicial);
    			var hasErrorVigenciaFinal = !isPeriodoValido(parcela.dataEntrega, $scope.cronogramaExecucao.contrato.dataUltimaVigenciaFinal);
    			if (hasErrorVigenciaInicial || hasErrorVigenciaFinal) {

    				if (hasInconsistencia) {
    					textParcelasComEntregaForaVigencia += ", " + parcela.sequencia.toString();
            		} else {
            			textParcelasComEntregaForaVigencia += parcela.sequencia.toString();
            			hasInconsistencia = true;
            		}
    			}
    		});

    		if (hasInconsistencia) {

    			$scope.showAlert('error', $translate.instant('CONTRATOS.VALIDACAO.DATA_ENTREGA_CRONOGRAMA_FORA_VIGENCIA_CONTRATO') + textParcelasComEntregaForaVigencia + ".");
    		} else {

    			itemCronogramaExecucao.contratoItem = $scope.contratoItem;
    			itemCronogramaExecucao.pagamentoParcelado = itemCronogramaExecucao.parcelas.length > 1;
    			if (itemCronogramaExecucao.$index != undefined && itemCronogramaExecucao.$index != null) {
    				$scope.cronogramaExecucao.itensCronogramaExecucao[itemCronogramaExecucao.$index] = itemCronogramaExecucao;
    			} else {
    				$scope.cronogramaExecucao.itensCronogramaExecucao.push(itemCronogramaExecucao);
    			}

    			limparItemCronogramaExecucao();
    			if (continuar) {
    				limparItemCronogramaExecucaoForm(formDialogItemCronogramaExecucao);
    			} else {
    				$scope.parcelaItem = {};
    				$scope.contratoItem = {};

    				$scope.$modalInstance.dismiss('cancel');
    			}
    			$scope.atualizarListaSizeSelect();
    		}
    	} else {
    		$scope.showAlert('error', $translate.instant('CONTRATOS.VALIDACAO.NECESSARIO_ITEM_CRONOGRAMA'), " ", false);
    	}
    };


    // ####################################################################################################################### \\
    // ##################### DAQUI PARA BAIXO SAO TODOS OS METODOS RELACIONADOS À WATCH DE CAMPOS DA TELA #################### \\
    // ####################################################################################################################### \\

    // watch para calcular valor total das parcelas do item do cronograma
    $scope.$watchCollection('itemCronogramaExecucao.parcelas', function() {
    	calcularValorTotalItemDialog();
    });

    function calcularValorTotalItemDialog() {
    	$scope.itemCronogramaExecucao.valorTotalItem = 0.0;
    	angular.forEach($scope.itemCronogramaExecucao.parcelas, function(parcela) {
    		$scope.itemCronogramaExecucao.valorTotalItem += (parcela.valorUnitario * parcela.quantidade);
    	});
    };

    // watch para calcular valor total dos itens do cronograma
    $scope.$watchCollection('cronogramaExecucao.itensCronogramaExecucao', function() {
    	$scope.cronogramaExecucao.valorTotal = 0.0;
    	angular.forEach($scope.cronogramaExecucao.itensCronogramaExecucao, function(item) {
    		$scope.cronogramaExecucao.valorTotal += item.valorTotalItem;
    	});
    });

	$scope.atualizarListaSizeSelect = function() {
		if(!$scope.listaSize){
			$scope.listaSize = 0;
		};
		var lista = recuperarItensSelecionadas();
		$scope.listaSize = lista.length;
	};

}]);