'use strict';
citApp.controller('CronogramaExecucaoListController', ['$scope', '$translate', '$timeout', 'CronogramaExecucaoRepository',
          function CronogramaExecucaoListController($scope, $translate, $timeout, CronogramaExecucaoRepository) {

    $scope.resetForm = function() {
        angular.element("#editCronogramaExecucao").scope().resetForm();
    };

	$scope.headers = [
	{
		value : 'codigo',
		title : $translate.instant('CONTRATOS.LABEL.CODIGO_CRONOGRAMA'),
		tamanho : 25
	},{
		value : 'contrato.numeroAnoContrato',
		title : $translate.instant('LABEL.NUMERO_CONTRATO'),
		typeMask: 'MASK_CONTRATO',
		filter : 'maskContrato',
		tamanho : 35
	},{
		value : 'valorTotal',
		title : $translate.instant('CONTRATOS.LABEL.VALOR_TOTAL'),
		filter : 'currency',
		tamanho : 35,
		align : 'text-right'
	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
			start : 1,
			dir : 'asc',
			sort : 'id',
			limit : 10,
			fields: ['id', 'codigo', 'contrato.numeroAnoContrato', 'valorTotal'],
			filters : [{type: 'string', field: 'codigo'},
			           {type: 'string', field: 'contrato.numeroAnoContrato'},
			           {type: 'money', field: 'valorTotal'}]
	};

	// abrir um cronograma de execução para visualizar ou editar dependendo do parametro edit
	$scope.abrirCronogramaExecucao = function(edit){
		var cronogramaExecucao = $scope.cronogramaExecucaoChecked;

		if(!edit && !cronogramaExecucao) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && !cronogramaExecucao) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}

		angular.element('#editCronogramaExecucao').scope().getCronogramaExecucao(cronogramaExecucao, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};

    $scope.gerarRelatorioCronograma = function() {

        if(!$scope.cronogramaExecucaoChecked){
        	$scope.showAlert("warning", $translate.instant("MSG.SELECIONE_UM_ITEM_PARA_RELATORIO") + " ", false );
        } else{
        	$scope.url = '/cit-contratos-web/rest/cronogramaExecucao/gerarCronograma?idCronograma='+ $scope.cronogramaExecucaoChecked.id;
        	$scope.visualizarRelatorio($scope.url, $translate.instant("CONTRATOS.LABEL.CRONOGRAMA_EXECUCAO"));
        }

    };

	$scope.remover = function() {
		var cronogramaExecucao = $scope.cronogramaExecucaoChecked;

		if(cronogramaExecucao) {

			$scope.$openModalConfirm({
				message: $translate.instant("CONTRATOS.MSG.CONFIRMA_EXCLUSAO_CRONOGRAMA"),
				callback: function () {
					$scope.setLoadingRemove(true);

					CronogramaExecucaoRepository.remove(cronogramaExecucao).then(function() {
						$scope.showAlert('success', $translate.instant('CONTRATOS.MSG.SUCESSO_EXCLUSAO_CRONOGRAMA'), " ", false);
						$scope.fetchResult();
					});

					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.setLoading(false);
				}
			});

		} else {

			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
		}
	};

}]);