'use strict';

citApp.controller('EncerramentoContratoController', ['$scope', 'EncerramentoContratoRepository', '$timeout', '$translate', 'FileUploader', '$filter', 'ContratoRepository', 'OrganizacaoRepository',
                                                     'ConfiguracaoParametroSistemaRepository',
                                             function EncerramentoContratoController($scope, EncerramentoContratoRepository, $timeout, $translate, FileUploader, $filter, ContratoRepository,
                                            		 OrganizacaoRepository, ConfiguracaoParametroSistemaRepository) {

	$scope.edit = true;
	$scope.encerramentoContrato = {
			existePendencia : false,
			dataEmissao : $filter('date')(new Date(), "dd/MM/yyyy")
	};
	
	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 99,
        url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });
	
	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#EncerramentoContratoListControllerId').scope().fetchResult();
		$scope.$showPageSearchWorkspace($scope.workspace);
	};
	
	$scope.findContrato = function(value) {
		return ContratoRepository.findContrato(value, $scope.usuarioLogado.organizacao.id).then(function(result) {
			return result;
		});
	};
	
	$scope.findOrganizacao =  function(value) {
		return OrganizacaoRepository.listarOrganizacaoPorNome(value).then(function(result) {
			return result;
	   });
	};
	
	ConfiguracaoParametroSistemaRepository.getParametro("TERMO_ENCERRAMENTO_INTRODUCAO").then(function(result) {
		$scope.introducaoTemplate = result.valor;
	});
	
	ConfiguracaoParametroSistemaRepository.getParametro("TERMO_ENCERRAMENTO_ENCAMINHAMENTOS").then(function(result) {
		$scope.encaminhamentosTemplate = result.valor;
	});
	
	ConfiguracaoParametroSistemaRepository.getParametro("TERMO_ENCERRAMENTO_DECISAO").then(function(result) {
		$scope.decisaoTemplate = result.valor;
	});

	// limpa o formulario
    $scope.resetForm = function() {
        $scope.edit = true;
        
        $scope.encerramentoContrato = {
        		introducao : angular.copy($scope.introducaoTemplate),
        		encaminhamentos : angular.copy($scope.encaminhamentosTemplate),
        		decisao : angular.copy($scope.decisaoTemplate),
        		dataEmissao : $filter('date')(new Date(), "dd/MM/yyyy"),
        		existePendencia : false
        };
        
        $scope.setCurrentStep(0);

        $timeout(function() {
            $scope.encerramentoContratoForm.informacaoContratoForm.$submitted = false;
            $scope.encerramentoContratoForm.termoEncerramentoForm.$submitted = false;
            $scope.encerramentoContratoForm.informacaoContratoForm.$setPristine();
            $scope.encerramentoContratoForm.termoEncerramentoForm.$setPristine();
        });
    };

    $scope.visualizarEditar = function(idEncerramentoContrato, edit) {
    	$scope.resetForm();
        $scope.setLoadingGet(true);

        EncerramentoContratoRepository.get(idEncerramentoContrato).then(function(result) {

            $scope.encerramentoContrato = result.originalElement;
            $scope.encerramentoContrato.existePendencia = $scope.encerramentoContrato.pendencias ? true : false;

            $scope.edit = edit;

            $scope.setCurrentStep(0);

            $scope.setLoading(false);
        });
    };

  //INICIO DE METODOS REFERENTES AOS RELATORIOS

    $scope.gerarRelatorioEncerramentoContrato = function() {
        $scope.url = '/cit-contratos-web/rest/encerramentoContrato/gerarTermoEncerramentoContrato?idEncerramentoContrato='+ $scope.encerramentoContrato.id;
        $scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.TITULO_ENCERRAMENTO_CONTRATO"));
    };

    //Salva o estudo técnico
    $scope.saveOrUpdate = function() {
        // atributo para visualizar os itens obrigatorios na tela
        $scope.encerramentoContratoForm.informacaoContratoForm.$submitted = true;
        $scope.encerramentoContratoForm.termoEncerramentoForm.$submitted = true;
        $scope.encerramentoContratoForm.anexosObservacoesForm.$submitted = true;

        // verifica formulario valido
        if ($scope.encerramentoContratoForm.$valid) {

            $scope.setLoadingSalva(true);

            var encerramentoContratoTemp = {
                documentos: []
            };

            // preencher lista de documentos temporarios para verificacao dos anexos. verificar qual anexo e de qual documento pelo numero do documento
			angular.forEach($scope.encerramentoContrato.documentos, function(documento, key) {
				if (documento.uploadsDocumento) {
					documento.uuid = getUuid();
					encerramentoContratoTemp.documentos.push({
						file: documento.uploadsDocumento[0],
						uuid: clone(documento.uuid)
					});
					// deleta esse atributo por causa de um erro de ciclo no json
					delete documento.uploadsDocumento;
				}
			});

            EncerramentoContratoRepository.save($scope.encerramentoContrato).then(function(result) {

                $scope.encerramentoContrato = result.originalElement;
                $scope.encerramentoContrato.existePendencia = $scope.encerramentoContrato.pendencias ? true : false;

                // depois de salvar a baixa ja tendo uma referencia do id da baixa, esse metodo ira salvar os documentos se existir
                salvarListaDocumentos(encerramentoContratoTemp);
                
                $scope.atualizaPaginaPesquisa();
                
                $scope.setLoading(false);

                $scope.showAlert("success", $translate.instant("MSG.SUCESSO_ENCERRAMENTO_CONTRATO"));

            });
        } else {
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'));
        }
    };

    // metodo utilizado para percorrer a lista de documentos persistidos e verificar com a lista de documentos temporarios para verificacao de qual anexo pertence a qual documento
    var salvarListaDocumentos = function(encerramentoContratoTemp) {
    	$scope.uploader.queue = [];
    	
        angular.forEach($scope.encerramentoContrato.documentos, function(documentoSalvo, key) {
            // verifica se a entidade documento foi salva
            if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
                // percorre todos os documentos temporarios para verificacao do numero do documento
                angular.forEach(encerramentoContratoTemp.documentos, function(documentoTransient, key) {
                    // verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
                    if (documentoTransient.file && documentoSalvo.uuid === documentoTransient.uuid) {
                        //salva o anexo ao documento que tiver o mesmo numero do documento transient
                    	
                    	documentoTransient.file.url = documentoTransient.file.url + documentoSalvo.id;
                    	$scope.uploader.queue.push(documentoTransient.file);
                    }
                });
            }
        });
        
        if($scope.uploader.queue.length > 0){
        	$scope.uploader.uploadAll();
        }
        delete $scope.encerramentoContratoTemp;
    };

    // ###########################################
    // ########################################### DAQUI PARA BAIXO SAO TODOS OS METODOS RELACIONADOS AO WIZARD
    // ###########################################

	$scope.templates = [{ name: 'encerramentoContratoPg1.html', url: '/cit-contratos-web/html/encerramentoContrato/encerramentoContratoPg1.html', show: true},
	                    { name: 'encerramentoContratoPg2.html', url: '/cit-contratos-web/html/encerramentoContrato/encerramentoContratoPg2.html', show: false},
	                    { name: 'encerramentoContratoPg3.html', url: '/cit-contratos-web/html/encerramentoContrato/encerramentoContratoPg3.html', show: false} ];

    $scope.steps = ['one', 'two', 'three'];
    $scope.step = 0;

 // Verifica se o passo é o primeiro passo do wizard
    $scope.isFirstStep = function() {
        return $scope.step === 0;
    };

    // Verifica se o passo é o ultimo passo do wizard
    $scope.isLastStep = function() {
        return $scope.step === ($scope.steps.length - 1);
    };

    // Verifica se o passo é o passo atual do wizard
    $scope.isCurrentStep = function(step) {
        return $scope.step === step;
    };

	// Utilizado pelo wizard para mudar para o passo desejado
    $scope.setCurrentStep = function(step) {

        $scope.setLoadingGet(true);

        if (step <= ($scope.step - 1)) {
        	$scope.templates[$scope.step].show = false;
            $scope.step = step;
            $scope.templates[step].show = true;
        } else {
            if (!$scope.edit) {
                $scope.permitidoAvancar = true;
            } else {
                permiteAvancoFluxo();
            }

            if ($scope.permitidoAvancar) {
            	$scope.templates[$scope.step].show = false;
                $scope.step = step;
                $scope.templates[step].show = true;
            }
        }

        $scope.setLoading(false);
    };

    // Controla se o usuario pode avancar o fluxo no wizard, validando os itens obrigatórios
    function permiteAvancoFluxo() {

        $scope.permitidoAvancar = true;

        if ($scope.step === 0) {
            if (!validStepInformacaoContrato() && $scope.permitidoAvancar) {
                $scope.encerramentoContratoForm.informacaoContratoForm.$submitted = true;
            }
        } else if ($scope.step === 1) {
       	 	if (!validStepTermo() && $scope.permitidoAvancar) {
       	 		$scope.encerramentoContratoForm.termoEncerramentoForm.$submitted = true;
            }
        } else if ($scope.step === 2) {
            $scope.observacoesEntradaForm.$submitted = true;
        }

        if (!$scope.permitidoAvancar && !$scope.stepValid) {
            //Mensagem de erro de campos obrigatorios nao preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        }
    };

 // Recupera o passo atual do wizard
    $scope.getCurrentStep = function() {
        return $scope.steps[$scope.step];
    };

    // Faz com o que o wizard retroceda um passo
    $scope.handlePrevious = function() {

        $scope.setLoadingGet(true);

        $scope.templates[$scope.step].show = false;
        $scope.step -= ($scope.isFirstStep()) ? 0 : 1;
        $scope.templates[$scope.step].show = true;

        $scope.setLoading(false);

    };

    // Faz com o que o wizard avance um passo
    $scope.handleNext = function () {
  	  	$scope.setLoadingGet(true);

  	  	if(!$scope.edit){
  	  		$scope.permitidoAvancar = true;
  	  	} else {
  	  		permiteAvancoFluxo();
  	  	}

  	  	if ($scope.permitidoAvancar){
  	  		$scope.templates[$scope.step].show = false;
  	  		$scope.step += 1;
  	  		$scope.templates[$scope.step].show = true;
  	  	}

  	  	$scope.setLoading(false);
	};

	// Funcao especifica que retorna se o formulario de dados da entrada esta valido
    function validStepInformacaoContrato() {
    	$scope.stepValid = true;

        if ($scope.encerramentoContratoForm.informacaoContratoForm.$invalid) {
            $scope.stepValid = false;
        }

        return $scope.stepValid;
    };

    // Funcao especifica que retorna se o formulario de requisitos esta valido
    function validStepTermo() {
        $scope.stepValid = true;

        if ($scope.encerramentoContratoForm.termoEncerramentoForm.$invalid) {
            $scope.stepValid = false;
        }

        return $scope.stepValid;
    };

}]);