'use strict';

citApp.controller('EncerramentoContratoListController', ['$scope', 'EncerramentoContratoRepository', '$translate',
                                                 function EncerramentoContratoListController($scope, EncerramentoContratoRepository, $translate) {

	$scope.resetForm = function(){
		angular.element("#encerramentoContratoControllerId").scope().resetForm();
	};

	$scope.visualizarEncerramentoContrato = function(idEncerramentoContrato){
		angular.element("#encerramentoContratoControllerId").scope().visualizarEncerramentoContrato(idEncerramentoContrato);
	};

	$scope.headers = [
	{
		value : 'numTermo',
		title : $translate.instant('LABEL.NUM_TERMO'),
		tamanho : 20,
		align : 'text-center',
		typeMask: 'MASK_10_CODE_ANO',
		mask : [new RegExp(/(\d{2})(\d{1,4})$/), '$1/$2']
	},{
		value : 'contrato.numeroAnoContrato',
		title : $translate.instant('LABEL.NUM_CONTRATO'),
		filter : 'maskContrato',
		typeMask: 'MASK_CONTRATO',
		tamanho : 20,
		align : 'text-center'
	},{
		value : 'contrato.fornecedor.pessoa.nome',
		title : $translate.instant('LABEL.CONTRATADA'),
		align : 'text-left',
		tamanho : 35
	},{
		value : 'dataEmissao',
		title : $translate.instant('LABEL.DATA_EMISSAO'),
		filter : 'dateBR',
		align : 'text-center',
		tamanho : 20
	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
			start : 1,
			dir : 'asc',
			sort : 'id',
			limit : 10,
			fields: ['id', 'contrato.numeroAnoContrato', 'contrato.fornecedor.pessoa.nome', 'dataEmissao', 'numTermo'],
			filters : [{type: 'string', field: 'contrato.numeroAnoContrato'},
			           {type: 'string', field: 'contrato.fornecedor.pessoa.nome'},
			           {type: 'date-range', field: 'dataEmissao'},
			           {type: 'string', field: 'numTermo'}]
	};

	// abrir uma EncerramentoContrato para visualizar ou editar dependendo do parametro edit
	$scope.abrirEncerramentoContrato = function(edit){
		// recupera a EncerramentoContrato selecionada
		var encerramentoContrato = $scope.encerramentoContratoChecked;

		if(!edit && !encerramentoContrato) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && !encerramentoContrato) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}

		angular.element("#encerramentoContratoControllerId").scope().visualizarEditar(encerramentoContrato.id, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};

	//INICIO DE METODOS REFERENTES AOS RELATORIOS

    $scope.gerarRelatorioEncerramentoContrato = function() {

        if(!$scope.encerramentoContratoChecked){
        	$scope.showAlert("warning", $translate.instant("MSG.SELECIONE_UM_ITEM_PARA_RELATORIO") + " ", false );
        } else{
        	$scope.url = '/cit-contratos-web/rest/encerramentoContrato/gerarTermoEncerramentoContrato?idEncerramentoContrato='+ $scope.encerramentoContratoChecked.id;
        	$scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.TITULO_ENCERRAMENTO_CONTRATO"));
        }

    };

}]);


