'use strict';

citApp.controller('EstudoTecnicoListController', ['$scope', 'EstudoTecnicoRepository', '$translate',
                                                 function EstudoTecnicoListController($scope, EstudoTecnicoRepository, $translate) {

	$scope.resetForm = function(){
		angular.element("#estudoTecnicoControllerId").scope().resetForm();
	};

	$scope.visualizarEstudoTecnico = function(idEstudoTecnico){
		angular.element("#estudoTecnicoControllerId").scope().visualizarEstudoTecnico(idEstudoTecnico);
	};

	$scope.headers = [
	{
		value : 'registroSequencial.sequencialAno',
		title : $translate.instant('LABEL.NUM_ESTUDO_TECNICO'),
		align : 'text-center',
		tamanho : 20,
		typeMask: 'MASK_10_CODE_ANO',
		mask : [new RegExp(/(\d{3})(\d{1,4})$/), '$1/$2']
	},{
		value : 'descricaoSolucao',
		title : $translate.instant('CONTRATOS.LABEL.DESCRICAO_SOLUCAO'),
		tamanho : 75,
		align : 'text-justify'
	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
			start : 1,
			dir : 'asc',
			sort : 'id',
			limit : 10,
			fields: ['id', 'registroSequencial.sequencialAno', 'descricaoSolucao'],
			filters : [{type: 'numeric-range', field: 'registroSequencial.sequencialAno'},
			           {type: 'string', field: 'descricaoSolucao'}]
	};

	// abrir uma EstudoTecnico para visualizar ou editar dependendo do parametro edit
	$scope.abrirEstudoTecnico = function(edit){
		// recupera a EstudoTecnico selecionada
		var estudoTecnico = $scope.estudoTecnicoChecked;

		if(!edit && !estudoTecnico) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && !estudoTecnico) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}

		angular.element("#estudoTecnicoControllerId").scope().visualizarEditar(estudoTecnico.id, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};

	$scope.cadastrarNovoEstudoTecnico = function(){
		angular.element("#estudoTecnicoControllerId").scope().resetForm();
	};

	// prepara a exclusão da EstudoTecnico abrindo o modal de confirmação
	$scope.removerEstudoTecnico = function(){
		if($scope.estudoTecnicoChecked && $scope.estudoTecnicoChecked.id){
			$scope.$openModalConfirm({message: $translate.instant('MSG.EXCLUIR_ESTUDO_TECNICO'), callback: $scope.excluirEstudoTecnico, item: $scope.estudoTecnicoChecked});
		}else{
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_ITEM_EXCLUIR'));
		}
	};

	// realiza a exclusão da EstudoTecnico
	$scope.excluirEstudoTecnico = function(){
		var estudoTecnico = this.item;

		$scope.$modalConfirmInstance.dismiss('cancel');

		EstudoTecnicoRepository.remove(estudoTecnico).then(function(result) {
			$scope.showAlert('success', $translate.instant('MSG.REGISTRO_EXCLUIDO'));
			angular.element('#EstudoTecnicoListControllerId').scope().fetchResult();
		});

	};

	//INICIO DE METODOS REFERENTES AOS RELATORIOS

    $scope.gerarRelatorioEstudoTecnico = function() {

        if(!$scope.estudoTecnicoChecked){
        	$scope.showAlert("warning", $translate.instant("MSG.SELECIONE_UM_ITEM_PARA_RELATORIO") + " ", false );
        } else{
        	$scope.url = '/cit-contratos-web/rest/estudoTecnico/gerarEstudoTecnico?idEstudoTecnico='+ $scope.estudoTecnicoChecked.id;
        	$scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.TITULO_ESTUDO_TECNICO"));
        }

    };

}]);


