'use strict';

citApp.controller('EventoAditivoController', ['$scope', '$timeout', '$translate', 'EventoAditivoRepository', 'DominioRepository', 'FileUploader', '$filter',
                                              function EventoAditivoController($scope, $timeout, $translate, EventoAditivoRepository, DominioRepository, FileUploader, $filter) {


	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 1,
        url : '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	$scope.resetForm = function(form) {
		if(!$scope.evento){
			$scope.aditivo = {
					contrato : $scope.contratoTemp,
					dataEmissao : $filter('date')(new Date(), "dd/MM/yyyy")
			};
		}
		
		if($scope.contratoTemp == undefined){
			$scope.contratoTemp = angular.copy($scope.contrato);
		}
		$scope.contrato = angular.copy($scope.contrato);
		
		EventoAditivoRepository.findAllByContrato($scope.contrato.id).then(function(result) {
			var aditivo = 0;
			for (var i = 0; i < result.length; i++) {
				if (result[i].aditivoDeValor && result[i].tipoAlteracaoValor.codigo == 1) {
					aditivo = aditivo + result[i].valorAditivo;
				}
				if (result[i].aditivoDeValor && result[i].tipoAlteracaoValor.codigo == 2) {
					aditivo = aditivo - result[i].valorAditivo;
				}
			}
			$scope.contrato.valorTotalContrato = $scope.contrato.valorContrato + aditivo;
		});
		
		if(form){
			$scope.contrato = angular.copy($scope.contratoTemp);
			$timeout(function(){
				form.$submitted = true;
				form.$setPristine();
			});
		}
	};
	
	DominioRepository.findAllDominio('tipoAlteracaoContrato').then(function(result) {
		$scope.tiposAlteracaoContrato = result;
	});
	
	DominioRepository.findAllDominio('tipoAlteracaoValor').then(function(result) {
		$scope.tiposAlteracaoValor = result;
	});
	
	DominioRepository.findAllDominio('tipoObjeto').then(function(result) {
		$scope.tiposObjeto = result;
	});

	$scope.salvarAditivo = function(form){
		form.$submitted = true;
		if(form.$valid){
        	$scope.setLoadingSalva(true);
        	
			if($scope.aditivoTemp == null){
				$scope.aditivoTemp = {
						documentos : []
				};

				angular.forEach($scope.aditivo.documentos, function(documento, key) {

					$scope.aditivoTemp.documentos.push({
						uploadsDocumento : clone(documento.uploadsDocumento),
						numero : clone(documento.numero)
					});
					delete documento.uploadsDocumento;
				});
			}
			
			if (!$scope.aditivo.contrato) {
				$scope.aditivo.contrato = $scope.contratoTemp;
			}
			
			EventoAditivoRepository.save($scope.aditivo).then(function(result) {
				$scope.aditivo = result.originalElement;

				angular.forEach($scope.aditivo.documentos, function(documentoSalvo, key) {
					if(documentoSalvo !== undefined && documentoSalvo.id !== undefined){
						angular.forEach($scope.aditivoTemp.documentos, function(documentoTransient, key) {
							if(documentoSalvo.numero === documentoTransient.numero){
								salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
							}
						});
					}
				});

				$scope.setLoading(false);
				$scope.$modalInstance.dismiss('cancel');
				$scope.showAlert("success", $translate.instant("MSG.MG001"));
				$scope.resetForm();
				$scope.buscaOcorrencias($scope.contrato);
			});

		}else{
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		}
	};

	  var salvarUpload = function(documento, uploadsDocumento){
		  if (uploadsDocumento != undefined && uploadsDocumento != null) {
			  $scope.uploader.queue = uploadsDocumento;
			  angular.forEach($scope.uploader.queue, function(item, key) {
				  item.url =  item.url + documento.id;
			  });
			  $scope.uploader.uploadAll();
			  $scope.uploader.isUploading = false;
		  }
	  };
	  
  $scope.gerarRelatorioAditivo = function () {
	    $scope.url = '/cit-contratos-web/rest/eventoAditivo/gerarAditivo?idAditivo=' + $scope.aditivo.id;
	    $scope.visualizarRelatorio($scope.url, $translate.instant('CONTRATOS.LABEL.TITULO_RELATORIO_ADITIVO'));
	    $scope.$modalInstance.dismiss('cancel');
	  };
	
	//Watch para calcular valor da porcentagem do aditivo
    $scope.$watch('aditivo.valorAditivo', function() {
        if ($scope.aditivo && $scope.aditivo.valorAditivo) {
        	$scope.aditivo.percentualAditivo = ( $scope.aditivo.valorAditivo / $scope.contrato.valorContrato );
        } else {
        	$scope.aditivo.percentualAditivo = 0.0;
        }
    });
    
    //Watch para calcular nova data de inicio do aditivo
    $scope.$watch('aditivo.aditivoDePrazo', function() {
        if ($scope.aditivo && $scope.aditivo.aditivoDePrazo && !$scope.aditivo.id) {
        	var timeStr = $scope.contrato.ultimaDataVigenciaFinal ? $scope.contrato.ultimaDataVigenciaFinal : $scope.contrato.dataVigenciaFinal;
        	var data = converterStringEmDate(timeStr);
        	data.setDate(data.getDate() + 1); // Adiciona 1 dia
        	$scope.aditivo.dataInicio = $filter('date')(data, "dd/MM/yyyy");
        } else if ($scope.aditivo && !$scope.aditivo.aditivoDePrazo) {
        	delete $scope.aditivo.dataInicio;
        	delete $scope.aditivo.dataFim;
        }
    });
    
    //Watch para limpar campos quando desmarcado checkbox
    $scope.$watch('aditivo.aditivoDeValor', function() {
        if ($scope.aditivo && !$scope.aditivo.aditivoDeValor) {
        	delete $scope.aditivo.tipoAlteracaoValor;
        	delete $scope.aditivo.tipoObjeto;
        	delete $scope.aditivo.valorAditivo;
        }
    });
    
    //Watch para limpar campos quando desmarcado checkbox
    $scope.$watch('aditivo.aditivoDeClausula', function() {
        if ($scope.aditivo && !$scope.aditivo.aditivoDeClausula) {
        	delete $scope.aditivo.descricaoAditivo;
        }
    });
    
    $scope.resetForm();

}]);
