'use strict';

citApp.controller('EventoApostilamentoController', ['$scope',
                                                    '$filter',
                                                    '$timeout',
                                                    '$translate',
                                                    'FileUploader',
                                                    'ContratoEventoRepository',
                                                    'EventoApostilamentoRepository',
                                                    'ContratoRepository',
                                                    'DominioRepository',
                                                    function EventoApostilamentoController($scope,
                                                                                           $filter,
                                                                                           $timeout,
                                                                                           $translate,
                                                                                           FileUploader,
                                                                                           ContratoEventoRepository,
                                                                                           EventoApostilamentoRepository,
                                                                                           ContratoRepository,
                                                                                           DominioRepository) {


  $scope.resetForm = function (form){

		if($scope.contratoTemp == undefined){
			$scope.contratoTemp= angular.copy($scope.contrato);
		}

    $scope.eventoApostilamento = {
          contrato : $scope.contratoTemp,
          cnpjFornecedor : $scope.contrato.fornecedor.pessoa.pessoaJuridica.cnpj,
          dataEmissao : $filter('date')(new Date(), "dd/MM/yyyy"),
          justificativa : "",
          registroAdministrativo : "",
          documentos : [],
          observacoes : []
      };
      if($scope.evento){
        $scope.setLoadingGet(true);
        EventoApostilamentoRepository.get($scope.evento.id).then(function(result){
          $scope.eventoApostilamento = result.originalElement;
          $scope.setLoading(false);
        });
      }
      $timeout(function (){
        if(form && !form.$submitted){
          form.$submitted = false;
          form.$setPristine();
        }
      });
  };

  $scope.resetForm();

  $scope.uploader = new FileUploader({
    autoUpload: false,
    queueLimit: 1,
    url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
  });

  $scope.saveOrUpdate = function (form){

    form.$submitted = true;

    if(form.$invalid){
    	$scope.showAlert('error', $translate.instant('MSG.MN001'));
    }
    if($scope.isInformacoesValidas()){

      var apostilamentoTemp = {
          documentos: []
      };
      // preencher lista de documentos temporarios para verificacao dos anexos. Verifica qual anexo e de qual documento pelo numero do documento
      angular.forEach($scope.eventoApostilamento.documentos, function(documento, key) {
    	  if (documento.uploadsDocumento) {
    		  apostilamentoTemp.documentos.push({
    			  uploadsDocumento: clone(documento.uploadsDocumento),
    			  numero: clone(documento.numero)
    		  });
    		  // deleta esse atributo por causa de um erro de ciclo no json
    		  delete documento.uploadsDocumento;
    	  }
      });

      $scope.setLoadingSalva(true);
      EventoApostilamentoRepository.save($scope.eventoApostilamento).then(function(result) {

        $scope.eventoApostilamento = result.originalElement;
        $scope.setLoading(false);
        $scope.showAlert("success", $translate.instant('CONTRATOS.MSG.SUCESSO_SALVAR_APOSTILAMENTO'));

        if($scope.eventoApostilamento.documentos){

          // depois de salvar, ja tendo uma referencia do id da mesma, esse metodo ira salvar os documentos se houver
          salvarListaDocumentos(apostilamentoTemp);
        }
        $scope.resetForm();
        $scope.$modalInstance.dismiss('cancel');
        $scope.buscaOcorrencias($scope.contrato);
      });
    };
  };


  $scope.isInformacoesValidas = function () {

    var informacoesValidas = true;

    if(!isDataDentroPeriodo($scope.eventoApostilamento.contrato.dataVigenciaInicial,
        $scope.eventoApostilamento.contrato.dataUltimaVigenciaFinal, $scope.eventoApostilamento.dataEmissao)) {
        $scope.showAlert('error', $translate.instant('CONTRATOS.MSG.DATA_APOSTILAMENTO_INVALIDA'));
        informacoesValidas = false;
    }

    return informacoesValidas;
  };

  // metodo utilizado para percorrer a lista de documentos persistidos e verificar com a lista de documentos temporarios para verificacao de qual anexo pertence a qual documento
  var salvarListaDocumentos = function(eventoApostilamento) {

      angular.forEach($scope.eventoApostilamento.documentos, function(documentoSalvo, key) {
          // verifica se a entidade documento foi salva
          if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
              // percorre todos os documentos temporarios para verificacao do numero do documento
              angular.forEach(eventoApostilamento.documentos, function(documentoTransient, key) {
                  // verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
                  if (documentoSalvo.numero === documentoTransient.numero) {
                      //salva o anexo ao documento que tiver o mesmo numero do documento transient
                      salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
                  }
              });
          }
      });
  };

  // metodo responsavel para salvar os anexos depois de ter salvo o registro de ocorrencia, utilizando a referencia do id registroOcorrencia.id
  var salvarUpload = function(documento, uploadsDocumento) {
      $scope.uploader.queue = uploadsDocumento;
      // percorre a lista de uploads a serem enviados
      angular.forEach($scope.uploader.queue, function(item, key) {
          item.url = item.url + documento.id;
      });
      if ($scope.uploader.queue && $scope.uploader.queue.length > 0) {
          // realiza o upload se existir no minimo um arquivo para upload
          $scope.uploader.uploadAll();
      }
  };

  $scope.gerarRelatorioApostilamento = function () {
    $scope.url = '/cit-contratos-web/rest/eventoApostilamento/pdfGerarApostilamento?idApostilamento=' + $scope.eventoApostilamento.id;
    $scope.visualizarRelatorio($scope.url, $translate.instant('CONTRATOS.LABEL.TITULO_DOCUMENTO_APOSTILAMENTO'));
    $scope.$modalInstance.dismiss('cancel');
  };

}]);

