'use strict';
citApp.controller('EventoEmpenhoController', ['$scope', '$filter', '$timeout', '$translate', 'FileUploader',
                                                       'ContratoEmpenhoRepository', 'DominioRepository',
                                                       function EventoEmpenhoController($scope, $filter, $timeout, $translate, FileUploader,
                                                    		   ContratoEmpenhoRepository, DominioRepository) {


	$scope.resetForm = function (form){

		$scope.contratoEmpenho = {
				contrato : $scope.contrato
		};

		if($scope.evento){
			$scope.setLoadingGet(true);
			ContratoEmpenhoRepository.get($scope.evento.id).then(function(result){
				$scope.contratoEmpenho = result.originalElement;
				$scope.contratoEmpenho.contrato =  $scope.contrato;
				$scope.setLoading(false);
			});
		}
		$timeout(function (){
			if(form && !form.$submitted){
				form.$submitted = false;
				form.$setPristine();
			}
		});
	};

	$scope.resetForm();

	$scope.uploader = new FileUploader({
		autoUpload: false,
		queueLimit: 1,
		url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
	});

	DominioRepository.findAllDominio('tipoFinalidadeEmpenhoContrato').then(function(result) {
		$scope.dominioFinalidadeEmpenhoList = result;
	});
	DominioRepository.findAllDominio('tipoCategoriaEconomicaEmpenhoContrato').then(function(result) {
		$scope.dominioCategoriaEconomicaList = result;
	});

	$scope.saveOrUpdate = function (form){

		form.$submitted = true;

		if(form.$invalid){
			if(!$scope.contratoEmpenho.numeroEmpenho || $scope.contratoEmpenho.numeroEmpenho == ''){
				$scope.numeroEmpenhoObrigatorio = true;
			}else{
				$scope.numeroEmpenhoObrigatorio = false;
			}
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		} else {

			if ($scope.contratoEmpenho.dominioFinalidadeEmpenho.codigo === 3) {
				$scope.contratoEmpenho.valorEmpenhado = ($scope.contratoEmpenho.valorEmpenhado * (-1));
			}

			if ($scope.contratoEmpenho.valorEmpenhado > parseFloat($scope.valorResidual.toFixed(2))) {
				$scope.showAlert('error', $translate.instant('CONTRATOS.VALIDACAO.VALOR_TOTAL_EMPENHADO_SUPERIOR_VALOR_RESIDUAL'));
				return;
			}
			$scope.empenhoTemp = {
					documentos : []
			};

			angular.forEach($scope.contratoEmpenho.documentos, function(documento, key) {

				$scope.empenhoTemp.documentos.push({
					uploadsDocumento : clone(documento.uploadsDocumento),
					numero : clone(documento.numero)
				});
				delete documento.uploadsDocumento;
			});

			$scope.setLoadingSalva(true);
			ContratoEmpenhoRepository.save($scope.contratoEmpenho).then(function(result) {

				$scope.contratoEmpenho = result.originalElement;
				$scope.setLoading(false);
				$scope.showAlert("success", $translate.instant('CONTRATOS.MSG.SUCESSO_SALVAR_EMPENHO'));

				if($scope.contratoEmpenho.documentos){

					// preencher lista de documentos temporarios para verificacao dos anexos. Verifica qual anexo e de qual documento pelo numero do documento
					angular.forEach($scope.contratoEmpenho.documentos, function(documento, key) {
						if (documento.uploadsDocumento) {
							$scope.empenhoTemp.documentos.push({
								uploadsDocumento: clone(documento.uploadsDocumento),
								numero: clone(documento.numero)
							});
							// deleta esse atributo por causa de um erro de ciclo no json
							delete documento.uploadsDocumento;
						}
					});
					// depois de salvar, ja tendo uma referencia do id da mesma, esse metodo ira salvar os documentos se houver
					salvarListaDocumentos($scope.empenhoTemp);
				}
				$scope.resetForm();
				$scope.$modalInstance.dismiss('cancel');
				$scope.buscaOcorrencias($scope.contrato);
			});
		};
	};


	// metodo utilizado para percorrer a lista de documentos persistidos e verificar com a lista de documentos temporarios para verificacao de qual anexo pertence a qual documento
	var salvarListaDocumentos = function(contratoEmpenho) {

		angular.forEach($scope.contratoEmpenho.documentos, function(documentoSalvo, key) {
			// verifica se a entidade documento foi salva
			if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
				// percorre todos os documentos temporarios para verificacao do numero do documento
				angular.forEach(contratoEmpenho.documentos, function(documentoTransient, key) {
					// verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
					if (documentoSalvo.numero === documentoTransient.numero) {
						//salva o anexo ao documento que tiver o mesmo numero do documento transient
						salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
					}
				});
			}
		});
	};

	// metodo responsavel para salvar os anexos depois de ter salvo o registro de ocorrencia, utilizando a referencia do id registroOcorrencia.id
	var salvarUpload = function(documento, uploadsDocumento) {
		$scope.uploader.queue = uploadsDocumento;
		// percorre a lista de uploads a serem enviados
		angular.forEach($scope.uploader.queue, function(item, key) {
			item.url = item.url + documento.id;
		});
		if ($scope.uploader.queue && $scope.uploader.queue.length > 0) {
			// realiza o upload se existir no minimo um arquivo para upload
			$scope.uploader.uploadAll();
			$scope.uploader.isUploading = false;
		}
	};


	$scope.gerarRelatorioRegistroEmpenho = function () {
		$scope.url = '/cit-contratos-web/rest/contratoEmpenho/pdfGerarEmpenho?idEmpenho=' + $scope.contratoEmpenho.id;
		$scope.visualizarRelatorio($scope.url, $translate.instant('CONTRATOS.LABEL.TITULO_DOCUMENTO_EMPENHO'));
		$scope.$modalInstance.dismiss('cancel');
	};

}]);

