'use strict';

citApp.controller('EventoLiberacaoPagamentoController', ['$scope', '$timeout', '$translate', 'EventoLiberacaoPagamentoRepository', 'ContratoRepository', 'DominioRepository', 'ContratoEnvolvidoRepository', 'ContratoEmpenhoRepository', 'FileUploader',
                                                         'OrdemFornecimentoRepository', 'RegistroOcorrenciaRepository',
                                              function EventoLiberacaoPagamentoController($scope, $timeout, $translate, EventoLiberacaoPagamentoRepository, ContratoRepository, DominioRepository, ContratoEnvolvidoRepository,
                                            		  ContratoEmpenhoRepository, FileUploader, OrdemFornecimentoRepository, RegistroOcorrenciaRepository) {


	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 1,
        url : '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	$scope.resetForm = function(form) {
		if($scope.contratoTemp == undefined){
			$scope.contratoTemp = angular.copy($scope.contrato);
		}
		$scope.valorTotalLiberado = $scope.contrato.valorTotalLiberado;
		delete $scope.contrato.valorTotalLiberado;
		$scope.contrato = angular.copy($scope.contrato);
		$scope.buscaProfissionaisEnvolvidos();
		$scope.buscaEmpenhos();
		$scope.buscaOrdemFornecimento();

		if($scope.evento){
			$scope.getLiberacaoPagamento();
        }else{
			$scope.liberacaoPagamento = {contrato: $scope.contrato, ordensFornecimento : [], empenhos : [], totalLiberadoOS : 0, totalEmpenhoLiberado : 0};
		}
		if(form){
			$scope.contrato = angular.copy($scope.contratoTemp);
			$timeout(function(){
				form.$submitted = true;
				form.$setPristine();
			});
		}
	};

	$scope.getLiberacaoPagamento = function(){
		var idLiberacaoPagamento = $scope.evento.id;
		$scope.setLoadingGet(true);
		EventoLiberacaoPagamentoRepository.get(idLiberacaoPagamento).then(function(result) {
			$scope.liberacaoPagamento = result.originalElement;

			if($scope.liberacaoPagamento.empenhos == undefined || $scope.liberacaoPagamento.empenhos.length == 0){
				$scope.listaEmpenhos = [];
				$scope.liberacaoPagamento.valorLiberadoPagamento = $scope.liberacaoPagamento.totalLiberadoPagamento;
			}

			$scope.setLoading(false);
		});
	};

	$scope.salvarLiberacaoPagamento = function(form){
		form.$submitted = true;
		if(form.$valid){
        	if(!isPeriodoValido($scope.contrato.dataVigenciaInicial, $scope.liberacaoPagamento.dataLiberacao)) {
				$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.DATA_LIBERACAO'));
				return;
			}

        	if(!$scope.liberacaoPagamento.liberarPagamentoValorContrata && $scope.listaOrdemFornecimento.length > 0 && $scope.liberacaoPagamento.ordensFornecimento.length == 0){
        		$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.SELECIONE_OS'));
				return;
        	}

        	if($scope.listaEmpenhos.length > 0 && $scope.liberacaoPagamento.empenhos.length == 0 && !$scope.liberacaoPagamento.liberarPagamentoValorContrata){
        		$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.SELECIONE_EMPENHO'));
				return;
        	}

        	if(($scope.valorTotalLiberado + $scope.liberacaoPagamento.totalLiberadoPagamento) > $scope.contrato.valorTotalContrato){
        		$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.VALOR_TOTAL_A_LIBERAR_ULTRAPASSA_VALOR_CONTRATO'));
				return;
        	}

        	$scope.setLoadingSalva(true);
        	delete $scope.liberacaoPagamento.ordemFornecimento;
        	delete $scope.liberacaoPagamento.empenho;

			if($scope.liberacaoTemp == null){
				$scope.liberacaoTemp = {
						documentos : []
				};

				angular.forEach($scope.liberacaoPagamento.documentos, function(documento, key) {

					$scope.liberacaoTemp.documentos.push({
						uploadsDocumento : clone(documento.uploadsDocumento),
						numero : clone(documento.numero)
					});
					delete documento.uploadsDocumento;
				});
			}
				EventoLiberacaoPagamentoRepository.save($scope.liberacaoPagamento).then(function(result) {
					$scope.liberacaoPagamento = result.originalElement;

					angular.forEach($scope.liberacaoPagamento.documentos, function(documentoSalvo, key) {
						if(documentoSalvo !== undefined && documentoSalvo.id !== undefined){
							angular.forEach($scope.liberacaoTemp.documentos, function(documentoTransient, key) {
								if(documentoSalvo.numero === documentoTransient.numero){
									salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
								}
							});
						}
					});

					$scope.setLoading(false);
					$scope.$modalInstance.dismiss('cancel');
					$scope.showAlert("success", $translate.instant("MSG.MG001"));
					$scope.resetForm();
					$scope.buscaOcorrencias($scope.contrato);
				});

		}else{
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		}
	};

	  var salvarUpload = function(documento, uploadsDocumento){
		  if (uploadsDocumento != undefined && uploadsDocumento != null) {
			  $scope.uploader.queue = uploadsDocumento;
			  angular.forEach($scope.uploader.queue, function(item, key) {
				  item.url =  item.url + documento.id;
			  });
			  $scope.uploader.uploadAll();
			  $scope.uploader.isUploading = false;
		  }
	  };

	$scope.buscaProfissionaisEnvolvidos = function(){
		$scope.listaProfissionaisEnvolvidos = {};
		ContratoEnvolvidoRepository.findProfissionaisContrato($scope.contrato.id).then(function(result) {
			$scope.listaProfissionaisEnvolvidos = result;
		});
	};

	$scope.buscaEmpenhos = function(){
		$scope.listaEmpenhos = {};
		ContratoEmpenhoRepository.findEmpenhoParaLiberacao($scope.contrato.id).then(function(result) {
			$scope.listaEmpenhos = result;
		});
	};

	$scope.buscaOrdemFornecimento = function(){
		$scope.listaOrdemFornecimento = {};
		OrdemFornecimentoRepository.findOrdemFornecimentoPorContrato($scope.contrato.id).then(function(result) {
			$scope.listaOrdemFornecimento = result;
		});
	};

	$scope.buscaValorTotalGlosa = function(){
		$timeout(function(){
			if($scope.liberacaoPagamento.ordemFornecimento != undefined && $scope.liberacaoPagamento.ordemFornecimento.id != undefined){
				RegistroOcorrenciaRepository.getValorTotalGlosaOrdem($scope.liberacaoPagamento.ordemFornecimento.id).then(function(result) {
					$scope.liberacaoPagamento.valorGlosa = result;
				});
			}else{
				$scope.liberacaoPagamento.valorGlosa = null;
			}
		});
	};

	$scope.buscaValorTotalOS = function(){
		$timeout(function(){
			if($scope.liberacaoPagamento.ordemFornecimento != undefined && $scope.liberacaoPagamento.ordemFornecimento.id != undefined){
				OrdemFornecimentoRepository.getValorTotalOS($scope.liberacaoPagamento.ordemFornecimento.id).then(function(result) {
					$scope.liberacaoPagamento.valorTotalOS = result;
				});
			}else{
				$scope.liberacaoPagamento.valorTotalOS = null;
			}
		});
	};

	$scope.addOrdemFornecimento = function(){

		if($scope.liberacaoPagamento.ordemFornecimento){
			if($scope.liberacaoPagamento.valorTotalOS == undefined){
				$scope.liberacaoPagamento.valorTotalOS = 0;
			}

			if($scope.liberacaoPagamento.valorGlosa == undefined){
				$scope.liberacaoPagamento.valorGlosa = 0;
			}

			var ordemFornecimento = {
					ordemFornecimento : $scope.liberacaoPagamento.ordemFornecimento,
					valorTotalOS : $scope.liberacaoPagamento.valorTotalOS,
					valorGlosa : $scope.liberacaoPagamento.valorGlosa
			};
			$scope.liberacaoPagamento.totalLiberadoOS += (ordemFornecimento.valorTotalOS - ordemFornecimento.valorGlosa);

			$scope.liberacaoPagamento.totalLiberadoPagamento = $scope.liberacaoPagamento.totalLiberadoOS;

			if($scope.liberacaoPagamento.ordensFornecimento.length == 0){
				$scope.liberacaoPagamento.empenhos = [];
				$scope.liberacaoPagamento.totalEmpenhoLiberado = 0;
			}
			$scope.liberacaoPagamento.ordensFornecimento.push(ordemFornecimento);
			$scope.liberacaoPagamento.ordemFornecimento = {};
			delete $scope.liberacaoPagamento.valorGlosa;
			delete $scope.liberacaoPagamento.valorTotalOS;
			delete $scope.liberacaoPagamento.ordemFornecimento;
			delete $scope.liberacaoPagamento.empenho;
			$scope.liberacaoPagamento.valorExecutado = 0;
			$scope.liberacaoPagamento.percentualEmpenho = 0;
			$scope.liberacaoPagamento.valorLiberadoPagamento = 0;

		}else{
			$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.SELECIONE_ORDEM'));
			return;
		}

	};

	$scope.buscaValorExecutadoEmpenho = function(){
		$scope.liberacaoPagamento.valorLiberadoPagamento = 0;
		$timeout(function(){
			$scope.valorExecutadoFix = 0;
			if($scope.liberacaoPagamento.empenho != undefined && $scope.liberacaoPagamento.empenho.id != undefined){
				var buscarValorJaEmpenhado = true;
				if($scope.liberacaoPagamento.empenhos && $scope.liberacaoPagamento.empenhos.length > 0){
					angular.forEach($scope.liberacaoPagamento.empenhos, function (empenho) {
						if(empenho.contratoEmpenho.id == $scope.liberacaoPagamento.empenho.id){
							if(empenho.valorLiberado != undefined){
								$scope.valorExecutadoFix += empenho.valorLiberado;
							}
						}
					});

					if($scope.valorExecutadoFix > 0){
						$scope.liberacaoPagamento.valorExecutado = $scope.valorExecutadoFix;
						buscarValorJaEmpenhado = false;
						calculaValorExecutadoPercentual();
					}
				}
				if(buscarValorJaEmpenhado){
					EventoLiberacaoPagamentoRepository.getValorTotalExecutadoEmpenho($scope.liberacaoPagamento.empenho.id).then(function(result) {
						if(!$scope.liberacaoPagamento.valorExecutado){
							$scope.liberacaoPagamento.valorExecutado = 0;
						}
						if(result != undefined){
							$scope.valorExecutadoFix = result;
							$scope.liberacaoPagamento.valorExecutado = result;
							$scope.liberacaoPagamento.percentualEmpenho = (($scope.liberacaoPagamento.valorExecutado * 100) / $scope.liberacaoPagamento.empenho.valorEmpenhado).toFixed(1);
						}else{
							$scope.valorExecutadoFix = 0;
							$scope.liberacaoPagamento.valorExecutado = 0;
							$scope.liberacaoPagamento.percentualEmpenho = 0;
						}
					});
				}
			}else{
				$scope.liberacaoPagamento.valorExecutado = 0;
				$scope.liberacaoPagamento.percentualEmpenho = 0;
			}

		});
	};

	$scope.addEmpenho = function(){

		if($scope.liberacaoPagamento.empenho){
			if($scope.liberacaoPagamento.totalLiberadoOS != undefined && $scope.liberacaoPagamento.totalLiberadoOS > 0 && $scope.liberacaoPagamento.totalEmpenhoLiberado != undefined){
				if(parseFloat(($scope.liberacaoPagamento.totalEmpenhoLiberado+$scope.liberacaoPagamento.valorLiberadoPagamento).toFixed(2)) > parseFloat($scope.liberacaoPagamento.totalLiberadoOS.toFixed(2))){
					$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.VALOR_LIBERADO_MAIOR_VALOR_OFBS'));
					return;
				}
			}

			if($scope.liberacaoPagamento.valorLiberadoPagamento == undefined || $scope.liberacaoPagamento.valorLiberadoPagamento == 0){
				$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.INFORME_VALOR_EMPENHO'));
				return;
			}

			if($scope.liberacaoPagamento.valorExecutado > $scope.liberacaoPagamento.empenho.valorEmpenhado){
				$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.LIBERACAO_NO_LIMITE'));
				return;
			}else if($scope.liberacaoPagamento.empenhos.length > 0){
				var valorLiberadoEmpenhoJadd = 0;
				angular.forEach($scope.liberacaoPagamento.empenhos, function (empenho) {
					if(empenho.contratoEmpenho.id == $scope.liberacaoPagamento.empenho.id){
						if(empenho.valorLiberado != undefined){
							valorLiberadoEmpenhoJadd += empenho.valorLiberado;
						}
					}
				});
			}

			if($scope.liberacaoPagamento.valorLiberadoPagamento > $scope.contrato.valorTotalContrato ){
				$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.VALOR_LIBERADO_MAIOR_VALOR_CONTRATO'));
				return;
			}

			if((($scope.liberacaoPagamento.valorExecutado * 100) / $scope.liberacaoPagamento.empenho.valorEmpenhado) > 100){
				$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.LIBERACAO_ULTRAPASSA_LIMITE'));
				return;
			}

			var empenho = {
					contratoEmpenho : $scope.liberacaoPagamento.empenho,
					valorExecutado : $scope.liberacaoPagamento.valorExecutado,
					percentualExecutado : $scope.liberacaoPagamento.percentualEmpenho,
					valorLiberado : $scope.liberacaoPagamento.valorLiberadoPagamento
			};
			$scope.liberacaoPagamento.totalEmpenhoLiberado += empenho.valorLiberado;
			$scope.liberacaoPagamento.empenhos.push(empenho);
			$scope.liberacaoPagamento.empenho = {};
			$scope.liberacaoPagamento.valorLiberadoPagamento = null;
			delete $scope.liberacaoPagamento.valorExecutado;
			delete $scope.liberacaoPagamento.valorLiberadoPagamento;
			delete $scope.liberacaoPagamento.percentualEmpenho;
			delete $scope.liberacaoPagamento.empenho;
		}else{
			$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.SELECIONE_EMPENHO'));
			return;
		}

	};

	$scope.selectAllOrdemFornecimento = function(value) {
		angular.forEach($scope.liberacaoPagamento.ordensFornecimento, function (item) {
			item.$checked = value;
        });
	};

	var recuperarOrdensSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.liberacaoPagamento.ordensFornecimento, function (item) {
			if(item.$checked){
				lista.push(item);
			}
        });
		return lista;
	};

	$scope.removerOrdemFornecimento = function(){
		var lista = recuperarOrdensSelecionadas();
		if(lista.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {
					angular.forEach(lista, function (itemRem, key) {
						var i = 0;
						var index = 0;
						angular.forEach($scope.liberacaoPagamento.ordensFornecimento, function (item) {
							if(itemRem.ordemFornecimento.id == item.ordemFornecimento.id){
								index = i;
							}
							i ++;
						});
						$scope.liberacaoPagamento.totalLiberadoOS -= ($scope.liberacaoPagamento.ordensFornecimento[index].valorTotalOS -  $scope.liberacaoPagamento.ordensFornecimento[index].valorGlosa);

						$scope.liberacaoPagamento.valorLiberadoPagamento = $scope.liberacaoPagamento.totalLiberadoOS;
						$scope.liberacaoPagamento.totalLiberadoPagamento = $scope.liberacaoPagamento.totalLiberadoOS;

						$scope.liberacaoPagamento.ordensFornecimento.splice(index, 1);
						$scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
					});
					$scope.$modalConfirmInstance.dismiss('cancel');
	            }
			});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	$scope.selectAllEmpenho = function(value) {
		angular.forEach($scope.liberacaoPagamento.empenhos, function (item) {
			item.$checked = value;
        });
	};

	var recuperarEmpenhosSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.liberacaoPagamento.empenhos, function (item) {
			if(item.$checked){
				lista.push(item);
			}
        });
		return lista;
	};

	$scope.removerEmpenho = function(){
		var lista = recuperarEmpenhosSelecionadas();
		if(lista.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {
					angular.forEach(lista, function (itemRem, key) {
						var i = 0;
						var index = 0;
						angular.forEach($scope.liberacaoPagamento.empenhos, function (item) {
							if(itemRem.contratoEmpenho.id == item.contratoEmpenho.id){
								index = i;
							}
							i ++;
						});
						$scope.liberacaoPagamento.totalEmpenhoLiberado -= $scope.liberacaoPagamento.empenhos[index].valorLiberado;
						$scope.liberacaoPagamento.empenhos.splice(index, 1);
						$scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
					});
					$scope.$modalConfirmInstance.dismiss('cancel');
	            }
			});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	$scope.$watch('liberacaoPagamento.liberarPagamentoValorContrata', function(){

		if($scope.validacaoValorContratoEmpenho){
			$scope.validacaoValorContratoEmpenho = false;
			return;
		}

		if($scope.liberacaoPagamento && $scope.liberacaoPagamento.liberarPagamentoValorContrata){
			if($scope.valorTotalEmpenhado < $scope.contrato.valorTotalContrato){
				$scope.showAlert('warning', $translate.instant('LIBERACAO_PAGAMENTO.MSG.LIBERACAO_PAGAMENTO_VALOR_EMPENHO_MENOR_VALOR_TOTAL_CONTRATO'));
				$scope.liberacaoPagamento.liberarPagamentoValorContrata = false;
				$scope.validacaoValorContratoEmpenho = true;
				return ;
			}
			$scope.liberacaoPagamento.valorLiberadoPagamento = $scope.contrato.valorTotalContrato;
			$scope.liberacaoPagamento.totalLiberadoPagamento = $scope.contrato.valorTotalContrato;
			$scope.liberacaoPagamento.empenhos = [];
			$scope.liberacaoPagamento.ordensFornecimento = [];
			$scope.liberacaoPagamento.totalLiberadoOS = 0;
			$scope.liberacaoPagamento.totalEmpenhoLiberado = 0;
		}
	}, true);

	function calculaValorExecutadoPercentual(){
		$scope.liberacaoPagamento.percentualEmpenho = (($scope.liberacaoPagamento.valorExecutado * 100) / $scope.liberacaoPagamento.empenho.valorEmpenhado).toFixed(1);
	};

	$scope.$watch('liberacaoPagamento.valorLiberadoPagamento', function(){
		if($scope.liberacaoPagamento && $scope.liberacaoPagamento.empenho){

			if(!$scope.liberacaoPagamento.valorExecutado){
				$scope.liberacaoPagamento.valorExecutado = 0;
			}
			if(!$scope.valorLiberadoPagamentoTemp){
				$scope.valorLiberadoPagamentoTemp = 0;
			}

			$scope.liberacaoPagamento.valorExecutado = $scope.valorExecutadoFix + $scope.liberacaoPagamento.valorLiberadoPagamento;
			$scope.valorLiberadoPagamentoTemp = $scope.liberacaoPagamento.valorLiberadoPagamento;

			calculaValorExecutadoPercentual();
		}
	}, true);

	$scope.resetForm();

}]);
