'use strict';

citApp.controller('EventoPenalidadeController', ['$scope', '$filter', '$timeout', '$translate', 'FileUploader',
                                                    'EventoPenalidadeRepository', 'RegistroOcorrenciaRepository', 'TermoReferenciaPenalidadeRepository',
                                                    function EventoPenalidadeController($scope,
                                                    		$filter, $timeout, $translate, FileUploader,
                                                    		EventoPenalidadeRepository, RegistroOcorrenciaRepository, TermoReferenciaPenalidadeRepository) {

	$scope.resetForm = function (form){

		if(!$scope.contrato.termoReferencia){
	    	$scope.showAlert('warning', $translate.instant('CONTRATOS.LABEL.CONTRATO_SEM_VINCULO_TERMO'));
		}else{
			$scope.listaPenalidades = $scope.contrato.termoReferencia.penalidades;
		}

		if ($scope.edit) {
			$scope.eventoPenalidade = {
					contrato : $scope.contrato, cnpjFornecedor : $scope.contrato.fornecedor.pessoa.pessoaJuridica.cnpj,
					ocorrencias : [], documentos : [], observacoes : []
			};
		}

		if($scope.evento){
			$scope.setLoadingGet(true);
			EventoPenalidadeRepository.get($scope.evento.id).then(function(result){
				$scope.eventoPenalidade = result.originalElement;
				$scope.eventoPenalidade.dataPenalizacao = $filter('date')($scope.eventoPenalidade.dataPenalizacao, "dd/MM/yyyy");
				$scope.setLoading(false);
			});
		}

		$timeout(function (){
			if(form && !form.$submitted){
				form.$submitted = false;
				form.$setPristine();
			}
		});
	};

	$scope.resetForm();

	$scope.uploader = new FileUploader({
		autoUpload: false,
		queueLimit: 1,
		url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
	});

	RegistroOcorrenciaRepository.findAllOcorrenciasPorContrato($scope.contrato.id).then(function(result) {
		$timeout(function (){
			$scope.listaOcorrenciasFixa = result;
			$scope.listaOcorrencias = angular.copy($scope.listaOcorrenciasFixa);

			removerOcorrenciasExistentes();

		});
	});

	function removerOcorrenciasExistentes() {
		for(var i = $scope.eventoPenalidade.ocorrencias.length - 1; i >= 0; i--) {
			for (var j = $scope.listaOcorrencias.length -1; j >= 0; j--) {
				if ($scope.listaOcorrencias[j].id === $scope.eventoPenalidade.ocorrencias[i].registroOcorrencia.id) {
					$scope.listaOcorrencias.splice(j, 1);
				}
			}
		}
	};

	$scope.addRegistroOcorrencia = function(){

		if($scope.eventoPenalidade.registroOcorrencia){

			var eventoPenalidadeRegistroOcorrencia = {
					registroOcorrencia : $scope.eventoPenalidade.registroOcorrencia
			};

			$scope.eventoPenalidade.ocorrencias.push(eventoPenalidadeRegistroOcorrencia);

			$scope.listaOcorrencias = $filter('idNotObject')($scope.listaOcorrencias, $scope.eventoPenalidade.registroOcorrencia);

			$scope.eventoPenalidade.registroOcorrencia = null;
		} else {
			$scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.SELECIONE_PARA_ADICIONAR'));
			return;
		}

	};

	$scope.removerRegistroOcorrencia = function() {
		var listaSelecionados = [];
		angular.forEach($scope.eventoPenalidade.ocorrencias, function (item) {
			if(item.$checked){
				listaSelecionados.push(item);
			}
        });

		if(listaSelecionados.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {
					for(var i = $scope.eventoPenalidade.ocorrencias.length - 1; i >= 0; i--) {
						if ($scope.eventoPenalidade.ocorrencias[i].$checked) {
							$scope.eventoPenalidade.ocorrencias.splice(i, 1);
						}
					}

					atualizarListaOcorrencias();

					$scope.$modalConfirmInstance.dismiss('cancel');
				}
			});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	function atualizarListaOcorrencias() {

		$scope.listaOcorrencias = [];

		$scope.listaOcorrencias = $scope.listaOcorrenciasFixa;

		angular.forEach($scope.eventoPenalidade.ocorrencias, function (itemOcor){
			$scope.listaOcorrencias = $filter('idNotObject')($scope.listaOcorrencias, itemOcor.registroOcorrencia);
		});
	};

	$scope.selectAllOcorrencias = function(value) {
		angular.forEach($scope.eventoPenalidade.ocorrencias, function (item) {
			item.$checked = value;
        });
	};

	$scope.saveOrUpdate = function (form){

		form.$submitted = true;

		if(form.$invalid){
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		} else if ($scope.listaOcorrencias.length > 0 && $scope.eventoPenalidade.ocorrencias.length < 1) {
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		} else {
			$scope.penalidadeTemp = {
					documentos : []
			};

			angular.forEach($scope.eventoPenalidade.documentos, function(documento, key) {

				$scope.penalidadeTemp.documentos.push({
					uploadsDocumento : clone(documento.uploadsDocumento),
					numero : clone(documento.numero)
				});
				delete documento.uploadsDocumento;
			});

			delete $scope.eventoPenalidade.registroOcorrencia;
			$scope.setLoadingSalva(true);
			EventoPenalidadeRepository.save($scope.eventoPenalidade).then(function(result) {

				$scope.eventoPenalidade = result.originalElement;
				$scope.setLoading(false);
				$scope.showAlert("success", $translate.instant('CONTRATOS.MSG.SUCESSO_SALVAR_PENALIDADE'));

				if($scope.eventoPenalidade.documentos){

					// preencher lista de documentos temporarios para verificacao dos anexos. Verifica qual anexo e de qual documento pelo numero do documento
					angular.forEach($scope.eventoPenalidade.documentos, function(documento, key) {
						if (documento.uploadsDocumento) {
							$scope.penalidadeTemp.documentos.push({
								uploadsDocumento: clone(documento.uploadsDocumento),
								numero: clone(documento.numero)
							});
							// deleta esse atributo por causa de um erro de ciclo no json
							delete documento.uploadsDocumento;
						}
					});
					// depois de salvar, ja tendo uma referencia do id da mesma, esse metodo ira salvar os documentos se houver
					salvarListaDocumentos($scope.penalidadeTemp);
				}
				$scope.resetForm();
				$scope.$modalInstance.dismiss('cancel');
				$scope.buscaOcorrencias($scope.contrato);
			});
		};
	};


	// metodo utilizado para percorrer a lista de documentos persistidos e verificar com a lista de documentos temporarios para verificacao de qual anexo pertence a qual documento
	var salvarListaDocumentos = function(eventoPenalidade) {

		angular.forEach($scope.eventoPenalidade.documentos, function(documentoSalvo, key) {
			// verifica se a entidade documento foi salva
			if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
				// percorre todos os documentos temporarios para verificacao do numero do documento
				angular.forEach(eventoPenalidade.documentos, function(documentoTransient, key) {
					// verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
					if (documentoSalvo.numero === documentoTransient.numero) {
						//salva o anexo ao documento que tiver o mesmo numero do documento transient
						salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
					}
				});
			}
		});
	};

	// metodo responsavel para salvar os anexos depois de ter salvo o registro de ocorrencia, utilizando a referencia do id registroOcorrencia.id
	var salvarUpload = function(documento, uploadsDocumento) {
		$scope.uploader.queue = uploadsDocumento;
		// percorre a lista de uploads a serem enviados
		angular.forEach($scope.uploader.queue, function(item, key) {
			item.url = item.url + documento.id;
		});
		if ($scope.uploader.queue && $scope.uploader.queue.length > 0) {
			// realiza o upload se existir no minimo um arquivo para upload
			$scope.uploader.uploadAll();
		}
	};

}]);

