'use strict';

citApp.controller('EventoRegistroNaoConformidadeController', ['$scope', '$timeout', '$translate', 'EventoRegistroNaoConformidadeRepository',
                                                              'ContratoRepository', 'DominioRepository', 'FileUploader','TermoReferenciaSancaoRepository','$filter',
                                              function EventoRegistroNaoConformidadeController($scope, $timeout, $translate, EventoRegistroNaoConformidadeRepository,
                                            		  ContratoRepository, DominioRepository,FileUploader, TermoReferenciaSancaoRepository, $filter) {


	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 1,
        url : '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	$scope.resetForm = function(form) {

		if(!$scope.contrato.termoReferencia){
        	$scope.showAlert('warning', $translate.instant('CONTRATOS.LABEL.CONTRATO_SEM_VINCULO_TERMO'));
		}else{
			$scope.tiposNaoConformidade = $scope.contrato.termoReferencia.sancoes;
		}

		if($scope.contrato == undefined){
			$scope.contrato= angular.copy($scope.contrato);
		}

		if($scope.evento){
			$scope.setLoadingGet(true);
			EventoRegistroNaoConformidadeRepository.get($scope.evento.id).then(function(result){
				$scope.registroNaoConformidade = result.originalElement;
				$scope.registroNaoConformidade.contrato = $scope.contrato;
				$scope.registroNaoConformidade.dataNaoConformidade = $filter('date')($scope.registroNaoConformidade.dataNaoConformidade, "dd/MM/yyyy");
				$scope.setLoading(false);
			});
		}

		if($scope.edit && !$scope.evento){
			$scope.registroNaoConformidade = {};
			$scope.registroNaoConformidade = {contrato: $scope.contrato};
		}
		if(form != undefined){
			$timeout(function(){
				form.$submitted = true;
				form.$setPristine();
			});
		}
	};

	$scope.salvarEvento = function(form){
		form.$submitted = true;
		if(form.$valid){

	        if(!$scope.evento && !isDataDentroPeriodo($scope.registroNaoConformidade.contrato.dataVigenciaInicial,
	                $scope.registroNaoConformidade.contrato.dataUltimaVigenciaFinal, $scope.registroNaoConformidade.dataNaoConformidade)){
	        	$scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.DATA_NAO_CONFORMIDADE_INVALIDA'));
	        	return;
	        }

	        EventoRegistroNaoConformidadeRepository.aplicacoesNaoConformidadeContrato($scope.contrato.id).then(function(result) {
				var eventosNaoConformidadeContrato = result;
				if(eventosNaoConformidadeContrato && eventosNaoConformidadeContrato.length > 0){
					var advertenciaGerado = false;
					var multaGerado = false;
					var suspensaoGerado = false;
					angular.forEach(eventosNaoConformidadeContrato, function(evento) {
						if(evento == 'TIPO_NOTIFICACAO_ADVERTENCIA'){
							advertenciaGerado = true;
						}else if(evento == 'TIPO_NOTIFICACAO_MULTA'){
							multaGerado = true;
						}else if(evento == 'TIPO_NOTIFICACAO_SUSPENSAO_TEMPORARIA'){
							suspensaoGerado = true;
						}
					});
					if($scope.registroNaoConformidade.tipoNaoConformidade.dominioTipoSancao.nome == 'TIPO_NOTIFICACAO_MULTA'){
						if(!advertenciaGerado){
							$scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.EVENTO_NAO_CONFORMIDADE_CRIAR_ADVERTENCIA'));
							return;
						}
					}else if($scope.registroNaoConformidade.tipoNaoConformidade.dominioTipoSancao.nome == 'TIPO_NOTIFICACAO_SUSPENSAO_TEMPORARIA'){
						if(!multaGerado){
							$scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.EVENTO_NAO_CONFORMIDADE_CRIAR_MULTA'));
							return;
						}
					}else if($scope.registroNaoConformidade.tipoNaoConformidade.dominioTipoSancao.nome == 'TIPO_NOTIFICACAO_DECLARACAO_INIDONEIDADE'){
						if(!suspensaoGerado){
							$scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.EVENTO_NAO_CONFORMIDADE_CRIAR_SUSPENSAO'));
							return;
						}
					}

					salvarEventoValidao();

				}else{
					if($scope.registroNaoConformidade.tipoNaoConformidade.dominioTipoSancao.nome != 'TIPO_NOTIFICACAO_ADVERTENCIA'){
			        	$scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.EVENTO_NAO_CONFORMIDADE_CRIAR_ADVERTENCIA'));
			        	return;
					}

					salvarEventoValidao();
				}

			});

		}else{
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		}
	};

	  var salvarEventoValidao = function(){
      	$scope.setLoadingSalva(true);

		if($scope.registroNaoConformidadeTemp == null){
			$scope.registroNaoConformidadeTemp = {
					documentos : []
			};
			angular.forEach($scope.registroNaoConformidade.documentos, function(documento, key) {
				$scope.registroNaoConformidadeTemp.documentos.push({
					uploadsDocumento : clone(documento.uploadsDocumento),
					numero : clone(documento.numero)
				});
				delete documento.uploadsDocumento;
			});
		}
			EventoRegistroNaoConformidadeRepository.save($scope.registroNaoConformidade).then(function(result) {
				$scope.registroNaoConformidade = result.originalElement;

				angular.forEach($scope.registroNaoConformidade.documentos, function(documentoSalvo, key) {
					if(documentoSalvo !== undefined && documentoSalvo.id !== undefined){
						angular.forEach($scope.registroNaoConformidadeTemp.documentos, function(documentoTransient, key) {
							if(documentoSalvo.numero === documentoTransient.numero){
								salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
							}
						});
					}
				});

				$scope.setLoading(false);
				$scope.$modalInstance.dismiss('cancel');
				$scope.showAlert("success", $translate.instant("MSG.MG001"));
				$scope.buscaOcorrencias($scope.contrato);
			});
	  }

	  var salvarUpload = function(documento, uploadsDocumento){
		  if (uploadsDocumento != undefined && uploadsDocumento != null) {
			  $scope.uploader.queue = uploadsDocumento;
			  angular.forEach($scope.uploader.queue, function(item, key) {
				  item.url =  item.url + documento.id;
			  });
			  $scope.uploader.uploadAll();
			  $scope.uploader.isUploading = false;
		  }
	  };

	  $scope.calcularValorMulta = function(){
		$scope.registroNaoConformidade.valorMulta =  parseFloat(($scope.contrato.valorContrato * $scope.registroNaoConformidade.percentualMulta).toFixed(2));
	  };

	$scope.resetForm();

}]);
