'use strict';

citApp.controller('EventoRescisaoContratoController', ['$scope', '$timeout', '$translate', 'EventoRescisaoContratoRepository', 'ContratoRepository', 'DominioRepository', 'FileUploader','ContratoEventoRepository',
                                              function EventoRescisaoContratoController($scope, $timeout, $translate, EventoRescisaoContratoRepository, ContratoRepository, DominioRepository,
                                            		  FileUploader, ContratoEventoRepository) {


	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 1,
        url : '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	  $scope.filterCriteria = {
	      start : 1,
	      dir : 'asc',
	      sort : 'dataEvento',
	      limit : 5,
	  };

	  $scope.fields = [ 'id', 'tipoEvento.descricao', 'dataEvento'];

	  $scope.headers = [{
		    title : $translate.instant('CONTRATOS.RESCISAO_CONTRATO.TIPO_ACAO'),
		    value : 'tipoEvento.descricao',
		    align : 'left'
		  },{
		    title : $translate.instant('CONTRATOS.RESCISAO_CONTRATO.DATA_ACAO'),
		    value : 'dataEvento',
		    date : true
	  }];

	  $scope.filters = [{
		    type : 'string',
		    field : 'tipoEvento.descricao'
		  },{
		    type : 'date-range',
		    field : 'dataEvento',
	  }];

	$scope.resetForm = function(form) {
		if($scope.contratoTemp == undefined){
			$scope.contratoTemp = angular.copy($scope.contrato);
		}
		$scope.listaAcoesContrato = [];
		if($scope.contratoEventos[0] && $scope.contratoEventos[0].eventos){
			angular.forEach($scope.contratoEventos[0].eventos, function(evento, key) {
				$scope.listaAcoesContrato.push(evento.id);
			});
		}
		if($scope.edit){
			$scope.rescisaoContrato = {};
			$scope.rescisaoContrato = {contrato: $scope.contratoTemp};
		}
		if(form != undefined){
			$timeout(function(){
				form.$submitted = true;
				form.$setPristine();
			});
		}
	};

	$scope.salvarEvento = function(form){
		form.$submitted = true;
		if(form.$valid){

	        if(!isDataDentroPeriodo($scope.rescisaoContrato.contrato.dataVigenciaInicial, $scope.rescisaoContrato.contrato.dataUltimaVigenciaFinal, $scope.rescisaoContrato.dataRescisao)) {
	        	$scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.DATA_RESCISAO_INVALIDA'));
	        	return;
	        }

      	  if($scope.rescisaoContrato.devolucaoValor && $scope.rescisaoContrato.valorADevolver && $scope.rescisaoContrato.valorADevolver > $scope.contratoTemp.valorTotalContrato){
    		  $scope.showAlert('warning', $translate.instant('CONTRATOS.MSG.RESCISAO_CONTRATO_VALOR_DEVOLVIDO_MAIOR_CONTRATO'));
          	return;
    	  }

        	$scope.setLoadingSalva(true);

			if($scope.rescisaoContratoTemp == null){
				$scope.rescisaoContratoTemp = {
						documentos : []
				};

				angular.forEach($scope.rescisaoContrato.documentos, function(documento, key) {

					$scope.rescisaoContratoTemp.documentos.push({
						uploadsDocumento : clone(documento.uploadsDocumento),
						numero : clone(documento.numero)
					});
					delete documento.uploadsDocumento;
				});
			}
				EventoRescisaoContratoRepository.save($scope.rescisaoContrato).then(function(result) {
					$scope.rescisaoContrato = result.originalElement;

					angular.forEach($scope.rescisaoContrato.documentos, function(documentoSalvo, key) {
						if(documentoSalvo !== undefined && documentoSalvo.id !== undefined){
							angular.forEach($scope.rescisaoContratoTemp.documentos, function(documentoTransient, key) {
								if(documentoSalvo.numero === documentoTransient.numero){
									salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
								}
							});
						}
					});

					$scope.setLoading(false);
					$scope.$modalInstance.dismiss('cancel');
					$scope.showAlert("success", $translate.instant("MSG.MG001"));
					$scope.buscaOcorrencias($scope.contratoTemp);
				});

		}else{
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		}
	};

	  var salvarUpload = function(documento, uploadsDocumento){
		  if (uploadsDocumento != undefined && uploadsDocumento != null) {
			  $scope.uploader.queue = uploadsDocumento;
			  angular.forEach($scope.uploader.queue, function(item, key) {
				  item.url =  item.url + documento.id;
			  });
			  $scope.uploader.uploadAll();
			  $scope.uploader.isUploading = false;
		  }
	  };

	DominioRepository.findAllDominio('tipoRescisaoContrato').then(function(result) {
		$scope.rescisoesContrato = result;
	});

	$scope.resetForm();

	$scope.gerarTermoRescisaoContrato = function(){
		$scope.url = '/cit-contratos-web/rest/eventoRescisaoContrato/pdfGerarTermoRescisaoContrato?idRescisaoContrato=' + $scope.rescisaoContrato.id;
	    $scope.visualizarRelatorio($scope.url, $translate.instant('CONTRATOS.LABEL.TITULO_DOCUMENTO_RESCISAO_CONTRATO'));
	    $scope.$modalInstance.dismiss('cancel');
	};

}]);
