'use strict';
citApp.controller('NotificacaoContratoController', ['$scope', '$translate', '$timeout', 'DominioRepository', 'NotificacaoContratoRepository',
                                                    'ContratoRepository', 'RegistroOcorrenciaRepository','FileUploader', '$filter', 'NotificacaoContratoOcorrenciaRepository',
                                             function NotificacaoContratoController($scope, $translate, $timeout, DominioRepository,
                                            		 NotificacaoContratoRepository, ContratoRepository, RegistroOcorrenciaRepository, FileUploader, $filter, NotificacaoContratoOcorrenciaRepository) {

	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 99,
        url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#searchNotificacaoContrato').scope().fetchResult();
	};

	DominioRepository.findAllDominio('tipoNotificacaoContratoSancao').then(function(result) {
		$scope.dominioTiposSancaoList = result;
	});

	$scope.limparOcorrencias = function(){
		$scope.listaOcorrenciasContrato = [];
		$scope.listaOcorrenciasContratoOrigem = [];
		$scope.registroOcorrencia = {};
		$scope.notificacao.associarOcorrencia = null;
	};

	$scope.resetForm = function() {
		$scope.notificacao = { emissao : $filter('date')(new Date(), "dd/MM/yyyy")};
		$scope.selectedAll = false;
		$scope.edit = true;
		$timeout(function(){
			$scope.notificacaoContratoForm.$submitted = false;
			$scope.notificacaoContratoForm.$setPristine();
		});
	};

	$scope.saveOrUpdate = function(){
		$scope.notificacaoContratoForm.$submitted = true;
		try{
			if($scope.notificacaoContratoForm.$valid){
				if($scope.notificacao.id != undefined){
					$scope.notificacao.emissao = converterStringUSemDate($scope.notificacao.emissao);
				}
				if($scope.notificacao.associarOcorrencia == undefined){
					$scope.showAlert('error', $translate.instant('NOTIFICACAO_CONTRATO.VALIDA_ASSOCIAR_OCORRENCIA'), " ", false);
					return;
				}else if(!isDataDentroPeriodo($scope.notificacao.contrato.dataVigenciaInicial, $scope.notificacao.contrato.dataUltimaVigenciaFinal, $scope.notificacao.emissao)){
					$scope.showAlert('error', $translate.instant('NOTIFICACAO_CONTRATO.DATA_EMISSAO_INVALIDA'), " ", false);
					return;
				}else{
					$scope.setLoadingSalva(true);

					if($scope.notificaoTemp == null){
						$scope.notificaoTemp = {
								documentos : []
						};
						angular.forEach($scope.notificacao.documentos, function(documento, key) {
							if (documento.uploadsDocumento) {
								documento.uuid = getUuid();
								$scope.notificaoTemp.documentos.push({
									file: documento.uploadsDocumento[0],
									uuid: clone(documento.uuid)
								});
								// deleta esse atributo por causa de um erro de ciclo no json
								delete documento.uploadsDocumento;
							}
						});
					}

					NotificacaoContratoRepository.save($scope.notificacao).then(function(result) {
						$scope.notificacao = result.originalElement;
						$scope.notificacaoContratoForm.$submitted = false;
						salvarListaDocumentos($scope.notificaoTemp);
						$scope.setLoading(false);
						$scope.notificacaoContratoForm.$submitted = false;
						$scope.showAlert("success", $translate.instant('NOTIFICACAO_CONTRATO.SUCESSO_SALVAR_NOTIFICACAO'));
						$scope.$showPageSearchWorkspace($scope.workspace);
						$scope.atualizaPaginaPesquisa();
					});
				}
			}else{
				$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
			}
		}catch(exception){
			$scope.setLoadingSalva(false);
		}
	};

	var salvarListaDocumentos = function(notificaoTemp) {
		angular.forEach($scope.notificacao.documentos, function(documentoSalvo, key) {
			if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
				angular.forEach(notificaoTemp.documentos, function(documentoTransient, key) {
					if (documentoSalvo.numero === documentoTransient.numero) {
						salvarUpload(documentoSalvo, documentoTransient.uploadsDocumento);
					}

				});

			}
		});
	};
    var salvarListaDocumentos = function(notificaoTemp) {
    	$scope.uploader.queue = [];
    	
        angular.forEach($scope.notificacao.documentos, function(documentoSalvo, key) {
            // verifica se a entidade documento foi salva
            if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
                // percorre todos os documentos temporarios para verificacao do numero do documento
                angular.forEach(notificaoTemp.documentos, function(documentoTransient, key) {
                    // verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
                    if (documentoTransient.file && documentoSalvo.uuid === documentoTransient.uuid) {
                        //salva o anexo ao documento que tiver o mesmo numero do documento transient
                    	
                    	documentoTransient.file.url = documentoTransient.file.url + documentoSalvo.id;
                    	$scope.uploader.queue.push(documentoTransient.file);
                    }
                });
            }
        });
        
        if($scope.uploader.queue.length > 0){
        	$scope.uploader.uploadAll();
        }
        delete $scope.notificaoTemp;
    };

	$scope.getNotificacaoContrato = function(notificacaoContrato, edit) {
		$scope.setLoadingGet(true);
		NotificacaoContratoRepository.get(notificacaoContrato.id).then(function(result) {
			$scope.notificacao = result.originalElement;
			$scope.notificacao.contrato = $scope.notificacao.contrato;
			$scope.setCamposContrato($scope.notificacao.contrato, true);
			$scope.edit = edit;
			$scope.setLoading(false);
		});
	};

    $scope.findContrato = function(value) {
        return ContratoRepository.findAutoCompleteContratoByNumeroAndOrganizacao(value, $scope.usuarioLogado.organizacao.id).then(function(result) {
            return result;
        });
    };

    $scope.setCamposContrato = function(item, isGetNotificacao){
    	$scope.listaOcorrenciasContrato = [];
    	RegistroOcorrenciaRepository.findAllOcorrenciasPorContrato(item.id).then(function(result){
    		if(!isGetNotificacao){
    			if(result.length == 0){
    				$scope.notificacao.associarOcorrencia = false;
    			}else{
    				$scope.notificacao.associarOcorrencia = null;
    			}
    		}
    		$scope.listaOcorrenciasContratoOrigem = angular.copy(result);
    		if(isGetNotificacao){
    			if($scope.notificacao.ocorrencias.length == 0){
    				$scope.listaOcorrenciasContrato = angular.copy(result);
    			}else{
    				var isPresent = false;
    				angular.forEach($scope.listaOcorrenciasContratoOrigem, function (itemOrigem) {
    					angular.forEach($scope.notificacao.ocorrencias, function (ocorrencia) {
    						if(ocorrencia.registroOcorrencia.id == itemOrigem.id ){
    							isPresent = true;
    						}
    					});
    					if(!isPresent){
    						$scope.listaOcorrenciasContrato.push(itemOrigem);
    					}
    					isPresent = false;
    				});
    			}
    		}else{
    			$scope.listaOcorrenciasContrato = angular.copy(result);
    		}
    	});
    };

    $scope.verificarListOcorrencia = function(){
    	if($scope.notificacao.contrato == undefined){
    		$scope.showAlert('warning', $translate.instant('NOTIFICACAO_CONTRATO.VALIDA_CONTRATO_NAO_SELECIONA'));
    		$scope.notificacao.associarOcorrencia = false;
    	}else{
    		if($scope.listaOcorrenciasContratoOrigem == undefined || $scope.listaOcorrenciasContratoOrigem.length == 0){
    			$scope.showAlert('warning', $translate.instant('NOTIFICACAO_CONTRATO.VALIDA_CONTRATO_SEM_OCORRENCIA'));
    			$scope.notificacao.associarOcorrencia = false;
    		}
    	}
    };

    $scope.addOcorrenciaTable = function(){
    	if($scope.registroOcorrencia == undefined || $scope.registroOcorrencia.id == undefined){
			$scope.showAlert('warning', $translate.instant('NOTIFICACAO_CONTRATO.SELECIONE_OCORRENCIA'));
			return ;
    	}else{
    		if($scope.notificacao.ocorrencias == undefined){
    			$scope.notificacao.ocorrencias = [];
    		}
    		var ocorrencia = { registroOcorrencia : $scope.registroOcorrencia};
    		$scope.notificacao.ocorrencias.push(ocorrencia);
    		var i = 0;
    		var index = 0;
    		angular.forEach($scope.listaOcorrenciasContrato, function (item) {
    			if($scope.registroOcorrencia.id == item.id){
    				index = i;
    			}
    			i ++;
    		});
    		$scope.listaOcorrenciasContrato.splice(index, 1);
    		$scope.registroOcorrencia = {};
    	}
    };

	$scope.selectedAllOcorrencias = function(value) {
		angular.forEach($scope.notificacao.ocorrencias, function (item, key) {
			item.$selected = value;
        });
	};

	var recuperarNecessidadesSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.notificacao.ocorrencias, function (item) {
			if(item.$selected){
				lista.push(item);
			}
        });
		return lista;
	};

	$scope.removerOcorrencia = function(){
		var lista = recuperarNecessidadesSelecionadas();
		if(lista.length > 0){

			var listIdsRemocao = [];
			angular.forEach(lista, function(item){
				if (item.id){
					listIdsRemocao.push(item.id);
				}
			});

			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {

					if (listIdsRemocao && listIdsRemocao.length > 0) {
						NotificacaoContratoOcorrenciaRepository.removeAll(listIdsRemocao).then();
					}

					angular.forEach(lista, function (itemRem, key) {
						var i = 0;
						var index = 0;
						angular.forEach($scope.notificacao.ocorrencias, function (item) {
							if(itemRem.id == item.id){
								index = i;
							}
							i ++;
						});
						delete itemRem.$selected;

						angular.forEach($scope.listaOcorrenciasContratoOrigem, function (itemOrigem) {
							if(itemRem.registroOcorrencia.id == itemOrigem.id){
								$scope.listaOcorrenciasContrato.push(itemOrigem);
							}
						});

						$scope.notificacao.ocorrencias.splice(index, 1);
						$scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
					});
					$scope.$modalConfirmInstance.dismiss('cancel');
	            }
			});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

    $scope.gerarRelatorioNotificacaoContrato = function() {
        $scope.url = '/cit-contratos-web/rest/notificacaoContrato/gerarTermoNotificacaoContrato?idEncerramentoContrato='+ $scope.notificacao.id;
        $scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.NOTIFICACAO"));
    };

}]);


