'use strict';
citApp.controller('NotificacaoContratoListController', ['$scope', '$translate', '$timeout', 'NotificacaoContratoRepository', 'DominioRepository',
          function NotificacaoContratoListController($scope, $translate, $timeout, NotificacaoContratoRepository, DominioRepository) {

    $scope.resetForm = function() {
        angular.element("#editNotificacaoContrato").scope().resetForm();
    };

	$scope.headers = [
	{
		value : 'emissao',
		title : $translate.instant('NOTIFICACAO_CONTRATO.DATA_EMISSAO'),
		tamanho : 20,
		filter : 'dateBR'
	},{
		value : 'codigo',
		title : $translate.instant('NOTIFICACAO_CONTRATO.NUMERO_NOTIFICACAO_ABREV'),
		tamanho : 25,
		typeMask: 'MASK_10_CODE_ANO',
		mask : [new RegExp(/(\d{2})(\d{1,4})$/), '$1/$2']
	},{
		value : 'dominioTipoSancao.descricao',
		title : $translate.instant('NOTIFICACAO_CONTRATO.TIPO_SANCAO'),
		tamanho : 25
	},{
		value : 'contrato.numeroAnoContrato',
		title : $translate.instant('NOTIFICACAO_CONTRATO.NUMERO_CONTRATO_ABREV'),
		tamanho : 25,
		typeMask: 'MASK_CONTRATO',
		mask : [new RegExp(/(\d{3})(\d{1,4})$/), '$1/$2']
	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
			start : 1,
			dir : 'asc',
			sort : 'id',
			limit : 10,
			fields: ['id', 'emissao', 'codigo', 'dominioTipoSancao.descricao', 'contrato.numeroAnoContrato'],
			filters : [{type: 'date-range', field: 'emissao'},
			           {type: 'string', field: 'codigo'},
			           {type: 'string', field: 'dominioTipoSancao.descricao', listaDominio : []},
			           {type: 'string', field: 'numeroAnoContrato'}]
	};

	DominioRepository.findAllDominio('tipoNotificacaoContratoSancao').then(function(result) {
		$scope.filterCriteria.filters[2].listaDominio = result;
	});

	$scope.abrirNotificacao = function(edit){
		var notificao = $scope.notificacaoChecked;

		if(!edit && !notificao) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && !notificao) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}

		angular.element('#editNotificacaoContrato').scope().getNotificacaoContrato(notificao, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};

	$scope.remover = function() {
		$scope.showAlert('warning', $translate.instant('NOTIFICACAO_CONTRATO.REMOCAO_NAO_PERMITIDA'));
	};

    $scope.gerarRelatorioNotificacaoContrato = function() {
        $scope.url = '/cit-contratos-web/rest/notificacaoContrato/gerarTermoNotificacaoContrato?idEncerramentoContrato='+ $scope.notificacaoChecked.id;
        $scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.NOTIFICACAO"));
    };

}]);