'use strict';
citApp.controller('ObjetivoEstrategicoController', ['$scope', '$translate', '$timeout', 'ObjetivoEstrategicoRepository', 'ObjetivoEstrategicoNecessidadeRepository', '$sce',
                                             function ObjetivoEstrategicoController($scope, $translate, $timeout, ObjetivoEstrategicoRepository, ObjetivoEstrategicoNecessidadeRepository, $sce) {

	$scope.objetivoEstrategico = {
			necessidadesElencadas : []
	};

	$scope.listaNecessidade = [];
	$scope.selectedAll = false;

	$scope.objetivoEstrategicoNecessidade = {};

	// Limpa formulário para novo cadastro
	$scope.resetForm = function() {
		$scope.limparObjetivoEstrategico();
		$scope.edit = true;
		$scope.pgEdit = true;
		$scope.atualizarListaSizeSelect();
		$timeout(function(){
			limparObjetivoEstrategicoForm();
		});
	};

	function limparObjetivoEstrategicoForm() {
		$scope.objetivoEstrategicoForm.$submitted = false;
		$scope.objetivoEstrategicoForm.$setPristine();
	}

	// Atualiza pagina de pesquisa
	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#searchObjetivoEstrategico').scope().fetchResult();
	};

    $scope.openModalNecessidade = function() {

        $scope.editNecessidade = true;
        $scope.showButtonSalvar = false;

        $scope.objetivoEstrategicoNecessidade = {};

        $timeout(function() {
        	$scope.$openModal('modal-necessidade.html', 'lg');
        });
    };

	$scope.addNecessidade = function(formDialogNecessidade, continuar) {

		formDialogNecessidade.$submitted = true;

		var necessidade = $scope.objetivoEstrategicoNecessidade;

		if (formDialogNecessidade.$valid) {
			if (necessidade.$index != undefined && necessidade.$index != null) {
				$scope.objetivoEstrategico.necessidadesElencadas[necessidade.$index] = necessidade;
			} else {
				$scope.objetivoEstrategico.necessidadesElencadas.push(necessidade);
			}

			$scope.limparObjetivoEstrategicoNecessidade();
			$scope.atualizarListaSizeSelect();
			if (continuar) {
				formDialogNecessidade.$submitted = false;
				formDialogNecessidade.$setPristine();
			} else {
				$scope.$modalInstance.dismiss('cancel');
			}
		} else {
			$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.NECESSIDADE_NAO_PREENCHIDA'));
		}
	};

	// MODAL QUE CONFIRMA REMOÇÃO DO OBJETIVO ESTRATÉGICO
	$scope.remove = function(objetivoEstrategico){
		$scope.$openModalConfirm({
			message: $translate.instant('CONTRATOS.MSG.CONFIRMA_EXCLUSAO_OBJETIVO'),
			callback: function () {
				ObjetivoEstrategicoRepository.remove(objetivoEstrategico).then(function() {
					$scope.limparObjetivoEstrategico();

					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.showAlert("success", $translate.instant('MSG.SUCESSO_EXCLUSAO_OBJETIVO_ESTRATEGICO'));
					angular.element('#searchObjetivoEstrategico').scope().fetchResult();

					$scope.resetForm();
				});
			}
		});
	};

	// SALVA O OBJETIVO ESTRATEGICO
	$scope.saveOrUpdate = function(){
		$scope.objetivoEstrategicoForm.$submitted = true;

		try{
			//verifica se o formulario está valido para salvar
			if($scope.objetivoEstrategicoForm.$valid){

				$scope.setLoadingSalva(true);

				ObjetivoEstrategicoRepository.save($scope.objetivoEstrategico).then(function(result) {
					$scope.objetivoEstrategico = result.originalElement;
					$scope.showAlert("success", $translate.instant('MSG.SUCESSO_SALVAR_OBJETIVO_ESTRATEGICO'));
					$scope.objetivoEstrategicoForm.$submitted = false;

					$scope.$showPageSearchWorkspace($scope.workspace);
					$scope.atualizaPaginaPesquisa();
				});
				$scope.setLoading(false);
			}else{
				//Mensagem de erro de campos obrigatorios não preenchidos
				$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
			}
		}catch(exception){
			// se der algum erro no save seta o loading false
			$scope.setLoadingSalva(false);
		}
	};

	// Limpa a necessidade preenchida
	$scope.limparObjetivoEstrategicoNecessidade = function(){
		$scope.objetivoEstrategicoNecessidade = {};
	};

	// Limpa o formulario preenchido
	$scope.limparObjetivoEstrategico = function(){
		$scope.objetivoEstrategico = {
				necessidadesElencadas : []
		};
		$scope.limparObjetivoEstrategicoNecessidade();
	};

	// Seleciona todos as necessidades na lista
	$scope.checkAll = function(value) {
		$scope.listaNecessidade = [];
		angular.forEach($scope.objetivoEstrategico.necessidadesElencadas, function (item, key) {
			item.$selected = value;
			item.$index = key;
			$scope.listaNecessidade.push(item);
        });
		if(!value){
			$scope.listaNecessidade = [];
		}
		$scope.atualizarListaSizeSelect();
	};

	$scope.checkNecessidade = function(item){
		if(item.$selected){
			$scope.listaNecessidade.push(item);
		}else{
			$scope.listaNecessidade.splice($scope.listaNecessidade.indexOf(item), 1);
		}

	};

	// Abre modal perguntando se deseja excluir e exclui os elementos selecionados na lista de necessidades caso confirmado
	$scope.excluirNecessidadesSelecionados = function(){
		var lista = recuperarNecessidadesSelecionadas();
		if(lista.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {
					angular.forEach(lista, function (itemRem, key) {
						angular.forEach($scope.objetivoEstrategico.necessidadesElencadas, function (item, idx) {
							if ($scope.objetivoEstrategico.necessidadesElencadas.length === 1 && item.id) {
								$scope.showAlert("warning", $translate.instant('CONTRATOS.VALIDACAO.NECESSIDADES_VAZIAS'));
								return;
							}

							if (item.$selected) {
								if (item.id) {
									ObjetivoEstrategicoNecessidadeRepository.remove(item).then(function(result) {
										$scope.objetivoEstrategico.necessidadesElencadas.splice(idx, 1);
										$scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
									});
								} else {
									$scope.objetivoEstrategico.necessidadesElencadas.splice(idx, 1);
									$scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
								}
							}
						});
					});
					$scope.atualizarListaSizeSelect();
					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.listaNecessidade = [];
	            }
			});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	$scope.editarNecessidade = function(edit) {

		$scope.editNecessidade = edit;
		$scope.showButtonSalvar = true;

		var lista = recuperarNecessidadesSelecionadas();
		if (lista.length === 0) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}
		if (lista.length > 1) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_APENAS_UM_ITEM_PARA_EDICAO'));
			return;
		}

		$scope.objetivoEstrategicoNecessidade = clone(lista[0]);

		$timeout(function() {
			$scope.$openModal('modal-necessidade.html', 'lg');
		});
	};

	//Recupera todas as necessidades selecionadas na lista
	var recuperarNecessidadesSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.objetivoEstrategico.necessidadesElencadas, function (item) {
			if(item.$selected){
				lista.push(item);
			}
        });
		return lista;
	};

	// Consulta entidade e mostra no formulario
	$scope.getObjetivoEstrategico = function(objetivoEstrategico, edit) {
		$scope.setLoadingGet(true);
		$scope.selectedAll = false;
		$scope.listaNecessidade = [];
		ObjetivoEstrategicoRepository.get(objetivoEstrategico).then(function(result) {
			$scope.objetivoEstrategico = result.originalElement;

			if($scope.objetivoEstrategico.dataBloqueio === null || $scope.objetivoEstrategico.dataBloqueio === undefined) {
				$scope.edit = edit;
			} else {
				$scope.edit = false;
			}

			$scope.atualizarListaSizeSelect();
			$scope.pgEdit = edit;
			$scope.setLoading(false);
		});
	};

	$scope.atualizarListaSizeSelect = function() {
		if(!$scope.listaSize){
			$scope.listaSize = 0;
		};
		var lista = recuperarNecessidadesSelecionadas();
		$scope.listaSize = lista.length;
	};

}]);


