'use strict';

citApp.controller('OrdemFornecimentoController', ['$scope', 'OrdemFornecimentoRepository', '$translate', '$timeout', 'DominioRepository', 'ContratoRepository', 'ContratoItemRepository',
                                                  '$filter', 'FileUploader',
                                             function OrdemFornecimentoController($scope, OrdemFornecimentoRepository, $translate, $timeout, DominioRepository, ContratoRepository,
                                            		 ContratoItemRepository, $filter, FileUploader) {

	$scope.edit = true;
	$scope.ordemFornecimento = {};

	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 99,
        url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	DominioRepository.findAllDominio('tipoSituacaoOrdemFornecimento').then(function(result) {
        $scope.listaSituacaoOrdemFornecimentos = result;
    });

	$scope.atualizaPaginaPesquisa = function () {
		$scope.$showPageSearchWorkspace($scope.workspace);
		angular.element('#searchOrdemFornecimento').scope().fetchResult();
	};
	
	$scope.findContrato = function(value) {
		return ContratoRepository.findContrato(value, $scope.usuarioLogado.organizacao.id).then(function(result) {
			return result;
		});
	};

	// limpa o formulario
    $scope.resetForm = function() {
        $scope.edit = true;
        $scope.ordemFornecimento = {};
        $scope.atualizarListaSizeSelect();
        $timeout(function() {
            $scope.ordemFornecimentoForm.$submitted = false;
        });
    };

    $scope.visualizarEditar = function(idOrdemFornecimento, edit) {
    	$scope.resetForm();
        $scope.setLoadingGet(true);

        OrdemFornecimentoRepository.get(idOrdemFornecimento).then(function(result) {

            $scope.ordemFornecimento = result.originalElement;
            $scope.atualizarListaSizeSelect();
            $scope.edit = edit;

            $scope.setLoading(false);
        });
    };

    // recupera os itens selecionados
    $scope.getItensChecked = function() {
        var itensChecked = [];

        if ($scope.ordemFornecimento.itensBensServicos) {
        	for (var i = 0; i < $scope.ordemFornecimento.itensBensServicos.length; i++) {
            	if ($scope.ordemFornecimento.itensBensServicos[i].$checked) {
            		itensChecked.push(angular.copy($scope.ordemFornecimento.itensBensServicos[i]));
            	}
            }
        }

        return itensChecked;
    };

    $scope.openModalItensBensServicos = function() {

        $scope.editItem = true;
        $scope.showButtonSalvarItem = false;

        $scope.item = {};

        if ($scope.ordemFornecimento.contrato) {
        	ContratoItemRepository.findAllByContrato($scope.ordemFornecimento.contrato.id).then(function(result) {
            	$scope.itensContrato = result;
            });
        } else {
        	$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_CONTRATO_ESPECIFICACAO'));
        	return;
        }

        $timeout(function() {
        	$scope.$openModal('modal-itens-bens-servicos.html', 'lg');
        });
    };

    $scope.abrirItensBensServicos = function(edit) {

        $scope.editItem = edit;
        $scope.showButtonSalvarItem = edit;

        $scope.item = null;
        // recupera a EstudoTecnico selecionada
        var itensSelecionados = $scope.getItensChecked();

        if (itensSelecionados.length > 1) {
        	$scope.showAlert('warning', edit? $translate.instant('MSG.SELECIONE_APENAS_UM_ITEM_PARA_EDICAO') : $translate.instant('MSG.SELECIONE_APENAS_UM_ITEM_PARA_VISUALIZACAO'));
			return;
        }

		if(itensSelecionados.length === 0) {
			$scope.showAlert('warning', edit? $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO') : $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		ContratoItemRepository.findAllByContrato($scope.ordemFornecimento.contrato.id).then(function(result) {
        	$scope.itensContrato = result;
        });

		$scope.item = itensSelecionados[0];

		$scope.item.dataInicioPrevisto = $filter('date')( $scope.item.dataInicioPrevisto, "dd/MM/yyyy");;
		$scope.item.dataTerminoPrevisto = $filter('date')( $scope.item.dataTerminoPrevisto, "dd/MM/yyyy");;

        $timeout(function() {
        	$scope.$openModal('modal-itens-bens-servicos.html', 'lg');
        });
    };

    // ADICIONAR item
    $scope.salvaItem = function(formDialogItem, continuarEditando) {

    	formDialogItem.$submitted = true;

        //verifica se o formulario está valido para salvar
        if (formDialogItem.$invalid) {
        	//Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
            return;
        }

    	if(!isPeriodoValido($scope.item.dataInicioPrevisto, $scope.item.dataTerminoPrevisto)) {
			$scope.showAlert('warning', $translate.instant('VALIDACAO.DATA_INICIO_PREVISTO_MAIOR_DATA_TERMINO_PREVISTO_CONTRATO'));
			return;
		}
    	
    	if(!isPeriodoValido($scope.ordemFornecimento.contrato.dataVigenciaInicial, $scope.item.dataInicioPrevisto) ||
    			!isPeriodoValido($scope.item.dataTerminoPrevisto, $scope.ordemFornecimento.contrato.dataUltimaVigenciaFinal)) {
			$scope.showAlert('warning', $translate.instant('VALIDACAO.PERIODO_INICIO_TERMINO_DENTRO_VIGENCIA_CONTRATO'));
			return;
		}
    	
    	if ($scope.item.quantidade == 0) {
    		$scope.showAlert('warning', $translate.instant('VALIDACAO.QUANTIDADE_MAIOR_ZERO'));
    		return;
    	}
    	
    	var qntParc = $scope.item.quantidade;
    	
    	if ($scope.ordemFornecimento.itensBensServicos) {
    		for (var i = 0; i < $scope.ordemFornecimento.itensBensServicos.length; i++) {
				if (!$scope.ordemFornecimento.itensBensServicos[i].$checked && $scope.ordemFornecimento.itensBensServicos[i].itemBemServico.id == $scope.item.itemBemServico.id) {
					qntParc = parseFloat(qntParc) + parseFloat($scope.ordemFornecimento.itensBensServicos[i].quantidade);
            	}
            }
    	}
    	
    	if (qntParc > $scope.item.itemBemServico.quantidade) {
    		
    		var qntRestante = parseFloat($scope.item.itemBemServico.quantidade) - (parseFloat(qntParc) - parseFloat($scope.item.quantidade));
    		
    		if (qntRestante < 0) {
    			qntRestante = 0;
    		}
    		
    		$scope.showAlert('warning', $translate.instant('VALIDACAO.QUANTIDADE_INSERIDA_ITEM_MAIOR_QUANTIDADE_CONTRATO') + qntRestante);
			return;
    	}
    	
    	if ($scope.item.itemBemServico.originalElement) {
    		$scope.item.itemBemServico = $scope.item.itemBemServico.originalElement;
    	}
    	
    	if ($scope.ordemFornecimento.itensBensServicos) {
    		if ($scope.item.$checked) {
    			for (var i = 0; i < $scope.ordemFornecimento.itensBensServicos.length; i++) {
    				if ($scope.ordemFornecimento.itensBensServicos[i].$checked) {
                		$scope.ordemFornecimento.itensBensServicos[i] = $scope.item;
                	}
                }
    		} else {
    			$scope.item.$index = $scope.ordemFornecimento.itensBensServicos.length + 1;
    			$scope.ordemFornecimento.itensBensServicos.push($scope.item);
    		}
    	} else {
    		$scope.item.$index = 1;
    		$scope.ordemFornecimento.itensBensServicos = [$scope.item];
    	}
    	
    	var qnt = 0;
		var total = 0;
		
		for (var i = 0; i < $scope.ordemFornecimento.itensBensServicos.length; i++) {
			var item = $scope.ordemFornecimento.itensBensServicos[i];
			qnt = parseFloat(qnt) + parseFloat(item.quantidade);
			total = parseFloat(total) + parseFloat(item.valorTotalItem);
        }
		
		$scope.quantidadeTotal = qnt;
		$scope.totalGeral = total;
    	
    	if(continuarEditando){
    		formDialogItem.$submitted = false;
    		formDialogItem.$setPristine();
    		$scope.item = {};
        } else {
        	$scope.$modalInstance.dismiss('cancel');
        }
		$scope.atualizarListaSizeSelect();
    };

    $scope.removerItensBensServicos = function() {
        // recupera a EstudoTecnico selecionada
    	var itensSelecionados = $scope.getItensChecked();

        if (itensSelecionados.length > 0) {
            $scope.$openModalConfirm({
                message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
                callback: function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');

                    var itens = [];
                    for (var i = 0; i < $scope.ordemFornecimento.itensBensServicos.length; i++) {
                    	if (!$scope.ordemFornecimento.itensBensServicos[i].$checked) {
                    		itens.push($scope.ordemFornecimento.itensBensServicos[i]);
                    	}
                    }
                    $scope.ordemFornecimento.itensBensServicos = itens;
                    $scope.atualizarListaSizeSelect();
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                }
            });
        } else {
            $scope.showAlert('warning', $translate.instant('MSG.SELECIONE_AO_MENOS_UM_ITEM_PARA_SER_REMOVIDO'));
        }
    };

    //Salva a ordem de fornecimento
    $scope.saveOrUpdate = function() {
        // atributo para visualizar os itens obrigatorios na tela
        $scope.ordemFornecimentoForm.$submitted = true;

        // verifica formulario valido
        if ($scope.ordemFornecimentoForm.$valid) {

        	if (!$scope.ordemFornecimento.itensBensServicos || $scope.ordemFornecimento.itensBensServicos.length == 0) {
        		$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        		return;
        	}

        	if ($scope.ordemFornecimento.dominioSituacao && $scope.ordemFornecimento.dominioSituacao.originalElement) {
        		$scope.ordemFornecimento.dominioSituacao = $scope.ordemFornecimento.dominioSituacao.originalElement;
        	}

            $scope.setLoadingSalva(true);

            var ordemFornecimentoTemp = {
                documentos: []
            };

            // preencher lista de documentos temporarios para verificacao dos anexos. verificar qual anexo e de qual documento pelo numero do documento
			angular.forEach($scope.ordemFornecimento.documentos, function(documento, key) {
				if (documento.uploadsDocumento) {
					documento.uuid = getUuid();
					ordemFornecimentoTemp.documentos.push({
						file: documento.uploadsDocumento[0],
						uuid: clone(documento.uuid)
					});
					// deleta esse atributo por causa de um erro de ciclo no json
					delete documento.uploadsDocumento;
				}
			});

            OrdemFornecimentoRepository.save($scope.ordemFornecimento).then(function(result) {

                $scope.ordemFornecimento = result.originalElement;
                // depois de salvar a ordem de fornecimento ja tendo uma referencia do id, esse metodo ira salvar os documentos se existir
                salvarListaDocumentos(ordemFornecimentoTemp);
                $scope.atualizaPaginaPesquisa();
                $scope.setLoading(false);

                $scope.showAlert("success", $translate.instant("MSG.SUCESSO_ORDEM_FORNECIMENTO"));

            });
        } else {
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'));
        }
    };

    // metodo utilizado para percorrer a lista de documentos persistidos e verificar com a lista de documentos temporarios para verificacao de qual anexo pertence a qual documento
    var salvarListaDocumentos = function(ordemFornecimentoTemp) {
    	$scope.uploader.queue = [];
    	
        angular.forEach($scope.ordemFornecimento.documentos, function(documentoSalvo, key) {
            // verifica se a entidade documento foi salva
            if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
                // percorre todos os documentos temporarios para verificacao do numero do documento
                angular.forEach(ordemFornecimentoTemp.documentos, function(documentoTransient, key) {
                    // verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
                    if (documentoTransient.file && documentoSalvo.uuid === documentoTransient.uuid) {
                        //salva o anexo ao documento que tiver o mesmo numero do documento transient
                    	
                    	documentoTransient.file.url = documentoTransient.file.url + documentoSalvo.id;
                    	$scope.uploader.queue.push(documentoTransient.file);
                    }
                });
            }
        });
        
        if($scope.uploader.queue.length > 0){
        	$scope.uploader.uploadAll();
        }
        delete $scope.ordemFornecimentoTemp;
    };

    //Watch para buscar o gestor do contrato
    $scope.$watch('ordemFornecimento.contrato', function() {
        if ($scope.ordemFornecimento.contrato && ($scope.ordemFornecimento.contrato.id)) {
        	ContratoRepository.getGestor($scope.ordemFornecimento.contrato.id).then(function(result) {
        		$scope.contratoGestor = result;
        	});
        } else {
        	$scope.contratoGestor = null;
        }
    });

    //Watch para atualizar valores do item
    $scope.$watch('item.itemBemServico', function() {
    	if ($scope.item && $scope.item.itemBemServico && $scope.item.itemBemServico.originalElement) {
    		if ($scope.item.itemBemServico) {

    			var qnt = 0;
    			if ($scope.ordemFornecimento.itensBensServicos) {
    				for (var i = 0; i < $scope.ordemFornecimento.itensBensServicos.length; i++) {
            			var item = $scope.ordemFornecimento.itensBensServicos[i];
            			if (item.itemBemServico.id == $scope.item.itemBemServico.id) {
            				qnt = parseFloat(qnt) + parseFloat(item.quantidade);
            			}
            		}
    			}

            	$scope.item.quantidade = parseFloat($scope.item.itemBemServico.quantidade) - parseFloat(qnt);
            	$scope.item.valorUnitarioItem = $scope.item.itemBemServico.valorUnitarioItem;
            	$scope.item.valorTotalItem = $scope.item.quantidade * $scope.item.valorUnitarioItem;
            } else {
            	$scope.item.quantidade = null;
            	$scope.item.valorUnitarioItem = 0;
            	$scope.item.valorTotalItem = 0;
            }
    	}
    });

    //Watch para atualizar o valor total do item
    $scope.$watch('item.quantidade', function() {
    	if ($scope.item) {
    		if ($scope.item.quantidade > 0) {
            	$scope.item.valorTotalItem = $scope.item.quantidade * $scope.item.valorUnitarioItem;
            } else {
            	$scope.item.valorTotalItem = 0;
            }
    	}
    });

    //Watch para atualizar o valor total do item
    $scope.$watch('ordemFornecimento.itensBensServicos', function() {
    	if ($scope.ordemFornecimento.itensBensServicos && $scope.ordemFornecimento.itensBensServicos.length > 0) {
    		var qnt = 0;
    		var total = 0;
    		for (var i = 0; i < $scope.ordemFornecimento.itensBensServicos.length; i++) {
    			var item = $scope.ordemFornecimento.itensBensServicos[i];
    			qnt = parseFloat(item.quantidade) + parseFloat(qnt);
    			total = parseFloat(item.valorTotalItem) + parseFloat(total);
    		}
    		$scope.quantidadeTotal = qnt;
    		$scope.totalGeral = total;
    	} else {
    		$scope.quantidadeTotal = 0;
    		$scope.totalGeral = 0;
    	}
    });

    //INICIO DE METODOS REFERENTES AOS RELATORIOS

    $scope.gerarRelatorioOrdemFornecimento = function() {
        $scope.url = '/cit-contratos-web/rest/ordemFornecimento/gerarOrdemFornecimento?idOrdemFornecimento='+ $scope.ordemFornecimento.id;
        $scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.TITULO_ORDEM_FORNECIMENTO"));
    };

	$scope.checkAll = function(value) {
		angular.forEach($scope.ordemFornecimento.itensBensServicos, function (item, key) {
			item.$checked = value;
			item.$index = key;
        });
		$scope.atualizarListaSizeSelect();
	};

	$scope.atualizarListaSizeSelect = function() {
		if(!$scope.listaSize){
			$scope.listaSize = 0;
		};
		var lista = $scope.getItensChecked();
		$scope.listaSize = lista.length;
	};

}]);