'use strict';

citApp.controller('OrdemFornecimentoListController', ['$scope', 'OrdemFornecimentoRepository', '$translate', 'DominioRepository',
                                                 function OrdemFornecimentoListController($scope, OrdemFornecimentoRepository, $translate, DominioRepository) {

	$scope.resetForm = function(){
		angular.element("#editOrdemFornecimento").scope().resetForm();
	};

	$scope.visualizarOrdemFornecimento = function(idOrdemFornecimento){
		angular.element("#editOrdemFornecimento").scope().visualizarOrdemFornecimento(idOrdemFornecimento);
	};

	$scope.headers = [
	{ title : $translate.instant('LABEL.NUM_ORDEM_FORNECIMENTO'), value : 'numOrdemFornecimento', typeMask: 'MASK_10_CODE_ANO', mask : [new RegExp(/(\d{3})(\d{1,4})$/), '$1/$2'], tamanho : 25 },
	{ title : $translate.instant('LABEL.DATA_EMISSAO'), value : 'dataEmissao', filter : 'dateBR', tamanho : 20 },
	{ title : $translate.instant('LABEL.SITUACAO'), value : 'dominioSituacao.descricao', tamanho : 25 },
	{ title : $translate.instant('LABEL.NUMERO_ANO_CONTRATO'), value : 'contrato.numeroAnoContrato', typeMask: 'MASK_CONTRATO', filter : 'maskContrato', tamanho : 25 }];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
			start : 1,
			dir : 'asc',
			sort : 'id',
			limit : 10,
			fields: ['id', 'numOrdemFornecimento', 'dataEmissao', 'dominioSituacao.descricao', 'contrato.numeroAnoContrato'],
			filters : [ { type : 'numeric-range', field : 'numOrdemFornecimento' } ,
			            { type : 'date-range', field : 'dataEmissao' } ,
	                    { type : 'string', field : 'dominioSituacao.descricao', listaDominio : [] } ,
	                    { type : 'numeric-range', field : 'contrato.numeroAnoContrato' }]
	};

	DominioRepository.findAllDominio('tipoSituacaoOrdemFornecimento').then(function(result) {
		$scope.filterCriteria.filters[2].listaDominio = result;
	});

	// abrir uma OrdemFornecimento para visualizar ou editar dependendo do parametro edit
	$scope.abrirOrdemFornecimento = function(edit){
		// recupera a OrdemFornecimento selecionada
		var ordemFornecimento = $scope.ordemFornecimentoChecked;

		if(!edit && !ordemFornecimento) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && !ordemFornecimento) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}

		angular.element("#editOrdemFornecimento").scope().visualizarEditar(ordemFornecimento.id, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};

	$scope.cadastrarNovaOrdemFornecimento = function(){
		angular.element("#editOrdemFornecimento").scope().resetForm();
	};

	//INICIO DE METODOS REFERENTES AOS RELATORIOS

    $scope.gerarRelatorioOrdemFornecimento = function() {

        if(!$scope.ordemFornecimentoChecked){
        	$scope.showAlert("warning", $translate.instant("MSG.SELECIONE_UM_ITEM_PARA_RELATORIO") + " ", false );
        } else{
        	$scope.url = '/cit-contratos-web/rest/ordemFornecimento/gerarOrdemFornecimento?idOrdemFornecimento='+ $scope.ordemFornecimentoChecked.id;
        	$scope.visualizarRelatorio($scope.url, $translate.instant("LABEL.TITULO_ORDEM_FORNECIMENTO"));
        }

    };

}]);


