'use strict';
citApp.controller('ProgramaController', ['$scope', '$translate', '$timeout', 'ProgramaRepository', 'DominioRepository', 'ProgramaAcaoRepository',
                                             function ProgramaController($scope, $translate, $timeout, ProgramaRepository, DominioRepository, ProgramaAcaoRepository) {

	$scope.programa = {
			acoes : []
	};

	$scope.programaAcao = {};

	//Obter os dominiosSituacao
	DominioRepository.findAllDominio('tipoPrograma').then(function(result) {
		$scope.listaTiposProgramas = result;
	});

	// Limpa formulário para novo cadastro
	$scope.resetForm = function() {
		$scope.limparPrograma();
		$scope.edit = true;
		$scope.pgEdit = true;
		$scope.atualizarListaSizeSelect();
		$timeout(function(){
			limparProgramaForm();
		});
	};

	function limparProgramaForm() {
		$scope.programaForm.$submitted = false;
		$scope.programaForm.$setPristine();
	}

	// Atualiza pagina de pesquisa
	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#searchPrograma').scope().fetchResult();
	};

    $scope.openModalProgramaAcao = function() {

        $scope.editProgramaAcao = true;
		$scope.showButtonSalvar = false;

        $scope.programaAcao = {};

        $timeout(function() {
        	$scope.$openModal('modal-programa-acao.html', 'lg');
        });
    };

	$scope.addAcao = function(formDialogProgramaAcao, continuar) {
		formDialogProgramaAcao.$submitted = true;

		var acao = $scope.programaAcao;

		if (formDialogProgramaAcao.$valid) {
			if (acao.$index != undefined && acao.$index != null) {
				$scope.programa.acoes[acao.$index] = acao;
			} else {
				$scope.programa.acoes.push(acao);
			}

			$scope.limparProgramaAcao();
			
			if (continuar) {
				formDialogProgramaAcao.$submitted = false;
				formDialogProgramaAcao.$setPristine();
			} else {
				$scope.$modalInstance.dismiss('cancel');
			}
			$scope.atualizarListaSizeSelect();
		} else {
			//Mensagem de erro de campos obrigatorios não preenchidos
			$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
		}
	};

	// MODAL QUE CONFIRMA REMOÇÃO DO PROGRAMA
	$scope.remove = function(programa){
		$scope.$openModalConfirm({
			message: $translate.instant('CONTRATOS.MSG.CONFIRMA_EXCLUSAO_PROGRAMA'),
			callback: function () {
				ProgramaRepository.remove(programa).then(function() {
					$scope.limparPrograma();

					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.showAlert("success", $translate.instant('CONTRATOS.MSG.SUCESSO_EXCLUSAO_PROGRAMA'));
					angular.element('#searchPrograma').scope().fetchResult();

					$scope.resetForm();
				});
			}
		});
	};

	// SALVA O PROGRAMA
	$scope.saveOrUpdate = function(){
		$scope.programaForm.$submitted = true;

		try{
			//verifica se o formulario está valido para salvar
			if($scope.programaForm.$valid){

				$scope.setLoadingSalva(true);

				ProgramaRepository.save($scope.programa).then(function(result) {
					$scope.programa = result.originalElement;
					$scope.showAlert("success", $translate.instant('MSG.SUCESSO_SALVAR_PROGRAMA'));
					$scope.programaForm.$submitted = false;

					$scope.$showPageSearchWorkspace($scope.workspace);
					$scope.atualizaPaginaPesquisa();
				});
				$scope.setLoading(false);
			}else{
				//Mensagem de erro de campos obrigatorios não preenchidos
				$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
			}
		}catch(exception){
			// se der algum erro no save seta o loading false
			$scope.setLoadingSalva(false);
		}
	};

	// Limpa a acao preenchida
	$scope.limparProgramaAcao = function(){
		$scope.programaAcao = {};
	};

	// Limpa o formulario preenchido
	$scope.limparPrograma = function(){
		$scope.programa = {
				acoes : []
		};
		$scope.limparProgramaAcao();
	};

	// Seleciona todos as acoes na lista
	$scope.checkAll = function(value) {
		angular.forEach($scope.programa.acoes, function (item, key) {
			item.$selected = value;
			item.$index = key;
        });
		$scope.atualizarListaSizeSelect();
	};

	// Abre modal perguntando se deseja excluir e exclui os elementos selecionados na lista de acoes caso confirmado
	$scope.excluirAcoesSelecionados = function(){
		var lista = recuperarAcoesSelecionadas();
		if(lista.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {
					angular.forEach(lista, function (itemRem, key) {
						angular.forEach($scope.programa.acoes, function (item, idx) {
							if ($scope.programa.acoes.length === 1 && item.id) {
								$scope.showAlert("warning", $translate.instant('CONTRATOS.VALIDACAO.ACOES_VAZIAS'));
								return;
							}

							if (item.$selected) {
								if (item.id) {
									ProgramaAcaoRepository.remove(item).then(function(result) {
										$scope.programa.acoes.splice(idx, 1);
										$scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
									});
								} else {
									$scope.programa.acoes.splice(idx, 1);
									$scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
								}
							}
						});
					});
					$scope.atualizarListaSizeSelect();
					$scope.$modalConfirmInstance.dismiss('cancel');
				}
			});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	$scope.atualizarListaSizeSelect = function() {
		if(!$scope.listaSize){
			$scope.listaSize = 0;
		};
		var lista = recuperarAcoesSelecionadas();
		$scope.listaSize = lista.length;
	};

	$scope.editAcao = function(edit) {

		$scope.editProgramaAcao = edit;
		$scope.showButtonSalvar = true;

		var lista = recuperarAcoesSelecionadas();
		if (lista.length === 0) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}
		if (lista.length > 1) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_APENAS_UM_ITEM_PARA_EDICAO'));
			return;
		}

		$scope.programaAcao = clone(lista[0]);

		$timeout(function() {
			$scope.$openModal('modal-programa-acao.html', 'lg');
		});

	};

	//Recupera todas as acoes selecionadas na lista
	var recuperarAcoesSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.programa.acoes, function (item) {
			if(item.$selected){
				lista.push(item);
			}
        });
		return lista;
	};

	// Consulta entidade e mostra no formulario
	$scope.getPrograma = function(programa, edit) {
		$scope.setLoadingGet(true);

		ProgramaRepository.get(programa.id).then(function(result) {
			$scope.programa = result.originalElement;

			if($scope.programa.dataBloqueio === null || $scope.programa.dataBloqueio === undefined) {
				$scope.edit = edit;
			} else {
				$scope.edit = false;
			}

			$scope.atualizarListaSizeSelect();
			$scope.pgEdit = edit;
			$scope.setLoading(false);
		});
	};
}]);


