'use strict';
citApp.controller('ProgramaListController', ['$scope', '$translate', '$timeout', 'ProgramaRepository', 'DominioRepository',  
          function ProgramaListController($scope, $translate, $timeout, ProgramaRepository, DominioRepository) {

    $scope.resetForm = function() {
        angular.element("#editPrograma").scope().resetForm();
    };

	$scope.headers = [
	{
		value : 'codigo',
		title : $translate.instant('CONTRATOS.LABEL.CODIGO_PROGRAMA'),
		tamanho : 10
	},{
		value : 'nome',
		title : $translate.instant('CONTRATOS.LABEL.NOME_PROGRAMA'),
		tamanho : 60,
		align : 'text-justify'
	},{
		value : 'dominioTipoPrograma.descricao',
		title : $translate.instant('CONTRATOS.LABEL.TIPO_PROGRAMA'),
		tamanho : 15
	},{
		value : 'anoExercicio',
		title : $translate.instant('CONTRATOS.LABEL.ANO_EXERCICIO'),
		//filter : 'dateAno',
		tamanho : 10
	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
			start : 1,
			dir : 'asc',
			sort : 'id',
			limit : 10,
			fields: ['id', 'codigo', 'nome', 'dominioTipoPrograma.descricao', 'anoExercicio'],
			filters : [{type: 'numeric-range', field: 'codigo'},
			           {type: 'string', field: 'nome'},
			           {type: 'string', field: 'dominioTipoPrograma.descricao', listaDominio : []},
			           {type: 'numeric-range', field: 'anoExercicio'}]
	};

	DominioRepository.findAllDominio('tipoPrograma').then(function(result) {
		$scope.filterCriteria.filters[2].listaDominio = result;
	});

	// abrir um programa para visualizar ou editar dependendo do parametro edit
	$scope.abrirPrograma = function(edit){
		var programa = $scope.programaChecked;

		if(!edit && !programa) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && !programa) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}

		angular.element('#editPrograma').scope().getPrograma(programa, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};

	$scope.remover = function() {
		var programa = $scope.programaChecked;

		if(programa) {

			$scope.$openModalConfirm({
				message: $translate.instant("CONTRATOS.MSG.CONFIRMA_EXCLUSAO_PROGRAMA"),
				callback: function () {
					$scope.setLoadingRemove(true);

					ProgramaRepository.remove(programa).then(function() {
						$scope.showAlert('success', $translate.instant('CONTRATOS.MSG.SUCESSO_EXCLUSAO_PROGRAMA'), " ", false);
						$scope.fetchResult();
					});

					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.setLoading(false);
				}
			});

		} else {

			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
		}
	};

}]);