'use strict';

citApp.controller('QuestionarioController', ['$scope', 'QuestionarioRepository', 'QuestionarioPerguntaRepository', 'PossivelRespostaQuestionarioPerguntaRepository', 'DominioRepository', '$translate',
                                             '$timeout',
                                             function QuestionarioController($scope, QuestionarioRepository, QuestionarioPerguntaRepository, PossivelRespostaQuestionarioPerguntaRepository, DominioRepository,
                                            		 $translate, $timeout) {
	$scope.questionario = {
			dominioTipoQuestionario : {codigo : 1},
			questoes : []
	};

	$scope.questionarioPergunta = {
			respostasEscolha : []
	};

	$scope.possivelResposta = {};

	$scope.isBloquear = false;

	$scope.isDesbloquear = false;

	DominioRepository.findAllDominio('tipoRespostaQuestao').then(function(result) {
		$scope.dominiosTipoResposta = result;
	});

	DominioRepository.findAllDominio('tipoQuestionario').then(function(result) {
        $scope.dominiosTipoQuestionario = result;
    });

	// Limpa formulário para novo cadastro
	$scope.resetForm = function() {
		$scope.limparQuestionario();
		$scope.edit = true;
		$scope.editTipoResposta = true;
		$scope.pgEdit = true;
		$scope.isBloquear = false;
		$scope.isDesbloquear = $scope.isBloquear;
		$scope.atualizarListaSizeSelect();
		$timeout(function(){
			$scope.questionarioForm.$submitted = false;
			$scope.questionarioForm.$setPristine();
		});
	};

	// Atualiza pagina de pesquisa
	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#searchQuestionario').scope().fetchResult();
	};

	$scope.addPossivelResposta = function() {
		var resposta = $scope.possivelResposta.resposta;
		if ($scope.questionarioPergunta.respostasEscolha.length > 10) {
			$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.MAXIMO_POSSIVEL_RESPOSTA'));
			return;
		}
		if (!resposta) {
			$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.SEM_POSSIVEL_RESPOSTA'));
		} else {
			$scope.questionarioPergunta.respostasEscolha.push($scope.possivelResposta);
			$scope.possivelResposta = {};
		}
	};

	$scope.addQuestao = function() {
		var questao = angular.copy($scope.questionarioPergunta);
		if (!questao.questao || !questao.dominioTipoResposta) {
			$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.QUESTAO_NAO_PREENCHIDA'));
		} else {
			if ((questao.dominioTipoResposta.codigo == 2 || questao.dominioTipoResposta.codigo == 4) && questao.respostasEscolha.length < 2) {
				//Se o tipo de resposta for escolha e tiver menos que duas possiveis adicionadas
				$scope.showAlert('warning', $translate.instant('CONTRATOS.VALIDACAO.MINIMO_POSSIVEIS_RESPOSTAS'));
				return;
			}
			if ((questao.dominioTipoResposta.codigo != 2 && questao.dominioTipoResposta.codigo != 4) && questao.respostasEscolha.length > 0) {
				//Se o tipo de resposta não for escolha e tiver respostas de escolha eu deleto as mesmas
				$timeout(function(){
					angular.forEach(questao.respostasEscolha, function (item, idx) {
						if (item.id) {
							PossivelRespostaQuestionarioPerguntaRepository.remove(item);
						}
			        });
					questao.respostasEscolha = [];
				});
			}
			if (questao.$index != undefined && questao.$index != null) {
				$scope.questionario.questoes[questao.$index] = questao;
			} else {
				$scope.questionario.questoes.push(questao);
			}
			$scope.limparPergunta();
			$scope.atualizarListaSizeSelect();
		}
	};

	$scope.mudaTipoResposta = function() {
		$timeout(function(){
			angular.forEach($scope.dominiosTipoResposta, function (item) {
				if (item.codigo == $scope.questionarioPergunta.dominioTipoResposta.codigo) {
					$scope.questionarioPergunta.dominioTipoResposta = item.originalElement;
				}
	        });
		});
	};

	// Bloqueia o questionario
	$scope.confirmarBloqueioQuestionario = function(){

		if(!$scope.questionario.dataBloqueio){

			$scope.showAlert('error', $translate.instant('MSG.MG011'));
		}else{

			try{

				$scope.setLoadingSalva(true);

				QuestionarioRepository.save($scope.questionario).then(function(result){
					$scope.setLoadingSalva(false);

					$scope.questionario = result.originalElement;

					$scope.isBloquear = false;
					$scope.isDesbloquear = !$scope.isBloquear;
					$scope.edit = false;
					$scope.showAlert('success', $translate.instant('MSG.SUCESSO_BLOQUEIO_QUESTIONARIO'));
				});
			}catch(exception){
				$scope.setLoadingSalva(false);
			}
		}
	};

	// Desbloqueia o questionario
	$scope.desbloquear = function() {
		$scope.setLoadingSalva(true);

		$scope.questionario.dataBloqueio =  null;

		QuestionarioRepository.save($scope.questionario).then(function(result){
			$scope.setLoadingSalva(false);

			$scope.questionario = result.originalElement;

			$scope.edit = true;
			$scope.isBloquear = true;
			$scope.isDesbloquear = !$scope.isBloquear;
			$scope.showAlert("success", $translate.instant('MSG.SUCESSO_DESBLOQUEIO_QUESTIONARIO'));
		});
	};

	// MODAL QUE CONFIRMA REMOVER DO Questionário
	$scope.remove = function(questionario){
		$scope.$openModalConfirm({
			message: $translate.instant('MSG.CONFIRMA_EXCLUIR_QUESTIONARIO'),
			callback: function () {
				QuestionarioRepository.remove(questionario).then(function() {
					$scope.limparQuestionario();

					$scope.$modalConfirmInstance.dismiss('cancel');
					$scope.showAlert("success", $translate.instant('MSG.SUCESSO_EXCLUSAO_QUESTIONARIO'));
					angular.element('#searchQuestionario').scope().fetchResult();

					$scope.resetForm();
				});
			}
		});
	};

	// SALVA O QUESTIONARIO
	$scope.saveOrUpdate = function(){
		$scope.questionarioForm.$submitted = true;

		try{
			//verifica se o formulario está valido para salvar
			if($scope.questionarioForm.$valid){

				$scope.setLoadingSalva(true);

				QuestionarioRepository.save($scope.questionario).then(function(result) {
					$scope.questionario = result.originalElement;
					$scope.isBloquear = !$scope.questionario.dataBloqueio;
					$scope.isDesbloquear = !$scope.isBloquear;
					$scope.showAlert("success", $translate.instant('MSG.SUCESSO_SALVAR_QUESTIONARIO'));
					$scope.questionarioForm.$submitted = false;
					
					$scope.$showPageSearchWorkspace($scope.workspace);
					$scope.atualizaPaginaPesquisa();
				});
				$scope.setLoading(false);
			}else{
				//Mensagem de erro de campos obrigatorios não preenchidos
				$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
			}
		}catch(exception){
			// se der algum erro no save seta o loading false
			$scope.setLoadingSalva(false);
		}
	};

	// Limpa a pergunta preenchida
	$scope.limparPergunta = function(){
		if ($scope.editTipoResposta) {
			$scope.questionarioPergunta = {
					respostasEscolha : []
			};
		} else {
			delete $scope.questionarioPergunta.questao;
		}
	};

	// Limpa o formulario preenchido
	$scope.limparQuestionario = function(){
		$scope.questionario = {
				dominioTipoQuestionario : {codigo : 1},
				questoes : []
		};
		$scope.limparPergunta();
	};

    // SE ESTIVER EDITANDO CARREGA A Regra Evento
	if ($scope.$parent != undefined && (!$scope.$parent.newObejct || $scope.$parent.idObejct != 0)) {
		$scope.setLoadingGet(true);

			QuestionarioRepository.get($scope.$parent.idObejct).then(function(result) {
				$scope.questionario = result.originalElement;
				$scope.atualizarListaSizeSelect();
				$scope.setLoading(false);
			});
	}

	// Seleciona todos as questoes na lista
	$scope.checkAll = function(value) {
		angular.forEach($scope.questionario.questoes, function (item, key) {
			item.$selected = value;
			item.$index = key;
        });
		$scope.atualizarListaSizeSelect();
	};

	// Seleciona todos as respostas na lista
	$scope.checkAllRespostas = function(value) {
		angular.forEach($scope.questionarioPergunta.respostasEscolha, function (item, key) {
			item.$selected = value;
			item.$index = key;
        });
	};

	// Abre modal perguntando se deseja excluir e exclui os elementos selecionados na lista de questões caso confirmado
	$scope.prepararExcluirQuestoesSelecionados = function(){
		var lista = recuperarQuestoesSelecionadas();
		if(lista.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {
					angular.forEach(lista, function (itemRem, key) {
						angular.forEach($scope.questionario.questoes, function (item, idx) {
							if (item.$index == itemRem.$index) {
								if (item.id) {
									QuestionarioPerguntaRepository.remove(item);
								}
								$scope.questionario.questoes.splice(idx, 1);
							}
				        });
			        });
					$scope.atualizarListaSizeSelect();
					$scope.$modalConfirmInstance.dismiss('cancel');
	            }
				});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	$scope.prepararExcluirRespostasSelecionadas = function() {
		var lista = recuperarRespostasSelecionadas();
		if(lista.length > 0){
			$scope.$openModalConfirm({
				message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
				callback: function () {
					angular.forEach(lista, function (itemRem, key) {
						angular.forEach($scope.questionarioPergunta.respostasEscolha, function (item, idx) {
							if (item.$index == itemRem.$index) {
								if (item.id) {
									PossivelRespostaQuestionarioPerguntaRepository.remove(item);
								}
								$scope.questionarioPergunta.respostasEscolha.splice(idx, 1);
							}
				        });
			        });
					$scope.$modalConfirmInstance.dismiss('cancel');
	            }
				});
		} else {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}
	};

	$scope.editQuestao = function() {
		var lista = recuperarQuestoesSelecionadas();
		if (lista.length === 0) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}
		if (lista.length > 1) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_APENAS_UM_ITEM_PARA_EDICAO'));
			return;
		}
		$scope.questionarioPergunta = lista[0];
	};

	//Recupera todas as questões selecionadas na lista
	var recuperarQuestoesSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.questionario.questoes, function (item) {
			if(item.$selected){
				lista.push(item);
			}
        });
		return lista;
	};

	//Recupera todas as possiveis respostas selecionadas na lista
	var recuperarRespostasSelecionadas = function(){
		var lista = [];
		angular.forEach($scope.questionarioPergunta.respostasEscolha, function (item) {
			if(item.$selected){
				lista.push(item);
			}
        });
		return lista;
	};

	// Consulta entidade e mostra no formulario
	$scope.getQuestionario = function(questionario, edit){
		$scope.setLoadingGet(true);

		QuestionarioRepository.get(questionario.id).then(function(result) {
			$scope.questionario = result.originalElement;

			if($scope.questionario.dataBloqueio === null || $scope.questionario.dataBloqueio === undefined){
				$scope.isBloquear = true;
				$scope.isDesbloquear = !$scope.isBloquear;
				$scope.edit = edit;
				$scope.editTipoResposta = edit;
			}else{
				$scope.isBloquear = false;
				$scope.isDesbloquear = !$scope.isBloquear;
				$scope.edit = false;
				$scope.editTipoResposta = false;
			}
			$scope.atualizarListaSizeSelect();
			$scope.pgEdit = edit;
			$scope.setLoading(false);
		});
	};

    //Watch para calcular valor unitario
    $scope.$watch('questionario.dominioTipoQuestionario', function() {
		$timeout(function() {
            if ($scope.questionario.dominioTipoQuestionario && ($scope.questionario.dominioTipoQuestionario.codigo == 2)) {
                QuestionarioRepository.findAllByDominioTipoQuestionario({codigo : 2}).then(function(result) {
                	if ($scope.questionario.id != result[0].originalElement.id) {
                		$scope.questionario = result[0].originalElement;
                		$scope.showAlert('warning', $translate.instant('MSG.CADASTRO_CHECKLIST_ANALISE_SOLUCAO'));
                	}
                });

            	$scope.questionarioPergunta.dominioTipoResposta = {codigo : 4};
            	$scope.editTipoResposta = false;
            	$scope.questionarioPergunta.respostasEscolha = [{resposta : $translate.instant('LABEL.SIM')},
            	                                                {resposta : $translate.instant('LABEL.NAO')},
            	                                                {resposta : $translate.instant('LABEL.NAO_APLICA')}];
            } else {
            	if (!$scope.questionario.id) {
	            	$scope.questionario = {
	            			dominioTipoQuestionario : $scope.questionario.dominioTipoQuestionario,
	            			questoes : []
	            	};
            	}
            	$scope.questionarioPergunta = {
            			respostasEscolha : []
            	};
            	$scope.editTipoResposta = true;
            }
        });
    });

	$scope.atualizarListaSizeSelect = function() {
		if(!$scope.listaSize){
			$scope.listaSize = 0;
		};
		var lista = recuperarQuestoesSelecionadas();
		$scope.listaSize = lista.length;
	};

}]);


