'use strict';
citApp.controller('RegistroOcorrenciaController', ['$scope', '$translate', '$timeout', '$filter',
                                                   'RegistroOcorrenciaRepository', 'DominioRepository', 'ContratoRepository', 'OrdemFornecimentoRepository', 'FileUploader',
                                                   function RegistroOcorrenciaController($scope, $translate, $timeout, $filter,
                                                		   RegistroOcorrenciaRepository, DominioRepository, ContratoRepository, OrdemFornecimentoRepository, FileUploader) {

	$scope.resetForm = function () {

		$scope.registroOcorrencia = {
				contrato : null,
				cnpjFornecedor : "",
				ordemFornecimento : {},
				dataEmissaoOcorrencia : $filter('date')(new Date(), "dd/MM/yyyy"),
				emitirGlosa : false,
				valorGlosa : "",
				justificativaGlosa : "",
				tituloOcorrencia : "",
				descricaoOcorrencia : "",
				prazoCumprimento : "",
				documentos : [],
				observacoes : []
		};

		$scope.edit = true;
		$scope.pgEdit = true;

		$timeout(function (){
			if($scope.registroOcorrenciaForm && $scope.registroOcorrenciaForm.$submitted){
				$scope.registroOcorrenciaForm.$submitted = false;
				$scope.registroOcorrenciaForm.$setPristine();
			}
		});
	};

	$scope.resetForm();

	$scope.uploader = new FileUploader({
        autoUpload: false,
        queueLimit: 99,
        url: '/cit-tabelas-corp-web/rest/anexo/uploadAnexo?idDocumento='
    });

	$scope.editar = function () {
		$scope.edit = true;
		$scope.pgEdit = true;
	};

	$scope.$watch('registroOcorrencia.emitirGlosa', function() {
		if($scope.registroOcorrencia.id == "" || !$scope.registroOcorrencia.id){
			$scope.registroOcorrencia.valorGlosa = "";
			$scope.registroOcorrencia.justificativaGlosa = "";
		} else {
			if(!$scope.registroOcorrencia.emitirGlosa) {
				$scope.registroOcorrencia.valorGlosa = "";
				$scope.registroOcorrencia.justificativaGlosa = "";
			}
			if ($scope.registroOcorrencia.contrato.fornecedor.pessoa.pessoaJuridica.cnpj && $scope.registroOcorrencia.contrato.fornecedor.pessoa.pessoaJuridica.cnpj !== ""){
				$scope.setCNPJFornecedorContrato($scope.registroOcorrencia.contrato.fornecedor.pessoa.pessoaJuridica.cnpj);
			}
		}
	});

	// Atualiza pagina de pesquisa
	$scope.atualizaPaginaPesquisa = function () {
		$scope.resetForm();
		$scope.$showPageSearchWorkspace($scope.workspace);
		angular.element('#searchRegistroOcorrencia').scope().fetchResult();
	};

	$scope.saveOrUpdate = function (){

		$scope.registroOcorrenciaForm.$submitted = true;

		if($scope.registroOcorrenciaForm.$invalid){
			$scope.showAlert('error', $translate.instant('MSG.MN001'));
		} else if ($scope.isInformacoesValidas()) {

			var registroOcorrenciaTemp = {
					documentos: []
			};

			// preencher lista de documentos temporarios para verificacao dos anexos. verificar qual anexo e de qual documento pelo numero do documento
			angular.forEach($scope.registroOcorrencia.documentos, function(documento, key) {
				if (documento.uploadsDocumento) {
					documento.uuid = getUuid();
					registroOcorrenciaTemp.documentos.push({
						file: documento.uploadsDocumento[0],
						uuid: clone(documento.uuid)
					});
					// deleta esse atributo por causa de um erro de ciclo no json
					delete documento.uploadsDocumento;
				}
			});

			$scope.setLoadingSalva(true);

			RegistroOcorrenciaRepository.save($scope.registroOcorrencia).then(function(result) {

				$scope.registroOcorrencia = result.originalElement;
				$scope.setLoading(false);
				$scope.showAlert("success", $translate.instant('MSG.SUCESSO_SALVAR_REGISTRO_OCORRENCIA'));
				// depois de salvar a ocorrencia ja tendo uma referencia do id da mesma, esse metodo ira salvar os documentos se houver
				salvarListaDocumentos(registroOcorrenciaTemp);
				$scope.atualizaPaginaPesquisa();
			});
		}
	};

	// metodo utilizado para percorrer a lista de documentos persistidos e verificar com a lista de documentos temporarios para verificacao de qual anexo pertence a qual documento
    var salvarListaDocumentos = function(registroOcorrenciaTemp) {
    	$scope.uploader.queue = [];
    	
        angular.forEach($scope.registroOcorrencia.documentos, function(documentoSalvo, key) {
            // verifica se a entidade documento foi salva
            if (documentoSalvo !== undefined && documentoSalvo.id !== undefined) {
                // percorre todos os documentos temporarios para verificacao do numero do documento
                angular.forEach(registroOcorrenciaTemp.documentos, function(documentoTransient, key) {
                    // verifica se o numero do documento salvo e igual ao numero do documento transient que o anexo ainda esta vinculado
                    if (documentoTransient.file && documentoSalvo.uuid === documentoTransient.uuid) {
                        //salva o anexo ao documento que tiver o mesmo numero do documento transient
                    	
                    	documentoTransient.file.url = documentoTransient.file.url + documentoSalvo.id;
                    	$scope.uploader.queue.push(documentoTransient.file);
                    }
                });
            }
        });
        
        if($scope.uploader.queue.length > 0){
        	$scope.uploader.uploadAll();
        }
        delete $scope.registroOcorrenciaTemp;
    };

	$scope.isInformacoesValidas = function () {

		var formValido = true;

		if(!$scope.registroOcorrencia.id && $scope.registroOcorrencia.contrato.dominioSituacaoAtual.nome !== "VIGENTE"){
			$scope.showAlert('error', $translate.instant('MSG.OCORRENCIA_CONTRATO_FORA_DA_VIGENCIA'));
			formValido = false;
		}

		if(!isDataDentroPeriodo($scope.registroOcorrencia.contrato.dataVigenciaInicial, $scope.registroOcorrencia.contrato.dataUltimaVigenciaFinal, $scope.registroOcorrencia.dataEmissaoOcorrencia)) {
			$scope.showAlert('error', $translate.instant('MSG.DATA_OCORRENCIA_INVALIDA'));
			formValido = false;
		}

		//A data para cumprimento das exigencias deve ser maior que a data de registro da ocorrencia e menor que a data de vigencia do contrato
		if($scope.registroOcorrencia.prazoCumprimento !== null && $scope.registroOcorrencia.prazoCumprimento !== undefined && $scope.registroOcorrencia.prazoCumprimento !== ""
			&& !isDataDentroPeriodo($scope.registroOcorrencia.dataEmissaoOcorrencia, $scope.registroOcorrencia.contrato.dataUltimaVigenciaFinal, $scope.registroOcorrencia.prazoCumprimento)) {
			$scope.showAlert('error', $translate.instant('MSG.DATA_PRAZO_CUMPRIMENTO_INVALIDA'));
			formValido = false;
		}

		if($scope.registroOcorrencia.emitirGlosa && ($scope.registroOcorrencia.valorGlosa > $scope.registroOcorrencia.contrato.valorTotalContrato || $scope.registroOcorrencia.valorGlosa === 0.00)){
			$scope.showAlert('error', $translate.instant('MSG.VALOR_GLOSA_INVALIDO'));
			formValido = false;
		}

		return formValido;
	};

	// Consulta entidade e mostra no formulario
	$scope.getRegistroOcorrencia = function (idRegistroOcorrencia, edit){
		$scope.resetForm();
		$scope.setLoadingGet(true);

		RegistroOcorrenciaRepository.get(idRegistroOcorrencia).then(function(result){
			$scope.registroOcorrencia = result.originalElement;
			$scope.edit = edit;
			if($scope.registroOcorrencia.valorGlosa || $scope.registroOcorrencia.justificativaGlosa) {
				$scope.registroOcorrencia.emitirGlosa = true;
			} else {
				$scope.registroOcorrencia.emitirGlosa = false;
			}
			$scope.setLoading(false);
		});
	};

	$scope.findAutoCompleteContrato = function (numeroAnoContrato) {

		return ContratoRepository.findContrato(numeroAnoContrato, $scope.usuarioLogado.organizacao.id).then(function(result) {
			return result;
		});
	};

	$scope.findOrdemFornecimentoServicoPorContrato = function (idContrato) {

		return OrdemFornecimentoRepository.findOrdemFornecimentoPorContrato(idContrato).then(function(result){

			$scope.listaNumeroOrdemFornecimento = [];
			angular.forEach(result, function(ordemFornecimento){
				$scope.listaNumeroOrdemFornecimento.push(ordemFornecimento.originalElement);
			});
		});
	};

	$scope.setCNPJFornecedorContrato = function(cnpj){
		$scope.registroOcorrencia.cnpjFornecedor = cnpj;
	};

	$scope.findGlosasAnterioresPorContrato = function(){
		//select numeroAnoSequencial, dataEmissaoOcorrencia, autor, Justificativa, valorGlosa from registroOcorrencia inner join contrato on registroOcorrencia.contrato_id = contrato.id
		//where registroOcorrencia.contrato_id = :p and valorGlosa is not null;
	};
}]);
