'use strict';

citApp.controller('RegistroOcorrenciaListController', ['$scope',
                                                       'RegistroOcorrenciaRepository',
                                                       'DominioRepository',
                                                       'ContratoRepository',
                                                       '$translate',
                                                       '$timeout',
                                                       function RegistroOcorrenciaListController($scope,
                                                                                                 RegistroOcorrenciaRepository,
                                                                                                 DominioRepository,
                                                                                                 ContratoRepository,
                                                                                                 $translate,
                                                                                                 $timeout) {
			$scope.$showAdvancedFilters = false;

			$scope.totalPages = 10;
			$scope.totalItens = 0;
			$scope.limit = 10;

			$scope.resetForm = function() {
				angular.element("#editRegistroOcorrencia").scope().resetForm();
			};

			$scope.headers = [{
				value : 'dataEmissaoOcorrencia',
				title : $translate.instant('LABEL.DATA_EMISSAO_OCORRENCIA'),
				filter : 'dateBR',
				tamanho : 10
			}, {
				value : 'registroSequencial.sequencialAno',
				title : $translate.instant('LABEL.NUMERO_OCORRENCIA'),
				tamanho : 10,
				typeMask: 'MASK_10_CODE_ANO',
				mask : [new RegExp(/(\d{1})(\d{1,4})$/), '$1/$2']
			}, {
				value : 'tituloOcorrencia',
				title : $translate.instant('LABEL.TITULO_OCORRENCIA'),
				tamanho : 60,
				align : 'text-justify'
			}, {
				value : 'contrato.numeroAnoContrato',
				title : $translate.instant('LABEL.NUMERO_ANO_CONTRATO'),
				tamanho : 15,
				typeMask: 'MASK_CONTRATO',
				mask : [new RegExp(/(\d{3})(\d{1,4})$/), '$1/$2']
			}
			];

			// default criteria that will be sent to the server
			$scope.filterCriteria = {
				start : 1,
				dir : 'asc',
				sort : 'id',
				limit : 10,
				fields : ['id', 'dataEmissaoOcorrencia', 'registroSequencial.sequencialAno', 'tituloOcorrencia', 'contrato.numeroAnoContrato'],
				filters : [{
					type : 'date-range',
					field : 'dataEmissaoOcorrencia'
				}, {
					type : 'string',
					field : 'registroSequencial.sequencialAno'
				}, {
					type : 'string',
					field : 'tituloOcorrencia'
				}, {
					type : 'string',
					field : 'contrato.numeroAnoContrato'
				}]
			};

			// Will be called when filtering the grid, will reset the page number to one
			$scope.filterResult = function(generic) {
				if (generic == undefined) {
					$scope.filterCriteria.keywordValue = null;
				}
				$scope.filterCriteria.start = 1;
				$scope.fetchResult().then(function() {
					// The request fires correctly but sometimes the ui doesn't update, that's a fix
					$scope.filterCriteria.start = 1;
				});
			};

			// Recupera lista de registros de ocorrenciass e seta resultado na grid
			$scope.fetchResult = function(page) {
				$scope.setLoadingPesquisa(true);

				return RegistroOcorrenciaRepository.getListPage($scope.filterCriteria).then(function(result) {
					$scope.registrosOcorrencias = result.originalElement.objects;
					$scope.totalPages = result.originalElement.totalPages;
					$scope.totalItens = result.originalElement.totalItens;

					$scope.setLoading(false);
				}, function() {
					$scope.registrosOcorrencias = [];
					$scope.totalPages = 0;
					$scope.totalItens = 0;
				});
			};

			// Busca lista de registros de ocorrencia
			$scope.fetchResult();

			$scope.abrirRegistroOcorrencia = function(edit) {
				$scope.setLoadingGet(true);

				var registroOcorrencia = $scope.registroOcorrenciaChecked;

				if (edit && !registroOcorrencia) {
					$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
					$scope.setLoadingGet(false);
					return;
				} else if (!edit && !registroOcorrencia) {
					$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
					$scope.setLoadingGet(false);
					return;
				}
				angular.element('#editRegistroOcorrencia').scope().getRegistroOcorrencia(registroOcorrencia.id, edit);
				$scope.$showPageEditWorkspace($scope.workspace);
			};

			$scope.gerarRelatorioRegistroOcorrencia = function(edit) {
				var registroOcorrencia = $scope.registroOcorrenciaChecked;
				if (!edit && !registroOcorrencia) {
					$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_GERAR_REGISTRO_OCORRENCIA'));
					$scope.setLoadingGet(false);
					return;
				}
				$scope.url = '/cit-contratos-web/rest/registroOcorrencia/pdfGerarRegistroOcorrencia?registroOcorrenciaId=' + registroOcorrencia.id;
				$scope.visualizarRelatorio($scope.url, $translate.instant('LABEL.DOCUMENTO_REGISTRO_OCORRENCIA'));
			};

		}]);
