'use strict';

citApp.controller('RegraEventoController', ['$scope', 'RegraEventoRepository', 'DominioRepository', '$translate', '$timeout',
                                            function RegraEventoController($scope, RegraEventoRepository, DominioRepository, $translate, $timeout) {
	$scope.regraEvento = {};
	$scope.tiposImplementacao = {};	
	$scope.chavesDominio = [];

	$scope.isBloquear = false;

	$scope.isDesbloquear = false;
	
	// Limpa formulário para novo cadastro
	$scope.resetForm = function() {
		$scope.limparRegraEvento();
		$scope.edit = true;
		$scope.pgEdit = true;
		$scope.isBloquear = false;
		$scope.isDesbloquear = $scope.isBloquear;
		$timeout(function(){
			$scope.regraEventoForm.$submitted = false;
			$scope.regraEventoForm.$setPristine();
		});
	};
	
	// Atualiza pagina de pesquisa
	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#searchRegraEvento').scope().fetchResult();
	};
	
	// Bloqueia a Regra Evento
	$scope.confirmarBloqueioRegraEvento = function(){
		
		if(!$scope.regraEvento.dataBloqueio){
			
			$scope.showAlert('error', $translate.instant('MSG.MG011'));
		}else{
			
			try{
			
				$scope.setLoadingSalva(true);
			
				RegraEventoRepository.save($scope.regraEvento).then(function(result){
					$scope.setLoadingSalva(false);
					
					$scope.regraEvento = result.originalElement;
					
					$scope.isBloquear = false;
					$scope.isDesbloquear = !$scope.isBloquear;
					$scope.edit = false;
					$scope.showAlert('success', $translate.instant('MSG.SUCESSO_BLOQUEIO_REGRA_EVENTO'));
				});
			}catch(exception){
				$scope.setLoadingSalva(false);
			}			
		}
	};

	// Desbloqueia a Regra Evento 
	$scope.desbloquear = function() {
		$scope.setLoadingSalva(true);
		
		$scope.regraEvento.dataBloqueio =  null;
		
		RegraEventoRepository.save($scope.regraEvento).then(function(result){
			$scope.setLoadingSalva(false);
			
			$scope.regraEvento = result.originalElement;
			
			$scope.edit = true;
			$scope.isBloquear = true;
			$scope.isDesbloquear = !$scope.isBloquear;
			$scope.showAlert("success", $translate.instant('MSG.SUCESSO_DESBLOQUEIO_REGRA_EVENTO'));
		});
	};
	
	DominioRepository.findAllDominio('tipoImplementacao').then(function(result) {
		$scope.dominiosTipoImplementacao = result;
	});	
	
	
	DominioRepository.findAllChavesDominio().then(function(result) {
		$scope.chavesDominio = result;
	});
	
	//Metodo responsavel por apresentar o dialog de confirmacao de remocao da regra evento
	$scope.apresentarDialogConfirmacaoRemocaoRegra = function(){
		
		$scope.$openModalConfirm({message: $translate.instant('MSG.CONFIRMA_EXCLUIR_REGRA_EVENTO'), callback: $scope.remove});
	};	
	
	// Método para remover regra evento
	$scope.remove = function(){
		//$scope.$openModalConfirm({ 
			//message: $translate.instant('MSG.CONFIRMA_EXCLUIR_REGRA_EVENTO'), 
			//callback: function () {
				RegraEventoRepository.remove($scope.regraEvento).then(function() {
					$scope.regraEvento = {};
					
				
					$scope.showAlert("success", $translate.instant('MSG.SUCESSO_EXCLUSAO_REGRA_EVENTO'));
					angular.element('#searchRegraEvento').scope().fetchResult();
					
					$scope.resetForm();
				});
				$scope.$modalConfirmInstance.dismiss('cancel');
			//}
	//	});
	};
	
	// SALVA A REGRA EVENTO
	$scope.saveOrUpdate = function(){
		$scope.regraEventoForm.$submitted = true;
		
		try{
			//verifica se o formulario está valido para salvar
			if($scope.regraEventoForm.$valid){
				
				$scope.setLoadingSalva(true);
				
				RegraEventoRepository.save($scope.regraEvento).then(function(result) {
					$scope.regraEvento = result.originalElement;
					$scope.isBloquear = !$scope.regraEvento.dataBloqueio;
					$scope.isDesbloquear = !$scope.isBloquear;
					$scope.showAlert("success", $translate.instant('MSG.SUCESSO_SALVAR_REGRA_EVENTO'));
					$scope.regraEventoForm.$submitted = false;
				});
				$scope.setLoading(false);
			}else{
				//Mensagem de erro de campos obrigatorios não preenchidos
				$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
			}
		}catch(exception){
			// se der algum erro no save seta o loading false
			$scope.setLoadingSalva(false);
		}
	};
	
	// Limpa o formulario preenchido
	$scope.limparRegraEvento = function(){
		$scope.regraEvento = {};
	};

    // SE ESTIVER EDITANDO CARREGA A Regra Evento
	if ($scope.$parent != undefined && (!$scope.$parent.newObejct || $scope.$parent.idObejct != 0)) {
		$scope.setLoadingGet(true);

			RegraEventoRepository.get($scope.$parent.idObejct).then(function(result) {
				$scope.regraEvento = result.originalElement;

				$scope.setLoading(false);
			});
	}
	
	// Consulta entidade e mostra no formulario
	$scope.getRegraEvento = function(regraEvento, edit){
		$scope.setLoadingGet(true);
	
		RegraEventoRepository.get(regraEvento.id).then(function(result) {
			$scope.regraEvento = result.originalElement;
			
			if($scope.regraEvento.dataBloqueio === null || $scope.regraEvento.dataBloqueio === undefined){
				$scope.isBloquear = true;
				$scope.isDesbloquear = !$scope.isBloquear;
				$scope.edit = edit;
			}else{
				$scope.isBloquear = false;
				$scope.isDesbloquear = !$scope.isBloquear;
				$scope.edit = false;
			}
			
			$scope.pgEdit = edit;
			$scope.setLoading(false);
		});
	};
}]);


