'use strict';

citApp.controller('RegraEventoListController', ['$scope', 'RegraEventoRepository', 'DominioRepository', '$translate',
                                                function RegraEventoListController($scope, RegraEventoRepository, DominioRepository, $translate) {
	$scope.$showAdvancedFilters = false;

	// CHECK Regra Evento
	$scope.removeCheckedRegraEvento = function () {
		var regrasEvento = $scope.regrasEvento;

		regrasEvento.forEach(function (regraEvento) {
			regraEvento.$checked = false;
		});
	};
	
	$scope.getCheckedRegrasEvento = function () {
		var regraEventoChecked = null;
		var regrasEvento = $scope.regrasEvento;

		regrasEvento.forEach(function (regraEvento) {
			if(regraEvento.$checked) {
				regraEventoChecked = regraEvento;				
			}
		});

		return regraEventoChecked;
	};
	
	$scope.checkRegraEvento = function (regraEvento) {
		$scope.removeCheckedRegraEvento();

		regraEvento.$checked = true;
	};

	DominioRepository.findAllDominio('tipoImplementacao').then(function(result) {
		$scope.dominiosTipoImplementacao = result;
	});

	$scope.totalPages = 10;
	$scope.totalItens = 0;
	$scope.limit = 10;

	$scope.resetForm = function() {
		angular.element("#editRegraEvento").scope().resetForm();
	};

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
		start : 1,
		dir : 'asc',
		sort : 'id',
		limit : 10,
		fields: ['id', 'nome', 'descricao', 'dominioTipoImplementacao', 'dataBloqueio'],
		filters : [{type: 'string', field: 'nome'},{type: 'string', field: 'descricao'},{type: 'string', field: 'dominioTipoImplementacao.descricao'}]
	};

	// Will be called when filtering the grid, will reset the page number to one
	$scope.filterResult = function(generic) {
        if (generic == undefined) {
            $scope.filterCriteria.keywordValue = null;
        }
		$scope.filterCriteria.start = 1;
		$scope.fetchResult().then(function() {
			// The request fires correctly but sometimes the ui doesn't update,
			// that's a fix
			$scope.filterCriteria.start = 1;
		});
	};

	// call back function that we passed to our custom directive sortBy, will be
	// called when clicking on any field to sort
	$scope.onSort = function(sortedBy, sortDir) {
		$scope.filterCriteria.dir = sortDir;
		$scope.filterCriteria.sort = sortedBy;
		$scope.filterCriteria.start = 1;
		$scope.fetchResult().then(function() {
			// The request fires correctly but sometimes the ui doesn't update,
			// that's a fix
			$scope.filterCriteria.start = 1;
		});
	};

	// RECUPERA LISTA DE Regras Evento E SETA RESULTADO NA GRID
	$scope.fetchResult = function(page) {
		$scope.setLoadingPesquisa(true);

		return RegraEventoRepository.getListPage($scope.filterCriteria).then(function(result) {
			$scope.regrasEvento = result.originalElement.objects;
			$scope.totalPages = result.originalElement.totalPages;
			$scope.totalItens =  result.originalElement.totalItens;

			$scope.setLoading(false);
		}, function() {
			$scope.regrasEvento = [];
			$scope.totalPages = 0;
			$scope.totalItens = 0;
		});
	};

	// MODAL QUE CONFIRMA REMOVER DA Regra Evento
	$scope.remove = function(){
		var regraEvento = $scope.getCheckedRegrasEvento();

		if(regraEvento === null) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));

			return ;
		}
		
		angular.element('#editRegraEvento').scope().regraEvento = regraEvento;

		angular.element('#editRegraEvento').scope().apresentarDialogConfirmacaoRemocaoRegra();
	};

	// BUSCA LISTA DE Regras Evento
	$scope.fetchResult();

	// ABRI CARACTERISTICA SELECIONADA
	$scope.abrirRegraEvento = function(edit){
		var regraEvento = $scope.getCheckedRegrasEvento();

		if(!edit && regraEvento === null) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && regraEvento === null) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));

			return;
		}

		angular.element('#editRegraEvento').scope().getRegraEvento(regraEvento, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};
}]);


