'use strict';

citApp.controller('TermoReferenciaListController', ['$scope', 'TermoReferenciaRepository', 'DominioRepository', '$translate', '$timeout',
                                            function TermoReferenciaListController($scope, TermoReferenciaRepository, DominioRepository, $translate, $timeout) {
	$scope.$showAdvancedFilters = false;

	$scope.totalPages = 10;
	$scope.totalItens = 0;
	$scope.limit = 10;

	$scope.resetForm = function() {
		angular.element("#editTermoReferencia").scope().resetForm();
	};

	$scope.headers = [
	              	{
	              		value : 'codigo',
	              		title : $translate.instant('LABEL.CODIGO'),
	              		tamanho : 15,
	              		typeMask: 'MASK_10_CODE_ANO',
	              		mask : [new RegExp(/(\d{3})(\d{1,4})$/), '$1/$2']
	              	},{
	              		value : 'objetoContratacao',
	              		title : $translate.instant('LABEL.OBJETO_CONTRATACAO'),
	              		tamanho : 80,
	              		align : 'text-justify'
	              	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
		start : 1,
		dir : 'asc',
		sort : 'id',
		limit : 10,
		fields: ['id', 'codigo', 'objetoContratacao', 'dataTermoGerado'],
		filters : [{type: 'string', field: 'codigo'}, {type: 'string', field: 'objetoContratacao'}]
	};

	// Will be called when filtering the grid, will reset the page number to one
	$scope.filterResult = function(generic) {
        if (generic == undefined) {
            $scope.filterCriteria.keywordValue = null;
        }
		$scope.filterCriteria.start = 1;
		$scope.fetchResult().then(function() {
			// The request fires correctly but sometimes the ui doesn't update,
			// that's a fix
			$scope.filterCriteria.start = 1;
		});
	};

	// RECUPERA LISTA DE TERMOS E SETA RESULTADO NA GRID
	$scope.fetchResult = function(page) {
		$scope.setLoadingPesquisa(true);

		return TermoReferenciaRepository.getListPage($scope.filterCriteria).then(function(result) {
			$scope.termosReferencia = result.originalElement.objects;
			$scope.totalPages = result.originalElement.totalPages;
			$scope.totalItens =  result.originalElement.totalItens;

			$scope.setLoading(false);
		}, function() {
			$scope.termosReferencia = [];
			$scope.totalPages = 0;
			$scope.totalItens = 0;
		});
	};

	// BUSCA LISTA DE TERMOS
	$scope.fetchResult();

	$scope.abrirTermo = function(edit){
		$scope.setLoadingGet(true);

		var termo = $scope.termoReferenciaChecked;

		if (edit && !termo){
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			$scope.setLoadingGet(false);
			return;
		} else if (!edit && !termo){
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			$scope.setLoadingGet(false);
			return;
		}
		angular.element('#editTermoReferencia').scope().getTermo(termo.id, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};

	// MODAL QUE CONFIRMA REMOVER Do termo
	$scope.remover = function(){
		var termo = $scope.termoReferenciaChecked;

		if(termo === null) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
			return ;
		}

	    $scope.$openModalConfirm({
	        message: $translate.instant('MSG.CONFIRMA_EXCLUIR_TERMO_REFERENCIA'),
	        callback: function () {
	          TermoReferenciaRepository.remove(termo).then(function() {
	            $scope.termoReferencia = {};

	            $scope.showAlert("success", $translate.instant('MSG.SUCESSO_EXCLUSAO_TERMO_REFERENCIA'));
	            angular.element('#searchTermoReferencia').scope().fetchResult();

	          });
	          
	          $scope.$modalConfirmInstance.dismiss('cancel');
	          $scope.resetForm();
	        }
	      });
	};

	$scope.gerarRelatorioTermoReferencia = function (edit) {
		var termo = $scope.termoReferenciaChecked;
		if (!edit && !termo){
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_GERAR_TERMO'));
			$scope.setLoadingGet(false);
			return;
		}
		$scope.url = '/cit-contratos-web/rest/termoReferencia/pdfGerarTermoReferencia?termoReferenciaId='+ termo.id;
		$scope.visualizarRelatorio($scope.url, $translate.instant('LABEL.TITULO_TERMO_REFERENCIA'));
	};

}]);


