package br.com.centralit.listener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;

import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ConfiguracaoService;
import br.com.centralit.api.service.DefaultFileService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.framework.model.Configuracao;
import br.com.centralit.framework.model.ConfiguracaoParametroSistema;
import br.com.centralit.framework.model.DefaultFile;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Internacionalizacao;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.model.UtilStartup;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 22/02/2016 - 16:46:35
 *
 * @version 1.0.0
 *
 * @author rogerio.cassimiro
 *
 */
@Component
public class StartupListenerContratos extends UtilStartup implements ApplicationListener<ContextRefreshedEvent> {

	@Autowired
	private DominioService dominioService;

	@Autowired
	private DefaultFileService defaultFileService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ModuloService moduloService;

	@Autowired
	private ConfiguracaoService configuracaoService;

	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	private final String DEVERES_RESPONSABILIDADES_CONTRATANTE = "DEVERES_RESPONSABILIDADES_CONTRATANTE";
	private final String DEVERES_RESPONSABILIDADES_CONTRATADA = "DEVERES_RESPONSABILIDADES_CONTRATADA";
	private final String DEVERES_RESPONSABILIDADES_ORGAO = "DEVERES_RESPONSABILIDADES_ORGAO";
	private final String MECANISMOS_FORMAIS_COMUNICACAO = "MECANISMOS_FORMAIS_COMUNICACAO";
	private final String VALOR_CONTRATO_REFERENCIA_GARANTIA = "VALOR_CONTRATO_REFERENCIA_GARANTIA";
	private final String TERMO_ENCERRAMENTO_INTRODUCAO = "TERMO_ENCERRAMENTO_INTRODUCAO";
	private final String TERMO_ENCERRAMENTO_ENCAMINHAMENTOS = "TERMO_ENCERRAMENTO_ENCAMINHAMENTOS";
	private final String TERMO_ENCERRAMENTO_DECISAO = "TERMO_ENCERRAMENTO_DECISAO";

	@Override
	public void onApplicationEvent(final ContextRefreshedEvent event) {

		criarDominios();
		criarDefaultFile();
		gerarInternacionalizacao();
		Configuracao configuracao = configuracaoService.findAll().get(0);
		if (configuracao != null) {
			criarConfiguracoesGeraisContrato(configuracao);
		}
	}

	/**
	 * Método responsável por criar domínios
	 *
	 * @author rogerio.cassimiro
	 *
	 */
	private void criarDominios() {

		List<Dominio> list = new ArrayList<Dominio>();

		// CHAVE TIPO_CONTRATO
		list.add(new Dominio("tipoContrato", "Adminstrativo", "ADMINISTRATIVO", 1L, false));
		list.add(new Dominio("tipoContrato", "Por Empenho", "POR_EMPENHO", 2L, false));
		list.add(new Dominio("tipoContrato", "Dispensa", "DISPENSA", 3L, false));
		list.add(new Dominio("tipoContrato", "Inexigibilidade", "INEXIGIBILIDADE", 4L, false));

		// CHAVE TIPO_AQUISICAO_CONTRATO
		list.add(new Dominio("tipoAquisicaoContrato", "Serviço", "SERVICO", 1L, false));
		list.add(new Dominio("tipoAquisicaoContrato", "Material", "MATERIAL", 2L, false));
		list.add(new Dominio("tipoAquisicaoContrato", "Material e serviço", "MATERIAL_SERVICO", 3L, false));

		// CHAVE TIPO_FORMA_PAGAMENTO_CONTRATO
		list.add(new Dominio("tipoFormaPagamentoContrato", "Único", "UNICO", 1L, false));
		list.add(new Dominio("tipoFormaPagamentoContrato", "Parcelado", "PARCELADO", 2L, false));
		list.add(new Dominio("tipoFormaPagamentoContrato", "Sob demanda", "SOB_DEMANDA", 3L, false));

		// CHAVE TIPO_FINALIDADE_EMPENHO_CONTRATO
		list.add(new Dominio("tipoFinalidadeEmpenhoContrato", "Despesa Contrato", "DESPESA_CONTRATO", 1L, false));
		list.add(new Dominio("tipoFinalidadeEmpenhoContrato", "Reforço", "REFORCO", 2L, false));
		list.add(new Dominio("tipoFinalidadeEmpenhoContrato", "Anulação", "ANULACAO", 3L, false));

		// CHAVE TIPO_CATEGORIA_ECONOMICA_EMPENHO_CONTRATO
		list.add(new Dominio("tipoCategoriaEconomicaEmpenhoContrato", "Despesa Corrente", "DESPESA_CORRENTE", 1L, false));
		list.add(new Dominio("tipoCategoriaEconomicaEmpenhoContrato", "Despesa de Capital", "DESPESA_CAPITAL", 2L, false));

		// CHAVE TIPO_MODALIDADE_LICITACAO_CONTRATO
		list.add(new Dominio("tipoModalidadeLicitacaoContrato", "Pregão", "PREGAO", 1L, false));
		list.add(new Dominio("tipoModalidadeLicitacaoContrato", "Concurso", "CONCURSO", 2L, false));
		list.add(new Dominio("tipoModalidadeLicitacaoContrato", "Concorrência", "CONCORRENCIA", 3L, false));
		list.add(new Dominio("tipoModalidadeLicitacaoContrato", "Convite", "CONVITE", 4L, false));
		list.add(new Dominio("tipoModalidadeLicitacaoContrato", "Tomada de Preço", "TOMADA_PRECO", 5L, false));
		list.add(new Dominio("tipoModalidadeLicitacaoContrato", "Leilão", "LEILAO", 6L, false));

		// CHAVE TIPO_LICITACAO_CONTRATO
		list.add(new Dominio("tipoLicitacaoContrato", "Menor preço", "MENOR_PRECO", 1L, false));
		list.add(new Dominio("tipoLicitacaoContrato", "Melhor técnica", "MELHOR_TECNICA", 2L, false));
		list.add(new Dominio("tipoLicitacaoContrato", "Técnica e Preço", "TECNICA_PRECO", 3L, false));
		list.add(new Dominio("tipoLicitacaoContrato", "Maior lance ou oferta", "MAIOR_LANCE_OFERTA", 4L, false));

		// CHAVE TIPO_PAPEL_ENVOLVIDO_CONTRATO
		list.add(new Dominio("tipoPapelEnvolvidoContrato", "Gestor", "GESTOR", 1L, false));
		list.add(new Dominio("tipoPapelEnvolvidoContrato", "Gestor Substituto", "GESTOR_SUBSTITUTO", 2L, false));
		list.add(new Dominio("tipoPapelEnvolvidoContrato", "Fiscal Técnico", "FISCAL_TECNICO", 3L, false));
		list.add(new Dominio("tipoPapelEnvolvidoContrato", "Fiscal Administrativo", "FISCAL_ADMINISTRATIVO", 4L, false));
		list.add(new Dominio("tipoPapelEnvolvidoContrato", "Fiscal Solicitante", "FISCAL_SOLICITANTE", 5L, false));
		list.add(new Dominio("tipoPapelEnvolvidoContrato", "Preposto", "PREPOSTO", 6L, false));

		// CHAVE TIPO_MODALIDADE_GARANTIA_CONTRATO
		list.add(new Dominio("tipoModalidadeGarantiaContrato", "Seguro-garantia", "SEGURO_GARANTIA", 1L, false));
		list.add(new Dominio("tipoModalidadeGarantiaContrato", "Caução em dinheiro ou títulos da dívida pública", "CAUCAO_DINHEIRO_TITULO_DIVIDA_PUBLICA", 2L, false));
		list.add(new Dominio("tipoModalidadeGarantiaContrato", "Fiança bancária", "FIANCA_BANCARIA", 3L, false));

		// CHAVE TIPO_SITUACAO_CONTRATO
		list.add(new Dominio("tipoSituacaoContrato", "Vigente", "VIGENTE", 1L, false));
		list.add(new Dominio("tipoSituacaoContrato", "Encerrado", "ENCERRADO", 2L, false));
		list.add(new Dominio("tipoSituacaoContrato", "Suspenso", "SUSPENSO", 3L, false));
		list.add(new Dominio("tipoSituacaoContrato", "Rescindido", "RESCINDIDO", 4L, false));

		// CHAVE TIPO_NOTIFICACAO_ADVERTENCIA
		list.add(new Dominio("tipoNotificacaoContratoSancao", "Advertência", "TIPO_NOTIFICACAO_ADVERTENCIA", 1L, false));
		list.add(new Dominio("tipoNotificacaoContratoSancao", "Multa", "TIPO_NOTIFICACAO_MULTA", 2L, false));
		list.add(new Dominio("tipoNotificacaoContratoSancao", "Suspensão Temporária", "TIPO_NOTIFICACAO_SUSPENSAO_TEMPORARIA", 3L, false));
		list.add(new Dominio("tipoNotificacaoContratoSancao", "Declaração de Inidoneidade", "TIPO_NOTIFICACAO_DECLARACAO_INIDONEIDADE", 4L, false));

		// CHAVE tipoEventoContrato
		list.add(new Dominio("tipoEventoContrato", "Alteração Contratual (Aditivo)", "ADITIVO", 1L));
		list.add(new Dominio("tipoEventoContrato", "Aplicação de Penalidade", "PENALIDADE", 2L));
		list.add(new Dominio("tipoEventoContrato", "Apostilamento", "APOSTILAMENTO", 3L));
		list.add(new Dominio("tipoEventoContrato", "Atualização de Status", "ATUALIZA_STATUS", 4L));
		list.add(new Dominio("tipoEventoContrato", "Liberação para Pagamento", "LIBERA_PAGAMENTO", 5L));
		list.add(new Dominio("tipoEventoContrato", "Registro de Empenho", "EMPENHO", 6L));
		list.add(new Dominio("tipoEventoContrato", "Registro de Não Conformidade", "NAO_CONFORMIDADE", 7L));
		list.add(new Dominio("tipoEventoContrato", "Registro de Recebimento de Objeto", "RECEBIMENTO_OBJETO", 8L));
		list.add(new Dominio("tipoEventoContrato", "Repactuação", "REPACTUACAO", 9L));
		list.add(new Dominio("tipoEventoContrato", "Rescisão de Contrato", "RESCISAO", 10L));

		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Provisório", "PROVISORIO", 1L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Definitivo", "DEFINITIVO", 2L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Recebido", "RECEBIDO", 3L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Devolvido", "DEVOLVIDO", 4L));

		list.add(new Dominio("tipoEventoContratoRecebimentoObjetoAvaliacaoPrazo", "Entrega no prazo", "ENTREGA_NO_PRAZO", 1L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjetoAvaliacaoPrazo", "Entrega com atraso", "ENTREGA_COM_ATRASO", 2L));

		list.add(new Dominio("tipoEventoContratoRecebimentoObjetoAvaliacaoAdesao", "Aderente aos termos contratuais", "ADERENTE_AOS_TERMOS", 1L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjetoAvaliacaoAdesao", "Descumprimento contratual", "DESCUMPRIMENTO_CONTRATUAL", 2L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjetoAvaliacaoAdesao", "Passível de correção", "PASSIVEL_CORRECAO", 3L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjetoAvaliacaoAdesao", "Outros", "OUTROS", 4L));

		// CHAVE tipoAlteracaoContrato
		list.add(new Dominio("tipoAlteracaoContrato", "Unilateral pela Administração", "UNILATERAL", 1L));
		list.add(new Dominio("tipoAlteracaoContrato", "Por acordo das partes", "ACORDO", 2L));

		// CHAVE tipoAlteracaoValor
		list.add(new Dominio("tipoAlteracaoValor", "Acréscimo", "ACRESCIMO", 1L));
		list.add(new Dominio("tipoAlteracaoValor", "Supressão", "SUPRESSAO", 2L));

		// CHAVE tipoObjeto
		list.add(new Dominio("tipoObjeto", "Obras, serviços ou compras", "OBRAS_SERVICOS_COMPRAS", 1L));
		list.add(new Dominio("tipoObjeto", "Reforma de edifícios (Imóveis) ou de equipamentos", "REFORMA_IMOVEIS_EQUIPAMENTOS", 2L));

		// CHAVE tipoRescisaoContrato
		list.add(new Dominio("tipoRescisaoContrato", "Unilateralmente por parte da Administração", "RESCISAO_UNILATERAL_ADM", 1L));
		list.add(new Dominio("tipoRescisaoContrato", "Por acordo das partes", "RESCISAO_ACORDO_PARTE", 2L));

		list.add(new Dominio("tipoPrograma", "Temático", "TEMATICO", 1L));
		list.add(new Dominio("tipoPrograma", "Gestão e manutenção", "GESTAO_MANUTENCAO", 2L));
		list.add(new Dominio("tipoPrograma", "Operações especiais", "OPERACOES_ESPECIAIS", 3L));

		list.add(new Dominio("tipoSituacao", "Ativo", "ATIVO", 1L));
		list.add(new Dominio("tipoSituacao", "Inativo", "INATIVO", 2L));

		list.add(new Dominio("tipoSituacaoOrdemFornecimento", "Em andamento", "EM_ANDAMENTO", 1L));
		list.add(new Dominio("tipoSituacaoOrdemFornecimento", "Finalizado", "FINALIZADO", 2L));

		list.add(new Dominio("tipoPapelEquipeContratos", "Gestor", "GESTOR", 1L));
		list.add(new Dominio("tipoPapelEquipeContratos", "Gestor substituto", "GESTOR_SUBSTITUTO", 2L));
		list.add(new Dominio("tipoPapelEquipeContratos", "Fiscal técnico", "FISCAL_TECNICO", 3L));
		list.add(new Dominio("tipoPapelEquipeContratos", "Fiscal administrativo", "FISCAL_ADMINISTRATIVO", 4L));
		list.add(new Dominio("tipoPapelEquipeContratos", "Fiscal solicitante", "FISCAL_SOLICITANTE", 5L));
		list.add(new Dominio("tipoPapelEquipeContratos", "Preposto", "PREPOSTO", 6L));
		list.add(new Dominio("tipoAquisicao", "Serviço", "SERVICO", 1L));
		list.add(new Dominio("tipoAquisicao", "Material", "MATERIAL", 2L));
		list.add(new Dominio("tipoAquisicao", "Material e serviço", "MATERIAL_SERVICO", 3L));
		list.add(new Dominio("tipoSituacaoContrato", "Vigente", "VIGENTE", 1L));
		list.add(new Dominio("tipoSituacaoContrato", "Encerrado", "ENCERRADO", 2L));
		list.add(new Dominio("tipoSituacaoContrato", "Suspenso", "SUSPENSO", 3L));
		list.add(new Dominio("tipoSituacaoContrato", "Rescindido", "RESCINDIDO", 4L));
		list.add(new Dominio("tipoContrato", "Administrativo", "ADMINISTRATIVO", 1L));
		list.add(new Dominio("tipoContrato", "Por Empenho", "POR_EMPENHO", 2L));
		list.add(new Dominio("tipoContrato", "Por Modalidade", "POR_MODALIDADE", 3L));
		list.add(new Dominio("tipoModalidadeContrato", "Concorrência (pregão)", "CONCORRENCIA", 1L));
		list.add(new Dominio("tipoModalidadeContrato", "Convite", "CONVITE", 2L));
		list.add(new Dominio("tipoModalidadeContrato", "Tomada de Preço", "TOMADA_PRECO", 3L));
		list.add(new Dominio("tipoModalidadeContrato", "Concurso", "CONCURSO", 4L));
		list.add(new Dominio("tipoModalidadeContrato", "Leilão", "LEILAO", 5L));
		list.add(new Dominio("tipoGarantia", "Contratual", "CONTRATUAL", 1L));
		list.add(new Dominio("tipoGarantia", "Fornecedor", "FORNECEDOR", 2L));
		list.add(new Dominio("tipoModalidadeGarantia", "Calção em dinheiro ou títulos da dívida pública", "CALCAO_DINHEIRO_TITULOS", 1L));
		list.add(new Dominio("tipoModalidadeGarantia", "Seguro-garantia", "SEGURO_GARANTIA", 2L));
		list.add(new Dominio("tipoModalidadeGarantia", "Fiança bancária", "FIANCA_BANCARIA", 3L));
		list.add(new Dominio("tipoQuestionario", "Geral", "GERAL", 1L));
		list.add(new Dominio("tipoQuestionario", "Checklist de análise de Solução", "ANALISE_SOLUCAO", 2L));
		list.add(new Dominio("tipoRespostaQuestao", "Texto livre", "TEXTO_LIVRE", 1L));
		list.add(new Dominio("tipoRespostaQuestao", "Múltipla Escolha", "MULTIPLA_ESCOLHA", 2L));
		list.add(new Dominio("tipoRespostaQuestao", "Booleana", "BOOLEAN", 3L));
		list.add(new Dominio("tipoRespostaQuestao", "Única Escolha", "UNICA_ESCOLHA", 4L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Apostilamento", "TIPO_OCORRENCIA_APOSTILAMENTO_EXECUCAO", 1L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Termo aditivo", "TIPO_OCORRENCIA_CADASTRO_TERMO_ADITIVO", 2L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Acionamento garantia", "TIPO_OCORRENCIA_ACIONAMENTO_GARANTIA", 3L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Infração", "TIPO_OCORRENCIA_INFRACAO", 4L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Penalização", "TIPO_OCORRENCIA_PENALIZACAO", 5L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Rescisão", "TIPO_OCORRENCIA_RESCISAO", 6L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Verificação realizada", "TIPO_OCORRENCIA_VERIFICACAO", 7L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Liberação de pagamento", "TIPO_OCORRENCIA_PAGAMENTO", 8L));
		list.add(new Dominio("tipoOcorrenciaContrato", "Empenho", "TIPO_OCORRENCIA_EMPENHO", 9L));

		list.add(new Dominio("tipoFormaPagamento", "Único", "UNICO", 1L));
		list.add(new Dominio("tipoFormaPagamento", "Parcelado", "PARCELADO", 2L));
		list.add(new Dominio("tipoFormaPagamento", "Sob demanda", "SOB_DEMANDA", 3L));

		list.add(new Dominio("tipoEmpreitada", "Preço global", "PRECO_GLOBAL", 1L));
		list.add(new Dominio("tipoEmpreitada", "Preço unitário", "PRECO_UNITARIO", 2L));

		list.add(new Dominio("tipoAdjudicacaoObjeto", "Global", "GLOBAL", 1L));
		list.add(new Dominio("tipoAdjudicacaoObjeto", "Por item", "ITEM", 2L));

		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Provisório", "PROVISORIO", 1L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Definitivo", "DEFINITIVO", 2L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Recebido", "RECEBIDO", 3L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Devolvido", "DEVOLVIDO", 4L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Entrega no prazo", "ENTREGA_NO_PRAZO", 5L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Entrega com atraso", "ENTREGA_COM_ATRASO", 6L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Aderente aos termos contratuais", "ADERENTE_AOS_TERMOS", 7L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Descumprimento contratual", "DESCUMPRIMENTO_CONTRATUAL", 8L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Passível de correção", "PASSIVEL_CORRECAO", 9L));
		list.add(new Dominio("tipoEventoContratoRecebimentoObjeto", "Outros", "OUTROS", 10L));

		this.dominioService.saveListIfNotExist(list);
		Logger.getLogger(StartupListenerContratos.class).info("Domínios executados - StartupListenerContratos!");
	}

	/**
	 *
	 * Método responsável por criar arquivos padrao para inicializacao do modulo
	 *
	 * @author iago.almeida
	 *
	 */
	private void criarDefaultFile() {

		dominioCSS = this.dominioService.findByChaveAndNome(TIPO_FILE, TIPO_FILE_CSS);
		this.defaultFileService.saveIfNotExist(new DefaultFile("/cit-contratos-web/assets/css/module-custom.css", dominioCSS, 9));
	}

	/**
	 *
	 * M&eacute;todo respons&aacute;vel por criar as defini&ccedil;&otilde;es de contratos nas configura&ccedil;&otilde;es gerais do sistema
	 *
	 * @author ciro.junior (<a href="mailto:ciro.junior@centralit.com.br">ciro.junior@centralit.com.br</a>)
	 *
	 * @param modulo
	 */
	public void criarConfiguracoesGeraisContrato(Configuracao configuracao) {

		Logger.getLogger(StartupListenerContratos.class).info("Inicializando configura\u00e7\u00e3o dos parametros de sistema para o m\u00f3dulo de contratos...");

		List<ConfiguracaoParametroSistema> configuracaoParametroSistema = configuracaoParametroSistemaService.findAll();
		Map <String, ConfiguracaoParametroSistema> parametrosEncontrados = new HashMap<>();

		for (ConfiguracaoParametroSistema item : configuracaoParametroSistema) {
			switch (item.getChave()) {
				case DEVERES_RESPONSABILIDADES_CONTRATANTE :
					parametrosEncontrados.put(DEVERES_RESPONSABILIDADES_CONTRATANTE, item);
					break;
				case DEVERES_RESPONSABILIDADES_CONTRATADA :
					parametrosEncontrados.put(DEVERES_RESPONSABILIDADES_CONTRATADA, item);
					break;
				case DEVERES_RESPONSABILIDADES_ORGAO :
					parametrosEncontrados.put(DEVERES_RESPONSABILIDADES_ORGAO, item);
					break;
				case MECANISMOS_FORMAIS_COMUNICACAO :
					parametrosEncontrados.put(MECANISMOS_FORMAIS_COMUNICACAO, item);
					break;
				case TERMO_ENCERRAMENTO_INTRODUCAO :
					parametrosEncontrados.put(TERMO_ENCERRAMENTO_INTRODUCAO, item);
					break;
				case TERMO_ENCERRAMENTO_ENCAMINHAMENTOS :
					parametrosEncontrados.put(TERMO_ENCERRAMENTO_ENCAMINHAMENTOS, item);
					break;
				case TERMO_ENCERRAMENTO_DECISAO :
					parametrosEncontrados.put(TERMO_ENCERRAMENTO_DECISAO, item);
					break;
				case VALOR_CONTRATO_REFERENCIA_GARANTIA :
					parametrosEncontrados.put(VALOR_CONTRATO_REFERENCIA_GARANTIA, item);
					break;
			}
		}

		List<ConfiguracaoParametroSistema> parametrosContratos = new ArrayList<ConfiguracaoParametroSistema>();

		if (!parametrosEncontrados.containsKey(DEVERES_RESPONSABILIDADES_CONTRATANTE)){

			StringBuilder sbDeveresResponsabilidadesContratante = new StringBuilder();
			sbDeveresResponsabilidadesContratante.append("Conforme Instru\u00e7\u00e3o Normativa 04/2014 - SLTI/MPOG:\n");
			sbDeveresResponsabilidadesContratante.append("a) Nomear Gestor e Fiscais T\u00e9cnico, Administrativo e Requisitante do contrato para acompanhar e fiscalizar a execu\u00e7\u00e3o dos contratos, conforme o disposto no art. 30 desta Instru\u00e7\u00e3o Normativa;\n");
			sbDeveresResponsabilidadesContratante.append("b) Encaminhar formalmente a demanda, preferencialmente por meio de Ordem de Servi\u00e7o ou Fornecimento de Bens, de acordo com os crit\u00e9rios estabelecidos no Termo de Refer\u00eancia ou Projeto B\u00e1sico;\n");
			sbDeveresResponsabilidadesContratante.append("c) Receber o objeto fornecido pela contratada que esteja em conformidade com a proposta aceita, conforme inspe\u00e7\u00f5es realizadas, observando o disposto no art. 21;\n");
			sbDeveresResponsabilidadesContratante.append("d) Aplicar \u00e0 contratada as san\u00e7\u00f5es administrativas regulamentares e contratuais cab\u00edveis, comunicando ao \u00f3rg\u00e3o gerenciador da Ata de Registro de Pre\u00e7os, quando se tratar de contrato oriundo de Ata de Registro de Pre\u00e7os;\n");
			sbDeveresResponsabilidadesContratante.append("e) Liquidar o empenho e efetuar o pagamento \u00e0 contratada, dentro dos prazos preestabelecidos em Contrato;\n");
			sbDeveresResponsabilidadesContratante.append("f) Comunicar \u00e0 contratada todas e quaisquer ocorr\u00eancias relacionadas com o fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o;\n");
			sbDeveresResponsabilidadesContratante.append("g) Definir produtividade ou capacidade m\u00ednima de fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o por parte da contratada, com base em pesquisas de mercado, quando aplic\u00e1vel;\n");
			sbDeveresResponsabilidadesContratante.append("h) Realizar, no momento da licita\u00e7\u00e3o e sempre que poss\u00edvel, dilig\u00eancias e/ou Prova de Conceito com o licitante classificado provisoriamente em primeiro lugar, para fins de comprova\u00e7\u00e3o de atendimento das especifica\u00e7\u00f5es t\u00e9cnicas; e \n");
			sbDeveresResponsabilidadesContratante.append("i) Prever que os direitos de propriedade intelectual e direitos autorais da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o sobre os diversos artefatos e produtos produzidos ao longo do contrato, incluindo a documenta\u00e7\u00e3o, o c\u00f3digo-fonte de aplica\u00e7\u00f5es, os modelos de dados e as bases de dados, perten\u00e7am \u00e0 Administra\u00e7\u00e3o, justificando os casos em que isso n\u00e3o ocorrer.\n");

			parametrosContratos.add(new ConfiguracaoParametroSistema(DEVERES_RESPONSABILIDADES_CONTRATANTE, sbDeveresResponsabilidadesContratante.toString(), configuracao, true ));

		}

		if (!parametrosEncontrados.containsKey(DEVERES_RESPONSABILIDADES_CONTRATADA)){

			StringBuilder sbDeveresResponsabilidadesContratada = new StringBuilder();
			sbDeveresResponsabilidadesContratada.append("Conforme Instru\u00e7\u00e3o Normativa 04/2014 - SLTI/MPOG:\n");
			sbDeveresResponsabilidadesContratada.append("a) Indicar formalmente preposto apto a represent\u00e1-la junto \u00e0 contratante, que dever\u00e1 responder pela fiel execu\u00e7\u00e3o do contrato;\n");
			sbDeveresResponsabilidadesContratada.append("b) Atender prontamente quaisquer orienta\u00e7\u00f5es e exig\u00eancias do fiscal do contrato, inerentes \u00e0 execu\u00e7\u00e3o do objeto contratual;\n");
			sbDeveresResponsabilidadesContratada.append("c) Reparar quaisquer danos diretamente causados \u00e0 contratante ou a terceiros por culpa ou dolo de seus representantes legais, prepostos ou empregados, em decorr\u00eancia da rela\u00e7\u00e3o contratual, n\u00e3o excluindo ou reduzindo a responsabilidade da fiscaliza\u00e7\u00e3o ou o acompanhamento da execu\u00e7\u00e3o dos servi\u00e7os pela contratante;\n");
			sbDeveresResponsabilidadesContratada.append("d) Propiciar todos os meios e facilidades necess\u00e1rias \u00e0 fiscaliza\u00e7\u00e3o da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o pela contratante, cujo representante ter\u00e1 poderes para sustar o fornecimento, total ou parcialmente, em qualquer tempo, sempre que considerar a medida necess\u00e1ria; \n");
			sbDeveresResponsabilidadesContratada.append("e) Manter, durante toda a execu\u00e7\u00e3o do contrato, as mesmas condi\u00e7\u00f5es da habilita\u00e7\u00e3o;\n");
			sbDeveresResponsabilidadesContratada.append("f) Quando especificada, manter, durante a execu\u00e7\u00e3o do Contrato, equipe t\u00e9cnica composta por profissionais devidamente habilitados, treinados e qualificados para fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o;\n");
			sbDeveresResponsabilidadesContratada.append("g) Manter a produtividade ou a capacidade m\u00ednima de fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o durante a execu\u00e7\u00e3o do contrato, conforme art. 18, inciso I, al\u00ednea \u201cg\u201d.\n");
			sbDeveresResponsabilidadesContratada.append("i) Fornecer, sempre que solicitado, amostra para realiza\u00e7\u00e3o de Prova de Conceito para fins de comprova\u00e7\u00e3o de atendimento das especifica\u00e7\u00f5es t\u00e9cnicas, conforme art. 18, inciso I, al\u00ednea \u201ch\u201d;\n");
			sbDeveresResponsabilidadesContratada.append("j) Ceder os direitos de propriedade intelectual e direitos autorais da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o sobre os diversos artefatos e produtos produzidos ao longo do contrato, incluindo a documenta\u00e7\u00e3o, os modelos de dados e as bases de dados, \u00e0 Administra\u00e7\u00e3o.\n");

			parametrosContratos.add(new ConfiguracaoParametroSistema(DEVERES_RESPONSABILIDADES_CONTRATADA, sbDeveresResponsabilidadesContratada.toString(), configuracao, true ));

		}

		if (!parametrosEncontrados.containsKey(DEVERES_RESPONSABILIDADES_ORGAO)){

			StringBuilder sbDeveresResponsabilidadesOrgao = new StringBuilder();
			sbDeveresResponsabilidadesOrgao.append("Conforme Instru\u00e7\u00e3o Normativa 04/2014 - SLTI/MPOG:\n");
			sbDeveresResponsabilidadesOrgao.append("a) Efetuar o registro do licitante fornecedor e firmar a correspondente Ata de Registro de Pre\u00e7os;\n");
			sbDeveresResponsabilidadesOrgao.append("b) Conduzir os procedimentos relativos a eventuais renegocia\u00e7\u00f5es de condi\u00e7\u00f5es, produtos ou pre\u00e7os registrados;\n");
			sbDeveresResponsabilidadesOrgao.append("c) Aplicar as penalidades por descumprimento do pactuado na Ata de Registro de Pre\u00e7os;\n");
			sbDeveresResponsabilidadesOrgao.append("d) Autorizar ou n\u00e3o o fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o para \u00f3rg\u00e3o n\u00e3o participante da Ata de Registro de Pre\u00e7os, desde que prevista no instrumento convocat\u00f3rio, consultando o benefici\u00e1rio da Ata e verificando as condi\u00e7\u00f5es de fornecimento, de forma a evitar extrapola\u00e7\u00f5es dos limites de produtividade ou de capacidade m\u00ednima de fornecimento da Solu\u00e7\u00e3o;\n");
			sbDeveresResponsabilidadesOrgao.append("e) Definir mecanismos de comunica\u00e7\u00e3o com os \u00f3rg\u00e3os participantes, n\u00e3o participantes, contendo:\n");
			sbDeveresResponsabilidadesOrgao.append("\tI. As formas de comunica\u00e7\u00e3o entre os envolvidos, a exemplo de of\u00edcio, telefone, e-mail, ou sistema informatizado, quando dispon\u00edvel;\n");
			sbDeveresResponsabilidadesOrgao.append("\tII. Defini\u00e7\u00e3o dos eventos a serem reportados ao \u00f3rg\u00e3o gerenciador, com a indica\u00e7\u00e3o de prazo e respons\u00e1vel, a exemplo de ordem de servi\u00e7o ou fornecimento de bens, aplica\u00e7\u00e3o de san\u00e7\u00f5es administrativas, altera\u00e7\u00e3o de item registrado em Ata por modelo equivalente ou superior;\n");
			sbDeveresResponsabilidadesOrgao.append("f) Definir mecanismos de controle de fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o, observando, entre outros:\n");
			sbDeveresResponsabilidadesOrgao.append("\tI. A defini\u00e7\u00e3o da produtividade ou da capacidade m\u00ednima de fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o;\n");
			sbDeveresResponsabilidadesOrgao.append("\tII. Regras para fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o aos \u00f3rg\u00e3os n\u00e3o participantes, desde que previsto no instrumento convocat\u00f3rio, cujo fornecimento n\u00e3o poder\u00e1 prejudicar os compromissos j\u00e1 assumidos e as futuras contrata\u00e7\u00f5es dos \u00f3rg\u00e3os participantes do registro de pre\u00e7os;\n");
			sbDeveresResponsabilidadesOrgao.append("\tIII. Regras para gerenciamento da fila de fornecimento da Solu\u00e7\u00e3o de Tecnologia da Informa\u00e7\u00e3o aos \u00f3rg\u00e3os participantes e n\u00e3o participantes, contendo prazos e formas de negocia\u00e7\u00e3o e redistribui\u00e7\u00e3o da demanda, quando esta ultrapassar a produtividade definida ou a capacidade m\u00ednima de fornecimento e for requerida pela Contratada;\n");
			sbDeveresResponsabilidadesOrgao.append("g) Atualiza\u00e7\u00e3o da ata de registro de pre\u00e7os, na forma estabelecida pela legisla\u00e7\u00e3o, por meio da substitui\u00e7\u00e3o da Solu\u00e7\u00e3o registrada, garantida a realiza\u00e7\u00e3o de Prova de Conceito, observado o disposto no inciso III, al\u00ednea \u201ce\u201d, item 2 deste artigo;\n");
			sbDeveresResponsabilidadesOrgao.append("h) Previs\u00e3o da exig\u00eancia para realiza\u00e7\u00e3o de dilig\u00eancias e/ou Prova de Conceito com o licitante provisoriamente classificado em primeiro lugar para fins de comprova\u00e7\u00e3o de atendimento das especifica\u00e7\u00f5es t\u00e9cnicas.\n");

			parametrosContratos.add(new ConfiguracaoParametroSistema(DEVERES_RESPONSABILIDADES_ORGAO, sbDeveresResponsabilidadesOrgao.toString(), configuracao, true ));

		}

		if (!parametrosEncontrados.containsKey(MECANISMOS_FORMAIS_COMUNICACAO)){

			StringBuilder sbMecanismosFormaisComunicacao = new StringBuilder();
			sbMecanismosFormaisComunicacao.append("* S\u00e3o mecanismos formais de comunica\u00e7\u00e3o entre a Contratada e a Contratante:\n");
			sbMecanismosFormaisComunicacao.append("- E-mails: forma r\u00e1pida de comunica\u00e7\u00e3o para tratar de informa\u00e7\u00f5es pouco cr\u00edticas;\n");
			sbMecanismosFormaisComunicacao.append("- Of\u00edcios: Comunica\u00e7\u00e3o para tratar de assuntos gerais;\n");
			sbMecanismosFormaisComunicacao.append("- Ordem de Servi\u00e7o:  elaborada, por demanda, pela Contratante e encaminhada via sistema eletr\u00f4nico, correio eletr\u00f4nico ou liga\u00e7\u00e3o telef\u00f4nica \u00e0 Contratada, com a fun\u00e7\u00e3o de demandar servi\u00e7os contratados;\n");
			sbMecanismosFormaisComunicacao.append("- Termo de Recebimento Provis\u00f3rio:  termo elaborado pela Contratante e encaminhado \u00e0 Contratada;\n");
			sbMecanismosFormaisComunicacao.append("- Termo de Recebimento Definitivo:  termo elaborado pela Contratante e encaminhado \u00e0 Contratada.\n");

			parametrosContratos.add(new ConfiguracaoParametroSistema(MECANISMOS_FORMAIS_COMUNICACAO, sbMecanismosFormaisComunicacao.toString(), configuracao, true ));
		}

		//Se o valor do contrato for maior que RS37.500.000,00 a garantia eh de no maximo 10 por cento se for menor o valor maximo eh de no maximo 5 por cento.
		if (!parametrosEncontrados.containsKey(VALOR_CONTRATO_REFERENCIA_GARANTIA)){
			parametrosContratos.add(new ConfiguracaoParametroSistema(VALOR_CONTRATO_REFERENCIA_GARANTIA, "37500000", configuracao, true ));
		}

		if(!parametrosEncontrados.containsKey(TERMO_ENCERRAMENTO_INTRODUCAO)){
			StringBuilder sbTermoEncerramentoIntroducao = new StringBuilder();
			sbTermoEncerramentoIntroducao.append("Por este instrumento, as partes acima identificadas resolvem registrar o encerramento do contrato em ep\u00edgrafe e ressaltar o que segue:\r\nO presente contrato est\u00e1 sendo encerrado por motivo de:");
			parametrosContratos.add(new ConfiguracaoParametroSistema(TERMO_ENCERRAMENTO_INTRODUCAO, sbTermoEncerramentoIntroducao.toString(), configuracao, true ));
		}

		if(!parametrosEncontrados.containsKey(TERMO_ENCERRAMENTO_ENCAMINHAMENTOS)){
			StringBuilder sbTermoEncerramentoEncaminhamentos = new StringBuilder();
			sbTermoEncerramentoEncaminhamentos.append("As partes concedem-se mutuamente plena, geral, irrestrita e irrevog\u00e1vel quita\u00e7\u00e3o de todas as obriga\u00e7\u00f5es diretas e indiretas decorrentes deste contrato, n\u00e3o restando mais nada a reclamar de parte a parte, exceto as relacionadas no par\u00e1grafo a seguir.\r\nN\u00e3o est\u00e3o abrangidas pela quita\u00e7\u00e3o ora lan\u00e7ada e podem ser objeto de exig\u00eancia ou responsabiliza\u00e7\u00e3o, mesma ap\u00f3s o encerramento do v\u00ednculo contratual:\r\n\r\na) As obriga\u00e7\u00f5es relacionadas a processos iniciados de penaliza\u00e7\u00e3o contratual;\r\nb) As garantias sobre bens e servi\u00e7os entregues ou prestados, tanto legais quanto convencionais;\r\nc) A reclama\u00e7\u00e3o de qualquer tipo sobre defeitos ocultos nos produtos ou servi\u00e7os entregues ou prestados.");
			parametrosContratos.add(new ConfiguracaoParametroSistema(TERMO_ENCERRAMENTO_ENCAMINHAMENTOS, sbTermoEncerramentoEncaminhamentos.toString(), configuracao, true ));
		}

		if(!parametrosEncontrados.containsKey(TERMO_ENCERRAMENTO_DECISAO)){
			StringBuilder sbTermoEncerramentoDecisao = new StringBuilder();
			sbTermoEncerramentoDecisao.append("E assim tendo lido e concordado com todos os seus termos, firmam as partes o presente instrumento, em duas vias iguais, para que surta seus efeitos jur\u00eddicos.");
			parametrosContratos.add(new ConfiguracaoParametroSistema(TERMO_ENCERRAMENTO_DECISAO, sbTermoEncerramentoDecisao.toString(), configuracao, true ));
		}

		if (CollectionUtils.isNotEmpty(parametrosContratos)){
			configuracaoParametroSistemaService.saveList(parametrosContratos);
			Logger.getLogger(StartupListenerContratos.class).info("Par\u00e2metros cadastrados !");
		} else {
			Logger.getLogger(StartupListenerContratos.class).info("Nenhum par\u00e2metro de configura\u00e7\u00e3o do m\u00f3dulo de contratos \u00e9 necess\u00e1rio ser adicionado, pois j\u00e1 existem no banco de dados.");
		}
	}
	/**
	 *
	 * Método responsável por gerar internacionalização do módulo Contratos
	 *
	 * @author rogerio.cassimiro
	 *
	 */
	public void gerarInternacionalizacao() {

		dominioPT_BR = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		moduloSelecionado = this.moduloService.getModuloPorBaseUrl(BASE_URL_CONTRATO, null);
		List<Internacionalizacao> internacionalizacaoList = new ArrayList<Internacionalizacao>();

		gerarMensagem(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		gerarValidacao(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		gerarLabel(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		gerarLabelsContrato(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		gerarValidacaoContrato(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		internacionalizacaoService.saveListIfNotExist(internacionalizacaoList);

		Logger.getLogger(StartupListenerContratos.class).info("Internacionalização executada - StartupListenerContratos!");
	}

	private void gerarLabelsContrato(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VIGENCIA", "Vigência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.GESTOR", "Gestor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.ADICIONAR_PERGUNTA", "Adicionar pergunta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.IMPRIMIR", "Imprimir", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.CODIGO_PROGRAMA_DUPLICADO", "Já existe um programa cadastrado com o mesmo código.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.ACOES_VAZIAS", "Deve existir no mínimo uma ação cadastrada para este programa!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NOME_PROGRAMA_DUPLICADO", "Já existe um programa cadastrado com o mesmo nome.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.CODIGO_ACAO_DUPLICADO", "Já existe uma ação cadastrada com o mesmo código  ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NOME_ACAO_DUPLICADO", "Já existe uma ação cadastrada com o mesmo nome  ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NECESSIDADES_VAZIAS", "Deve existir no mínimo uma necessidade elencada para este objetivo estratégico!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.BENS_E_SERVICOS_JA_ADD", "Já existe um detalhamento da solução com o mesmo bens e serviços selecionado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.DATA_ENTREGA_CRONOGRAMA_FORA_VIGENCIA", "A data da entrega deve estar dentro da vigência do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_TOTAL_EMPENHADO_SUPERIOR_VALOR_CONTRATADO", "O valor total empenhado é superior ao valor contratado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_TOTAL_ANULACAO_SUPERIOR_VALOR_TOTAL", "O valor total das anulações não pode ser superior ao valor total empenhado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_TOTAL_ITENS_SUPERIOR_VALOR_CONTRATADO", "O valor total dos itens do contrato é superior ao valor contratado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NECESSARIO_VALOR_CONTRATO_INFO_EMPENHO", "Para adicionar informações de empenho, é necessário informar o valor do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NECESSARIO_VALOR_CONTRATO_INFO_ITENS_CONTRATO", "Para adicionar informações de itens do contrato, é necessário informar o valor do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NECESSARIO_CONTRATO_INFO_CRONOGRAMA", "Para adicionar itens ao cronograma, é necessário selecionar um contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NECESSARIO_ITEM_CRONOGRAMA", "Você deve adicioar pelo menos um item à tabela.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.DATA_ENTREGA_CRONOGRAMA_FORA_VIGENCIA_CONTRATO", "A data da entrega deve estar dentro da vigência do contrato. Verifique as parcelas  ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_TOTAL_CRONOGRAMA_ACIMA_VALOR_CONTRATO", "A soma dos valores dos itens adicionados à tabela deverá ser menor ou igual ao valor total do contrato vinculado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.PAPEIS_OBRIGATORIOS_CONTRATO_FALTANDO", "Estão faltando um ou mais papéis obrigatórios dos envolvidos no contrato. Verifique quais dentre os seguintes papéis não forma informados  ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.FINALIDADE_EMPENHO_FALTANDO", "Não é permitido informar as finalidades 'Anulação' e 'Reforço' antes de informar a finalidade 'Despesa contrato'.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_GARANTIA_ACIMA_LIMITE_MAXIMO", "O valor da garantia do contrato está acima do valor máximo. Valor máximo da garantia  ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.ALTERACAO_ENVOLVIDOS_EXIGE_JUSTIFICATIVA", "É obrigatório informar uma justificativa para alterar os envolvidos do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_LIBERADO_MAIOR_VALOR_CONTRATO", "Valor liberado não pode ser maior que o valor do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_LIBERADO_MAIOR_VALOR_OFBS", "O valor liberado para pagamento não pode ser maior que o valor liberado referente às ordem de fornecimento de bens e serviços", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_TOTAL_CONTRATO_JA_LIBERADO", "Já liberado o pagamento no valor do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_TOTAL_A_LIBERAR_ULTRAPASSA_VALOR_CONTRATO", "Valor liberado para pagamento superior ao valor do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NUMERO_CONTRATO_DUPLICADO", "Já existe um contrato cadastrado com o mesmo número/ano.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.NUMERO_CONTRATO_DUPLICADO", "Já existe um contrato cadastrado com o mesmo número/ano.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.ANO_EXERCICIO", "Ano exercício", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_PROGRAMA", "Tipo do programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CODIGO_PROGRAMA", "Código do programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NOME_PROGRAMA", "Nome do programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CODIGO_ACAO", "Código da ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NOME_ACAO", "Nome da ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DESCRICAO_ACAO", "Descrição da ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.BASE_LEGAL", "Base legal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_CONTRATO_REFERENCIA_GARANTIA", "Valor de referência do contrato para o percentual da garantia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CODIGO_CRONOGRAMA", "Código do cronograma", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CRONOGRAMA_EXECUCAO", "Cronograma de Execução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.ITENS_CRONOGRAMA", "Itens do cronograma", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CONTRATADA", "Contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_TOTAL", "Valor total", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NOME_ITEM_CRONOGRAMA", "Nome do item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.PAGAMENTO_PARCELADO_QUESTAO", "Pagamento parcelado?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.PAGAMENTO_PARCELADO", "Pagamento parcelado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUMERO_PARCELAS", "Número de parcelas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUMERO_PARCELAS", "Número de parcelas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_UNITARIO_ITEM", "Valor unitário do item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_TOTAL_ITEM", "Valor total do item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_ITEM", "Valor do item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_ENTREGA", "Data da entrega", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.PARCELAS", "Parcelas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVAS", "Justificativas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR_CRONOGRAMA", "Gerar cronograma", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR_APOSTILAMENTO", "Gerar apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUMERO_APOSTILAMENTO", "Número do apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_EMISSAO_APOSTILAMENTO", "Data de emissão do apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.JUSTIFICATIVA_APOSTILAMENTO", "Justificativa do apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_APOSTILAMENTO", "Justificativa do apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_REGISTRO_ADMINISTRATIVO", "Descrição do registro administrativo.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DESCRICAO_REGISTRO_ADMINISTRATIVO", "Descrição do registro administrativo.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_DOCUMENTO_APOSTILAMENTO", "Termo de Solicitação de Apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.JUSTIFICATIVA_EXCLUSAO_EVENTO", "Justificativa da exclusão do evento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_ATUALIZACAO_STATUS", "Atualização Status", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_PENALIZACAO", "Data de penalização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SELECIONE_PARA_ADICIONAR", "Selecione um item para ser adicionado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SUCESSO_SALVAR_PENALIDADE", "Sucesso ao salvar a penalidade!", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_APOSTILAMENTO", "Apostilamento de número: %s realizado no dia %s.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SUCESSO_EXCLUSAO_PROGRAMA", "Programa excluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.CONFIRMA_EXCLUSAO_PROGRAMA", "Existem ações cadastradas neste programa. Deseja excluir este programa e TODAS as ações vinculadas?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.CONFIRMA_EXCLUSAO_OBJETIVO", "Existem necessidades elencadas cadastradas neste objetivo estratégico. Deseja excluir este objetivo estratégico e TODAS as necessidades elencadas vinculadas?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.MOSTRAR_CONTRATOS_APOSTILAMENTO", "Mostrar apenas contratos com apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.PARAMETRO_VALOR_CONTRATO_REFERENCIA_GARANTIA", "O parâmetro valor de referência do contrato para o percentual da garantia não está cadastrado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.CONFIRMA_EXCLUSAO_CRONOGRAMA", "Deseja excluir este cronograma e TODAS as parcelas vinculadas?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SUCESSO_EXCLUSAO_CRONOGRAMA", "Cronograma excluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.VALOR_TOTAL_EMPENHADO_MENOR_VALOR_CONTRATO", "O valor total empenhado é menor que o valor do contrato. Deseja continuar?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.VALOR_TOTAL_ITENS_MENOR_VALOR_CONTRATO", "O valor total dos itens do contrato é menor que o valor do contrato. Deseja continuar?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.TODOS_PAPEIS_ENVOLVIDOS_ADICIONADOS", "Todos os papeis de envolvidos já foram adicionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SUCESSO_SALVAR_CRONOGRAMA", "Cronograma de execução salvo com sucesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EMPENHO", "Empenho de valor: :VALOR: realizado no dia :DATAEVENTO:.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SUCESSO_SALVAR_APOSTILAMENTO", "Sucesso ao salvar o apostilamento.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_APOSTILAMENTO_INVALIDA", "A data de emissão do apostilamento deve estar dentro do intervalo de vigência do contrato selecionado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_ATUALIZACAO_STATUS", "Atualização da Situação do Contrato realizado no dia %s. (Situação anterior: %s - Nova situação: %s)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_NAO_CONFORMIDADE", "Registro não conformidade %s, realizado no dia %s.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SANCAO_VINCULADA_NAO_CONFORMIDADE", "Não é permitido remover essa sanção, pois há um evento de não conformidade vinculada a ela.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.VALOR_TOTAL_EMPENHADO_SUPERIOR_VALOR_RESIDUAL", "O valor empenhdo não pode ser maior que o valor residual", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.EVENTO_NAO_CONFORMIDADE_CRIAR_ADVERTENCIA", "Para gerar essa não conformodidade, é necessário gerar uma não conformidade de advertência primeiramente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.EVENTO_NAO_CONFORMIDADE_CRIAR_MULTA", "Para gerar essa não conformodidade, é necessário gerar uma não conformidade de multa primeiramente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.EVENTO_NAO_CONFORMIDADE_CRIAR_SUSPENSAO", "Para gerar essa não conformodidade, é necessário gerar uma não conformidade de suspensão primeiramente.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.ANEXOS", "Anexos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.QUESTAO", "Questão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.PERGUNTAS", "Perguntas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.POSSIVEL_RESPOSTA", "Possível resposta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.FALSO", "Falso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.TIPO_RESPOSTA", "Tipo resposta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VERDADEIRO", "Verdadeiro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.TODAS", "Todas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.ALTA_COMPLEXIDADE", "Alta complexidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RELATORIO_CONTRATOS", "Relatório de contratos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.QUESTIONARIOS", "Questionário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUMERO_ORDEM_SERVICO_ABREV", "Nº ordem de serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_TOTAL_OS", "Valor total da OS", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_COM_GLOSSA", "Valor com glossa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TOTAL_LIBERADO", "Total liberado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.DESCRICAO_EVENTO_EMPENHO", "Empenho (Nº %s) gerado em %s.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_TOTAL_GLOSSA", "Valor total da glossa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_ENCERRAMENTO", "Data encerramento", dominio, modulo));

		//Novas labels referente ao Evento de aditivo (16/03/2016)
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_ADITIVO", "Aditamento de contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_RELATORIO_ADITIVO", "Termo de Solicitação de Aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR_SOLICITACAO_ADITIVO", "Gerar termo de solicitação de aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUM_ADITIVO", "Nº do aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_ADITIVO", "Tipo de aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DE_PRAZO", "De prazo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DE_VALOR", "De valor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DE_CLAUSULA_CONTRATUAL", "De cláusula contratual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.ALTERACAO_PRAZO", "Alteração de prazo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_INICIO_ADITIVO", "Data início do aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_FINAL_ADITIVO", "Data final do aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.ALTERACAO_VALOR", "Alteração de valor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_OBJETO", "Tipo de objeto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.VALOR_ADITIVO", "Valor do aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.PERCENTUAL_ADITIVO", "Percentual do aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.ALTERACAO_CLAUSULA_CONTRATUAL", "Alteração de cláusula contratual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DESCRICAO_ADITIVO", "Descrição do aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.JUSTIFICATIVA_ADITIVO", "Justificativa do aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.MANIFESTACAO_CONTRATADA", "Manifestação da contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_VIGENCIA_INICIAL_CONTRATO", "Data vigência inicial do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_VIGENCIA_FINAL_CONTRATO", "Data vigência final do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_VIGENCIA_INICIAL_ADITIVO", "Aditivo de data vigência inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_VIGENCIA_FINAL_ADITIVO", "Aditivo de data vigência final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUM_REPACTUACAO", "Nº da repactuação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_REPACTUACAO", "Data da repactuação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.JUSTIFICATIVA_REPACTUACAO", "Justificativa da repactuação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_REPACTUACAO", "Repactuação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR_SOLICITACAO_REPACTUACAO", "Gerar relatório de repactuação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_RELATORIO_REPACTUACAO", "Relatório de Repactuação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR_TERMO_REPACTUACAO","Gerar termo de repactuação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_LIST_REPACTUACOES", "Repactuações do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_LIST_ADITIVOS", "Aditivos do contrato", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_INICIO_ADITIVO_MAIOR_DATA_FINAL", "A data de início do aditivo deve ser maior que a data final do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_INICIO_ADITIVO_MAIOR_DATA_FIM_ADITIVO", "A data de início do aditivo deve ser menor que a data fim!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_EMISSAO_DENTRO_REFERENCIA", "A data de emissão do aditivo deve estar dentro da referência do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_EMISSAO_12_MESES", "É preciso ter um intervalo de 12 meses após o último aditivo de prazo!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.MAXIMO_ADITIVO_PRAZO", "Você já cadastrou 5 aditivos de prazo para este contrato. Não é possível cadastrar outro!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.PERCENTAGEM_MAIOR_OBRAS_SERVICOS_COMPRAS", "O percentual total de aditivos para Obras, serviços ou compras ultrapassou 25% do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.PERCENTAGEM_MAIOR_REFORMA_IMOVEIS", "O percentual total de aditivos para Reforma de edifícios (Imóveis) ou de equipamentos ultrapassou 50% do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO", "Aditivo de número %s emitído no dia %s. ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO_PRAZO", "Aditivo de prazo (Novo período de %s à %s). ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO_VALOR", "Aditivo de valor (%s de %s). ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_ADITIVO_CLAUSULA", "Aditivo de cláusula contratual. ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_REPACTUACAO", "Repactuação de número %s ocorrida em %s.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.DATA_REPACTUACAO_VIGENCIA", "Data da repactuação deve estar dentro da vigência do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUANTIDADE_INSERIDA_TOTAL_MAIOR_QUANTIDADE_CONTRATO", "A quantidade total de algum dos itens inseridos em todas as ordens de fornecimento deste contrato supera a definida no contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ITEM_PERIODO_FORA_VIGENCIA_CONTRATO", "Item fora do período de vigência do contrato.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.DATA_EMISSAO","Data da emissão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.NUMERO_NOTIFICACAO_ABREV","Nº da notificação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.TIPO_SANCAO","Tipo da sanção",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.NUMERO_CONTRATO_ABREV","Nº do contrato",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.SUCESSO_EXCLUSAO_NOTIFICAO","Notificação excluída com sucesso!",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.CONFIRMA_EXCLUSAO_NOTIFICACAO","Deseja excluir esta notificação?",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.GERAR_NOTIFICACAO_RELATORIO","Gerar notificação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.FORNECEDOR_CONTRATADA","Fornecedor (Contratada)",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.ASSOCIAR_OCORRENCIA","Associar a uma ocorrência?",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.NUMERO_OCORRENCIA_ABREV","Nº da ocorrência",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.TITULO_OCORRENCIA","Título da ocorrência",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.DESCRICAO_NOTIFICACAO","Descrição da notificação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.SELECIONE_OCORRENCIA","Selecione uma ocorrência",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.VALIDA_ASSOCIAR_OCORRENCIA","Informe se há associação de ocorrência",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.SUCESSO_SALVAR_NOTIFICACAO","Notificação salva com sucesso!",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.DATA_EMISSAO_INVALIDA","Data da emissão não pode estar fora da vigência do contrato",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.VALIDA_CONTRATO_SEM_OCORRENCIA","Contrato selecionado não possui ocorrências vinculadas",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.DATA_EMISSAO_VALIDA","Data da emissão válida entre, ",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.VALIDA_CONTRATO_NAO_SELECIONA","Selecione um contrato primeiramente",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.FORNECEDOR_CONTRATADA_CNPJ","CNPJ do fornecedor (Contratada)",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.NOTIFICACAO_TITULO_TERMO","Notificação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("NOTIFICACAO_CONTRATO.REMOCAO_NAO_PERMITIDA","Não é permitido remover uma notificação", dominio, modulo));


		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.LIBERACAO_ULTRAPASSA_LIMITE", "Valor a ser liberado ultrapassa o valor do empenho selecionado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.LIBERACAO_PAGAMENTO_VALOR_EMPENHO_MENOR_VALOR_TOTAL_CONTRATO", "Não é possível realizar a liberação do valor total do contrato, pois não há registros de empenhos para cobrir esse valor.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.TITULO_LIBERAR_PAGAMENTO","Liberar para pagamento",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.NUMERO_LIBERACAO","Nº da liberação de pagamento",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.DATA_LIBERACAO","Data da liberação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.RESPONSAVEL_LIBERACAO","Responsável pela liberação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.NUMERO_ORDEM_FORNECIMENTO","Nº ordem de fornecimento de bens e serviços",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.VALOR_EXECUTADO","Valor executado",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.PERCENTUAL_EXECUTADO","Percentual executado",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.LIBERAR_PAGAMENTO_VALOR_CONTRATA","Liberar pagamento do valor do contrato",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.VALOR_LIBERADO","Valor liberado para pagamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.DATA_LIBERACAO","A data da liberação deverá ser posterior a data de início da vigência do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.DESCRICAO_EVENTO","Liberação para Pagamento (Nº {numeroLiberacaoPagamento}) no valor de {valor} realizado na data de {data}", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.VALOR_TOTAL_LIBERACAO","Valor total da liberação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.SELECIONE_ORDEM","Selecione uma ordem de fornecimento de serviço para adicionar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.SELECIONE_EMPENHO","Selecione um empenho para adicionar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.LIBERACAO_NO_LIMITE","O empenho selecionado encontra-se em seu limite permitido para liberação de pagamento.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.REGISTRO_NAO_CONFORMIDADE.DATA","Data da não conformidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.REGISTRO_NAO_CONFORMIDADE.APLICAVEL","Não conformidade aplicável", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.REGISTRO_NAO_CONFORMIDADE.PERCENTUAL_MULTA","Percentual da multa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.REGISTRO_NAO_CONFORMIDADE.VALOR_MULTA","Valor da multa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.REGISTRO_NAO_CONFORMIDADE.DESCRICAO","Descrição de não conformidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.REGISTRO_NAO_CONFORMIDADE.RESPOSTA_CONTRATADA","Resposta da contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_NAO_CONFORMIDADE_INVALIDA", "Data de não conformidade deve estar dentro do intervalo de vigência do contrato.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_CONTRATO","Tipo de contrato",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_AQUISICAO","Tipo de aquisição",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.SITUACAO_ATUAL","Situação atual",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NOVA_SITUACAO","Nova situação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_ATUALIZACAO","Data da atualização",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.JUSTIFICATIVA_ATUALIZACAO_STATUS","Justificativa da atualização do status",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_ATUALIZACAO_STATUS_INVALIDA", "A data de atualização deve estar dentro do intervalo de vigência do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.ALTERACAO_STATUS_NEGADA", "A situação do Contrato não poderá ser alterado, uma vez que o mesmo encontra-se ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.ALTERACAO_STATUS_IGUAL", "Não é permitido realizar uma alteração selecionando o mesma stituação do contrato atual.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.TITULO_RESCISAO_CONTRATO","Rescisão de contrato",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.NUM_RESCISAO","Nº da rescisão",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.DATA_RESCISAO","Data da rescisão",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.TIPO_RESCISAO","Tipo de rescisão",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.HA_DEVOLUCAO_VALOR","Há devolução de valor?",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.VALOR_A_DEVOLVER","Valor a ser devolvido",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.ACOES_APLICADAS","Ações aplicadas ao contrato",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.TIPO_ACAO","Tipo de ação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.DATA_ACAO","Data da ação",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_RESCISAO_CONTRATO_SEM_DEVOLUCAO_VALOR","Rescisão de contrato (Nº %s) realizado em %s.",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_RESCISAO_CONTRATO_COM_DEVOLUCAO_VALOR","Rescisão de contrato (Nº %s) realizado em %s, com devolução de R$%s",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.RESCISAO_CONTRATO_VALOR_DEVOLVIDO_MAIOR_CONTRATO","Valor da devolução não pode ultrapassar o valor do contrato.",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.RESCISAO_CONTRATO.GERAR_TERMO","Gerar termo de rescisão de contrato",dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_DOCUMENTO_RESCISAO_CONTRATO", "Termo de Rescisão de Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_LIST_RESCISOES", "Rescisões do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.SELECIONE_CONTRATO", "Selecione o contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_RESCISAO_INVALIDA", "A data de rescisão deve estar dentro do intervalo de vigência do contrato.", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_ANO_CONTRATO", "Número/Ano do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROGRAMA", "Programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROGRAMA_ACAO", "Ação do programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CNPJ_FORNECEDOR", "CNPJ do fornecedor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CONTRATO_RENOVAVEL", "Contrato pode ser renovado?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_PUBLICACAO_LICITACAO", "Data da publicação no DOU", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODALIDADE_LICITACAO", "Modalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_LICITACAO", "Tipo de licitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_EMPENHO", "Informações de empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_LICITACAO", "Informações de licitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_EMPENHO", "Data do empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.FINALIDADE_EMPENHO", "Finalidade do empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CATEGORIA_ECONOMICA", "Categoria econômica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CLASSIFICACAO_DESPESA", "Classificação da despesa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ITEM_CONTRATO", "Item do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_CONTRATO", "Valor do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_PARCELA", "Valor da parcela", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_VENCIMENTO_PARCELA", "Data de vencimento da parcela", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.FORMA_PAGAMENTO_CONTRATO", "Forma de pagamento do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PARCELA", "N° de parcelas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_GARANTIA", "Informações de garantia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_VIGENCIA_GARANTIA", "Data de vigência da garantia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_GARANTIA", "Valor da garantia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERCENTUAL_GARANTIA", "Percentual da garantia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ENVOLVIDO", "Envolvido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_ENVOLVIDO", "Nome do envolvido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_ANO_EDITAL", "Número/Ano do Edital", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_ITEM", "Nome do item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_ITENS_CONTRATO", "Informações de itens do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CONTRATO_EMPENHOS", "Informações de empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_REFERENCIA", "Termo de Referência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.GERAR_TERMO_REFERENCIA", "Gerar termo de referência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DEFINICOES_CONTRATOS", "Definições de Contratos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.GERAR_TERMO_CONTRATO", "Gerar termo de contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CARACTERES_RESTANTES", "Caracteres restantes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_EMPENHADO", "Valor total empenhado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_ITENS_CONTRATO", "Valor total dos itens do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_EMISSAO_OCORRENCIA","Data da emissão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_OCORRENCIA","Título da ocorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_OCORRENCIA","Descrição da ocorrênca (Irregularidades)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EMITIR_GLOSA", "Emitir glosa ?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_GLOSA","Valor da glosa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_OCORRENCIA","Nº da ocorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.GERAR_REGISTRO_OCORRENCIA","Gerar registro de ocorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRO_OCORRENCIA","Registro de Ocorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_ORDEM_SERVICO","Nº ordem de fornecimento de serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRAZO_CUMPRIMENTO_EXIGENCIAS", "Prazo para o cumprimento das exigências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.FORNECEDOR_CONTRATADA", "Fornecedor (Contratada)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_ORDEM_FORNECIMENTO", "Nº da ordem de fornecimento de bens serviços", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_CONTRATO", "Nº do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_GESTOR_CONTRATO", "Nome do gestor do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_PRESTACAO_SERVICO", "Local de prestação do serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ESPECIFICACAO_PRODUTOS_SERVICOS_VOLUMES", "Epecificação dos produtos/serviços e volumes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INICIO_PREVISTO", "Inicio previsto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMINO_PREVISTO", "Término previsto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTDE_TOTAL", "Qtde. total", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TOTAL_GERAL", "Total geral", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.GERAR_ORDEM_FORNECIMENTO", "Gerar ordem de fornecimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ORDEM_FORNECIMENTO", "Ordem de Fornecimento de Bens e Serviços", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_CONTRATANTE", "Selecione contratante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_TERMO", "Nº termo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_TERMO_ANO", "Nº termo/ano", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTRODUCAO", "Introdução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ENCAMINHAMENTOS", "Encaminhamentos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DECISAO", "Decisão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CONTRATADA", "Contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CNPJ_CONTRATADA", "CNPJ da contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.GERAR_ENCERRAMENTO_CONTRATO", "Gerar termo de encerramento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACAO_CONTRATO", "Informações do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_ENCERRAMENTO", "Termos do encerramento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_CELEBRACAO_CONTRATO", "Data da celebração do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_INICIO_VIGENCIA", "Data de início da vigência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_TERMINO_VIGENCIA", "Data de término da vigência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_ENCERRAMENTO_INTRODUCAO", "Introdução do termo de encerramento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_ENCERRAMENTO_ENCAMINHAMENTOS", "Encaminhamentos do termo de encerramento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_ENCERRAMENTO_DECISAO", "Decisão do termo de encerramento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EXISTEM_PENDENCIAS", "Existem pendências?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_PENDENCIAS", "Descrição das pendências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENCERRAMENTO_CONTRATO", "Termo de Encerramento de Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_REGISTRO_OCORRENCIA", "Registro de Ocorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DOCUMENTO_REGISTRO_OCORRENCIA", "Documento de Registro de Ocorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_PROGRAMA", "Informações do programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ACOES_PROGRAMA", "Ações do programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ADICIONAR_CONTINUAR", "Adicionar e continuar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETIVO_ESTRATEGICO", "Objetivo estratégico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NECESSIDADES", "Necessidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ID_OBJETIVO_ESTRATEGICO", "Identificação do objetivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETIVO_ESTRATEGICO", "Objetivo estratégico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ID", "ID", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_NECESSIDADE", "Justificativa da necessidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_SOLUCAO", "Justificativa da solução escolhida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ALTERNATIVA", "Alternativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_ALTERNATIVA", "Descrição da alternativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CHECKLIST_ANALISE", "Checklist de análise", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_BEM_SERVICO", "Nome do bem ou serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.BEM_OU_SERVICO", "Bem ou serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_BEM_SERVICO_MODAL", "Descrição dos bens e serviços que compõem a solução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_BENS_SERVICO", "Descrição dos bens e serviços que compõem a solução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ACAO_CONTINGENCIA", "Ação de contingência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ACAO_PROGRAMA", "Ação do programa", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("LABEL.BENEFICIO_ESPERADO", "Benefício esperado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_BENEFICIO_ESPERADO", "Descrição do benefício esperado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_NECESSIDADES", "Descrição das necessidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NECESSIDADES_ADEQUACOES", "Necessidades de adequações do ambiente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSOS_NECESSARIOS", "Recursos necessários", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSOS_HUMANOS", "Recursos humanos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSOS_MATERIAIS", "Recursos materiais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ESTRATEGIA_CONTINUIDADE_CONTRATUAL", "Estratégia de continuidade contratual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DISPONIVEL", "Disponível", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.FORMACAO", "Formação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ATIVIDADES", "Atividades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ACAO_OBTENCAO", "Ação para obtenção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ROI", "ROI", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSO_MATERIAL", "Recurso material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ESPECIFICACOES_TECNICAS", "Especificações Técnicas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODELO_EXECUCAO_CONTRATO", "Modelo de Execução do Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SANCOES_APLICAVEIS", "Sanções Aplicáveis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CRITERIOS_SELECAO", "Critérios de Seleção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.BENEFICIO", "Benefício", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DOCUMENTACAO_MINIMA", "Documentação mínima exigida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCEDIMENTOS_INSPECAO", "Procedimentos de teste e inspeção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSOS_HUMANOS_FISCALIZACAO", "Recursos Humanos para Atividades de Fiscalização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SANCOES_APLICAVEIS", "Sanções Aplicáveis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SANCOES", "Sanções", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OCORRENCIA", "Ocorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SANCAO", "Sanção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_SANCAO", "Descrição da sanção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ADEQUACAO_ORCAMENTARIA", "Adequação Orçamentária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCREVER_SANCAO", "Descrever sanção", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("LABEL.EMPREITADA", "Empreitada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ADJUDICACAO", "Adjudicação do objeto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUALIFICACAO_TECNICA", "Qualificação técnica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ADEQUACAO_ORCAMENTARIA", "Adequação Orçamentária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_MODALIDADE", "Justificativa modalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_PREFERENCIA", "Justificativa para aplicação do direito de preferência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_CONTRATACAO_DIRETA", "Justificativa para a contratação direta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CRITERIOS_HABILITACAO", "Critérios técnicos de habilitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUCESSO_SALVAR_TERMO_REFERENCIA", "Termo de referência salvo com sucesso!", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_GERAIS", "Informações gerais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_CONTRATO", "Informações do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_EMPENHO_LICITACAO", "Informações do empenho e licitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_GARANTIA_ITENS_CONTRATO", "Informações de garantia e itens do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFORMACOES_ENVOLVIDOS", "Informações de envolvidos do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EXERCICIO", "Exercício", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ESTUDO_TECNICO", "Estudo técnico", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA", "Justificativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_QUESTIONARIO", "Tipo do questionário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ACAO_PROGRAMA", "Ação do programa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ESPECIFICACOES_TECNICAS", "Especificações Técnicas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODELO_GESTAO_CONTRATO", "Modelo de Gestão do Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SANCOES_APLICAVEIS", "Sanções Aplicáveis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CRITERIOS_SELECAO", "Critérios de Seleção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.BENEFICIO", "Benefício", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ROI", "ROI", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DOCUMENTACAO_MINIMA", "Documentação mínima exigida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCEDIMENTOS_INSPECAO", "Procedimentos de teste e inspeção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSOS_HUMANOS_FISCALIZACAO", "Recursos Humanos para Atividades de Fiscalização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SANCOES", "Sanções", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OCORRENCIA", "Ocorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_SANCAO", "Descrição da sanção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SANCAO", "Sanção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ADEQUACAO_ORCAMENTARIA", "Adequação Orçamentária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EMPREITADA", "Empreitada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ADJUDICACAO", "Adjudicação do objeto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUALIFICACAO_TECNICA", "Qualificação técnica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_MODALIDADE", "Justificativa modalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_PREFERENCIA", "Justificativa para aplicação do direito de preferência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_CONTRATACAO_DIRETA", "Justificativa para a contratação direta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CRITERIOS_HABILITACAO", "Critérios técnicos de habilitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSO_MATERIAL", "Recurso material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSO_HUMANO", "Recurso humano", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITO", "Requisito", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_CONTRATO", "Termo de Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ESTUDO_TECNICO", "Documento de Estudo Técnico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_APLICA", "Não se aplica", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_ENCERRAMENTO_CONTRATO", "Encerramento de contrato salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SELECIONE_CONTRATO_ESPECIFICACAO", "Selecione um contrato antes de inicializar a especificação!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_SALVAR_PROGRAMA", "Programa salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_SALVAR_TERMO_REFERENCIA", "Termo de referência salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SELECIONE_UM_ITEM_PARA_GERAR_REGISTRO_OCORRENCIA", "Selecione um item para gerar o registro de ocorrência.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_SALVAR_REGISTRO_OCORRENCIA", "Sucesso ao salvar o registro de ocorrência.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.DATA_OCORRENCIA_INVALIDA", "A data de emissão informada não pode estar fora do intervalo de vigência do contrato selecionado !", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.DATA_PRAZO_CUMPRIMENTO_INVALIDA", "A data para cumprimento das exigências não pode ser menor que a data de emissão da ocorrência ou maior que a data de vigência do contrato !", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.CONTRATO_FORA_DA_VIGENCIA", "Não é possível gerar registro de ocorrência para contratos que não estão em execução/vigente, por favor escolha um contrato em situação vigente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.VALOR_GLOSA_INVALIDO", "O valor da glosa não pode ser superior ao valor total do contrato nem igual a zero!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_ESTUDO_TECNICO", "Estudo técnico salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_ORDEM_FORNECIMENTO", "Ordem de fornecimento salva com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.CARREGANDO_PARCELAS", "Carregando parcelas...", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.QUESTAO_VAZIA", "Preencha a questão para salvar!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.TERMO_REFERENCIA_JA_GERADO", "O termo de referência selecionado já foi gerado para aprovação e não pode ser editado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.OCORRENCIA_CONTRATO_FORA_DA_VIGENCIA", "Não é possível gerar registro de ocorrência para contratos que não estão em execução/vigente, por favor escolha um contrato em situação vigente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.CADASTRO_CHECKLIST_ANALISE_SOLUCAO", "Pode existir apenas um único cadastro de 'Checklist de Análise de Solução'!", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_EMISSAO_MENOR_VIGENCIA_FINAL", "A data de emissão não pode ser menor que a data de término do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PERIODO_INICIO_TERMINO_DENTRO_VIGENCIA_CONTRATO", "O período de início à término deve estar dentro da vigência do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_CELEBRACAO_MAIOR_DATA_VIGENCIA_INICIAL", "A data de celebração não pode ser maior que a data de vigência inicial do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_VIGENCIA_INICIAL_MAIOR_DATA_VIGENCIA_FINAL_CONTRATO", "A data de vigência inicial não pode ser maior que a data de vigência final!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CONTRATO_EXIGE_MINIMO_TRES_ENVOLVIDOS", "O contrato exige no minino três envolvidos.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_VENCIMENTO_PARCELA_FORA_VIGENCIA_CONTRATO", "O(s) seguinte(s) item(ns) das parcelas não pode(m) estar fora da vigência do contrato: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_EMPENHO_MAIOR_DATA_VIGENCIA_FINAL_CONTRATO", "A data do empenho não pode ser maior que a data de vigência final do contrato.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INICIO_PREVISTO_MAIOR_DATA_TERMINO_PREVISTO_CONTRATO", "A data de inicio previsto não pode ser maior que a data de término previsto.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUANTIDADE_INSERIDA_ITEM_MAIOR_QUANTIDADE_CONTRATO", "A quantidade deste item não pode ser maior que a quantidade definida no contrato. Quantidade restante: ", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.DATA_PENALIZACAO_INVALIDA", "A data da penalização deve estar dentro do intervalo de vigência do contrato selecionado.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.APLICAR_PENALIDADE", "Aplicar penalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.OCORRENCIA_PENALIDADE", "Ocorrência da penalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.RESPOSTA_CONTRATADA", "Resposta da contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_APLICAR_PENALIDADE", "Aplicação de penalidade para a ocorrência da penalidade: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DESCRICAO_EVENTO_APLICAR_PENALIDADE_DATA", "Penalidade gerada em %s", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NOTA_EMPENHO", "Nota de empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.JUSTIFICATIVA_EMPENHO", "Justificativa do empenho", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.SUCESSO_SALVAR_EMPENHO", "Sucesso ao salvar o empenho!", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.DATA_EMISSAO_INVALIDA","Data da emissão não pode estar fora da vigência do contrato",dominio, modulo));

		 internacionalizacaoList.add(new Internacionalizacao("LABEL.RECEBIMENTO_OBJETO", "Recebimento de Objeto", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_RECEBIMENTO_OBJETO", "Data do recebimento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_RECEBIMENTO_OBJETO", "Data do recebimento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_RECEBIMENTO", "Tipo do recebimento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_OBJETO", "Nome do objeto", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NOME_OBJETO", "Nome do objeto", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("LABEL.AVALIACAO_OBJETO", "Avaliação do objeto", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("LABEL.GERAR_TERMO_RECEBIMENTO", "Gerar termo de recebimento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_RECEBIMENTO_OBJETO", "Nº do recebimento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_REGISTRAR_NAO_CONFORMIDADE", "Registrar não conformidade", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_DOCUMENTO_RECEBIMENTO", "Termo de Recebimento de Objeto", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_DOCUMENTO_EMPENHO", "Registro de Nota de Empenho", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR_REGISTRO_EMPENHO", "Gerar registro de nota de empenho", dominio, modulo));

		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR", "Gerar", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.EMITIR_CONSULTA_TERMOS", "Emitir consulta de termos", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NAO_HA_TERMOS", "Não há termos para este contrato", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_INICIAL", "Data inicial", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_FINAL", "Data final", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_TERMO", "Tipo de termo", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TERMOS", "Termos do contrato", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.SELECIONE", "Selecione os tipos do termo que deseja gerar", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TITULO_CONSULTA_TERMOS", "Consulta de Termos", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.PERIODO_NAO_INFORMADO", "Período não informado", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.SEM_TERMOS_A_SER_GERADO", "Não há termos para ser gerado nos filtros selecionados.", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_INICIAL_NAO_INFORMADA", "Data inicial não informada", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DATA_FINAL_NAO_INFORMADA", "Data final não informada", dominio, modulo));

		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.EMITIR_CONSULTA_RELATORIOS", "Emitir consulta relatórios", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TIPO_RELATORIOS", "Tipo de relatórios", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.RELATORIO_REGISTRO_EMPENHO", "Relatório de Registro de Empenho", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.RELATORIO_LIBERACAO_PAGAMENTO", "Relatório de Liberação de Pagamento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.RELATORIO_REPACTUACAO", "Relatório de Repactuação", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.RELATORIO_NAO_CONFORMIDADE", "Relatório de Não Conformidade", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.RELATORIO_PENALIDADE_APLICADA", "Relatório de Penalidades Aplicadas", dominio, modulo));

		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.RELATORIOS", "Relatórios do contrato", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.ADICIONAR_JUSTIFICATIVA", "Adicionar Justificativa", dominio, modulo));

		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.TERMO_REFERENCIA_PROJETO_BASICO", "Termo de referência/Projeto Básico", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CONTRATO_SEM_VINCULO_TERMO", "Contrato selecionado está sem vínculo a um termo de referência.", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.ENCERRAMENTO_CONTRATO", "Encerramento de contrato", dominio, modulo));

		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.EXCLUSAO_ADITIVO_NEGADA", "Exclusão não permitida, o aditivo de prazo já está vigente ou há aditivos de prazo subsequentes a este.", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.DATA_EMISSAO_12_MESES_POSTERIOR", "Data de emissão não permitida, pois infere intervalo de 12 meses em relação ao aditivo subsequente.", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.ADITIVO_DATA_EMISSAO_MAIOR_DATA_INICIO", "Data de emissão deve ser menor que a data de início do aditivo.", dominio, modulo));

		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CONSULTA_RELATORIOS", "Consulta de Relatórios", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.CONSULTA_TERMOS", "Consulta de termos", dominio, modulo));
		 
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_ADITIVO", "Alteração Contratual (Aditivo)", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_PENALIDADE", "Aplicação de Penalidade", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_APOSTILAMENTO", "Apostilamento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_ATUALIZA_STATUS", "Atualização de Status", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_LIBERA_PAGAMENTO", "Liberação para Pagamento", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_EMPENHO", "Registro de Empenho", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_NAO_CONFORMIDADE", "Registro de Não Conformidade", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_RECEBIMENTO_OBJETO", "Registro de Recebimento de Objeto", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_REPACTUACAO", "Repactuação", dominio, modulo));
		 internacionalizacaoList.add(new Internacionalizacao("CONTRATO_LABEL_EVENTO_RESCISAO", "Rescisão de Contrato", dominio, modulo));
	}

	private void gerarMensagem(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("MSG.CONFIRMA_EXCLUIR_QUESTIONARIO", "Deseja excluir questionário?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.CONFIRMA_EXCLUIR_TERMO_REFERENCIA", "Confirma exclusão do Termo de Referência?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.CONFIRMACAO_EXCLUSAO_LIBERACAO_PAGAMENTO", "Tem certeza que deseja excluir a liberação de pagamento? Essa operação é definitiva e não poderá ser desfeita.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.CONTRATO_RESCINDIDO", "O Contrato já está rescindido!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.QUANTIDADE_DIAS_DISPARO_NAO_DEFINIDA", "A quantidade de dias disparo deve ser maior que 0!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.QUANTIDADE_TENTATIVAS_NAO_DEFINIDA", "A quantidade de tentativas deve ser maior que 0!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.REGISTRO_EMPENHO_SALVO", "Registro de empenho salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.RESCISAO_CONTRATO_SALVA", "Contrato rescindido com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SEM_INFRACAO_PARA_PENALIZAR", "Este contrato não possui infrações para ser possível realizar penalizações!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_ATUALIZACAO_INFRACAO", "Infração salva com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_ATUALIZACAO_LIBERACAO_PAGAMENTO", "Liberação de pagamento salva com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_ATUALIZACAO_STATUS", "Atualização de status realizado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_BLOQUEIO_PENALIDADE", "A penalidade foi bloqueada com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_BLOQUEIO_QUESTIONARIO", "Questionário bloqueado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_BLOQUEIO_REGRA_EVENTO", "As regras dinâmicas foram bloqueadas com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_CONTRATO", "Contrato salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_DESBLOQUEIO_PENALIDADE", "A penalidade foi desbloqueada com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_DESBLOQUEIO_QUESTIONARIO", "Questionário desbloqueado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_DESBLOQUEIO_REGRA_EVENTO", "As regras dinâmicas foram desbloqueadas com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_EXCLUSAO_OBJETIVO_ESTRATEGICO", "Objetivo estratégico excluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_EXCLUSAO_QUESTIONARIO", "Questionário excluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_EXCLUSAO_REGRA_EVENTO", "As regras dinâmicas foram excluídas com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_EXCLUSAO_TERMO_REFERENCIA", "Termo de Referência excluído com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_LIBERACAO_PAGAMENTO_EXCLUIDA", "Liberação de pagamento excluída com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_SALVAR_OBJETIVO_ESTRATEGICO", "Objetivo estratégico salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_SALVAR_QUESTIONARIO", "Questionário salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_SALVAR_REGRA_EVENTO", "As regras dinâmicas foram salvas com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.CONFIRMA_EXCLUIR_REGRA_EVENTO", "Tem certeza que deseja excluir essas regras dinâmicas?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.EXCLUIR_ESTUDO_TECNICO", "Confirma a exclusão do Estudo técnico selecionado?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SELECIONE_AO_MENOS_UM_ITEM_PARA_SER_REMOVIDO", "Selecione ao menos um item para ser removido!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SUCESSO_SALVAR", "Salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.SELECIONE_UM_ITEM_PARA_GERAR_TERMO", "Selecione um item para gerar o termo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.DATA_RECEBIMENTO_INVALIDA", "A data de recebimento do objeto deve estar dentro do período de vigência do contrato selecionado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("MSG.DESCRICAO_EVENTO_RECEBIMENTO_OBJETO", "Recebimento de objeto nº: %s realizado no dia %s", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.MSG.TODOS_PAPEIS_ENVOLVIDOS_ADICIONADOS", "Todos os papéis envolvidos já adicionados.", dominio, modulo));


	}

	private void gerarValidacao(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_LIBERACAO_PAGAMENTO_MENOR_DATA_EMPENHO", "A data da liberação de pagamento não deve ser inferior a data do empenho!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.LIBERACAO_PAGAMENTO_PARA_VALOR_CONTRATO", "Contrato já teve liberação do seu valor total!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALOR_LIBERACAO_MAIOR_VALOR_EMPENHO", "Valor da liberação de pagamento maior que o valor do empenho!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALOR_LIBERACAO_NAO_PODE_ZERO", "O valor da liberação de pagamento não pode ser zero!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MAXIMO_POSSIVEL_RESPOSTA", "Você pode adicionar no máximo 10 possíveis respostas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MINIMO_POSSIVEIS_RESPOSTAS", "Você deve adicionar pelo menos duas possíveis respostas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NECESSIDADE_NAO_PREENCHIDA", "Você deve informar a descrição da necessidade elencada para adicionar a mesma!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.QUESTAO_NAO_PREENCHIDA", "Você deve preencher todos os dados da questão para adicionar a mesma!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.VALIDACAO.SEM_POSSIVEL_RESPOSTA", "Você deve digitar uma possível resposta!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.SELECIONE_OS", "Adicione uma ordem de fornecimento de bens e serviços.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LIBERACAO_PAGAMENTO.MSG.INFORME_VALOR_EMPENHO", "Informe o valor liberado para pagamento do empenho.", dominio, modulo));
	}

	private void gerarLabel(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("LABEL.ADICIONAR_PERGUNTA", "Adicionar pergunta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.GESTOR", "Gestor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.IMPRIMIR", "Imprimir", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRAR_CONTRATOS_APOSTILAMENTO", "Mostrar apenas contratos com apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERGUNTAS", "Perguntas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUESTAO", "Questão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_CONTRATOS", "Relatório de contratos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_RESPOSTA", "Tipo resposta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TODAS", "Todas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VERDADEIRO", "Verdadeiro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ACOES_DECORRENTES", "Ações Decorrentes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.AGENDAMENTO", "Agendamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.APOSTILAMENTO", "Apostilamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.BEM_SERVICO_SOLUCAO", "Bens e serviços que compõem a solução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.BENEFICIOS_NECESSIDADES", "Benefícios e necessidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CRITERIO_ACEITACAO", "Critérios de Aceitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_INFRACAO", "Data da infração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_VIGENCIA_FINAL", "Data da vigência final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_VIGENCIA_INICIAL", "Data vigência inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DEFINICAO_REQUISITOS", "Definições de requisitos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DEVERES_RESPONSABILIDADES_CONTRATADA", "Deveres e Responsabilidades da Contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DEVERES_RESPONSABILIDADES_CONTRATANTE", "Deveres e Responsabilidades da Contratante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DEVERES_RESPONSABILIDADES_ORGAO", "Deveres e Responsabilidades do Órgão Gerenciador do Registro de Preço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EMPENHOS", "Empenhos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EQUIPE_PLANEJAMENTO_CONTRATACAO", "Equipe de planejamento da contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ESTIMATIVA_PRECO", "Estimativa de Preço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ESTRATEGIA_CONTRATUAL", "Estratégia contratual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EXIBIR_DETALHES", "Exibir detalhes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EXPRESSAO_CRON", "Expressão cron", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.FLUXO_TRABALHO", "Fluxo de trabalho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.FORMA_PAGAMENTO", "Forma de pagamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INFRACOES", "Infrações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERCORRENCIAS", "Intercorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LEVANTAMENTO_ALTERNATIVAS", "Levantamento das alternativas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIBERACOES_PAGAMENTO", "Liberações Pagamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIBERAR_PAGAMENTO_CONTRATO", "Liberar pagamento do valor do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MECANISMOS_FORMAIS_COMUNICACAO", "Mecanismos Formais de Comunicação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODELO_EMAIL", "Modelo de email", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOTIVO", "Motivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NECESSIDADE_NEGOCIO", "Necessidade de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NENHUM_CONTRATO_ENCONTRADO", "Nenhum contrato encontrado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOTIFICACOES", "Notificações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUM_ESTUDO_TECNICO", "Nº do estudo técnico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_CONTRATO", "Número de contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PARCELAS", "Número de parcelas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETIVO_EMPENHO", "Objetivo empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OCORRENCIAS", "Ocorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIDADE", "Penalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIZACAO", "Penalização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLANEJAMENTO_CONTRATACAO", "Planejamento da contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLANO_ACAO", "Plano de Ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRAZO_ENTREGA", "Prazo de entrega", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREDECESSORES", "Predecessores", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCEDIMENTOS_GLOSA", "Procedimentos de Glosa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DIAS_DISPARO", "Quantidade de dias disparo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DIAS_INTERCORRENCIA", "N° dias até intercorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DIAS_PLANO_ACAO", "N° dias até plano de ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_INTERCORRENCIA", "Quantidade de intercorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_OCORRENCIAS", "Quantidade de Ocorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_TENTATIVAS", "Quantidade de tentativas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSOS_NECESSARIOS", "Recursos necessários", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRAR_INFRACAO", "Registrar Infração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRO_INFRACAO", "Registo de infração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITO_NEGOCIO", "Requisito de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITOS_CAPACITACAO", "Requisitos de capacitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITOS_NEGOCIO", "Requisitos de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITOS_TECNICOS", "Requisitos técnicos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SEM_DATA_TERMINO", "Sem data de término", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ANALISE", "Tipo análise", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_AQUISICAO", "Tipo aquisicao", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ATRIBUICAO", "Tipo atribuição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_CONDICAO", "Tipo condição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DESTINO", "Tipo destino", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DISPARO", "Tipo disparo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_IMPLEMENTACAO", "Tipo de implementação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_INTERCORRENCIA", "Tipo de intercorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_REITERACAO", "Tipo reiteração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_REPETICAO", "Tipo repetição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_CONTRATO", "Valor total do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.POSSIVEL_RESPOSTA", "Possível resposta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VIGENCIA", "Vigência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ANEXOS", "Anexos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ANEXOS_OBSERVACOES", "Anexos e observações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ATUALIZAR_STATUS", "Atualizar Status", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.BENEFICIOS_ESPERADOS", "Benefícios Esperados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CONTRATO", "Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.DESCRICAO_SOLUCAO", "Descrição da solução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DETALHAMENTO_SOLUCAO", "Detalhamento da Solução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EMPENHO", "Empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EQUIPE_CONTRATACAO", "Equipe de planejamento da contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EXECUTAR_QUESTIONARIO", "Executar questionário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.GERAR_ESTUDO_TECNICO", "Gerar estudo técnico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_CONTRATACAO", "Justificativa da Contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIBERAR_PAGAMENTO", "Liberar Pagamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NECESSIDADE_ELENCADA", "Necessidade elencada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.LABEL.NUMERO_EMPENHO", "N° do empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETO_CONTRATACAO", "Objeto contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIDADES", "Penalidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIDADES_PREVISTAS", "Penalidades previstas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRAR_EMPENHO", "Registrar empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRO_PENALIDADE", "Penalizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESCINDIR_CONTRATO", "Rescindir Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_ATUAL", "Situacao atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SOLUCAO", "Solução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_CONTRATO", "Tipo contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_PERSPECTIVA", "Tipo perspectiva", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_EMPENHADO", "Valor empenhado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL", "Valor total", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.FINAL", "Final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.INFORME_GESTOR", "Informe o gestor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.INICIAL", "Inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CONTRATOS.NOME_QUESTIONARIO", "Nome do checklist", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.COMECA_EM", "Começa em", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.COMPENSACAO_SANCAO_FINANCEIRA", "Compensação sanção financeira", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CONTEUDO", "Conteúdo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_CELEBRACAO", "Data celebração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_EXECUCAO", "Data da execução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DATA_INSTITUICAO_EQUIPE", "Data da instituição da equipe", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.DESCRICAO_GLOSA", "Descrição da Glosa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.EXIGE_ANEXO", "Exige Anexo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.FUNDAMENTO_LEGAL", "Fundamento legal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INCISO", "Inciso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INDICADOR", "Indicador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_ENTREGA", "Local Entrega/Execução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.METRICA", "Métrica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODALIDADE", "Modalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEL_SERVICO", "Nível de Serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVO_STATUS_CONTRATO", "Novo status do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_PORTARIA", "Nº da portaria", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_LICITACAO", "Número licitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_TERMO", "Nº do termo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETO_CONTRATUAL", "Objeto contratual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGAO_EXTERNO", "Órgão externo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAPEL", "Papel", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREDECESSOR", "Predecessor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PUBLICACAO", "Publicação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGRA_EVENTO", "Regra Evento ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_INFRACAO_PENALIZAR", "Selecione uma infração para penalizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMINA_EM", "Termina em", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_COLETA", "Tipo perspectiva", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_CONTRATUAL", "Valor contratual", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTEGRANTE_ADMINISTRATIVO", "Integrante administrativo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTEGRANTE_REQUISITANTE", "Integrante requisitante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTEGRANTE_TECNICO", "Integrante técnico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERCORRENCIA", "Intercorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERCORRENCIAS", "Intercorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERESSADO", "Interessado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERESSADOS", "Interessados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERNACIONALIZACAO", "Internacionalização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERNACIONALIZACAOS", "Internacionalizações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_AQUISICAO_FINAL", "Data aquisição final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_AQUISICAO_INICIAL", "Data aquisição inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_BAIXA_FINAL", "Data baixa final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_BAIXA_INICIAL", "Data baixa inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_CONTABILIZACAO_FINAL", "Data contabilização final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_CONTABILIZACAO_INICIAL", "Data contabilização inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_INCLUSAO_FINAL", "Data de inclusão final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_DATA_INCLUSAO_INICIAL", "Data de inclusão inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INTERVALO_NUMERO_PATRIMONIAL", "Intervalo de número patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVALIDO", " é inválido!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVENTARIAR", "Inventariar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVENTARIAR_BENS", "Inventariar bens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVENTARIAR_SELECIONADOS", "Inventariar selecionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVENTARIO", "Inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVENTARIO_DO_BEM", "Inventário do bem patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVENTARIO_DO_NOVO_BEM", "Inventário do novo bem patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.INVENTARIOS", "Inventários", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.IR_FIM", "Ir para o fim", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.IR_TOPO", "Ir para o topo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ITEM", "Item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ITEM_PLANO_CLASSIFICACAO", "Itens do plano de classificação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ITENS", "Itens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ITENS_DOMINIO", "Itens do domínio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ITENS_POR_PAGINA", "Por página: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JSON", "Json", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JURIDICA", "Jurídica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.JUSTIFICATIVA_CONTRATACAO", "Justificativa da Contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LABEL", "Label", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LATITUDE", "Latitude", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LEVANTAMENTO_ALTERNATIVAS", "Levantamento das alternativas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIBERACOES_PAGAMENTO", "Liberações Pagamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIBERAR_PAGAMENTO", "Liberar Pagamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIBERAR_PAGAMENTO_CONTRATO", "Liberar pagamento do valor do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIMITE_APROVACAO", "Limite de aprovação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIMITE_ESTOQUE", "Limite de estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIMITES_VALOR", "Limites de valor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIMPAR", "Limpar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LIMPAR_FILTROS", "Limpar filtros", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LINGUAGEM_DROOLS", "Mapeamento drools", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LINGUAGEM_NATURAL", "Expressão em linguagem natural", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LINHA", "Linha", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LINK", "Link", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LISTA_FAVORITOS", "Lista de páginas favoritas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LISTA_FILTROS", "Lista de filtros por página", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LISTA_HELP", "Comportamento do help por página", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_ENTREGA", "Local Entrega/Execução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_ESTOQUE", "Local estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_ESTOQUE_SALDO", "Saldo do local estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_ESTOQUE_SALDOS", "Saldos do local estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_ESTOQUES", "Local estoques", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCAL_IDENTIFICACAO_INCONSISTENCIA", "Local identificação inconsistência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCALIZACAO", "Localização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCALIZACAO_ATUAL", "Localização atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOCALIZACOES", "Localizações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOGO_PAGINA_LOGIN", "Logo página login", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOGO_RELATORIO", "Logo cabeçalho relatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOGRADOURO", "Logradouro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LONGITUDE", "Longitude", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.LOTES", "Lotes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MANTER_BEM_ESTRUTURA", "Manter nesta estrutura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MANTER_CAMPOS_BEM_NO_INVENTARIO", "Manter os campos do bem patrimonial abaixo ao realizar o inventário?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MANUTENCAO_FLUXOS", "Manutenção de Fluxos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MAPA_EXCLUIDO_SALVO_SUCESSO", "Exclusão do Mapa Organizacional realizada com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MAPA_ORGANIZACIONAL", "Mapa organizacional", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MAPA_ORGANIZACIONAL_SALVO_SUCESSO", "Mapa Organizacional cadastrado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MAPAS_ORGANIZACIONAIS", "Mapas organizacionais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MAPEAMENTO", "Mapeamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MAPEAMENTOS", "Mapeamentos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MARCA", "Marca", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MARCAR_TODAS", "Marcar todas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MARGEM_PRECO_MEDIO", "Porcentagem de margem de erro para o preço médio de entrada de materiais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MASCARA", "Máscara", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MASCARA_ANO_SEQUENCIAL", "Ano atual e número sequencial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MASCARA_NUMERO_IDENTIFICACAO", "Máscara do número de identificação da operação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MASCARA_NUMERO_PATRIMONIO", "Máscara do número patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MASCARA_SEQUENCIAL", "Número sequencial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAIS", "Materiais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAIS_CONSUMO", "Materiais de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAIS_CONSUMO_REQUISICAO", "Materiais de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAIS_CONSUMO_USADOS_POR_MATERIAL_PERMANENTE", "Materiais de consumo utilizados por esse material permanente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAL", "Material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAL_CONSUMO", "Material de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAL_CONSUMO_TIPO_UNIDADE_MEDIDA_ENTRADA", "Unidade de medida da entrada de material de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATERIAL_CONSUMO_TIPO_UNIDADE_MEDIDA_ENTRADAS", "Unidades de medidas das entradas de materiais de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATRICULA", "Matrícula", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MATRICULA_SIAPE", "Matrícula SIAPE", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MECANISMOS_FORMAIS_COMUNICACAO", "Mecanismos Formais de Comunicação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MENSAGEM", "Mensagem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MENU", "Menu", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MENU_ASSOCIADO", "Menu associado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MENU_DESATIVADO", "Menu desativado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MES_REFERENCIA", "Mês de referência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MESES", "Meses", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.METRICA", "Métrica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODALIDADE", "Modalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODALIDADE_GARANTIA", "Modalidade garantia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODELO", "Modelo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODELO_EMAIL", "Modelo de email", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODULO", "Módulo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODULOS", "Módulos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MODULOS_SELECIONADOS", "Módulos selecionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MONITOR", "Monitor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MONITORES", "Monitores", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRANDO", "Mostrando", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRAR", "Mostrar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRAR_ARVORE", "Mostrar árvore", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRAR_LISTA", "Mostrar lista", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRAR_PROXIMO_NUMERO", "Mostrar próximo número disponível", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARAJUDA", "Mostrar ajuda", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARBLOQUEAR", "Mostrar bloquear", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARCADASTRAR", "Mostrar cadastrar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRAREDITAR", "Mostrar editar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARFAVORITO", "Mostrar favorito", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARFILTROS", "Mostrar filtros", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARLIMPAR", "Mostrar limpar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARREMOVER", "Mostrar remover", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARSALVAR", "Mostrar salvar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOSTRARVISUALIZAR", "Mostrar visualizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOTIVO", "Motivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOVER_PARA_MENU", "Mover para o menu:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOVIMENTACOES", "Movimentações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOVIMENTO_CONTABIL", "Movimento contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOVIMENTO_CONTACONTABIL", "Movimento conta contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOVIMENTO_ESTOQUE", "Movimento do estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOVIMENTO_ESTOQUES", "Movimento do estoques", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MOVIMENTOS_CONTABEIS", "Movimentos contábeis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.MUDAR_PARENT_MENU", "Alterar menu pai", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.N_DOCUMENTO", "Nº do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.N_PATRIMONIAL", "Nº patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO", "Não", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_ACHOU_PESQUISE", "Não achou o que procura? Pesquise!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_ATENDE_REGEX", "Não atende a expressão regular", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_DEFINIDO", "Não definido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_DEPRECIAR", "Não depreciar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_ENCONTRADO", "Não encontrado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_ENCONTRADO_NESTA_ESTRUTURA", "Não encontrado nesta estrutura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NAO_POSSUI_DETENTOR", "Não possui detentor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NATUREZA", "Natureza", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NECESSIDADE_ELENCADA", "Necessidade elencada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NECESSIDADE_NEGOCIO", "Necessidade de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NECESSIDADES_ESTRATEGICAS", "Necessidades estratégicas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NENHUM", "Nenhum", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NENHUM_CONTRATO_ENCONTRADO", "Nenhum contrato encontrado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NENHUM_ITEM_DEPRECIADO", "Nenhum bem patrimonial foi depreciado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEIS_AUTORIDADE", "Níveis de autoridade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEL_ACESSO", "Nível Acesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEL_ACESSO_PERMITIDO", "Níveis de Acesso Permitidos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEL_AUTORIDADE", "Nível de autoridade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEL_AUTORIDADE_SELECIONADAS", "Níveis de autoridade selecionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEL_SERVICO", "Nível de Serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVEL_SIGILO", "Nível de sigilo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NIVELACESSO", "Nivel Acesso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME", "Nome", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_DOMINIO", "Nome domínio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_EXIBIDO_MENU", "Nome exibido no menu", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_FANTASIA", "Nome fantasia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_FILTRO", "Nome filtro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_MAE", "Nome da mãe", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_OBRIGATORIO", "O nome do usuário é obrigatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_PACKAGE", "Nome do package", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_PAGINA", "Nome página", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_PAI", "Nome do pai", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOME_UNIDADE_REQUISITANTE", "Nome unidade requisitante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOMEENTIDADE", "Nome da entidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOTA_ENTRADA", "Nota Entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOTA_RECEBIMENTO", "Nota recebimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOTIFICACAO", "Notificação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOTIFICACAO_VISUALIZADA", "Notificação visualizada?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOTIFICACOES", "Notificações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVA_SENHA", "Nova senha", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVA_SIT_FISICA", "Nova situação física", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVA_TAXA_RESIDUAL", "Nova taxa residual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVA_UNIDADE", "Cadastrar nova unidade.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVA_VERSAO", "Nova Versão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVO_NUMERO_PATRIMONIAL", "Novo número patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVO_STATUS_BEM", "Novo status do bem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVO_STATUS_CONTRATO", "Novo status do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVO_VALOR_MERCADO", "Novo valor patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVO_VALOR_RECUPERAVEL", "Novo valor patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVO_VALOR_RESIDUAL", "Novo valor residual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NOVOPROCESSO", " Novo Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NR_DOCUMENTO_FINAL", "Nº documento final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NR_DOCUMENTO_INICIAL", "Nº documento inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NR_PATRIMONIAL_FINAL", "Nº patrimonial final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NR_PATRIMONIAL_INICIAL", "Nº patrimonial inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NR_TERMO_FINAL", "Nº termo final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NR_TERMO_INICIAL", "Nº termo inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_ESTUDO_TECNICO", "Nº do estudo técnico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_PATRIMONIAL", "Núm. patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUM_PORTARIA", "Nº da portaria", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO", "Número", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_ADICAO", "Número da adição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_ATENDIMENTO", "Número do atendimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_BAIXA", "Número da baixa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_CONTRATO", "Número de contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_DEFINICAO_DETENTOR", "Número da definição do detentor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_EMPENHO", "N° do empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_ENTRADA", "Nº entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_LICITACAO", "Número licitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_LOTE", "Número do lote", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PARCELAS", "Número de parcelas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PASSAPORTE", "Número passaporte", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PATRIMONIAL", "Número patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PATRIMONIAL_ABREVIADO", "Nº patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PATRIMONIAL_ATUAL", "Número patrimonial atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PATRIMONIAL_BEM", "Número patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PATRIMONIAL_BEM_PRINCIPAL", "Número patrimonial do bem principal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PATRIMONIAL_GERADO_INVENTARIO", "Número patrimonial confirmado no inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PATRIMONIAL_INICIAL", "Nº patrimonial inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_PROCESSO", "N° processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_REQUISICAO", "Número da requisição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_REQUISICAO_FIM", "Nº requisição fim", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_REQUISICAO_INICIO", "Nº requisição início", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_SAIDA", "Número da saída", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_SELECAO", "Número da seleção", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_TERMO", "Nº do termo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_TRANSFERENCIA", "Número da transferência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUMERO_ULTIMA_REQUISICAO", "Nº última requisição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.NUP", "Nup", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.O_CAMPO_OBRIGATORIO", "O campo é obrigatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.CAMPO_OBRIGATORIO", "Campo obrigatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETIVO", "Objetivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETIVO_EMPENHO", "Objetivo empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETO_CONTRATACAO", "Objeto contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBJETO_CONTRATUAL", "Objeto contratual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBRIGATORIO", "Obrigatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBSERVACAO", "Observação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBSERVACAOGERAL", "Observacao Geral", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OBSERVACOES", "Observações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OCORRENCIAS", "Ocorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OK", "Ok", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OPACIDADE_COR_MENU", "Opacidade da cor do menu", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORDEM", "Ordem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGANIZACAO", "Órgão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGANIZACAO_EXTERNO", "Órgão externo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGANIZACAO_PRINCIPAL", "Orgão principal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGANIZACOES", "Órgãos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGAO", "Órgão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGAO_EXTERNO", "Órgão externo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGAO_PRINCIPAL", "Orgão principal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORGAOS", "Órgãos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ORIGEM", "Origem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.OUTROS", "Outros", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PADRAO", "Padrão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAGAMENTO_VALOR_CONTRATO", "Pagamento do valor do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAGINA", "Página", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAGINA_LOGIN", "Página Login", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAGINAS_USUARIO", "Páginas usuário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAINEIS", "Painéis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAINEL", "Painel", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAIS", "País", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAIS_ATUALIZADO_SUCESSO", "País atualizado com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAIS_SALVO_SUCESSO", "País salvo com sucesso!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAISES", "Países", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PAPEL", "Papel", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PARA", "para", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PARAMETROS", "Parâmetros", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PARAMETROS_SISTEMA", "Parâmetros sistema", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PARCEIRO", "Parceiro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PATRIMONIO", "Patrimônio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PATRIMONIO_MASCARA_NUMERO_PATRIMONIO", "Máscara do número patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PEDIDO_MEDIO", "Pedido Médio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIDADE", "Penalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIDADES", "Penalidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIDADES_PREVISTAS", "Penalidades previstas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PENALIZACAO", "Penalização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERCENTUAL", "Percentual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERCENTUAL_NAO_DEPRECIACAO", "Não depreciável (0-100%)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERFIL", "Perfil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERIODICIDADE", "Periodicidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERIODO", "Período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERMISSOES", "Permissões", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERMISSOES_SELECIONADAS", "Permissões selecionadas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERMITE_APROVACAO_NIVEL_INFERIOR", "Aprovação em nível inferior quando solicitante igual ao autorizador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERMITE_APROVACAO_PROPRIA", "Permite aprovação própria", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERMITE_REQUISICAO", "Permitir requisição de produto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PERMITIR_USO_CLASSIFICACAO", "Permitir utilizar como classificação?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PESQUISA_ITEM", "Pesquisa item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PESQUISAR", "Pesquisar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PESSOA", "Pessoa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PESSOAS", "Pessoas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLACA", "Placa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLACEHOLDER_GENERICO", "Informe ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLANEJAMENTO_CONTRATACAO", "Planejamento da contratação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLANO_ACAO", "Plano de Ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLANO_CLASSIFICACAO_SUPERIOR", "Elemento superior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PLANOCLASSIFICACAO", "Plano classificação, temporalidade e destinação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PONTO_PEDIDO", "Ponto de pedido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PORTA", "Porta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PORTADOR", "Portador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PORTAL", "Portal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PORTE", "Porte", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRAZO_ENTREGA", "Prazo de entrega", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRE_VISUALIZACAO", "Pré visualização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRECO_MEDIO", "Preço Médio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREDECESSOR", "Predecessor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREDECESSORES", "Predecessores", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREENCHA_DADOS_OBRIGATORIOS", "Preencha os dados obrigatórios!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREENCHA_UM_ITEM_DEVOLUCAO", "Ao menos uma quantidade a ser devolvida deverá ser preenchida!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREFERENCIAS", "Preferências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PREVISAO_RETORNO", "Data prevista para o retorno", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRIMEIRO", "Primeiro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRIMEIRO_TITULO", "Primeiro título", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRIVILEGIOS", "Privilégios", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCEDIMENTOS_GLOSA", "Procedimentos de Glosa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCESSO", "Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCESSO_NEGOCIO", "Processo de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCESSORELACIONADOS", " Processos Relacionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCESSOS", "Processos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCESSOS_APENSADOS", " Processos Apensados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROCESSOS_NEGOCIO", "Processos de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROGRESSO", "Progresso", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROIBIDO_EDITAR_APOS_USO_COMISSAO", "Proibido editar após o uso da comissão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROIBIDO_EDITAR_APOS_USO_CONTA", "Proibido editar após uso da conta contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROJETO", "Portal brazuka", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PRONTO", "Pronto!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROTOCOLO", "Protocolo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PROXIMO", "Próximo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.PUBLICACAO", "Publicação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QNT_ITENS_DEPRECIADOS", "Qnt. itens depreciados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QNT_ITENS_NAO_DEPRECIADOS_VALOR", "Bens não depreciados por estarem no valor mínimo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QNT_ITENS_NAO_DEPRECIADOS_VIDA_UTIL", "Bens não depreciados por atingirem o tempo de vida útil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD", "Quantidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_EM_ESTOQUE", "Qtd. em estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_ENTRADA", "Qtde. Entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_ESTOQUE", "Qtde. Estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_FORNECIDA_REQUISICAO", "Qtd. fornecida última requisição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_POR_ALMOXARIFADO", "Qtde por Almoxarifado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_POR_ORGAO", "Qtde por Órgão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_RECEBIDA", "Qtd. Recebida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTD_REQUERIDA_REQUISICAO", "Qtd. requerida última requisição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QTDE_LINHAS", "Quantidade de linhas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE", "Quantidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_ATENDIDA", "Quantidade atendida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DEVOLVIDA", "Quantidade devolvida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DIAS_DISPARO", "Quantidade de dias disparo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DIAS_INTERCORRENCIA", "N° dias até intercorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DIAS_PLANO_ACAO", "N° dias até plano de ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_DOMINIO_TIPO_UNIDADE_ENTRADA", "Tipo de unidade de entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_ESTOCADA", "Quantidade que irá ser armazenada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_ESTOQUE", "Quantidade Estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_INTERCORRENCIA", "Quantidade de intercorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_ITENS_UNIDADE_ARMAZENAMENTO", "Quantidade de itens da unidade de entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_OCORRENCIAS", "Quantidade de Ocorrências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_PADRAO", "Qtde. Padrão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_PEDIDA", "Quantidade requisitada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_PENDENTE", "Quantidade Pendente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_REQUERIDA", "Quantidade requerida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_TENTATIVAS", "Quantidade de tentativas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUANTIDADE_TOTAL", "Quantidade total", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.QUESTIONARIO", "Questionário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RAMOS_ATIVIDADES", "Ramos de atividades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RAZAO_SOCIAL", "Razão social", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REABRIR", "Reabrir", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REAVALIACAO", "Reavaliação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECEBE_ATUALIZACAO", "Recebe Atualização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECEBEDOR", "Recebedor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RECURSOS_NECESSARIOS", "Recursos necessários", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REDEF_SENHA_MOBILE", "Redefinir senha mobile", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REDEFINIR_SENHA", "Redefinir senha", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REDUCAO", "Redução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REFERENCIA", "Referência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REFERENCIA_FINAL", "Referência final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REFERENCIA_INICIAL", "Referência inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REFERENCIA_SUBSEQUENTE", "Referência subsequente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REFERENCIA_VIGENTE", "Referência vigente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGIAO", "Região", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGIOES", "Regiões", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRAR_EMPENHO", "Registrar empenho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRAR_INFRACAO", "Registrar Infração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRO_DEVOLUCAO", "Registros de Devoluções", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRO_INFRACAO", "Registo de infração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRO_NO_SISTEMA", "Registro no sistema", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGISTRO_PENALIDADE", "Penalizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGRA", "Regra", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGRA_EVENTO", "Regra Evento ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGRA_NEGOCIO", "Regra de Negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGRAS", "Regras", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REGRAS_EVENTO", "Regras Evento ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELACIONARPROCESSO", " Relacionar Processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO", "Relatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_ANALITICO", "Relatório analítico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_ANUAL", "Relatório anual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_BENS_ESTRUTURA", "Relatório inventário por estrutura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_BENS_ESTRUTURA_ALT", "Imprimir relatório de bens inventariáveis de forma manual pela estrutura selecionada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_BENS_PATRIMONIAIS_ESTRUTURA", "Relatório bens da estrutura", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_CONSUMO_POR_PERIODO", "Relatório de consumo por período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_ENTRADA_ALMOXARIFADO_POR_PERIODO", "Relatório de entrada por período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_ENTRADA_POR_PERIODO", "Emitir relatório de entrada por período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_INVENTARIO", "Relatório de inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_MATERIAL_ALMOXARIFADO", "Relatório de Material - Serviço Almoxarifado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_MOVIMENTACAO_DE_ALMOXARIFADO", "Relatório de movimentação do almoxarifado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_MOVIMENTACAO_DE_BENS", "RMB – Relatório de Movimentação de Bens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_POSICAO_ESTOQUE_POR_CONTA", "Posição do Estoque Analítico - Por Conta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_DE_SAIDA_ALMOXARIFADO_POR_PERIODO", "Relatório de saida por período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_ENTRADA_MATERIAL_ANALITICO", "Relatório analítico de entrada de materiais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_FICHA_INDIVIDUAL", "Ficha individual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_INVENTARIO", "Relatório inventário vazio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_INVENTARIO_ALT", "Imprimir relatório vazio de bens inventariáveis de forma manual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_PRIMEIRO_TITULO", "Primeiro título", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_SAIDA_MATERIAL_SINTETICO", "Saída de Material - Por Conta (Sintético)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_SEGUNDO_TITULO", "Segundo título", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_SINTETICO", "Relatório sintético", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIO_TERCEIRO_TITULO", "Terceiro título", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIOS", "Relatórios", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIOS_DE_BAIXAS_POR_PERIODO", "Relatório de baixas por período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIOS_DE_ENTRADA", "Relatório de entrada por período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RELATORIOS_INVENTARIO", "Relatórios de inventário de bens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER", "Remover", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_COLABORADOR", "Remover colaborador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_DETENTORES_TRANSFERENCIA", "detentor(es) no(s) bem(ns) selecionado(s). Deseja remover o(s) detentor(es) após a transferência?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_FORNECEDOR", "Remover fornecedor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_ITEM", "Remover item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_MENU", "Remover menu", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_ORGANIZACAO_EXTERNO", "Remover órgão externo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_ORGAO_EXTERNO", "Remover órgão externo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_PORTADOR", "Remover portador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_SEGURADORA", "Remover seguradora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_SELECIONADO", "Remover selecionado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_SERVIDOR", "Remover servidor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVER_TERCEIRIZADO", "Remover terceirizado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REMOVERRELACIONAMENTO", "Remover Relacionamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RENAVAM", "Renavam", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REPRESENTANTE_LEGAL", "Representante legal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUERIDA", "Requerida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO", "Requisição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO_CONSUMO", "Requisição de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO_CONSUMO_ITEM", "Item da requisição de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO_CONSUMO_ITEMS", "Itens da requisição de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO_CONSUMO_OBSERVACAO", "Observação da requisição de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO_CONSUMO_OBSERVACOES", "Observações da requisição de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO_CONSUMOS", "Requisições de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICAO_MATERIAL_CONSUMO", "Requisição de material de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICOES", "Requisições", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISICOES_PENDENTE", "Requisições Pendentes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITO_NEGOCIO", "Requisito de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITOS_CAPACITACAO", "Requisitos de capacitação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITOS_NEGOCIO", "Requisitos de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REQUISITOS_TECNICOS", "Requisitos técnicos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESCINDIR_CONTRATO", "Rescindir Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESIDUO", "Resíduo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESOLVER_INCONSISTENCIA", "Resolver inconsistências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESP_RECEBIMENTO", "Resp. recebimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEIS", "Responsáveis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL", "Responsável", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL_ATUAL", "Responsável atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL_GERADO_INVENTARIO", "Responsável confirmado no inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL_INVENTARIADO", "Responsável inventariado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL_PELO_INVENTARIO", "Responsável pelo inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL_RECEBIMENTO", "Responsável pelo recebimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL_SAIDA", "Responsável pela saída", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPONSAVEL_TRANSFERENCIA", "Responsável transferência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESPOSTA_CONTRATADA", "Resposta da contratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESSUPRIMENTO", "Ressuprimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESTANGULAR", "Restangular", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESTRITO", " Restrito", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RESULTADO_DA_BUSCA", "Resultado da busca", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RETORNO_INVENTARIO_BEM_PATRIMONIAL", "Retorno inventário do bem patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RETORNO_INVENTARIO_NOVO_BEM_PATRIMONIAL", "Retorno inventário do novo bem patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.REVOGAR", "Revogar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.RG", "RG", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SAIDA_TEMPORARIA", "Saída temporária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SAIDA_TEMPORARIA_ITENS", "Bens em saída temporária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SAIDAS", "Saídas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SAIDAS_TEMPORARIAS", "Saídas temporárias de bens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALDOS", "Saldos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR", "Salvar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_ALTERACAO_ITEM", "Salvar alteração item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_ALTERACOES", "Salvar alterações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_APENAS", "Salvar apenas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_COMO_NAO_CONCLUIDA", "Salvar com status de não concluída", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_COMO_PRONTA_PARA_ATENDIMENTO", "Salvar com status de pronta para atendimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_CONFIGURACAO", "Salvar configuração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_CONTA_CONTABIL", "Salvar conta contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_E_FINALIZAR_REQUISICOES", "Salvar e finalizar requisições", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVAR_NOVO", "Salvar novo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SALVO", " salvo!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SEGUNDO_TITULO", "Segundo título", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SEGUNDOS_RESTANTES_SESSAO", " segundo(s).", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SEGURADORA", "Seguradora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECAO_BENS", "Selecione os bens para alteração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECAO_TIPO_OPERACAO", "Selecione o tipo de alteração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONAR_CONTAS_RELATORIO", "Selecionar contas para o relatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE", "Selecione", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_DETALHE_MATERIAL", "Selecione o nível detalhe para edição:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_ESTRUTURA_INVENTARIAR", "Selecione uma estrutura para mostrar os bens a se inventariar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_GRUPOS", "Selecione os grupos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_INFRACAO_PENALIZAR", "Selecione uma infração para penalizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_INVENTARIO", "Selecione o inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_ITEM_ENTRADA", "Selecione um item de entrada logo acima!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_LOCAL_DESTINO", "Selecione o local de destino", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_MATERIAL", "Escolha o detalhe que deseja incluir/editar:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_MATERIAL_CONSUMO", "Selecione o material de consumo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_PERMISSOES", "Selecione as permissões", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_RELATORIO", "Selecione o relatório", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_UM_ITEM", "Selecione um item para realizar esta ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_UMA_OPCAO", "Selecione uma opção ...", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SELECIONE_UNIDADE_INVENTARIO", "Selecione as unidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SEM_DATA_TERMINO", "Sem data de término", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SEM_RESULTADOS", "Sem resultados!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SENHA", "Senha", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SENHA_ANTIGA", "Senha antiga", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SENHA_MOBILE", "Senha mobile", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SENHAS_DIFERENTES", "Senhas diferentes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SENHASSH", "Senha SSH", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SERIE", "Série", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SERVIDOR", "Servidor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SESSAO_EXPIRADA", "Sessão Expirada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SEXO", "Sexo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SIGLA", "Sigla", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SIM", "Sim", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SIT_FISICA", "Sit. física", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SIT_FISICA_ATUAL", "Situação física atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITE", "Site", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO", "Situação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_ATUAL", "Situacao atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_BAIXA", "Situação da baixa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_ESTORNO", "Situação Estorno", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_FISICA", "Situação física", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_FISICA_ATUAL", "Situação física atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_FISICA_GERADO_INVENTARIO", "Situação física confirmado no inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SITUACAO_FISICA_INVENTARIADO", "Situação física inventariado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SMTP_HOST", "SMTP (host)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SMTP_PORTA", "SMTP (porta)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SOBRE", "Sobre", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SOBRE_GRP", "Sobre o Citsmart - GRP", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SOLUCAO", "Solução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SOMENTE_MENOR_HIERARQUIA", "Seleciona somente autoridade do nível mais baixo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS", "Status", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_ATUAL", "Status atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_BEM", "Status do bem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_DEPRECIACAO", "Status da depreciação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_GERADO_INVENTARIO", "Status confirmado no inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_INVENTARIADO", "Status inventariado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_INVENTARIO_BEM_PATRIMONIAL", "Status inventário bem patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_LEVANTAMENTO", "Status Levantamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_REPRESENTA_EXTRAVIADO", "Status que representa extraviado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_REQUISICAO", "Status da requisição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.STATUS_RESTRITO", "Restrito por status: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUB_GRUPO", "Subgrupo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUBSTITUICAO_RESPONSAVEL", "Substituição de Responsável", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUBSTITUIR", "Substituir", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUBSTITUTO", "Substituto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUBTOTAL", "Subtotal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUGESTAODEASSUNTO", "Sugestão de Assuntos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.SUGESTOES_ASSUNTO", "Sugestões de assunto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TABELA_VAZIA", "Tabela vazia!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TABLEVAZIA", "Nenhum resultado encontrado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TAG", "Tag", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TAMANHO", "Tamanho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TAMANHO_MASCARA", "Tamanho total da máscara", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TAREFA", "Tarefa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TAXA_DEPRECIACAO", "Taxa utilizada (a.m.)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TAXA_DEPRECIACAO_MENSAL", "Taxa mensal (0-100%)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TAXA_RESIDUAL", "Taxa residual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TELA", "Tela", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TELAS", "Telas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TELEFONE", "Telefone", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TELEFONES", "Telefones", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TEMPLATE", "Template", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TEMPO_ATUALIZACAO", "Tempo Atualização em minutos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TEMPORALIDADE", "Temporalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERCEIRIZADO", "Terceirizado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERCEIRO_TITULO", "Terceiro título", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMINA_EM", "Termina em", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_ADITIVO", "Termo aditivo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_ADITIVOS", "Termos aditivos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_BAIXA_COM_RESUMO_CONTABIL", "Termo de Baixa com Resumo Contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMO_RESPONSABILIDADE", "Termo de responsabilidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMOS_RESPONSABILIDADE", "Termos de responsabilidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TERMOS_RESPONSABILIDADE_TRANSFERENCIA", "Termos de responsabilidade / transferência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO", "Tipo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ABRANGENCIA", "Abrangência do centro de resultado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ALTERACAO", "Tipo de alteração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ANALISE", "Tipo análise", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_AQUISICAO", "Tipo aquisicao", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ATRIBUICAO", "Tipo atribuição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_AVALIACAO_MONETARIA_ESTOQUE", "Tipo de avaliação monetária de estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_BAIXA", "Tipo da baixa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_BAIXA_PADRAO", "Tipo da baixa padrão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_BEM", "Tipo bem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_CHAVE_DOMINIO", "Chave domínio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_CLASSIFICACAO_MATERIAL", "Tipo de classificação de materiais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_COLETA", "Tipo perspectiva", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_COMISSAO", "Tipo comissão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_CONDICAO", "Tipo condição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_CONTA_CONTABIL", "Tipo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_CONTRATO", "Tipo contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DADO", "Tipo dado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DE_ESTRUTURA_ORGANIZACIONAL", "Tipo de estrutura organizacional", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DEP_REAVALIACAO_REDUCAO", "Método de depreciação em Reavaliação/Redução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DESTINO", "Tipo destino", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DISPARO", "Tipo disparo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_DOCUMENTO", "Tipo de documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_EMISSOR", "Tipo de emissor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ENDERECO", "Tipo de endereço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_ENTRADA", "Tipo entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_EXPRESSAO", "Tipo de expressão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_FECHAMENTO", "Tipo fechamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_FEDERAL_SUPPLY", "Tipo federal supply", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_GARANTIA", "Tipo garantia", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_GRAFICO", "Tipo de gráfico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_IMPLEMENTACAO", "Tipo de implementação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_INCONSISTENCIA_BEM_SEM_PLAQUETA", "BEM SEM PLAQUETA", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_INCONSISTENCIA_ESTRUTURA_ORGANIZACIONAL_DIVERGENTE", "Estrutura organizacional divergente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_INTERCORRENCIA", "Tipo de intercorrência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_LIMITE", "Tipo de limite", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_MATERIAL", "Tipo do material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_MODALIDADE_CONTRATO", "Modalidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_MOV_REAVALIACAO_REDUCAO", "Método de movimentação em conta da Reavaliação/Redução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_MOVIMENTACAO", "Tipo de movimentação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_MOVIMENTO", "Tipo do movimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_NOTIFICACAO", "Tipo de notificação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_NUMERICO", "Tipo numérico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_OPERACAO", "Tipo de operação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_PARCEIRO", "Tipo parceiro", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_PERSPECTIVA", "Tipo perspectiva", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_PESSOA", "Tipo pessoa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_PLANO_CLASSIFICACAO", "Tipo plano classificação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_PRIORIDADE", "Tipo de prioridade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_RECEBIMENTO", "Tipo recebimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_REITERACAO", "Tipo reiteração", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_REPETICAO", "Tipo repetição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_RESTRICAO", "Tipo restrição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_SAIDA", "Tipo de saída", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_SAIDA_TEMP", "Tipo de saída", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_TERMO_RESPONSABILIDADE", "Tipo do Termo de responsabilidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_UNIDADE_MEDIDA", "Tipo da unidade de medida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_UNIDADE_MEDIDA_ENTRADA", "Unidades de medida de entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_UTILIZACAO", "Utilização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPO_VALOR", "Tipo de valor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPODOCUMENTO", "Tipo Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPODOMINIOIDIOMA", "Idioma", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPOPROCESSO", "Tipo de processo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPOPROTOCOLO", "Tipo de Protocolo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPOSDOCUMENTO", "Tipos de Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TIPOUNIDA_DEMEDIDA", "Unidade de medida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITLE_ALTERA_BENS_PATRIMONIAIS", "Serão alterados ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITLE_INCONSISTENCIA", "Inconsistências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITLE_TRATAR_INCONSISTENCIA", "Serão tratadas ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO", "Título", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ANUAL_RMA", "Relatório de Movimentação Anual de Almoxarifado - RMAA", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ANUAL_RMB", "Relatório de Movimentação Anual de Bens Móveis - RMAB", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_AUTORIZACAO_BAIXA_BENS_PATRIMONIO", "Autorização de Saída de Bens Patrimoniais Baixados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_BAIXAS_PERIODO", "Baixas no Período – Por Conta Contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_BEM_PATRIMONIAL_ANALITICO_UA_ENDERECO", "Relatório Analítico de Bens Móveis (Ordenado por Descrição - Agrupado por U.A./Endereço)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_BEM_PATRIMONIAL_FICHA_INDIVIDUAL", "Ficha Individual do Bem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_BENS_PATRIMONIAIS_INVENTARIAR", "Relatório de bens patrimoniais a se inventariar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_CONSUMO_ALMOXARIFADO", "Relação de Consumo no Período - Por Material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ELEITOR", "Título de eleitor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENTRADA_ALMOXARIFADO", "Relação de Entradas no Período - Por Material - Almoxarifado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENTRADA_MATERIAL_ANALITICO", "Entrada de Material - Analítico - Por Entrada (Nota Fiscal)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENTRADA_MATERIAL_NOTA_RECEBIMENTO", "Nota de recebimento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENTRADA_PATRIMONIO_ANALITICO", "Entrada de Materiais - Analítico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENTRADA_PATRIMONIO_RESUMO_CONTABIL", "Resumo por Código Contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENTRADA_PATRIMONIO_SINTETICO", "Entrada de Materiais - Sintético", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_ENTRADA_PERIODO_PATRIMONIO", "Relação de Entradas no Período - Por Fornecedor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_EXPORT_CSV_BEM_PATRIMONIAL", "csv_bem_patrimonial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_ANALITICO", "Relatório de Inventário Geral Agrupado por Conta Contábil Analítico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_BENS_ENCONTRADOS_EXTRAVIADO", "Bens Encontrados com Status Extraviado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_BENS_ENCONTRADOS_EXTRAVIADO_LOCALIZADOS", "Bens Encontrados com Status Extraviado (Localizados)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_BENS_NAO_ENCONTRADOS", "Bens Não Encontrados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_BENS_NAO_ENCONTRADOS_LOCALIZADOS", "Bens Não Encontrados (Localizados)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_GERAL_ADMINISTRATIVO", "Relatório de Inventário Geral Por Unidade Administrativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_GERAL_CONTA_CONTABIL", "Relatório de Inventário Geral Agrupado Por Conta Contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_GERAL_LOCALIZACAO", "Relatório de Inventário Geral Por Localização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_GERAL_LOCALIZADORA", "Relatório de Inventário Geral Por Localização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_GERAL_PATRIMONIO", "Relatório de Inventário Geral Por Patrimônio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_GERAL_RESUMO_CONTA_CONTABIL", "Relatório Inventário Geral – Resumo por Contas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_QUANTITATIVO_LEVANTAMENTO", "Relatório Quantitativo Por Levantamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_INVENTARIO_SINTETICO", "Relatório de Inventário Geral Agrupado por Conta Contábil Sintético", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_MATERIAL_ESTOQUE_ALMOXARIFADO", "Relatório de Material - Serviço Almoxarifado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_MENSAL_RMA", "Relatório de Movimentação Mensal de Almoxarifado - RMMA", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_MENSAL_RMB", "Relatório de Movimentação Mensal de Bens Móveis - RMMB", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_PAGINA", "Título página", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_POSICAO_ESTOQUE_ALMOXARIFADO_ANALITICO", "Posição do Estoque Analítico - Por Conta", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_POSICAO_ESTOQUE_ALMOXARIFADO_RESUMO_ANALITICO", "Resumo da Posição do Estoque (Agrupado por Conta) - Analítico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_RECIBO_BAIXAS_BENS_PATRIMONIO", "Recibo de Bens Baixados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_RELATORIO_DE_DEPRECIACAO", "Relatório de Depreciação – Sintético", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_RESUMO_BAIXAS_CONTA_CONTABIL", "Resumo por Código Contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_RESUMO_BAIXAS_PERIODO", "Resumo de Baixas no Período – Por Conta Contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_RESUMO_INVENTARIO_POR_LEVANTAMENTO", "Resumo de inventário por levantamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_SAIDA_ALMOXARIFADO", "Relação de Saídas no Período - Por Material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_SAIDA_MATERIAL_CONTA_SINTETICO", "Saída de Material - Por Conta (Sintético)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_BAIXAS_BENS_PATRIMONIO", "Termo de Baixa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_DEFINICAO_DETENTOR", "Termo de Responsabilidade para Definição de Detentor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_DEFINICAO_DETENTOR_LIST", "Transferência de Detentor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_DETENTOR_PATRIMONIO", "Termo de Responsabilidade por Detentor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_RESPONSABILIDADE_INVENTARIO_POR_UNIDADE", "Termo de responsabilidade do inventário por U.L.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_RETORNO_MATERIAL_SAIDA_TEMPORARIA", "Termo de Retorno de Material em Saída Temporária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_TRANSFERENCIA", "Termo de Transferência Interna", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_TRANSFERENCIA_EXTERNA", "Termo de Transferência Externa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TITULO_TERMO_UNIDADE_PATRIMONIO", "Termo de Responsabilidade por Unidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TODOS", "Todos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TOTAL_ITENS_HISTORICO", "Total de históricos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERECENCIA_ENTRE_ESTRUTURAS", "Transferência entre estruturas organizacionais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIA", "Transferência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIA_CONTA_CONTABIL", "Transferência conta contábil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIA_CONTAS_CONTABEIS", "Transferência contas contábeis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIA_ESTRUTURA", "Transferência de almoxarifado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIA_INTERNA", "Transferência interna", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIACONTACONTABIL", "Transferências entre contas contábeis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIACONTACONTABILS", "TransferenciaContaContabils", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERENCIAS", "Transferências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSFERIR", "Transferir", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRANSIENTE", "Transiente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRATADA", "Tratada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRATAR_INCONSISTENCIA_MULTPLO", "Tratar Inconsistências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TRATAR_INCOSISTENCIA_MULTPLO", "Tratar Inconsistências dos Bens Selecionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.TSL_SSL", "TSL/SSL", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UA", "U.A.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UL", "U.L.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ULTIMO", "Último", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.ULTIMO_SALDO", "&Uacute;ltimo saldo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UN_ADMINISTRATIVA", "Unidade administrativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UN_GESTORA", "Unidade gestora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UN_LOCALIZADORA", "Unidade localizadora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE", "Unidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_ADMINISTRATIVA", "Unidade administrativa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_GESTORA", "Unidade Gestora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_LOCALIZADORA", "Unidade Localizadora", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_LOCALIZADORA_BENS", "Unidade localizadora de bens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_MEDIDA", "Unidade de medida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_MEDIDA_ARMAZENAMENTO", "Unidade de medida de armazenamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_MEDIDA_ENTRADA", "Unidade de medida de entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_MEDIDA_MATERIAL", "Unidade de medida do material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_MEDIDA_SIGLA", "U.M.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_PRINCIPAL", "Unidade principal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_REQUISITANTE", "Unidade requisitante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_REQUISITANTE_SEM_RESTRICAO_DIAS_REQUISICAO", "Unidade requisitante sem restrição de dias em que pode fazer requisições.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADE_RM", "Unidade RM", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADES", "Unidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADES_REQUISITANTES", "Unidades requisitantes", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UNIDADES_SIGLA", "Unds.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UPLOAD", "Enviar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UPLOAD_FILE", "Clique para escolher", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UPLOAD_MESSAGE", "Arraste e solte os arquivos ou clique para escolher", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UPLOAD_MSG_ARRASTE", "Arraste e solte o arquivo ou clique para escolher", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.UR_POSSUI_CATALOGO_MATERIAL", "Unidade requisitante possui catálogo de materiais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.URL", "Url", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.URL_SERVICO", "URL do serviço", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.USERNAME", "Username", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.USUARIO", "Usuário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.USUARIOS", "Usuários", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.USUARIOS_SELCIONADOS", "Usuários selecionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.USUARIOS_SELECIONADOS", "Usuários selecionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.USUARIOSSH", "Usuário SSH", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALIDAR_PRECO_MEDIO", "Validar preço médio do material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR", "Valor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_ANTERIOR", "Valor anterior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_AQUISICAO", "Valor aquisição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_ATUAL", "Valor após depreciação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_ATUAL_BEM", "Valor líquido atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_BRUTO", "Valor bruto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_BRUTO_ATUAL", "Valor bruto atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_CARACTERISTICA", "Valor caracter&iacute;stica", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_CONSERTO", "Valor do conserto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_CONTRATUAL", "Valor contratual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_EMPENHADO", "Valor empenhado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_ESTOQUE", "Valor Estoque", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_GERADO_INVENTARIO", "Valor confirmado no inventário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_INICIAL", "Valor inicial", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_INVENTARIADO", "Valor inventariado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_ITEM", "Valor total do material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_LIQUIDO", "Valor líquido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_PADRAO", "Valor padrão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_PENDENTE", "Valor pendente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_RESIDUAL", "Valor residual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_SOBRA", "Valor de sobra", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_SUBTOTAL", "Valor subtotal", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL", "Valor total", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_CONTRATO", "Valor total do contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_INCLUIDO", "Valor dos materiais adicionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_MATERIAIS", "Valor total materiais", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_MATERIAL", "Valor total do material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_NOTA", "Valor total da nota", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TOTAL_UNITARIO", "Valor Total unitário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_TRANSFERENCIA", "Valor da transferência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_UNITARIO", "Valor unitário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VALOR_UNITARIO_MATERIAL", "Valor unitário do material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VARIAVEL", "Variável", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VER_TODOS_FAVORITOS", "Ver todos os favoritos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VERSAO", "Versão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VERSAO_ATUAL", "Versão Atual", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VERSAO_DOCUMENTO", "Versões do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VERSAO_INTERMEDIARIA", "Versão Intermediária", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VERSOESDODOCUMENTO", "Versões do Documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VIDA_UTIL", "Vida útil", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VIDA_UTIL_EM_MESES", "Vida útil (Em meses)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VIDA_UTIL_ORIGINAL", "Vida útil original", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VIDA_UTIL_RESTANTE", "Vida útil restante", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VIDA_UTIL_RESTANTE_EM_MESES", "Vida útil restante (Em meses)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VINCULAR_BENS", "Vincular bens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VISAO_CONTRATO", "Visão Contrato", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VISUALIZAR", "Visualizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VISUALIZAR_HISTORICO_RESPONSAVEIS", "Visualizar histórico de responsavéis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VISUALIZAR_INCONSISTENCIAS", "Visualizar inconsistências", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VISUALIZAR_INVENTARIADO", "Visualizar inventariado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VISUALIZAR_SUBMENUS", "Visualizar menus filhos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.VOCE_SERA_DESLOGADO", "Você será deslogado em: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.WIDGET", "Widget", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.WIDGET_ITEM", "Widget item", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.WIDGET_ITENS", "Widget itens", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.WIDGETS", "Widgets", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LABEL.WORK_DAY", "Jornadas de Trabalho", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LOG.ATUALIZACAO_NAO_PERMITIDA", "Atualização não permitida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LOG.CONSULTA_LOG", "Consultar Histórico", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LOG.DADOS_BASICOS", "Dados básicos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LOG.HISTORICO", "Histórico de Execução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("LOG.SALVAR_NA_VERSAO_ORIGINAL", "Na versão original", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.CANCELAR", "Cancelar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.DATA_HORA_INICIO", "Início", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.DELEGAR", "Delegar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.EXECUTANDO_TAREFA", "Executando tarefa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.EXECUTAR", "Executar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.INICIAR", "Iniciar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.REATIVAR", "Reativar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.SUSPENDER", "Suspender", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PERMISSAO.VISUALIZAR", "Visualizar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PLACEHOLDER.FILTRO_IVENTARIAR_BEM_PATRIMONIAL", "Identificação Patrimonial ou Descrição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PLACEHOLDER.PESQUISA_PATRIMONIO_PLACE", "Consulta por Nº Patrimonial ou descrição material", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ACAO", "Ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ACAO_ENTRADA", "Ação de entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ACAO_SAIDA", "Ação de saída", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ACOES_TAREFA", "Selecione uma ou mais ações disponíveis ao usuário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ALVO", "Alvo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ATORES_TAREFA", "Atores da tarefa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.CONDICAO", "Condição", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.COR_FUNDO", "Cor de fundo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.COR_TEXTO", "Cor do texto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.DATA_HORA_CRIACAO_TAREFA", "Criada em", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.DESCRICAO_TAREFA", "Descrição da tarefa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.DIAS", "Dia(s)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.EDITAR_VERSAO_ANTERIOR", "Editar versão anterior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ENGINE_EXECUCAO", "Engine", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.ESTADO_FINAL", "Estado final", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.EXECUTADO", "Executado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.FLUXOS", "Fluxos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.HORAS", "h", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.IDENTIFICADOR_ALVO", "Identificador do alvo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.MINUTOS", "min", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.NENHUMA_ACAO", "Nenhuma ação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.NOME_FLUXO", "Nome do fluxo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.NOME_TAREFA", "Nome da tarefa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.SCRIPT", "Script", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.SEGUNDOS", "seg", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.SINCRONO", "Síncrono", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.TEMPO_ESTIMADO", "Tempo estimado de execução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.TIPO_ALTERACAO_SITUACAO", "Tipo de alteração da situação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.TIPO_ESTIMATIVA", "Tipo de estimativa do tempo de execução", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.TIPO_INSTANCIA", "Tipo de instância", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.TIPO_INSTANCIA_UNICA", "Única", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.TIPO_INSTANCIA_USUARIO", "Por usuário", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.TIPO_INSTANCIA_VARIAS", "Várias (controlada pelo fluxo)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("PROPRIEDADE.VARIAVEIS_TAREFA", "Vari&aacute;vel(eis) de entrada/sa&iacute;da da tarefa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.INVENTARIO_BENS_ENCONTRADOS", "Relatório Bens Encontrados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.INVENTARIO_BENS_EXTRAVIADO", "Relatório Bens Encontrados com Status Extraviado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.INVENTARIO_BENS_INCORPORADOS", "Relatório Bens Incorporados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.INVENTARIO_BENS_NAO_ENCONTRADOS", "Relatório Bens não Encontrados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.INVENTARIO_BENS_OUTRAS_LOCALIDADE", "Relatório Bens Encontrados em Outras Localidades", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.QUANTITATIVO_INVENTARIO_LEVANTAMENTO", "Quantitativo de Inventário por Levantamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.RELATORIO_RESUMO_INVENTARIO_LEVANTAMENTO", "Resumo de Inventário por Levantamento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("RELATORIO.TERMO_RESPONSABILIDADE_INVENTARIO_UNIDADE", "Termo de Responsabilidade do Inventário por Unidade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("SOLICITACAO_ALCADA.CENTRO_RESULTADO", "Centro de resultado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("SOLICITACAO_ALCADA.DATA_REFERENCIA", "Data de referência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("SOLICITACAO_ALCADA.ID_ORIGEM_SOLICITACAO", "Identificação da origem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("SOLICITACAO_ALCADA.ORIGEM_SOLICITACAO", "Requisição origem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("SOLICITACAO_ALCADA.TIPO_UTILIZACAO", "Tipo de utilização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.CLASSE", "Classe java", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.ENTRADA", "Entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.FLUXO_SUSPENSAO", "Regra de negócio para validação da suspensão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.MULTIPLO", "Lista de valores", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.NOME", "Nome da variável", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.NOVO_PROCESSO", "Iniciar processo de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.PERSISTENTE", "Armazena no banco de dados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.PROPRIEDADE_NAO_PREENCHIDA", "Pelo menos uma propriedade obrigatória do elemento '#elemento#' não foi preenchida", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.RETORNO", "Variável de retorno", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.SAIDA", "Variável de saída da interface do fluxo", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VARIAVEL.TIPO", "Tipo da variável", dominio, modulo));

	}

	private void gerarValidacaoContrato(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {

		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.400", "Falha inesperada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.700", "Os seguintes campos apresentam erros de validação:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.701", "Os seguintes campos são obrigatórios:", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.702", "Erro na regra de negócio", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.DATA_FIM_GARANTIA_MENOR_DATA_INICIO", "Data final da garantia deve ser maior que a data inicial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.MN008", "Número patrimonial informado já está associado a outro bem!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.NOVO_VALOR_REAVALIACAO", "O novo valor deve ser maior que o valor atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.NOVO_VALOR_REDUCAO", "O novo valor deve ser menor que o valor atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("CODIGOERRO.NOVO_VALOR_RESIDUAL", "Novo valor residual não pode ser maior que o valor líquido atual do bem!", dominio, modulo));

		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALERTA_OBRIGATORIOS", "Campos obrigatórios não preenchidos ou dados inválidos. Verifique os campos em destaque.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALERTA_OBRIGATORIOS_BEM_PATRIMONIAL", "Existe algum bem patrimonial que não está com os dados obrigatórios preenchidos!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALERTA_OBRIGATORIOS_LIMITEAPROVACAO_VALOR", "É obrigatório informar o tipo de valor, a utilização e o valor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALERTA_OBRIGATORIOS_LISTA_DOCUMENTOS", "É obrigatório informar pelo menos um documento!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALERTA_OBRIGATORIOS_LISTA_MATERIAL", "É obrigatório informar pelo menos um material!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALERTA_OBRIGATORIOS_NIVELAUTORIDADE_ALCADA", "É obrigatório informar o nivel de autoridade e a antecedência mínima para aprovação!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALERTA_OBRIGATORIOS_RESPONSAVEIS", "É obrigatório informar o responsável e pelo menos uma alçada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ALTERACAO_DADOS_BEM_ANTERIOR_ULTIMA_OPERACAO", "Você não pode alterar um bem patrimonial com data anterior a da última operação do bem! Verifique a operação: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ANEXOS", "Anexos", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.BAIRRO_UTILIZANDO_CIDADE", "Existe bairro que utiliza esta cidade!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.BAIXA_NAO_POSSUI_ITEM", "Movimento não possui item de baixa!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.BEM_AINDA_NAO_INVENTARIADO", "Bem patrimonial ainda não inventariado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.BEM_JA_TRATADO", "Não é possível editar um inventário de um bem patrimonial com ao menos uma inconsistência tratada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.BEM_JA_VINCULADO", "Bem patrimonial já vinculado a este detentor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.BEM_TRANSFERIDO_MESMA_DATA", "Você não pode transferir um bem patrimonial mais de uma vez no mesmo dia! Verifique o bem: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.BENS_ERRO_VALIDACAO", "Existem Bens Patrimoniais com erro de validação. Verifique!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CAMPO_DATA_INICIO_FIM", "Data início deve ser menor que data fim!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CAMPO_DATA_INICIO_MAPA", "Só pode haver um mapa organizacional vigente, portanto, o novo mapa deve ter a data início no mínimo 2 dias superior a data início do último mapa vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CARREGUE_ESTRUTURAS", "Carregue as estruturas organizacionais para salvar o formulário!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CENTRO_RESULTADO_CODIGO_IGUAL", "Já existe um centro de resultado com esse código!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CENTRO_RESULTADO_NOME_IGUAL", "Já existe um centro de resultado com esse nome!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CIDADE_UTILIZANDO_ESTADO", "Existe cidade que utiliza este estado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CNPJ_EXISTENTE", "O CNPJ informado já está cadastrado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CODIGO_CLASSIFICACAO_UNICO", "Código informado já está associado a outra classificação de materiais!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CODIGO_UNICO", "Código informado já está associado a outro registro!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CONFIRMAR_INVENTARIO_SEM_ESTRUTURA", "Não é possível confirmar um inventário sem ter ao menos uma estrutura organizacional selecionada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CPF_EXISTENTE", "O CPF informado já está cadastrado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CPF_INVALIDO_CONTATO", "CPF do contato está inválido.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CPF_OBRIGATORIO_REPRESENTANTE_LEGAL", "CPF é obrigatório para o representante legal.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.CRON_EXPRESSION_INVALIDO", "Cron expression invalido!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_AQUISICAO_INICIAL_MAIOR_FINAL", "A data de aquisição final deve ser maior que a data de aquisição inicial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ATENDIMENTO_FORA_REFERENCIA", "Data de atendimento fora da referência vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ATRIBUICAO_DENTRO_VIGENCIA", "Data de atribuição deve estar dentro da vigência!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_BAIXA_INICIAL_MAIOR_FINAL", "A data de baixa final deve ser maior que a data de baixa inicial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_BAIXA_MAIOR_DATA_CONCLUSAO", "A data da baixa não pode ser maior que a data de conclusão!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_CELEBRACAO_MAIOR_DATA_VIGENCIA", "A data da celebração não pode ser maior que a data de vigência inicial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_CONCLUSAO_FORA_REFERENCIA_VIGENTE", "A data de conclusão da baixa está fora da referência vigente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_CONTABIL_ALTERADA_TRANSFERENCIA", "Você não pode alterar a data de contabilização após realizar movimentações nos bens da mesma!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_CONTABIL_DATA_RECEBIMENTO", "A data de contabilização não pode ser menor que a data de recebimento!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_CONTABIL_VIGENCIA", "Data de contabilização não pode ser inferior a data de vigência!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_CONTABILIZACAO_INICIAL_MAIOR_FINAL", "A data de contabilização final deve ser maior que a data de contabilização inicial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_DENTRO_VIGENCIA", "Data deve estar dentro da vigência!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_DEVOLUCAO_DATA_CONSUMO", "Data de devolução não pode ser inferior a data da requisição!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_DEVOLUCAO_DATA_FINALIZACAO", "Data de devolução não pode ser inferior a data de finalização da requisição!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_EMISSAO_DATA_ATUAL", "Data de emissão não pode ser maior que a data atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ENCERRAMENTO_DATA_INICIO", "Data de encerramento não pode ser inferior a data início!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ENCERRAMENTO_DATA_FINAL", "Data de encerramento não pode ser inferior a data de vitgência final!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ESTORNO_FORA_REFERENCIA_VIGENTE", "Data de estorno tem que estar dentro da referência vigente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ESTORNO_MENOR_DATA_ALMOXARIFADO", "A data do estorno não pode ser menor que a data da baixa!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_FIM_GARANTIA_MENOR_DATA_INICIO", "Data final da garantia deve ser maior que a data inicial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_FIM_MENOR_DATA_INICIAL", "A data fim não pode ser menor que a data início!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_FIM_REFERENCIA", "Data fim não pode ser menor que a referência vigente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_FINALIZACAO_FORA_REFERENCIA", "Data de finalização tem que estar dentro da referência vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_FORA_REFERENCIA_EXCLUSAO", "Não pode ser excluído pois está fora da referência vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_FORA_REFERENCIA_EXCLUSAO_ENTRADA", "Essa entrada não pode ser excluída porque ela não foi contabilizada na referência vigente.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_GARANTIA_INVALIDA", "Data de garantia inválida.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INCLUSAO_INICIAL_MAIOR_FINAL", "A data de inclusão final deve ser maior que a data de aquisição inicial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INICIO_COM_VIGENCIA", "Data início deve estar no mesmo ano da referência vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INICIO_FIM_ATRIBUICAO", "Data fim da atribuição deve ser maior que a data início da atribuição!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INICIO_GARANTIA", "Informe a data de início da garantia!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INICIO_GARANTIA_SAIDA_TEMP", "O inicio da garantia deve ser após a saída do bem para conserto!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INICIO_MAPA_VIGENTE", "Data início não pode ser inferior a data início do Mapa Organizacional vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_INVALIDA", "Data inválida!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_LIBERACAO_PAGAMENTO_MENOR_DATA_EMPENHO", "A data da liberação de pagamento não deve ser inferior a data do empenho!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_OPERACAO_FORA_REFERENCIA", "Data da operação não pode estar fora da referência vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_PERIODO_GARANTIA", "O término da garantia deve ser maior que o início!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_PREVISAO_FIM_DATA_INICIO", "Data prevista fim não pode ser inferior a data início!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_PREVISTA_HOJE", "A data prevista fim não pode ser inferior a data atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_RECEBIMENTO", "Data de recebimento não poder ser maior que a data atual ou maior que a data de contabilização!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_REFERENCIA_ADICAO", "deve estar com a data de referência dentro da vigência!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_REQUISICAO_CONSUMO_FORA_VIGENCIA_ORGANIZACAO", "Data da requisição de consumo fora da vigência do órgão!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_REQUISICAO_CONSUMO_FORA_VIGENCIA_ORGAO", "Data da requisição de consumo fora da vigência do órgão!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_RESCISAO_DENTRO_VIGENCIA", "A data da rescisão deve estar dentro da vigência do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_TERMINO_GARANTIA", "Informe a data de término da garantia!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_TERMINO_GARANTIA_DATA_ATUAL", "O término da garantia não pode ser menor que a data atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_TRANSFERENCIA_FORA_REFERENCIA", "A data de transferência deve estar dentro da referência vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_TRANSFERENCIA_MAIOR_ATUAL", "A data de transferência não pode ser maior que a data atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_TRANSFERENCIA_VIGENCIA", "Data de transferência não pode ser inferior à vigência!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ULTIMA_TRANSFERENCIA", "Data da transferência deve ser maior que da última transferência realizada ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_ULTIMA_TRANSFERENCIA_EXCLUSAO", "Esta transferência possui bem patrimonial com transferência mais recente ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DATA_VIGENCIA_INICIAL_MAIOR_DATA_ATUAL", "A situação atual do contrato está definida como 'Concluído', a data de vigência inicial do contrato deve ser menor que a data corrente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DEFINICAO_DETENTOR_ANTERIOR_ULTIMA_OPERACAO", "Você não pode definir um detentor de um bem patrimonial com data anterior a da última operação do bem! Verifique a operação: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DESEJA_ENCERRAR_PARTICIPACAO_DE", "Deseja encerrar a participação de ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.DESTINO_BEM_IGUAL_ORIGEM", "O bem escolhido não pode ser transferido, pois o destino é o mesmo da origem do bem!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EDICAO_BEM_BAIXADO", "Bens baixados não podem ser editados!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EDICAO_BEM_NAO_CONTABILIZADO", "Se você deseja editar bens não contabilizados ou com data de contabilização maior ou igual a da referência vigente, edite pela entrada do bem!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EDITAR_ENTRADA_CONTABIL", "Você só pode editar entradas sem data contabil ou com data maior ou igual ao do mês de referência!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ENDERECO_PRINCIPAL_JA_CADASTRADO", "Já existe um endereço principal cadastrado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ENDERECO_PRINCIPAL_NAO_CADASTRADO", "Não existe um endereço principal cadastrado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ENTRADA", "Entrada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ENTRADA_DESCONTABILIZACAO_COM_TRANSFERENCIA", "Você não pode descontabilizar esta entrada pois as seguintes transferências possuem bens da mesma: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ENTRADA_EDICAO_COM_TRANSFERENCIA", "Você não pode editar este item de entrada pois as seguintes transferências possuem bens deste item: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EQUIPE_GESTORA_VALIDACAO", "É obrigatório o cadastro do gestor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_EDICAO_EXISTE_MOVIMENTACAO", "Não foi possível realizar a edição pois existe movimentações após o atendimento.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_ESTRUTURA_VINCULADA_DATA_FIM", "A Estrutura não pode ser salva com data fim pois contém vínculo com outra(s) entidade(s)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_EXCLUSAO_CLASSIFICACAO", "Essa classificação e(ou) seus nós filhos já estão vinculados a materiais no sistema e não podem ser excluídos!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_EXCLUSAO_CLASSIFICACAO_RAMO_ATIVIDADE", "Essa classificação e(ou) seus nós filhos já estão vinculados a algum ramo de atividade de fornecedores!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_EXCLUSAO_ESTRUTURA", "A Estrutura não pode ser removida pois contem vínculo com outra(s) entidade(s)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_EXCLUSAO_EXISTE_MOVIMENTACAO", "Não foi possível realizar a exclusão pois existe movimentações após o atendimento.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_EXCLUSAO_PESSOA", "Exclusão não permitida, pois o registro está sendo referenciado em outros cadastros do sistema!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_LOCAL_ESTOQUE", "Erro ao tentar salvar local estoque!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_PESSOA_JURIDICA_COLABORADOR", "Pessoas jurídicas não podem ser do tipo parceiro colaborador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_UNIDADE_ECM_VIGENTE_USUARIO", "O usuário vigente não possui unidade organizacional ECM vinculada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ERRO_VINCULACAO_USUARIO", "O usuário já está vinculado a outra pessoa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ESCOLHA_DIAS_REQUISICOES", "Selecione ao menos um dia em que a unidade requisitante poderá realizar requisições!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ESTRUTURA_NAO_POSSUI_CONTA", "Conta contábil Almoxarifado não está configurada nas definições gerais do sistema! Escolha uma conta contábil para o almoxarifado nas configurações do Patrimônio para prosseguir.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ESTRUTURA_VINCULADA", "Estrutura não pode ser removida pois contém vínculo com outra(s) entidade(s).", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EXCLUIR_SAIDA_RETORNADA", "Você não pode excluir uma saída que tenha registro de retorno!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EXCLUSAO_BAIXA", "A baixa ou um dos seus itens só pode ser excluído se a situação for 'EM ANDAMENTO'", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EXCLUSAO_ENTRADA_IMPOSSIVEL", "Já houve movimentação com bens provenientes dessa entrada, ela não poderá mais ser excluída do sistema.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EXCLUSAO_INVENTARIO", "Somente inventário com status Aberto pode ser excluído!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EXCLUSAO_TRANSFERENCIA_IMPOSSIVEL", "Não é possível excluir a transferência, pois houve movimentação após a transferência!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.EXISTE_ORGANIZACAO_COM_MESMO_CODIGO", "Já existe um orgão cadastrado com esse código!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.FUNCAO_CODIGO_IGUAL", "Já existe uma função cadastrada com esse código!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.FUNCAO_NOME_IGUAL", "Já existe uma função cadastrada com esse nome!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.GARANTIA_OBRIGATORIA", "Deve existir pelo menos uma garantia no contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_DATA_EMISSAO_DOCUMENTO", "Informe a data de emissão do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_EMIENTE_DOCUMENTO", "Informe o emitente do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_MATERIAL", "Informe o material!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_NUMERO_DOCUMENTO", "Informe o número do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_SITUACAO_MATERIAL", "Informe a situação do material!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_STATUS_MATERIAL", "Informe o status da disponibilidade do material!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_TIPO_DOCUMENTO", "Informe o tipo do documento", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_UNIDADE_RELATORIO", "Informe uma ou mais unidades do inventário para gerar o relatório.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INFORME_VALOR_TOTAL_MATERIAL", "Informe o valor total do material!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INTERVALO_VAZIO_DATA_AQUISICAO", "Preencha o intervalo corretamente da data de aquisição!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INTERVALO_VAZIO_DATA_BAIXA", "Preencha o intervalo corretamente da data de baixa!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INTERVALO_VAZIO_DATA_CONTABILIZACAO", "Preencha o intervalo corretamente da data de contabilização!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INTERVALO_VAZIO_DATA_INCLUSAO", "Preencha o intervalo corretamente da data de inclusão!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INTEVALO_INVALIDO", "Intervalo inválido!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INTEVALO_VAZIO", "Preencha o intervalo completo!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.INVENTARIO_ANUAL_JA_EXISTE", "Já existe inventário anual para o mesmo ano da vigência referente! Deseja continuar?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.JA_VINCULADO", "já vinculado a este detentor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.JBOSS_ERRO_IDENTIFICACAO", "Esta operação não pode ser realizada para esta configuração definida no servidor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.JBOSS_ERRO_LEITURA_WAR", "Não foi possível identificar os módulos contidos no servidor. Não será possível ativar o módulo!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.LIBERACAO_PAGAMENTO_PARA_VALOR_CONTRATO", "Contrato já teve liberação do seu valor total!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.LISTA_BENS_VAZIA", "Lista de bens deve conter pelo menos um bem patrimonial!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MATERIAL_UTILIZANDO_CARACTERISTICA", "Existe material que utiliza esta característica!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MAXIMO_BEM_REAVALIACAO", "Você não pode realizar reavaliação para mais de 1 bem patrimonial por vez!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MAXIMO_BEM_REDUCAO", "Você não pode realizar redução para mais de 1 bem patrimonial por vez!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MAXIMO_POSSIVEL_RESPOSTA", "Você pode adicionar no máximo 10 possíveis respostas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MINIMO_INTEGRANTES", "Deve ser informado pelo menos 3 integrantes!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MINIMO_POSSIVEIS_RESPOSTAS", "Você deve adicionar pelo menos duas possíveis respostas!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MN005", "Data de bloqueio não pode ser menor que a data atual!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MODULO_NAO_ENCONTRADO", "Não é possível ativar o módulo. Ele não está presente no servidor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MOVIMENTO_NAO_POSSUI_BAIXA", "É necessario baixa para realizar movimento!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MOVIMENTO_NAO_POSSUI_ENTRADA", "É necessaria entrada para realizar movimento!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MOVIMENTO_NAO_POSSUI_TRANSFERENCIA", "É necessario baixa para realizar transferencia!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.MUDANCA_VALOR_CONTRATO", "O valor do contrato foi modificado. Deseja recalcular os percentuais das garantias de acordo com o novo valor do contrato?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NAO_HA_INCONSISTENCIA_PARA_TRATAMENTO_ITENS_SELECIONADOS", "Não há inconsistência para ser tratada para os itens selecionados", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NAO_INFORMADO", "Não informado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NECESSARIO_CONTA_CONTABIL_ALMOXARIFADO", "É necessário informar uma conta contábil almoxarifado para a(s) estrutura(s) almoxarifado(s)!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NIVEL_AUTORIDADE_NOME_IGUAL", "Já existe um nivel de autoridade com esse nome!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NO_PAPEL_DE", "no papel de ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NOTA_FISCAL_IGUAL_VALOR_INFORMADO", "O valor informado para nota fiscal está divergente do valor calculado para os materiais adicionados!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NUMERO_BEM_PATRIMONIAL_REPETIDO", "Já existe um bem patrimonial com esse número!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.NUMERO_CONTRATO_IGUAL", "Já existe um contrato cadastrado com esse número!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ORGANIZACAO_NAO_VINCULADO_AO_USUARIO", "Usuário não possui vínculo a nenhum orgão!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ORGAO_NAO_VINCULADO_AO_USUARIO", "Usuário não possui vínculo a nenhum orgão!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PAINEL_EXISTENTE", "Já existe um painel no sistema com esse nome!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PERIODO_INVALIDO", "Período inválido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PERIODO_INVALIDO_DATAINICIO_MAIOR_DATAFIM", "Período inválido. A data início deve ser menor que a data fim da referência", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PERIODO_INVALIDO_INFERIOR_DOZE_MESES", "Período inválido. O período da referência deve ser inferior a 12 meses", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PLANO_CLASSIFICACAO_ULTRAPASSA_NIVEL_PERMITIDO", "Não é permitido cadastrar mais itens deste nível", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.POSSIVEl_RESPOSTA_JA_ADD", "Você já adicionou esta resposta!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PREENCHA_CAMPO_OBSERVACAO", "Preencha o campo observação!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PREENCHA_UM_ITEM_DEVOLUCAO", "Quantidade a ser devolvida dever ser maior que zero!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.PRESIDENTE_NAO_INFORMADO", "Você deve informar pelo menos um presidente", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QTDADE_DEVOLVIDA_MAIOR_ATENDIDA", "A quantidade a ser devolvida não pode ser maior que a atendida!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUANTIDADE_A_ATENDER_SUPERIOR_A_QTD_ESTOQUE", "Quantidade a atender superior a do estoque para o material: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUANTIDADE_ATENDIDA_E_QUANTIDADE_SUPERIOR", "A soma da quantidade atendida e da quantidade a atender é superior a quantidade requisitada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUANTIDADE_MAIOR_ZERO", "Quantidade tem que ser maior do que zero!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUANTIDADE_SUPERIOR_SALDO", "A quantidade do item da baixa não pode ser maior que a quantidade estoque!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUANTIDADE_ZERADA_NEGATIVA", "A quantidade a atender está zerada ou vazia do seguinte material: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUESTAO", "Questão", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.QUESTAO_NAO_PREENCHIDA", "Você deve preencher todos os dados da questão para adicionar a mesma!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REGIAO_UTILIZANDO_PAIS", "Existe região que utiliza este país!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REGISTRO_BLOQUEADO_NAO_PODE_SER_ALTERADO", "Registro bloquado não pode ser alterado, tente desbloqueá-lo.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REGRA_EDICAO_BAIXA", "Essa baixa de bens não pode ser editada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REGRA_ESTORNO_DATA_ESTORNO_MENOR_DATA_CONCLUSAO", "Data de estorno não pode ser inferior à data de conclusão da baixa", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REGRA_ESTORNO_RN002", "Para uma baixa de bens ou um item da baixa ser estornado a situação deve ser 'AUTORIZADA' e a data de conclusão deve estar dentro da referência vigente!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REMOCAO_ALTERACAO_BAIXA_ALMOXARIFADO", "Não é possível remover/alterar a baixa, pois existe requisição ou Atendimento com a data posterior!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REMOCAO_COM_VINCULO", "Não é possível excluir pois o mesmo encontra-se em uso por outro cadastro no sistema!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REMOCAO_EDICAO_BAIXA_ALMOXARIFADO", "A baixa não pode ser editada ou removida pois possui requisição posterior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REMOCAO_REGRA_EVENTO", "A regra evento não pode ser excluída pois ela está vinculada a um monitor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.REPRESENTANTE_LEGAL_UNICO", "Só é permitido adicionar um representante legal para os contatos.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SAIDA_TEMPORARIA_ANTERIOR_ULTIMA_OPERACAO", "Você não pode realizar uma saída temporária de um bem patrimonial com data anterior a da última operação do bem! Verifique a operação: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SELECAO_DATA_INFERIOR_ENTRADA", "Não foi possível realizar a seleção do bem patrimonial pois a data informada é inferior a data contábil da entrada do bem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SELECIONAR_UM_RESPONSAVEL", "Você precisa selecionar um responsável!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SELECIONAR_UMA_FUNCAO", "Você precisa selecionar uma função!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SELECIONE_SITUCAO_FISICA", "Selecione uma situação física", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SELECIONE_SITUCAO_STATUS", "Selecione o status", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SELECIONE_UM_BEM_ALTERACAO", "Pelo menos um bem deve ser selecionado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SELECIONE_UM_ITEM_PARA_ATENDIMENTO", "Selecione pelo menos um item para o atendimento!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SEM_POSSIVEL_RESPOSTA", "Você deve digitar uma possível resposta!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SENHAS_DIFERENTES", "As senhas digitadas não coincidem!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SOMENTE_BAIXA_AUTORIZADA_PODE_ESTORNADA", "Somente baixa autorizada pode ser estornada!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SOMENTE_STATUS_EM_INCONSISTENCIA", "Somente bem patrimonial com status Em inconsistência pode ser visualizado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SOMENTE_STATUS_NAO_INFORMADO", "Somente bem patrimonial com status Não informado pode ser inventariado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.SOMENTE_UM_POR_PAPEL", "Já existe uma pessoa que exerce esse papel no contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.TAMANHO_CODIGO_CLASSIFICACAO", "Código da classificação pode ter no máximo 30 caracteres", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.TAMANHO_DESCRICAO_CLASSIFICACAO", "Descrição da classificação pode ter no máximo 100 caracteres", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.TOTAL_EMPENHOS_SUPERIOR_PERMITIDO", "O total de empenhos já atingiu o total do contrato!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.TRANSFERENCIA_ANTERIOR_ULTIMA_OPERACAO", "Você não pode transferir um bem patrimonial com data anterior a da última operação do bem! Verifique a operação: ", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.TRANSFERENCIA_DATA_INFERIOR_ENTRADA", "Não foi possível realizar a transferência pois a data informada é inferior a data contábil da entrada do bem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ULTIMO_HISTORICO_NAO_ENCONTRADO", "Não foi encontrado o último histórico referente a esse bem! Não é possível prosseguir!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.ULTIMO_MAPA_VIGENTE", "O sistema deve sempre ter um Mapa Organizacional vigente, portanto esse Mapa não pode ser excluído. Caso precise de um novo Mapa é só inserir um mapa e o sistema imediatamente finalizará a vigência do mapa anterior.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.UNIDADE_CONSUMIDORA_NAO_FAZ_REQUISICAO_NO_DIA_INFORMADO", "Unidade de requisição não faz requisição no dia informado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.UNIDADE_MAIOR_ZERO", "Unidade(s) deve ser maior que zero.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.UNIDADE_MEDIDA_VALIDACAO_EXCLUSAO", "Não é possível excluir esta unidade de medida. Existe vínculo com uma ou mais entradas.", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALIDA_ATRIBUICAO", "O(s) bem(ns) patrimonial(s) listado(s) já está(ão) atribuído(s) a um detentor com data de atribuição e/ou data fim de atribuição superior à data definida!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALIDACAO.ERRO_VINCULO_FLUXO_BPE_ESI", "O processo deverá estar vinculado a um fluxo no BPE/ESI!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALIDACAO_CNPJ", "CNPJ inválido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALIDACAO_CPF", "CPF inválido", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALOR_GARANTIA_TERMO_ADITIVO_INCORRETO", "A garantia do termo aditivo deve ser de no máximo 5%!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALOR_GARANTIA_TERMO_ADITIVO_INCORRETO_ALTA_COMPLEXIDADE", "Para termos de garantia de contratos de alta complexidade a garantia deve ser de no máximo 10%!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALOR_LIBERACAO_MAIOR_VALOR_EMPENHO", "Valor da liberação de pagamento maior que o valor do empenho!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VALOR_LIBERACAO_NAO_PODE_ZERO", "O valor da liberação de pagamento não pode ser zero!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("VALIDACAO.VIDA_UTIL_NAO_INFORMADA", "O campo Tipo recebimento selecionado como doação, deve ser informada a vida útil para todos os bens patrimoniais adicionados!", dominio, modulo));

	}
}
